/*
*
* $Author: bhunt $
* $Revision: 1.1.1.1 $
* $Log: Action.java,v $
* Revision 1.1.1.1  2000/09/19 19:39:44  bhunt
* Initial Population
*
* Revision 1.1.1.1  2000/08/17 17:50:08  bhunt
* Initial Code Upload
*
* Revision 1.2  1999/09/19 07:46:32  bhunt
* Action is now it's own java file.
*
* Revision 1.1  1999/09/19 07:23:41  bhunt
* Initial revision
*
*/
/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/Parsergen/Action.java,v 1.1.1.1 2000/09/19 19:39:44 bhunt Exp $


ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

One or more classes in ICEcubes have been written by:

V. Bruce Hunt,                  Technical Lead & Vice Chair, ICE-AG, 
    vbhunt@sourceforge.net      Adobe Systems, Inc.
    
Jonathan Herbach                Developer           
    jherbach@adobe.com          Princeton University & Adobe Systems, Inc.
    
Will Conant                     Developer
                                University of Utah & Adobe Systems Inc.

Andrew Francis                  Developer           
    afrancis@adobe.com          Adobe Systems, Inc.
    
Anna Shamble                    Developer           
    ashamble@adobe.com          Santa Clara University & Adobe Systems, Inc.

Authors of various classes are noted in the headings 

*/
/*----------------------------------------------------------------------------------+
 |                                                                                  |
 |                                  Action Class                                    |
 |                                                                                  |
 +----------------------------------------------------------------------------------*/

package com.adobe.rice.common.Parsergen;

import java.io.*;
import java.text.*;
import java.util.*;
//$JDK1.1$ import com.sun.java.util.collections.*;

/*----------------------------------------------------------------------------------+
 |                                                                                  |
 |                     CLASS Action - What to do in a LR Parse Table                |
 |                                                                                  |
 +----------------------------------------------------------------------------------*/
public class Action
{
    public final static short SHIFT = 0;
    public final static short REDUCE = 1;
    public final static short ACCEPT = 2;
    public final static short GOTO = 3;
    public final static short ERROR = 4;

    short  action = ERROR;
    short  nextState = 0;      //
    short  reduceIndex = 0;    // reduce symbol index

    public Action() { action = ERROR; nextState = 0; reduceIndex = 0; }

    public Action( short b, short ns )
    {
         this();
         action = b;
         nextState = ns;
    }

    public Action( short b, short ns, short rs )
    {
        this();
        action = b;
        nextState = ns;
        reduceIndex = rs;
    }

    public short getAction() { return action; }
    public short getNextState() { return nextState; }
    public short getReduceIndex() { return reduceIndex; }

    public boolean equals( Action a )
    {
        if ( action != a.action )
            return false;
        if ( nextState != a.nextState )
            return false;
        if ( reduceIndex != a.reduceIndex )
            return false;
        return true;
    }

    static final String actions[] = { "SHIFT", "REDUCE", "ACCEPT", "GOTO", "ERROR" };
    static final String actAbbr[] = {   "S",      "R",      "A",    "G",      "E"  };

    public String toShortString( NumberFormat nf )
    {
        String ac = actAbbr[action];
        switch( action )
        {
        case SHIFT:
            ac += nf.format( (long) nextState );
            break;
        case REDUCE:
            ac += nf.format( (long) reduceIndex );
            break;
        case ACCEPT:
            ac += "CC";
            break;
        case GOTO:
            ac += nf.format( (long) nextState );
            break;
        case ERROR:
            ac += nf.format( (long) nextState );
            break;
        default:
            ac = "DEAD";
            break;
        }
        return ac;
    }

    public String toStringCore( int ind )
    {
        String ac = LRconst.indent(ind) + "["+ actAbbr[action];
        if ( action == REDUCE )
        {
            Integer ri = new Integer( reduceIndex );
            ac += ri.toString();
        }
        else
        {
            Integer nxt = new Integer( nextState );
            ac += nxt.toString();
        }
        ac += "] ";
        return ac;
    }

    public String toString() { return toStringCore( 0 ); }

}


