
/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/Parsergen/LRparser.java,v 1.1.1.1 2000/09/19 19:39:44 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/
/**
 * LRparser.java
 *
 *
 * Created: Mon Feb  7 11:00:17 2000
 *
 * @author V. Bruce Hunt
 * @version
 */

/*--------------------------------------------------------------------------+
 |                                                                          |
 |                   CLASS LRparser - LR Parser generator                   |
 |                                                                          |
 +--------------------------------------------------------------------------*/

package com.adobe.rice.common.Parsergen;

import java.io.*;
import java.text.*;
import java.lang.*;
import java.util.*;
//$JDK1.1$ import com.sun.java.util.collections.*;
import com.adobe.rice.common.XMLassembly.*;

public class LRparser
{
    Grammar    grammar;     // the grammar for this parser
    HashMap    symIndex;    // symbol index mapping
    ArrayList  symArray;    // Array of symbols
    Action[][] action;      // parser action table
    // Stack      stack;       // parser stack
    // ParseState current;     // current ParseState  ( current == stack.peek() )
        
    public LRparser() 
    {
        grammar = null;
        symIndex = null;
        symArray = null;
        action = null;
        // stack = null;
        // current = null;
    }

    public LRparser( Grammar g )
    {
            // we assume that Grammar g is complete.
            grammar = g;
            // now compute the action table:
            LRparsergen lrpg = new LRparsergen();
            // compute the action & goto tables:
            action = lrpg.canonicalLR( grammar );
            // setup the indexing maps:
            symIndex = new HashMap(   (grammar.getSymbols()).size() );
            symArray = new ArrayList( (grammar.getSymbols()).size() );
            Iterator sii = grammar.getSymbols().iterator();
            for( short ofs = 0; sii.hasNext() ; ofs++ )
            {
                Symbol s = (Symbol) sii.next();
                Short ofo = new Short( ofs );
                symIndex.put( s, ofo );
                symArray.add( ofs, s );
            }
            // now setup the stack:
            // by definition, the top of the stack contains S0
            // the first state in the table; we must make it so (S0 ;-)).
            // 
            // following commented out (provided by user of object).
            // current = new ParseState( (short) 0, null );
            // stack = new Stack( );
            // stack.ensureCapacity( LRconst.STACK_DEPTH );
            // stack.push(current);
    }

    // 
    // methods:
    //

    //
    // isTokenSymbol - returns symbol if it is; null otherwise.
    //
    public Symbol isTokenSymbol( ScanToken t )
    {
        Symbol sym = new Symbol( t.string, LRconst.TERMINAL );
        if ( grammar.containsSymbol( sym ) )
            return sym;
        return null;
    }
    //
    // isStringSymbol - returns symbol if it is; null otherwise.
    //
    public Symbol isStringSymbol( String name )
    {
        Symbol sym = new Symbol( name, LRconst.TERMINAL );
        if ( grammar.containsSymbol( sym ) )
            return sym;
        return null;
    }
    //
    // tokenToSymbol - convert input token to symbol and throw exception if won't convert.
    //
    public Symbol tokenToSymbol( ScanToken t ) throws LRparserException
    {
        // First, look up the token to make sure that it is a symbol:
        Symbol sym = isTokenSymbol( t );

        if ( sym == null )
        {
            Object[] args = { t.string, grammar.getStartSymbol().get() };
            String msg = MessageFormat.format( Messages.badTokenInGrammar, args );
            LRparserException pBad = new LRparserException( msg );
            throw pBad; 
        }
        // OK!, This symbol is in the grammar.
        return sym;
    }
    //
    // stringToSymbol - convert input token to symbol and throw exception if won't convert.
    //
    public Symbol stringToSymbol( String name ) throws LRparserException
    {
        // First, look up the token to make sure that it is a symbol:
        Symbol sym = isStringSymbol( name );

        if ( sym == null )
        {
            Object[] args = { name, grammar.getStartSymbol().get() };
            String msg = MessageFormat.format( Messages.badTokenInGrammar, args );
            LRparserException pBad = new LRparserException( msg );
            throw pBad; 
        }
        // OK!, This symbol is in the grammar.
        return sym;
    }

    //
    // parseStepToken - steps the parser for a single input token.
    //
    public Action parseStepToken( ScanToken t, Stack sp ) throws LRparserException
    {
        Symbol s = isTokenSymbol( t );
        if ( s == null )
        {
            Object[] args = { t.string, grammar.getStartSymbol().get() };
            String msg = MessageFormat.format( Messages.badTokenInGrammar, args );
            LRparserException pBad = new LRparserException( msg );
            throw pBad;
        }
        return parseStep( s, sp );
    }
    //
    // parseStepString - steps the parser for a single input string.
    //
    public Action parseStepString( String name, Stack sp ) throws LRparserException
    {
        Symbol s = isStringSymbol( name );
        if ( s != null )
        {
           Object[] args = { name, grammar.getStartSymbol().get() };
           String msg = MessageFormat.format( Messages.badTokenInGrammar, args );
          LRparserException pBad = new LRparserException( msg );
          throw pBad;
        }
        return parseStep( s, sp );
    }
    //
    // parseStep - steps the parser for a single input.
    //
    public Action parseStep( Symbol s, Stack sp ) throws LRparserException
    {
        ParseState current = (ParseState) sp.peek(); // current ParseState  ( current == stack.peek() )
        Action a = new Action();
        String msg = null;
        LRparserException pBad = null;
        
        if ( s == null )
        { 
            Object[] args0 = { "(null) Symbol", grammar.getStartSymbol().get() };
            msg = MessageFormat.format( Messages.badTokenInGrammar, args0 );
            pBad = new LRparserException( msg );
            throw pBad;
        }
        // now parse the new symbol
        int stateIndex = current.getIndex();
        int symbolIndex = ((Short) symIndex.get( s )).intValue();
        
        a = action[ stateIndex ][ symbolIndex ];
        switch ( a.getAction() )
        {
            case Action.SHIFT:
                ParseState tos = new ParseState( a.nextState, s  );
                sp.push( tos );
                current = tos;
                break;
            case Action.REDUCE:
                Production rp = grammar.get( a.reduceIndex );
                Symbol lhs = rp.getLHS();
                int lhsIndex = ( (Short) symIndex.get( lhs )).intValue();
                // pop rhs symbols off of the stack.
                for ( int count = rp.size(); count > 0 ; count-- ) sp.pop();
                current = (ParseState) sp.peek();
                Action gotoState = action[ current.getIndex() ][ lhsIndex ];
                if ( gotoState.action != Action.GOTO )
                {   // bad error - throw a fit.
                    Object[] args1 = { rp.toString(), gotoState.toString() };
                    msg = MessageFormat.format( Messages.expectingGoTo, args1 );
                    pBad = new LRparserException( msg );
                    throw pBad; 
                }
                tos = new ParseState( gotoState.nextState, lhs );
                sp.push( tos );
                current = tos;
                // call semantic routine here!
                break;
            case Action.ACCEPT:
                // it's a good parse
                break;
            case Action.GOTO:
                // This cannot happen in theory -- throw a fit
                // We got a GOTO state on a terminal symbol -- there is a bug in Parsergen.
                Object[] args2 = { s.get(), a.toString() };
                msg = MessageFormat.format( Messages.unexpectedGoTo, args2 );
                pBad = new LRparserException( msg );
                throw pBad; 
            case Action.ERROR:
                Object[] args1 = { s.get(), grammar.getStartSymbol().get() };
                msg = MessageFormat.format( Messages.badTokenInGrammar, args1 );
                pBad = new LRparserException( msg );
                throw pBad; 
            default:
                // unknown action type -- This cannot happen in theory -- throw a fit.
                // bad bug in parserGen
                Object[] args3 = { s.get(), a.toString() };
                msg = MessageFormat.format( Messages.invalidLRtable, args3 );
                pBad = new LRparserException( msg );
                throw pBad;
        }
        return a;
    }

    //
    //  return list of acceptable strings for the current state.
    //      Each entry in the list is an object of class Symbol.
    //
    public List acceptableSymbols(Stack sp)
    {
        if ( grammar == null )
            return null;
        int terminalCount = grammar.getTerminalCount();
        ArrayList al = new ArrayList( terminalCount );
        ParseState current = (ParseState) sp.peek();
        int stateIndex = current.getIndex();
        Symbol s = null;
        short count = 0;

        for ( int i = 0; i < terminalCount ; i++ )
        {
            Action a = action[ stateIndex ][ i ];
            switch( a.getAction() )
            {
                case Action.SHIFT:
                case Action.REDUCE:
                case Action.ACCEPT:
                    s = (Symbol) symArray.get(i);
                    al.add( count++, s.get() );
                    break;
                default:
                    break;
            }
        }
        return al;
    }

    //
    // acceptSymbol -- returns, true if symbol is acceptable to parse; otherwise false.
    //
    public boolean acceptSymbol( Symbol s, Stack sp )
    {
        ParseState current = (ParseState) sp.peek();
        int stateIndex = current.getIndex();
        int symbolIndex = ((Short) symIndex.get( s )).intValue();
        
        Action a = action[ stateIndex ][ symbolIndex ];
        boolean itIsAcceptable = false;

        switch( a.action )
        {
            case Action.REDUCE:
            case Action.SHIFT:
                itIsAcceptable = true;
                break;
        }
        return itIsAcceptable;
    }

    //
    //  acceptToken -- returns, true if token is acceptable; otherwise, false.
    //          
    public boolean acceptToken( ScanToken t, Stack sp )
    {
        Symbol s = isTokenSymbol( t );
        return ( ( s == null ) ? false : acceptSymbol( s, sp ) );
    }
    //
    // acceptString -- returns, true if String is acceptable; otherwise, false.
    //
    public boolean acceptString( String name, Stack sp )
    {
        Symbol s = isStringSymbol( name );
        return ( (s == null)? false: acceptSymbol( s, sp ) );
    }
    
} // LRparser





