/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/Parsergen/LRparsergen.java,v 1.1.1.1 2000/09/19 19:39:44 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/
/*
*
* $Author: bhunt $
* $Revision: 1.1.1.1 $
* $Log: LRparsergen.java,v $
* Revision 1.1.1.1  2000/09/19 19:39:44  bhunt
* Initial Population
*
* Revision 1.1.1.1  2000/08/17 17:50:08  bhunt
* Initial Code Upload
*
* Revision 1.2  1999/09/19 07:47:31  bhunt
* LRparsergen is now it's own java file.
*
* Revision 1.1  1999/09/19 07:35:38  bhunt
* Initial revision
*
*/
/*--------------------------------------------------------------------------+
 |                                                                          |
 |            CLASS LRparsergen - LR 0 and LR 1 Parser generator            |
 |                                                                          |
 +--------------------------------------------------------------------------*/

package com.adobe.rice.common.Parsergen;

import java.io.*;
import java.text.*;
import java.lang.*;
import java.util.*;
//$JDK1.1$ import com.sun.java.util.collections.*;

/*--------------------------------------------------------------------------+
 |                                                                          |
 |      CLASS LRparsergen - Generate LR(1) parse tables for a grammar       |
 |                                                                          |
 +--------------------------------------------------------------------------*/
public class LRparsergen
{
    public LRparsergen() {}
	
    /**
       @method void closureLR0
	
       This routine generates the complete list of items that can form a 
       state in the parsing table for an element model.  This is an 
       implementation of the closure algorithm in "Compliers, Principles, 
       Techniques & Tools" by Aho, Sethi, Ullman March 1988, Page 232, 
       Addison Wesley, ISBN 0-201-10088-6.
	
       Note:  This algorithm runs as n*n where n is the number of productions 
       in the grammar.  Any ideas on how to speed it up?
	
    **/
    public void closureLR0( List stateItems, Grammar g )
    {
       	ProductionItem pi;
       	ListIterator li = stateItems.listIterator();
       	HashSet added = new HashSet();
       	// We use a HashSet to keep track of the items of the form [ p, 0 ] so we can 
       	// index directly on the production, p.
       	Production p;
       	int dp;
       	// run through and initially populate added
       	while ( li.hasNext() )
       	{
            pi = (ProductionItem) li.next();			// get next production item
            p = pi.getP();
            if ( (pi.getDp() == 0) && ( p != null ) )
       	    {
                added.add( p );
            } 
       	}
       	// setup for the second run for closure - re-init li.
       	li = stateItems.listIterator();
       	// compute the closure of the state items list.
       	while ( li.hasNext() )
       	{
            pi = (ProductionItem) li.next();
            p = pi.getP();			// get production
            dp = pi.getDp();		// get distinguished point
            if ( p != null )
            {
                Symbol s = p.get( dp );
                if ( s.isNonTerminal() )
                {   // add all of the productions in g of the form s -> rhs
                    // we run through all of the productions in g looking for those
                    // that have a non-terminal s.  This makes closureLR0 an n**2 operation.
                    for ( int j = 0; j < g.size(); j++ )
                    {	// get the jth production from the grammar.
                        Production gp = g.get(j);
                        // is it a production of the form s -> rhs?
                        if ( gp.getLHS().equals( s ) )
                        {   // have we already got it in our list?
                            if ( !added.contains( gp ) )
                            {
                                added.add( gp );
                                ProductionItem np = 
                                      new ProductionItem( gp, 0, Symbol.empty );
                                li.add( np );
                            }
                        }
                    } // end looking through the grammar $FIX$ n**2 loop.
                } // end s is a non terminal
            } // end production is not empty.
        } // end iteration through state items.
    }
    /** 
	@method public void closureLR1
	
	This routine computes the closure function for LR1 grammars.
	
	The following parameters are defined:
	   List sis is a list of ProductionItems.  
	   Each ProductionItem contains:
	      (1) a production, 
	      (2) a dp (distinquished point) and
	      (3) a lookahead terminal symbol.
	   of the form:
	   [ production,  dp,   terminal ]
                 |         |           |
                 v         v           v
	   [ A   ->  alpha . B gamma , a ]
			
	   Grammar g.  This is the complete grammar, including root, productions and
		       a hashset of symbols.
			
	The algorithm is defined as follows:
	
	for each production item, [ A -> alpha . B beta, a ], in stateItems, 
	     each production  B -> gamma in the grammar g,
	     each terminal c in FIRST( beta a )
	such that [ B -> . gamma, c ] is not in stateItems
	do
		add [ B -> .gamma, c ] to stateItems
	until no more items can be added to stateItems.
	return stateItems.
	
	This algorithm is from Aho, Sethi and Ullman; page 232.
	
    */
    public void closureLR1( StateSet stateset, Grammar g )
    {
        final int DEBUGLEVEL = 8;	// closureLR1's debug level --

        int piAdded = 0;   // counter to indicate number of new PI's added to closure list.
        HashMap added = new HashMap();
        // We use the "added" HashMap to keep track of the items of the form 
        // [ production, dp, a ] so we have a production indexed Map, each 
        // value is a symbol indexed HashSet.
       	// run through and add the items in stateItems
       	ProductionItem pi;
       	ListIterator li = stateset.getItems().listIterator();
       	Production p;	   // production for an item.
       	int dp;		   // distinguished point in a production from an item.
       	Symbol a;	   // look ahead symbol for an item.
       	// run through and initially populate added
       	while ( li.hasNext() )
       	{
            pi = (ProductionItem) li.next();  // get next production item
            p = pi.getP();
            dp = pi.getDp();
            a = pi.getLa();
            // [  A -> alpha . B beta, a ] 
            // [     pi      dp        a ]  
            if ( dp == 0 )
            {	// create a set of look ahead symbols
                HashSet laSyms = new HashSet(LRconst.SYMBOL_COUNT);
                // place the look ahead symbol in the set.
                laSyms.add( a );
                // place the production and it's look ahead symbol set in the added list.
                added.put( p, laSyms);
            }
        }
	
        do // while ( piAdded > 0 )
        {
            piAdded = 0;	// reset to count list stateset expansion.
            // setup for the next iteration for closure - re-init li.
            li = stateset.getItems().listIterator();
            // compute the next step in the closure of the state items list.
            while ( li.hasNext() )
            {
                pi = (ProductionItem) li.next(); // get next production item
                p = pi.getP();   // get the production p of form [ A -> alpha . B beta, a ]
                dp = pi.getDp(); // where the "." is the distinguished position, dp.
                a = pi.getLa();  // and a is the look ahead symbol.
			
                if ( dp >= p.size() ) // skip if at end of production.
                    continue;
                // get the symbol at the distinguished position.
                Symbol B = p.get(dp);
                // if B is a nonterminal then we have a production item of the form:
                // [ A -> alpha . B beta, a ] 
                if ( B.isNonTerminal() )
                {
                    // construct the follower list (i.e Beta a )
                    List beta_a = new ArrayList();
                    // construct beta
                    for ( int i = dp+1; i < p.size(); i++ )
                        beta_a.add( p.get(i));
                    // append a
                    beta_a.add( a );
                    // Compute the list of terminals in FIRST( beta_a )
                    // note:  is there a more efficient way to produce this list? 
                    //	      $INVESTIGATE$
                    HashSet isFIRSTofBeta_a = new HashSet(LRconst.SYMBOL_COUNT);
                    // iterate through the symbols of the grammar.
                    Iterator symit = g.symbols.iterator();
                    while ( symit.hasNext() )
                    {
                        Symbol s = (Symbol) symit.next();
                        if ( s.isNonTerminal() )
                            continue;
                        if ( g.inFIRST( beta_a, s) )
                            isFIRSTofBeta_a.add( s );
                    }
				
                    // find all grammar productions of the form B -> gamma.	
                    for ( int i = 0; i < g.size(); i++ )
                    {   // get next production
                        Production gp = g.get(i);
                        // if it is of the form B -> gamma then
                        if ( gp.getLHS().equals( B ) )
                        {   // for each terminal c in FIRST( beta a )
                            Iterator laSymIt = isFIRSTofBeta_a.iterator();
                            HashSet terminalList = null;
                            while ( laSymIt.hasNext() )
                            {   // get terminal c
                                Symbol c = (Symbol) laSymIt.next();
                                // if we already have B -> gamma ; i.e., gp
                                if ( added.containsKey( gp ) )
                                {   // check to see if have [ B -> .gamma, c ], 
                                    // continue if so.
                                    terminalList = (HashSet) added.get( gp );
                                    // if previously seen both gp and c, then 
                                    // continue with next
                                    if ( terminalList.contains( c ) )
                                        continue;
                                    // never seen production item [ gp, 0, c ] so
                                    // put c in our list and add a new production item.
                                    terminalList.add( c );
                                }
                                else // never seen the production gp == [ B -> gamma ]
                                {   // so create terminal list for gp,
                                    terminalList = new HashSet( LRconst.SYMBOL_COUNT );
                                    // put c in the terminal list.
                                    terminalList.add( c );
                                    // add gp & c to added so we don't later add 
                                    // idential production items.
                                    added.put( gp, terminalList );
                                }
                                // production item of the form [ B -> .gramma, c ] 
                                // not in state items.
                                // create the new production item:
                                ProductionItem npi = new ProductionItem( gp, 0, c );
                                // add it to our growing closure list 
                                // (using the list iterator).
                                li.add( npi );
                                piAdded++;	// count additions to closure list
                            } // end iterating through the terminals c in FIRST( beta a )
                        } // end of have production of the form B -> gamma. 
                    } // end iterating with  productions of grammar looking for B -> gamma.
                } // end B is a non-terminal symbol.
            } // end iterating through the production items in the current state set.
        } while ( piAdded > 0 ); // end repeat as long as something has been added.
        if ( LRconst.DEBUG > DEBUGLEVEL )
        {
            li = stateset.getItems().listIterator();
            System.out.println(" LR1 Closure [");
            while ( li.hasNext() )
            {
                pi = (ProductionItem) li.next();  // get next production item
                System.out.println( pi.toStringCore(1) ); // print it!
            }
            System.out.println("] --->| end LR1 Closure |<---");
       	} // end debug output
    } // end closureLR1();
	
    /**
       @method public List stateGoto
	
       This routine computes the list of goto states for each reachable 
       production of the grammar.
	
       This is an implementation of an algorithm slavishly copied from 
       the discussion of goto in "Compliers, Principles, Techniques & Tools" 
       by Aho, Sethi, Ullman March 1988, Page 232, Addison Wesley, 
       ISBN 0-201-10088-6.
       
       goto( I, X ) where I is a set of production items and X is 
                    grammar symbol.  
	
       goto( I, X ) is defined to be the closure of the set of all 
                    production items { A -> alpha X . beta, a } such 
                    that { A -> alpha . X beta } is in I.  
	
       Intuitively, if I is the set of items that are valid for 
       some viable prefix g, then goto( I, X ) is the set of items 
       that are valid for the viable prefix gX.
	
       The input to goto is always a list of productions items and a Symbol. 
	
       function goto( List items, Symbol X )
       begin
           let J be the set of production items [ A -> alpha X . beta, a ] such that
	       [ A -> alpha . X beta, a ] is in items.
       ????? Something is missing ???? 
	
    **/
    public StateSet stateGoto( StateSet si, Symbol x, Grammar g )
    {
        ArrayList ji = new ArrayList( LRconst.STATE_COUNT );
        int basisCount = 0;  // count the goto basis.	
        ListIterator sii = si.getItems().listIterator();
        while ( sii.hasNext() )
        {		
            ProductionItem pi = (ProductionItem) sii.next();
            int dp = pi.getDp();   // get the distinguished point	
            Production p = pi.getP();
            if ( p == null )
                continue;
            if ( dp >= p.size() )
                continue;
            Symbol dsX = p.get( dp );
            if ( dsX == null )
                continue;
            if ( ! dsX.equals( x ) )
                continue;
			
            ProductionItem npi = new ProductionItem( p, dp+1, pi.getLa() );
            ji.add( npi );
            basisCount++;
        }
        // we now have the basis for the state -- now we need the closure
        if ( basisCount > 0 )
        {
            StateSet gotoState = new StateSet( ji );
            closureLR1( gotoState, g );
            return gotoState;
        }
        else
            return null;
    }
	
     /**
       @method stateItems( grammar g )
	
       This routine computes the canonical collections of sets of LR(1) 
       ProductionItems for the grammar g.
	
    **/
    public ArrayList stateItems( Grammar g )
    {
        final int DEBUGLEVEL = 8;		// Routine's Debug level
        int stateAdded = 0;
        Production base = g.get(0);
        if ( base == null )
            return null;
        // construct the base production items from which the states for the parser grow	
        ProductionItem pi = new ProductionItem( base, 0, Symbol.endParse );
        // C is a list of elements, each element is a state set which 
        // contains production items.
        ArrayList emergingStateSetList = new ArrayList( LRconst.STATE_COUNT );
        // construct the basic initial state set.
        StateSet stateSet = new StateSet( pi );	
        // the initial state is the closure of the basic initial state set. 
        closureLR1( stateSet, g );
        // Put the initial state in the emerging list of state sets.	
        emergingStateSetList.add( stateSet );
    
        // now iterate through all the state sets in the emerging list
        do
        {   stateAdded = 0;	// repeat as long as states are added 
	    ListIterator cli = emergingStateSetList.listIterator();
	    while ( cli.hasNext() )
	    {   // get the next state set
            stateSet = (StateSet) cli.next();
            // iterate through the symbols (for each symbol x in grammar g )
            Iterator si = (g.getSymbols()).iterator();
            while ( si.hasNext() )
            {
                Symbol x = (Symbol) si.next();
                // compute the Goto state set of ( stateSet, x ) for grammar g.
                if ( LRconst.DEBUG > DEBUGLEVEL+1 )
                {
                    System.out.println( "Goto (" + x.toString() + ", stateSet) where:" );
                    System.out.println( stateSet.toString() );
                }
                StateSet nextState = stateGoto( stateSet, x, g );
                if ( nextState != null )
                {   // is the next state already in our list of states?
                    StateSet aNextStateSet = nextState.inStateSets( emergingStateSetList );
                    // if there is no StateSet that contains nextState then
                    if ( aNextStateSet == null )
                    {  
                        if ( LRconst.DEBUG > DEBUGLEVEL+2 )
                        {
                            System.out.println("Adding nextState =" + nextState.toString());
                        }
                        cli.add( nextState );
                        stateAdded++;
                        aNextStateSet = nextState;
                    }
                    else
                        nextState.release();	// explicitly free the next state.
                    nextState = null;		// and shout about it.

                    stateSet.put( x, aNextStateSet );  // store the goto indicator
                } // end non empty goto state seen
            } // end symbols in grammar iteration	
	    } // end stateset iteration
        } while( stateAdded > 0 );  // repeat as long as states are being added.
      
        if ( LRconst.DEBUG > DEBUGLEVEL )
        {
            System.out.println("State Items are:");
            ListIterator cli = emergingStateSetList.listIterator();
            System.out.println( "[" );
            while( cli.hasNext() )
            {
                stateSet = (StateSet) cli.next();
                System.out.print(stateSet.toStringCore(1));
            }
            System.out.println( "] <-- end State Items.\n");
        }
        return emergingStateSetList;
    }
	
    // 
    //	work routines and state for printing the LR1 parsing tables
    //
    int	  		indLevel = 0;		// the indent level to print at
    NumberFormat 	stateFormat = null;
    NumberFormat 	symbolFormat = null;
    String		errorEntry = null;	// filled in with an error state
    static final String blankLine = 
	"                                                                                               ";
    static final String	headerLine =
	       "+-----------------------------------------" + 
           "-----------------------------------------------------";

    void outline( String s )
    {
        System.out.println( LRconst.indent(indLevel) + s );
    }

    void outline( int i )
    {
        Integer io = new Integer(i);
        System.out.println( LRconst.indent( indLevel ) + io.toString() );
    }

    void outline() { System.out.println( LRconst.indent( indLevel) ); }

    void outstr( String s )
    {
        System.out.print(s);
    }

    void outstr( int i )
    {
        Integer io = new Integer(i);
        System.out.print( io.toString() );
    }

    String rowGroupLabel( int labelWidth )
    {
        return "|" + blankLine.substring(0, labelWidth-1 );
    }
    
    String colGroupLabel( int colWidth )
    {
        return headerLine.substring(0, colWidth-1 ) + "+";
    }

    String stateLabel( int state )
    {
        return  "| I" + stateFormat.format(state) + " ";
    }

    String stateLabelLine( int rowLabelWidth, int colWidth, int page, 
                           int cols, int maxColumns )
    {
        String line = rowGroupLabel( rowLabelWidth );
        for ( int col = 0; col < cols; col++ )
	    {
            line += stateLabel( (page * maxColumns ) + col );
	    }
        line += "|";
        return line;
    }

    String tableValue( Action a )
    {
        if ( a != null )
            return "| " + a.toShortString( stateFormat ) + " ";
        else
            return "| " + errorEntry + " ";
    }

    String rowLabel( Symbol s, int symOfs, int labelWidth  )
    {
        String r = "| ";
        String symOffset = symbolFormat.format( symOfs );
	
        r += s.get();
        while ( r.length() + symOffset.length() < (labelWidth - 1) )
            r += " ";
        r += symOffset;
        r += " ";
        return r;
    }

    String groupline( int rowLW, int pageCols, int colWidth )
    {
        String line = headerLine.substring(0, rowLW);
        for ( int i = 0; i < pageCols; i++ )
            line += headerLine.substring(0,colWidth);
        line += "+";
        return line;
    }

    public void printLR1core( int ind, Action table[][], ArrayList stateitems, Grammar g )
    {
        String line = "";
        indLevel = ind;
	
        line = "\nC a n o n i c a l   L R 1   s t a t e   t a b l e s";
        outline( line );
        // output the grammar:
        indLevel++;
        outline();
        line = "The grammar and symbols are:";
        indLevel++;
        outline( line );
        outstr( g.toStringCore(indLevel) );
        outline();
        indLevel--;
        // output the state items table
        indLevel++;
        outline();
        line = "The state items are: ";
        outline( line );
        indLevel++;
        for ( int i = 0; i < stateitems.size(); i++ )
        {
            StateSet ss = (StateSet) stateitems.get(i);
            outline( ss.toStringCore( indLevel ));
        }
        indLevel--;
        // output the state table now:
        HashMap symIndex = new HashMap( (g.getSymbols()).size() );
        ArrayList symArray = new ArrayList( (g.getSymbols()).size() );
        Iterator sii = g.getSymbols().iterator();
        short ofs = 0;
        int maxSymbolWidth = 0;
        while ( sii.hasNext() )
        {
            Symbol s = (Symbol) sii.next();
            Short ofo = new Short( ofs );
            symIndex.put( s, ofo );
            symArray.add( ofs, s );
            // compare to find maxSymbolWidth
            if ( s.getS().length() > maxSymbolWidth )
                maxSymbolWidth = s.getS().length();
            ofs++;
        }
        int symbolCount = ofs;			// get the number of symbols
	
        // compute column width as the number of chars needed to rep the state.
        int stateCount = stateitems.size();	// get the number of states
        // compute the columnWidth:
        int columnWidth = 1;
        int i = stateCount;
        while ( i > 0 ) { columnWidth++; i /= 10; }
        // construct the (default) error state string:
        errorEntry = blankLine.substring(0,columnWidth );
        // add in seperator  spaces  e.g: | S999 |
        columnWidth += 3; // for the |, and two blanks.

        // create a number format for a state:
        stateFormat = NumberFormat.getNumberInstance();
        stateFormat.setMaximumFractionDigits(0);
        stateFormat.setMaximumIntegerDigits( (columnWidth-4) );
        stateFormat.setMinimumIntegerDigits( (columnWidth-4) );
	
        // compute the row label width:  | ssssssss nnn |
        int rowLabelWidth = maxSymbolWidth + 4;	
        // initial value = maxSymbolWidth + 3 blanks + 1 bar
        // other bar provided by column entry.
        i = ofs;
        while( i > 0 ) { rowLabelWidth++; i /= 10; } // poor mans log base 10 computation

        // create a number format for a symbol offset:
        symbolFormat = NumberFormat.getNumberInstance();
        symbolFormat.setMaximumFractionDigits(0);
        symbolFormat.setMaximumIntegerDigits( ( rowLabelWidth - maxSymbolWidth - 4 ) );
        symbolFormat.setMinimumIntegerDigits( ( rowLabelWidth - maxSymbolWidth - 4 ) );
	
        // compute the number of columns that can be printed
        int maxLineLength = 90;		
        // arbitrary for portrait style assuming 12 chars per inch.
        int columns = ( maxLineLength - rowLabelWidth - 1 ) / columnWidth;
        int rowWidth = rowLabelWidth + ( columns * columnWidth ) + 1;  
        // don't forget closing bar at end.

        // compute the number of rows that can be printed on a page
        int colLabelWidth = 2 ;
        int maxRowCount = 95;       // for portrait style assuming 10 rpi and 9.5 inches.
        int rowGroupHeight = 6;		// arbitrary 5 rows and alignment seperator
        int rowGroups = ( maxRowCount - colLabelWidth -1 ) / rowGroupHeight ;
        int rows = rowGroups * ( rowGroupHeight - 1 ) ;
        int colHeight = colLabelWidth + ( rowGroupHeight * rowGroups ) + 1; 
        // don't forget closing line 

        // now we are ready to print it
        // The strategy is to print pages left to right 
        // ( i.e. states first ) then top to bottom ( i.e. symbols second ).
        int col = 0;
        int row = 0;

        // compute the number of width and height pages:
        int widthPages = stateCount / columns + 1;
        int heightPages = symbolCount / rows + 1;

        // figure out how to eject a page here!
        outstr( "\f" );
        outline("  LR1 Canonical Parsing Table \n");

        for ( int heightPage = 0; heightPage < heightPages ; heightPage++ )
        {
            for ( int widthPage = 0; widthPage < widthPages; widthPage++ )
            {
                int stateNum = widthPage * columns;
                int symbolNum = heightPage * rows;
		
                // output the next page here:
                // 1.  Compute the number of page columns and rows:
                int pageColumns = Math.min( columns, stateCount - (widthPage * columns) );
                int pageRows = Math.min( rows, symbolCount - (heightPage * rows ) );

                // first output the header label
                //    the groupline looks like +--------------+----+----+...+----+
                outline( groupline( rowLabelWidth, pageColumns, columnWidth ) );
                // now output the header
                outline( stateLabelLine( rowLabelWidth, columnWidth, 
                                         widthPage, pageColumns, columns ) );
                // now output lower groupline
                outline( groupline( rowLabelWidth, pageColumns, columnWidth ) );
                //
                // now output the (partial) rows
                //
                for ( row = 0; row < pageRows ; row++ )
                {
                    symbolNum = heightPage * rows + row;
			    
                    // output row label
                    Symbol s = (Symbol) symArray.get(symbolNum);
                    outstr(  LRconst.indent(indLevel) + 
                             rowLabel( s, symbolNum, rowLabelWidth ) );
                    // now output the columns
                    for ( col = 0; col < pageColumns; col++ )
                    {
                        stateNum = widthPage * columns + col;
				
                        Action a = table[ stateNum ][ symbolNum ];
                        outstr( tableValue( a ) );
                    }
                    outstr("|");
                    outline();
                    // have we done a group yet ?
                    if ( ( (row + 1) % (rowGroupHeight - 1) ) == 0 )
                        outline( groupline( rowLabelWidth, pageColumns, columnWidth ) );
                }
                // output a groupline if we didn't just do one:
                if ( ( row % ( rowGroupHeight -1 ) ) != 0 )
                    outline( groupline( rowLabelWidth, pageColumns, columnWidth ) );
                // now output the state label again:
                outline( stateLabelLine( rowLabelWidth, columnWidth, 
                                         widthPage, pageColumns, columns ) );
                // output the final groupline:
                outline( groupline( rowLabelWidth, pageColumns, columnWidth ) );
            }
        }
    }

    //
    //	Construction of the canonical LR1 parsing table:
    //
	
    public Action[][] canonicalLR( Grammar g )
    {
        ArrayList C = stateItems( g );
        ArrayList sortedC = new ArrayList( C.size() );
        for ( int idx = 0; idx < C.size(); idx++ )
        {
            for ( int jdx = 0; jdx < C.size(); jdx++ )
            {
                StateSet ssj = (StateSet) C.get(jdx);
                if ( idx == ssj.getIndex() )
                {
                    sortedC.add( ssj );
                    break;
                }
            }
        }
        C = null;	// release the extraneous C
        Action lr[][] = new Action[ sortedC.size()][ ( g.getSymbols() ).size() ];
		
        HashMap symbolIndex = new HashMap( g.getSymbols().size() );
        Iterator sii = g.getSymbols().iterator();
        short ofs = 0;
        while ( sii.hasNext() )
        {
            Symbol s = (Symbol) sii.next();
            Short offset = new Short( ofs++ );
            symbolIndex.put( s, offset );
        }
		
        for ( int i = 0; i < sortedC.size(); i++ )
        {
            StateSet sbi = (StateSet) sortedC.get(i);
            ArrayList Ii = sbi.getItems();	
            for ( int k = 0; k < Ii.size() ; k++ )
            {
                ProductionItem pi = (ProductionItem) Ii.get(k);
                // pi == [ A -> alpha . a beta, b ]
                Production p = pi.getP();
                // p == A -> alpha a beta
                int dp = pi.getDp();
                if ( dp < p.size() )
                {
                    Symbol a = p.get(dp);
                    short aIndex = ( (Short) symbolIndex.get(a) ).shortValue();
                    if ( a.isTerminal() )
                    {
                        if ( sbi.containsGotoSymbol( a ) )
                        {
                            StateSet gtj = sbi.getGoto( a );
                            for ( int j = 0; j < sortedC.size(); j++ )
                            {
                                if ( gtj == sortedC.get(j) )
                                {  
                                    Action na = new Action( Action.SHIFT, (short) j );
                                    // set action (i, a) to [Action.SHIFT, m];
                                    // $FIX$
                                    if ( lr[ i ][ aIndex ] != null )
                                    {   // possible conflict, already an action here.
                                        if ( ! na.equals( lr[i][aIndex] ) )
                                        {
                                            System.out.print( "Action Conflict at lr[");
                                            System.out.print( i );
                                            System.out.print( "," );
                                            System.out.print( aIndex );
                                            System.out.println( "] = " + lr[i][aIndex].toString() );
                                            System.out.println( "Assign attempt: " + na.toString() );
                                        }
                                    }
                                    else
                                    {
                                        lr[ i ][ aIndex ] = na;
                                    }			
                                }
                            }	
                        }
                    }
                    else
                    {
                        if ( sbi.containsGotoSymbol( a ) )
                        {
                            StateSet gtj = sbi.getGoto( a );
                            for ( int j = 0; j < sortedC.size(); j++ )
                            {
                                if ( gtj == sortedC.get(j) )
                                {
                                    Action na = new Action( Action.GOTO, (short) j );
                                    if ( lr[ i][ aIndex ] != null )
                                    {
                                // possible conflict
                                        if ( ! na.equals( lr[i][aIndex] ) )
                                        {
                                            System.out.print( "Goto Conflict!" );
                                            System.out.print( "at lr[" );
                                            System.out.print( i );
                                            System.out.print( "," );
                                            System.out.print( aIndex );
                                            System.out.println( "] = " + 
                                                                lr[i][aIndex].toString() );
                                            System.out.println( "Assign attempt: " + 
                                                                na.toString() );
                                        }
                                    }
                                    else
                                    {
                                        lr[ i ][ aIndex ] = na;
                                    }
                                }
                            }
                        }
                    }
                }
                else // we have the reduce case since dp points at end
                {
                    // if [ A -> alpha . , a ] is in sbi and A is not root
                    // then set action[ i, a ] to reduce[ A -> alpha ]
                    // ( that is pop alpha off the stack and replace it with A )
                    // if [ root -> r ., @END ] is in sbi then action[ i, @END ] is "accept".
                    Symbol a = pi.getLa();
                    short aIndex = ( (Short) symbolIndex.get( a ) ).shortValue();
					
                    Symbol lhs = p.getLHS();
                    short lhsIndex = ( (Short) symbolIndex.get( lhs ) ).shortValue();
					
                    if ( lhs.getS().equals( LRconst.ROOT ) &&
                         a.get().equals( LRconst.ATEND ) )
                    {
                        Action na = new Action( Action.ACCEPT, (short) 0 );
                        if ( lr[ i ][ aIndex ] != null )
                        {
                                // possible conflict
                            if ( ! na.equals( lr[i][aIndex] ) )
                            {	
                                System.out.print( " Accept Conflict " );
                                System.out.print( "at lr[" );
                                System.out.print( i );
                                System.out.print( "," );
                                System.out.print( aIndex );
                                System.out.println( "] = " + lr[i][aIndex].toString() );
                                System.out.println( "Assign attempt: " + na.toString() );
                            }
                        }
                        else
                        {
                            lr[i ][aIndex] = na;
                        }
                    }
                    else
                    {
                        // get the production index in g of p.
                        short prodIndex = (short) g.getProductionIndex( p );
                        Action act = new Action( Action.REDUCE, (short) p.size(), prodIndex );
                        if ( lr[i][aIndex ] != null )
                        {
                                // Possible conflict
                            if ( ! act.equals( lr[i][aIndex] ) )
                            {
                                System.out.print("Reduce Conflict");
                                System.out.print( "at lr[" );	
                                System.out.print( i );	
                                System.out.print( "," );
                                System.out.print( aIndex );
                                System.out.println( "] = " + lr[i][aIndex].toString() );
                                System.out.println( "Assign attempt: " + act.toString() );
                            }
                        }
                        else
                        {
                            lr[ i ][ aIndex ] = act;
                        }
                    }
                }
            }
        }
        printLR1core( 0, lr, sortedC , g ); 
        return lr;
    }
}














