/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/Parsergen/StateSet.java,v 1.1.1.1 2000/09/19 19:39:45 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/
/*
*
* $Author: bhunt $
* $Revision: 1.1.1.1 $
* $Log: StateSet.java,v $
* Revision 1.1.1.1  2000/09/19 19:39:45  bhunt
* Initial Population
*
* Revision 1.1.1.1  2000/08/17 17:50:08  bhunt
* Initial Code Upload
*
* Revision 1.2  1999/09/19 07:47:31  bhunt
* LRparsergen is now it's own java file.
*
* Revision 1.1  1999/09/19 07:35:38  bhunt
* Initial revision
*
*/
/*--------------------------------------------------------------------------+
 |                                                                          |
 | CLASS StateSet - Set of possible states (Production Items) for a grammar |
 |                                                                          |
 +--------------------------------------------------------------------------*/

package com.adobe.rice.common.Parsergen;

import java.io.*;
import java.text.*;
import java.util.*;
//$JDK1.1$ import com.sun.java.util.collections.*;

   /**
       @class StateSet
	
       This class contains an ArrayList of ProductionItems and a HashMap 
       of the symbols to other StateBags that this StateBag will goto.
	
       The ArrayList of Production Items is the closure of the basis 
       production items either from the initial basis [ root -> . r @END ]; 
       or from the closure of each of the goto sets of Production Items 
       and symbols.
	
       The purpose of the StateSet is to keep track of the transitions 
       out of this proto-state caused by a terminal or non-terminal 
       grammar symbol.
	
    **/
public class StateSet
{
    static int	stateIndex = 0;
    int		    index;
    ArrayList 	items;
    HashMap	    gotoSymbol;
		
    StateSet()
    {
        index = stateIndex++;
        items = new ArrayList( LRconst.STATE_COUNT );
        gotoSymbol = new HashMap( LRconst.SYMBOL_COUNT );
    }
		
    StateSet( ProductionItem pi )
    {
        this();
        items.add( pi );
    }
		
    StateSet( ArrayList ni )
    {
        index = stateIndex++;
        items = ni;
        gotoSymbol = new HashMap( LRconst.SYMBOL_COUNT );
    }
		
    public void add( ProductionItem pi ) { items.add( pi ); }
    public ProductionItem get( int i ) 
    { 
        return (ProductionItem) items.get(i); 
    }
    public int getIndex() { return index; }
    public ArrayList getItems() { return items; }
    public void setItems( ArrayList ni ) { items = ni; }	
    public void put( Symbol s, StateSet sb ) 
    { 
        gotoSymbol.put( s, sb ); 
    }
    public boolean containsGotoSymbol( Symbol s ) 
    { 
        return gotoSymbol.containsKey( s ); 
    }
    public boolean containsGotoStateSet( StateSet sb ) 
    { 
        return gotoSymbol.containsValue( sb ); 
    }
    public StateSet getGoto( Symbol s ) 
    { 
        return (StateSet) gotoSymbol.get( s ); 
    }
    public int size() { return items.size(); }

    public void release() 
    { 
        // if releasing the top of the list; then reuse the index.
        if ( this.index == stateIndex - 1 ) stateIndex--;
        items = null;
        gotoSymbol = null;
        index = 0;
    }

    String toStringCore( int indentLevel )
    {
        String sset = "\n";
        int ind = indentLevel;
        int count = 0;
        Integer intIndex = new Integer( index );
        sset += LRconst.indent( ind );
        sset += "[\n";
        ++ind;
        sset += LRconst.indent( ind );
        sset += "index = " + intIndex.toString() + ",\n";
        sset += LRconst.indent( ind );
        sset += "items = [\n";
        ++ind;
        for ( count = 0; count < items.size(); count++ )
	    {
            sset += LRconst.indent( ind );
            sset += this.get(count).toString() + "\n";
	    }
        --ind;
        sset += LRconst.indent( ind ) + "],\n";
        sset += LRconst.indent( ind ) + "GOTO functions = [ ";
        ++ind;
        // iterate through the goto symbols
        Iterator gtsi = gotoSymbol.keySet().iterator();
        while ( gtsi.hasNext() )
        {
            Symbol s = (Symbol) gtsi.next();
            Integer sssIndex = new Integer( this.getGoto( s ).getIndex() );
            sset += "GOTO(I" + intIndex.toString() + "," + 
                    s.getS() + ") = I" +sssIndex.toString();
            if ( gtsi.hasNext() ) sset += ", ";
        }
        sset += "]\n";
        --ind;
        --ind;
        sset += LRconst.indent( ind ) + "]";
        return sset;
    }
	
    public String toString()
    {
        return toStringCore( 0 );
    }
	
    /**
       @method StateSet inList( List losb )
       List losb is a list of StateSets.  
       This routine examines each StateSet's Arraylist, c, in losb 
       and compares it to this object's Arraylist, designated a here.
       if the list a is equal to the list c , the StateSet 
       containing a is returned; otherwise comparison continues.  
       If there is no c in some StateSet in losb 
       such that a equals c, null is returned.

       The wrinkle is that the order of Production Items in a 
       state set is not identical.

    **/
    public StateSet inStateSets( List setOfStateSets )
    {
        final int DEBUGLEVEL = 8;	// set the debug level for this routine
        boolean isin = false;
        ArrayList thisProdList = this.getItems();
        if ( LRconst.DEBUG > DEBUGLEVEL )
        {
            System.out.println( "Entering inStateSets for:" );
            System.out.println( this.toString() );
        }
        ListIterator si = setOfStateSets.listIterator();
        while ( si.hasNext() )
        {
            StateSet stateset = (StateSet) si.next();
            ArrayList siProdList = stateset.getItems();

            if ( LRconst.DEBUG > DEBUGLEVEL )
            {
                System.out.println("Checking following for equality:");
                System.out.println( stateset.toStringCore(2) );
            }
			
            if ( siProdList.size() != thisProdList.size() )
            {
                if ( LRconst.DEBUG > DEBUGLEVEL )
                {
                    System.out.print("Item count in stateset = ");
                    System.out.println( siProdList.size() );
                }
                continue;
            }
            if ( LRconst.DEBUG > DEBUGLEVEL )
            {
                System.out.println("Comparing for set equality");
            }
            if ( setEqual( thisProdList, siProdList) )
            {
                if ( LRconst.DEBUG > DEBUGLEVEL )
                    System.out.println(" !!! Sets are EQUAL !!!\nLeaving inStateSets.");
                return stateset;
            }
        }
        if ( LRconst.DEBUG > DEBUGLEVEL )
            System.out.println(" ### Sets UNEQUAL! ### \nLeaving inStateSets.");
        return null;
    }

    //
    // note:  setEqual( List s, List t ) compares items in List s to items in List t
    //        without regard to order.  setEqual returns true iff every item in s
    //        is equal to some unique item in t; and every item in t is equal to
    //        some unique item in s.
    public boolean setEqual( List s, List t )
    {
        final int DEBUGLEVEL = 8;		// Routine's DEBUGLEVEL 

        if ( LRconst.DEBUG > DEBUGLEVEL )
	    {
            System.out.println("Entering setEqual");
            System.out.print(" List S has size = ");
            System.out.println( s.size() );
            System.out.print(" List T has size = ");
            System.out.println( t.size() );
	    }
        if ( s.size() != t.size() )
        {
            if ( LRconst.DEBUG > DEBUGLEVEL )
            {
                System.out.println(" !!!! item counts differ -- they cannot be equal.");
                System.out.println("Leaving setEqual with false.");
            }
            return false;
        }

        boolean isin[] = new boolean[ s.size() ];
        int i, j;

        for ( i = 0; i < s.size(); i++ )
        {
            Object os = s.get(i);
            if ( LRconst.DEBUG > DEBUGLEVEL )
            {
                System.out.println("===============================================");
                System.out.println( "Class = " + os.getClass().getName() );
                System.out.println( os.toString() );
                System.out.println("-----------------------------------------------");
            }
            for ( j = 0; j < t.size(); j++ )
            {
                Object ot = t.get(j);
                // objects cannot be equal if not in same class
                if ( LRconst.DEBUG > DEBUGLEVEL )
                {
                    System.out.println( ot.toString() );
                    System.out.println( "Class = " + ot.getClass().getName() );
                }
                if ( ! os.getClass().getName().equals( ot.getClass().getName() ) )
                    continue;
                // if ot has already been matched, don't rematch
                // remove this statement if you want to support
                // define set equality that can have instance replications.
                if ( isin[j] )
                {
                    if ( LRconst.DEBUG > DEBUGLEVEL )
                        System.out.println(" Object has already been matched!" );
                }
                else if ( ( (ProductionItem) os ).equals( (ProductionItem) ot ) )
                {
                    if ( LRconst.DEBUG > DEBUGLEVEL )
                        System.out.println(" Objects match!" );
                    isin[j] = true;
                    break;
                }
                if ( LRconst.DEBUG > DEBUGLEVEL )
                    System.out.println("- - - - - - - - - - - - - - - - - - - -");
            }
            // if j >= t.size() then os was not equal to any item in the list, t
            if ( j >= t.size() )
            {
                if ( LRconst.DEBUG > DEBUGLEVEL )
                {
                    System.out.println(" Object os didn't match any ot in t!");
                    System.out.println("-----------------------------------------------");
                }
                break;
            }
            if ( LRconst.DEBUG > DEBUGLEVEL )
                System.out.println("-----------------------------------------------");
        }
        // if i >= s.size() then each item in s was identical to some item in t, and
        // each item in t was identical to only one item in s.
        if ( i >= s.size() )
        {
            if ( LRconst.DEBUG > DEBUGLEVEL )
                System.out.println("!!! Sets are EQUAL !!!\nleaving setEqual.");
            return true;
        }
        if ( LRconst.DEBUG > DEBUGLEVEL )
            System.out.println("### Sets are UNEQUAL ###\nleaving setEqual.");
        return false;
    }
}








