/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/Parsergen/Symbol.java,v 1.1.1.1 2000/09/19 19:39:45 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/
/*
*
* $Author: bhunt $
* $Revision: 1.1.1.1 $
* $Log: Symbol.java,v $
* Revision 1.1.1.1  2000/09/19 19:39:45  bhunt
* Initial Population
*
* Revision 1.1.1.1  2000/08/17 17:50:08  bhunt
* Initial Code Upload
*
* Revision 1.2  1999/09/19 07:49:06  bhunt
* Symbol is now it's own java file.
*
* Revision 1.1  1999/09/19 07:01:47  bhunt
* Initial revision
*
*/

/*--------------------------------------------------------------------------+
 |                                                                          |
 | CLASS StateSet - Set of possible states (Production Items) for a grammar |
 |                                                                          |
 +--------------------------------------------------------------------------*/

package com.adobe.rice.common.Parsergen;

import java.io.*;
import java.text.*;
import java.util.*;
//$JDK1.1$ import com.sun.java.util.collections.*;

/*--------------------------------------------------------------------------+
 |                                                                          |
 |               CLASS Symbol -- repreesnt a Grammar Symbol                 |
 |                                                                          |
 +--------------------------------------------------------------------------*/

public class Symbol implements Comparable
{
    String	    s;     // symbol string.
    boolean     nt;    // True if this is a non-terminal; false otherwise
	
    public Symbol()
    {
        s = LRconst.EMPTY;		// The explicit empty symbol
        nt = LRconst.TERMINAL;		// a terminal symbol
    }
	
    public Symbol( String ns )
    {
        this();				
        s = ns;				//
        nt = LRconst.NONTERMINAL;	// named symbols are by default non-terminal
    }
  
    public Symbol( String ns, boolean b )
    {
        this();
        s = ns;
        nt = b;
    }
	
    public boolean isEmpty() { return s.equals(LRconst.EMPTY); }
    public boolean isRoot() { return s.equals(LRconst.ROOT); }
    public boolean isNonTerminal() { return nt; }
    public boolean isTerminal() { return !nt; }
    public boolean isNt() { return nt; }
    public String get() { return s; }
    public String getS() { return s; }
      
    public void setNt( boolean n ) { nt = n; }
	
    public void setS( String ns ) { s = ns; }
    public void setSymbol( String ns, boolean n ) { s = ns; nt = n; }
    public int  length() { return s.length(); }
	
    public boolean equals( Symbol ns )
    {
        if ( nt != ns.nt )
            return false;
        if ( ! s.equals(ns.s) )
            return false;
        return true;
    }

    public int compareTo( Object ons )
    {
        Symbol ns = (Symbol) ons;
	
        // if this is non terminal and ns is terminl, this is the greater.
        if ( nt && !ns.nt )
            return 1;
        // if this is terminal and ns is non terminal, this is the lesser.
        if ( !nt && ns.nt )
            return -1;
        // otherwise compare the strings
        return s.compareTo( ns.s );
    }

    public String toStringCore( int ind )
    {
        return LRconst.indent(ind)+(( nt )? "~":"" ) + s;
    }

    public String toString( ) { return toStringCore( 0 ); }
    
    // following are standard Symbols ( empty symbol, endParse symbol and root symbol)
    public final static Symbol empty = new Symbol( LRconst.EMPTY, LRconst.TERMINAL );
    public final static Symbol endParse = new Symbol( LRconst.ATEND, LRconst.TERMINAL );
    public final static Symbol root = new Symbol( LRconst.ROOT, LRconst.NONTERMINAL );
}




