/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/Tray.java,v 1.5 2000/11/27 20:27:07 bhunt Exp $


ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

One or more classes in ICEcubes have been written by:

V. Bruce Hunt,                  Technical Lead & Vice Chair, ICE-AG, 
    vbhunt@sourceforge.net      Adobe Systems, Inc.
    
Jonathan Herbach                Developer           
    jherbach@adobe.com          Princeton University & Adobe Systems, Inc.
    
Will Conant                     Developer
                                University of Utah & Adobe Systems Inc.

Andrew Francis                  Developer           
    afrancis@adobe.com          Adobe Systems, Inc.
    
Anna Shamble                    Developer           
    ashamble@adobe.com          Santa Clara University & Adobe Systems, Inc.

Authors of various classes are noted in the headings 

*/

/*----------------------------------------------------------------------------------------+
 |                                                                                        |
 |                              Reference Implementation of ICE                           |
 |                                                                                        |
 |                    Tray - container for the core list of ICEcubes managers             |
 |                                                                                        |
 +----------------------------------------------------------------------------------------*/ 

package com.adobe.rice.common;

import com.adobe.rice.common.storage.logging.*;
import com.adobe.rice.common.policies.*;
import com.adobe.rice.common.payloadAssembly.*;
import com.adobe.rice.common.XMLassembly.*;
import com.adobe.rice.kernel.*;
import com.adobe.rice.common.*;

import java.util.*;

/** Tray contains the core list of managers known to Rice.
 ** Tray only initially knows about a single manager, the PolicyManager.
 ** The PolicyManager's job at tray initialization time is to register the 
 ** other managers.  Once the other managers have been registered,
 ** they are initialized according to the phased initializion policy
 ** defined by the Manager interface.
 **
 ** @see Manager
 **
 **/

import java.io.*;

import com.adobe.rice.util.*;
import com.adobe.rice.common.storage.*;
import com.adobe.rice.common.policies.PolicyManager;

public class Tray extends IceCubeManager
{
    private class SourceLink
    {
        String source;
        Object message;
        
        SourceLink( String src, Object msg )
        {
            source = src;
            message = msg;
        }
    }
    
    
    // Core managers available anywhere:
    //
    // 
    public static StorageManager storageManager;
    public static PolicyManager  policyManager;
    public static PayloadFactory payloadFactory;

    // A mapping from manager name to manager object.
    // This approach permits a manager dynamic loader to be built here.
    // Thus, for example, there may be no need to load the Subscriber Manager if
    // this use of RICE is for Syndication only.
    //
    // It also decouples Tray dependencies on all managers except the 
    // distinguished policy manager.
    //
    private static HashMap managerMap;
    private static HashMap queueMap;
    
    //
    // The UUID for this service ( syndicator or subscriber )
    //
    static UUID theUUID;

    // Rice runs in phases as follows:
    //	note: Always use the manifest constants and never the values.
    //	to assure that the insertion of a new phase does not
    //  break your code.
    //
    public static final byte PRE_INIT   = 0;
    public static final byte IN_INIT    = 1;
    public static final byte POST_INIT  = 2;
    public static final byte RUNNING    = 3;
    public static final byte PRE_FINAL  = 4;
    public static final byte IN_FINAL   = 5;
    public static final byte POST_FINAL = 6;
    public static final byte FINALIZED  = 7;

    private static byte phase = PRE_INIT;

    private static final String[] phases = 
    { 
        "Pre-initialization",
        "Initializtion",
        "Post-initialziation",
        "Run",
        "Pre-finalization",
        "Finalization",
        "Post-Finalization"
    };
	
    private ThreadGroup managerThreads = new ThreadGroup("managers");

    public Tray () throws DispatcherException
    {
        super ("tray", MessageDispatcher.instance ());
        
        // Initialize the managerMap exactly once.
        if ( managerMap == null )
        {
            managerMap = new HashMap( 50 );
            queueMap = new HashMap( 50 );
        }
        // 
        // initialize storage manager first - so others can use it as early as possible.
        //
        storageManager = StorageManager.instance();
        
        // initialize policy manager second -- so others can use policies as early as possible.
        //
        policyManager = PolicyManager.instance(); 
        //
        // Now read in the configuration files
        //
        try
        {
            this.policyManager.initializeConfiguration();
        }
        catch ( FileNotFoundException fnf )
        {
            // $FIX$ output the exception message and log it if possible.
            System.out.println( " Configuration File NOT FOUND! " );
            System.out.println( fnf.getMessage() );
            System.exit(-9001);
        } 
        catch ( PolicyException pe )
        {
            // $FIX$ output the exeception message and log it if possible
            System.out.println(" Policy Exception -- Policy Manager Failure!");
            System.out.println( pe.getMessage() );
            System.exit(-9002);
        }   
        
        // now register the policy manager.
        registerManager( "Policy", policyManager );
        // now run the policy that registers managers
        policyManager.managerRegister( this );
        //
        // now initialize the basic information about us:
        //
        payloadFactory = PayloadFactory.instance();
        /*
        try { payloadFactory = PayloadFactory.instance(); }
        catch ( ElementException ee ) 
        {
            // $FIX$ output the exeception message and log it if possible
            System.out.println(" Payload Factory Failure -- Element Exception!");
            System.out.println( ee.getMessage() );
            System.exit(-8001);
        }
        catch ( DocTypeException dte ) 
        {
            // $FIX$ output the exeception message and log it if possible
            System.out.println(" Payload Factory Failure -- ICE DTD Exception!");
            System.out.println( dte.getMessage() );
            System.exit(-8002);
        }
        catch ( IOException ioe ) 
        { 
            // $FIX$ output the exeception message and log it if possible
            System.out.println(" Payload Factory Failure -- I/O Exception!");
            System.out.println( ioe.getMessage() );
            System.exit(-8003);
        }
        catch ( XMLparseException xpe ) 
        { 
            // $FIX$ output the exeception message and log it if possible
            System.out.println(" Payload Factory Failure -- ICE DTD Parsing Exception!");
            System.out.println( xpe.getMessage() );
            System.exit(-8004);
        }
        catch ( XMLdocException xde ) 
        { 
            // $FIX$ output the exeception message and log it if possible
            System.out.println(" Payload Factory Failure -- XML Document Exception!");
            System.out.println( xde.getMessage() );
            System.exit(-8005);
        }
        */
        
    }
    
    
    public void postInitialize ()
    {
        
        
    }
    
    
    public void preFinalize ()
    {
        
    }
    
    public void initialize ()
    {
        
    }
    
    
    public void postFinalize ()
    {
        
    }

    
    public void preInitialize ()
    {
        
    }
    
    
    public void finalize ()
    {
        
    }
    
    public void run( )
    {
        //
        // Run the phases of this execution of icecubes:
        //
        for ( int thePhase = 0; thePhase < phases.length; thePhase++ ) 
        {
            Object[] phaseParms = { phases[thePhase] }; 
            LogMsg phaseStart = new LogMsg("Entering phase {0}.", phaseParms );
            phaseStart.prolog("Common.Tray", 1001, "Phase Start");
            phaseStart.log();	
            phaseStart.flush();	// make sure that this is recorded.
            // run through each of the managers for this phase.
            Iterator it = managerMap.keySet().iterator();
            while ( it.hasNext() ) 
            {
                String managerName = (String) it.next();
                //
                // $FIX$  check for logging of managers at phases policy.
                //
                Object[] callParms = { phases[thePhase] , managerName };
                LogMsg preCall = new LogMsg(" {0} phase, call to manager {1}.", callParms );
                preCall.prolog("Common.Tray", 1010, "Manager Call");
                // log that we are calling the manager. 
                preCall.log();
                // Make sure that it is logged to permanent storage before continuing.
                preCall.flush();
                //
                IceCubeManager manager = (IceCubeManager) managerMap.get( managerName );
                switch ( thePhase ) 
                {
                    // Initialization Phases
                    case PRE_INIT:
                        manager.preInitialize();
                        break;
                    case IN_INIT:
                        manager.initialize();
                        break;
                    case POST_INIT:
                        manager.postInitialize();
                        break;
                        // Running Phases ( only one for now )
                    case RUNNING:
                        //
                        //Launch a thread for each manager in the manager group:
                        //
                        Thread managerThread = new Thread(managerThreads, manager, managerName );
                        manager.setThread(managerThread);
                        managerThread.start();
                        break;
                        // Finalization Phases
                    case PRE_FINAL:
                        manager.preFinalize();
                        break;
                    case IN_FINAL:
                        manager.finalize();
                        break;
                    case POST_FINAL:
                        manager.postFinalize();
                        break;
                        // Disaster Phase -- new phase added without switch.
                    default:
                        break;	
                }
                // log the return from the manager:
                if ( thePhase != RUNNING )
                {
                    LogMsg postCall = new LogMsg(" {0} phase, return from manager {1}.", callParms );
                    postCall.prolog("Common.Tray", 1011, "Manager Return");
                    // Log the message
                    postCall.log();
                    postCall.flush();
                }
                else
                {
                    LogMsg startedThread = new LogMsg(" {0} phase, started thread for manager {1}",  callParms );
                    startedThread.prolog("Common.Tray", 1012, "Manager Thread Start");
                    // Log the message:
                    startedThread.log();
                    startedThread.flush();
                }
            }
            if ( thePhase == RUNNING )
            {
                // wait until all the managers have finished running before executing the
                // next phase.
                //
                // Note - to signal them to terminate use the tray terminate call below!
                //
                // $FIX$ figure out a better way of doing the following:
                // make each manager -- as it terminates, decrement a counter
                //
                while ( managerThreads.activeCount() > 0 )
                {   
                    // let everyone else run
                    Thread.yield();
                    // now sleep a bit
                    try
                    {
                        Thread.sleep( 100 );
                    }
                    catch ( InterruptedException ie ) {}
                }
                Object[] managersDone = { phases[RUNNING] };
                LogMsg postCall = new LogMsg(" {0} phase, completed - no active managers.", managersDone );
                postCall.prolog("Common.Tray", 1013, "Managers Returned");
                // Log the message
                postCall.log();
                postCall.flush();
            }
            LogMsg phaseEnd = new LogMsg("Leaving phase {0}.", phaseParms );
            phaseEnd.prolog("Common.Tray", 1002, "Phase End");
            phaseEnd.log();
            phaseEnd.flush();	// make sure that this is recorded.
        }
	
    }
    
    public static PolicyManager getPolicyManager() { return policyManager; }

    public static void registerManager( String name, Manager manager ) 
    {
        if ( !managerMap.containsKey( name ) )
        {
            managerMap.put( name, manager );
            queueMap.put(name, new LinkedList() );
        }
    }

    public static Manager getManager( String name ) 
    { 
        return (Manager) managerMap.get( name ); 
    }
    
    public static UUID getUUID() { return theUUID; }
    
    
    // $DEPRECATED -- use PayloadFactory.instance() instead!
    public static PayloadFactory getPayloadFactory() { return payloadFactory; }
    

}






