/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/XMLassembly/Attribute.java,v 1.3 2000/12/02 23:31:44 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/
/*----------------------------------------------------------------------------------+
 |                                                                                  |
 |                             Attribute for XMLassembly                            |
 |                                                                                  |
 +----------------------------------------------------------------------------------*/

//$FIX$ -- much work to do here to assure that value's are valid.

package com.adobe.rice.common.XMLassembly;

import java.util.*;
import java.io.*;
import java.text.*;
//$JDK1.1$ import com.sun.java.util.collections.*;


public class Attribute implements Serializable, XMLexpandable
{
    //$DEPENDENCY$ private XMLdoc xmlDoc = null;
    private AttributeDecl attributeDecl = null;
    private String prefix;  			// for name spaces
    private String name;
    private String value;
    private Element element;			// element this attribute is on

    public Attribute()
    {
        attributeDecl = new AttributeDecl();
        name = "";
        value = "";
        element = null;
    }

    public void setValue(String newValue) throws AttributeException
    {
        AttributeType at = this.attributeDecl.getType();
        String msg;
        if ( !at.isValidType(newValue) )
        {
            String errorMessage = Messages.attributeTypeIllegal;
            String listString = "";
            switch ( at.getType() )
            {
            case AttributeType.AT_ENUMERATION:
                errorMessage = Messages.attributeNotInList;
                listString = "Enumeration" + at.toXMLstring();
                break;
            case AttributeType.AT_NOTATION:
                errorMessage = Messages.attributeNotInList;
                listString = "Notation" + at.toXMLstring();
                break;
            default:
                errorMessage = Messages.attributeTypeIllegal;
                listString = at.toXMLstring();
                break;
            }
            Object[] args = { this.name, newValue, listString };
            msg = MessageFormat.format( errorMessage, args );
            AttributeException pBad = new AttributeException( msg );
            throw pBad; 
        }
        switch ((int) attributeDecl.getType().getType())
        {
            case AttributeType.AT_CDATA:
                //
                // The value here can be anything.
                // $CHECK$ make sure that all general entities are appropriately 
                // expanded.
                value = newValue;
                break;
            case AttributeType.AT_ID:
                //
                // $FIX$ look up the value in the id table -- it MUST NOT be there.
                //       and it must have the characteristics of an NMTOKEN
                this.value = newValue;
                break;
            case AttributeType.AT_IDREF:
                //
                // $FIX$ look up the value in the id table -- it MUST be there.
                //   
                this.value = newValue;
                break;
            case AttributeType.AT_IDREFS:
                //
                // $FIX$ look up the list of values in the id table -- each MUST
                //       be there.
                this.value = newValue;
                break;
            case AttributeType.AT_ENTITY:
                //
                // $FIX$ look up the value in the general entity table -- it MUST
                //       be there.
                this.value = newValue;
                break;
            case AttributeType.AT_ENTITIES:
                //
                // $FIX$ break out each entity out of the newValue and look it up 
                //       in the general entity table -- it MUST be there.
                this.value = newValue;
                break;
            case AttributeType.AT_NMTOKEN:
                this.value = newValue;
                break;
            case AttributeType.AT_NMTOKENS:
                this.value = newValue;
                break;
            case AttributeType.AT_NOTATION:
                this.value = newValue;
                break;
            case AttributeType.AT_ENUMERATION:
                this.value = newValue;
                break;
            default:    // IMPOSSIBLE TO GET HERE -- WHY?
                break;
        }
    }

    public void setName(String newName)
    {
    	name = newName;
    }

    public String getValue()
    {
    	return value;
    }

    public String getName()
    {
    	return name;
    }

    public void setElement(Element newElement)
    {
    	element = newElement;
    }


    public String toString()
    {
	    // $FIX$ -- make sure that value is properly quoted.
    	return name + "=\"" + value + "\" ";
    }

    public boolean canDoCharData() { return false; }
    public String toCDSect() { return ""; }
    public String toCharData() { return ""; }

    public String toXMLstring()
    {
	// $FIX$ -- make sure that value is properly quoted.
    	return name + "=\"" + value + "\"";
    }

    public void setAttributeDecl(AttributeDecl newAttributeDecl)
    {
        this.attributeDecl = newAttributeDecl;
    }

    public AttributeDecl getAttributeDecl()
    {
    	return attributeDecl;
    }
}










