/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/XMLassembly/AttributeDecl.java,v 1.3 2000/11/27 20:20:41 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/
/*----------------------------------------------------------------------------------+
 |                                                                                  |
 |                        Attribute Declarations for XMLassembly                    |
 |                                                                                  |
 +----------------------------------------------------------------------------------*/

package com.adobe.rice.common.XMLassembly;

//<class-dependencies>
//  <class> ElementDecl </>
//  <class> AttributeType </>
//  <class> AttributeDefault </>
//  <class> XMLscanner </>
//</class-dependencies>
//
import java.util.*;
import java.io.*;
import java.text.*;
//$JDK1.1$ import com.sun.java.util.collections.*;

public class AttributeDecl implements Serializable, XMLexpandable
{
    private String name;				        // attribute name
    private AttributeType type  ;		       	// attribute type
    private AttributeDefault defaultValue;  	// default value
    private ElementDecl elementDecl;	       	// element it shows up on
    private XMLscanner scan;		       		// XML scanner for attribute

    public AttributeDecl()
    {
        type = new AttributeType();
        defaultValue = new AttributeDefault();
        elementDecl = null;
    }
	
    public AttributeDecl( ElementDecl ed )
    {
        this();
        elementDecl = ed;
    }

    public AttributeDecl( ElementDecl ed, String n )
    {
        this();
        elementDecl = ed;
        name = n;		
    }

    public AttributeDecl( ElementDecl ed, String n, AttributeType t )
    {
        this();
        elementDecl = ed;
        name = n;
        type = t;
    }
    
    
    public AttributeDecl( XMLscanner s )
    {
        this();
        scan = s;
    }
	
    public AttributeDecl( XMLscanner s, ElementDecl ed )
    {
        this();
        scan = s;
        elementDecl = ed;
    }
	
    public AttributeDecl( XMLscanner s, ElementDecl ed, String n )
    {
        this();
        scan = s;
        elementDecl = ed;
        name = n;		
    }
	
    public AttributeDecl( XMLscanner s, ElementDecl ed, String n, AttributeType t )
    {
        this();
        scan = s;
        elementDecl = ed;
        name = n;
        type = t;
    }
    //
    // methods
    //
	
    public String toString()
    {
        // AttDef ::= Name AttType DefaultDecl
        if ( name != null )
            return  name + "= (" + type.toXMLstring() + ") " + defaultValue.toXMLstring();
        return "(declared, unnamed attribute)";
    }

    public boolean canDoCharData() { return false; }
    public String toCDSect() { return ""; }
    public String toCharData() { return ""; }

    public String toXMLstring()
    {
        if ( name != null )
            // AttDef ::= Name AttType DefaultDecl
            return  name + " " + type.toXMLstring() + " " + defaultValue.toXMLstring();
        // $FIX$ - The following will NOT work if we are expanding an attributedecl.
        // We should probably throw an exception at this point so that this rogue attribute
        // error can be squashed.
        return "<!-- declared, unnamed attribute --!>";
    }
    //
    // get and set
    //
    public String getName() { return name; }
    public AttributeType getType() { return type; }
    public AttributeDefault getDefaultValue() { return defaultValue; }
    public ElementDecl getElementDecl() { return elementDecl; }
    public XMLscanner getScan() { return scan; }
    
    public void setName( String n ) { name = n; }
    public void setType( AttributeType t ) { type = t; }
    public void setDefaultValue( AttributeDefault d ) { defaultValue = d; }
    public void setElementDecl( ElementDecl e ) { elementDecl = e; }
    public void setScan( XMLscanner s ) { scan = s; }
    //
    // Recursive Descent parsing routines
    //
    //
    // AttlistDecl ::= '<!ATTLIST' S Name AttDef* S? '>'
    //      AttDef ::= S Name S AttType S DefaultDecl
    //
    
    public int parse( ScanToken t ) throws XMLparseException
    {
    	int count = 0;
    	int gotOne = 0;
    	String msg = null;
    	XMLparseException pBad = null;
    	
    	if ( t.type != ScanToken.NAME )
    	{	// expecting a name
			Object[] args = { "Attribute name", t.string };
			msg = MessageFormat.format( Messages.unexpected, args );
			pBad = new XMLparseException( msg );
			throw pBad; 
    	}
    	name = t.string;
    	// now look for attribute type
    	t = scan.getToken();
    	AttributeType at = new AttributeType( scan );
    	try { gotOne = at.parse( t ); }
    	catch ( XMLparseException p ) { throw p; }
    	type = at;
    	// now look the default value
    	t = scan.getToken();
    	AttributeDefault ad = new AttributeDefault( scan );
    	try { gotOne = ad.parse( t ); }
    	catch ( XMLparseException p ) { throw p; }
    	defaultValue = ad;
    	count++;
    	return count;
    }
    
}






