/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/XMLassembly/AttributeDefault.java,v 1.1.1.1 2000/09/19 19:39:50 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/
/*----------------------------------------------------------------------------------+
 |                                                                                  |
 |                        Attribute Defaults for XMLassembly                        |
 |                                                                                  |
 +----------------------------------------------------------------------------------*/

package com.adobe.rice.common.XMLassembly;

import java.util.*;
import java.io.*;
import java.text.*;
//$JDK1.1$ import com.sun.java.util.collections.*;


public class AttributeDefault
{
    private boolean required = false;
    private boolean implied = false;
    private boolean fixed = false;
    private boolean hasDefaultValue = false;
    private String defaultValue;
    private XMLscanner scan = null;
    
    public AttributeDefault()
    {
    	required = false;
    	implied = false;
    	fixed = false;
    	hasDefaultValue = false;
    	defaultValue = "";
    }
    
    public AttributeDefault( XMLscanner s )
    {
    	this();
    	scan = s;
    }
    
    public boolean getRequired() { return required; }
    public void setRequired( boolean rqd ) { required = rqd; }
    public boolean getImplied() { return implied; }
    public void setImplied( boolean imp ) { implied = imp; }
    public boolean getFixed() { return fixed; }
    public void setFixed( boolean fix ) { fixed = fix; }
    public boolean getHasDefaultValue() {return hasDefaultValue; } 
    public void setHasDefaultValue( boolean hdv ) { hasDefaultValue = hdv; }
    public XMLscanner getScan() { return scan; }
    public void setScan( XMLscanner xs ) { scan = xs; }
    
    public void setDefaultValue( String attributeDefault )
    {
    	defaultValue = attributeDefault;
    	hasDefaultValue = ( defaultValue != null)? true : false;
    }
    
    public String getDefaultValue( )
    {
    	if ( hasDefaultValue )
    		return defaultValue;
    	return "";
    }
    
    public void clearDefaultValue( )
    {
        defaultValue = null;
        hasDefaultValue = false;
    }

    public String toString()
    {
    	return "";
    }

    public boolean canDoCharData() { return false; }
    public String toCDSect() { return ""; }
    public String toCharData() { return ""; }

    public String toXMLstring()
    {
        String ad = "";
        if ( required )
            ad += "#REQUIRED";
        else if ( implied )
            ad += "#IMPLIED" ;
        else if ( hasDefaultValue )
	    {
            if ( fixed )
                ad += "#FIXED ";
            // $FIX$ -- make sure that value is properly quoted.
            ad += "\"" + defaultValue + "\"";
	    }
        return ad;
    }

    //
    // Recursive Descent Parse routines
    //
    // DefaultDecl ::= '#REQUIRED' | '#IMPLIED' | (('#FIXED' S)? AttValue)
    //
    public int parse( ScanToken t ) throws XMLparseException
    {
        int count = 0;
        String msg = null;
        XMLparseException pBad = null;
	
        if ( t.type == ScanToken.QUOTED_STRING )
        {
            defaultValue = t.string;
            hasDefaultValue = true;
            count = 1;
        } 
        else if ( t.string.equals("#") )
        {
            t = scan.getToken();
            if ( t.type == ScanToken.NAME )
            {	
                if ( t.string.equals("REQUIRED") )
                {
                    required = true;
                    count = 1;
                }
                else if ( t.string.equals("IMPLIED") )
                {
                    implied = true;
                    count = 1;
                }
                else if ( t.string.equals("FIXED") )
                {
                    fixed = true;
                    t = scan.getToken();
                    if ( t.type != ScanToken.QUOTED_STRING )
                    {	// must have an attribute value if have #FIXED
                        Object[] args = { "Attribute (quoted) default value", t.toString() };
                        msg = MessageFormat.format( Messages.unexpected, args );
                        pBad = new XMLparseException( msg );
                        throw pBad; 
                    }
                    defaultValue = t.string;
                    hasDefaultValue = true;
                    count = 1;
                }
                else
                {	// must be one of the above
                    Object[] args = { "#REQUIRED, #IMPLIED or #FIXED", t.toString() };
                    msg = MessageFormat.format( Messages.unexpected, args );
                    pBad = new XMLparseException( msg );
                    throw pBad; 
                }
            }
            else
            {	// must be a name
                Object[] args = { "#REQUIRED, #IMPLIED or #FIXED", t.toString() };
                msg = MessageFormat.format( Messages.unexpected, args );
                pBad = new XMLparseException( msg );
                throw pBad; 
            }
        }
        else
        {	// must have a sharp
            Object[] args = { "'#'", t.toString() };
            msg = MessageFormat.format( Messages.unexpected, args );
            pBad = new XMLparseException( msg );
            throw pBad; 
        }
        return count;
    }
}





