/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/XMLassembly/AttributeType.java,v 1.2 2000/12/02 23:31:44 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/
/*---------------------------------------------------------------------+
 |                                                                     |
 |                     Attribute Types for XMLassembly                 |
 |                                                                     |
 +---------------------------------------------------------------------*/

package com.adobe.rice.common.XMLassembly;

import java.util.*;
import java.io.*;
import java.text.*;
//$JDK1.1$ import com.sun.java.util.collections.*;


public class AttributeType
{
    private byte type;

    public final static byte AT_CDATA = 0;
    public final static byte AT_ID = 1;
    public final static byte AT_IDREF = 2;
    public final static byte AT_IDREFS = 3;
    public final static byte AT_ENTITY = 4;
    public final static byte AT_ENTITIES = 5;
    public final static byte AT_NMTOKEN = 6;
    public final static byte AT_NMTOKENS = 7;
    public final static byte AT_NOTATION = 8;
    public final static byte AT_ENUMERATION = 9;
    private final static byte AT_MIN = AT_CDATA;
    private final static byte AT_MAX = AT_ENUMERATION;
    
    private List enumsOrNotas = new ArrayList( Constants.ENUMERATION_COUNT );
    
    private XMLscanner scan;
    
    public AttributeType()
    {
        type = AT_CDATA;
        scan = null;
    }
	
    public AttributeType( XMLscanner s )
    {
        type = AT_CDATA;
        scan = s;
    }
	
    public void setType(byte newType) throws AttributeException
    {
        if (newType >= AT_MIN && newType <= AT_MAX)
        {
            type = newType;
        }
        else
        {
            Byte badByte = new Byte(newType);
            String msg =
              "Out of range XML attribute type (" + badByte.toString() + ")!";
            throw new AttributeException(msg);
        }
    }

    public byte getType()
    {
        return type;
    }
    
    public boolean isInEnumeration( String val )
    {
        if ( this.type != this.AT_ENUMERATION )
            return false;
        return this.enumsOrNotas.contains(val);
    }
    
    public boolean isInNotation( String val )
    {
        if ( this.type != this.AT_NOTATION )
            return false;
        return this.enumsOrNotas.contains(val);
    }
    
    public static boolean isNMTOKEN( String val )
    {
        if ( ( val == null ) || ( val.length() == 0 ) )
            return false;
        for ( int i = 0; i < val.length(); i++ )
        {
            if ( !XMLscanner.isNameChar(val.charAt(i)) )
                return false;
        }
        return true;
        
    }
    
    public static boolean isNMTOKENS( String val )
    {
        if ( ( val == null ) || ( val.length() == 0 ) )
            return false;
        for ( int i = 0; i < val.length(); i++ )
        {
            if ( !XMLscanner.isNameChar(val.charAt(i)) &&
                 !XMLscanner.isWhiteSpace(val.charAt(i)) )
                 return false;
        }
        return true;
    }
    
    public static boolean isID( String val )
    {
        if ( ( val == null ) || ( val.length() == 0 ) )
            return false;
        if ( !XMLscanner.isNameStart(val.charAt(0)) )
            return false;
        for ( int i = 0; i < val.length(); i++ )
        {
            if ( !XMLscanner.isNameChar(val.charAt(i)) )
                return false;
        }
        //
        return true;
    }
    
    public static boolean isIDS( String val )
    {
        if ( ( val == null ) || ( val.length() == 0 ) )
            return false;
        boolean inWhite = false;
        for ( int i = 0; i < val.length(); i++ )
        {
            if ( XMLscanner.isWhiteSpace(val.charAt(i)) )
                inWhite = true;
            else if ( XMLscanner.isNameChar(val.charAt(i)) )
            {
                if ( inWhite && !XMLscanner.isNameStart(val.charAt(i)) )
                    return false;
                inWhite = false;
            }
            else
                return false;
        }
        return true;
    }
    
    
    public boolean isValidType( String val )
    {
        switch ( this.type )
        {
            case AttributeType.AT_CDATA:
                //
                // The value here can be anything.
                return true;
            case AttributeType.AT_ID:
                return this.isID(val);
            case AttributeType.AT_IDREF:
                return this.isID(val);
            case AttributeType.AT_IDREFS:
                return this.isIDS(val);
            case AttributeType.AT_ENTITY:
                return this.isID(val);
            case AttributeType.AT_ENTITIES:
                return this.isIDS(val);
            case AttributeType.AT_NMTOKEN:
                return this.isNMTOKEN( val );
            case AttributeType.AT_NMTOKENS:
                return this.isNMTOKENS( val );
            case AttributeType.AT_NOTATION:
                return this.isInNotation( val );
            case AttributeType.AT_ENUMERATION:
                return this.isInEnumeration( val );
            default:
                //
                return false;
        }
    }
    
    //
    // Recursive Descent Parsing methods
    //
    // AttType       ::=  StringType | TokenizedType | EnumeratedType 
    // StringType    ::=  'CDATA'
    // TokenizedType ::=  'ID' | 'IDREF' | 'IDREFS' | 'ENTITY' | 'ENTITIES'
    //                |   'NMTOKEN' | 'NMTOKENS'
    // EnumeratedType ::= NotationType | Enumeration 
    // NotationType   ::= 'NOTATION' S '(' S? Name (S? '|' S? Name)* S? ')'
    // Enumeration    ::= '(' S? Nmtoken (S? '|' S? Nmtoken)* S? ')'
    //
    private class AttStrType
    {
        String	name;
        byte	type;
	
        AttStrType( )
        {
            name = "";
            type = 0;
        }
	
        AttStrType( String n, byte t )
        {
            name = n;
            type = t;
        }
    };
	
    private static AttStrType[] astMap = new AttStrType[10];
    {
        AttributeType.astMap[0] = new AttStrType( "CDATA", AT_CDATA );
        AttributeType.astMap[1] = new AttStrType( "ID", AT_ID );
        AttributeType.astMap[2] = new AttStrType( "IDREF", AT_IDREF );
        AttributeType.astMap[3] = new AttStrType( "IDREFS", AT_IDREFS );
        AttributeType.astMap[4] = new AttStrType( "ENTITY", AT_ENTITY );
        AttributeType.astMap[5] = new AttStrType( "ENTITIES", AT_ENTITIES );
        AttributeType.astMap[6] = new AttStrType( "NMTOKEN", AT_NMTOKEN );
        AttributeType.astMap[7] = new AttStrType( "NMTOKENS", AT_NMTOKENS );
        AttributeType.astMap[8] = new AttStrType( "NOTATION", AT_NOTATION );
        AttributeType.astMap[9] = new AttStrType( "(", AT_ENUMERATION );
    }

    public boolean canDoCharData() { return false; }
    public String toCDSect() { return ""; }
    public String toCharData() { return ""; }

    public String toXMLstring()
    {
        // AttType  ::=  StringType | TokenizedType  | EnumeratedType 
        // StringType ::= 'CDATA'
        // TokenizedType ::=  'ID' | 'IDREF'  | 'IDREFS'  | 'ENTITY' | 'ENTITIES' | 'NMTOKEN' | 'NMTOKENS'

        String at = "";
        switch ( type )
        {
            case AT_CDATA:
            case AT_ID:
            case AT_IDREF:
            case AT_IDREFS:
            case AT_ENTITY:
            case AT_ENTITIES:
            case AT_NMTOKEN:
            case AT_NMTOKENS:
                at =  AttributeType.astMap[type].name;
                break;
                // enumerated types
            case AT_NOTATION:
            case AT_ENUMERATION:
                if ( type == AT_NOTATION )
                    at += "NOTATION ";
                at += "( ";
                for ( int i = 0; i < enumsOrNotas.size(); i++ )
                {
                    at += enumsOrNotas.get(i) + " ";
                    if ( i+1 < enumsOrNotas.size() )
                        at += "| ";
                }
                at += ")";
                break;
            default:
                at += "<!-- $FIX$ - Logic error in Attribute_Type -- unknown type -->";
                break;
        }
        return at;
    }

    public int parse( ScanToken t ) throws XMLparseException
    {
    	int count = 0;
    	int i = 0;			/* index value */
    	XMLparseException pBad = null;
    	String msg = null;
    	
    	if ( t.type == ScanToken.NAME )
    	{
            for ( i = 0; i < astMap.length ; i++ )
    	    {
                if ( t.string.equals( astMap[i].name ) )
                {
                    this.type = astMap[i].type;
                    count = 1;
                    break;
                }
            }
            if ( i >= astMap.length )
    	    {	// didn't find an attribute type -- parse error
                Object[] args = { "Attribute type", t.toString() };
                msg = MessageFormat.format( Messages.unexpected, args );
                pBad = new XMLparseException( msg );
                throw pBad; 
            }
            // NotationType   ::= 'NOTATION' S '(' S? Name (S? '|' S? Name)* S? ')'
            if ( this.type == AT_NOTATION )
    	    {
                t = scan.getToken();
                if ( !t.string.equals( "(" ) )
                {
                    Object[] args = { "Notation '('", t.toString() };
                    msg = MessageFormat.format( Messages.unexpected, args );
                    pBad = new XMLparseException( msg );
                    throw pBad; 
                }
                do 
                {
                    t = scan.getToken();
                    if ( t.type != ScanToken.NAME )
                    {	// must be a name or parse error
                        Object[] args = { "Notation name", t.toString() };
                        msg = MessageFormat.format( Messages.unexpected, args );
                        pBad = new XMLparseException( msg );
                        throw pBad; 
                    }
                    enumsOrNotas.add( t.string );
                    t = scan.getToken();
                    count++;
                }
                while ( t.string.equals("|") );
                if ( !t.string.equals(")") )
                {	// must be a closing paren or parse error
                    Object[] args = { "Notation ')'", t.toString() };
                    msg = MessageFormat.format( Messages.unexpected, args );
                    pBad = new XMLparseException( msg );
                    throw pBad; 
                }
            }
    	}	
    	else if ( t.string.equals("(") )
    	{	// it is an enumeration
            do 
            {
                t = scan.getToken();
                if ( (t.type != ScanToken.NAME_TOKEN) && (t.type != ScanToken.NAME) )
                {	// must be name or name token or it is a parse error
                    Object[] args = { "Enumeration name token", t.toString() };
                    msg = MessageFormat.format( Messages.unexpected, args );
                    pBad = new XMLparseException( msg );
                    throw pBad; 
                }
                enumsOrNotas.add( t.string );
                t = scan.getToken();
                count++;
            }
            while ( t.string.equals("|") );
            if ( !t.string.equals(")") )
            {	// must be a closing paren or parse error
                Object[] args = { "Enumeration ')'", t.toString() };
                msg = MessageFormat.format( Messages.unexpected, args );
                pBad = new XMLparseException( msg );
                throw pBad; 
            }
            type = AT_ENUMERATION;
     	}    	
    	return count;
    }
}




