/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/XMLassembly/Children.java,v 1.1.1.1 2000/09/19 19:39:50 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/
/*----------------------------------------------------------------------------------+
 |                                                                                  |
 |                       Childern Content Model for XMLassembly                     |
 |                                                                                  |
 +----------------------------------------------------------------------------------*/

package com.adobe.rice.common.XMLassembly;

import java.io.*;
import java.text.*;
import java.util.*;
//$JDK1.1$ import com.sun.java.util.collections.*;
import com.adobe.rice.common.Parsergen.*;

/*----------------------------------------------------------------------------------+
 |                                                                                  |
 |                           Children Element Content Class                         |
 |                                                                                  |
 +----------------------------------------------------------------------------------*/

/**
 * Children.java
 *
 *
 * Created: Tue Oct 19 08:50:13 1999
 *
 * @author V. Bruce Hunt
 * @version
 */

/**
   children	 ::=  (choice | seq) ('?' | '*' | '+')?
   cp            ::=  (Name | choice | seq) ('?' | '*' | '+')?
   choice        ::=  '(' S? cp ( S? '|' S? cp )* S? ')' [ VC: Proper Group/PE Nesting ]
   seq           ::=  '(' S? cp ( S? ',' S? cp )* S? ')' [ VC: Proper Group/PE Nesting ]

**/

public class Children extends ChoiceSeq implements XMLexpandable 
{

    public Children() 
    {
        super();
    }

    public Children( byte t )
    {
        this();
        if ( t >= CP_CHOICE && t <= CP_SEQ )
            type = t;
        else
            type = CP_CHOICE;
    }

    public Children( ElementDecl ed )
    {
        this();
        container = ed;
    }

    public Children( byte t, ElementDecl ed )
    {
        this();
        if ( t >= CP_CHOICE && t <= CP_SEQ )
            type = t;
        else
            type = CP_CHOICE;
	
        parent = null;
        container = ed;
    }
    //
    // Recursive descent parsing routines
    //
    //
    // contentspec ::= 'EMPTY' | 'ANY' | Mixed | children 
    //
    // Mixed	::= '(' S? '#PCDATA' ( S? '|' S? Name)*  S? ')*' 
    //         |  '(' S? '#PCDATA' S? ')' 
    //
    // children ::= (choice | seq) ('?' | '*' | '+')?
    //
    // choice ::= '(' S? cp ( S? '|' S? cp )* S? ')'
    //
    // seq    ::= '(' S? cp ( S? ',' S? cp )* S? ')'
    //
    // cp     ::= (Name | choice | seq) ('?' | '*' | '+')?
    //
    //
    // parse enters with t being the first "(" and 
    // parse leaves with t at the token after the trailing ")"
    public int parse( ScanToken t ) throws XMLparseException
    {
    	int count = 0;
    	int gotOne = 0;
    	String msg = null;
    	XMLparseException pBad = null;
    	XMLscanner scan = container.getScan();
    	
        if ( t.string.equals("(") )
    	{   // go find an choice or a seq 
            ChoiceSeq chse = new ChoiceSeq( (Cp) this, container );
            chse.setIsPossibleMixed( true );
            try { gotOne = chse.parse( t ); }
            catch ( XMLparseException p ) { throw p; }
            count = 1;
            // set the child
            child = chse;
    	}
    	else
    	{
            Object[] args0 = { "ANY, EMPTY, Mixed or children", t.toString() }; 
            msg = MessageFormat.format( Messages.unexpected, args0 );
            pBad = new XMLparseException( msg );
            throw pBad; 
    	}
    	// now look for the repetitions
    	if ( t.type == ScanToken.SPECIAL )
    	{
            switch ( t.string.charAt(0) )
    	    {
                case '?':
                    repetition = REP_ZERO_OR_ONE;
                    t = scan.getToken();
                    break;
                case '*':
                    repetition = REP_ZERO_OR_MORE;
                    t = scan.getToken();
                    break;
                case '+':
                    repetition = REP_ONE_OR_MORE;
                    t = scan.getToken();
                    break;
                default:
                    repetition = REP_ONCE;
                    break;
            }
    	}
    	else
    	{
            repetition = REP_ONCE;
    	}
    	return count;
    }
    

    public String toXMLstring()
    {
        String ch = "";
        if ( child != null )
        {
            ch += child.toXMLstring();
            switch( type )
            {
                case REP_ZERO_OR_MORE:
                    ch += "*";
                    break;
                case REP_ONE_OR_MORE:
                    ch += "+";
                    break;
                case REP_ZERO_OR_ONE:
                    ch += "?";
                    break;
                case REP_ONCE:
                    break;
            }
        }
        return ch;
    }
    /**
       @method genGrammar
	
       This method canonicalizes Children by constructing a left recursive 
       grammar for the children by replacing all repetition operators with
       their (left recursive) grammar equivalents.  This is basically a 
       recursive scan through the children tree...
	
    **/
	
    public void genGrammar( Grammar g )
    {
        // now construct the production representation.
        Production chpr = null; 
        if ( child != null )
        {
            // generate lower level productions first to eliminate recursion problems.
            child.genGrammar( g );
            switch( repetition )
            {
                case REP_ZERO_OR_MORE:
                    // children -> EMPTY
                    chpr = new Production( this.id );
                    chpr.add( Symbol.empty );
                    g.add( chpr );
                    // children -> child
                    chpr = new Production( this.id );
                    chpr.add( child.id );
                    g.add( chpr );
                    // children -> children child
                    chpr = new Production( this.id );
                    chpr.add( this.id );
                    chpr.add( child.id );
                    g.add( chpr );
                    break;
                case REP_ONE_OR_MORE:   // children -> child+
                    // children -> child
                    chpr = new Production( this.id );
                    chpr.add( child.id );
                    g.add( chpr );
                    // children -> children child
                    chpr = new Production( this.id );
                    chpr.add( this.id );
                    chpr.add( child.id );
                    g.add( chpr );
                    break;
                case REP_ZERO_OR_ONE:	// children -> child?
                    // children -> EMPTY
                    chpr = new Production( this.id );
                    chpr.add( Symbol.empty );
                    g.add( chpr );
                    // children -> child
                    chpr = new Production( this.id );
                    chpr.add( child.id );
                    g.add( chpr );
                    break;
                case REP_ONCE:		// children -> child
                    // children -> child
                    chpr = new Production( this.id );
                    chpr.add( child.id );
                    g.add( chpr );
                    break;
                default:
                    // This is very bad --> Content model is messed up big time.
                    break;	
            }
        }
        else
        {
            chpr = new Production( this.id );
            chpr.add( Symbol.empty );
        }
    }
} // Children
























