/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/XMLassembly/ChoiceSeq.java,v 1.1.1.1 2000/09/19 19:39:50 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.XMLassembly;

import java.io.*;
import java.text.*;
import java.util.*;
//$JDK1.1$ import com.sun.java.util.collections.*;
import com.adobe.rice.common.Parsergen.*;

/*----------------------------------------------------------------------------------+
 |                                                                                  |
 |                           Choice Element Content Class                           |
 |                                                                                  |
 +----------------------------------------------------------------------------------*/

/**
 * ChoiceSeq.java
 *
 *
 * Created: Tue Oct 19 09:02:47 1999
 *
 * @author V. Bruce Hunt
 * @version
 */

public class ChoiceSeq extends Cp implements XMLexpandable, Serializable 
{
    
    public ChoiceSeq( ) 
    {
        super();
        items = new ArrayList( Constants.CP_COUNT );
    }

    public ChoiceSeq( Cp p, ElementDecl ed )
    {
        super( p, ed );
        items = new ArrayList( Constants.CP_COUNT );
    }

    public void setItems( List l ) { items = l; }
    public List getItems() { return items; }
    public Object getItem( int i ) { return items.get(i); }
    public int getItemsSize() { return items.size(); }
    public boolean getIsPossibleMixed( ) { return isPossibleMixed; }
    public void setIsPossibleMixed( boolean isM ) { isPossibleMixed = isM; }

    //
    // Recursive descent parsing routines
    //
    //
    // contentspec ::= 'EMPTY' | 'ANY' | Mixed | children 
    //
    // Mixed	::= '(' S? '#PCDATA' ( S? '|' S? Name)*  S? ')*' 
    //         |  '(' S? '#PCDATA' S? ')' 
    //
    // children ::= (choice | seq) ('?' | '*' | '+')?
    // choice ::= '(' S? cp ( S? '|' S? cp )* S? ')'
    // seq    ::= '(' S? cp ( S? ',' S? cp )* S? ')'
    // cp     ::= (Name | choice | seq) ('?' | '*' | '+')?
    //
    // upon entry, t must be the entering "(".
    // upon exit, t points at the token after the closing ")".
    //
    public int parse( ScanToken t ) throws XMLparseException
    {
        int count = 0;
        int gotOne = 0;
        String msg = null;
        XMLparseException pBad = null;
        byte mcsType = -1;			// keeps track of choice, mixed or seq
        String op = "", newOp=")";	// contains the first and latest operator
        XMLscanner scan = container.getScan();
        DocType dtd = container.getDtd();	// basis of the scan.

        // upon entry, token t must contain "(".
		if ( !t.string.equals("(") )
        {
            Object[] args = { "(", t.toString() };
            msg = MessageFormat.format( Messages.unexpected, args );
            pBad = new XMLparseException( msg );
            throw pBad; 
        }
        // get the first item ...
        t = scan.getToken();
        if ( isPossibleMixed )
        {
            // first character must be a # if it is a mixed; look for #PCDATA to signal a Mixed
            // scan: { "#PCDATA" ( "|" name )* ( ")" | ")*" ) }
            if ( t.string.equals("#") )
            {
                // must be Mixed with possible #PCDATA
                t = scan.getToken();
                // $FIX$ note that there could be white space between '#' and 'PCDATA'
                if ( !t.string.equals("PCDATA") )
                {   // bad, must be #PCDATA
                    Object[] args = { "#PCDATA", t.toString() };
                    msg = MessageFormat.format( Messages.unexpected, args );
                    pBad = new XMLparseException( msg );
                    throw pBad; 
                }
                // Don't save the #PCDATA since it's value is recorded 
                // in MCS_MIXED.
                // hmm -- will this cause construction to fail since we may not notice the 
                // possibility to allow parsed character data later?  $CHECK$
                mcsType = CP_MIXED;
                count++;
                t = scan.getToken();
                // token must be '|' or ')'
                if ( t.string.equals(")") )
                {
                    // point at next token
                    t = scan.getToken(); 
                    type = mcsType;
                    return count;
                }
                if ( !t.string.equals("|") )
                {
                    Object[] args = { "'|' or ')'", t.toString() };
                    msg = MessageFormat.format( Messages.unexpected, args );
                    pBad = new XMLparseException( msg );
                    throw pBad; 
                }
                // scan ( '|' name )+ ')*'
                while ( t.string.equals("|") )
                {
                    t = scan.getToken();
                    if ( t.type != ScanToken.NAME )
                    {
                        Object[] args = { "Element name", t.toString() };
                        msg = MessageFormat.format( Messages.unexpected, args );
                        pBad = new XMLparseException( msg );
                        throw pBad; 
                    }
                    // look up the name in the Document's ElementDecl list.
                    ElementDecl ned = dtd.getElementDecl( t.string );
                    if ( ned == null )
                    {
                        // it is NOT an XML error to have an undeclared element name. 
                        ned = new ElementDecl( dtd, t.string );
                        ned.setIsUndeclared( true );
                    }
                    items.add( new Cp( Cp.CP_NAME, this, ned ) );
                    count++;
                    t = scan.getToken();
                }
                if ( t.string.equals(")") )
                {
                    t = scan.getToken();
                    if ( t.string.equals("*") )
                    {
                        type = mcsType;
                        t = scan.getToken();	// point at next token.
                        return count;
                    }
                }
                Object[] args = { "closing ')*' for Mixed expression", t.toString() };
                msg = MessageFormat.format( Messages.unexpected, args );
                pBad = new XMLparseException( msg );
                throw pBad; 
            }  	
        }
        // scan:  { ( ( cp '|' )+ ) | ( ( cp ',' )+ ) ')' } 
        Cp mcsCp = new Cp( ContentSpec.CP_EMPTY, this, container );
        do
        {   // go try to get a Cp
            try { gotOne = mcsCp.parse( t ); } 
            catch ( XMLparseException p ) { throw p; };
            if ( gotOne > 0 )
            {
                items.add( mcsCp );
                count++;
                // On first pass, set choice (|) or sequence (,) based on operator:
                if ( mcsType == -1 )
                {
                    switch( t.string.charAt(0) )
                    {
                    case '|':
                        mcsType = CP_CHOICE;
                        op = "|";
                        break;
                    case ',':
                        mcsType = CP_SEQ;
                        op = ",";
                        break;
                    case ')':
                        mcsType = CP_SEQ; // only occurs if we have { "(" cp ")" }
                        op = ",";
                        break;
                    default:
                        Object[] args = { "'|', ',' or ')'", t.toString() };
                        msg = MessageFormat.format( Messages.unexpected, args );
                        pBad = new XMLparseException( msg );
                        throw pBad; 
                    }
                }
                newOp = t.string;
                if ( newOp.equals(op) )
                {
                    mcsCp = new Cp( ContentSpec.CP_EMPTY, this, container );
                    t = scan.getToken();
                }
            }
        }
        while ( (gotOne > 0) && newOp.equals(op) );

        if ( !t.string.equals(")") )
        {
            Object[] args = { "')' closing choice or sequence", t.toString() };
            msg = MessageFormat.format( Messages.unexpected, args );
            pBad = new XMLparseException( msg );
            throw pBad; 
        }
        t = scan.getToken(); // point past the closing right paren.
        type = mcsType;
        return count;
    }

    //
    // implements XMLexpandable
    //
    public boolean canDoCharData() { return false; }
    public String toCDSect() { return ""; }
    public String toCharData() { return ""; }

    public String toXMLstring()
    {
        String scs = "(";
        if ( type == CP_MIXED )
            scs += " #PCDATA";
        int itemsSize = items.size();
        for ( int i = 0; i < itemsSize; i++ )
        {
            Cp cscp = (Cp) items.get( i );
            scs += cscp.toXMLstring();
            if ( (i+1) < itemsSize )
                scs += ",";
        }
        scs += ")";
        return scs;
    }

    //
    // generate a left recursive grammar 
    // 
    //	This routine handles the following cases in generating a grammar for 
    //  the content model of this container element.
    /**
       Consider the content specification for an element e:
       e -> ( e1 , ( e2 | e3 | e4 )+ , ( e5 | e6 )* , e7 )
	
       It is represented internally as: 
	
       ContentSpec -> Children
       Children -> seq[ Cp0, Cp1, Cp3, Cp4 ]:1  <- repetitions - can be 1, ?, + or *
                         |    |    |    |
                         |    |    |    +-> e7
                         |    |    +-> choice[ Cp31, Cp32 ]:*
                         |    |                 |     |
                         |    |                 |     +-> e6
                         |    |                 +-> e5
                         |    +-> choice[ Cp11, Cp12, Cp13 ]:1
                         |                 |     |     |
                         |                 |     |     +-> e4
                         |                 |     +-> e3
                         |                 +->e2
                         +->e1

	The general form for a content specification is:
	                        
        ContentSpec -> choice[ ANY, EMPTY, Mixed, Children ]:1
                                             |       |
                  +--<-----<------<-----<----+       |
                  |       +--<-----<------<-----<----+
                  |       |
                  |       +--> selector[ cp0, cp1, ..., cpi, ..., cpn ]: rep
                  +---> choice[ #PCDATA, cp0, cp1, ..., cpi, ..., cpn ]: [1,*]
       
        ANY -- allow any mixture of PCDATA and elements (and don't validate)
        EMPTY -- no content is allowed; tag can be <name attributes /> or <name attributes></name>
        Mixed -- allow mixed PCDATA and any sequence of only the elements cp0 through cpn.
                 Note that for each i, cpi must be an element.
        Children -- follow the grammar specified (as in XML 1.0 spec.); 
	      -- the initial selector for children is either choice or seq;
	      -- And where each cpi can be a choice, sequence or element;
	      -- And where selector -> element | choice | seq ;
              -- element means that the Cp is an element with a repetition treated as if it is 1.
	      -- choice means that there is a list of Cp alternatives with repetition rep;
	      -- seq means that there is a list of Cp in sequence with repetition rep;
              -- And, where rep -> 1 | ? | + | *
	      -- 1 means one repetition.
	      -- ? means zero or one repetitions
	      -- * means zero or more repetitions
	      -- + means one or more repetitions.

	The purpose of the routine is to generate a grammar that replaces repetition operators 
	with left recursion.
	
	The cases to consider are:
	
	A.  cpe -> element[ e0 ]:rep 
		                 
	B.  cpe -> choice[ c0, c1, ..., ci, ..., ck ]:rep 
	
	    Note that cpe -> mixed[ c0, c1, ..., ci, ..., ck ]:{1|*} is a special case of B in 
	    which rep is limited to 1 or *; and c0 is the element string  "#PCDATA".
	
	C.  cpe -> seq[ c0, c1, ..., ci, ..., cn ]:rep 

	So Consider in turn, A, B and C (expanding rep ):
	
	A.  Consider the cases, cpe -> element[ e0 ]: rep : 

        1.  cpe -> element[ e0 ]:1	This generates cpe -> e0.
        2.  cpe -> element[ e0 ]:?  This generates ( cpe-> e0 | empty )
        3.  cpe -> element[ e0 ]:+ 	This generates ( cpe-> e0 | cpe e0 )
        4.  cpe -> element[ e0 ]:*  This generates ( cpe -> e0 | cpe e0 | empty )

    B.  Consider the cases, choice[ c0, c1, ..., ci, ..., ck ]: rep : 

        1.  cpe -> choice[ c0, c1, ..., ci, ..., ck ]: 1 
            This generates cpe -> c0 | c1 | ... | ci | ... | ck
        2.  cpe -> choice[ c0, c1, ..., ci, ..., ck ]: ?
            This generates: cpe -> empty | c0 | c1 | ... | ci | ... | ck
        3.  cpe -> choice[ c0, c1, ..., ci, ..., ck ]: +
            This generates: cpe -> cp1, cpe -> cpe cp1 and
            cp1 -> c0 | c1 | ... | ci | ... | ck
        4.  cpe -> choice[ c0, c1, ..., ci, ..., ck ]: *
            This generates: 
            cpe -> empty | cp1 | cpe cp1
            cp1 -> c0 | c1 | ... | ci | ... | ck
		                     
	C.  Consider the cases, seq[ c0, c1, ..., ci, ..., cn ]: rep :
	
        1.  cpe -> seq[ c0, c1, ..., ci, ..., cn ]:1
            This generates: cpe -> c0 c1 ... ci ... cn		
        2.  cpe -> seq[ c0, c1, ..., ci, ..., cn ]:?
	        This generates:
            cpe -> empty | c0 c1 ... ci ... cn
        3.  cpe -> seq[ c0, c1, ..., ci, ..., cn ]:+
            This generates:
            cp1 -> c0 c1 ... ci ... cn
            cpe -> cp1 | cpe cp1
        4.  cpe -> seq[ c0, c1, ..., ci, ..., cn ]:*
            This generates:
            cp1 -> c0 c1 ... ci ... cn
            cpe -> empty | cp1 | cpe cp1

    */
    
    public void genGrammar( Grammar g )
    {
        // now construct the production representation.
        Production csp = null;
        // run through and generate any lower level productions
        for ( int i = 0; i < items.size(); i++ )
        {
            Cp cpi = (Cp) items.get(i);
            switch( cpi.getType() )
            {
                case CP_CHOICE:
                case CP_SEQ:
                case CP_MIXED:
                    cpi.genGrammar( g );
                    break;
                default:
                    break;
            }
        }
        // now generate this level's productions
        switch( type )
        {
            case CP_MIXED:	 // id -> ( #PCDATA | C0 | C1 | ... | ck )*
                // id -> #PCDATA
                if ( items.size() > 0 )
                {
                    // id -> EMPTY
                    this.id.setNt( LRconst.NONTERMINAL );
                    csp = new Production( this.id );
                    csp.add( Symbol.empty );
                    g.add( csp );
                    // id -> p1
                    Symbol p1 = new Symbol( id.get()+"1", LRconst.NONTERMINAL );
                    csp = new Production( p1 );
                    csp.add( this.id );
                    g.add( csp );
                    // p1 -> #PCDATA
                    csp = new Production( p1 );
                    csp.add( new Symbol( "#PCDATA", LRconst.TERMINAL ) );
                    g.add( csp );
                    // p1 -> ( C0 | C1 | C2 ... | Ck )
                    // for each i:  ( id -> Ci | id -> id Ci )
                    for ( int i = 0; i < items.size() ; i++ )
                    {   // id -> Ci
                        csp = new Production( p1 );
                        Cp cpi  = (Cp) items.get(i);

                        csp.add( cpi.getId() );
                        g.add( csp );
                    }
                    // id -> id p1				!! RECURSION !!
                    csp = new Production( this.id );
                    csp.add( this.id );
                    csp.add( p1 );
                    g.add( csp );
                }
                else	// id -> #PCDATA	(#PCDATA is a terminal)
                {
                    csp = new Production( this.id );
                    csp.add( new Symbol( "#PCDATA", LRconst.TERMINAL ) );
                    g.add( csp );
                }
                break;
            case CP_CHOICE:  // id -> ( C0 | C1 | ... | Ck ) [ + | * | ? ]
                switch ( repetition )
                {
                    case REP_ZERO_OR_MORE:	// id -> ( C0 | C1 | ... | ck ) *
                        // id -> EMPTY
                        csp = new Production( this.id );
                        csp.add( Symbol.empty );
                        g.add( csp );
                        // id -> p1
                        Symbol p1 = new Symbol( id.get()+"1", LRconst.NONTERMINAL );
                        csp = new Production( p1 );
                        csp.add( this.id );
                        g.add( csp );
                        // p1 -> ( C0 | C1 | C2 ... | Ck )
                        // for each i:  ( id -> Ci | id -> id Ci )
                        for ( int i = 0; i < items.size() ; i ++ )
                        {   // id -> Ci
                            csp = new Production( p1 );
                            Cp cpi  = (Cp) items.get(i);
                            csp.add( cpi.getId() );
                            g.add( csp );
                        }
                        // id -> id p1				!! RECURSION !!
                        csp = new Production( this.id );
                        csp.add( this.id );
                        csp.add( p1 );
                        g.add( csp );
                        break;
                    case REP_ONE_OR_MORE:  // id -> ( C0 | C1 | ... | Ck ) +
                        // id -> p1
                        p1 = new Symbol( id.get()+"1", LRconst.NONTERMINAL );
                        csp = new Production( p1 );
                        csp.add( this.id );
                        g.add( csp );
                        // p1 -> ( C0 | C1 | C2 ... | Ck )
                        // for each i:  ( id -> Ci | id -> id Ci )
                        for ( int i = 0; i < items.size() ; i ++ )
                        {   // id -> Ci
                            csp = new Production( p1 );
                            Cp cpi  = (Cp) items.get(i);
                            csp.add( cpi.getId() );
                            g.add( csp );
                        }
                        // id -> id p1				!! RECURSION !!
                        csp = new Production( this.id );
                        csp.add( this.id );
                        csp.add( p1 );
                        g.add( csp );
                        break;
                    case REP_ZERO_OR_ONE:
                        // id -> EMPTY
                        csp = new Production( this.id );
                        csp.add( Symbol.empty );
                        g.add( csp );
                        // id -> ( C0 | C1 | C2 ... | Ck )
                        for ( int i = 0; i < items.size() ; i ++ )
                        {   // id -> Ci
                            csp = new Production( this.id );
                            Cp cpi  = (Cp) items.get(i);
                            csp.add( cpi.getId() );
                            g.add( csp );
                        }
                        break;
                    case REP_ONCE:
                        // id -> ( C0 | C1 | C2 ... | Ck )
                        for ( int i = 0; i < items.size() ; i ++ )
                        {   // id -> Ci
                            csp = new Production( this.id );
                            Cp cpi  = (Cp) items.get(i);
                            csp.add( cpi.getId() );
                            g.add( csp );
                        }
                        break;
                    default:
                        // $FIX$ something is broken!
                        break;
                }	    
                break;
            case CP_SEQ:	 // id -> ( C0 C1 ... Ck ) [ + | * | ? ]
                switch ( repetition )
                {
                    case REP_ZERO_OR_MORE:	// id -> ( C0 C1 ... Ck )*
                        // id -> EMPTY
                        csp = new Production( this.id );
                        csp.add( Symbol.empty );
                        g.add( csp );
                        // p1 -> C0 C1 ... Ck
                        Symbol p1 = new Symbol( id.get()+"1", LRconst.NONTERMINAL );
                        csp = new Production( p1 );
                        for ( int i = 0; i < items.size(); i++ )
                        {
                            Cp cpi = (Cp) items.get(i);
                            csp.add( cpi.getId() );
                        }
                        g.add( csp );
                        // id -> p1
                        csp = new Production( this.id );
                        csp.add( p1 );
                        g.add( csp );
                        // id -> id p1				     !RECURSION!
                        csp = new Production( this.id );
                        csp.add( this.id );
                        csp.add( p1 );
                        g.add( csp );
                        break;
                    case REP_ONE_OR_MORE:
                        // p1 -> C0 C1 ... Ck
                        p1 = new Symbol( id.get()+"1", LRconst.NONTERMINAL );
                        csp = new Production( p1 );
                        for ( int i = 0; i < items.size(); i++ )
                        {
                            Cp cpi = (Cp) items.get(i);
                            csp.add( cpi.getId() );
                        }
                        g.add( csp );
                        // id -> p1
                        csp = new Production( this.id );
                        csp.add( p1 );
                        g.add( csp );
                        // id -> id p1
                        csp = new Production( this.id );
                        csp.add( this.id );
                        csp.add( p1 );
                        g.add( csp );
                        break;
                    case REP_ZERO_OR_ONE:
                        // id -> EMPTY
                        csp = new Production( this.id );
                        csp.add( Symbol.empty );
                        g.add( csp );
                        // id -> C0 C1 ... Ck
                        csp = new Production( this.id );
                        for ( int i = 0; i < items.size(); i++ )
                        {
                            Cp cpi = (Cp) items.get(i);
                            csp.add( cpi.getId() );
                        }
                        g.add( csp );
                        break;
                    case REP_ONCE:
                        // id -> C0 C1 ... Ck
                        csp = new Production( this.id );
                        for ( int i = 0; i < items.size(); i++ )
                        {
                            Cp cpi = (Cp) items.get(i);
                            csp.add( cpi.getId() );
                        }
                        g.add( csp );
                        break;
                    default:
                        // $FIX$ something is broken if ever get here.
                        break;
                }
                break;
            default:
                // very bad
                break;
        }
    }
} // ChoiceSeq



























