/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/XMLassembly/Constants.java,v 1.1.1.1 2000/09/19 19:39:50 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/
/*----------------------------------------------------------------------------------+
 |                                                                                  |
 |                             Constants for XMLassembly                            |
 |                                                                                  |
 +----------------------------------------------------------------------------------*/

package com.adobe.rice.common.XMLassembly;

final public class Constants
{
    // defining class for package wide constants
    //
    //
    // define the default XML version from the following choice(s)
    //      {There is only one option.}
    final static String XML_VERSION_1_0 = "1.0";
    //
    //  DEFAULT XML VERSION
    //
    final static String XML_VERSION = XML_VERSION_1_0;
    //
    // define the default encoding from the following choices
    //
    final static String XML_ENCODING_UTF16 = "UTF-16";
    final static String XML_ENCODING_UTF8 = "UTF-8";
    //
    // Note that Java supports only the UTF-8 (reading/writing) and 
    // UTF-16 encoding (internal) -- That is UTF-8 is converted to 
    // UTF-16 on reading and UTF-16 is converted to UTF-8 on writing when
    // UTF-8 is the selected encoding.
    //
    final static String XML_ENCODING_UCS2 = "ISO-10646-UCS-2";
    final static String XML_ENCODING_UCS4 = "ISO-10646-UCS-4";
    //
    // The following encodings are supported only in-so-far as they are represented
    // in UTF-16.
    //
    final static String XML_ENCODING_8859_1 = "ISO-8859-1";
    final static String XML_ENCODING_8859_2 = "ISO-8859-2";
    final static String XML_ENCODING_8859_3 = "ISO-8859-3";
    final static String XML_ENCODING_8859_4 = "ISO-8859-4";
    final static String XML_ENCODING_8859_5 = "ISO-8859-5";
    final static String XML_ENCODING_8859_6 = "ISO-8859-6";
    final static String XML_ENCODING_8859_7 = "ISO-8859-7";
    final static String XML_ENCODING_8859_8 = "ISO-8859-8";
    final static String XML_ENCODING_8859_9 = "ISO-8859-9";
    //
    final static String XML_ENCODING_2022_JP = "ISO-2022-JP";
    final static String XML_ENCODING_Shift_JIS = "Shift_JIS";
    final static String XML_ENCODING_EUC_JP = "EUC-JP";
    //
    // If an XML document uses UTF-16, it must begin with:
    //	otherwise it is assumed to be encoded in UTF-8
    //
    final static char XML_BYTE_ORDER_MARK = '\uFEFF';
    //
    // DEFAULT XML CHARACTER ENCODING
    //
    final static String XML_ENCODING = XML_ENCODING_UTF8;
    //
    // DEFAULT INITIAL NUMBER OF COMMENTS OR PROCESSING INSTRUCTIONS.
    //
    final static int XML_MISC_CAPACITY = 5;
    //
    // DEFAULT XML STANDALONE DOCUMENT STATUS
    //
    final static boolean XML_STANDALONE = false;
    //
    // DEFAULT BUFFER SIZE FOR BUILDING DocTypes
    //
    final static int XML_DOCTYPE_BUILDER_BUFFER_SIZE = 8192;
    //
    // DEFAULT XML Delimiters
    //
    final static String XML_SPACE_DELIMITERS = " \t\n\r";
    //
    //	Expected number of attributes on an Element.
    //
    final static int EXPECTED_ELEMENT_ATTRIBUTE_COUNT = 75;
    //
    //	Expected size of an attribute in characters
    //
    final static int EXPECTED_ATTRIBUTE_LENGTH = 50;
    //
    //	Expected number of (unique) elements in a DTD
    //
    final static int EXPECTED_DOCTYPE_ELEMENT_COUNT  = 50;
    //
    //  Expected total number of markup declarations in a DTD
    //
    final static int EXPECTED_MARKUP_DECLARATIONS = 200;
    //
    //	Expected content character legnth in an Element.
    //
    final static int EXPECTED_ELEMENT_CHARDATA_LENGTH = 500;
    //
    //	Expected number of Processing Instructions in an XML document
    //
    final static int EXPECTED_PROCESSING_INSTRUCTION_COUNT = 20;
    //
    //	Expected number of Elements in an XML document
    //
    final static int EXPECTED_DOCUMENT_ELEMENT_COUNT = 200;
    //
    //	Expected number of characters in an XML token
    //
    final static int EXPECTED_TOKEN_LENGTH = 64;
    //
    //	Expected number of characters in an XML character Entity
    //
    final static int EXPECTED_CHARACTER_ENTITY_LENGTH = 16;
    //
    //	Expected number of characters in an XML entity reference
    //
    final static int EXPECTED_ENTITY_REFERENCE_LENGTH = 64;
    //
    //	Expected number of characters in an XML entity expansion.
    //
    final static int EXPECTED_ENTITY_LENGTH = 128;
    //
    //	Expected number of characters in an XML quoted String
    //
    final static int EXPECTED_QUOTED_STRING_LENGTH = 128;
    //
    //	Maximum number of charactes in an XML comment retained for comment token
    // 
    final static int MAXIMUM_COMMENT_LENGTH = 16384;
    //
    //	Expected number of characters in a scan buffer
    //
    final static int EXPECTED_SCAN_BUFFER_LENGTH = 2048;
    //
    //	Expected number of characters in a CDATA section
    //
    final static int EXPECTED_CDATA_LENGTH = 2048;
    //
    //	Initial number of general entity defintions in a document
    //
    final static int GENERAL_ENTITY_COUNT = 127;
    //
    //	Initial number of parameter entity definitions in a document
    //
    final static int PARAMETER_ENTITY_COUNT = 67;
    //
    //	Expected PI Instruction length
    //
    final static int PI_LENGTH = 255;
    //
    //	Expected number of Content Particles in an Element declartion
    //
    final static int CP_COUNT = 10;
    //
    //	Expected number of names or name tokens in a notation or enumeration
    //
    final static int ENUMERATION_COUNT = 10;
    //
    //  Expected number of name spaces in use per document.
    //
    final static int EXPECTED_NAME_SPACE_COUNT = 8;
    //
    //  Expected length of an XML Section
    //
    final static int EXPECTED_SECTION_LENGTH = 32768;
    //
    //  Expected depth of a document element stack.
    //
    final static int EXPECTED_ELEMENT_STACK_DEPTH = 50;
    
}



