/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/XMLassembly/ContentSpec.java,v 1.1.1.1 2000/09/19 19:39:50 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/
/*----------------------------------------------------------------------------------+
 |                                                                                  |
 |                ContentSpec:  Element Content specification Class                 |
 |                                                                                  |
 +----------------------------------------------------------------------------------*/

package com.adobe.rice.common.XMLassembly;

import java.io.*;
import java.text.*;
import java.util.*;
//$JDK1.1$ import com.sun.java.util.collections.*;
import com.adobe.rice.common.Parsergen.*;

/**
 * ContentSpec.java
 *
 *
 * Created: Wed Oct 20 13:14:06 1999
 *
 * @author V. Bruce Hunt
 * @version
 */

public class ContentSpec extends Cp implements XMLexpandable 
{
    
    final static byte CP_ANY   = (byte)( Cp.CP_LAST + 1);  // 
    final static byte CP_EMPTY = (byte)( Cp.CP_LAST + 2);  //

    // Parser cmParserFactory = null;     //
    
    public ContentSpec() 
    {
        super();
        parent = null;			    // this MUST be null for a ContentSpec.
        repetition = REP_ONCE;		// this MUST be REP_ONCE for a ContentSpec.
    }
    
    public ContentSpec( byte t )
    {
        this();
        if ( t > CP_NAME && t <= CP_EMPTY )
            type = t;
        else
            type = CP_ANY;
    }

    public ContentSpec( byte t, ElementDecl ed )
    {
        this();
        if ( t > CP_NAME && t <= CP_EMPTY )
            type = t;
        else
            type = CP_ANY; 
        container = ed;
    }

    // cannot change parent nor repetitions in a ContentSpec
    public void setParent( ChoiceSeq cs ) { parent = null; }
    public void setRepetition( byte r ) { repetition = REP_ONCE; }

    //
    // Recursive descent parsing routines
    //
    //
    // contentspec ::= 'EMPTY' | 'ANY' | Mixed | children 
    //
    // Mixed	::= '(' S? '#PCDATA' ( S? '|' S? Name)*  S? ')*' 
    //         |  '(' S? '#PCDATA' S? ')' 
    //
    // children ::= (choice | seq) ('?' | '*' | '+')?
    //
    // choice ::= '(' S? cp ( S? '|' S? cp )* S? ')'
    //
    // seq    ::= '(' S? cp ( S? ',' S? cp )* S? ')'
    //
    // cp     ::= (Name | choice | seq) ('?' | '*' | '+')?
    //
    // parse enters with t pointing at the first token in content spec: 
    //     'EMPTY' | 'ANY' | '('
    // parse leaves with t pointing at the token after a valid content spec:
    //     i.e. token after 'EMPTY' | 'ANY' | trailing ')'.
    public int parse( ScanToken t ) throws XMLparseException
    {
        int count = 0;
        int gotOne = 0;
        String msg = null;
        XMLparseException pBad = null;
        XMLscanner scan = container.getScan();
        DocType dtd = container.getDtd();
    	
        if ( t.type == ScanToken.NAME )
        {
            if ( t.string.equals("EMPTY") )
            {
                this.setType(CP_EMPTY);
                count = 1;
            }
            else if ( t.string.equals("ANY") )
            {
                this.setType(CP_ANY);
                count = 1;
            }
            else
            {
                Object[] args = { "EMPTY or ANY", t.toString() };
                msg = MessageFormat.format( Messages.unexpected, args );
                pBad = new XMLparseException( msg );
                throw pBad; 		
            }
            // point at next token
            t = scan.getToken();
        }
        else if ( t.string.equals("(") )
        {   // go find children or Mixed
            // enter children but could also be Mixed...             
            Children ach = new Children( CP_CHOICE, container );
            ach.setIsPossibleMixed(true);    // it is possible to be a Mixed...
            try { gotOne = ach.parse( t ); }
            catch ( XMLparseException p ) { throw p; }
            child = ach;
            count = 1;
        }
        else
        { 
            Object[] args1 = { "ANY, EMPTY, Mixed or children", t.toString() }; 
            msg = MessageFormat.format( Messages.unexpected, args1 );
            pBad = new XMLparseException( msg );
            throw pBad; 
        }
        return count;
    }

    //
    //	implements XMLexpandable:
    //
    //	cannot do CharData so doesn't do toCDSect or toCharData
    //		probably should throw and exception.
    //
    public boolean canDoCharData() { return false; }
    public String toCDSect() { return ""; }
    public String toCharData() { return ""; }

    public String toXMLstring()
    {
        String scp = "";
        
        switch( type )
        {
            case CP_NAME:
                scp += "<!-- ***LOGIC ERROR in ContentSpec - cannot be element type name *** --->";
                break;
            case CP_CHOICE:
            case CP_SEQ:
                scp += child.toXMLstring();
                break;
            case CP_MIXED:
                scp +=  child.toXMLstring();
            case CP_EMPTY:
                scp +=  " EMPTY ";
                break;
            case CP_ANY:
                scp += " ANY ";
                break;
            default:
                scp += "<!-- ***LOGIC ERROR in CP - unknown Cp type -- ignored *** -->"; 
        }
        return scp;
    }

    /**
       @method genContentGrammar
	
       This method constructs a (left recursive) grammer for the content model by replacing all 
       repetition operators with their (left recursive) grammar equivalents.  
       This is basically a recursive scan through the
       Content tree to repCanonical as we go....
	
    **/
	
    public void genGrammar( )
    {
        Grammar g = container.getGrammar();
        
        if ( g != null )
            return;
        
        Production p = new Production( this.id );	// construct the production
	
        if ( child != null )
        {
            child.genGrammar( g );
            p.add( child.id );
        }
        else
        {
            p.add( Symbol.empty );
        }
    }
} // ContentSpec




