/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/XMLassembly/ContentType.java,v 1.1.1.1 2000/09/19 19:39:50 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/
/*================================================================================+
 |                                                                                |
 |                     ContentType - XML Document Content Type                    |
 |                                                                                |
 |	This class defines the available content types in an XML document.            |
 |                                                                                |
 +================================================================================*/

package com.adobe.rice.common.XMLassembly;

import java.util.*;
import java.io.*;
import java.text.*;

public class ContentType
{
    // note:  following constants are ordered for the string constants below
    //        If you change these values, update the string table as well.
    //
    private static final byte CTYPE_MIN    = 0;

    public final static byte PROLOG 	   = 0;	// envelope for xmldecl, doctype 
    // decl, comments & PIs at beginning of XML document
    public final static byte XMLDECL   	   = 1; // XML version, encoding and 
    // standalone declaration.
    public final static byte DOCTYPEDECL   = 2; // Doctype declarations
    public final static byte TEXTDECL	   = 3; // External Entity definition
    public final static byte EXTSUBSETDECL = 4; // External subset declaration
    public final static byte IGNORE        = 5; // IGNORE section
    public final static byte INCLUDE	   = 6; // INCLUDE section
    public final static byte PARMENTITYREF = 7; // parameter entity reference
    // the following types are valid content for an element:
    public final static byte ATTRIBUTE	   = 8; // XML element attribute content 
                                                // reference (name & value)
    public final static byte ATTRVALUE	   = 9; // XML element attribute value 
    public final static byte ELEMENT	   = 10;// XML element
                                                // content references possible:
    public final static byte COMMENT 	   = 11;// XML comment
    public final static byte PROCINST	   = 12;// processing instruction declaration
    public final static byte GENENTITYREF  = 13;// general entity reference
    public final static byte CHARREF	   = 14;// character reference
    public final static byte CHARDATA	   = 15;// regular characters (not markup)
    public final static byte CDATA	       = 16;// CDATA section
    
    private static final byte CTYPE_MAX = CDATA;  // maximum value
	
    private byte type;		                // This holds the ContentType defined above
	
    //
    // note: The following strings are ordered as defined by the manifest 
    //       constants above. Thus, "Prolog" is at index zero because 
    //       PROLOG is defined as zero above; and "CharData" is at 
    //       index 15 because CHARDATA is defined as 15 above. THEY MUST 
    //       REMAIN ordered as bound. The names as defined below are 
    //       EXACTLY as defined in the XML 1.0 specification.
    //
    private final static String[] names = 
    { 
        "Prolog",        "XMLDecl",    "doctypedecl", "TextDecl", 
        "ExtSubsetDecl", "IgnoreSect", "IncludeSect", "PEReference",
        "Attribute",     "AttValue",   "Element",     "Comment",
        "PI",            "EntityRef",  "CharRef",     "CharData",   
        "CDSect"  
    };
		
    public ContentType()
    {
        type = CHARDATA;
    }
	
    public ContentType( String s ) throws ContentException
    {
        this();

        byte i = 0;

        for ( ; i < names.length ; i++ )
        {
            if ( s.equals( names[i] ) )
            {	
                type = i;
                break;
            }
        }
        if ( i >= names.length )
        {
            Object[] args = { s, Messages.frag_ContentTypeName };
            MessageFormat undecl = new MessageFormat( Messages.undeclared );
            ContentException cBad = new ContentException( undecl.format(args) );
            throw cBad;
        }
    }
	
    public ContentType( byte b ) throws ContentException
    {
        this();
        if ( CTYPE_MIN <= b && b <= CTYPE_MAX )
        {
            type = b;
            return;
        }
        // msg: "Content type index, x, is not in the defined range from &CTYPE_MIN through &CTYPE_MAX."
        Byte ob = new Byte( b ), low = new Byte( CTYPE_MIN ), hi = new Byte( CTYPE_MAX );
        Object[] args = { Messages.frag_ContentType, ob, low, hi };
        MessageFormat outOfRange = new MessageFormat( Messages.indexOutOfRange );
        // throw an exception
        ContentException cBad = new ContentException( outOfRange.format(args));
        throw cBad;
    }
	
    //
    // set the Content type -- return true if set, false if out of range.
    //
    public boolean set( byte b )
    {
        if ( CTYPE_MIN <= b && b <= CTYPE_MAX )
        {
            type = b;
            return true;
        }
        return false;
    }
	
    public boolean set( String s )
    {		
        for ( byte i = 0; i < names.length ; i++ )
	    {
            if ( s.equals( names[i] ) )
		    {
                type = i;
                return true;
		    }
	    }
        return false;
    }
	
    public byte get()
    {
        return type;
    }
	
    public String getString()
    {
        if ( ( CTYPE_MIN <= type) && ( type <= CTYPE_MAX ) )
            return names[type];
        return "Unknown";
    }

    public String toString() { return getString(); }
}












