/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/XMLassembly/Cp.java,v 1.1.1.1 2000/09/19 19:39:50 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/
/*----------------------------------------------------------------------------------+
 |                                                                                  |
 |                             Cp Content Particle Class                            |
 |                                                                                  |
 +----------------------------------------------------------------------------------*/

package com.adobe.rice.common.XMLassembly;

import java.io.*;
import java.text.*;
import java.util.*;
//$JDK1.1$ import com.sun.java.util.collections.*;
import com.adobe.rice.common.Parsergen.*;

//<class-dependencies>
//  <class codependent=yes> ElementDecl </>
//  <class> Symbol </>
//</class-dependencies>

/**
   children	 ::=  ( choice | seq ) ('?' | '*' | '+')?
   cp            ::=  ( Name | choice | seq ) ('?' | '*' | '+')?
   choice        ::=  '(' S? cp ( S? '|' S? cp )* S? ')' [ VC: Proper Group/PE Nesting ]
   seq           ::=  '(' S? cp ( S? ',' S? cp )* S? ')' [ VC: Proper Group/PE Nesting ]

**/
public class Cp implements XMLexpandable, Serializable
{
    // Cp types:
    public final static byte CP_NAME = 0;
    public final static byte CP_CHOICE = 1;
    public final static byte CP_SEQ = 2;
    public final static byte CP_MIXED = 3;	        // used at ElementDecl Level
    public final static byte CP_ANY = 4;            // used at ElementDecl Level

    public final static byte CP_LAST = CP_MIXED;   // defined last element ( so easy for object extension )

    // Cp repetitions:
    public final static byte REP_ZERO_OR_MORE = 0; // '*' operator
    public final static byte REP_ONE_OR_MORE = 1;  // '+' operator
    public final static byte REP_ZERO_OR_ONE = 2;  // '?' operator
    public final static byte REP_ONCE = 3;	    // '' operator (empty)
    
    // Cp fields:
    protected Cp          parent = null; 	    // parent of this Cp; Must be ChoiceSeq or derivative
    protected Cp          child = null;	        // contains the Choice, Seq or null if CP_NAME or CP_MIXED
    protected byte        type = CP_NAME;
    protected byte        repetition = REP_ONCE;
    protected Symbol      id = null;		    // non-terminal id for grammar 
    protected ElementDecl container = null;     // The element declartion that contains this Cp
    protected ElementDecl elementDecl = null;   // if CP_NAME, the element declaration for the cp.
    protected List        items = null;         // used only if a ChoiceSeq
    protected boolean     isPossibleMixed = false;

    //
    // constructors
    //
    public Cp()
    {
    	elementDecl = null;
        container = null;
    	type = CP_NAME;
    	repetition = REP_ONCE;
    	parent = null;
        child = null;
        id = null;
    }

    public Cp( byte t )
    {
        this();
        if ( t >= CP_NAME && t <= CP_MIXED )
            type = t;
        else
            type = CP_NAME;
        makeId();
    }
    
    public Cp( Cp p )
    {
    	this();
    	parent = p;
        makeId();
    }

    public Cp( ElementDecl ed )
    {
        this();
        container = ed;
        makeId();
    }

    public Cp( byte t, ChoiceSeq p )
    {
        this();
        parent = p;
        if ( t >= CP_NAME && t <= CP_MIXED )
            type = t;
        else
            type = CP_NAME;
        makeId();
    }
    
    public Cp( Cp p, ElementDecl ed )
    {
    	this();
    	parent = p; 
    	container = ed;
        makeId();
    }

    public Cp( byte t, Cp p, ElementDecl ed )
    {
        this();
        if ( t >= CP_NAME && t <= CP_MIXED )
            type = t;
        else
            type = CP_NAME; 
        parent = p;
        container = ed;
        makeId();
    }
    
    //
    //	setters and getters
    //
    public void setType( byte t ) { type = t; }
    public byte getType() { return type; }
    
    public void setRepetition( byte r ) { repetition = r; }
    public byte getRepetition() { return repetition; }
    
    public void setContainer( ElementDecl ed ) 
    { 	
        container = ed;
        // make sure that there is a production name.
        makeId();
    }
    public ElementDecl getContainer() { return container; }

    public void setElementDecl( ElementDecl ed ) { elementDecl = ed; makeId(); }
    public ElementDecl getElementDecl( ) { return elementDecl; }
    
    public void setParent( Cp p ) { parent = p; }
    public Cp getParent( ) { return parent; }
    
    public String getName() 
    { 
        if ( type == CP_NAME )
            return (String) elementDecl.getName();
        return "";
    }
    
    public Symbol getId() { return id; }


    //
    // methods:
    //
    public ElementDecl findElement( String elementType )
        {
            switch (type)
            {
                case Cp.CP_NAME:
                    if ( elementType.equals( this.getName() ) )
                        return this.elementDecl;
                    break;
                case Cp.CP_CHOICE:
                case Cp.CP_SEQ:
                case Cp.CP_MIXED:
                    if ( items == null )
                        break;
                    for ( int i = 0; i < items.size(); i++ )
                    {
                        Cp tcp = (Cp) items.get(i);
                        if ( ( tcp.type == Cp.CP_NAME ) &&
                             elementType.equals( tcp.getName() ) )
                            return tcp.getElementDecl();
                    }
                    break;
                case Cp.CP_ANY:
                    break;
                default:
                    break;
            }
            return null;
        }

    //
    // ID constructor:
    //
    protected void makeId()
    {
        if ( id != null )
            return;
        switch( type )
        {
            case CP_NAME:
                if ( elementDecl != null )
                {
                    id = new Symbol( elementDecl.getName(), LRconst.TERMINAL );
                }
                break;
            default:
                if ( container != null )
                {
                    container.incrementCount();
                    Integer idint = new Integer( container.getCount() );
                    String prid = container.getName() + "_P" + idint.toString();
                    id = new Symbol( prid, LRconst.NONTERMINAL );
                }
                break;
        }
    }
    
    //
    // Recursive descent parsing routines
    //
    //
    // contentspec ::= 'EMPTY' | 'ANY' | Mixed | children 
    //
    // Mixed	::= '(' S? '#PCDATA' ( S? '|' S? Name)*  S? ')*' 
    //         |  '(' S? '#PCDATA' S? ')' 
    //
    // children ::= (choice | seq) ('?' | '*' | '+')?
    //
    // choice ::= '(' S? cp ( S? '|' S? cp )* S? ')'
    //
    // seq    ::= '(' S? cp ( S? ',' S? cp )* S? ')'
    //
    // cp     ::= (Name | choice | seq) ('?' | '*' | '+')?
    //
    //
    public int parse( ScanToken t ) throws XMLparseException
    {
    	int count = 0;
    	int gotOne = 0;
    	String msg = null;
    	XMLparseException pBad = null;
    	XMLscanner scan = container.getScan();
        DocType dtd = container.getDtd();
    	
    	if ( t.type == ScanToken.NAME )
    	{
            // look up the name in the Document's ElementDecl list.
            elementDecl = dtd.getElementDecl( t.string );
            if ( elementDecl  == null )
            {	
                // it is NOT an XML error to have an undeclared element name. 
                elementDecl = new ElementDecl( dtd, t.string );
                // but note it so we can warn user later if remains undeclared.
                elementDecl.setIsUndeclared( true );
                // put it in the list of element declarations
                dtd.addElementDecl( elementDecl );
            }
            count = 1;	// indicate something was parsed.
            t = scan.getToken();
    	}
    	else if ( t.string.equals("(") )
    	{   // go find an choice or a seq 
            ChoiceSeq acs = new ChoiceSeq( this, container );
            try { gotOne = acs.parse( t ); }
            catch ( XMLparseException p ) { throw p; }
            child = acs;
            count = 1;
    	}
    	else
    	{ 
            Object[] args1 = { "Name, Choice or Sequence", t.toString() }; 
            msg = MessageFormat.format( Messages.unexpected, args1 );
            pBad = new XMLparseException( msg );
            throw pBad; 
    	}
    	// now look for the repetitions
    	if ( t.type == ScanToken.SPECIAL )
    	{
            switch ( t.string.charAt(0) )
    	    {
                case '?':
                    repetition = REP_ZERO_OR_ONE;
                    t = scan.getToken();
                    break;
                case '*':
                    repetition = REP_ZERO_OR_MORE;
                    t = scan.getToken();
                    break;
                case '+':
                    repetition = REP_ONE_OR_MORE;
                    t = scan.getToken();
                    break;
                default:
                    repetition = REP_ONCE;
                    break;
            }
    	}
    	else
    	{
            repetition = REP_ONCE;
    	}
    	return count;
    }

    public boolean canDoCharData() { return false; }
    public String toCDSect() { return ""; }
    public String toCharData() { return ""; }

    public String toXMLstring()
    {
        String scp = "";
	
        switch( type )
        {
            case CP_NAME:
                scp += elementDecl.getName();
                break;
            case CP_CHOICE:
            case CP_SEQ:
                scp += child.toXMLstring();
                break;
            case CP_MIXED:
                scp += "<!-- MIXED in CP! (How does this happen?) --> " + child.toXMLstring();
            default:
                scp += "<!-- ***LOGIC ERROR in CP - unknown Cp type -- ignored *** -->"; 
        }
        return scp;
    }

    //
    // genGrammar - construct a production representation for rapid parsing
    //
    public void genGrammar( Grammar g )
    {
        // now construct the production representation.
        Production cpr = null;
        if ( child != null )
        {   // generate any lower level productions for the grammar
            switch( type )
            {
                case CP_CHOICE:
                case CP_SEQ:
                    child.genGrammar( g );
                    break;
                default:
                    break;
            }
            // now generate productions for this content particle.
            switch( repetition )
            {
                case REP_ZERO_OR_MORE: // id -> child*
                    // id -> EMPTY
                    cpr = new Production( this.id );
                    cpr.add( Symbol.empty );
                    g.add( cpr );
                    // id -> child
                    cpr = new Production( this.id );
                    cpr.add( child.id );
                    g.add( cpr );
                    // id -> id child
                    cpr = new Production( this.id );
                    cpr.add( this.id );
                    cpr.add( child.id );
                    g.add( cpr );
                    break;
                case REP_ONE_OR_MORE:   // id -> child+
                    // id -> child
                    cpr = new Production( this.id );
                    cpr.add( child.id );
                    g.add( cpr );
                    // id -> id child
                    cpr = new Production( this.id );
                    cpr.add( this.id );
                    cpr.add( child.id );
                    g.add( cpr );
                    break;
                case REP_ZERO_OR_ONE:	// id -> child?
                    // id -> EMPTY
                    cpr = new Production( this.id );
                    cpr.add( Symbol.empty );
                    g.add( cpr );
                    // id -> child
                    cpr = new Production( this.id );
                    cpr.add( child.id );
                    g.add( cpr );
                    break;
                case REP_ONCE:		// id -> child
                    // id -> child
                    cpr = new Production( this.id );
                    cpr.add( child.id );
                    g.add( cpr );
                    break;
                default:
                    // This is very bad --> Content model is messed up big time.
                    break;	
            }
        }
    }
}

















