/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/XMLassembly/DocType.java,v 1.3 2000/11/27 20:20:41 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/
/*----------------------------------------------------------------------------------+
 |                                                                                  |
 |                                DocType (DTD)                                     |
 |                                                                                  |
 +----------------------------------------------------------------------------------*/

package com.adobe.rice.common.XMLassembly;

import java.util.*;
import java.io.*;
import java.text.*;
import java.net.*;
//$JDK1.1$ import com.sun.java.util.collections.*;


public class DocType implements Serializable, XMLexpandable
{
    private String      name;                // MUST match the root element name.
    private String      systemLiteral = "";  // optional external subset URI
    private String      pubidLiteral = "";   // optional initial external subset URI.
    private List        markup = new ArrayList(Constants.EXPECTED_MARKUP_DECLARATIONS);
    //
    //  External Subset 
    //
    private ExtSubset   externalSubset = null; // points at a single instance
    //
    //  Document Type Definition Operating Section
    //
    HashMap             elementDecls;       // element declarations in the document.
    HashMap             processorInst;	    // processing Instructions in the document.
    //
    //	Entity Management 
    //
    private HashMap     parameterEntities;  // parameter Entity declarations in document.
    private HashMap     generalEntities;    // general Entity declarations in document.
    //
    //  Each XML document points at a unique Doctype -- but a unique Doctype can
    //  be pointed at by many XML documents.
    //
    private XMLscanner scan;    // used only to scan the DocType.
    //
    //  Constructors
    //
    public DocType()
    {
    	systemLiteral = "";
    	pubidLiteral = "";
        elementDecls = new HashMap(Constants.EXPECTED_DOCTYPE_ELEMENT_COUNT);
        processorInst = new HashMap(Constants.EXPECTED_PROCESSING_INSTRUCTION_COUNT);
        parameterEntities = new HashMap( Constants.PARAMETER_ENTITY_COUNT );
        generalEntities = new HashMap( Constants.GENERAL_ENTITY_COUNT );
    }
    
    public DocType( XMLscanner s )
    {
    	this();
    	scan = s;
    }

    public DocType( ExtSubset ess )
    {
        this();
        externalSubset = ess;
    }

    public DocType( ExtSubset ess, XMLscanner s )
    {
        this();
        externalSubset = ess;
        scan = s;
    }

    public DocType( String sysId )
        throws MalformedURLException, IOException, XMLparseException, DocTypeException
    {
        this();
        systemLiteral = sysId;
        try { setupParseExternalSubset( systemLiteral ); }
        catch ( MalformedURLException mfURL ) { throw mfURL; }
        catch ( IOException ioe ) { throw ioe; }
        catch ( XMLparseException xmlEx ) { throw xmlEx; }
        catch ( DocTypeException dtdEx ) { throw dtdEx; }
        //
        // now setup the default root element -- i.e. the first element decl in
        // the external subset
        //
    }

    public DocType( String sysId, String pubId ) 
        throws MalformedURLException, IOException, XMLparseException, DocTypeException
    {
        //$FIX$ - What should be done about pubID?
        this();
        systemLiteral = sysId;
        pubidLiteral = pubId;
        try { setupParseExternalSubset( systemLiteral ); }
        catch ( MalformedURLException mfURL ) { throw mfURL; }
        catch ( IOException ioe ) { throw ioe; }
        catch ( XMLparseException xmlEx ) { throw xmlEx; }
        catch ( DocTypeException dtdEx ) { throw dtdEx; }
    }
            
    //
    // DocType Methods
    //
    
    //*
    //* Set Methods
    //*
    public void         setScan( XMLscanner s )             { scan = s; }
    public void         setExternalSubset( ExtSubset es )   { externalSubset = es; }
    public void         setSystemLiteral( String sl )       { systemLiteral = sl; }
    public void         setPubidLiteral( String pl )        { pubidLiteral = pl; }
    public void         setName( String nm )
    {
        if ( this.elementDecls != null )
        {
            if ( this.getElementDecl(nm) != null )
                this.name = nm;
        }
        else
        {
            this.name = nm;
        }
    }

    //*
    //* Get Methods
    //*
    public XMLscanner   getScan( )                  { return scan; }
    public String       getName( )                  { return name; }
    public String       getSystemLiteral()          { return systemLiteral; }
    public String       getPubidLiteral()           { return pubidLiteral; }
    public List         getMarkup()                 { return markup; }
    public ExtSubset    getExternalSubset()         { return externalSubset; }
    public HashMap      getElementDecls()           { return elementDecls; }
    public HashMap      getProcessorInst()          { return processorInst; }
    public HashMap      getParameterEntities()      { return parameterEntities; }
    public HashMap      getGeneralEntities()        { return generalEntities; }


    //
    // Setup Access to Extneral Subset
    //
    private void setupParseExternalSubset( String subsetURL ) 
        throws MalformedURLException, IOException, XMLparseException, DocTypeException
    {
        int gotExtSubset = 0;

        if ( subsetURL.length() > 0 )
        {
            externalSubset = new ExtSubset();
            // $DELETE$ externalSubset.setScan( new XMLscanner() );
            externalSubset.setDtd(this);        // must tell him about the DTD!
            externalSubset.setSubsetURL( subsetURL );
            try { externalSubset.parseSetup(); }
            catch( MalformedURLException mfURL ) 
            {   // $POLICY$
                // A:  Ignore invalid URL -- pretend it didn't happen
                // B:  Throw MalformedURLException -- 
                // C:  Throw DocTypeException -- Can't construct valid document.
                if ( 1 == 0 ) 
                {
                    DocTypeException dtdex = new DocTypeException( "Can't construct DTD" );
                    throw dtdex;
                }
                else
                    throw mfURL; 
            }
            catch( IOException ioe ) 
            {   // $POLICY$
                // A:  Ignore error -- pretend it didn't happen.
                // B:  Throw IOException -- Could be a timeout problem -- so wait and retry?
                // C:  Throw DocTypeException -- Can't construct valid document.
                throw ioe; 
            };
            // now parse it.
            try { gotExtSubset = externalSubset.parse(); }
            catch( XMLparseException p ) 
            {   // $POLICY$
                // A:  Ignore error -- pretend it didn't happen.
                // B:  Throw XMLparseException -- error in DTD.
                // C:  Throw DocTypeException -- Can't construct valid document.
                throw p; 
            }
        }
    }

    //*
    //*	Element definition Table management
    //*
    public ElementDecl getElementDecl( String elemName )
    {
        return (ElementDecl) elementDecls.get( elemName );
    }
    
    public void addElementDecl( String elementName )
    {
        // if it is already defined, then don't add it again.
        if ( elementDecls.containsKey( elementName ) )
            return;
        ElementDecl ed = new ElementDecl( elementName );
        elementDecls.put( elementName, ed );
        return;
    }

    public void addElementDecl( ElementDecl ed )
    {	// if it is already defined, then don't add it again.
        if ( elementDecls.containsKey( ed.getName() ) )
            return;
        elementDecls.put( ed.getName() , ed );
        return;
    }
		 
    //*
    //*	Parameter Entity Table management
    //*
    public EntityDecl setParameterEntity( String parmName, EntityDecl parmValue )
    {
        return (EntityDecl) parameterEntities.put( parmName, parmValue );
    }
	
    public EntityDecl getParameterEntity( String parmName )
    {
        return (EntityDecl) parameterEntities.get( parmName );
    }
	
    public EntityDecl removeParameterEntity( String parmName )
    {
        return (EntityDecl) parameterEntities.remove( parmName );
    }
	
    public void clearParameterEntities( )
    {
        parameterEntities.clear();
    }
	
    public int sizeParameterEntities()
    {
        return parameterEntities.size();
    }
    
    public boolean definedParameterEntity( String peName )
    {
        return parameterEntities.containsKey( peName );
    }
	
    public boolean isEmptyParameterEntities()
    {
        return parameterEntities.isEmpty();
    }
    //*
    //*	General Entity table management
    //*
    
    public EntityDecl setGeneralEntity( String entityName, EntityDecl entity )
    {
        return (EntityDecl) generalEntities.put( entityName, entity );
    }
    
    public EntityDecl getGeneralEntity( String entityName )
    {
        return (EntityDecl) generalEntities.get( entityName );
    }
	
    public EntityDecl removeGeneralEntity( String entityName )
    {
        return (EntityDecl) generalEntities.remove( entityName );
    }
	
    public void clearGeneralEntities( )
    {
        generalEntities.clear();
    }
	
    public int sizeGeneralEntities()
    {
        return generalEntities.size();
    }
	
    public boolean definedGeneralEntity( String geName )
    {
        return generalEntities.containsKey( geName );
    }
	
    public boolean isEmptyGeneralEntities()
    {
        return generalEntities.isEmpty();
    }

    //* 
    //* Implementation of XMLexpandable
    //*

    public boolean canDoCharData() { return false; }
    public String toCDSect() { return ""; }
    public String toCharData() { return ""; }

    public String toXMLstring()
    {
        //
        // doctypedecl ::= '<!DOCTYPE' S Name (S ExternalID)? S? 
        //						('[' (markupdecl | PEReference | S)* ']' S?)? '>'
        //
        String dtd = "<!DOCTYPE " + name + " ";
        //
        // ExternalID ::= 'SYSTEM' S SystemLiteral
        //             |  'PUBLIC' S PubidLiteral S SystemLiteral 
        //
        if ( pubidLiteral.length() > 0 )
        {
            dtd += "PUBLIC \"" + pubidLiteral + "\" ";
            if ( systemLiteral.length() > 0 )
                dtd += "\"" + systemLiteral + "\" ";
        }
        else if ( systemLiteral.length() > 0 )
        {
            dtd += "SYSTEM \"" + systemLiteral + "\" ";
        }
        if ( markup.size() > 0 )
        {
            dtd += "[ \n";
            for ( int i = 0; i < markup.size(); i++ )
            {
                MarkUpDecl mdecl = (MarkUpDecl) markup.get(i);
                dtd += mdecl.toXMLstring() + "\n";
            }
            dtd += "] \n";
        }
        dtd += ">\n";
        return dtd;
    }
    //*
    //* Parsing Routines
    //*

    //
    // doctypedecl ::= '<!DOCTYPE' S Name (S ExternalID)? S? 
    //						('[' (markupdecl | PEReference | S)* ']' S?)? '>'
    //
    public int parse( ScanToken t ) 
        throws MalformedURLException, IOException, XMLparseException, DocTypeException
    {
    	int count = 0;
    	
    	if ( t.type != ScanToken.BANG_NAME )
    		return count;
    	if ( !t.string.equals("DOCTYPE") )
    		return count;
    	// we have a DOCTYPE prefix
     	XMLparseException pBad = null;
    	String msg = null;
   	
    	t = scan.getToken();
    	if ( t.type != ScanToken.NAME )
    	{
            Object[] args = { "DOCTYPE name", t.toString() };
            msg = MessageFormat.format( Messages.unexpected, args );
            pBad = new XMLparseException( msg );
            throw pBad; 
    	}
    	name = t.string;  // so now we have the root element
        count++;
    	//
    	// now look for optional ExternalID -- SYSTEM or PUBLIC identifers
    	//
    	t = scan.getToken();
    	{
            int gotExternalID = 0;
            int gotExtSubset  = 0;
            try { gotExternalID = parseExternalID( t ); }
            catch( XMLparseException p ) { throw p; }
            if ( gotExternalID > 0 )
            {
                try { setupParseExternalSubset( systemLiteral ); }
                catch ( MalformedURLException mfURL ) { throw mfURL; }
                catch ( IOException ioe ) { throw ioe; }
                catch ( XMLparseException xmlEx ) { throw xmlEx; }
                catch ( DocTypeException dtdEx ) { throw dtdEx; }
            }
    	}
        // 
        //  ('[' (markupdecl | PEReference | S)* ']' S? )? 
        //
    	if ( (t.type == ScanToken.SPECIAL) && t.string.equals("[") )
    	{
            int gotPMD = 0;
            MarkUpDecl mdecl = new MarkUpDecl( this );
            scan.setInDTD(true);
            //  (markupdecl | PEReference | S)* 
            do
    	    {	// get next token
                t = scan.getToken();
                try { gotPMD = mdecl.parse( t ); }
                catch ( XMLparseException p ) { throw p; }
                if ( gotPMD > 0 )
                {
                    markup.add(mdecl);
                    count++;
                    // get a new one for next try
                    mdecl = new MarkUpDecl( this );
                }
            }
            while ( gotPMD > 0 );
            // no longer scanning DTD.
            scan.setInDTD(false);
            if ( (t.type != ScanToken.SPECIAL) || (! t.string.equals("]") ) )
    	    {
                // expecting ] to close out the DTD
                // parse Invalid without it.
                Object[] args = { "closing ] for DOCTYPE", t.string };
                msg = MessageFormat.format( Messages.unexpected, args );
                pBad = new XMLparseException( msg );
                throw pBad; 
            }
            t = scan.getToken();
    	}
    	if ( t.type != ScanToken.MARKUP_END )
    	{
            // missing closing ">" 
            Object[] args = { "closing > for DOCTYPE", t.string };
            msg = MessageFormat.format( Messages.unexpected, args );
            pBad = new XMLparseException( msg );
            throw pBad; 
    	}
    	return count;
    }

    //
    // ExternalID ::= 'SYSTEM' S SystemLiteral
    //             |  'PUBLIC' S PubidLiteral S SystemLiteral 
    //
    public int parseExternalID( ScanToken t ) throws XMLparseException
    {
    	int count = 0;
    	String msg = null;
    	XMLparseException pBad = null;
    	
    	if ( t.type == ScanToken.NAME )
    	{
            if ( t.string.equals("SYSTEM") )
    	    {
                // 'SYSTEM' SystemLiteral
                t = scan.getToken();
                if ( t.type != ScanToken.QUOTED_STRING )
                {
                    Object[] args = { "System literal value (quoted)", t.toString() };
                    msg = MessageFormat.format( Messages.unexpected, args );
                    pBad = new XMLparseException( msg );
                    throw pBad; 
                }
                systemLiteral = t.string;
                count = 1;
            }
            else if ( t.string.equals("PUBLIC") )
    	    {
                // 'PUBLIC' S PubidLiteral S SystemLiteral 
                t = scan.getToken();
                if ( t.type != ScanToken.QUOTED_STRING )
                {
                    Object[] args = { "PubId literal value (quoted)", t.toString() };
                    msg = MessageFormat.format( Messages.unexpected, args );
                    pBad = new XMLparseException( msg );
                    throw pBad; 
                }
                pubidLiteral = t.string;
                // get SystemLiteral
                t = scan.getToken();
                if ( t.type != ScanToken.QUOTED_STRING )
                {
                    Object[] args = { "System literal value (quoted)", t.toString() };
                    msg = MessageFormat.format( Messages.unexpected, args );
                    pBad = new XMLparseException( msg );
                    throw pBad; 
                }
                systemLiteral = t.string;
                count = 1;
            }
            else
    	    {
                // name must be either SYSTEM or PUBLIC -- this is in error
                Object[] args = { "SYSTEM or PUBLIC", t.toString() };
                msg = MessageFormat.format( Messages.unexpected, args );
                pBad = new XMLparseException( msg );
                throw pBad; 
            }
    	}
    	return count;
    }
}













