/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/XMLassembly/Element.java,v 1.5 2000/12/02 23:31:44 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

/*-------------------------------------------------------------------+
 |                                                                   |
 |                       Element Content Class                       |
 |                                                                   |
 +-------------------------------------------------------------------*/

package com.adobe.rice.common.XMLassembly;

import java.util.*;
import java.io.*;
import java.text.*;
//$JDK1.1$ import com.sun.java.util.collections.*;
import com.adobe.rice.common.storage.*;
import com.adobe.rice.common.storage.logging.*;
import com.adobe.rice.common.Parsergen.*;


public class Element implements Serializable, Loggable , XMLexpandable
{
    private static final byte DELETE = 0;   //  delete selected content
    private static final byte INSERT = 1;   //  insert new content
    private static final byte REPLACE = 2;  //  replace selected content
    
    private String  name = "";		        // element (local) name
    private String  prefix = "";	        // element (name space prefix) name
    private HashMap attributes = null;      // element attributes
    private List    content = null;	        // element content
    //
    // internal management attributes
    //
    private ElementDecl elementDecl = null;	// associated element declaration
    private Element     parent = null;		// parent element if not root
//  private XMLdoc      doc = null;			// The containing XML document
    private DocType     dtd = null;         // The controlling DTD for the element.
    
    private boolean     isRoot = false;		// True if this is the root element
    private boolean     isEmpty = false;    // True if this is an empty element
    private boolean     isAny = false;      // True if this content model is "ANY"
    private boolean     isOpen = false;     // True if element is open 
    //
    // internal parsing machinery
    //
    /*
     * The parser is called with the following parseStack.  That way, only a single LR parser
     * is needed for each element declaration.  The current state of the
     * content model's parse is maintained in this stack.  Thus, all parsing
     * is done by calling the LRparser (that resides in the ElementDecl.)
     * Since all declared elements point at the defining ElementDecl, the 
     * single parser is there.  
     */
    private Stack    parseStack = null;    // The element's content model parse stack.
    // - one per content item.
    private int      index = 0;	            // current content insertion point.
    //
    // the following are formatting options
    //
    private int      nest = 0;              // signals how deep in this element is.
    private boolean  okToFormat = false;    // if true, it means that we can insert
                                            // linefeeds and whitespace to quasi-format
                                            // the XML.  This flag must be explicity
                                            // set to true.
    
    public Element()
    {
        dtd = null;
        name = "";              // this is an element model
        prefix = "";
        attributes = new HashMap(Constants.EXPECTED_ELEMENT_ATTRIBUTE_COUNT);
        content = new ArrayList(Constants.EXPECTED_DOCUMENT_ELEMENT_COUNT);
        // find and set our elementDecl
        elementDecl = null;
        this.isRoot = false;
        this.isAny = true;
        this.isEmpty = false;  // must be declared to be held empty
        this.isOpen = false;
        parent = null;
        parseStack = null;        
    }

	/* $REMOVED$ -- Elements should NOT depend on XMLdocs!
    public Element( String n, Element p, XMLdoc d ) throws ElementException
    {
        this();
        doc = d;
        name = n;
        parent = p;
        // find and set our elementDecl
        if ( (doc != null ) )
            dtd = doc.dtd;
        if ( (dtd != null ) && dtd.elementDecls.containsKey( name ) )
            elementDecl = (ElementDecl) dtd.elementDecls.get( name );
        else
        {
            // $POLICY$
            Object[] args = { name };
            String msg = MessageFormat.format( Messages.element_undeclared, args );
            ElementException pBad = new ElementException( msg );
            throw pBad; 
        }
        
        this.setIsRoot( elementDecl.getIsRoot() );
        this.setIsEmpty( elementDecl.getIsEmpty() );
        this.setIsAny( elementDecl.getIsAny() );
        
        // no stack is needed if Any content is allowd; or no content is allowed.
        if ( !this.isAny && !this.isEmpty )
        {
            parseStack = new Stack( );
            parseStack.ensureCapacity( LRconst.STACK_DEPTH );
            parseStack.push( (ParseState) new ParseState( (short) 0, null ));
        }
    }
    */
	
    public Element( String n, Element p, DocType theDTD ) throws ElementException
    {
        this();
        this.dtd = theDTD;
        this.name = n;
        this.parent = p;
        
        // find and set our elementDecl
        if ( (dtd != null ) && dtd.elementDecls.containsKey( name ) )
            elementDecl = (ElementDecl) dtd.elementDecls.get( name );
        else
        {
            // $POLICY$
            Object[] args = { name };
            String msg = MessageFormat.format( Messages.element_undeclared, args );
            ElementException pBad = new ElementException( msg );
            throw pBad; 
        }
        
        this.setIsRoot( elementDecl.getIsRoot() );
        this.setIsEmpty( elementDecl.getIsEmpty() );
        this.setIsAny( elementDecl.getIsAny() );
        
        // no stack is needed if Any content is allowd; or no content is allowed.
        if ( !this.isAny && !this.isEmpty )
        {
            parseStack = new Stack( );
            parseStack.ensureCapacity( LRconst.STACK_DEPTH );
            parseStack.push( (ParseState) new ParseState( (short) 0, null ));
        }
        
        // set the nesting level 
        if ( this.parent != null )
            this.nest = this.parent.nest + 1;
    }
	
	/* $REMOVED$ - elements should NOT depend on XMLdocs
    public Element( String n, String p, Element pn, XMLdoc d ) throws ElementException
    {
        this();
        name = n;
        prefix = p;
        parent = pn;
        doc = d;
        if ( doc != null )
            this.dtd = doc.dtd;
        //$FIX$ what happens if doc is null?
        if ( ( dtd!= null ) && dtd.elementDecls.containsKey( name ) )
            elementDecl = (ElementDecl) dtd.elementDecls.get( name );
        else
        {
            // $POLICY$
            Object[] args = { name };
            String msg = MessageFormat.format( Messages.element_undeclared, args );
            ElementException pBad = new ElementException( msg );
            throw pBad; 
        }
        if ( elementDecl != null )
        {
            this.setIsRoot( elementDecl.getIsRoot() );
            this.setIsEmpty( elementDecl.getIsEmpty() );
            this.setIsAny( elementDecl.getIsAny() );
        }
        else
            this.setIsAny(true);
        // no stack is needed if Any content is allowd; or NO content is allowed.
        if ( !this.isAny && !this.isEmpty )
        {
            parseStack = new Stack( );
            parseStack.ensureCapacity( LRconst.STACK_DEPTH );
            parseStack.push( (ParseState) new ParseState( (short) 0, null ));
        }
    }	
    */
    
    public Element( String n, String p, Element pn, DocType theDTD ) throws ElementException
    {
        this();
        this.name = n;
        this.prefix = p;
        this.parent = pn;
        this.dtd = theDTD;
        if ( ( dtd!= null ) && dtd.elementDecls.containsKey( name ) )
            elementDecl = (ElementDecl) dtd.elementDecls.get( name );
        else
        {
            // $POLICY$
            Object[] args = { name };
            String msg = MessageFormat.format( Messages.element_undeclared, args );
            ElementException pBad = new ElementException( msg );
            throw pBad; 
        }
        this.setIsRoot( elementDecl.getIsRoot() );
        this.setIsEmpty( elementDecl.getIsEmpty() );
        this.setIsAny( elementDecl.getIsAny() );
        // no stack is needed if Any content is allowd; or NO content is allowed.
        if ( !this.isAny && !this.isEmpty )
        {
            parseStack = new Stack( );
            parseStack.ensureCapacity( LRconst.STACK_DEPTH );
            parseStack.push( (ParseState) new ParseState( (short) 0, null ));
        }
        // set the nesting level 
        if ( this.parent != null )
            this.nest = this.parent.nest + 1;
    }	

    public LRparser getParser() 
    { 
        if ( elementDecl == null ) 
            return null;
        return elementDecl.getLrParser();
    }
	
	public String getAttribute( String aName )
	{
	    if ( attributes == null )
	    {
	        attributes = new HashMap();
	        setupAttributes();
	    }
	    if ( attributes.containsKey( aName ) )
	    {
    	    Attribute theAttribute = (Attribute) attributes.get( aName );
    	    return theAttribute.getValue();
    	}
    	return null;
	}
	
    public void setAttribute(String aName, String aValue) throws AttributeException
    {
    	Attribute theAttribute;
    	
        if ( this.attributes == null )
        {
            this.attributes = new HashMap();
            setupAttributes();
        }
    	// make sure that an attribute declaration exists....
        //
        // if the policy were to allow undeclared attributes to be specified
        // this would be the place to set their value.
        // $POLICY$  	
        if ( !this.attributes.containsKey( aName ) )
    	{
            // $POLICY$ 
            if ( !this.elementDecl.getAttributeDecls().containsKey( aName ) )
            {
                // POLICY - How to handle undeclared attributes
                //		A.  Define default attribute according to XML 1.0 specification.
                //		B.  Throw an InvalidAttribute Exception
                //		C.  Ignore the attribute completly and optionally log it.
                char policy_selection = 'A'; /* $FIX$ */
                switch ( policy_selection )
                {
                case 'A':
                    AttributeDecl newAttributeDecl = 
                      new AttributeDecl( elementDecl, aName, new AttributeType() );
                    elementDecl.getAttributeDecls().put( aName, newAttributeDecl );
                    break;
                case 'B':
                    Object[] args = { aName, name };
                    String msg = MessageFormat.format( 
                           Messages.element_attribute_undeclared, args );
                    AttributeException aBad = new AttributeException( msg );
                    throw aBad; 
                case 'C':
                    // $FIX$ $LOG$ log the ignored attribute.
                    return;
                }		
            }
            HashMap ad = elementDecl.getAttributeDecls();
            AttributeDecl theAttributeDecl = (AttributeDecl) ad.get(aName);
            AttributeDefault adf = theAttributeDecl.getDefaultValue();
            theAttribute = new Attribute();
            theAttribute.setAttributeDecl(theAttributeDecl);
            theAttribute.setElement(this);
            theAttribute.setName(aName);
            if ( adf.getHasDefaultValue() )
            {   
                theAttribute.setValue(adf.getDefaultValue());
            }
            this.attributes.put(aName, theAttribute);
    	}
    	else
    	    theAttribute = (Attribute) attributes.get( aName );
        try 
        {
            theAttribute.setValue( aValue );
        }
        catch ( AttributeException badAttribute )
        {
            throw badAttribute;
        }
    }

    public void setName( String newName ) throws ElementException
    {
        /* $REMOVED$ how should namespaces be handled?
        if ( !name.equals(newName) && doc.dtd.elementDecls.containsKey( prefix + name ) )
        {
            Object args[] = { newName };
            String msg = MessageFormat.format( Messages.element_conflict, args);
            ElementException badElement = new ElementException( msg );
            throw badElement;
        }
        */
        name = newName;
    }

    public String getName() {  return name;  }
    
    public void setPrefix( String p ) throws ElementException
    {
        /* $REMOVED$ - how should Namespaces be handled?
        if ( !prefix.equals(p) && !doc.nameSpaces.containsKey( prefix ) )
    	{
            Object args[] = { name, prefix, p };
            String msg = MessageFormat.format( Messages.element_prefix_conflict, args);
            ElementException bad = new ElementException( msg );
            throw bad;
    	}
    	*/
    	prefix = p;
    }
    
    public String getPrefix( ) { return prefix; }
    
    public boolean getIsRoot() { return isRoot;	}
    public void setIsRoot( boolean rooted ) { isRoot = rooted;  }
    
    public boolean getIsEmpty() { return isEmpty; }
    public void setIsEmpty( boolean e ) { isEmpty = e; }

    public boolean getIsAny() { return isAny; }
    public void setIsAny( boolean a ) { isAny = a; }
    
    public void setParent( Element p ) { parent = p; }
    public Element getParent( ) { return parent; }
    
    public Object getContent( int ind ) { return content.get(ind); }
    
    public int getCount( ) { return content.size(); }
    
    public int getIndex() { return index; }
      
    /*$REMOVED - element should NOT depend on XMLdoc 
    public XMLdoc getDoc() { return this.doc; }
    
    public void setDoc( XMLdoc aDoc ) { this.doc = aDoc; }
    */
    
    public DocType getDtd() { return this.dtd; }
    
    public void setDtd( DocType theDTD ) 
    { 
        // $FIX$ this will work for right now, but we really need to revalidate
        // the content against the incoming dtd.
        this.dtd = theDTD; 
    }
    
    public int getNest() { return this.nest; }
    
    public void setNest( int newNest ) { this.nest = newNest; }
    
    public boolean getOkToFormat() { return this.okToFormat; }
    
    public void setOkToFormat( boolean format ) { this.okToFormat = format; }
    
    
    /**
    * clearContent erases all of the content in this element.  This is
    * a draconian act and should be used with care.  
    *
    **/
    public void clearContent()
    {
        this.parseStack = null;
        this.content = null;
        this.index = 0;
    }
    private void setupAttributes( ) {
        // run through the element decl and setup the parameters
        // for this element.  Set the parameters to their default values.
        if ( this.elementDecl == null )
            return;
        HashMap at = this.elementDecl.getAttributeDecls();
        Collection atc = at.entrySet();
        Iterator atci = atc.iterator();
        
        while ( atci.hasNext() )
        {
            Map.Entry me = (Map.Entry) atci.next();
            String aName = (String) me.getKey();
            AttributeDecl ad = (AttributeDecl) me.getValue();
            AttributeDefault adf = ad.getDefaultValue();
            
            try {
                if ( adf.getHasDefaultValue() )
                {   
                        this.setAttribute(aName, adf.getDefaultValue() );
                }
                else
                {
                    this.setAttribute(aName, null );
                }
            } catch ( AttributeException ae )
            {
                    //$LOG$
            }
        }
    }
    
    /*----------------------------------------------------------------------+
     |                                                                      |
     |               	Content management methods                          |
     |                                                                      |
     +----------------------------------------------------------------------*/

    /*
     * Element Open and Close routines.
     *
     */
    public boolean open( )
    {
        // 
        // An element can be opened if it is the root; or if it has no
        // parent.  Having no parent indicates that it is a fragment being
        // constructed for insertion later.  Permit an null-parented element
        // to be opened as if it were a root.
        //
        if ( this.isRoot || (parent == null) )
        {
            this.isOpen = true;
            return true;
        }
        // parent is not null.
        if ( parent.isOpen )
        {
            this.isOpen = true;
            return true;
        }
        return false;
    }
    
    // close - return previous open state and close the element.
    public boolean close( )
    {
        boolean wasOpen = this.isOpen;
        this.isOpen = true;
        return wasOpen;
    }
    /*
     * The following routines determine if an ElementException will be thrown in an
     * attempt to insert the element's content.
     */

    public boolean addElementTypeValid( String name )
    {
        return addStringValid( name );
    }
    
    public boolean addElementValid( Element e )
    {
        return addStringValid( e.getName() );
    }
    
    public boolean addCommentValid( String c )
    {
        return addStringValid( "#Comment" );
    }
    
    public boolean addPIValid( String pi )
    {
        return addStringValid( "#PI" );
    }
    
    private String getReferenceName( ContentReference cr )
    {
        String s = "";
        switch( cr.getCtype().get() )
        {
        case ContentType.ELEMENT:       // XML element
            s = ((Element) cr.getRef()).getName();
            break;
        case ContentType.COMMENT:       // XML comment
        case ContentType.PROCINST:      // processing instruction declaration
            s = "#SKIP";
            break;
        case ContentType.GENENTITYREF:  // general entity reference
            // this is the tough one.  can't really parse it yet;
            s = "#SKIP";
            break;
        case ContentType.CHARREF:       // character reference
        case ContentType.CHARDATA:      // regular characters (not markup)
        case ContentType.CDATA:         // CDATA section
            s = "#PCDATA";
            break;
        default:
            s = "";
            break;
        }
        return s;
    }
    
    public boolean addContentValid( ContentReference cr )
    {
        switch( cr.getCtype().get() )
        {
            case ContentType.ELEMENT: // Can I insert this element type?
                // look up the element name.
                return addElementValid( (Element) cr.getRef());
            case ContentType.CDATA:	    // Character Data section
            case ContentType.CHARDATA:  // character data.
            case ContentType.CHARREF:   // insert those characters.
                // either characters, a CDATA section, or a character reference.
                // all of these are permitted if "#PCDATA" is permitted.
                return addStringValid( "#PCDATA");
            case ContentType.GENENTITYREF: // must be general entity ref here.
                //
                // A general entity reference may contain any partially 
                // well formed collection of elements, etc.  In theory, we
                // should parse the the content to determine if it will
                // be acceptable.
                // The way to do this is to evaluate the reference and
                // parse it.  Do this later.$FIX$
                return true;
            default: // everything else is invalid
                return false;
        }
    }
    
    // 
    // This lowlevel routine checks to see if the terminal symbol can be added to
    // the element's content.  It is used by addContentValid above.
    //
    public boolean addStringValid(String theTerminal ) 
    {
        if ( theTerminal.equals("#Comment") )
        {
            return true;
        }
        else if ( theTerminal.equals("#PI") )
        {
            return true;
        }
        // ok check for other stuff
        LRparser theParser = this.getParser();
        if ( theParser != null)
        {
           return theParser.acceptString(theTerminal, parseStack);
        }
        // the parser is null indicating that either the content model is ANY or 
        // there is no parser for the content model yet.
        if ( isAny )
            return true;
        // can't insert if empty, or you haven't initialized the element yet.
        return false;  // if it must be empty, then theTerminal cannot be added.
    }
    
    /*
     * The following routines add ("insert") content if the content model for 
     * the element says it is ok.  
     *
     */
     
    /**
     * Add (insert) an element at the index specified.  
     *
     * @param index - integer from 0 to content.size() where the element is
     *      to be inserted.  Note that index = 0 inserts before the first
     *      element; while index = content.size inserts after the last element.
     * @param element - Element to be inserted.
     *
     * @exception - ElementException is thrown if the insertion is not 
     *      permitted by the content model of this element.
     *
     * @note - because comments and processing instructions can be inserted;
     *      at any point, care should be taken in using indexes.
     **/
    public void add( int index, Element e ) throws ElementException
    {
        Stack newStack = null;
    	if ( index < 0 || index > content.size() )
    	{
            Object args[] = { name, new Integer( 0 ), new Integer( content.size()),
                              new Integer(index) };
            String msg = MessageFormat.format( Messages.element_index_invalid, args);
            ElementException bad = new ElementException( msg );
            throw bad;
    	}
    	try {
    	    newStack = this.updateContentModel(INSERT, index, e );
    	}
    	catch ( ElementException ee ) { throw ee; }
    	// got here so add is valid.  Do it!

    	// set the new stack as well
    	this.parseStack = newStack;  // set the new stack
    	// setup the new element
    	e.setParent( this );
    	/* $REMOVED - 
    	e.doc = this.doc;
    	// make sure that the current element is us!
     	e.doc.setCurrent( e );
     	*/
        // insert e in this element's content list.
    	content.add(index, e);		// put it in this element's content list
    }
    
    public void add( int index, Comment cm ) throws ElementException
    {
        // comments are permitted almost anywhere.
        content.add( index, cm );
    }

    public void add( int index, PI pi ) throws ElementException
    {
        content.add( index, pi );
    }
    

    public void add(int index, ContentReference newContent) throws ElementException
    {
        Stack newStack = null;
    	if ( index < 0 || index > content.size() )
    	{
            Object args[] = { name, new Integer( 0 ), new Integer( content.size()),
                              new Integer(index) };
            String msg = MessageFormat.format( Messages.element_index_invalid, args);
            ElementException bad = new ElementException( msg );
            throw bad;
    	}
    	try {
    	    newStack = this.updateContentModel(INSERT, index, newContent);
    	}
    	catch ( ElementException e ) { throw e; }
    	// got here so add is valid.  Do it!
    	content.add( index, newContent );
    	// set the new stack as well
    	this.parseStack = newStack;  // set the new stack
    }
    
    
    public void add( ContentReference newContent) throws ElementException
    {
    	try { this.add( index, newContent ); }
    	catch ( ElementException ee ) {	throw ee; }
    	index++;
    }
    
    public void add( Element el ) throws ElementException
    {
    	try { this.add( index, el ); }
    	catch ( ElementException ee ) {	throw ee; }
    	index++;
    }
    
    public void add( Comment comment ) throws ElementException
    {
        try { this.add(index, comment); }
        catch ( ElementException ee ) { throw ee; }
        index++;
    }
    
    public void add( PI pi ) throws ElementException
    {
        try { this.add(index, pi); }
        catch ( ElementException ee ) { throw ee; }
        index++;
    }
    
    //
    // replace operations
    //

    public void replace(int index, Element e) throws ElementException
    {
        Stack newStack = null;
    	if ( index < 0 || index > content.size() )
    	{
            Object args[] = { name, new Integer( 0 ), new Integer( content.size()),
                              new Integer(index) };
            String msg = MessageFormat.format( Messages.element_index_invalid, args);
            ElementException bad = new ElementException( msg );
            throw bad;
    	}
    	try {
    	    newStack = this.updateContentModel(REPLACE, index, e);
    	}
    	catch ( ElementException ee ) { throw ee; }
    	// got here so replace is valid.  Do it!
    	content.set( index, e );
    	// set the new stack as well
    	this.parseStack = newStack;  // set the new stack
    }

    public void replace(int index, ContentReference newContent) throws ElementException
    {
        Stack newStack = null;
    	if ( (index < 0) || (index >= content.size()) )
    	{
            Object args[] = { name, new Integer( 0 ), new Integer( content.size()),
                              new Integer(index) };
            String msg = MessageFormat.format( Messages.element_index_invalid, args);
            ElementException bad = new ElementException( msg );
            throw bad;
    	}
    	try {
    	    newStack = this.updateContentModel(REPLACE, index, newContent);
    	}
    	catch ( ElementException e ) { throw e; }
    	// got here so replace is valid.  Do it!
    	content.set( index, newContent );
    	// set the new stack as well
    	this.parseStack = newStack;  // set the new stack
    }
    
    public void replace( int index, Comment cm ) throws ElementException
    {
        if ( (index < 0) || ( index >= content.size() ))
        {
            Object args[] = { "Replace comment", 
                              new Integer( 0 ), 
                              new Integer( content.size() ),
                              new Integer( index ) 
                            };
            String msg = MessageFormat.format( Messages.element_index_invalid, args);
            ElementException bad = new ElementException( msg );
            throw bad;
        }
        Object ob = content.get( index );
        if ( ( ob instanceof Comment ) || ( ob instanceof PI) )
        {
            content.set(index, cm);  // replace the old with the new.
        }
        else // this is effective delete except we delete and replace with comment.
        {
            Stack newStack = null;
            try { newStack = this.updateContentModel(DELETE, index, content.get(index)); }
            catch ( ElementException e ) { throw e; }
            content.set( index, cm );
            this.parseStack = newStack;
        }
    }
   
    public void replace( int index, PI pi ) throws ElementException
    {
        if ( (index < 0) || ( index >= content.size() ))
        {
            Object args[] = { "Replace PI", 
                              new Integer( 0 ), 
                              new Integer( content.size() ),
                              new Integer( index ) 
                            };
            String msg = MessageFormat.format( Messages.element_index_invalid, args);
            ElementException bad = new ElementException( msg );
            throw bad;
        }
        Object ob = content.get( index );
        if ( ( ob instanceof Comment ) || ( ob instanceof PI) )
        {
            content.set(index, pi);  // replace the old with the new.
        }
        else // this is effective delete except we delete and replace with comment.
        {
            Stack newStack = null;
            try { newStack = this.updateContentModel(DELETE, index, content.get(index)); }
            catch ( ElementException e ) { throw e; }
            content.set( index, pi );
            this.parseStack = newStack;
        }
    }
    
    //
    // delete operations
    //

    public void delete(int index) throws ElementException
    {
        Stack newStack = null;
    	if ( index < 0 || index > content.size() )
    	{
            Object args[] = { name, new Integer( 0 ), new Integer( content.size()),
                              new Integer(index) };
            String msg = MessageFormat.format( Messages.element_index_invalid, args);
            ElementException bad = new ElementException( msg );
            throw bad;
    	}
    	try {
    	    Object item = content.get(index);
    	    if ( item instanceof Element )
    	        newStack = this.updateContentModel(DELETE, index, (Element) item );
    	    else if ( item instanceof ContentReference )
    	        newStack = this.updateContentModel(DELETE, index, (ContentReference) item);
    	    else if ( item instanceof Comment )
    	        newStack = this.updateContentModel(DELETE, index, (Comment) item );
    	    else if ( item instanceof PI )
    	        newStack = this.updateContentModel(DELETE, index, (PI) item );
    	    else 
    	    {
    	        // serious logic error, item in content is unknown type.
    	        // this should never happen.
    	        // $FIX$ throw an elementException!
    	    }
    	}
    	catch ( ElementException e ) { throw e; }
    	// got here so delete is valid.  Do it!
    	content.remove( index );
    	
    	this.parseStack = newStack;  // set the new stack
    }

    
    /*
     * updateContentModel checks to see if the content model for this element 
     * can be updated.  If it can, the new stack representing the update is
     * returned.  If it cannot, an appropriate elementException is thrown.
     *
     * The parameter, newIndex, must be in the range 0 .. content.size().
     *
     * The parameter's meaning depends on the updateType as follows:
     *
     *  DELETE -  check to see if content at newIndex can be deleted. If 
     *            targetContent is non-null, then it must be identical ( i.e.
     *            same object as that at newIndex.  Thus, newIndex must be
     *            in the range 0 .. (content.size()-1).
     *  INSERT -  check to see if targetContent at newIndex can be inserted.
     *            targetContent must be non-null and it is the content that
     *            is inserted.
     *  REPLACE - check to see if targetContent at newIndex can replace the
     *            existing content.  newIndex must be in the range
     *            0..(content.size() - 1);
     *            
     */
     
    
    private Stack updateContentModel( byte updateType, int newIndex, 
                                      Object target ) throws ElementException
    {
        String targetName = "";
        String targetErrorName = "";
        
        // first identify the target object class
        if ( target instanceof Element )
        {
            targetName = ((Element) target).getName();
            targetErrorName = targetName;
        }
        else if ( target instanceof ContentReference )
        {
            ContentReference cr = (ContentReference) target;
            ContentType ct = cr.getCtype();
            targetName = this.getReferenceName(cr);
            switch ( ct.get() )
            {
            case ContentType.ELEMENT:       // XML element
                targetErrorName = targetName;
                break;
            case ContentType.COMMENT:       // XML comment
                {
                    targetErrorName = "Comment ";
                    String theComment = ((Comment) cr.getRef()).get();
                    int theLast = ( theComment.length() > 10 ) ? 10 : theComment.length() - 1;
                    if ( theLast > 0 )
                        targetErrorName += "|" + theComment.substring( 0, theLast ) + "|";
                }
                break;
            case ContentType.PROCINST:      // processing instruction declaration
                targetErrorName = ( (PI) cr.getRef() ).getTarget();
                break;
            case ContentType.GENENTITYREF:  // general entity reference
                targetErrorName = "Entity ";
                break;
            case ContentType.CHARREF:       // character reference
                targetErrorName = "Character ";
                break;
            case ContentType.CHARDATA:      // regular characters (not markup)
                targetErrorName = "CharData ";
                break;
            case ContentType.CDATA:         // CDATA section
                targetErrorName = "CDATA ";
                break;
            default:
                targetErrorName = "Unknown ";
                break;
            }
        }
        else if ( target instanceof Comment )
        {
            targetName = "#SKIP";
            targetErrorName = "Comment ";
            String theComment = ((Comment) target).get();
            int theLast = ( theComment.length() > 10 ) ? 10 : theComment.length() - 1;
            if ( theLast > 0 )
                targetName += "|" + theComment.substring( 0, theLast ) + "|";
        }
        else if ( target instanceof PI )
        {
            targetErrorName = ( (PI) target).getTarget();
            targetName = "#SKIP";
        }
        else
        {
            targetErrorName = "Logic Error, unknown content object.";
            targetName = "";
        }
            
        LRparser theParser = this.getParser();
        if ( !this.isOpen )
        {   // You have to open it first!
            Object args[] = { name, targetErrorName };
            String msg = MessageFormat.format( Messages.element_not_open, args);
            ElementException bad = new ElementException( msg );
            throw bad;
            //You have to open it first!
        }
        if ( theParser == null )
        {
            // consider the cases where the ContentSpec is EMPTY or ANY.
            // EMPTY:  Cannot delete, replace or insert -- throw exception.
            // ANY:    Can delete, replace or insert -- with no restrictions.
            if ( this.getIsEmpty() )
            {
                Object args[] = { name, targetErrorName };
                String msg = MessageFormat.format( Messages.element_is_empty, args);
                ElementException bad = new ElementException( msg );
                throw bad;
            }
            else if ( this.getIsAny() )
            {
                return null; // no sense in parsing -- any old thing is ok.
            }
            Object args[] = { name, Messages.frag_NoParser };
            String msg = MessageFormat.format( Messages.element_logic_error, args);
            ElementException bad = new ElementException( msg );
            throw bad;  
        }
        if ( (newIndex < 0) || (newIndex > this.content.size()) )
        {   // element insertion must be from 0 to content.size(); newIndex isn't in range.
            Object args[] = { name, new Integer(0), 
                              new Integer(content.size()), new Integer( newIndex) };
            String msg = MessageFormat.format( Messages.element_index_invalid, args);
            ElementException bad = new ElementException( msg );
            throw bad;
            // newIndex must be in range from 0 to content.size();
        }
        // use following if you want to accept indices < 0 or > content.size().
        if ( false )
            newIndex = ( newIndex > this.content.size())? this.content.size(): 
                        (newIndex < 0)? 0 : newIndex ;
        // build a temporary stack and reparse the input!
        Stack saveStack = parseStack;
        // 
        parseStack = new Stack( );
        parseStack.ensureCapacity( LRconst.STACK_DEPTH );
        parseStack.push( (ParseState) new ParseState( (short) 0, null ));
        String s = null;
        boolean failure = false;
        
        try {
            for ( int i = 0; i < content.size() ; i++ )
            {   // get the content item
                Object item = content.get(i);
                // if we are at the action point, figure out what to do.
                if ( i == newIndex )
                {   // check to see if the new element can be updated here.
                    // if not valid - an LRparserException will be thrown.
                    s = targetName;
                    switch( updateType)
                    {
                    case INSERT:
                        if ( targetName.equals(""))
                        {   
                            // caller is confused.  -- called with bad content.
                            Object args[] = { name, Messages.frag_InvalidContent };
                            String msg = MessageFormat.format( Messages.element_logic_error, args);
                            ElementException bad = new ElementException( msg );
                            throw bad;  
                        }
                        // by inserting the parse step here, it effectively inserts
                        // the content.
                        theParser.parseStepString( targetName, parseStack );
                        break;
                    case DELETE:
                        if ( target != null )
                        {   // if targetContent is not null then it must be the item.
                            if ( item != target)
                            {   // the caller is confused about the deletion.
                                Object args[] = { name, targetErrorName + " DELETE" };
                                String msg = MessageFormat.format( Messages.element_content_invalid, args);
                                ElementException bad = new ElementException( msg );
                                throw bad;
                            };
                        }
                        // treat as if it weren't there -- that is continue.
                        continue;   // start back at top with a new iteration.
                    case REPLACE:
                        item = target; // temporarily replace the item with the target.
                        break;
                    default:
                        // we are confused.  -- called with an invalid updateType.
                        Object args[] = { name, Messages.frag_UpdateTypeInvalid };
                        String msg = MessageFormat.format( Messages.element_logic_error, args);
                        ElementException bad = new ElementException( msg );
                        throw bad;  
                    }
                }
                if ( item instanceof Element )
                {
                    s = ((Element) item).getName();
                    theParser.parseStepString(s, parseStack);
                }
                else if ( item instanceof ContentReference )
                {
                    s = getReferenceName( (ContentReference) item );
                    if ( s.equals("") )
                    {   // serious error -- unknown reference type in content or replace.
                        // $LOG$ the error.
                        Object args[] = { name, Messages.frag_InvalidContent };
                        String msg = MessageFormat.format( Messages.element_logic_error, args);
                        ElementException bad = new ElementException( msg );
                        throw bad;  
                    }
                    if ( !s.equals("#SKIP") )
                    {   // the following may throw an LRparserException
                        theParser.parseStepString(s, parseStack);
                    }
                } // end item is a ContentReference
                else if ( item instanceof Comment )
                {
                    // treat as #skip; 
                }
                else if ( item instanceof PI )
                {
                    // treat as #skip;
                }
                else
                {
                    // item is unknown.  It should be a Comment, PI, ContentReference or
                    // an element.  Assume that the constructor of the
                    // content placed it here intentionally.
                    // $LOG$ the event
                    Object args[] = { name, Messages.frag_InvalidContent };
                    String msg = MessageFormat.format( Messages.element_logic_error, args);
                    ElementException bad = new ElementException( msg );
                    throw bad;  
                }
            } // end for loop
        }
        catch ( LRparserException lrpe )
        {   // insertion of the element is invalid.
            Object args[] = { name, s };
            String msg = MessageFormat.format( Messages.element_content_invalid, args);
            ElementException bad = new ElementException( msg );
            throw bad;
        }
        // we made it through the current content model and it parsed.
        // so the update is valid.  Restore the savedStack (The caller is
        // under no obligation to complete the update.) and return
        // the successful stack in case the caller wants to do the update.
        Stack tempStack = parseStack;
        parseStack = saveStack;
        return tempStack;        
    }    
    
    //
    // Attribute Management
    //
    //      addAttribute( String aName, String aValue ) throws AttributeException
    //
    public void addAttribute( String aName, String aValue ) throws AttributeException
    {
        this.setAttribute(aName, aValue);
    }
    
    public void deleteAttribute(String aName) throws AttributeException
    {
        if ( attributes == null )
        {
            attributes = new HashMap();
            setupAttributes();
        }
    	// make sure that an attribute declaration exists....
        //
        // if the policy were to allow undeclared attributes to be specified
        // this would be the place to set their value.
        // $POLICY$  	
        if ( !attributes.containsKey( aName ) )
    	{
            // $POLICY$ 
            if ( !elementDecl.getAttributeDecls().containsKey( aName ) )
            {
                // POLICY - How to handle undeclared attributes
                //		A.  Define default attribute according to XML 1.0 specification.
                //		B.  Throw an InvalidAttribute Exception
                //		C.  Ignore the attribute completly and optionally log it.
                char policy_selection = 'A'; /* $FIX$ */
                switch ( policy_selection )
                {
                case 'A':
                    AttributeDecl newAttributeDecl = 
                        new AttributeDecl( elementDecl, aName, new AttributeType() );
                    elementDecl.getAttributeDecls().put( aName, newAttributeDecl );
                    break;
                case 'B':
                    Object[] args = { aName, name };
                    String msg = MessageFormat.format( 
                           Messages.element_attribute_undeclared, args );
                    AttributeException aBad = new AttributeException( msg );
                    throw aBad; 
                case 'C':
                    // $FIX$ $LOG$ log the ignored attribute.
                    return;
                }		
            }
            HashMap ad = elementDecl.getAttributeDecls();
            AttributeDecl theAttributeDecl = (AttributeDecl) ad.get(aName);
            AttributeDefault adf = theAttributeDecl.getDefaultValue();
            
            if ( adf.getHasDefaultValue() )
                attributes.put(aName, adf.getDefaultValue());
            else
                attributes.put(aName, "" );
            return;
                
        }
        // set the "deleted" attribute value back to it's default
        HashMap ad = elementDecl.getAttributeDecls();
        AttributeDecl theAttributeDecl = (AttributeDecl) ad.get(aName);
        AttributeDefault adf = theAttributeDecl.getDefaultValue();
                
        try {
            if ( adf.getHasDefaultValue() )
            {   
                this.setAttribute(aName, adf.getDefaultValue() );
            }
            else
            {
                this.setAttribute(aName, "" );
            }
        } 
        catch ( AttributeException ae )
            {  throw ae;  }
    }
    
    public void replaceAttribute(String aName, String aValue) throws AttributeException
    {
        if ( attributes == null )
        {
            attributes = new HashMap();
            setupAttributes();
        }
    	// make sure that an attribute declaration exists....
        //
        // if the policy were to allow undeclared attributes to be specified
        // this would be the place to set their value.
        // $POLICY$  	
        if ( !attributes.containsKey( aName ) )
    	{
            // $POLICY$ 
            if ( !elementDecl.getAttributeDecls().containsKey( aName ) )
            {
                // POLICY - How to handle undeclared attributes
                //		A.  Define default attribute according to XML 1.0 specification.
                //		B.  Throw an InvalidAttribute Exception
                //		C.  Ignore the attribute completly and optionally log it.
                char policy_selection = 'A'; /* $FIX$ */
                switch ( policy_selection )
                {
                case 'A':
                    AttributeDecl newAttributeDecl = 
                         new AttributeDecl( elementDecl, aName, new AttributeType() );
                    elementDecl.getAttributeDecls().put( aName, newAttributeDecl );
                    break;
                case 'B':
                    Object[] args = { aName, name };
                    String msg = MessageFormat.format( 
                           Messages.element_attribute_undeclared, args );
                    AttributeException aBad = new AttributeException( msg );
                    throw aBad; 
                case 'C':
                    // $FIX$ $LOG$ log the ignored attribute.
                    return;
                }		
            }
            HashMap ad = elementDecl.getAttributeDecls();
            AttributeDecl theAttributeDecl = (AttributeDecl) ad.get(aName);
            AttributeDefault adf = theAttributeDecl.getDefaultValue();
                
            if ( adf.getHasDefaultValue() )
            {  
                attributes.put(aName, adf.getDefaultValue());
            }
            else
            {
                attributes.put(aName, "" );
            }
    	}
    	Attribute theAttribute = (Attribute) attributes.get( aName );
        try 
        {
            theAttribute.setValue( aValue );
        }
        catch ( AttributeException badAttribute )
        {
            throw badAttribute;
        }
    }

    /*----------------------------------------------------------------------+
     |                                                                      |
     |               	XMLexpandable Interface implementation              |
     |                                                                      |
     +----------------------------------------------------------------------*/

    public boolean canDoCharData() { return false; 	}
    public String toCDSect() { return ""; }
    public String toCharData() { return ""; }
	
    public String toXMLstring()
    {
        final String EOL = "\n";
    	// 
    	// Generate the start tag
    	//
    	StringBuffer startTag = new StringBuffer( 10 + prefix.length() + name.length() +
    	       attributes.size() * Constants.EXPECTED_ATTRIBUTE_LENGTH );
    	       
    	if ( (this.parent == null) || this.parent.getOkToFormat() )
    	{
    	    startTag.append( getWhiteSpace( this.getNest(), 2 ) );
    	}
    	startTag.append("<");
    	if ( prefix.length() > 0)
    		startTag.append( prefix + ":");
    	startTag.append( name );
    	//	
    	//		Generate the attributes
    	//
    	if ( attributes.size() > 0 )
    	{
            Iterator i = attributes.entrySet().iterator();
            while ( i.hasNext() )
            {
                Map.Entry me = (Map.Entry) i.next();
                if ( me.getValue() != null )
                {
                    Attribute theAttr = (Attribute) me.getValue();
                    //
                    // if the value of an attribute is null, then it was
                    // defined but never set.  This meant that it was
                    // #IMPLIED or we have logged an error (or we should have!).
                    // In any case, only include attributes if they have non-null values.
                    if ( theAttr.getValue() != null )
                        startTag.append( " " + theAttr.toXMLstring() );
                }
            }
    	}
    	//
    	//		Close the start or empty tag.  If empty, return 
    	//
    	if ( isEmpty )
    	{
    	    // include an extra space before the ender incase XHTML is being generated.
            startTag.append(" />") ;
            return startTag.toString();
    	}
    	startTag.append(" >") ;
    	if ( this.getOkToFormat() )
    	    startTag.append(EOL);
    	//
    	// Generate the content
    	//
        StringBuffer interior = new StringBuffer( 
                          content.size() * Constants.EXPECTED_ELEMENT_CHARDATA_LENGTH );
            
    	for ( int j = 0; j < content.size(); j++ )
    	{
            Object c = content.get(j);
            if ( c instanceof Element )
    	    {
    	        Element el = (Element) c;
    	        
    	        if ( this.getOkToFormat() )
    	        {
    	            el.setNest( this.getNest() + 1 );
    	        }
                interior.append( el.toXMLstring() );
                if ( this.getOkToFormat() )
                    interior.append(EOL);
    	    }
    	    else if ( c instanceof ContentReference )
    	    {
    	        ContentReference cref = (ContentReference) c;
    	        
    	        if ( this.getOkToFormat() )
    	        {
    	            cref.setNest( this.getNest() + 1 );
    	        }
                interior.append( cref.toXMLstring() );
                if ( this.getOkToFormat() )
                    interior.append(EOL);
            }
            else
    	    {
                interior.append( "$FIX$ illegal content in element" );
                if ( this.getOkToFormat() )
                    interior.append(EOL);
    	    }
    	}
    	//
    	//	Generate the end tag with optional namespace prefix
    	//
    	StringBuffer endTag = new StringBuffer( 100 );
    	if ( this.getOkToFormat() )
    	{
    	    endTag.append( getWhiteSpace( this.getNest(), 2 ) );
    	}
    	endTag.append("</" + ((prefix.length() > 0) ? prefix + ":" : "") + name  +" >" );
    	//
    	// return the element
    	//
    	return startTag.toString() + interior.toString() + endTag.toString();
    }
    
    public String toLog()
    {
    	// $FIX$ what goes in here?
        return this.toXMLstring();
    }
    
    //
    // silly formatting routine that should be removed at some point
    //
    
    public static String getWhiteSpace( int nest, int indentSize )
    {   //                     0123456789012345678901234567890123456789
        //                     0         1         2         3
        final String blanks = "                                        ";
        
 
        if ( nest == 0 ) 
            return "";
            
        if ( (nest * indentSize) <= blanks.length() )
            return blanks.substring( 0, (nest * indentSize ) - 1 );
            
        StringBuffer moreBlanks = new StringBuffer( nest * indentSize );
        for ( int i = 0; i < moreBlanks.length(); i++ )
            moreBlanks.setCharAt(i, ' ');
        return moreBlanks.toString();
    }
    
    
}