/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/XMLassembly/ElementDecl.java,v 1.1.1.1 2000/09/19 19:39:50 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

/*-------------------------------------------------------------------+
 |                                                                   |
 |                     Element Declaration Class                     |
 |                                                                   |
 +-------------------------------------------------------------------*/

package com.adobe.rice.common.XMLassembly;

import java.util.*;
import java.io.*;
import java.text.*;
//$JDK1.1$ import com.sun.java.util.collections.*;
import com.adobe.rice.common.Parsergen.*; 				// grammar, symbol, production

//<class-dependencies>
//  <class> ContentSpec </>
//  <class> ContentAction </>
//  <class> DocType </>
//  <class> Grammar </>
//  <class> LRparser </>
//  <class> XMLscanner </>
//</class-dependencies>

public class ElementDecl implements Serializable, XMLexpandable
{
    private String        name;            // element name
    private ContentSpec   contentSpec;     // element's content specification
    private HashMap       attributeDecls;  // element's attributes
    private ContentAction assemblyAction;
    private boolean       isRoot = false;  // element is the root element.
    private boolean       isEmpty = false; // element has empty contentSpec
    private boolean       isAny = false;   // element has ANY contentSpec.
    private boolean	  isUndeclared = true; // no element declaration has been seen 
                                           // but the element type has been referenced.
	
    private DocType       dtd = null;      // owning dtd
    private XMLscanner    scan = null;     // scanner

    private int		      count = 0;	   // counts total number of CP's generated.
    private Grammar	      grammar = null;  // non-null if a content model in place.
    private LRparser      lrParser = null; // non-null if an LR parser is ready to roll:

    public ElementDecl( )
    {
        name = "";
        contentSpec = null;
        assemblyAction = null;
        isRoot = false;
        isEmpty = false;
        isAny = true;
        isUndeclared = false;
        attributeDecls = new HashMap(Constants.EXPECTED_ELEMENT_ATTRIBUTE_COUNT);
        dtd = null;
        scan = null;
        count = 0;
    }

    public ElementDecl( String newName )
    {
            this();
            name = newName;
    }
	
    public ElementDecl( DocType aDtd )
    {
        this();
        dtd = aDtd;
        if ( dtd != null )
            scan = dtd.getScan();
    }	
	
    public ElementDecl( DocType aDtd, String newName )
    {
        this();
        name = newName;
        dtd = aDtd;
        if ( dtd != null )
            scan = dtd.getScan();
    }
	
    public ElementDecl( DocType aDtd, String newName, ContentAction myAction )
    {
        this();
        name = newName;
        assemblyAction = myAction;
        dtd = aDtd;
        if ( dtd != null )
            scan = dtd.getScan();
    }		
	
    //
    // setters and getters
    //
    public DocType getDtd( ) { return dtd; }
    public void setDtd( DocType d ) { dtd = d; if ( dtd != null ) scan = dtd.getScan(); }
    
    public String getName() { return name; }
    public void setName( String s ) { name = s; }
	
    public ContentSpec getContentSpec() { return contentSpec; }
    public void setContentSpec( ContentSpec cs ) { contentSpec = cs; }
	
    public HashMap getAttributeDecls() { return attributeDecls; }
	
    public ContentAction getContentAction() { return assemblyAction;  }
    public void setContentAction( ContentAction ca ) { assemblyAction = ca; }
    
    public boolean getIsRoot( ) { return isRoot; }
    public void setIsRoot( boolean aRoot ) { isRoot = aRoot; }

    public boolean getIsEmpty() { return isEmpty; }
    public void setIsEmpty( boolean ise ) { isEmpty = ise; }

    public boolean getIsAny() { return isAny; }
    public void setIsAny( boolean isa ) { isAny = isa; }

    public boolean getIsUndeclared() { return isUndeclared; }
    public void setIsUndeclared( boolean isU ) { isUndeclared = isU; }
    
    public XMLscanner getScan() { return scan; }
    public void setScan( XMLscanner s ) { scan = s; };

    public int getCount() { return count; }
    public void setCount( int c ) { count = c; }
    public void incrementCount( int c ) { count += c; }
    public void incrementCount( ) { count++; }

    public Grammar getGrammar() { return grammar; }
    public void setGrammar( Grammar g ) { grammar = g; }

    public LRparser getLrParser() { return lrParser; }
    public void setLrParser( LRparser pr ) { lrParser = pr; }

    //
    // implements XMLexpandable interface:
    //
    public boolean canDoCharData() { return false; }
    public String toCDSect() { return ""; }
    public String toCharData() { return ""; }

    public String toXMLstring()
    {
        String sed = "<!ELEMENT " + name + " ";
        sed += contentSpec.toXMLstring() + " >\n";

        // now add in the attribute declarations:
        if ( !attributeDecls.isEmpty() )
        {
            sed += "<!ATTLIST " + name + " \n";
            Iterator adsit = attributeDecls.keySet().iterator();
            while ( adsit.hasNext() )
            {
                AttributeDecl atde = (AttributeDecl) attributeDecls.get( adsit.next() );
                sed += "        " + atde.toXMLstring() + " \n";
            }
            sed += ">\n";
        }
        return sed;
    }
    //
    //	Recursive descent parser Routines
    //
    
    //
    // elementdecl ::= '<!ELEMENT' S Name S contentSpec S? '>'
    //
    // parse enters with t.string.equals("ELEMENT").
    // parse leaves with t.string.equals(">").
    public int parse( ScanToken t ) throws XMLparseException
    {
    	int	count = 0;
    	String msg = null;
    	XMLparseException pBad = null;
    	
        if ( !t.string.equals("ELEMENT") )
            return count;
    	t = scan.getToken();
    	if ( t.type != ScanToken.NAME )
    	{
            Object[] args = { "ELEMENT name", t.string };
            msg = MessageFormat.format( Messages.unexpected, args );
            pBad = new XMLparseException( msg );
            throw pBad; 
        }    	
		// now have the element name
    	name = t.string;
    	// get a new ContentSpec
    	contentSpec = new ContentSpec(ContentSpec.CP_EMPTY, this );
    	// Get a new token and go parse the content spec.
        t = scan.getToken();
    	try { count = contentSpec.parse( t ); }
    	catch ( XMLparseException p ) { throw p; }
        if ( t.type != ScanToken.MARKUP_END )
        {
            Object[] args = { ">", t.string };
            msg = MessageFormat.format( Messages.unexpected, args );
            pBad = new XMLparseException( msg );
            throw pBad;
        }
    	return count;
    }

    //
    // Element Declaration Complete 
    //
    //  1.  Convert Content Model to Left Recursive Grammar
    //  2.  Construct the LR1 parse tables
    //  3.  Create Parser
    //  4.  Store a copy of the parser
    //
    // So how will this be used?
    //
    // Caller invokes XMLdoc and points at DTD.  Eventually inserts an element which is
    // declared with this elementDecl.  Caller can do the following:
    //
    //      Let item be any of 
    //          XMLdoc -- Content Model: Prolog? element ( PI | Comment )*
    //          Prolog -- Content Model: XMLDecl?, (PI | Comment )*, ( doctypedecl?, (PI | Comment)*)? 
    //          XMLDecl -- Content Model:  VersionInfo, EncodingDecl?, SDDecl? 
    //          doctypedecl -- Content Model: Name, ExternalID? ( '[' (markupdecl | PEReference )* ']' )? 
    //          PI -- Content Model: PITarget ( char* - ( char* '?>' char*) )?
    //          TextDecl - Content Model: VersionInfo? EncodingDecl
    //          ExtSubsetDecl - Content Model: ( markupdecl | conditionalSect | PEReference )*
    //          IgnoreSect - IGNORE section.
    //          IncludeSect - INCLUDE section
    //          Comment - Content Model: ( char* - ( char* '-->' char* ) )?
    //          Element - Content Model: EmptyElemTag | STag content ETag
    //          EmptyElemTag - Content Model: name (Attribute)*
    //          STag    - Content Model: name ( Attribute )*
    //          ETag - Content Model: name
    //          Attribute - Content Model:  name '=' AttrValue
    //          AttrValue - Content Model:  '"' ( char* - ( char* '"' char* ) ) '"' | 
    //                                      "'" ( char* - ( char* "'" char* ) ) "'"
    //          content - Content Model: (element | CharData | Reference | CDSect | PI | Comment)*
    //          CharData -- Content Model: character data ( Content Reference ) escaping <, &. 
    //          CDSect -- Content Model: CDATA section ( Content Reference ) escaping "]]>" if needed.
    //          EntityRef -- Content Model: General Entity reference
    //          CharRef -- Content Model: Character Entity reference )
    //          PEReference -- Content Model: Parameter Entity reference.
    //
    //  1.  Ask for a list of items that can be inserted as the content.  Each unique item type
    //      appears once in the list (i.e. #CDATA appears once, element names A and B both appear).
    //  2.  Ask if ok to insert item; if ok, true; otherwise, false is returned.
    //  3.  Insert item;  if item is illegal at this point, an Exception is thrown.
    //      (Which exception? should be a ContentException! $FIX$).
    //      Item insertion causes an action depending on the item.  This is 
    //      explained in comments in XMLdoc.
    //  4.  Ask if ok to close element.  returns true if ok, false if not.
    //      If in a sequence of elements, and not complete, then false can be returned.
    //  5.  Close element.  If close is illegal at this point, an Exception is thrown.
    //      (Which exception? should be a ContentException! $FIX$).


    //
    // complete - called when the element Decl scan has been completed.
    //
    public void complete( )
    {
        // Examine the content specification to determine the type of content model:
        switch ( contentSpec.type )
        {
            case Cp.CP_NAME:
                // technically this is illegal here:  $FIX$
                break;
            case Cp.CP_CHOICE:
            case Cp.CP_SEQ:
            case Cp.CP_MIXED:
                // Convert Content Model to Left Recursive Grammar
                contentSpec.genGrammar();
                // Construct LR1 parse tables, create parser and store it
                lrParser = new LRparser( grammar );
                this.setIsAny(false);
                this.setIsEmpty(false);
                break;
            case ContentSpec.CP_ANY:
                // no need for a parser if any old content is allowed.
                this.setIsAny( true );
                this.setIsEmpty(false);
                break;
            case ContentSpec.CP_EMPTY:
                // no need for a parser if no content is allowed.
                this.setIsEmpty( true );
                this.setIsAny(false);
                break;
            default:
                // This should NEVER happen -- $FIX$
                break;
        }
        // It is now fully declared!
        this.setIsUndeclared( false );
    }

    //
    // Entry Validation Routines
    //
    /**
    @parameter contentName - String name of content item.
    @function This routine determines if it is legal to insert an elment named
    **/

    static final byte Y = 1;        // Yes - content is ok
    static final byte N = 0;        // No - content not permitted
    static final byte L = 2;        // Look up in parse

    static final byte icv[][] = 
    {   /*                  +---+---+---+---+---+---+ 
                            | N | C | S | M | A | E |                 
                            | A | H | E | I | N | M |
                            | M | O | Q | X | Y | P |
                            | E | I |   | E |   | T |
                            |   | C |   | D |   | Y |
                            |   | E |   |   |   |   |
                            +---+---+---+---+---+---+    */
        /* ATTRIBUTE  08 */ { L , L , L , L , L , L },
        /* ATTRVALUE  09 */ { N , N , N , N , N , N },
        /* ELEMENT    10 */ { L , L , L , L , Y , N }, 
        /* COMMENT    11 */ { Y , Y , Y , Y , Y , N },
        /* PI         12 */ { Y , Y , Y , Y , Y , N },
        /* ENTITYREF  13 */ { N , N , N , L , Y , N },
        /* CHARREF    14 */ { N , N , N , L , Y , N },
        /* CHARDATA   15 */ { N , N , N , L , Y , N },
        /* CDATA      16 */ { N , N , N , L , Y , N },
    };

    static final byte base = ContentType.ATTRIBUTE;
    
    boolean contentInsertionValid( ContentType ct, String contentName )
    {
        // Examine the content specification to determine the type of content model:
        byte typeIndex = (byte) ( ct.get() - base ) ;
        // validate typeIndex
        if ( (typeIndex <= 0) || (typeIndex > (ContentType.CDATA - base) ))
        {
            // the content type is out of range
            // $POLICY$ -- As a matter of policy, throw an exception or return false.
            return false;
        }
        switch ( icv[ typeIndex ][ contentSpec.type ] )
        {
            case Y: return true;    // Content type is permitted in this context
            case N: return false;   // Content type is not permitted.
            case L: break;          // Must look in the content model
            default:                // icv array lookup failed....
                // the icv value is invalid.
                // $FATAL$ -- This is a really serious error
                return false;       // pretend that it is a no.
        }
        // got here because we need to look in the content specification
        switch ( contentSpec.type )
        {
            case Cp.CP_NAME:
                // return true if names are equal
                switch( typeIndex )
                {
                    case (ContentType.ATTRIBUTE - base):
                        // look up name in attribute name list
                        return attributeDecls.containsKey( contentName );
                     case ( ContentType.ELEMENT - base ):
                        return contentName.equals( contentSpec.getName() );
                    default:
                        return false;
                }
                // DEADPOINT
            case Cp.CP_CHOICE:
                switch( typeIndex )
                {
                    case (ContentType.ATTRIBUTE - base):
                        // look up name in attribute name list
                        break;
                     case ( ContentType.ELEMENT - base ):
                        return contentName.equals( contentSpec.getName() );
                    default:
                        return false;
                }
                break;
            case Cp.CP_SEQ:
                switch( typeIndex )
                {
                    case (ContentType.ATTRIBUTE - base):
                        // look up name in attribute name list
                        break;
                     case ( ContentType.ELEMENT - base ):
                        return contentName.equals( contentSpec.getName() );
                    default:
                        return false;
                }
                break;
            case Cp.CP_MIXED:
                switch( typeIndex )
                {
                    case (ContentType.ATTRIBUTE - base):
                        // look up name in attribute name list
                        break;
                     case ( ContentType.ELEMENT - base ):
                        return contentName.equals( contentSpec.getName() );
                    default:
                        return false;
                }
                break;
            case ContentSpec.CP_ANY:
                switch( typeIndex )
                {
                    case (ContentType.ATTRIBUTE - base):
                        // look up name in attribute name list
                        break;
                    case ( ContentType.ATTRVALUE - base):
                        return false;
                }
                return true;
            case ContentSpec.CP_EMPTY:
                switch( typeIndex )
                {
                    case (ContentType.ATTRIBUTE - base):
                        // look up name in attribute name list
                        break;
                }
                return false;
            default:
                // $FATAL$ -- This should NEVER happen -- $FIX$
                break;
        }
    	return false;	
    }
    
    boolean replaceContentValid( String contentType )
    {
    	return false;
    }


}

















