/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/XMLassembly/EntityDecl.java,v 1.1.1.1 2000/09/19 19:39:50 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/
/*----------------------------------------------------------------+
 |                                                                |
 |                   Entity Declaration  Class                    |
 |                                                                |
 +----------------------------------------------------------------*/

package com.adobe.rice.common.XMLassembly;

import java.util.*;
import java.io.*;
import java.text.*;
//$JDK1.1$ import com.sun.java.util.collections.*;

public class EntityDecl implements Serializable, XMLexpandable
{
    private String name = "";
    private String value = null;
    private boolean parameter = false;
    private boolean external = false;
    private boolean parsed = false;
    private String publicID = "";
    private String systemID = "";
    private boolean hasPublicID = false;
    private boolean hasSystemID = false;
    
    private String ndataName = "";
    private boolean hasNdataName = false;
    
    private DocType dtd = null;

    public EntityDecl()
    {
        name = "";
        value = "";
        parameter = false;
        parsed = false;
        //
        publicID = "";
        systemID = "";
        external = false;
        //
        hasPublicID = false;
        hasSystemID = false;
        //
        dtd = null;
    }
    
    public EntityDecl(DocType d)
    {
        this();
        dtd = d;
    }
	
    public EntityDecl( DocType d, String n )
    {
        this();
        dtd = d;
        name = n;
    }

    public EntityDecl(DocType d, String n, String v)
    {
    	this();
    	dtd = d;
        name = n;
        value = v;
    }

    public EntityDecl(DocType d, String n, String v, boolean p)
    {
    	this();
    	dtd = d;
        name = n;
        value = v;
        parameter = p;
    }

    public EntityDecl(DocType d, String n, String v, String s, String p)
    {
    	this();
    	dtd = d;
        name = n;
        value = v;
        //
        publicID = p;
        systemID = s;
        external = hasPublicID() || hasSystemID();
    }
    //
    // get & set methods
    //
    public String getValue() { return value; }
    public String getName()  { return name; }
    public boolean getExternal() { return external; }
    public boolean getParameter() { return parameter; }
    public boolean getParsed() { return parsed; }
    public String getPublicID(){ return publicID; }
    public String getSystemID(){ return systemID; }

    public void setName(String newName) { name = newName; }
    public void setValue(String newValue) { value = newValue; }
    public void setPublicID(String newPublicID) { publicID = newPublicID; }
    public void setSystemID(String newSystemID) { systemID = newSystemID; }
    public void setParsed( boolean isParsed ) { parsed = isParsed; }
    public void setParameter(boolean newParameter) { parameter = newParameter; }
    //
    // methods
    //
    public boolean hasSystemID()
    {
        if (systemID != null)
        {
            hasSystemID = (systemID.length() > 0) ? true : false;
        }
        else
        {
            hasSystemID = false;
        }
        return hasSystemID;
    }

    public boolean hasPublicID()
    {
        if (publicID != null)
        {
            hasPublicID = (publicID.length() > 0) ? true : false;
        }
        else
        {
            hasPublicID = false;
        }
        return hasPublicID;
    }

    public boolean canDoCharData() { return false; }
    public String toCDSect() { return ""; }
    public String toCharData() { return ""; }

    public String toXMLstring()
    {
        String ende = "<!ENTITY ";
        if ( parameter ) 
            ende += "% ";
        ende += name + " ";
        if ( external )
        {
            if ( hasPublicID )
                ende += "PUBLIC \"" + publicID + "\" \"" + systemID + "\" ";
            else
                ende += "SYSTEM \"" + systemID + "\" ";
            // ndata only allowed on general entities.
            if ( hasNdataName && !parameter )
                ende += "NDATA " + ndataName + " ";
        }
        else // entityvalue
        {
            ende += "\"" + value + "\"";
        }
        ende += " >";
        return ende;
    }
    //
    // Recursive descent parsing routines
    //
    // EntityDecl  ::=  GEDecl | PEDecl
    //     GEDecl  ::=  '<!ENTITY' S Name S EntityDef S? '>'
    //     PEDecl  ::=  '<!ENTITY' S '%' S Name S PEDef S? '>'
    //
    //  EntityDef  ::=  EntityValue | (ExternalID NDataDecl?)
    //      PEDef  ::=  EntityValue | ExternalID
    //
    // parse enters with t.string.equals("ENTITY") as a BANG_NAME.
    // parse leaves with t.string.equals(">") as MARKUP_END.
    public int parse( ScanToken t ) throws XMLparseException
    {
        int count = -1;
        XMLscanner scan = dtd.getScan();
        int gotOne = 0;
        String msg = null;
        XMLparseException pBad;
	
        if ( !t.string.equals("ENTITY") )
            return count;
        t = scan.getToken();
        if ( t.string.equals("%") )
        {   // it's a parameter entity declaration
            parameter = true;
            t = scan.getToken();
            if ( t.type != ScanToken.NAME )
            {	// parameter entity name must follow the percent.
                Object[] args = { "name (for parameter entity)", t.string };
                msg = MessageFormat.format( Messages.unexpected, args );
                pBad = new XMLparseException( msg );
                throw pBad; 
            }
            name = t.string;
            t = scan.getToken();	// now look for value
            if ( t.type == ScanToken.QUOTED_STRING )
            {
                value = t.string;
                parsed = true;		// we've parsed it.....
                t = scan.getToken();
            }
            else if ( t.string.equals("SYSTEM") || t.string.equals("PUBLIC") )
            {
                try { gotOne = parseExternalID( t ); }
                catch ( XMLparseException p ) { throw p; }
                external = true;
                parsed = false; // we have to go retrieve it and fit it in....
                t = scan.getToken();
            }
            else
            {
                Object[] args = { "value (quoted) or External ID", t.toString() };
                msg = MessageFormat.format( Messages.unexpected, args );
                pBad = new XMLparseException( msg );
                throw pBad; 
            }			
        }
        else if ( t.type == ScanToken.NAME )
        {   // it's a general entity
            name = t.string;		// collect the name
            t = scan.getToken();	// now look for value
            if ( t.type == ScanToken.QUOTED_STRING )
            {
                value = t.string;
                parsed = true;		// we've parsed it.....
                t = scan.getToken();
            }
            else if ( t.string.equals("SYSTEM") || t.string.equals("PUBLIC") )
            {
                try { gotOne = parseExternalID( t ); }
                catch ( XMLparseException p ) { throw p; }
                external = true;
                parsed = false; // First retrieve it, then parse it.
                // scan for an optional 
                // NDataDecl ::= S 'NDATA' S Name
                t = scan.getToken();
                if ( t.string.equals("NDATA") )
                {
                    t = scan.getToken();
                    if ( t.type == ScanToken.NAME )
                    {
                        ndataName = t.string;
                        hasNdataName = true;
                    }
                    else
                    {	// expecting NDATA name
                        Object[] args = { "NDATA name", t.string };
                        msg = MessageFormat.format( Messages.unexpected, args );
                        pBad = new XMLparseException( msg );
                        throw pBad; 
                    }
                    t = scan.getToken();
                }
                // now we have a token for the next parser particle.
            }
            else
            {
                Object[] args = { "value (quoted) or External ID", t.string };
                msg = MessageFormat.format( Messages.unexpected, args );
                pBad = new XMLparseException( msg );
                throw pBad; 
            }
        }
        else
        {   // it's a syntax error
            Object[] args = 
            { "name (general entity) or '%' (parameter entity)", t.string };
            msg = MessageFormat.format( Messages.unexpected, args );
            pBad = new XMLparseException( msg );
            throw pBad; 
        }
        // note that the next token has already been obtained.
        if ( !t.string.equals(">") )
        {
            Object[] args = { "'>' to close Entity declaration", t.string };
            msg = MessageFormat.format( Messages.unexpected, args );
            pBad = new XMLparseException( msg );
            throw pBad; 
        }
        return count;	
    }
    //
    // ExternalID ::= 'SYSTEM' S SystemLiteral
    //             |  'PUBLIC' S PubidLiteral S SystemLiteral 
    //
    public int parseExternalID( ScanToken t ) throws XMLparseException
    {
    	int count = 0;
    	String msg = null;
    	XMLparseException pBad = null;
    	XMLscanner scan = dtd.getScan();
    	
    	if ( t.type == ScanToken.NAME )
    	{
            if ( t.string.equals("SYSTEM") )
    	    {
                // 'SYSTEM' SystemLiteral
                t = scan.getToken();
                if ( t.type != ScanToken.QUOTED_STRING )
                {
                    Object[] args = { "System literal value (quoted)", t.string };
                    msg = MessageFormat.format( Messages.unexpected, args );
                    pBad = new XMLparseException( msg );
                    throw pBad; 
                }
                systemID = t.string;
                hasSystemID = true;
                count = 1;
            }
            else if ( t.string.equals("PUBLIC") )
    	    {
                // 'PUBLIC' S PubidLiteral S SystemLiteral 
                t = scan.getToken();
                if ( t.type != ScanToken.QUOTED_STRING )
                {
                    Object[] args = { "PubId literal value (quoted)", t.string };
                    msg = MessageFormat.format( Messages.unexpected, args );
                    pBad = new XMLparseException( msg );
                    throw pBad; 
                }
                publicID = t.string;
                hasPublicID = true;
                // get SystemLiteral
                t = scan.getToken();
                if ( t.type != ScanToken.QUOTED_STRING )
                {
                    Object[] args = { "System literal value (quoted)", t.string };
                    msg = MessageFormat.format( Messages.unexpected, args );
                    pBad = new XMLparseException( msg );
                    throw pBad; 
                }
                systemID = t.string;
                hasSystemID = true;
                count = 1;
            }
            else
    	    {	// name must be either SYSTEM or PUBLIC -- this is in error
                Object[] args = { "SYSTEM or PUBLIC", t.string };
                msg = MessageFormat.format( Messages.unexpected, args );
                pBad = new XMLparseException( msg );
                throw pBad; 
            }
    	}
    	return count;
    }		
}









