/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/XMLassembly/ExtSubset.java,v 1.3 2000/11/27 20:20:41 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/
/*----------------------------------------------------------------------+
 |                                                                      |
 |                extSubset - External DTD subset Class                 |
 |                                                                      |
 +----------------------------------------------------------------------*/

/**
 * extSubset.java
 *
 *
 * Created: Sun Dec 12 12:16:58 1999
 *
 * @author V. Bruce Hunt
 * @version 0.01
 */

package com.adobe.rice.common.XMLassembly;

import java.util.*;
import java.io.*;
import java.text.*;
import java.net.*;
//$JDK1.1$ import com.sun.java.util.collections.*;

/**
    extSubset ::= TextDecl? extSubsetDecl

    TextDecl ::= '<?xml' VersionInfo? EncodingDecl S? '?>'

    extSubsetDecl ::= ( markupdecl | conditionalSect | PEReference | S )*

**/
public class ExtSubset implements Serializable, XMLexpandable
{
    // begin TextDecl:
    String byteOrderMark = "\uFEFF";        /* optional UTF16 encoding */
    String version       = Constants.XML_VERSION;
    String encoding      = Constants.XML_ENCODING;
    // end TextDecl;
    //
    // private fields
    //
    private List            markup = new ArrayList(Constants.EXPECTED_MARKUP_DECLARATIONS);
    private BufferedReader  reader = null;
    private XMLscanner      scan = null;
    private String          subsetURL = null;
    private DocType         dtd = null;

    //
    // constructors
    //
    public ExtSubset()
    {
        version = Constants.XML_VERSION;
        encoding = Constants.XML_ENCODING;
    }

    public ExtSubset( String url )
    {
        this();
        subsetURL = url;
    }
    //
    // methods
    //
    //
    // getters and setters
    //
    public String getByteOrderMark() { return byteOrderMark; }
    public String getVersion() { return version; }
    public String getEncoding() { return encoding; }
    
    //
    //     $FIX$ The following set parameters should be validated against legal values
    //    
    public void setByteOrderMark( String bom ) { byteOrderMark = bom; }
    public void setVersion( String v ) { version = v; }
    public void setEncoding( String e ) { encoding = e; }
    //
    //  Internal fields
    //
    public void setReader( BufferedReader r ) { reader = r; }
    public BufferedReader getReader() { return reader; }
    public void setScan( XMLscanner s ) { scan = s; }
    public XMLscanner getScan() { return scan; }
    public void setSubsetURL( String d ) { subsetURL = d; }
    public String getSubsetURL() { return subsetURL; }
    public void setDtd( DocType d ) { dtd = d; }
    public DocType getDtd() { return dtd; }
  
    //
    //  Implement expandable:
    //
    public boolean canDoCharData() { return false; }
    public String toCDSect() { return ""; }
    public String toCharData() { return ""; }

    public String toXMLstring()
    {
        String textDecl = "<?xml " + 
            "version = \"" + version + "\" " +
            "encoding = \"" + encoding + "\" " +
            "?>\n";
    
        // 
        //   Add in the external subset
        //
        String docdecl = "";
        for ( int i = 0; i < markup.size(); i++ )
        {
            MarkUpDecl mud = (MarkUpDecl) markup.get(i);
            docdecl += mud.toXMLstring() + "\n";
        }
        return ( textDecl + docdecl );
    }
    
    //
    // Methods
    //

    // 
    // $FIX$ - hack to support local and remote
    //
    private boolean isFilePath( String suspect )
    {
        if ( suspect.charAt(0) == '/' )
            return true;
        if ( suspect.charAt(0) == '.' )
            return true;
        if ( suspect.startsWith("http:") )
            return false;
        return true;
    }
    /**
     * @author Bruce Hunt
     * @version 0.01 (11-01-2000)
     * @see XMLscanner
     *
     * @exception IOException
     * @exception MalformedURLException 
     * @exception XMLparseException
     * @exception FileNotFoundException
     * 
     **/    
    public void parseSetup( ) 
        throws MalformedURLException, IOException, XMLparseException, FileNotFoundException
    {  
      
        InputStream     inputStream;
        
        if ( isFilePath( this.subsetURL ) )
        {
            try { inputStream = setupLocalStream( this.subsetURL ); }
            catch ( XMLparseException xmlpe ) { throw xmlpe; }
            catch ( FileNotFoundException fnfe ) { throw fnfe; }
        }
        else
        {
            try { inputStream = setupRemoteStream( this.subsetURL ); }
            catch ( MalformedURLException mfue ) { throw mfue; }
            catch ( IOException ioe ) { throw ioe; }
        }
        
        this.reader = new BufferedReader( new InputStreamReader( inputStream ) );
  
        this.setScan(new XMLscanner( reader )); 
        //$CHECK$
        this.getDtd().setScan( this.scan );
        
        // $DEBUG$
        this.scan.setDebug( (byte)3 );
    }
    
    private InputStream setupRemoteStream( String url ) throws MalformedURLException, IOException
    {
        // set the URL
        URL               thisDTD;
        URLConnection     thisConnect;
        XMLparseException pBad;
        InputStream       inputStream;

        try 
        {
            thisDTD = new URL(url);
            thisConnect = thisDTD.openConnection();
            inputStream = thisConnect.getInputStream();
        } 
        catch (MalformedURLException e) 
        {   // url is poorly formed 
            throw e;
        } 
        catch (IOException e) 
        {   // openConnection() failed
            throw e;
        } 
        return inputStream;
    }
    
    private InputStream setupLocalStream(String filePath) throws XMLparseException, FileNotFoundException
    {
        //
        // 1.  get a file for the path
        // 
        File dtdFile = new File(filePath);
        //
        // 2.  Make sure that the DTD file exists
        //
        if ( !dtdFile.exists() )
        {
            // This is a FATAL icecubes error
            XMLparseException fatalError = new XMLparseException(
                "IceCubes Fatal Error" + "ICE DTD file, " + 
                filePath + ", does not exist!  " 
                );
            throw fatalError;
        }
        //
        // 3.  Make sure that it is a file
        //
        if ( !dtdFile.isFile() )
        {
            // This is also a Fatal icecubes error
            XMLparseException fatalError = new XMLparseException(
                "IceCubes Fatal Error" + "ICE DTD file, " + 
                filePath + ", is not a file!  " 
                );
            throw fatalError;
        }
        // construct an input stream for the file.
        FileInputStream dtdStream = new FileInputStream(dtdFile);
        return dtdStream;
    }
    /**
     * @author Bruce Hunt
     * @version 0.01 (14-08-1999)
     * @see XMLscanner
     *
     * @exception XMLparseException
     * 
     **/    
    public int parse( ) throws XMLparseException
    {  
        //
        // build the external DTD subset
        //
        int    count = 0;        // count parse items obtained.
        ScanToken t = null;      // scan token holder 
        // get the external subset dtd and set it in the scanner.
        scan.setDtd( this.getDtd() );
        // get a token to start off the parse.
        t = scan.getToken();
        // go attempt to get an TextDecl
        try { count = parseTextDecl( t ); }
        catch ( XMLparseException pTBad ) { throw pTBad; }
        // if we got one then get a new token
        if ( count > 0 )
            t = scan.getToken();
        // now get a dtd external subset
        try { count = this.parseExtSubset( t ); }
        catch ( XMLparseException pTBad ) { throw pTBad; }
        
        return count;
    }

    //
    // ExtSubset Recursive Descent Parse Methods
    //
    //  All the methods work as follows:
    //
    //  return 1 if a valid parse of the non-terminal occured.
    //  return 0 if the non-terminal didn't parse.
    //    in this case the caller is expected to attempt another
    //    alternative.  The caller may be required to back up the scan.
    //
    private int parseTextDecl( ScanToken t ) throws XMLparseException
    {
        String msg = null;
        XMLparseException pBad;
  
        if ( t == null )
            t = scan.getToken();
        // check for '<?xml' that signals a TextDecl
        if ( t.status != ScanToken.OK ) 
            return 0;
        if ( t.type != ScanToken.HUH_NAME )
            return 0;
        if ( !t.string.toLowerCase().equals("xml") )
            return 0;
        /* have '<?xml', now get version = {xmlVersion} ... */
        // from here on, a parsing error will throw an exception
        t = scan.getToken();
        if ( t.status != ScanToken.OK )
        {   //
            // TextDecl error
            //
            Object[] args1 = { "XML version", t.string };
            msg = MessageFormat.format( Messages.tokenError, args1 );
            pBad = new XMLparseException( msg );
            throw pBad; 
        }
        // version is optional in a textDecl!
        if ( (t.type == ScanToken.NAME) && ( t.string.equals("version") ) )
        {   //
            t = scan.getToken();
            if ( ! t.string.equals("=") )
            {
                Object[] args3 = { "=", t.string };
                msg = MessageFormat.format( Messages.unexpected, args3 );
                pBad = new XMLparseException( msg );
                throw pBad; 
            }
            t = scan.getToken();
            if ( t.type != ScanToken.QUOTED_STRING )
            {
                Object[] args4 = { "quoted value", t.string };
                msg = MessageFormat.format( Messages.unexpected, args4 );
                pBad = new XMLparseException( msg );
                throw pBad; 
            }
            if ( ! t.string.equals(Constants.XML_VERSION) )
            {
                Object[] args5 = { Constants.XML_VERSION, t.string };
                msg = MessageFormat.format( Messages.unexpected, args5 );
                pBad = new XMLparseException( msg );
                throw pBad; 
            }
            version = t.string;
            t = scan.getToken();
        }
        //
        // now get the encoding -- which is required!
        //
        if ( !t.string.equals("encoding" ) )
        {
                Object[] args6 = { "encoding", t.toString() };
                msg = MessageFormat.format( Messages.unexpected, args6 );
                pBad = new XMLparseException( msg );
                throw pBad;
            
        }
        t = scan.getToken();
        if ( ! t.string.equals("=") )
        {
            Object[] args6 = { "=", t.toString() };
            msg = MessageFormat.format( Messages.unexpected, args6 );
            pBad = new XMLparseException( msg );
            throw pBad;
        }
        t = scan.getToken();
        if ( t.type != ScanToken.QUOTED_STRING )
        {
            Object[] args7 = { "Quoted encoding value", t.string };
            msg = MessageFormat.format( Messages.tokenError, args7 );
            pBad = new XMLparseException( msg );
            throw pBad;
        }
        this.encoding = t.string;
        // $FIX$ Make sure value is legal

        t = scan.getToken();

        if ( !t.string.equals("?>") )
        {
            Object[] args11 = { "?>", t.toString() };
            msg = MessageFormat.format( Messages.unexpected, args11 );
            pBad = new XMLparseException( msg );
            throw pBad;
        }
        return 1;
    }
    //
    // extSubsetDecl ::= ( markupdecl | conditionalSect | PEReference | S )*
    //
    //  markupdecl is parsed by class MarkUpDecl
    //  conditionalSect is parsed as part of the scanner.
    //  PEReference is parsed as part of the scanner.
    //  S is whitespace and is handled by the scanner.
    // 
    public int parseExtSubset( ScanToken t ) throws XMLparseException
    {
        int        count = 0;
        int        gotPMD = 0;
        MarkUpDecl markUpDecl = new MarkUpDecl( dtd, scan );

        scan.setInDTD(true);
        do
        {   // get next token
            try { gotPMD = markUpDecl.parse( t ); }
            catch ( XMLparseException p ) { throw p; }
            if ( gotPMD > 0 )
            {
                markup.add(markUpDecl);
                count++;
                // get a new one for next try
                markUpDecl = new MarkUpDecl( dtd, scan );
                t = scan.getToken();
            }
        }
        while ( gotPMD > 0 );
        // no longer scanning DTD.
        scan.setInDTD(false);
        // markUpDecl = null;
      return count;
    }
    
    /**
     * return the first element name encountered as the default root 
     * element. 
     *
     **/
    public String getDefaultRoot()
    {
        MarkUpDecl mud = null;
        
        for ( int i = 0; i < this.markup.size(); i++ )
        {
            mud = (MarkUpDecl) markup.get(i);
            if ( mud.getType() == mud.MUD_ELEMENT )
            {
                ElementDecl ed = (ElementDecl) mud.get();
                return ed.getName();
            }
            
        }
        return null;
    }
    
} // extSubset














