/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/XMLassembly/MarkUpDecl.java,v 1.2 2000/11/22 02:05:43 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/
/*----------------------------------------------------------------------------------+
 |                                                                                  |
 |                             MarkUp Declaraction Class                            |
 |                                                                                  |
 +----------------------------------------------------------------------------------*/

package com.adobe.rice.common.XMLassembly;

import java.util.*;
import java.io.*;
import java.text.*;
//$JDK1.1$ import com.sun.java.util.collections.*;


public class MarkUpDecl implements Serializable
{
    // 
    //	Markup Decl types
    //
    public final static byte MUD_INVALID = -1;
    public final static byte MUD_ELEMENT = 0;
    public final static byte MUD_ATTRIBUTE = 1;
    public final static byte MUD_ENTITY = 2;
    public final static byte MUD_NOTATION = 3;
    public final static byte MUD_PI = 4;
    public final static byte MUD_COMMENT = 5;
    
    //
    // fields
    //
    
    private ElementDecl elementDecl = null;
    private ElementDecl elementAttributeDecl = null;
    private EntityDecl entityDecl = null;
    private NotationDecl notationDecl = null;
    private PI pi = null;
    private String comment = "";
    private byte type = MUD_ELEMENT;
    
    //
    // private work fields
    //
    XMLscanner scan = null;
    DocType dtd = null;
    
    MarkUpDecl()
    {
    	elementDecl = null;
    	elementAttributeDecl = null;
    	entityDecl = null;
    	notationDecl = null;
    	pi = null;
    	comment = "";
        dtd = null;
        type = this.MUD_INVALID;
    }
    
    MarkUpDecl( XMLscanner aScanner )
    {
    	this();
        scan = aScanner;
    }

    MarkUpDecl( DocType aDTD )
    {
        this();
        dtd = aDTD;
    }

    MarkUpDecl( DocType aDTD, XMLscanner aScanner )
    {
        this();
        dtd = aDTD;
        scan = aScanner;
    }

    public void setDtd( DocType aDTD ) { dtd = aDTD; }
    public void setScan( XMLscanner aScanner ) { scan = aScanner; }
    
    public byte getType( ) { return this.type; }
    
    public Object get( )
    {
        switch( this.type )
        {
            case  MUD_INVALID:
                return null;
            case  MUD_ELEMENT:
                return this.elementDecl;
            case  MUD_ATTRIBUTE:
                return this.elementAttributeDecl;
            case  MUD_ENTITY:
                return this.entityDecl;
            case  MUD_NOTATION:
                return this.notationDecl;
            case MUD_PI:
                return this.pi;
            case MUD_COMMENT:
                return this.comment;
        }
        return null;    // $FIX$ Fundamental error here!
    }
    
    //
    // implement XMLexpandable
    //

    public boolean canDoCharData() { return false; }
    public String  toCDSect() { return ""; }
    public String  toCharData() { return ""; }

    public String toXMLstring()
    {
        String mud = "";
        switch( type )
        {
            case MUD_ELEMENT:
                if ( elementDecl != null )
                    mud = elementDecl.toXMLstring();
                break;
            case MUD_ATTRIBUTE:
                if ( elementAttributeDecl != null )
                {
                    mud = "<!-- $FIX$ Invalid Attribute decl in MarkUpDecl: \n";
                    mud += elementAttributeDecl.toXMLstring();
                    mud = "\n--> ";
                }
                break;
            case MUD_ENTITY:
                if ( entityDecl != null )
                    mud = entityDecl.toXMLstring();
                break;
            case MUD_NOTATION:
                if ( notationDecl != null )
                    mud = notationDecl.toXMLstring();
                break;
            case MUD_PI:
                if ( pi != null )
                    mud = pi.toXMLstring();
                break;
            case MUD_COMMENT:
                if ( comment.length() > 0 )
                    mud = "<!-- " + comment + " -->";
                break;
            default:
                mud = "<!-- $FIX$ -- unknown markup type in MarkUpDecl.  LOGIC ERROR! -->";
                break;
        }
        return mud;
    }
    
    //
    // recursive descent parsing routines
    //
    //
    // markupdecl ::= elementdecl 
    //			   |  AttlistDecl
    //			   |  EntityDecl 
    //			   |  NotationDecl 
    //			   |  PI
    //             |  Comment 
    //
    int parse( ScanToken t ) throws XMLparseException
    {
    	int count = 0;
    	int gotOne = 0;
    	String msg = null;
    	XMLparseException pBad = null;
    	
    	switch( t.type )
    	{
            case ScanToken.COMMENT:		// '<!--' text '-->'
                this.comment = t.string;
                this.type = MUD_COMMENT;
                count = 1;
                break;
            case ScanToken.BANG_NAME: 	// '<!' name ...
                if ( t.string.equals("ELEMENT") )
                {
                    elementDecl = new ElementDecl( dtd );
                    try { gotOne = elementDecl.parse(t); }
                    catch ( XMLparseException p ) { throw p; };
                    // save the new element definition
                    if ( gotOne > 0 )
                        dtd.addElementDecl( elementDecl );
                    this.type = this.MUD_ELEMENT;
                }	
                else if ( t.string.equals("ATTLIST") )
                {
                    try { gotOne = parseAttListDecl(t); }
                    catch ( XMLparseException p ) { throw p; };
                    this.type = this.MUD_ATTRIBUTE;
                }
                else if ( t.string.equals("ENTITY") )
                {
                    entityDecl = new EntityDecl( dtd );
                    try { gotOne = entityDecl.parse(t); }
                    catch ( XMLparseException p ) { throw p; };
                    if ( entityDecl.getParameter() )
                    {
                        dtd.setParameterEntity( entityDecl.getName(), entityDecl );
                    }
                    else
                    {
                        dtd.setGeneralEntity( entityDecl.getName(), entityDecl );
                    }
                    this.type = this.MUD_ENTITY;
                }
                else if ( t.string.equals("NOTATION") )
                {
                    notationDecl = new NotationDecl( dtd );
                    try { gotOne = notationDecl.parse(t); }
                    catch ( XMLparseException p ) { throw p; };
                    this.type = this.MUD_NOTATION;
                }
                else
                {
                    Object[] args = { "ELEMENT, ATTLIST, ENTITY or NOTATION", t.string };
                    msg = MessageFormat.format( Messages.unexpected, args );
                    pBad = new XMLparseException( msg );
                    throw pBad; 
                }
                count = 1;
                break;
            case ScanToken.HUH_NAME:	// '<?' name ...
                pi = new PI(scan);
                try { gotOne = pi.parse(t); }
                catch ( XMLparseException p ) { throw p; };
                if ( gotOne == 0 )
                {
                    Object[] args = { t.string, "as a Processing Instruction target" };
                    msg = MessageFormat.format( Messages.illegal, args );
                    pBad = new XMLparseException( msg );
                    throw pBad; 
                }
                // put the processing instruction in the table
                dtd.processorInst.put( pi.getTarget(), pi );
                count = gotOne;
                this.type = this.MUD_PI;
                break;
            default:
                count = 0;
    	}
    	return count;
    }
    
    //
    // AttlistDecl ::= '<!ATTLIST' S Name AttDef* S? '>'
    //      AttDef ::= S Name S AttType S DefaultDecl
    //
    // parseAttListDecl enters with t a BANG_NAME with value "ATTLIST".
    // parseAttListDecl leaves with t a MARKUP_END with value ">".
    public int parseAttListDecl( ScanToken t ) throws XMLparseException
    {
    	int	count = 0;
     	int gotOne = 0;
    	String msg = null;
    	XMLparseException pBad = null;
    	
        if ( !t.string.equals("ATTLIST") )
            return count;
    	t = scan.getToken();
    	if ( t.type != ScanToken.NAME )
    	{
            Object[] args = { "ATTLIST element name", t.string };
            msg = MessageFormat.format( Messages.unexpected, args );
            pBad = new XMLparseException( msg );
            throw pBad; 
        }    	
        // now have the element name for this attribute list
        // look up its element declaration 
    	ElementDecl ed = (ElementDecl) dtd.elementDecls.get( t.string );
    	if ( ed == null )
    	{
            Object[] args = { t.string, "an element" };
            msg = MessageFormat.format( Messages.undeclared, args );
            pBad = new XMLparseException( msg );
            throw pBad; 
    	}
    	count = 0;				/* there is a valid (empty) ATTLIST */
    	t = scan.getToken();
    	while ( !t.string.equals(">") )
    	{   // loop scanning for attribute declarations
            AttributeDecl ad = new AttributeDecl( scan, ed );
            try{ gotOne = ad.parse( t ); }
            catch ( XMLparseException p ) { throw p; }
            ed.getAttributeDecls().put( ad.getName(), ad );
            count++;
            t = scan.getToken();
    	}
    	this.elementAttributeDecl = ed;
    	// note that we leave with the current token pointing at the '>' symbol.
       	return count;
    }
}






