/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/XMLassembly/Misc.java,v 1.1.1.1 2000/09/19 19:39:50 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

/*----------------------------------------------------------------------------------+
 |                                                                                  |
 |                               XML Document Misc Class                            |
 |                                                                                  |
 +----------------------------------------------------------------------------------*/

package com.adobe.rice.common.XMLassembly;

import java.io.*;
import java.util.*;
//$JDK1.1$ import com.sun.java.util.collections.*;
import java.text.*;


public class Misc implements Serializable, XMLexpandable
{
    private XMLdoc doc = null;  // $FIX$ this should be removed ASAP
    private String comment = "";
    private PI processingInstruction = null;
    private XMLscanner scan = null;
    
    public Misc( )
    {
    	doc = null;
    	comment = "";
    	processingInstruction = null;
    }
    
    public Misc( String newComment, XMLdoc d )
    {
    	this();
    	doc = d;
    	processingInstruction = null;
    }
    
    public Misc( PI newPI, XMLdoc d )
    {
    	this();
    	doc = d;
    	processingInstruction = newPI;
    }
    
    public Misc( XMLdoc d )
    {
    	this();
    	doc = d;
    }
    
    public Misc( XMLscanner s )
    {
        this();
        scan = s;
    }
    
    //
    // Misc methods
    //
    
    public void setScan( XMLscanner s ) { scan = s; }
    public XMLscanner getScan( ) { return scan; }
    
    //
    // XMLexpandable interface
    //
    
    public boolean canDoCharData() { return false; }
    public String toCDSect() { return ""; }
    public String toCharData() { return ""; }

    public String toXMLstring()
    {
    	if ( comment != null )
    	{
            return "<!--" + comment + "-->\n";
    	}
    	else if ( processingInstruction != null )
    	{
            return processingInstruction.toXMLstring();
    	}
    	else
    	{
            return "";
    	}
    }
    
    public int parse( ScanToken t ) throws XMLparseException
    {	// Misc			::= Comment | PI |  whitespace
        // Comments are handled automatically by the scanner.
        int count = 0;
	
        if ( t.type == ScanToken.COMMENT )
        {
            // place the comment in the Misc.
            comment = t.string;
            ++count;
        }
        // note that we can have either of the following cases:
        // '<?' pitarget ... '?>' or '<?' whitespace pitarget ... '?>'
        // The following handles handles either case. 
        else if ( (t.type == ScanToken.HUH_NAME) || ( t.type == ScanToken.MARKUP_BANG ) )
        {			
            PI aPI = new PI( scan );
            // try to parse the PI; it will throw parseInvalid if it fails
            try
            {
                count = aPI.parse( t );
            }
            catch ( XMLparseException pBad )
            {
                // How to recover from an invalid parse here.
                // don't let someone higher up do it.
                throw pBad;
            }
            if ( count > 0 )
            {
                processingInstruction = aPI;
            }
        }
        return count;
    }	
}







