/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/XMLassembly/Mixed.java,v 1.1.1.1 2000/09/19 19:39:50 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/
/*----------------------------------------------------------------------------------+
 |                                                                                  |
 |                           Mixed  Element  Content  Class                         |
 |                                                                                  |
 +----------------------------------------------------------------------------------*/

package com.adobe.rice.common.XMLassembly;

import java.io.*;
import java.text.*;
import java.util.*;
//$JDK1.1$ import com.sun.java.util.collections.*;
import com.adobe.rice.common.Parsergen.*;


public class Mixed extends ChoiceSeq implements XMLexpandable
{
    protected boolean isRoot = true;	// mixed is always at the root level
    //
    // constructors
    //
    public Mixed()
    {
        super();
        type = CP_MIXED;
        isRoot = true;
    }
  
    public Mixed( ElementDecl e )
    {
        this();
        container = e;
    }
  
    public Mixed( ElementDecl e, ChoiceSeq p )
    {
        this();
        container = e;
        parent = p;
    }
    //
    // getters and setters
    //

    public void setType( byte t ) { type = t; }
    public byte getType() { return type; }
    public void setParent( Cp p ) { parent = p; }
    public Cp getParent() { return parent; }
    public void setIsRoot( boolean r ) { isRoot = r; }
    public boolean getIsRoot() { return isRoot; }

    public void setItems( List l ) { items = l; }
    public List getItems() { return items; }
	
    public Object getItem( int i ) { return items.get(i); }
	
    public int getItemsSize() { return items.size(); }

  
    //
    // methods
    //
  
    //
    // Recursive descent parsing routines
    //
    //
    // contentspec ::= 'EMPTY' | 'ANY' | Mixed | children 
    //
    // Mixed	::= '(' S? '#PCDATA' ( S? '|' S? Name)*  S? ')*' 
    //         |  '(' S? '#PCDATA' S? ')' 
    //
    // children ::= (choice | seq) ('?' | '*' | '+')?
    //
    // choice ::= '(' S? cp ( S? '|' S? cp )* S? ')'
    //
    // seq    ::= '(' S? cp ( S? ',' S? cp )* S? ')'
    //
    // cp     ::= (Name | choice | seq) ('?' | '*' | '+')?
    //
    //
    public int parse( ScanToken t ) throws XMLparseException
    {
        int count = 0;
        int gotOne = 0;
        String msg = null;
        XMLparseException pBad = null;
        byte mcsType = -1;			// keeps track of choice, mixed or seq
        String op = "";				// contains the correct operator
        XMLscanner scan = container.getScan();
        DocType doc = container.getDtd();	// basis of the scan.
 	
        // look for #PCDATA
        if ( !t.string.equals("#") )
        {
            Object[] args = { "'#'", t.string };
            msg = MessageFormat.format( Messages.unexpected, args );
            pBad = new XMLparseException( msg );
            throw pBad;
        }
        // must be Mixed with possible #PCDATA
        t = scan.getToken();
        // $FIX$ note that there could be white space between '#' and 'PCDATA'
        if ( !t.string.equals("PCDATA") )
        {   // bad, must be #PCDATA
            Object[] args = { "#PCDATA", t.string };
            msg = MessageFormat.format( Messages.unexpected, args );
            pBad = new XMLparseException( msg );
            throw pBad; 
        }
        // Don't save the #PCDATA since it's value is recorded 
        // in CP_MIXED.
        mcsType = CP_MIXED;
        count++;
        t = scan.getToken();
        // token must be '|' or ')'
        if ( t.string.equals(")") )
        {
            type = mcsType;
            return count;
        }
        if ( !t.string.equals("|") )
        {
            Object[] args = { "'|' or ')'", t.string };
            msg = MessageFormat.format( Messages.unexpected, args );
            pBad = new XMLparseException( msg );
            throw pBad; 
        }
        // scan: ( '|' name )+ ')*'
        while ( t.string.equals("|") )
        {
            t = scan.getToken();
            if ( t.type != ScanToken.NAME )
            {
                Object[] args = { "Element name", t.string };
                msg = MessageFormat.format( Messages.unexpected, args );
                pBad = new XMLparseException( msg );
                throw pBad; 
            }
            // need to look up the Element Declaration
            ElementDecl ed = doc.getElementDecl( t.string );
            if ( ed == null )
            {   // this is an element that is not yet declared.
                // It is NOT an error (according to XML 1.0 spec.)
                doc.addElementDecl( t.string );
                ed = elementDecl.getDtd().getElementDecl( t.string );
                // keep track so we can list at the end
                ed.setIsUndeclared( true );
            }
            Cp aCp = new Cp( parent, ed );
            aCp.setType( Cp.CP_NAME );
            aCp.setRepetition( Cp.REP_ONCE );
            items.add(aCp);
            count++;
            t = scan.getToken();
        } // while ( t.string.equals("|")
        // must end on a right paren
        if ( t.string.equals(")") )
        {
            t = scan.getToken();
            if ( t.string.equals("*") )
            {
                // getToken here?
                t = scan.getToken();	// point at next
                return count;
            }
        }
        Object[] args = { "closing ')*' for Mixed content specification", t.string };
        msg = MessageFormat.format( Messages.unexpected, args );
        pBad = new XMLparseException( msg );
        throw pBad; 
    }

    // 
    // implements XMLexpandable
    //

    public boolean canDoCharData() { return false; }
    public String toCDSect() { return ""; }
    public String toCharData() { return ""; }

    public String toXMLstring()
    {
        String mx = "( #PCDATA";
        int itemsSize = items.size();
        for ( int i = 0 ; i < itemsSize; i++ )
        {
            Cp mcp = (Cp) items.get(i);
            mx += mcp.toXMLstring();
            if ( (i+1) < itemsSize )
                mx += ", ";
        }
        mx += ")";
        return mx;
    }
    //
    /**
     @method genGrammar
        This routine generates the grammar productions based on the Mixed content content
	model.

	The model of the form:

	Mixed ::= '(' '#PCDATA' ')' 
               |  '(' '#PCDATA' ( '|' Cp )+ ')*'

	generates:

	p1 -> #PCDATA | C0 | C2 | ... | Ci | ... | Ck 
	id -> empty 
            |  p1
            |  id p1

    **/

    public void genGrammar( Grammar g )
    {
        if ( type != CP_MIXED )
            return;

        Symbol pcdata = new Symbol( "#PCDATA", LRconst.TERMINAL );
        // construct the production representation.
        Production csp = null;
        // generate this level's productions
        if ( items.size() > 0 )
        {   // id -> ( #PCDATA | C0 | C1 | ... | ck )*
            // id -> EMPTY
            this.id.setNt( LRconst.NONTERMINAL );
            csp = new Production( this.id );
            csp.add( Symbol.empty );
            g.add( csp );
            // id -> p1
            Symbol p1 = new Symbol( id.get()+"_1", LRconst.NONTERMINAL );
            csp = new Production( p1 );
            csp.add( this.id );
            g.add( csp );
            // p1 -> #PCDATA
            csp = new Production( p1 );
            csp.add( pcdata );
            g.add( csp );
            // p1 -> ( C0 | C1 | C2 ... | Ck )
            // for each i:  ( id -> Ci | id -> id Ci )
            for ( int i = 0; i < items.size() ; i++ )
            {   // id -> Ci
                csp = new Production( p1 );
                Cp cpi  = (Cp) items.get(i);
                csp.add( cpi.getId() );
                g.add( csp );
            }
            // id -> id p1				!! RECURSION !!
            csp = new Production( this.id );
            csp.add( this.id );
            csp.add( p1 );
            g.add( csp );
        }
        else	// id -> #PCDATA	(#PCDATA is a terminal)
        {
            csp = new Production( this.id );
            csp.add( pcdata );
            g.add( csp );
        }
    }
}











