/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/XMLassembly/NotationDecl.java,v 1.1.1.1 2000/09/19 19:39:50 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

/*--------------------------------------------------------------------+
 |                                                                    |
 |                      Notation Declaration Class                    |
 |                                                                    |
 +--------------------------------------------------------------------*/

package com.adobe.rice.common.XMLassembly;

import java.util.*;
import java.io.*;
import java.text.*;
import com.sun.java.util.collections.*;

/** Generated by Together */
public class NotationDecl implements Serializable
{
    private String name = "";
    private String systemID = "";
    private String publicID = "";
    private boolean isExternal = false;
    private boolean isPublic = false;
    
    private DocType dtd = null;
    //
    // constructors
    //
    public NotationDecl()
    {
    	name = "";
    	systemID = "";
    	publicID = "";
    	isExternal = false;
    	isPublic = false;
    	dtd = null;
    }
    
    public NotationDecl( DocType d )
    {
    	this();
    	dtd = d;
    }
    
    public NotationDecl( DocType d, String n )
    {
    	this();
    	dtd = d;
    	name = n;
    }
    //
    // get and set methods
    //
    public void setName( String n ) { name = n; }
    public String getName() { return name; }
    public void setSystemID( String s ) { systemID = s; }
    public String getSystemID() { return systemID; }
    public void setPublicID( String p ) { publicID = p; }
    public String getPublicID() { return publicID; }
    public DocType getDtd() { return dtd; }
    public void setDtd( DocType d ) { dtd = d; }
    public boolean getIsExternal() { return isExternal; }
    public void setIsExternal( boolean i) { isExternal = i; }
    public boolean getIsPublic() { return isPublic; }
    public void setIsPublic( boolean p) { isPublic = p; }
    
    //
    // implement XMLexpandable
    //
    public boolean canDoCharData() { return false; }
    public String toCDSect() { return ""; }
    public String toCharData() { return ""; }

    public String toXMLstring()
    {
        String snd = "<!NOTATION " + name + " ";
        if ( isExternal )
        {
            if ( isPublic )
                snd += "PUBLIC \"" + publicID + "\" \"" + systemID + "\" ";
            else
                snd += "SYSTEM \"" + systemID + "\" ";
        }
        else
        {
            if ( isPublic )
                snd += "PUBLIC \"" + publicID + "\" ";
            else
                snd += "\"$FIX$ Logic error in NotationDecl!\" ";
        }
        snd += " > ";
        return snd;
    }
    //
    // recursive descent parsing routines
    //
    // NotationDecl  ::= '<!NOTATION' S Name S (ExternalID |  PublicID) S? '>' 
    //
    //     PublicID  ::= 'PUBLIC' S PubidLiteral 
    // 
    // parse enters with t a BANG_NAME with value "NOTATION".
    // parse exits with t a MARKUP_END with value ">".
    //
    public int parse( ScanToken t ) throws XMLparseException
    {
    	int count = 0;
    	int gotOne = 0;
    	XMLparseException pBad = null;
    	String msg = null;
    	XMLscanner scan = dtd.getScan();
    	
    	if ( !t.string.equals("NOTATION") )
    		return count;
    	
    	t = scan.getToken();
    	if ( t.type != ScanToken.NAME )
    	{	// must have a name
			Object[] args = { "NOTATION name", t.string };
			msg = MessageFormat.format( Messages.unexpected, args );
			pBad = new XMLparseException( msg );
			throw pBad; 
    	}
    	name = t.string;
    	t = scan.getToken();
    	if ( t.string.equals("SYSTEM") || t.string.equals("PUBLIC") )
    	{
    		try{ gotOne = parseExternalID( t ); }
    		catch( XMLparseException p ) { throw p; }
    		count = gotOne;
    	}
    	else
    	{
			Object[] args = { "SYSTEM or PUBLIC identifiers", t.string };
			msg = MessageFormat.format( Messages.unexpected, args );
			pBad = new XMLparseException( msg );
			throw pBad; 
    	}
    	if ( !t.string.equals(">") )
    	{
 			Object[] args = { "'>' closing Notation declaration", t.string };
			msg = MessageFormat.format( Messages.unexpected, args );
			pBad = new XMLparseException( msg );
			throw pBad; 
   		}	
    	return count;
    }
    //
    // ExternalID ::= 'SYSTEM' S SystemLiteral
    //             |  'PUBLIC' S PubidLiteral S SystemLiteral 
    //
    public int parseExternalID( ScanToken t ) throws XMLparseException
    {
    	int count = 0;
    	String msg = null;
    	XMLparseException pBad = null;
    	XMLscanner scan = dtd.getScan();
    	
    	if ( t.type == ScanToken.NAME )
    	{
    		if ( t.string.equals("SYSTEM") )
    		{
    			// 'SYSTEM' SystemLiteral
    			t = scan.getToken();
    			if ( t.type != ScanToken.QUOTED_STRING )
    			{
					Object[] args = { "System literal value (quoted)", t.string };
					msg = MessageFormat.format( Messages.unexpected, args );
					pBad = new XMLparseException( msg );
					throw pBad; 
    			}
    			systemID = t.string;
    			count = 1;
    			t = scan.getToken();
    		}
    		else if ( t.string.equals("PUBLIC") )
    		{
    			// 'PUBLIC' S PubidLiteral S SystemLiteral 
    			t = scan.getToken();
    			if ( t.type != ScanToken.QUOTED_STRING )
    			{
					Object[] args = { "PubId literal value (quoted)", t.string };
					msg = MessageFormat.format( Messages.unexpected, args );
					pBad = new XMLparseException( msg );
					throw pBad; 
    			}
    			publicID = t.string;
    			// get optional SystemLiteral 
    			t = scan.getToken();
    			if ( t.type == ScanToken.QUOTED_STRING )
    			{
    				systemID = t.string;
    				t = scan.getToken();
    			}
    			count = 1;
    		}
    		else
    		{	// name must be either SYSTEM or PUBLIC -- this is in error
				Object[] args = { "SYSTEM or PUBLIC", t.string };
				msg = MessageFormat.format( Messages.unexpected, args );
				pBad = new XMLparseException( msg );
				throw pBad; 
    		}
    	}
    	return count;
    }		
}








