/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/XMLassembly/PI.java,v 1.1.1.1 2000/09/19 19:39:50 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/
/*----------------------------------------------------------------------------------+
 |                                                                                  |
 |                             PI - Processor Instruction                           |
 |                                                                                  |
 +----------------------------------------------------------------------------------*/

package com.adobe.rice.common.XMLassembly;

import java.util.*;
import java.io.*;
import java.text.*;
import com.sun.java.util.collections.*;

public class PI implements Serializable, XMLexpandable
{
    private String target = "";
    private String instruction = "";
    private XMLscanner scan;
    
    PI() { target = ""; instruction = "";  }
    
    PI( String t, String i ) { target = t; instruction = i ; }
    
    PI( XMLscanner s ) { scan = s; }
    
    PI( XMLscanner s, String t, String i ) { scan = s; target = t; instruction = i; }

    public void setTarget( String aTarget ) { target = aTarget; }

    public String getTarget() { return target; }

    public void setInstruction( String i ) { instruction = i; }

    public String getInstruction() { return instruction; }
    
    public void setScan(XMLscanner s) { scan = s; }
    
    public XMLscanner getScan( ) { return scan; }

    //
    // implement XML expandable
    //
    
    public boolean canDoCharData() { return false; }
    public String toCDSect() { return ""; }
    public String toCharData() { return ""; }

    public String toXMLstring()
    {
    	return "<?" + target + " " + instruction + " ?>\n";
    }
    
    //
    // PI parser
    //

    public int parse( ScanToken t ) throws XMLparseException
    {
    	// PI ::= '<?' PITarget (S (Char* - (Char* '?>' Char*)))? '?>'
 
    	XMLparseException pBad = null;
    	String msg = null;
    	int count = 0;
    	
    	if ( t.type == ScanToken.HUH_NAME )
    	{
            target = t.string;
            // a PI cannot have any of the 8 case variants of "XML" as a target.
            if ( t.string.toUpperCase().equals("XML") )
                return count;
            // get the next token
            t = scan.getToken();
    	}
    	else {  return count; }
    	// from here on, we have a PI instruction....
    	// scan forward to get the processing instruction.
    	t = scan.getToken();
    	if ( t.status != ScanToken.OK )
     	{	// bad news scan is invalid... throw an exception
            Object[] args = { "PI instruction", t.toString() };
            msg = MessageFormat.format( Messages.tokenError, args );
            pBad = new XMLparseException( msg );
            throw pBad; 
        }
        if ( t.type != ScanToken.PROC_INSTR )
        {
            Object[] args = { "PI instruction", t.toString() };
            msg = MessageFormat.format( Messages.tokenError, args );
            pBad = new XMLparseException( msg );
            throw pBad; 
    	}
        instruction = t.string;
    	return ++count;
    }  
}








