/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/XMLassembly/ScanSource.java,v 1.1.1.1 2000/09/19 19:39:50 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

/*--------------------------------------------------------------+
 |                                                              |
 |                 Scanner Reading Source                       |
 |                                                              |
 +--------------------------------------------------------------*/

package com.adobe.rice.common.XMLassembly;

import java.io.*;
import java.util.*;
import com.sun.java.util.collections.*;
//
//	Things to fix.
//
//	(D) - DONE,  (HL) - Higher Level, (P) - partial
//
// (D)  1.  Have begin/end line number and positions for multi-line tokens.
//
	 
public class ScanSource
{
    // following are sources available to obtain scan strings 
    //
    static final byte NONE = 0;			
    static final byte READER = 1;
    static final byte PROVIDER = 2;
    static final byte STRING = 3;
	
    //
    // following are line end options available for scanning
    //
    static final byte LINEFEED = 0;		/* XML default  0x000A character */
    static final byte RETURN = 1;		/* carraige return */
    static final byte IGNORE = 2;		/* Line end characters are ignored (stripped off) */
    static final byte INCLUDE = 3;		/* Line end characters supplied by source */
    static final byte CRLF = 4;			/* Line end is 0x0A 0x0D (CR LF) */
	
    static final String[][] lineEnds =
    {
        { "lf",     "\n" },
        { "cr",     "\r" },
        { "ignore",   "" },
        { "include" , "" },		
        { "crlf", "\r\n" }
    };

    //
    // instance variables
    //
    ScanSource		previous = null;	/* previous scan source */
    String			line = null;		/* current line to scan */
    int				lineNumber = 0;		/* line number */
    int				linePosition = 0;	/* line position (offset into the line) */
    String			lineEnd = "";		/* default line end */
    BufferedReader	inputReader = null;	/* non-null if a buffered reader supplied */
    StringProvider	lineProvider = null;/* non-null if line provider registered */
    boolean			atEOS = true;		/* true if at end of scan - this source */
    byte			sourceType = NONE;	/* default source type */
    byte			endType = LINEFEED;	/* default line end type */
	
    // getters and setters
	
    String getLine() { return line; }
	
    int getLineNumber() { return lineNumber; }
	
    int getLinePosition() { return linePosition; }
	
    String getLineEnd() { return lineEnd; }
	
    BufferedReader getInputReader() { return inputReader; }
	
    ScanSource getPrevious() { return previous; }
	
    StringProvider getLineProvider() { return lineProvider; }
	
    boolean getAtEOS() { return atEOS; }
	
    void setAtEOS( boolean aBit ) { atEOS = aBit; }
	
    void setLineNumber( int lineNo ) { lineNumber = lineNo; }
	
    void setLinePosition( int linePos ) { linePosition = linePos; }
	
    void setLine( String newLine ) 
    {
        line = newLine;
        sourceType = STRING;
        setLineNumber( 0 );
        setLinePosition( 0 );
        setAtEOS( (line.length() > 0) ? false : true  );
    }
	
    void setInputReader( BufferedReader reader ) 
    {
        boolean atEnd;
        inputReader = reader;
        sourceType = READER;
        setLineNumber( 0 );
        if ( line.length() == 0 )
        {
            atEnd = getNextLine();
        }
    }
	
    void setLineProvider( StringProvider lineProv ) 
    {
        lineProvider = lineProv;
        sourceType = PROVIDER;
        setLineNumber( 0 );
        if ( line.length() == 0 )
        {
            boolean atEnd = getNextLine();
        }
    }
	
    void setLineEnd( String lineEndOption )
    {
        lineEnd = "\n";			/* set standard line end default */
        endType = LINEFEED;		/* set standard option */
        for ( int i = 0; i < lineEnds.length; i++ )
        {
            if ( lineEnds[i][0].equals( lineEndOption ) )
		    {
                lineEnd = lineEnds[i][1];
                endType = (byte) i;
                break;
		    }
	    }	
	// note that by default or error the standard linefeed option is selected.
    }
	
    // methods
    
    int incrementLineNumber( int incr ) 
    { 
        if ( incr >= 0 )
            lineNumber += incr; 
        return lineNumber; 
    }
    
    // 
    // increment the line position between 0 and line length.  
    // but never allow it out of bounds.
    //
    int incrementLinePosition( int incr ) 
    { 
        if ( incr < 0 )
        {
            if ( linePosition + incr >= 0 )
                linePosition += incr;
            else
                linePosition = 0;
        }
        else /* incr >= 0 */
        {
            if ( linePosition + incr <= line.length() )
                linePosition += incr; 
            else
                linePosition = line.length();
        }
        return linePosition;
    }
    
    int remanent() { return (line == null)? 0 : ( line.length() - linePosition ); }

    // getNextLine returns true if atEOS else returns false
    //
    private boolean getNextLine()
    {
        switch( sourceType )
        {
            case NONE:
            case STRING:
                atEOS = true;
                lineNumber = -1;
                break;
            case READER:
                // caller wants to use the buffered Reader.
                try
                {
                    if ( endType == INCLUDE )
                    {
                        char[] cbuf = new char[Constants.EXPECTED_SCAN_BUFFER_LENGTH];
                        int len = inputReader.read( cbuf, 0, cbuf.length );
                        line = new String( cbuf, 0, len );
                    }
                    else
                    {
                        line = inputReader.readLine() + lineEnd;
                    }
                }
                catch ( IOException io )
                {
                    line = null;
                    atEOS = true;
                    lineNumber--;   /* back down one so common increment below works. */	
                }
                break;
            case PROVIDER:
                line = lineProvider.getString() + lineEnd;
                linePosition = 0;
                if ( line == null )
                {
                    atEOS = true;
                    lineNumber--;   /* back down one so common increment below works. */
                }
                break;
        }
        linePosition = 0;			/* set the offset back to zero */
        lineNumber++;				/* bump the line count */
        return atEOS;				/* Return whether or not a line was read */
    }
	
    char getChar() 
    { 
        if ( atEOS || ( line == null ) || (linePosition >= line.length()) )
            return '\u0000';
        return line.charAt( linePosition );
    }
	
    char getNextChar()
    {
        if ( atEOS )
            return '\u0000';

        ++linePosition;		/* point at next character */
        if ( ( line == null ) || ( linePosition >= line.length() ) )
        {
            atEOS = getNextLine();
            if  ( atEOS )
                return '\u0000';
        }
        return line.charAt( linePosition );
    }

    ScanSource push( ScanSource newSource )
    {
        newSource.previous = this;
        return newSource;
    }
	
    boolean okToPop( )
    {
        return (( previous != null ) ? true : false ); 
    }
    //
    // following code doesn't allow poping off the bottom of stack
    //
    ScanSource pop( )
    {
        if ( okToPop() )
        {
            ScanSource temp = previous;
            previous = null;
            return temp;
        }
        return this;
    }
    //
    //	Constructors:
    //
    public ScanSource()
    {
        previous = null;
        line = null;
        lineNumber = 0;
        linePosition = 0;
        lineEnd = "";
        inputReader = null;
        lineProvider = null;
        sourceType = NONE;
        endType = IGNORE;
        atEOS = false;
    }
	
    public ScanSource( ScanSource prev, String newline, String endLine )
    {
        this();
        previous = prev;
        line = newline;
        setLineEnd(endLine);
        sourceType = STRING;
    }
    
    public ScanSource( ScanSource prev, BufferedReader reader, String endLine )
    {
        this();
        previous = prev;
        inputReader = reader;
        setLineEnd(endLine);
        sourceType = READER;
    }
    
    public ScanSource( ScanSource prev, StringProvider provider, String endLine )
    {
        this();
        previous = prev;
        lineProvider = provider;
        setLineEnd(endLine);
        sourceType = PROVIDER;
    }

    //
    // methods
    //

    private static final String db_Begin   = "->|";
    private static final String db_Header  = "  +"; // db_Header must be same length as db_Begin
    private static final String db_End     = "|<-";
    private static final String db_Trailer = "+  ";
    private static final String db_Point = "|";
    private static final String db_Dashes  = 
                                  "-------------------------------------------------------------";
    private static final String db_Blanks = 
                                  "                                                             ";
    private static final int PREFIX_MAX = 10;
    private static final int TAIL_MAX   = 10;
    private static final int PREFACE_LEN = 2;

    private static final String db_Preface = db_Blanks.substring(0, PREFACE_LEN);

    public String debugString()
    {
        // This generates the following string:
        // 
        // |<-preface->|<-header->|<-dashes->|<-pointer->|<-trail dashes->|<-trailer->| position = nn,
        // |<-preface->|<-begin ->|xxxxxxxxxx|   char    |xxxxxxxxxxxxxxxx|<-  end  ->| line = mmm,
        // |<-preface->|<-header->|<-dashes->|<-pointer->|<-trail dashes->|<-trailer->|
        // 
        // so it looks like:
        //
        //     +-----|-----+   position = mm, 
        //   ->|abcdeXfghij|<- line = nnnn, 
        //     +-----|-----+

        Integer db_LinePosition = new Integer( linePosition );
        Integer db_LineNumber = new Integer( lineNumber );

        final int stringLen = ( line == null )? 0 : line.length();
        int prefixLen = ( linePosition > PREFIX_MAX ) ? PREFIX_MAX : linePosition;
        int tailStart = ( linePosition < (stringLen - 1) ) ? linePosition + 1: stringLen - 1 ;
        int tailLen =  ((stringLen-tailStart) > TAIL_MAX) ? TAIL_MAX : ( stringLen - tailStart - 1 ) ;

        int prefixEnd = ( stringLen - PREFIX_MAX - 1 );      
        int prefixStart = linePosition - prefixLen ;
        int substrEnd = prefixStart + prefixLen + tailLen + 1;

        substrEnd = ( substrEnd > stringLen )? stringLen : substrEnd;
        
        String db_prefix = null;
        String db_tail = null;
        String db_substr = ( line == null )? "" : line.substring( prefixStart, substrEnd ) ;
        String db_target = ( line == null )? "" : line.substring( linePosition, linePosition + 1 );

        db_prefix = db_Preface + db_Header + db_Dashes.substring( 0, prefixLen );
        db_tail = db_Dashes.substring(0, tailLen ) + db_Trailer;

        // remove trailing "\n" if it exists
        if ( substrEnd == stringLen )
        {
            if ( line.substring( substrEnd - 1 ).equals("\n") )
                db_substr = db_substr.replace( '\n', '^' );
        }

        String ds = "\n";

        ds += db_prefix + db_Point + db_tail;
        // now put in the position number
        ds += " position = " + db_LinePosition.toString() + "\n";
        // build the display string:
        ds += db_Preface + db_Begin + db_substr + db_End;
        // now put in the line number
        ds += " line = " + db_LineNumber.toString() +"\n";
        // build the final string
        ds += db_prefix + db_Point + db_tail + "\n";

        return ds;
    }
}











