/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/XMLassembly/ScanToken.java,v 1.1.1.1 2000/09/19 19:39:50 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/
/*-------------------------------------------------------------+
 |                                                             |
 |                       ScanToken Class                       |
 |                                                             |
 +-------------------------------------------------------------*/

package com.adobe.rice.common.XMLassembly;

import java.io.*;
import java.util.*;
import com.sun.java.util.collections.*;
//
//	Things to fix.
//
//

public class ScanToken
{
    /* Token Types defined */
    public static final byte GENERAL_ENTITY_CHAR_HEX = 0;	/* token is { "&#x" hexchar+ ";" } */
    public static final byte GENERAL_ENTITY_CHAR_DIG = 1;	/* token is { "&#" digit+ ";" } */
    public static final byte GENERAL_ENTITY = 2;		/* token is { "&" name ";" } */
    public static final byte QUOTED_STRING = 3;			/* token is { quote char* quote } */
    public static final byte COMMENT = 4;				/* token is { "<!--" char* "-->" } */
    public static final byte NAME = 5;					/* token is { letter namechar* } */
    public static final byte NAME_TOKEN = 6;			/* token is { namechar+ } */
    public static final byte PARAMETER_ENTITY = 7;		/* token is { "%" name ";" } */
    public static final byte BANG_NAME = 8;				/* { "<!" name } */
    public static final byte BANG_BOX_NAME = 9;			/* { "<![" name } */
    public static final byte HUH_NAME = 10;				/* { "<?" name } */
    public static final byte TAG_NAME = 11;             /* { "< name } */
    public static final byte SPECIAL = 12;				/* single character special */
    public static final byte MARKUP_BEGIN = 13;			/* token is "<" */
    public static final byte MARKUP_BANG = 14;			/* token is "<!" */
    public static final byte MARKUP_QUESTION = 15;		/* token is "<?" */
    public static final byte MARKUP_BANG_BOX = 16;		/* token is "<![" */
    public static final byte MARKUP_EMPTY_END = 17;		/* token is "/>" */
    public static final byte MARKUP_BANG_DASH = 18;		/* token is "<!-" */
    public static final byte MARKUP_TAGEND = 19;		/* token is "</" */
    public static final byte MARKUP_END = 20;			/* token is ">" */
    public static final byte SECTION_CDATA = 21;		/* token is CDATA content */
    public static final byte SECTION_IGNORE = 22;		/* token is IGNORE section */
    public static final byte SECTION_INCLUDE = 23;		/* token is INCLUDE section */
    public static final byte SECTION_UNKNOWN = 24;		/* token is an unknown section */
    public static final byte SECTION_NAME = 25;         /* token is a SECTION name */
    /* i.e. "<![" s? name s? "[" which differs from BANG_BOX_NAME */ 
    public static final byte SECTION_END = 26;          /* token is "]]>" */
    public static final byte WHITESPACE = 27;			/* token is whitespace */
    public static final byte RBOX_SPAN = 28;            /* token is string: "]"+ */
    public static final byte PROC_INSTR = 29;           /* token is a pI in { "<?" name whitespace pI "?>" }*/
    /* note: the PI_END terminated the PROC_INSTR and is NOT reported as a seperate token. */
    public static final byte PI_END = 30;               /* token is a "?>" */
    
    public static final byte ILLEGAL = 31;				/* token is illegal XML character */
    public static final byte EMPTY = 32;				/* Scanned nothing at all */
    public static final byte ERROR = 33;				/* token is erroneous fragment */
	
    /* status definitions */
    public static final byte OK = 0;					/* scan is ok */
    public static final byte TRUNCATED = 1;		    	/* scanner truncated token */
    public static final byte ENDED = 2;			    	/* End of scan */
    public static final byte CHAR_ENTITY_ERROR = 3; 	/* scan terminated in char entity */
    public static final byte COMMENT_ERROR = 4;	    	/* scan terminated in comment */
    public static final byte LBBNAME_ERROR = 5;	    	/* scan terminated in CDATA, IGNORE or INCLUDE section */
    public static final byte DECL_ERROR = 6;			/* scan terminated in <!name string */
    public static final byte GEN_ENTITY_ERROR = 7;		/* scan terminated in &name (missing ;) */
    public static final byte PARM_ENTITY_ERROR = 8; 	/* scan terminated in %name (missing ;) */
    public static final byte ILLEGAL_CHAR = 9;			/* scan terminated, saw illegal XML char */
    public static final byte QUOTE_ERROR = 10;			/* scan terminated, in quoted string */
    public static final byte ENDED_IN_MARKUP = 11;		/* scan terminated in markup */
    public static final byte ENDED_IN_SECTION = 12; 	/* scan terminated in section */
    public static final byte ENDED_IN_PI = 13;			/* scan terminated in { "<?" name EOS } */
    public static final byte ENDED_IN_IGNORE = 14;      /* scan terminated in IGNORE section */
	
    public String	string;					/* the token string */
    public byte		type = EMPTY;			/* a Type byte identifying the token */
    public byte		status = OK;			/* scan status code */
    public int 		lineBegin = 0;			/* line number where token started */
    public int 		positionBegin = 0;		/* line offset where scan token started */
    public int 		lineEnd = 0;			/* line number where token ended */
    public int 		positionEnd = 0;		/* column offset where token ended. */
	
    public ScanToken()
    {
        string = null;
        type = EMPTY;
        status = OK;
        lineBegin = 0;
        positionBegin = 0;
        lineEnd = 0;
        positionEnd = 0;
    }
	
    private static final String[] typeDescriptions =
    {        
        "Hexadecimal character entity reference",
        "Digital character entity reference",
        "General entity reference",
        "Quoted string",
        "XML comment",
        "Name",
        "Name Token",
        "Parameter entity reference",
        "Doctype or markup declaration name token { '<!' name }",
        "Possible section name { '<![' name }",
        "XML declaration or processing instruction target, { '<?' name }",
        "Element Type { '<' name } ",
        "Special character",
        "Markup begin { '<' }",
        "Markup bang { '<!' }",
        "Markup question { '<?' }",
        "Markup bang box { '<![' }",
        "Markup empty element end { '/>' }",
        "Markup bang dash { '<!-' }",
        "Markup element end { '</' }",
        "Markup end { '>' }",
        "Section CDATA { '<![CDATA[' character* ']]>' }",
        "Section ignore { '<![IGNORE[' character* ']]>' }",
        "Section include { '<![INCLUDE[' character* ']]>' }",
        "Section unknown { '<![' name '[' }",
        "Section name ",
        "Section end { ']]>' } ",
        "White space { ( ' ' | '\n' | '\r' | '\t' )+ }",
        "Right box span { ']'+ }, i.e. one or more ']' ",
        "Processing Instruction ( text after targetname in { '<?' targetname whitespace text '?>' }",
        "End XMLdecl or PI, { '?>' } ",

        "Illegal character",
        "Empty - nothing scanned.",
        "Error - erroneous text fragment for XML"
    };
	
    private static final String[] statusDescriptions =
    {
        "OK",
        "Truncated",
        "End of Scan",
        "Character entity error",
        "Comment error",
        "Section name error",
        "Declaration error",
        "General entity reference error",
        "Parameter entity reference error",
        "Illegal XML character",
        "Quoted string error",
        "Ended in markup",
        "Ended in section",
        "Ended in processing instruction",
        "Ended in ignore section"
    };
    
    public String debugMessage()
    {
        Byte b_type = new Byte( type );
        Byte b_status = new Byte( status );
        Integer i_lineBegin = new Integer( lineBegin );
        Integer i_positionBegin = new Integer( positionBegin );
        Integer i_lineEnd = new Integer( lineEnd );
        Integer i_positionEnd = new Integer( positionEnd );
	
        String header = "ScanToken, type = " +  b_type.toString() + " (" + 
            typeDescriptions[ type ] + ")\n";
        String statline = "[\n    status = " +  b_status.toString() + " (" + 
            statusDescriptions[ status ] + ")\n";
        String tokenbgn = "    Begins at:  line number = " + i_lineBegin.toString() +
            ", Offset = " + i_positionBegin.toString() + "\n";
        String tokenEnd = "    Ends at:  line number = " + i_lineEnd.toString() + 
            ", Offset = " + i_positionEnd.toString() + "\n";
        String value = "    " + ( (string == null) ? "string = null" : "->|" + string + "|<-" )+ "\n";
        String footer = "];\n" ;
	
        return header + statline + tokenbgn + tokenEnd + value + footer;
    }
    
    public String toString()
    {
        Byte b_type = new Byte( type );
        Byte b_status = new Byte( status );
        Integer i_lineBegin = new Integer( lineBegin );
        Integer i_positionBegin = new Integer( positionBegin );
        Integer i_lineEnd = new Integer( lineEnd );
        Integer i_positionEnd = new Integer( positionEnd );
        
        String str = "Token = { ";
        str += "string = " + ((string == null) ? "null," : "\"" + string + "\", ");
        str += "type = " + b_type.toString() + ", ";
        str += "staus = " + b_status.toString() + ", ";
        str += "lineBegin = " + i_lineBegin.toString() + ", ";
        str += "positionBegin = " + i_positionBegin.toString() + ", ";
        str += "lineEnd = " + i_lineEnd.toString() + ", ";
        str += "positionEnd = " + i_positionEnd.toString() + "}\n";
	
        return str;
    }

    public void append( String s ) { string = string.concat(s); }
    public void append( char c ) { string.concat( String.valueOf(c) ); }
}	













