/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/XMLassembly/XMLdoc.java,v 1.5 2000/12/02 23:31:44 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/
/*------------------------------------------------------------------------+
 |                                                                        |
 |                            XML Document Class                          |
 |                                                                        |
 +------------------------------------------------------------------------*/

package com.adobe.rice.common.XMLassembly;

import java.util.*;
import java.io.*;
import java.net.*;
import java.text.*;
//$JDK1.1$ import com.sun.java.util.collections.*;
import com.adobe.rice.common.*;

/**
@author Bruce Hunt
@version 0.30 14 May, 2000
 
This is the package XMLassembly.  It's function is to construct
guaranteed valid "deflated" representations of XML documents. 
A "deflated" document carries around most of it's content by
reference.  The document is "inflated" either incrementally when
required or when all values are required to be present ( say just
before transmission, printing or archival storage.).
These documents are called deflated documents or just documents;

XML documents constructed using this package are guaranteed to 
be both well formed and valid according to the docuemnt's DTD.
Attempts to alter a document so that it does not conform to
it's DTD are rebuffed <i>prior to any document change</i> with
an appropriate exception.  This means that the document is
<b><i>ALWAYS</i></b> valid and well formed.

<i>Since the validity of an XML document is the PRIMARY goal of
this package, all readers of this code are invited to supply
corrections, updates (or even correctness proofs) to assure
that this goal is achieved. </i>

To assemble a deflated document, the first step is to provide a 
Document Type Definition(DTD).  This is done by creating an XML 
document object and providing it with either an external DTD 
reference or a stream containing a DTD.  The DTD is used to 
create a content model for any documents that are assembled.  
It should be noted that the class XMLdoc (used to construct an 
XML document object) can support any well defined DTD and create 
an XMLdoc object for each new DTD type.  (A well defined DTD 
is one that satisfies all of the constraints of the XML 
specification 1.0.)
 
A valid XML document is precisely defined as a single physical 
storage entity that satisfies the (current) XML specification 
together with the (current) XML namespace specification.  

A deflated document of an XML document is a tree of objects 
where each object corresponds to markup or character data.  
There is a one-to-one correspondence between an XML markup item 
and an object.  However, objects that correspond to character data 
may appear in (sequential) groups surrounded by markup objects. 
This latter ability for grouped character data objects is provided 
for convenience of assembly. [ The thinking here is that a file 
based record or other content character data partitioning
can be accomodated if multiple character sequences are permitted.]
  
A well formed and valid XML document prototype has the following 
structure:
  {XMLdoc object}	
	{Prolog}  
      This must appear and contains the following structure:
      {XML Declaration}
        This is REQUIRED (unlike in XML spec where it is optional).
        It has the following content model:
          {VresionInfo} -- Set by default.
          {EncodingDecl} -- Set by default.
          {SDDecl}? -- Optional (if selected, set by default).  
        {Misc object} -- optional, with content model:
          [ {XML comment object} | {processing instruction object} ]*
          {XML comment object} -- optional with content model:
            {text string up to the first occurrance of "-->"}
            A content reference is permitted here.
          end {XML comment object}
          {processing instruction object} -- optional with content model:
            {PITarget} {text string up to the first occurrance of "?>"}
              { PITarget } content model is:  
                {any name except those prefixed with "XML", "xML", "XmL", ...}
                The text string may be a content reference.
          end {procesing instruction object}
        end {Misc object}
      end {XML Declaration}
      {Document Type Declaration object}
         This is REQUIRED (unlike XML spec where it is optional).  
         Either an external reference to a DTD or an internal DTD 
         is required.  It's content model is:
         {element name} -- REQUIRED, name of the root element
         {ExternalID}? -- optional, URI for initial external subset DTD.
           This is a pointer to an external DTD.
         {markup}* -- optional, internal declarations (internal subset DTD).
           This is a list of entity, element, attribute, and notation declarations.
           It can also have processing instructions and comments.
      end { Document Type Declaration object}
      {Misc object} -- optional, identified as "MiscProlog" with same 
         definition as above.
    end {Prolog}
    {Element object}
       This is the REQUIRED root element.  It's content model is:
       {element name} -- REQUIRED, must match DTD root element name.
       {element prefix}? -- optional, name space prefix for the element.
       {attributes}* -- conditional, must match DTD attribute
         declaration (AttrDecl) requirements.
       {content}? -- conditional, must match DTD elementDecl content model
         which limits the following general content model:
         General Content Model is zero or more of in any order:
         {Element object}
         {CharData object} -- A content reference is permitted here
         {CharReference}
         {EntityReference}
         {CDSect object} -- A content reference is permitted here
         {processing instruction object} -- same definition as above.
         {XML comment object} -- same definition as above.
    end {Element object}
    {Misc object} -- optional, identified as "MiscEpilog" with the
         same definition as above.
  end {XMLdoc object}

Once an XMLdoc has been created, the XMLdoc object permits the 
following via methods:

    1.  Open -- readies the document for update.

        open( ) -- opens document to current part.  By default
        this is current part:  Prolog, Element, or Misc.

        open( partname ) -- partname is one of: Prolog, Element,
        MiscProlog, or MiscEpilog.  Opens the document and moves pointer
        to the current part.
        

    2.  Close -- closes the document.

        Close() -- closes the current part and the document.

        Close( partname ) -- partname is one of: Prolog, Element,
        or Misc. Closes the partname and the document.

    3.  Seek -- positions document to a selected part:

        Seek( partname ) -- positions document to selected part.
        returns partname object.
        
    4.  Any of the operations permitted on an element as defined
        below if open.  The element that is inferred is the current
        element.  If there is no current element, then the
        only operation's permitted are:
        
        addContent( root ) 
        addContent( root, 0 )

All contained objects in an XMLdoc have the following generic
methods:

    1.  Open -- opens sub objects of the object.

    2.  Close - closes sub objects of the object.

    3.  Seek - points at a specfic sub object of the object.
        (not very useful on fixed content objects)
        returns specified object.

The Prolog object permits the following via methods:

    1.  Open -- readies a subobject of Prolog for modification.
  
        {prolog object].Open() -- opens Prolog to current sub object.  By default
        this is the current sub object which is one of:
        XMLdecl, DocType, or Misc.

        Open( subobject name ) -- opens Prolog object to named sub object.
        Defined subobject names are:  
        XMLdecl, DocType or Misc.

    2.  Close -- closes a prolog subobject.

        Close() -- closes current prolog subobject.

        Close( subobject name ) -- closes named suboObject.

    3.  Seek -- positions Prolog object to the selected contained subobject.

        seek( partname ) -- positions prolog object to selected subobject.
        which can be one of: XMLdecl, DocType or Misc.
    

An element has the following methods:

    1.  Open 

        A:  open()
            open( int index ) 
        
        Open readies the element content for update.   If <i>this</i> element
        is in the content of the document's current element, then 
        <i>this</i> element is pushed on the document content stack and becomes
        the document's current element.
        Open positions the document to the element's current
        index point.  An optional parameter, index, sets the current 
        index point to index.   In this implementation, the sequence of 
        content is indexed starting at 0. (When an element is created, 
        its index point is set to zero.)  Attempts to position the 
        index below zero or beyond the number of items in the current content
        result in an element exception. Attempts to access the 
        element without opening it cause an ElementException to be thrown.
        Attempts to update the content model of a declared empty element
        result in an Element Exception.
        
        An element optionally contains a sequence of content that satisfys
        the element's content model.  A declared non-empty element may also have
        XML comments and Processing Instructions interspersed with the 
        content.  
        
        B:  openAttributes()
        
        OpenAttributes opens up the element for attribute update.
        The declared attributes (including the built-in attributes (e.g xml:lang))
        may have their values updated.  The value must satisfy the
        Attribute declaration of the value type; if it fails to do so, an 
        Attribute Exception is thrown.  Reference to undeclared attributes in
        an update call result in an Attribute Exception.  Attempts to update
        an element's attributes without opening the element for attribute 
        update result in an Element Exception.
        
	
	2.  Update
	    
	    A:  addContent( ContentReference cr )
	        addContent( ContentReference cr, int index ) 
	        addContent( Element e )
	        addContent( Element e, int index)
	        
	    Insert content (by reference) permitted by the DTD; 
	    If the optional parameter, index, is used, the content index point
	    is set to its value.  The content is placed at the content index point 
	    and the content index point is updated to point past the inserted 
	    content.  A content reference may contain any of:
	        character data, 
	        character entity, 
	        element, 
	        general entity, 
	        processing instruction 
	        comment.
	    Character data, character entity and element MUST be permitted
	    by the current element's content model for the insertion to succeed.
	    
	    A general entity, when the value is available, is included content piece 
	    by content piece.  The structure of a general entity is a 
	    character sequence of markup.  A general entity's content is parsed for 
	    insertion and may result in one of several different exceptions 
	    according to the type of content within the entity.  If the value of 
	    the general entity is not yet available, the document is marked for 
	    deferred entity validation.
	    
	    The element must be declared non-empty for insertion of any
	    content otherwise an Element Exception is thrown.
        Attempts to add content NOT satisfying the content model result in an 
        Element Exception.  This includes elements declared to be empty.  
        Attempts to update the content of an element
        without opening it cause an Element Exception to be thrown.
	    
	    Note: An XML document initially only permits the root 
        element to be inserted (as required by the DTD).  The root 
        element then becomes the current element.  The root element
        may be opened or closed; but it may not be popped off the stack 
        unless it is deleted.
        
        B.  addAttribute( String theName, String theValue )
        
        Update the attribute with name, theName, to have value, theValue.
        The attribute MUST have been declared in the DTD or it must be
        one of the built-in XML attributes (e.g. xml:lang ).  If the
        attribute is not declared, an Attribute Exception is thrown.
        The value, theValue, must satisfy any declared value constraints
        otherwise an Attribute Exception is thrown.
        
        
        C:  replaceContent( ContentReference cr )
            replaceContent( ContentReference cr, int index )
            replaceContent( Element e )
            replaceContent( Element e, int index )
            
	  
	    The content at the current index is replaced with the content
	    in the call.  If the optional parameter, index, is included, the
	    current index is first set to its value.  This operation succeeds 
	    if the replacement is valid according to the content model of 
	    the element.  The entire content in the current element MUST 
	    be valid with the replacing item in place of the item at the 
	    current index.  An element exception is thrown if the 
	    replacement fails.  Replacement can also fail if the index is 
	    less than zero or exceeds or equals the number of items in the
	    content. The current index is NOT updated by this operation.
	    
	    D:  replaceAttribute( String theName, String theValue )
	    
	    This is a synonym for B. above.
	    
	    E:  deleteContent( )
	        deleteContent( int index )

	    The item at the current index is removed from the element's
	    content if the remaining items constitute a valid element
	    content model.  If the optional parameter, index, is included,
	    the current index is first set to its value.  If there is
	    no item at the current index, this operation fails and an
	    Element Exception is thrown.  
	    
	    This operation succeeds if the remaining items in the element's 
	    content after the item at the current index is removed are 
	    valid according to the element's content model.  Otherwise 
	    the operation fails and an Element Exception is thrown.  
	    If the parameter index is below zero or, exceeds or equals 
	    the number of items in the element's content, an Element 
	    Exception is thrown.  
	    
	    The delete operation does NOT update the current index.
	    
	    F:  deleteAttribute( String theName )
	        resetAttribute( String theName )
	    
	    The value of the declared attribute with name, theName, is 
	    set back to it's default value.  Since a declared attribute
	    always has a default value, it cannot be removed from the
	    element.
	    
	3.  Seek
	
	    A:  ContentReference seek( int index )
	        
	    The parameter, index, is used to position the current index
	    to the item at it's value.  The item at this index is returned.
	    The value of index MUST be from 0 through the number of items
	    in <this> element's content.  If the index is set to the number
	    of items in the content, a null object is returned and no
	    Element Exception is thrown.
	    
	    If the index is less than zero or greater than the number of
	    items in the element's content, an Element Exception is thrown.
	    
			    
	4.  Close 
	    
	    A:  close()
	    
	    <i>This</i> element is closed.  If <i>this</i> element is on
	    the document's element stack, elements from the current element
	    through <i>this</i> element are closed and popped off the document's
	    element stack.  If <i>this</i> is the root element, (i.e. is the 
	    only element on the document's stack and has the root tagname) 
	    it is not popped off the document element stack, but is closed. 
	    
	    If <i>this</i> element is NOT on the document's element stack,
	    it is closed.

        Closing an element may cause an Element Exception to be thrown
        because additional content may be required to keep the document 
        valid.  Closing an element may also cause an Attribute Exception
        to be thrown (e.g. if a required attribute is not provided).
	    
	    If the root element is closed, the document prototype is open 
        but the only content that may be inserted are XML comments or 
        processing instructions.  Attempts to insert other
	    content cause an XML document exception to be thrown.
	    
		
If an exception is thrown, the state of the XMLdoc is such that it remains 
in exactly the same state it had just prior to the method call which caused 
the exception to be thrown.  This means that another insertion can tried.
		 

Once an XMLdoc has been created, the object permits the following access 
actions:

	1.  get the current element name.  An empty name returned indicates 
        a closed XMLdoc.
	2.  get a list of the content types that may be inserted.
		If the root element is not yet inserted, a single element list 
        which contains the root element type (string name) is returned.  
        If the document prototype is closed, then a list containing the
		sring, "#XMLdoc" is returned.
	3.  get a selected content object by type and occurance.


Let item be any of 
        XMLdoc -- Content Model: Prolog? element ( PI | Comment )*
        Prolog -- Content Model: XMLDecl?, (PI | Comment )*, ( doctypedecl?, (PI | Comment)*)? 
        XMLDecl -- Content Model:  VersionInfo, EncodingDecl?, SDDecl? 
        doctypedecl -- Content Model: Name, ExternalID? ( '[' (markupdecl | PEReference )* ']' )? 
        PI -- Content Model: PITarget ( char* - ( char* '?>' char*) )?
        TextDecl - Content Model: VersionInfo? EncodingDecl
        ExtSubsetDecl - Content Model: ( markupdecl | conditionalSect | PEReference )*
        IgnoreSect - IGNORE section.
        IncludeSect - INCLUDE section
        Comment - Content Model: ( char* - ( char* '-->' char* ) )?
        Element - Content Model: EmptyElemTag | STag content ETag
        EmptyElemTag - Content Model: name (Attribute)*
        STag    - Content Model: name ( Attribute )*
        ETag - Content Model: name
        Attribute - Content Model:  name '=' AttrValue
        AttrValue - Content Model:  '"' ( char* - ( char* '"' char* ) ) '"' | 
                                    "'" ( char* - ( char* "'" char* ) ) "'"
        content - Content Model: (element | CharData | Reference | CDSect | PI | Comment)*
        CharData -- Content Model: character data ( Content Reference ) escaping <, &. 
        CDSect -- Content Model: CDATA section ( Content Reference ) escaping "]]>" if needed.
        EntityRef -- Content Model: General Entity reference
        CharRef -- Content Model: Character Entity reference )
        PEReference -- Content Model: Parameter Entity reference.
*/

public class XMLdoc implements Serializable, XMLexpandable
{
    //
    // Document parts for content managment 
    //
    public static final int PARTSTART   = 0;
    
    public static final int PROLOG      = (PARTSTART + 0);
    public static final int PREMISC     = (PARTSTART + 1);
    public static final int DTD         = (PARTSTART + 2);
    public static final int POSTMISC    = (PARTSTART + 3);
    public static final int ROOT        = (PARTSTART + 4);
    public static final int ENDMISC     = (PARTSTART + 5);
    
    public static final int PARTLAST     = ENDMISC;
    
    
    static private String docParts[] = {
        "Prolog", "PreMisc", "DTD", "PostMisc", "Root", "EndMisc"
    };
    
    //-XMLdoc[
    boolean     PrologIsDefined = false;
    //--Prolog[ 
    //
    boolean     XMLDeclIsDefined = false;
    //---XMLDecl?[
    String      byteOrderMark = "\uFEFF";	 /* optional UTF16 encoding */
    //----VersionInfo[
    String      version = Constants.XML_VERSION;
    //----] VersionInfo;
    //----EncodingDecl?[
    String      encoding = Constants.XML_ENCODING;
    //----] EncodingDecl?;
    //----SDDecl?[
    boolean     standAlone = false;
    //----] SDDecl?;
    //---] XMLDecl?
    boolean     preMiscIsDefined = false;
    //---Misc*[
    List        preMisc = null;
    //---] Misc*; ------
    boolean     dtdIsDefined = false;
    //---doctypedecl?[          
    DocType     dtd = null;
    //---] doctypedecl?
    boolean     postMiscIsDefined = false;
    //---Misc*[
    List        postMisc = null;
    //---] Misc*;
    //--] Prolog;
    boolean     rootIsDefined = false;
    //--element[
    Element     root;               // the root element of the document.
    HashMap     nameSpaces;         // the name spaces in the document
    //--] element;
    boolean     endMiscIsDefined = false;
    //--Misc*[
    List        endMisc;            // trailing PIs and comments.
    //--] Misc*;
    
    //
    // private fields
    //
	private BufferedReader reader = null;   // reader for DTD or other stuff.
    private XMLscanner     scan = null;     // the scanner object for this document 
                                            // (used when DTD processed)
    private Element        current = null;	// points at the current element.
	private Stack          elementStack = null; // The stack of elements active.
	
	private int            docIndex = 0;
	
	private boolean        isOpen = false;
	
	private boolean        okToFormat = false;  // true if ok to format this document

    

    //-] XMLdoc;
    //
    //	Constructors
    //
    public XMLdoc()
    {
        // set defaults
        version = Constants.XML_VERSION;
        encoding = Constants.XML_ENCODING;
        standAlone = Constants.XML_STANDALONE;
        
        preMisc = new ArrayList(Constants.XML_MISC_CAPACITY);
        postMisc = new ArrayList(Constants.XML_MISC_CAPACITY);
        endMisc = new ArrayList(Constants.XML_MISC_CAPACITY);
        
        dtd = null;

        root = null;
        nameSpaces = new HashMap( Constants.EXPECTED_NAME_SPACE_COUNT );
        //
        // default element stack
        //
        elementStack = new Stack();
        elementStack.ensureCapacity(Constants.EXPECTED_ELEMENT_STACK_DEPTH);

        // The following is very expensive
        //
        scan = new XMLscanner();
    }

    public XMLdoc(DocType newDtd) throws ElementException
    {
        this();
        dtd = newDtd;
        // Note that the XML document may not have a root element selected.
        // or the caller may want to add a child of element.
    }

    public XMLdoc(String newEncoding, DocType newDtd, String newVersion) throws ElementException
    {
        this();
        version = newVersion;
        encoding = newEncoding;
        dtd = newDtd;
        // note that the XML doc may not have a root element selected yet!
    }

    public XMLdoc(String newEncoding, DocType newDtd) throws ElementException
    {
        this();
        encoding = newEncoding;
        dtd = newDtd;
    }
    //===================================================================
    // XML document utility methods
    //===================================================================

	//
    // Setters and Getters
    //
    //
    // Methods
    //
    //
    // getters and setters
    //
    public String     getByteOrderMark() { return byteOrderMark; }
    public String     getVersion()       { return version; }
    public String     getEncoding()      { return encoding; }
    public boolean    getStandAlone()    { return standAlone; }
	public DocType    getDtd()           { return dtd; }
    public List       getPreMisc()       { return preMisc; }
    public List       getPostMisc()      { return postMisc; }
    public List       getEndMisc()       { return endMisc; }
    public XMLscanner getScan()          { return scan; }
    public Element    getRoot()          { return root; }
    public HashMap    getNameSpaces()    { return nameSpaces; }
    public Element    getCurrent()       { return current; }
    public boolean    getOkToFormat()    { return okToFormat; }
    //
    // 		$FIX$ The following set parameters should be validated against legal values
    //		
    public void setByteOrderMark( String bom ) { byteOrderMark = bom; }
    public void setVersion( String v )         { version = v; }
    public void setEncoding( String e )        { encoding = e; }
    public void setStandAlone( boolean s )     { standAlone = s; }
    public void setPreMisc( List prM )         { preMisc = prM; }
    public void setDtd( DocType d )            { /* are you sure? */ dtd = d; }
    public void setPostMisc( List poM )        { postMisc = poM; }
    public void setEndMisc( List enM )         { endMisc = enM; }
    public void setOkToFormat( boolean ok )    { this.okToFormat = ok; }
    
    public void setSystemURL( String sURL )    { 
        if ( dtd != null )
            dtd.setSystemLiteral(sURL);
    }
    
    public void setPublicURL( String pURL )    {
        if ( dtd != null )
            dtd.setPubidLiteral(pURL);
    }
    //
    //	Internal fields setters and getters.
    //
    public void setReader( BufferedReader r )  { reader = r; }
    public void setScan( XMLscanner s )        { scan = s; }
    //
    public Element setRoot( Element em )       
    {   
        // May set root element only if the stack is empty!
        if ( elementStack.empty() )
        {   
             root = em; 
             setCurrent( em );
             this.root.setIsRoot(true); // also make sure that this is known to be the root.
        }
        return root;
    }
    
    public void setNameSpaces( HashMap ns )    { nameSpaces = ns; }
    
    public Element setCurrent( Element em )       
    { 
        Element re = (Element) elementStack.push(em);
        current = em;
        return re;
    }
    //
    //  Element stack management setters/getters
    //
    public Element push( Element em )  {
        return (Element) elementStack.push( em );
    }
    
    public Element pop( ) {
        return (Element) elementStack.pop();
    }
    
    public Element peek( ) {
        return (Element) elementStack.peek();
    }
    
    public int search( Element em ) {
        return elementStack.search( em );
    }
    
    public boolean empty( ) {
        return elementStack.empty();
    }
    //
    // methods
    //

    /*==========================================================================+
     |                                                                          |
     |		                U T I L I T Y   M E T H O D S                       |
     |                                                                          |
     +==========================================================================*/

    /**
    * @author Bruce Hunt
    * @version 0.01 (14-08-1999)
    * @see XMLscanner
    *
    * @exception FileNotFoundException 
    * @exception XMLparseException
    * 
    */		
    public void makeDOCfromURL( String aDTDurl ) 
        throws MalformedURLException, IOException, XMLparseException
    {	// set the URL
        URL	          iceDTD;
        URLConnection iceConnect;

        try 
        {
            iceDTD = new URL(aDTDurl);
            iceConnect = iceDTD.openConnection();
        } 
        catch (MalformedURLException e) 
        {   // new URL() failed
            throw e;
        } 
        catch (IOException e) 
        {   // openConnection() failed
            throw e;
        } 
		  
        reader = new BufferedReader( 
            new InputStreamReader( iceConnect.getInputStream() ) 
                );
        //
        // setup the scanner
        //
        scan.setReader( reader );
        //
        // now build the prolog
        //
        int	  count = 0;			// count parse items obtained.
        ScanToken t = null;			// scan token holder 
        // get a token to start off the parse.
        t = scan.getToken();
        // go attempt to get an XMLdecl
        try { count = parseXMLdecl( t ); }
        catch ( XMLparseException pBad )
        {   throw pBad; }
        // if we got one then get a new token
        if ( count > 0 )
            t = scan.getToken();
        try { count = parseMisc( t, preMisc ); }
        catch ( XMLparseException pBad ) { throw pBad; }
        // now see if we can get a dtd
        {
            dtd = new DocType( scan );
            try { count = dtd.parse( t ); }
            catch ( XMLparseException pBad ) { throw pBad; }
            catch ( DocTypeException dtBad ) 
            {   // convert DocTypeException to XMLparseException 
                String msg = dtBad.getMessage();
                XMLparseException pBad = new XMLparseException( msg );
                throw pBad;
            }
        }
        // can only have more misc if we got a dtd.
        if ( count > 0 )
        {
            t = scan.getToken();
            try { count = parseMisc( t, endMisc ); }
            catch ( XMLparseException pBad ) { throw pBad; }
        }
        // 
    }

    /*==========================================================================+
     |                                                                          |
     |		        X M L e x p a n d a b l e   M E T H O D S                   |
     |                                                                          |
     +==========================================================================*/

    public boolean canDoCharData() { return false; }
    public String  toCDSect() { return ""; }
    public String  toCharData() { return ""; }

    public String  toXMLstring()
    {
        final String EOL = "\n";
        //
        // [1] document ::= prolog element Misc*
        //
        StringBuffer result = new StringBuffer( /*$FIX$*/ 600 );
        result.append( prologToXMLstring() );
        if ( root != null )
        {
            result.append( root.toXMLstring() );
        }
        // put in "end misc"
        for ( int i = 0; i < endMisc.size(); i++ )
        {
            XMLexpandable holder = (XMLexpandable) endMisc.get(i);
            result.append( holder.toXMLstring() );
        }
        return result.toString();
    }

    public String prologToXMLstring()
    {
        final String EOL = "\n";
        
        StringBuffer XMLDecl = new StringBuffer( Constants.EXPECTED_ATTRIBUTE_LENGTH );
        
        XMLDecl.append( "<?xml " + 
            "version=\"" + version + "\" " +
            "encoding=\"" + encoding + "\" " +
            "standalone=\"" + ((standAlone)? "yes" : "no") + "\" " +
            "?>" );
        if ( this.getOkToFormat() )
            XMLDecl.append( EOL );
        StringBuffer prefixMisc = new 
            StringBuffer( Constants.XML_MISC_CAPACITY * Constants.PI_LENGTH );
        StringBuffer postfixMisc = new 
            StringBuffer( Constants.XML_MISC_CAPACITY * Constants.PI_LENGTH );
		
        if ( preMisc.size() > 0 )
        {
            for ( int i = 0; i < preMisc.size(); i++ )
            {
                Misc m = (Misc) preMisc.get(i);		// get the next Misc
                if ( this.getOkToFormat() )
                    prefixMisc.append( EOL );
                prefixMisc.append( m.toXMLstring() );
            }		
        }
		
        // $POLICY
        //		Include the DTD or reference it?
        //
        StringBuffer doctype = new StringBuffer( dtd.toXMLstring() );
        //
        // now include the endMisc
        //
        if ( endMisc.size() > 0 )
        {
            for ( int i = 0; i < endMisc.size(); i++ )
            {
                Misc m = (Misc) endMisc.get(i);		// get the next Misc
                if ( this.getOkToFormat() )
                    prefixMisc.append( EOL );
                postfixMisc.append( m.toXMLstring() );
            }		
        }

		XMLDecl.append( prefixMisc.toString() );
		XMLDecl.append( doctype.toString() );
		XMLDecl.append( postfixMisc.toString() );
        return XMLDecl.toString();
    }

    /*==========================================================================+
     |                                                                          |
     |		                  P A R S I N G  M E T H O D S                      |
     |                                                                          |
     +==========================================================================*/
	
    // prolog 		::= XMLDecl? Misc* (doctypedecl Misc*)?
    // XMLDecl 		::= '<?xml' VersionInfo EncodingDecl? SDDecl? S? '?>'
    // VersionInfo 	::= S 'version' Eq (' VersionNum ' | " VersionNum ")
    // Eq 			::= S? '=' S?
    // VersionNum   ::= ([a-zA-Z0-9_.:] | '-')+
    // Misc			::= Comment | PI |  S
    //
    // Recursive Descent Parse Methods
    //
    //	All the methods work as follows:
    //
    //	return positive count if a valid parse of the non-terminal occured.
    //	return zero if the non-terminal didn't parse.
    //		in this case the caller is expected to attempt another
    //		alternative.  
    //
    private int parseXMLdecl( ScanToken t ) throws XMLparseException
    {
        String msg = null;
        XMLparseException pBad;
	
        if ( t == null )
            t = scan.getToken();
        // check for '<?xml' that signals an XMLdecl
        if ( t.status != ScanToken.OK ) 
            return 0;
        if ( t.type != ScanToken.HUH_NAME )
            return 0;
        if ( !t.string.equals("xml") )
            return 0;
        /* have '<?xml', now get version = {xmlVersion} ... */
        // from here on, a parsing error will throw an exception
        t = scan.getToken();
        if ( t.status != ScanToken.OK )
        {
            // XMLDecl error
            //
            Object[] args1 = { "XML version", t.toString() };
            msg = MessageFormat.format( Messages.tokenError, args1 );
            pBad = new XMLparseException( msg );
            throw pBad; 
        }
        if ( (t.type != ScanToken.NAME) || (! t.string.equals("version") ) )
        {
            // XMLDecl version error
            // expecting version = 
            Object[] args2 = { "version", t.toString() };
            msg = MessageFormat.format( Messages.unexpected, args2 );
            pBad = new XMLparseException( msg );
            throw pBad; 
        }
        t = scan.getToken();
        if ( ! t.string.equals("=") )
        {
            Object[] args3 = { "=", t.toString() };
            msg = MessageFormat.format( Messages.unexpected, args3 );
            pBad = new XMLparseException( msg );
            throw pBad; 
        }
        t = scan.getToken();
        if ( t.type != ScanToken.QUOTED_STRING )
        {
            Object[] args4 = { "quoted value", t.toString() };
            msg = MessageFormat.format( Messages.unexpected, args4 );
            pBad = new XMLparseException( msg );
            throw pBad; 
        }
        if ( ! t.string.equals(Constants.XML_VERSION) )
        {
            Object[] args5 = { Constants.XML_VERSION, t.toString() };
            msg = MessageFormat.format( Messages.unexpected, args5 );
            pBad = new XMLparseException( msg );
            throw pBad; 
        }
        version = t.string;
        //
        // now scan for encoding
        //
        t = scan.getToken();
        if ( t.string.equals("encoding" ) )
        {
            t = scan.getToken();
            if ( ! t.string.equals("=") )
            {
                Object[] args6 = { "=", t.toString() };
                msg = MessageFormat.format( Messages.unexpected, args6 );
                pBad = new XMLparseException( msg );
                throw pBad; 
            }
            t = scan.getToken();
            if ( t.type != ScanToken.QUOTED_STRING )
            {
                Object[] args7 = { "Quoted encoding value", t.toString() };
                msg = MessageFormat.format( Messages.tokenError, args7 );
                pBad = new XMLparseException( msg );
                throw pBad; 
            }
            // Make sure value is legal
			 
            t = scan.getToken();
        }
        if ( t.string.equals("standalone") )
        {
            t = scan.getToken();
            if ( !t.string.equals("=") )
            {
                Object[] args8 = { "=", t.toString() };
                msg = MessageFormat.format( Messages.unexpected, args8 );
                pBad = new XMLparseException( msg );
                throw pBad; 
            }
            t = scan.getToken();
            if ( t.type != ScanToken.QUOTED_STRING )
            {
                Object[] args9 = { "Quoted standalone value", t.toString() };
                msg = MessageFormat.format( Messages.tokenError, args9 );
                pBad = new XMLparseException( msg );
                throw pBad; 
            }
            String answer = t.string.toLowerCase();
            if ( answer.equals("yes") )
            {
                standAlone = true;
            }
            else if ( answer.equals("no") )
            {
                standAlone = false;
            }
            else
            {
                Object[] args10 = { "'yes' or 'no'", t.toString() };
                msg = MessageFormat.format( Messages.unexpected, args10 );
                pBad = new XMLparseException( msg );
                throw pBad; 
            };
        }
        if ( !t.string.equals("?") )
        {
            Object[] args11 = { "?>", t.toString() };
            msg = MessageFormat.format( Messages.unexpected, args11 );
            pBad = new XMLparseException( msg );
            throw pBad; 
        }
        t = scan.getToken();
        if ( !t.string.equals(">") )
        {
            Object[] args12 = { "?>", t.toString() };
            msg = MessageFormat.format( Messages.unexpected, args12 );
            pBad = new XMLparseException( msg );
            throw pBad; 
        }
        return 1;		
    }    

    public int parseMisc( ScanToken t, List a ) throws XMLparseException
    {
        Misc m = new Misc( this );
        int gotMisc = 0;
        int count = 0;
	
        do 
        {
            try { gotMisc = m.parse( t ); }
            catch ( XMLparseException p )
            {
                throw p;
            }
            if ( gotMisc > 0 )
            {
                count++;
                a.add(m);
                m = new Misc( this );
                t = scan.getToken();
            }
        }
        while ( gotMisc > 0 );
        // got zero or more Misc 
        return count;
    }
    
    /*==========================================================================+
     |                                                                          |
     |		      C O N T E N T  M A N A G E M E N T  M E T H O D S			    |
     |                                                                          |
     +==========================================================================*/
    //
    //  Open Methods
    //
    
    //
    // open to current part
    //
    public int open() {
        
        if ( !isOpen )
        {
            isOpen = true;
            docIndex = PROLOG;
        }
        return docIndex;
        
    }
    //
    // open to selected part by name
    //
    public int open( String part ) {
        
        for ( int i = PARTSTART; i < docParts.length; i++ )
        {
            if ( part.equals(docParts[i]) )
            {
                docIndex = i;
                isOpen = true;
                return docIndex;
            }
        }
        return -1;
    }
    //
    // open to selected part by index
    //
    public int open( int index ) {
        if ( ( index >= 0 ) && ( index < docParts.length ) )
        {
            docIndex = index;
            isOpen = true;
            return docIndex;
        }
        return -1;
    }
    
    // Seek -- positions document to a selected part:
    //
    //  Seek( partname ) -- positions document to selected part.
    //  returns partname object.
    //
    
    //
    // seek to selected part by name
    //
    public int seek( String part )
    {
        for ( int i = PARTSTART; i < docParts.length; i++ )
        {
            if ( part.equals( docParts[i] ) )
            {
                docIndex = i;
                return docIndex;
            }
        }
        return -1;
    }
    //
    // seek to selected part by index
    //
    public int seek( int index )
    {
        if (( index >= 0 ) && ( index < docParts.length) )
        {
            docIndex = index;
            return docIndex;
        }
        return -1;
    }
    
    //	Update Methods
    //
        
    //  Any of the operations permitted on an element as defined
    //  below if open.  The element that is inferred is the current
    //  element.  If there is no current element, then the
    //  only operation's permitted are:
    //  
    //  addContent( root ) 
    //  addContent( root, 0 )
    
    public void add( Comment cmnt ) throws XMLdocException
    {
        XMLdocException docBad;
        String msg;
        
        switch( docIndex )
        {
                
        case XMLdoc.PREMISC:
            preMisc.add( cmnt );
            break;
        case XMLdoc.POSTMISC:
            postMisc.add( cmnt );
            break;
        case XMLdoc.ROOT:
            if ( current == null )
            {
                Object[] args = { dtd.getName(), "Comment" };
                msg = MessageFormat.format(Messages.documentEmpty , args );
                docBad = new XMLdocException(msg);
                throw docBad;
            }
            try 
            {
                current.add( cmnt );
            }
            catch ( ElementException ee )
            {
                docBad = new XMLdocException(ee.getMessage());
                throw docBad;
            }
            break;
        case XMLdoc.ENDMISC:
            endMisc.add( cmnt );
            break;
            // cannot add comments to Prolog or DTD.
        case XMLdoc.PROLOG:
        case XMLdoc.DTD:
            Object[] args = { Messages.frag_CommentThereBad };
            msg = MessageFormat.format( Messages.docUpdateInvalid, args );
            docBad = new XMLdocException( msg );
            throw docBad;
        default:
            break;
        }
    }

    public void add( Comment cmnt, int theIndex ) throws XMLdocException
    {
        XMLdocException docBad;
        String msg;
        
        switch ( docIndex )
        {
                
        case XMLdoc.PREMISC:
            preMisc.add( theIndex, cmnt );
            break;
        case XMLdoc.POSTMISC:
            postMisc.add( theIndex, cmnt );
            break;
        case XMLdoc.ROOT:
            if ( current == null )
            {
                Object[] args = { dtd.getName(), "Comment" };
                msg = MessageFormat.format(Messages.documentEmpty , args );
                docBad = new XMLdocException(msg);
                throw docBad;
            }
            try 
            {
                current.add( theIndex, cmnt );
            }
            catch ( ElementException ee )
            {
                docBad = new XMLdocException(ee.getMessage());
                throw docBad;
            }
            break;
        case XMLdoc.ENDMISC:
            endMisc.add( theIndex, cmnt );
            break;
            // cannot add comments to Prolog or DTD.
        case XMLdoc.PROLOG:
        case XMLdoc.DTD:
            Object[] args = { Messages.frag_CommentThereBad };
            msg = MessageFormat.format( Messages.docUpdateInvalid, args );
            docBad = new XMLdocException( msg );
            throw docBad;
        default:
            break;
        }
    }

    public void add( PI pi ) throws ElementException, XMLdocException
    {
        XMLdocException docBad;
        String msg;
        
        switch ( docIndex )
        {
        case XMLdoc.PREMISC:
            preMisc.add( pi );
            break;
        case XMLdoc.POSTMISC:
            postMisc.add( pi );
            break;
        case XMLdoc.ROOT:
            if ( current == null )
            {
                Object[] args = { dtd.getName(), "Processing Instruction" };
                msg = MessageFormat.format(Messages.documentEmpty , args );
                docBad = new XMLdocException(msg);
                throw docBad;
            }
            try 
            {
                current.add( pi );
            }
            catch ( ElementException ee )
            {
                docBad = new XMLdocException(ee.getMessage());
                throw docBad;
            }
            break;
        case XMLdoc.ENDMISC:
            endMisc.add( pi );
            break;
            // cannot add PIs to Prolog or DTD.
        case XMLdoc.PROLOG:
        case XMLdoc.DTD:
            Object[] args = { Messages.frag_CommentThereBad };
            msg = MessageFormat.format( Messages.docUpdateInvalid, args );
            docBad = new XMLdocException( msg );
            throw docBad;
        default:
            break;
        }
    }
 
    public void add( PI pi, int theIndex ) throws ElementException, XMLdocException
    {
        XMLdocException docBad;
        String msg;
        
        switch ( docIndex )
        {
                
        case XMLdoc.PREMISC:
            preMisc.add( theIndex, pi );
            break;
        case XMLdoc.POSTMISC:
            postMisc.add( theIndex, pi );
            break;
        case XMLdoc.ROOT:
            if ( current == null )
            {
                Object[] args = { dtd.getName(), "Processing Instruction" };
                msg = MessageFormat.format(Messages.documentEmpty , args );
                docBad = new XMLdocException(msg);
                throw docBad;
            }
            current.add( theIndex, pi );
            break;
        case XMLdoc.ENDMISC:
            endMisc.add( theIndex, pi );
            break;
            // cannot add comments to Prolog or DTD.
        case XMLdoc.PROLOG:
        case XMLdoc.DTD:
            Object[] args = { Messages.frag_CommentThereBad };
            msg = MessageFormat.format( Messages.docUpdateInvalid, args );
            docBad = new XMLdocException( msg );
            throw docBad;
        default:
            break;
        }
    }
    
    public void add( Element e ) throws XMLdocException, ElementException
    {
        // add e to the end of the current element
        add( e, -1 );
    }
    
    public void add( Element e , int theIndex ) throws ElementException, XMLdocException
    {
        ElementException eBad;
        XMLdocException docBad;
        String msg;
        
        if ( this.docIndex != ROOT )
        {
            Object[] args = { Messages.frag_ElementThereBad };
            msg = MessageFormat.format( Messages.docUpdateInvalid, args );
            docBad = new XMLdocException( msg );
            throw docBad;
        }
        // Check to add in the root:
        if ( this.root == null )
        {
            if ( current != null )
            {
                Object[] args = { e.getName(), current.getName(), "root element is NULL!" };
                msg = MessageFormat.format( Messages.element_logic_error, args );
                eBad = new ElementException( msg );
                throw eBad;
            }
            // this is the first element of the XML document...
            ElementDecl elementDecl = dtd.getElementDecl( e.getName() );
            // as a matter of policy, we could declare the element here...
            if ( (elementDecl == null) || ( elementDecl.getIsUndeclared() ) )
            {   // element is not declared and so the document is not valid. 
                Object[] args = { e.getName() };
                msg = MessageFormat.format( Messages.element_undeclared, args );
                eBad = new ElementException( msg );
                throw eBad;
            }
            // If a dtd's root is declared but this element is not the root
            if ( ( dtd.getName() != null) && !( e.getName().equals( dtd.getName() ) ) )
            {
                Object[] args = { e.getName(), dtd.getName() };
                msg = MessageFormat.format( Messages.element_not_root, args );
                eBad = new ElementException( msg );
                throw eBad;
            }
            // note that the root may not have been declared until now...
            this.setRoot(e);
            // make sure that this element knows which doc it is in...
            // $REMOVED$ e.setDoc(this);
            return;
        }
        // now - we MUST have a current element
        if ( current == null )
        {
            Object[] args = { e.getName(), "NULL", "current element is NULL but root exists!" };
            msg = MessageFormat.format( Messages.element_logic_error, args );
            eBad = new ElementException( msg );
            throw eBad;
        }
        try {
            if ( theIndex < 0 )
            {
                theIndex = current.getCount();
            }
            current.add( theIndex, e );
        }
        catch (ElementException ee )
        { throw ee; }
    }

    
    public void add( ContentReference cr ) throws ElementException, XMLdocException
    {
            add( cr, -1 );
    }
    

    public void add( ContentReference cr, int theIndex ) throws ElementException, XMLdocException
    {
        XMLdocException docBad;
        String msg;
        
        if ( (docIndex != ROOT) || (root == null) )
        {
            Object[] args = { Messages.frag_ContentThereBad };
            msg = MessageFormat.format( Messages.docUpdateInvalid, args );
            docBad = new XMLdocException( msg );
            throw docBad;
        }
        if ( current == null ) 
        {
            Object[] args = { Messages.frag_ContentThereBad };
            msg = MessageFormat.format( Messages.docUpdateInvalid, args );
            docBad = new XMLdocException( msg );
            throw docBad;
        }
        
        try {
            if ( theIndex < 0 )
            {
                theIndex = current.getCount();
            }
            current.add( theIndex, cr);
        }
        catch (ElementException ee )
        { throw ee; }
    }

    //
    // Add Attributes
    //
    
    
    // 
    // xmlDecl attributes
    //
    static private final String xmlDeclAttributes[] = 
    {
        "version", "encoding", "standalone"
    };
    
    //
    // version valid values:
    //
    static private final String versions[] =
    {
        Constants.XML_VERSION_1_0
    };
    
    //
    // encoding valid values:
    //
    
    static private final String encodings[] =
    {
        Constants.XML_ENCODING_UTF16,
        Constants.XML_ENCODING_UTF8,
        // Note that Java supports only the UTF-8 (reading/writing) and 
        // UTF-16 encoding (internal) -- That is UTF-8 is converted to 
        // UTF-16 on reading and UTF-16 is converted to UTF-8 on writing when
        // UTF-8 is the selected encoding.
        //
        Constants.XML_ENCODING_UCS2,
        Constants.XML_ENCODING_UCS4,
        //
        // The following encodings are supported only in-so-far as they are represented
        // in UTF-16.
        //
        Constants.XML_ENCODING_8859_1,
        Constants.XML_ENCODING_8859_2,
        Constants.XML_ENCODING_8859_3,
        Constants.XML_ENCODING_8859_4 ,
        Constants.XML_ENCODING_8859_5,
        Constants.XML_ENCODING_8859_6,
        Constants.XML_ENCODING_8859_7,
        Constants.XML_ENCODING_8859_8,
        Constants.XML_ENCODING_8859_9,
        //
        Constants.XML_ENCODING_2022_JP,
        Constants.XML_ENCODING_Shift_JIS,
        Constants.XML_ENCODING_EUC_JP
    };
    
    //
    // standalone valid values
    //
    
    static private final String standalones[] =
    {
        "no", "NO", "yes", "YES"
    };
    
        
    private void addPrologAttribute( String name, String val ) throws XMLdocException
    {
        XMLdocException docBad;
        String msg;
        int i, j;
        
        for ( i = 0; i < xmlDeclAttributes.length; i++ )
        {
        
            if ( name.equals( xmlDeclAttributes[i]))
                break;
        }
        if ( i >= xmlDeclAttributes.length )
        {
            Object[] args = { name, "unknown xmlDecl attribute" };
            msg = MessageFormat.format( Messages.xmlDeclAttributeInvalid, args );
            docBad = new XMLdocException( msg );
            throw docBad;
        }
        switch( i )
        {
            case 0: // version
                for ( j = 0; j < versions.length; j++ )
                    if ( val.equals(versions[j]) )
                        break;
                if ( j >= versions.length )
                {
                    // Version value is invalid.
                    Object[] args = { name, "version must be \"" + Constants.XML_VERSION_1_0 +"\"" };
                    msg = MessageFormat.format( Messages.xmlDeclAttributeInvalid, args );
                    docBad = new XMLdocException( msg );
                    throw docBad;
                }
                this.setVersion(val);
                break;
            case 1: // encoding
                for ( j = 0; j < encodings.length; j++ )
                    if ( val.equals(encodings[j]) )
                        break;
                if ( j >= encodings.length )
                {
                    // Encoding value is invalid.
                    Object[] args = { name, "encoding value, \"" + val + "\"!" };
                    msg = MessageFormat.format( Messages.xmlDeclAttributeInvalid, args );
                    docBad = new XMLdocException( msg );
                    throw docBad;
                }
                this.setEncoding(val);
                break;
            case 2: // standalone
                for ( j = 0; j < standalones.length; j++ )
                    if ( val.equals(standalones[j]) )
                        break;
                if ( j >= standalones.length )
                {
                    // Encoding value is invalid.
                    Object[] args = { name, "standalone value, \"" + val + "\"!" };
                    msg = MessageFormat.format( Messages.xmlDeclAttributeInvalid, args );
                    docBad = new XMLdocException( msg );
                    throw docBad;
                }
                this.setEncoding(val);
                break;
        }
        
    }
    
    static private final String docTypeAttributes[] =
    {
        "root", "SYSTEM", "PUBLIC"
    };
    
    private void addDocTypeAttribute( String name, String val ) throws XMLdocException
    {
        XMLdocException docBad;
        String msg;
        int i, j;
        
        for ( i = 0; i < docTypeAttributes.length; i++ )
        {
        
            if ( name.equals( docTypeAttributes[i]))
                break;
        }
        if ( i >= docTypeAttributes.length )
        {
            Object[] args = { name, "unknown DocType attribute" };
            msg = MessageFormat.format( Messages.docTypeAttributeInvalid, args );
            docBad = new XMLdocException( msg );
            throw docBad;
        }
        switch( i )
        {
        case 0:  // root element setting
            break;
        case 1:  // SYSTEM url setting
            break;
        case 2:  // PUBLIC url setting
            break;
        }
    }
    
    private void addElementAttribute( String name, String val ) throws AttributeException
    {
        AttributeException aBad = null;
        String msg = null;
        
        if ( elementStack.empty() )
        {
            Object[] args = { name, Messages.frag_DocumentEmpty};
            msg = MessageFormat.format( Messages.element_attribute_undeclared, args );
            aBad = new AttributeException( msg );
            throw aBad;
        }
        try {
            current.setAttribute( name, val );
        }
        catch ( AttributeException ae )
        {
            throw ae;
        }
    }


    public void addAttribute( String name, String val ) throws AttributeException, XMLdocException
    {
        AttributeException aBad;
        XMLdocException docBad;
        
        String msg;
        
        switch ( docIndex )
        {
        case XMLdoc.PROLOG:
            addPrologAttribute( name, val );
            break;
        case XMLdoc.DTD:
            addDocTypeAttribute( name, val );
            break;
        case XMLdoc.ROOT:
            addElementAttribute( name, val );
            break;
        case XMLdoc.PREMISC:
        case XMLdoc.POSTMISC:
        case XMLdoc.ENDMISC:
            // Comments and PIs don't have attributes
            Object[] args0 = { name + " = " + val };
            msg = MessageFormat.format( Messages.attributeMiscIllegal, args0 );
            aBad = new AttributeException( msg );
            throw aBad;
        default:
            // Document not open.
            Object[] args1 = { "addAttribute", name + " = " + val };
            msg = MessageFormat.format( Messages.documentNotOpen, args1 );
            docBad = new XMLdocException( msg );
            throw docBad;
        }
    }
    
    //
    //	Replace methods.
    //
    
    public void replace( Comment cmnt, int theIndex ) throws XMLdocException
    {
        
        XMLdocException docBad;
        String msg;
        
        switch ( docIndex )
        {
                
        case XMLdoc.PREMISC:
            if ( (theIndex < 0) || (theIndex >= preMisc.size() ) ) 
            {
                // bad -- cannot do the replacement
                Object[] args = { 
                    docParts[XMLdoc.PREMISC],
                    "replace",
                    new Integer( 0 ),
                    new Integer( preMisc.size() ),
                    new Integer( theIndex )
                };
                msg = MessageFormat.format( Messages.documentIndexInvalid, args );
                docBad = new XMLdocException( msg );
                throw docBad;
            }
            preMisc.set( theIndex, cmnt );
            break;
        case XMLdoc.POSTMISC:
            if ( (theIndex < 0) || (theIndex >= postMisc.size() ) ) 
            {
                // bad -- cannot do the replacement
                Object[] args = { 
                    docParts[XMLdoc.POSTMISC],
                    "replace",
                    new Integer( 0 ),
                    new Integer( postMisc.size() ),
                    new Integer( theIndex )
                };
                msg = MessageFormat.format( Messages.documentIndexInvalid, args );
                docBad = new XMLdocException( msg );
                throw docBad;
            }
            postMisc.set( theIndex, cmnt );
            break;
        case XMLdoc.ROOT:
            if ( current == null )
            {
                Object[] args = { dtd.getName(), "Comment" };
                msg = MessageFormat.format(Messages.documentEmpty , args );
                docBad = new XMLdocException(msg);
                throw docBad;
            }
            try 
            {
                current.replace( theIndex, cmnt);
            }
            catch ( ElementException ee )
            {
                docBad = new XMLdocException(ee.getMessage());
                throw docBad;
            }
            break;
        case XMLdoc.ENDMISC:
            if ( (theIndex < 0) || (theIndex >= endMisc.size() ) ) 
            {
                // bad -- cannot do the replacement
                // bad -- cannot do the replacement
                Object[] args = { 
                    docParts[XMLdoc.ENDMISC],
                    "replace",
                    new Integer( 0 ),
                    new Integer( endMisc.size() ),
                    new Integer( theIndex )
                };
                msg = MessageFormat.format( Messages.documentIndexInvalid, args );
                docBad = new XMLdocException( msg );
                throw docBad;
            }
            break;
            // cannot add comments to Prolog or DTD.
        case XMLdoc.PROLOG:
        case XMLdoc.DTD:
            Object[] args0 = { Messages.frag_CommentThereBad };
            msg = MessageFormat.format( Messages.docUpdateInvalid, args0 );
            docBad = new XMLdocException( msg );
            throw docBad;
        default:
            Object[] args1 = { "replace", cmnt.get() };
            msg = MessageFormat.format( Messages.documentNotOpen, args1 );
            docBad = new XMLdocException( msg );
            throw docBad;
        }
    }
    
    public void replace( PI pi, int theIndex ) throws XMLdocException
    {
        XMLdocException docBad;
        String msg;
        
        switch ( docIndex )
        {
                
        case XMLdoc.PREMISC:
            if ( (theIndex < 0) || (theIndex >= preMisc.size() ) ) 
            {
                // bad -- cannot do the replacement
                Object[] args = { 
                    docParts[XMLdoc.PREMISC],
                    "replace",
                    new Integer( 0 ),
                    new Integer( preMisc.size() ),
                    new Integer( theIndex )
                };
                msg = MessageFormat.format( Messages.documentIndexInvalid, args );
                docBad = new XMLdocException( msg );
                throw docBad;
            }
            preMisc.set( theIndex, pi );
            break;
        case XMLdoc.POSTMISC:
            if ( (theIndex < 0) || (theIndex >= postMisc.size() ) ) 
            {
                // bad -- cannot do the replacement
                Object[] args = { 
                    docParts[XMLdoc.POSTMISC],
                    "replace",
                    new Integer( 0 ),
                    new Integer( postMisc.size() ),
                    new Integer( theIndex )
                };
                msg = MessageFormat.format( Messages.documentIndexInvalid, args );
                docBad = new XMLdocException( msg );
                throw docBad;
            }
            postMisc.set( theIndex, pi );
            break;
        case XMLdoc.ROOT:
            if ( current == null )
            {
                Object[] args = { dtd.getName(), "PI" };
                msg = MessageFormat.format(Messages.documentEmpty , args );
                docBad = new XMLdocException(msg);
                throw docBad;
            }
            try 
            {
                current.replace( theIndex, pi );
            }
            catch ( ElementException ee )
            {
                docBad = new XMLdocException(ee.getMessage());
                throw docBad;
            }
            break;
        case XMLdoc.ENDMISC:
            if ( (theIndex < 0) || (theIndex >= endMisc.size() ) ) 
            {
                // bad -- cannot do the replacement
                Object[] args = { 
                    docParts[XMLdoc.ENDMISC],
                    "replace",
                    new Integer( 0 ),
                    new Integer( endMisc.size() ),
                    new Integer( theIndex )
                };
                msg = MessageFormat.format( Messages.documentIndexInvalid, args );
                docBad = new XMLdocException( msg );
                throw docBad;
            }
            endMisc.set( theIndex, pi );
            break;
            // cannot add PIs to Prolog or DTD.
        case XMLdoc.PROLOG:
        case XMLdoc.DTD:
            Object[] args0 = { Messages.frag_CommentThereBad };
            msg = MessageFormat.format( Messages.docUpdateInvalid, args0 );
            docBad = new XMLdocException( msg );
            throw docBad;
        default:
            Object[] args1 = { "replace", pi.getTarget() };
            msg = MessageFormat.format( Messages.documentNotOpen, args1 );
            docBad = new XMLdocException( msg );
            throw docBad;
        }
    }
    
    public void replace( Element e, int theIndex ) throws ElementException, XMLdocException
    {
        ElementException eBad;
        XMLdocException docBad;
        String msg;
        
        if ( docIndex != ROOT )
        {
            Object[] args = { Messages.frag_ElementThereBad };
            msg = MessageFormat.format( Messages.docUpdateInvalid, args );
            docBad = new XMLdocException( msg );
            throw docBad;
        }
        
        if ( current == null )
        {
            // this is the first element of the XML document...
            ElementDecl elementDecl = dtd.getElementDecl( e.getName() );
            // as a matter of policy, we could declare the element here...
            if ( (elementDecl == null) || ( elementDecl.getIsUndeclared() ) )
            {   // element is not declared and so the document is not valid. 
                Object[] args = { e.getName() };
                msg = MessageFormat.format( Messages.element_undeclared, args );
                eBad = new ElementException( msg );
                throw eBad;
            }
            if ( ! e.getName().equals( dtd.getName() ) )
            {
                Object[] args = { e.getName(), dtd.getName() };
                msg = MessageFormat.format( Messages.element_not_root, args );
                eBad = new ElementException( msg );
                throw eBad;
            }
            
            this.setCurrent(e);
        
        }
        try {
            current.replace( theIndex, e );
        }
        catch (ElementException ee )
        { throw ee; }
    }
        
    public void replace( ContentReference cr, int theIndex ) throws ElementException, XMLdocException
    {
        XMLdocException docBad;
        String msg;
        
        if ( (docIndex != ROOT) || (root == null) )
        {
            Object[] args = { Messages.frag_ContentThereBad };
            msg = MessageFormat.format( Messages.docUpdateInvalid, args );
            docBad = new XMLdocException( msg );
            throw docBad;
        }
        
        try { current.replace( theIndex, cr ); }
        catch (ElementException ee ) { throw ee; }
    }
    
    //
    // Attribute replacement
    //
    
    private void replaceElementAttribute( String name, String value ) throws AttributeException
    {
        AttributeException aBad = null;
        String msg = null;
        
        if ( elementStack.empty() )
        {
            Object[] args = { name, Messages.frag_DocumentEmpty };
            msg = MessageFormat.format( Messages.element_attribute_undeclared, args );
            aBad = new AttributeException( msg );
            throw aBad;
        }
        try { current.setAttribute( name, value ); }
        catch ( AttributeException ae ) {   throw ae;   }
    }
    
    private void replacePrologAttribute( String name, String value ) throws XMLdocException
    {
        XMLdocException docBad;
        String msg;
        int i, j;
        
        for ( i = 0; i < xmlDeclAttributes.length; i++ )
        {
        
            if ( name.equals( xmlDeclAttributes[i]))
                break;
        }
        if ( i >= xmlDeclAttributes.length )
        {
            Object[] args = { name, "unknown xmlDecl attribute" };
            msg = MessageFormat.format( Messages.xmlDeclAttributeInvalid, args );
            docBad = new XMLdocException( msg );
            throw docBad;
        }
        switch( i )
        {
            case 0: // version
                for ( j = 0; j < versions.length; j++ )
                    if ( value.equals(versions[j]) )
                        break;
                if ( j >= versions.length )
                {
                    // Version value is invalid.
                    Object[] args = { name, "version must be \"" + Constants.XML_VERSION_1_0 +"\"" };
                    msg = MessageFormat.format( Messages.xmlDeclAttributeInvalid, args );
                    docBad = new XMLdocException( msg );
                    throw docBad;
                }
                this.setVersion(value);
                break;
            case 1: // encoding
                for ( j = 0; j < encodings.length; j++ )
                    if ( value.equals(encodings[j]) )
                        break;
                if ( j >= encodings.length )
                {
                    // Encoding value is invalid.
                    Object[] args = { name, "encoding value, \"" + value + "\"!" };
                    msg = MessageFormat.format( Messages.xmlDeclAttributeInvalid, args );
                    docBad = new XMLdocException( msg );
                    throw docBad;
                }
                this.setEncoding(value);
                break;
            case 2: // standalone
                for ( j = 0; j < standalones.length; j++ )
                    if ( value.equals(standalones[j]) )
                        break;
                if ( j >= standalones.length )
                {
                    // Encoding value is invalid.
                    Object[] args = { name, "standalone value, \"" + value + "\"!" };
                    msg = MessageFormat.format( Messages.xmlDeclAttributeInvalid, args );
                    docBad = new XMLdocException( msg );
                    throw docBad;
                }
                if ( value.equals("yes") || value.equals("YES") )
                    this.setStandAlone(true);
                else 
                    this.setStandAlone(false);
                break;
        }
    }
    
    private void replaceDocTypeAttribute( String name, String value ) throws XMLdocException
    {
        XMLdocException docBad;
        String msg;
        int i, j;
        
        for ( i = 0; i < docTypeAttributes.length; i++ )
        {
        
            if ( name.equals( docTypeAttributes[i]))
                break;
        }
        if ( i >= docTypeAttributes.length )
        {
            Object[] args = { name, "unknown DocType attribute" };
            msg = MessageFormat.format( Messages.docTypeAttributeInvalid, args );
            docBad = new XMLdocException( msg );
            throw docBad;
        }
        switch( i )
        {
        case 0:  // root element setting
            if ( dtd == null )
            {
                //$FIX$ -- a new DTD needs built.
            }
            else
            {
                if ( !dtd.getName().equals(value) )
                {
                    Object[] args = { name, "unknown DocType attribute" };
                    msg = MessageFormat.format( Messages.docTypeAttributeInvalid, args );
                    docBad = new XMLdocException( msg );
                    throw docBad;
                }
            }
            break;
        case 1:  // SYSTEM url setting
            break;
        case 2:  // PUBLIC url setting
            break;
        }
    }
    
    private void replaceAttribute( String name, String value ) throws AttributeException, XMLdocException
    {
        AttributeException aBad;
        XMLdocException docBad;
        
        String msg;
        
        switch ( docIndex )
        {
        case XMLdoc.PROLOG:
            replacePrologAttribute( name, value );
            break;
        case XMLdoc.DTD:
            replaceDocTypeAttribute( name, value );
            break;
        case XMLdoc.ROOT:
            replaceElementAttribute( name, value );
            break;
        case XMLdoc.PREMISC:
        case XMLdoc.POSTMISC:
        case XMLdoc.ENDMISC:
            // Comments and PIs don't have attributes
            Object[] args0 = { name + " = " + value };
            msg = MessageFormat.format( Messages.attributeMiscIllegal, args0 );
            aBad = new AttributeException( msg );
            throw aBad;
        default:
            // Document not open.
            Object[] args1 = { "replaceAttribute", name + " = " + value };
            msg = MessageFormat.format( Messages.documentNotOpen, args1 );
            docBad = new XMLdocException( msg );
            throw docBad;
        }
    }
    
    //
    //  Delete Methods
    //

    public void delete( Comment cmnt ) throws ElementException, XMLdocException
    {
        XMLdocException docBad;
        String msg;
        int theIndex = -1;

        switch ( docIndex )
        {
        case XMLdoc.PREMISC:
            theIndex = preMisc.size() - 1;
            break;
        case XMLdoc.POSTMISC:
            theIndex = postMisc.size() - 1;
            break;
        case XMLdoc.ROOT:
            if ( current == null )
            {
                Object[] args = { dtd.getName(), "Comment" };
                msg = MessageFormat.format(Messages.documentEmpty , args );
                docBad = new XMLdocException(msg);
                throw docBad;
            }
            try 
            {
                Object ob = null;
                int i = current.getIndex();
                if ( i >= 0 )
                    ob = current.getContent( i );
                if ( ob instanceof Comment )
                {
                    current.delete( i );
                }
                else
                {
                    Object[] args = { current.getName() + " Comment deletion" };
                    msg = MessageFormat.format(Messages.docUpdateInvalid , args );
                    docBad = new XMLdocException(msg);
                    throw docBad;
                }
            }
            catch ( ElementException ee )
            {
                docBad = new XMLdocException(ee.getMessage());
                throw docBad;
            }
            return;
        case XMLdoc.ENDMISC:
            theIndex = endMisc.size() - 1;
            break;
            // cannot add comments to Prolog or DTD.
        case XMLdoc.PROLOG:
        case XMLdoc.DTD:
            Object[] args = { Messages.frag_CommentThereBad };
            msg = MessageFormat.format( Messages.docUpdateInvalid, args );
            docBad = new XMLdocException( msg );
            throw docBad;
        default:
            // doc not open error
            break;
        }
        if ( theIndex >= 0 )
            delete( cmnt, theIndex );
        
    }
    
    public void delete( Comment cmnt, int theIndex ) throws XMLdocException
    {
        XMLdocException docBad;
        String msg;
        
        switch ( docIndex )
        {
        case XMLdoc.PREMISC:
            preMisc.remove(theIndex);
            break;
        case XMLdoc.POSTMISC:
            postMisc.remove( theIndex );
            break;
        case XMLdoc.ROOT:
            if ( current == null )
            {
                Object[] args = { dtd.getName(), "Comment" };
                msg = MessageFormat.format(Messages.documentEmpty , args );
                docBad = new XMLdocException(msg);
                throw docBad;
            }
            try 
            {
                Object ob = current.getContent( theIndex );
                if ( ob instanceof Comment )
                    current.delete( theIndex );
                else
                {
                    Object[] args = { current.getName() + " Comment deletion" };
                    msg = MessageFormat.format(Messages.docUpdateInvalid , args );
                    docBad = new XMLdocException(msg);
                    throw docBad;
                }
            }
            catch ( ElementException ee )
            {
                docBad = new XMLdocException(ee.getMessage());
                throw docBad;
            }
            break;
        case XMLdoc.ENDMISC:
            endMisc.remove( theIndex );
            break;
            // cannot add comments to Prolog or DTD.
        case XMLdoc.PROLOG:
        case XMLdoc.DTD:
            Object[] args = { Messages.frag_CommentThereBad };
            msg = MessageFormat.format( Messages.docUpdateInvalid, args );
            docBad = new XMLdocException( msg );
            throw docBad;
        default:
            break;
        }
    }
    
    public void delete( PI pi ) throws ElementException, XMLdocException
    {
        XMLdocException docBad;
        String msg;
        int theIndex = -1;

        switch ( docIndex )
        {
        case XMLdoc.PREMISC:
            theIndex = preMisc.size() - 1;
            break;
        case XMLdoc.POSTMISC:
            theIndex = postMisc.size() - 1;
            break;
        case XMLdoc.ROOT:
            if ( current == null )
            {
                Object[] args = { dtd.getName(), "PI" };
                msg = MessageFormat.format(Messages.documentEmpty , args );
                docBad = new XMLdocException(msg);
                throw docBad;
            }
            try 
            {
                theIndex = current.getIndex();
                Object ob = null;
                if ( theIndex >= 0 )
                    ob = current.getContent( theIndex );
                if ( ob instanceof PI )
                    current.delete( theIndex );
                else
                {
                    Object[] args = { current.getName() + " PI deletion" };
                    msg = MessageFormat.format(Messages.docUpdateInvalid , args );
                    docBad = new XMLdocException(msg);
                    throw docBad;
                }
            }
            catch ( ElementException ee )
            {
                docBad = new XMLdocException(ee.getMessage());
                throw docBad;
            }
            return;
        case XMLdoc.ENDMISC:
            theIndex = endMisc.size() - 1;
            break;
            // cannot add comments to Prolog or DTD.
        case XMLdoc.PROLOG:
            Object[] args = { Messages.frag_PiThereBad };
            msg = MessageFormat.format( Messages.docUpdateInvalid, args );
            docBad = new XMLdocException( msg );
            throw docBad;
        case XMLdoc.DTD:
            // $FIX$ -- DTD's can contain PIs
        default:
            // doc not open error
            break;
        }
        if ( theIndex >= 0 )
            delete( pi, theIndex );
        // invalid delete index error.
    }
    
    public void delete( PI pi, int theIndex ) throws ElementException, XMLdocException
    {
        XMLdocException docBad;
        String msg;
        
        switch ( docIndex )
        {
        case XMLdoc.PREMISC:
            preMisc.remove( theIndex );
            break;
        case XMLdoc.POSTMISC:
            postMisc.remove( theIndex );
            break;
        case XMLdoc.ROOT:
            if ( current == null )
            {
                Object[] args = { "delete", "PI" };
                msg = MessageFormat.format(Messages.documentEmpty , args );
                docBad = new XMLdocException(msg);
                throw docBad;
            }
            try 
            {
                Object ob = current.getContent( theIndex );
                if ( ob instanceof PI )
                    current.delete( theIndex );
                else
                {
                    Object[] args = { current.getName() + " PI deletion" };
                    msg = MessageFormat.format(Messages.docUpdateInvalid , args );
                    docBad = new XMLdocException(msg);
                    throw docBad;
                }
            }
            catch ( ElementException ee )
            {
                docBad = new XMLdocException(ee.getMessage());
                throw docBad;
            }
            break;
        case XMLdoc.ENDMISC:
            endMisc.remove( theIndex );
            break;
            // cannot add comments to Prolog or DTD.
        case XMLdoc.PROLOG:
            Object[] args = { Messages.frag_PiThereBad };
            msg = MessageFormat.format( Messages.docUpdateInvalid, args );
            docBad = new XMLdocException( msg );
            throw docBad;
        case XMLdoc.DTD:
            // $FIX$ - DTD can contain PI's
        default:
            break;
        }
    }
    
    public void delete( Element e ) throws ElementException, XMLdocException
    {
        ElementException eBad;
        XMLdocException docBad;
        String msg;
        
        if ( docIndex != ROOT )
        {
            Object[] args = { Messages.frag_ElementThereBad };
            msg = MessageFormat.format( Messages.docUpdateInvalid, args );
            docBad = new XMLdocException( msg );
            throw docBad;
        }
        
        if ( current == null )
        {
            // this is the first element of the XML document...
            ElementDecl elementDecl = dtd.getElementDecl( e.getName() );
            // as a matter of policy, we could declare the element here...
            if ( (elementDecl == null) || ( elementDecl.getIsUndeclared() ) )
            {   // element is not declared and so the document is not valid. 
                Object[] args = { e.getName() };
                msg = MessageFormat.format( Messages.element_undeclared, args );
                eBad = new ElementException( msg );
                throw eBad;
            }
            if ( ! e.getName().equals( dtd.getName() ) )
            {
                Object[] args = { e.getName(), dtd.getName() };
                msg = MessageFormat.format( Messages.element_not_root, args );
                eBad = new ElementException( msg );
                throw eBad;
            }
            this.setCurrent(e);
        }
        try {
            int theIndex = current.getIndex();
            Object ob = null;
            if ( theIndex >= 0 )
                ob = current.getContent( theIndex );
            if ( ob instanceof Element )
                current.delete( theIndex );
            else
            {
                Object[] args = { current.getName() + " Element deletion" };
                msg = MessageFormat.format(Messages.docUpdateInvalid , args );
                docBad = new XMLdocException(msg);
                throw docBad;
            }
        }
        catch (ElementException ee ) { throw ee; }
    }
    
    public void delete( Element e, int theIndex ) throws ElementException, XMLdocException
    {
        ElementException eBad;
        XMLdocException docBad;
        String msg;
        
        if ( docIndex != ROOT )
        {
            Object[] args = { Messages.frag_ElementThereBad };
            msg = MessageFormat.format( Messages.docUpdateInvalid, args );
            docBad = new XMLdocException( msg );
            throw docBad;
        }
        
        if ( current == null )
        {
            // this is the first element of the XML document...
            ElementDecl elementDecl = dtd.getElementDecl( e.getName() );
            // as a matter of policy, we could declare the element here...
            if ( (elementDecl == null) || ( elementDecl.getIsUndeclared() ) )
            {   // element is not declared and so the document is not valid. 
                Object[] args = { e.getName() };
                msg = MessageFormat.format( Messages.element_undeclared, args );
                eBad = new ElementException( msg );
                throw eBad;
            }
            if ( ! e.getName().equals( dtd.getName() ) )
            {
                Object[] args = { e.getName(), dtd.getName() };
                msg = MessageFormat.format( Messages.element_not_root, args );
                eBad = new ElementException( msg );
                throw eBad;
            }
            this.setCurrent(e);
        }
        try { 
            
            Object ob = current.getContent( theIndex );
            if ( (ob instanceof Element) && (e.getName().equals( ( (Element)ob ).getName())) )
                current.delete( theIndex );
            else
            {
                Object[] args = { current.getName() + " Element deletion" };
                msg = MessageFormat.format(Messages.docUpdateInvalid , args );
                docBad = new XMLdocException(msg);
                throw docBad;
            }
        }
        catch (ElementException ee ) { throw ee; }
    }
    
    public void delete( ContentReference cr ) throws ElementException, XMLdocException
    {
        XMLdocException docBad;
        String msg;
        
        if ( docIndex != ROOT )
        {
            Object[] args = { Messages.frag_ElementThereBad };
            msg = MessageFormat.format( Messages.docUpdateInvalid, args );
            docBad = new XMLdocException( msg );
            throw docBad;
        }
        
        if ( current == null )
        {
            Object[] args = { "Delete", "ContentReference" };
            msg = MessageFormat.format( Messages.documentEmpty, args );
            docBad = new XMLdocException( msg );
            throw docBad;
        }
        try {
            int theIndex = current.getIndex();
            Object ob = current.getContent( theIndex );
            if ( ob instanceof ContentReference )
                current.delete( theIndex );
            else
            {
                Object[] args = { current.getName() + " Content Reference deletion" };
                msg = MessageFormat.format(Messages.docUpdateInvalid , args );
                docBad = new XMLdocException(msg);
                throw docBad;
            }
        }
        catch (ElementException ee ) { throw ee; }
    }
    
    public void delete( ContentReference cr, int theIndex ) throws ElementException, XMLdocException
    {
        XMLdocException docBad;
        String msg;
        
        if ( docIndex != ROOT )
        {
            Object[] args = { Messages.frag_ElementThereBad };
            msg = MessageFormat.format( Messages.docUpdateInvalid, args );
            docBad = new XMLdocException( msg );
            throw docBad;
        }
        
        if ( current == null )
        {
            Object[] args = { "Delete", "ContentReference" };
            msg = MessageFormat.format( Messages.documentEmpty, args );
            docBad = new XMLdocException( msg );
            throw docBad;
        }
        try { 
            Object ob = current.getContent( theIndex );
            if ( ob instanceof ContentReference )
                current.delete( theIndex );
            else
            {
                Object[] args = { current.getName() + " Content Reference deletion" };
                msg = MessageFormat.format(Messages.docUpdateInvalid , args );
                docBad = new XMLdocException(msg);
                throw docBad;
            }            
        }
        catch (ElementException ee ) { throw ee; }
    }
    
    //
    // Attribute deletion
    //
    
    private void deleteElementAttribute( String aName, String aValue ) throws AttributeException
    {
        AttributeException aBad = null;
        String msg = null;
        
        if ( elementStack.empty() )
        {
            Object[] args = { aName, Messages.frag_DocumentEmpty };
            msg = MessageFormat.format( Messages.element_attribute_undeclared, args );
            aBad = new AttributeException( msg );
            throw aBad;
        }
        // note ignore the value parameter -- it is not needed.
        try { current.deleteAttribute( aName ); }
        catch ( AttributeException ae ) {   throw ae;   }
    }
    
    private void deletePrologAttribute( String name, String value ) throws XMLdocException
    {
        XMLdocException docBad;
        String msg;
        int i, j;
        
        for ( i = 0; i < xmlDeclAttributes.length; i++ )
        {
        
            if ( name.equals( xmlDeclAttributes[i]))
                break;
        }
        if ( i >= xmlDeclAttributes.length )
        {
            Object[] args = { name, "unknown xmlDecl attribute" };
            msg = MessageFormat.format( Messages.xmlDeclAttributeInvalid, args );
            docBad = new XMLdocException( msg );
            throw docBad;
        }
        switch( i )
        {
            case 0: // version
                if ( !getVersion().equals( value ) )
                {
                    // Version value is invalid.
                    Object[] args = { name, "version must be \"" + Constants.XML_VERSION_1_0 +"\"" };
                    msg = MessageFormat.format( Messages.xmlDeclAttributeInvalid, args );
                    docBad = new XMLdocException( msg );
                    throw docBad;
                }
                this.setVersion(Constants.XML_VERSION);
                break;
            case 1: // encoding
                if ( !getEncoding().equals( value ) )
                {
                    // Encoding value is invalid.
                    Object[] args = { name, "encoding value, \"" + value + "\"!" };
                    msg = MessageFormat.format( Messages.xmlDeclAttributeInvalid, args );
                    docBad = new XMLdocException( msg );
                    throw docBad;
                }
                this.setEncoding( Constants.XML_ENCODING );
                break;
            case 2: // standalone
                boolean gsay =  getStandAlone() && ( value.equals("YES") || value.equals("yes") ) ;
                boolean gsan =  !getStandAlone() && ( value.equals("NO") || value.equals("no") );
                
                if ( !gsay || !gsan  )
                {
                    // Encoding value is invalid.
                    Object[] args = { name, "standalone value, \"" + value + "\"!" };
                    msg = MessageFormat.format( Messages.xmlDeclAttributeInvalid, args );
                    docBad = new XMLdocException( msg );
                    throw docBad;
                }
                this.setStandAlone( false /* "no" */ );
                break;
        }
    }
    
    private void deleteDocTypeAttribute( String name, String value ) throws XMLdocException
    {
        XMLdocException docBad;
        String msg;
        int i, j;
        
        for ( i = 0; i < docTypeAttributes.length; i++ )
        {
        
            if ( name.equals( docTypeAttributes[i]))
                break;
        }
        if ( i >= docTypeAttributes.length )
        {
            Object[] args = { name, "unknown DocType attribute" };
            msg = MessageFormat.format( Messages.docTypeAttributeInvalid, args );
            docBad = new XMLdocException( msg );
            throw docBad;
        }
        switch( i )
        {
        case 0:  // root element setting
            if ( dtd == null )
            {
                //$FIX$ -- a new DTD needs built.
                root = null;
            }
            else
            {
                if ( !dtd.getName().equals(value) )
                {
                    Object[] args = { name, "unknown DocType attribute" };
                    msg = MessageFormat.format( Messages.docTypeAttributeInvalid, args );
                    docBad = new XMLdocException( msg );
                    throw docBad;
                }
                // don't delete the root element type.
            }
            break;
        case 1:  // SYSTEM url setting
            this.setSystemURL( "" );
            break;
        case 2:  // PUBLIC url setting
            this.setPublicURL( "" );
            break;
        }
    }
    
    private void deleteAttribute( String name, String value ) throws AttributeException, XMLdocException
    {
        AttributeException aBad;
        XMLdocException docBad;
        
        String msg;
        
        switch ( docIndex )
        {
        case XMLdoc.PROLOG:
            deletePrologAttribute( name, value );
            break;
        case XMLdoc.DTD:
            deleteDocTypeAttribute( name, value );
            break;
        case XMLdoc.ROOT:
            deleteElementAttribute( name, value );
            break;
        case XMLdoc.PREMISC:
        case XMLdoc.POSTMISC:
        case XMLdoc.ENDMISC:
            // Comments and PIs don't have attributes
            Object[] args0 = { name + " = " + value };
            msg = MessageFormat.format( Messages.attributeMiscIllegal, args0 );
            aBad = new AttributeException( msg );
            throw aBad;
        default:
            // Document not open.
            Object[] args1 = { "replaceAttribute", name + " = " + value };
            msg = MessageFormat.format( Messages.documentNotOpen, args1 );
            docBad = new XMLdocException( msg );
            throw docBad;
        }
    }
    
    
    //
    //  Close -- closes the document.
    //
    //  Close() -- closes the current part and the document.
    //
    //  Close( partname ) -- partname is one of: Prolog, Element,
    //   or Misc. Closes the partname and the document.
    
    public int close() {
        if ( (docIndex >= 0) && ( docIndex < docParts.length ) )
            return this.close( docIndex );
        docIndex = -1;
        return docIndex;
    }

    //
    // close selected part by name
    //
    public int close( String part ) 
    {
        int i;
        
        for ( i = PARTSTART; i < docParts.length; i++ )
        {
            if ( part.equals(docParts[i]) )
                break;
        }
        if ( i >= docParts.length )
            return -1;
        // "seek" to the part and close it.
        docIndex = i;
        return this.close( docIndex );
    }
    //
    // open to selected part by index
    //
    public int close( int index ) {
        if ( ( index >= 0 ) && ( index < docParts.length ) )
        {
            docIndex = index;
            if ( docIndex == ROOT )
            {
                if ( current != null )
                {
                    current.close();
                }
            }
            isOpen = false;
            docIndex = -1;
            return 0;
        }
        return -1;
    }

    
}
