/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/XMLassembly/XMLexpandable.java,v 1.2 2000/09/25 00:50:36 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/
/*--------------------------------------------------------------------------------+
 |                                                                                |
 |		                        Class XMLexpandable                               |
 |                                                                                |
 +--------------------------------------------------------------------------------*/

package com.adobe.rice.common.XMLassembly;

/**
  XMLexpandable is the standard interface that an object implements when it can
  output itself as an XML entity.  It must implement the following:

 toXMLstring() -- this returns a possibly empty string that represents the object in
     it's XML format.  Markup is represented this way.
      An entity reference may be returned by toXMLstring().

 CanDoCharData() -- this returns true if BOTH toCDSect() and toCharData are implemented.
	Note that only toXMLstring and canDoCharData MUST be implemented.
	If either toCDSect or toCharData is not implemented, then canDoCharData
	MUST return false.  All objects contemplating calling toCDSect and/or
	toCharData MUST call canDoCharData() before calling either and MUST NOT
	call them if canDoCharData() returns false.

 toCDSect() -- this returns the data in a character data section:
	"<![CDATA[" { content string with no "]]>" } "]]>"
	if a '>' must be represented in the string, then a character entity
	reference to it is inserted in it's place. (i.e. '>' may be replaced
	with "&gt;" or a numeric character entity reference for it.)

 toCharData() -- this returns the data as parsed character data:
    { string that doesn't contain '<' or '&' }.  In the case that
	'<' or '&' is needed, then a character entity reference to it may be
	included in the string. (i.e. '<' is in the string as "&lt;" and '&'
    is "&amp;")
                                                                                          |
**/
public interface XMLexpandable
{
    String toXMLstring();        // returns content in XML format
    boolean canDoCharData();     // returns true if implements toCDSect() AND toCharData
    String toCDSect();		 // returns content in CharData Section
    String toCharData();
}









