/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/XMLdisassembly/Content.java,v 1.3 2000/12/02 23:36:53 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.XMLdisassembly;

import java.util.*;
import org.xml.sax.*;
// import com.ibm.xml.parsers.*;

public class Content {
    
    private TagEntry[] tags;                    // the static content model.     
    private XMLDisassembler disassembler;       // the dis-assembler.
    
    public Content() 
    {
        tags = null;
        disassembler = null;
    }
    
    public Content( TagEntry[] theTags, XMLDisassembler xdis )
    {
        this();
        tags = theTags;
        disassembler = xdis;
        // now register the handlers
        this.registerHandlers();
    }
    
    public void setTags( TagEntry[] theTags  )
    {   
        if ( this.tags != null )
        {
            this.clearHandlers();    
        }
        tags = theTags;    
    }
    
    public void setDisassembler( XMLDisassembler xdis )
    {   disassembler = xdis; }
    
    public int index( String tag )
    {
        if ( tags != null )
        {
            for ( int i = 0; i < tags.length; i++ )
                if ( tag.equals(tags[i].name) )
                    return i;
        }
        return -1;
    }
    
    public Handler getHandler( String tagName )
    {
        if ( tags != null )
        {
            for ( int i = 0; i < tags.length; i++ )
                if ( tagName.equals(tags[i].name ) )
                    return tags[i].handler;
        }
        return null;
    }
    
    /**
     * clear out the element handlers -- to regain memory
     *
     **/
    public void clearHandlers()
    {
        // first, unregister handlers
        this.unregisterHandlers();
        // now clear them out.
        if ( tags != null )
        {
            for ( int i = 0; i < tags.length; i++ )
            {
                tags[i] = null;
            }
        }
        tags = null;
    }
    
    /**
     * register the content handlers
     *
     **/
    public void registerHandlers()
    {
        if ( (this.disassembler != null) && ( this.tags != null ) )
        {
            for ( int i = 0; i < this.tags.length ; i++ )
            {
                TagEntry te = tags[i];
                this.disassembler.registerHandler( te.getName(), te.getHandler());
            }
        }
    }
     
    /**
     * un-register the content handlers
     *
     **/
    public void unregisterHandlers()
    {
        if ( ( this.disassembler != null) && ( this.tags != null ) )
        {
            for ( int i = 0; i < this.tags.length; i++ )
            {
                TagEntry te = tags[i];
                this.disassembler.unregisterHandler( te.getName() );
            }
        }
    }
}
