/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/asks/Ask.java,v 1.3 2000/11/27 20:01:46 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.asks;

//
// java imports
//
import java.util.*;
//
// icecubes imports
//
import com.adobe.rice.util.*;
import com.adobe.rice.util.time.*;
import com.adobe.rice.common.asks.*;
import com.adobe.rice.common.XMLassembly.*;
import com.adobe.rice.common.payloadAssembly.*;
import com.adobe.rice.common.storage.logging.*;

/**
 * The Ask abstract class is designed to abstract the core capability to generate
 * an ice-request and monitor it through the process.  Many managers need to
 * generate requests to their counterparts. 
 *
 **/

public abstract class Ask
{
    PayloadDoc  payload;            // the initial payload for the ask.
    Request     request;            // the initial request in the payload
    
    int         retryMax;           // the maximum number of retrys
    
    IceDuration timeout;            // the maximum time to wait before retrying each send.
    
    public Ask()
    {
        this.retryMax = AskConstants.MAX_RETRYS;
        
        
        this.timeout = (IceDuration) AskConstants.fixedDefaultTimeOut;
        
    }
    
    public Ask( String version )
    {
        this();
        
        // why doesn't the following require a try/catch block?
        this.payload = PayloadFactory.instance().get( version );
        
        // now add in a request
        addRequest();
    }
    
    public Ask( String version, String variant )
    {
        this();
        
        this.payload = PayloadFactory.instance().get( version, variant );
        
        // now add in a request
        addRequest();
    }  
    
    protected Request addRequest( )
    {
        request = null;
        this.payload.open();
            try 
            {
                request = new Request( this.payload.getRoot(), this.payload.getDtd() );
                this.payload.add( request );
            }
            catch ( ElementException ee ) {}
            catch ( XMLdocException xde ) {}
        this.payload.close();
        return request;
    }
    
    public void setPayload( PayloadDoc p ) { this.payload = p; }
    public PayloadDoc getPayload() { return this.payload; }
    
    public Request setRequest( Request rq )
    {
        Request old = request;
        request = rq;
        return old;
    }
    public Request getRequest() { return request; }
    
    public void setTimeout( IceDuration to ) { this.timeout = to; }
    public IceDuration getTimeout() { return this.timeout; }
    
    public void setRetryMax( int rt ) { this.retryMax = rt; }
    public int getRetryMax( ) { return this.retryMax; }
    
}

