/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/catalog/BusinessTerm.java,v 1.1.1.1 2000/09/19 19:39:44 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.catalog;

import java.util.*;
import java.io.*;
import java.net.*;
import java.text.*;
//$JDK1.1$ import com.sun.java.util.collections.*;
import com.adobe.rice.common.storage.*;

public class BusinessTerm implements Serializable 
{
    private static final byte MIN_TYPE = 0;
    //
    public static final byte CREDIT = 0;
    public static final byte LICENSING = 1;
    public static final byte PAYMENT = 2;
    public static final byte REPORTING = 3;
    //
    private static final byte MAX_TYPE = REPORTING;
    
    private byte type;
    private URL	url;
    
    private List terms;
    
    public BusinessTerm()
    {
	type = CREDIT;
	url = null;
	terms = null;
    }
	
    public BusinessTerm( String st )
    {
	this();
	setType(st);
	terms = new ArrayList( Constants.EXPECTED_BIZTERM_COUNT );
    }
	
    public BusinessTerm( String st, URL u )
    {
	this();
	setType(st);
	url = u;
	terms = new ArrayList( Constants.EXPECTED_BIZTERM_COUNT );
    }
	
	
    public void setType( byte t )
    {
	if ( MIN_TYPE <= t  && t <= MAX_TYPE )
	{
	    type = t;
	}
	else
	{
	    //$FIX$ -- throw an offer exception?
	}
    }
	
    private static final String[] businessTermTypes = 
    { "credit", "licensing", "payment", "reporting" };
	
    public void setType( String st )
    {
	for ( byte i = 0; i < businessTermTypes.length ; i++ )
	{
	    if ( businessTermTypes[i].equals( st ) )
	    {
		type = i;
		return;
	    }
	}
	// $FIX$ throw offer exception here
    }
	
    public byte getType( ) { return type; }
	
    public void setURL( URL u ) { url = u; }
    public URL getURL() { return url; }
	
    public void setTerms( List t ) { terms = t; }
    public List getTerms() { return terms; }
	
    public String toXMLstring()
    {
	String startTagTemplate = "<ice-business-term type = \"{1,String}\" " +
	    "url = \"{2,String}\" " +
	    ">\n" ;
	String endTag = "</ice-business-term>\n";
	StringBuffer content = new StringBuffer( Constants.EXPECTED_BIZTERM_COUNT* Constants.EXPECTED_BIZTERM_LENGTH );
		
	Object[] args = { (type == CREDIT)? "credit" : 
			  ( (type == LICENSING)? "licensing" :
			    ( ( (type == PAYMENT) ? "payment" :
				( ( ( (type == REPORTING) ? "reporting" : "unknown" )))))) ,
			  url.toString() };
						  
	String startTag = MessageFormat.format( startTagTemplate, args );
				
	if ( (terms != null) && (terms.size() > 0) )
	{
	    for ( int i = 0; i < terms.size(); i++ )
	    {
		Object o = terms.get(i);
				
		if ( o instanceof IceText )
		{
		    IceText it = (IceText) o ;
		    content.append( it.toXMLstring() );
		}
		else
		{
		    // $FIX$ throw an invalid offer exception
		}
	    }
	}

	return startTag + content.toString() + endTag;
    } 

}
