/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/catalog/DeliveryPolicy.java,v 1.1.1.1 2000/09/19 19:39:44 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org


initial 1999.08.10 Will Conant - original code.
updated 1999.08.22 Bruce Hunt - added toXMLstring() method 

*/

package com.adobe.rice.common.catalog;

//$JDK1.1$ import com.sun.java.util.collections.*;
import java.util.*;
import com.adobe.rice.util.time.*;
import com.adobe.rice.util.*;

public class DeliveryPolicy { 
    private List deliveryRules;
    private IceDateTime startdate;
    private IceDateTime stopdate;
    private UUID uuid;
	
    /** A DeliveryPolicy is a wrapper for many DeliveryRules.
     * 
     * @param uuid Unique identifier of IceParty. 
     */
    public DeliveryPolicy(UUID uuid) { 
	deliveryRules = new LinkedList();
	//start now
	startdate = new IceDateTime();
	//never stop. (time part is arbitrary.)
	stopdate = new IceDateTime(IceDate.getDateInfinite(),new IceTime());
	this.uuid=uuid;
    }
    
    public void setStartdate(IceDateTime startdate){
	this.startdate=startdate;
    }

    public IceDateTime getStartdate() {
	return startdate;
    }
	
    public void setStopdate(IceDateTime stopdate){
	this.startdate=startdate;
    }
	
    public IceDateTime getStopdate() {
	return stopdate;
    }
	
    /** addDeliveryRule cleans up a DeliveryRule and adds it to the DeliveryPolicy.
     * 
     * @param deliveryRule DeliveryRule to be added to the DeliveryPolicy
     */
    public void addDeliveryRule(DeliveryRule deliveryRule) {
	// The startdate-stopdate window intersection between DRule 
	// and DPolicy should be one
	deliveryRule.overrideDates(getStartdate(),getStopdate());
	deliveryRule.setUUID(uuid);
	deliveryRules.add(deliveryRule);
    }

    public String toXMLstring() throws CatalogException
    {
	String startTag = "<ice-delivery-policy " +
	    "startdate = \"{1,String}\" " +
	    "stopdate = \"{2,String}\" " +
	    // UUID is not in the DTD.
	    ">\n";
	String endTag = "</ice-delivery-policy>\n";
	StringBuffer content = new StringBuffer( deliveryRules.size() 
						 * Constants.EXPECTED_DELIVERY_RULE_LENGTH );
	if ( deliveryRules.size() > 0 )
	{
	    for ( int i = 0; i < deliveryRules.size(); i++ )
	    {
		DeliveryRule dr = (DeliveryRule) deliveryRules.get(i);
		try { content.append( dr.toXMLstring() ); }
		catch ( CatalogException ce )
		{
		    throw ce;
		}
	    }
	}			
	return startTag + content.toString() + endTag;
    }
	
    /** overrideDates will AND the start/stopdate of this with the arguments.
     *  Subscriptions should use this to clean up DeliveryPolicy children.
     * @param parentStartDate this IceDateTime will be ANDed with this StartDate
     * @param parentStopDate this IceDateTime will be ANDed with this StopDate
     */
    public void overrideDates(IceDateTime parentStartdate, IceDateTime parentStopdate) {
    // cleanup this DP
	if (getStartdate().before(parentStartdate))
	    setStartdate(parentStartdate);
	if (getStopdate().after(parentStopdate))
	    setStopdate(parentStopdate);
		
	// cleanup all DR
	Iterator it = deliveryRules.iterator();
	while (it.hasNext()) {
	    DeliveryRule dr = (DeliveryRule) it.next();
	    dr.overrideDates(parentStartdate,parentStopdate);
	}
    }
}













