/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/catalog/DeliveryRule.java,v 1.1.1.1 2000/09/19 19:39:44 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.catalog;

import java.util.*;
import java.text.*;
//$JDK1.1$ import com.sun.java.util.collections.*;
import com.adobe.rice.common.scheduling.*;
import com.adobe.rice.common.policies.*;
import java.io.Serializable;
import com.adobe.rice.util.time.*;
import com.adobe.rice.util.*;
/** DeliveryRule objects contain all infomration within an ICE Delivery Rule.  */

public class DeliveryRule implements Schedulable, Serializable {
    private byte mode;
    private String subscriptionId;
    private IceDateTime startdate;
    private IceDateTime stopdate;
    private IceDayGroup days; // IceDateGroup is comprised of Monthday and Weekday
    private IceTime starttime;
    private IceDuration duration;
    private IceDuration minfreq;
    private IceDuration maxfreq;
    private int mincount;
    private int maxcount;
    private String url;
    private UUID uuid;

    private List negotiables;

/**   Mode provides a namespace for DeliveryRule.   */
    public static class Mode {
	public static final byte MINVAL = 0;
	public static final byte PUSH = 1;
	public static final byte PULL = 2;
	public static final byte MAXVAL = 3;
    }

    public static class PolicyConstants {
	public static final byte MINVAL = 0;
	public static final byte MINDELIVERY = 1;
	public static final byte MAXDELIVERY = 2;
	public static final byte MAXVAL = 3;
    }

	
    /** Default constructor for DeliveryRule sets mode as Mode.PULL.
     * 
     */
    public DeliveryRule() {
	construct();
    }
	
    public DeliveryRule(byte mode) throws IllegalArgumentException {
	construct();
	setMode(mode);
    }
    // TODO: go back and javadoc the constructors with explinations of these as default values.
    private void construct() {
	mode=Mode.PULL;
	days=new IceDayGroup("ANY","ANY");
	startdate=new IceDateTime();
	stopdate=new IceDateTime(IceDate.getDateInfinite(), new IceTime());
	starttime=new IceTime("00:00:00"); // by default the SelectedTimeWindow is a full day
	duration=new IceDuration("P86400S"); // by default the SelectedTimeWindow is a full day
	
	constructPolicies();
    }

    private void constructPolicies() {
	// set up all relevent policies
	// note that we DO NOT want to observe, as DR isn't a singleton.
	RadioButtonPolicy freq = new RadioButtonPolicy(Byte.class);
	freq.addChoice(new Byte(PolicyConstants.MINDELIVERY));
	freq.addChoice(new Byte(PolicyConstants.MAXDELIVERY));
	try {
	    freq.select(new Byte(PolicyConstants.MINDELIVERY));
	}
	catch (PolicyException e) {}
	PolicyManager.getPolicyManager().setPolicy("DeliveryRule-simplefreq",freq);
    }

    public void setMode(byte mode) {
	if (mode<Mode.MAXVAL && mode>Mode.MINVAL)
	    this.mode=mode;
	else 
	    throw new IllegalArgumentException("Illegal mode set");
    }	
    
    public byte getMode(){
	return mode;
    }

    public void setSubscriptionId(String subcriptionId){
	this.subscriptionId=subcriptionId;
    }
	
    public String getSubscriptionId() {
	return subscriptionId;
    }
	
    void setUUID(UUID uuid){
	this.uuid=uuid;
    }
	
    UUID getUUID() {
	return uuid;
    }
	
	
    public void setStartdate(IceDateTime startdate){
	this.startdate=startdate;
    }
	
    public IceDateTime getStartdate() {
	return startdate;
    }
	
    public void setStopdate(IceDateTime stopdate){
	this.startdate=startdate;
    }

    public IceDateTime getStopdate() {
	return stopdate;
    }
	
	
    /** overrideDates will AND the start/stopdate of this with the arguments.
     * DeliveryPolicys should use this to clean up DeliveryRule children.
     * @param parentStartDate this IceDateTime will be ANDed with this StartDate
     * @param parentStopDate this IceDateTime will be ANDed with this StopDate
     */
    public void overrideDates(IceDateTime parentStartdate, IceDateTime parentStopdate) {
	if (getStartdate().before(parentStartdate))
	    setStartdate(parentStartdate);
	if (getStopdate().after(parentStopdate))
	    setStopdate(parentStopdate);
    }	
	
	
    public void setDays(IceDayGroup days){
	this.days=days;
    }
    public IceDayGroup getDays() {
	return days;
    }
	
    public void setStarttime(IceTime starttime){
	this.starttime=starttime;
    }
    public IceTime getStarttime() {
	return starttime;
    }
    
    public void setDuration(IceDurationDay duration) {
	// using an IceDurationDay does the nice check to ensure that 
	// we only have a duration
	// that is <= day as the spec requires!
	this.duration=(IceDuration)duration;
    }
    public IceDuration getDuration() {
	return duration;
    }
	
    public void setMinfreq(IceDuration minfreq){
	this.minfreq=minfreq;
    }
    public void setMaxfreq(IceDuration maxfreq){
	this.maxfreq=maxfreq;
    }
    public IceDuration getMinfreq()
	throws OptionalArgumentNotSpecifiedException {
	return getFreq(false);
    }
    public IceDuration getMaxfreq()
	throws OptionalArgumentNotSpecifiedException {
	return getFreq(true);
    }
	
    private IceDuration getFreq(boolean isMaxfreq)
	throws OptionalArgumentNotSpecifiedException {
	IceDuration thisfreq;
	if (isMaxfreq) 
	    thisfreq=maxfreq;
	else
	    thisfreq=minfreq;
	
	if (thisfreq!=null)
	    return thisfreq;
	else 
	    throw new OptionalArgumentNotSpecifiedException();	
    }	
	
    public void setMincount(int mincount) 
	throws IllegalArgumentException{
	if (mincount==0)
	    throw new IllegalArgumentException();
	this.mincount=mincount;
    }
    public void setMaxcount(int maxcount) 
	throws IllegalArgumentException{
	if (mincount==0)
	    throw new IllegalArgumentException();
	this.maxcount=maxcount;
    }
    public int getMincount()
	throws OptionalArgumentNotSpecifiedException {
	return getCount(false);
    }
    public int getMaxcount()
	throws OptionalArgumentNotSpecifiedException {
	return getCount(true);
    }
	
    private int getCount(boolean isMaxcount)
	throws OptionalArgumentNotSpecifiedException {
	int thiscount;
	if (isMaxcount) 
	    thiscount=maxcount;
	else
	    thiscount=mincount;
	
	if (thiscount!=0)
	    return thiscount;
	else 
	    throw new OptionalArgumentNotSpecifiedException();	
    }
	
    public void setUrl(String url) {
	this.url=url;
    }
    public String getUrl()
	throws OptionalArgumentNotSpecifiedException {
	if (url!=null)
	    return url;
	else 
	    throw new OptionalArgumentNotSpecifiedException();	
    }
	
	
    public IceDuration getScheduledDuration() {
	return getDuration();
    }
    public IceDateTime getScheduledStartdate(){
	return getStartdate();
    }
    public IceDateTime getScheduledStopdate(){
	return getStartdate();
    }
    public IceTime getScheduledStarttime(){
	return getStarttime();
    }
    public String getScheduledId() {
	return new String("DeliveryPolicy" +
			  "." + getSubscriptionId() +
			  "." + getUUID().toString());	
    }
	
	
    public ListIterator getSchedulables(IceDate scheduledDate){
	Byte freq = (Byte)
	    PolicyManager.getPolicyManager().getSelection("DeliveryRule-simplefreq");
	boolean isMaxSimple = (freq.byteValue()==
			       PolicyConstants.MAXDELIVERY) ? true : false;
		
	return simpleSchedulables(scheduledDate, isMaxSimple);
    }
	
    private ListIterator simpleSchedulables(IceDate scheduledDate, boolean isMaxSimple) {
	IceDateTime windowStart, windowStop;
	IceDuration windowDuration;
	// set up the SelectedTimeWindow for "normal"
	windowStart=new IceDateTime(scheduledDate,starttime);
	windowStop=windowStart.add(duration);
	
	// boundary case where we "truncate" beginning of selectedtimewindow
	if (startdate.getDate().equals(scheduledDate))
	    windowStart = startdate;
	// boundary case where we "truncate" ending of selectedtimewindow
	if (stopdate.getDate().equals(windowStop.getDate()))
	    windowStop = stopdate;
	windowDuration = windowStart.subtract(windowStop);
		
	/* "If there is a conflict between the frequency and count parameters,
	   frequency takes precedence." will be interpreted as follows:
	   -Given neither freq nor count, do not partition SelectedTimeWindow.
	   -Given only a count=L, create L partitions of size (SelTimeWind/L).
	   -Given count N and freq M, create (SelTimeWind/M) partitions of size M.
	*/
		
	int numpartitions;
	IceDuration partitionsize;
	// see if freq specified
	try {
	    partitionsize = getFreq(isMaxSimple);
	    numpartitions = (int) windowDuration.divide(partitionsize);
	}
	catch (OptionalArgumentNotSpecifiedException e) {
	    // else see if count specified
	    try {
		numpartitions=getCount(isMaxSimple);
		partitionsize=windowDuration.divide(numpartitions);
	    }
	    // else do not partition STW
	    catch (OptionalArgumentNotSpecifiedException e2) {
		numpartitions=1;
		partitionsize=windowDuration;
	    }
	}
		
	List SelectedTimeWindow = new LinkedList();
		
	int i=0;
	IceDateTime eventstart=windowStart;
	
	while (i<numpartitions) {
	    // TODO. What exactly is a runner? How do I give it the info I need?
	    Runner runner = null;
	    DispatchScheduleItem item;
	    item= new DispatchScheduleItem(runner, eventstart, partitionsize);
	    SelectedTimeWindow.add(item);
			
	    i++;
	    eventstart=windowStart.add(partitionsize);
	}
	return (ListIterator) SelectedTimeWindow.iterator();
    }
	
    public String getModeString()
    {
	switch( mode )
	    {
	    case Mode.PUSH:
		return "push";
	    case Mode.PULL:
		return "pull";
	    }
	return "UNKNOWN";
    }

    public String toXMLstring() throws CatalogException
    {
	String startTagTemplate = "<ice-delivery-rule " + 
	    "duration = \"{1,String}\" " +
	    "maxcount = \"{2,String}\" " +
	    "maxfreq = \"{3,String}\" " +
	    "mincount = \"{4,String}\" " +
	    "mode = \"{5,String}\" " +
	    "monthday = \"{6,String}\" " +
	    "startdate = \"{7,String}\" " +
	    "starttime = \"{8,String}\" " +
	    "stopdate = \"{9,String}\" " +
	    "url = \"{10,String}\" " +
	    "weekday = \"{11,boolean}\" " +
	    ">\n" ;
	String endTag = "</ice-delivery-rule>\n";
	StringBuffer content = new StringBuffer( Constants.EXPECTED_NEGOTIABLE_COUNT * 
						 Constants.EXPECTED_NEGOTIABLE_LENGTH );
	Integer maxcnt = new Integer( maxcount );
	Integer mincnt = new Integer( mincount );
		
	Object[] args = { duration.toString(), maxcnt.toString(), maxfreq.toString(), 
			  mincnt.toString(),
			  getModeString(), "$FIXME$", startdate.toString(), starttime.toString(), 
			  stopdate.toString(), url.toString(), "$FIXME$" } ;
						  
	String startTag = MessageFormat.format( startTagTemplate, args );
				
	if ( negotiables.size() > 0 )
	{
	    for ( int i = 0; i < negotiables.size(); i++ )
	    {
		Object o = negotiables.get(i);
		    
		if ( o instanceof Negotiable )
		{
		    Negotiable n = (Negotiable) o ;
		    content.append( n.toXMLstring() );
		}
		else
		{
		    Object[] eargs = { subscriptionId };
		    String msg = MessageFormat.format( Messages.DeliveryRule_IllegalObject, eargs );
		    CatalogException cBad = new CatalogException( msg );
		    throw cBad; 
		}
	    }
	}

	return startTag + content.toString() + endTag;
    }

}





