/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/catalog/Negotiable.java,v 1.1.1.1 2000/09/19 19:39:44 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.catalog;

import java.util.*;
import java.io.*;
import java.net.*;
import java.text.*;
//$JDK1.1$ import com.sun.java.util.collections.*;
import com.adobe.rice.common.storage.*;

public class Negotiable implements Serializable 
{
    private static final byte MIN_TYPE = 0;
    //
    public static final byte UNDEFINED = -1;
    public static final byte DURATION  = 0;
    public static final byte MAXCOUNT  = 1;
    public static final byte MAXFREQ   = 2;
    public static final byte MINCOUNT  = 3;
    public static final byte MINFREQ   = 4;
    public static final byte MONTHDAY  = 5;
    public static final byte STARTDATE = 6;
    public static final byte STARTTIME = 7;
    public static final byte STOPDATE  = 8;
    public static final byte WEEKDAY   = 9;
	
    //
    private static final byte MAX_TYPE = WEEKDAY;
	
    private byte type;
    private String max;
    private String min;
		
    public Negotiable()
    {
	type = UNDEFINED;
	max = "";
	min = "";
    }
	
    public Negotiable( String nt )
    {
	this();
	setType(nt);
    }
	
    public Negotiable( String nt, String mx, String mn )
    {
	this();
	setType(nt);
	max = mx;
	min = mn;
    }
	
    public void setType( byte t )
    {
	if ( MIN_TYPE <= t  && t <= MAX_TYPE )
	{
	    type = t;
	}
	else
	{
	    type = UNDEFINED;
	}
    }
	
    private static final String[] negotiableTypes = 
    { "duration", "maxcount", "maxfreq", "mincount", "minfreq", "monthday",
      "startdate", "starttime", "stopdate", "weekday" };
	
    public void setType( String st )
    {
	for ( byte i = 0; i < negotiableTypes.length ; i++ )
	{
	    if ( negotiableTypes[i].equals( st ) )
	    {
		type = i;
		return;
	    }
	}
	// $FIX$ throw offer exception here
    }
	
    public byte getType( ) { return type; }
	
    public String getTypeString( ) 
    {
	if ( (MIN_TYPE <= type) && (type <= MAX_TYPE) )
	    return negotiableTypes[ type ];
	return "UNKNOWN";
    }
	
    public String getMax() { return max; }
    public void setMax( String mx ) { max = mx; }
	
    public String getMin() { return min; }
    public void setMin( String mn ) { min = mn; }
		
    public String toXMLstring()
    {
	String startTagTemplate = "<ice-negotiable " + 
	    "max = \"{1,String}\" " +
	    "min = \"{2,String}\" " +
	    "type = \"{3,String}\" " +
	    "/>\n" ;		
	Object[] args = { max, min, (String) getTypeString() };
						  
	String startTag = MessageFormat.format( startTagTemplate, args );
	
	return startTag;
    } 
}
