/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/catalog/Offer.java,v 1.1.1.1 2000/09/19 19:39:44 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.catalog;

import java.util.*;
import java.io.*;
import java.net.*;
import java.text.*;
//$JDK1.1$ import com.sun.java.util.collections.*;
import com.adobe.rice.common.storage.*;

public class Offer implements Serializable 
{
    private boolean atomicUse;
    private String constraintsHash;
    private String constraintsHashMethod;
    private URL constraintsURL;
    private String description;
    private boolean editable;
    private String ipStatus;
    private String rightsHolder;
    private boolean showCredit;
    private String subscriptionID;
    private boolean usageRequired;
	
    private DeliveryPolicy delivery;		// ice-delivery-policy
    private List terms;				// ice-business-terms
	
    public Offer()
    {
	atomicUse = false;
	constraintsHash = "";
	constraintsHashMethod = "";
	constraintsURL = null;
	description = "";
	editable = false;
	ipStatus = "";
	rightsHolder = "";
	showCredit = false;
	subscriptionID = "";
	usageRequired = false;
	
	delivery = null;
	terms = new ArrayList( Constants.EXPECTED_BIZTERM_COUNT );
    }
	

    public String toXMLstring() throws OfferException
    {
	String startTagTemplate = "<ice-offer atomic-use = \"{1,boolean}\" " +
	    "constraints-hash = \"{2,String}\" " +
	    "constraints-hash-method = \"{3,String}\" " +
	    "constraints-url = \"{4,String}\" " +
	    "description = \"{5,String}\" " +
	    "editable = \"{6,boolean}\" " +
	    "ip-status = \"{7,String}\" " +
	    "rights-holder = \"{8,String}\" " +
	    "show-credit = \"{9,boolean}\" " +
	    "subscription-id = \"{10,String}\" " +
	    "usage-required = \"{11,boolean}\" " +
	    ">\n" ;
	String endTag = "</ice-offer>\n";
	StringBuffer content = new StringBuffer( (Constants.EXPECTED_BIZTERM_COUNT + 1)* Constants.EXPECTED_BIZTERM_LENGTH );
		
	Object[] args = { new Boolean(atomicUse), constraintsHash, constraintsHashMethod, 
			  constraintsURL.toString(), description, new Boolean(editable), 
			  ipStatus, rightsHolder, new Boolean(showCredit), subscriptionID,
			  new Boolean(usageRequired) };
						  
	String startTag = MessageFormat.format( startTagTemplate, args );
		
	String XMLdeliveryPolicy;
	try {  XMLdeliveryPolicy = delivery.toXMLstring(); }
	catch ( CatalogException ce )
	{ 	OfferException ob = new OfferException( ce.getMessage() ); throw ob; }
	
	if ( terms.size() > 0 )
	{
	    for ( int i = 0; i < terms.size(); i++ )
	    {
		Object o = terms.get(i);
		
		if ( o instanceof BusinessTerm )
		{
		    BusinessTerm b = (BusinessTerm) o ;
		    content.append( b.toXMLstring() );
		}
		else
		{
		    Object[] eargs = { description };
		    String msg = MessageFormat.format( Messages.Offer_IllegalObject, eargs );
		    OfferException oBad = new OfferException( msg );
		    throw oBad; 
		}
	    }
	}
	return startTag + XMLdeliveryPolicy + content.toString() + endTag;
    }
}













