/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/collections/Collection.java,v 1.1.1.1 2000/09/19 19:39:44 bhunt Exp $

<<ICECOPYRIGHT>>
*/

/*----------------------------------------------------------------------------------------+
 |                                                                                        |
 |                              Reference Implementation of ICE                           |
 |                                                                                        |
 |                         Collections -- Collection Class Definition                     |
 |                                                                                        |
 +----------------------------------------------------------------------------------------*/ 

package com.adobe.rice.common.collections;

import com.adobe.rice.common.*;
import com.adobe.rice.util.ExceptionMessages;

import java.util.*;
//$JDK1.1$ import com.sun.java.util.collections.*;

public class Collection 
{
    private static int presentInChecklist = 3;        // value for checklist
    private static int vacantInChecklist = 1;         // value for checklist
    private static int notPresentInChecklist = 0;     // value for checklist
    
    private String name;                    // name of the collection
    private int    id;                      // the number of this collection.
    private Model model;                    // 
    private ArrayList issues;               // list containing the status of elements within issues 
    private int currentIssue;               // the point in issue that holds id of the most recently completed issue of the collection
    private ArrayList issueElementArray;    // list containing the issue elements of the collection    
    private int issueElementMark;           // marks the place of the last issue element added to issueElementArray
    private LinkedList issueElementEmpties; // list linking all the spaces in issueElementArray that were left when issue elements were deleted - from bottom to top of the list
    private BitVector checklist;            // array to keep track of which elements will be included in the next issue
    private String currentState;            // in the subscriber, the state is opaque - a string 
    private Hashtable stateTable;         // maps the state of an issue to that issue's index in issues

    public Collection()
    {
        name = "";
        id = -1;
        model = new Model();
        issues = new ArrayList();
        issueElementArray = new ArrayList(); 
        issueElementMark = -1; 
        issueElementEmpties = new LinkedList();
        checklist = new BitVector();
        currentIssue = -1;
        currentState = CollectionManager.initialState;
        stateTable = new Hashtable();
    }
    
    public Collection( String collectionName )
    {
        name = collectionName;
        id = -1;
        model = new Model();
        issues = new ArrayList();
        issueElementArray = new ArrayList(); 
        issueElementMark = -1; 
        issueElementEmpties = new LinkedList();
        checklist = new BitVector();
        currentIssue = -1;
        currentState = CollectionManager.initialState;
        stateTable = new Hashtable();
    }
    
    public String getName() { return name; }
    public void setName( String newName ) { name = newName; }
    public int getId() { return id; }
    public void setId( int newId ) { if (newId >= -1) id = newId; }
    public int getCurrentIssue() { return currentIssue; }
    public void setCurrentIssue( int newCurrentIssue ) { if (newCurrentIssue >= 0)currentIssue = newCurrentIssue; }
    public String getCurrentState() { return currentState; }
    public void setCurrentState( String newCurrentState ) { currentState = newCurrentState; }
    public int getIssueElementMark() { return issueElementMark; }
    public void setIssueElementMark( int newIssueElementMark ) { issueElementMark = newIssueElementMark; }
    public Model getModel() {return model;}
    public Issue getIssue( int issueId ) { return (Issue)issues.get( issueId ); } 
    public IssueElement getIssueElement( int index ) { return (IssueElement) issueElementArray.get( index ); }
    
    /** 
    * Method called to add an issue to the end of issues.  The issue's id number will be changed to 
    * reflect that placement and a unique state will be assigned to the issue.
    *
    * @param    issueToAdd   The issue which will be added.
    */
    public void setIssue ( Issue issueToAdd ) 
    {        
        if (issueToAdd == null)
            return;  //$LOG$ the error - there are no elements in the issue
        PackageSequenceId packageState = new PackageSequenceId();
        String state = packageState.getPackageSequenceId();
        issueToAdd.setState(state);
        issues.add(issueToAdd);
        int index = issues.indexOf(issueToAdd);
        issueToAdd.setId( index ); 
        this.setStateTable(state, index); 
    }
   
    /** 
    * Method called to put an issue element into issueElementArray.  If the issue element 
    * has not appeared before, it will be put at the end of the issueElementArray and its id number will 
    * be changed to reflect that placement.
    *
    * @param    issueElementToAdd   The issue element which will be added.
    */
    public void setIssueElement( IssueElement issueElementToAdd )
    {
        int count = 0;
        
        while( count <= issueElementMark) // checking for elements already in issueElementArray
        {
            IssueElement testIssueElement = (IssueElement) issueElementArray.get( count );
            String testSERef = (String)testIssueElement.getSubscriptionElementReference();
            int testHash = testIssueElement.getHash();
            if ( ( testHash == issueElementToAdd.getHash() ) && ( testSERef.equals( (String)issueElementToAdd.getSubscriptionElementReference() ) ) )
                {
                    this.setCheckItemPresent( count );
                    return; // the element is already in issueElementArray - no use putting it in again
                }
            count++;
        }
        if ( issueElementEmpties.isEmpty() == false ) 
        { 
            // if there are empty spots, fill them and take them off of the list of empties
            IssueElement deletedElement = (IssueElement)issueElementEmpties.getFirst();
            int deletedElementIndex = deletedElement.getId();
            issueElementArray.add( deletedElementIndex, issueElementToAdd );
            issueElementToAdd.setId( deletedElementIndex );
            issueElementEmpties.removeFirst();
            setCheckItemPresent( deletedElementIndex );
            return;
        }   
        else 
        {
            // more of the array is being used now
            issueElementMark++;  
            // add the new issue element to the end of the array
            issueElementArray.add( issueElementMark, issueElementToAdd );
            issueElementToAdd.setId( issueElementMark );
            this.setCheckItemPresent( issueElementMark );
            return;
        }
    }
    
    /**
    * Method called to add an issue element into a specific place in issueElementArray.  
    * This specific place is indicated by the issue element's id. 
    *
    * @param issueElementToAdd  The issue element which will be added to the place in 
    *                           issueElementArray which is marked by its id.
    */
    public void addIssueElement( IssueElement issueElementToAdd )
    {
        int index = issueElementToAdd.getId();
        if ( index < 0)
            return;
        if ( issueElementMark<index )
            this.setIssueElementMark( index );
        while (index > issueElementArray.size())
        {
            // add a blank element at the end of the array until the array is big enough
            IssueElement blankIssueElement = new IssueElement();
            issueElementArray.add(blankIssueElement);
        }
        issueElementArray.add( index, issueElementToAdd );
        this.setCheckItemPresent( index );
        return;
    }
    
    /** 
    * Method called to completely delete an issue element from issueElementArray by using the element's 
    * index in the array.  That index will then be added to the linked list of issueElementEmpties.
    *
    * ** NOTE ** this method should ONLY be used when the issue element at this index has been deleted
    * or absent for the entire archived duration of the collection.
    *
    * @param    indexToVacant   The index of the issue element which will be deleted.
    */
    public void vacantIssueElement( int indexToVacant )
    // set the stuff in the array to a ptr to some other empty, set the ptr of some empty to this
    {
        if ( (indexToVacant < 0) || (indexToVacant >= issueElementArray.size()) )
            return; // invalid index
        IssueElement issueElementToVacant = (IssueElement) issueElementArray.get(indexToVacant); 
            
        // if has SE refs, make sure they're unlinked
        // 
        // if (issueElementToVacant.getSubscriptionElementReference() != null)
        issueElementEmpties.addFirst( issueElementArray.get(indexToVacant) ); // adds at the beginning of the linked list
        checklist.setItem( indexToVacant, vacantInChecklist);
        issueElementToVacant = null;
    }
    
    /**
    * Method called to put a state and the index of the corresponding issue into stateTable.
    *
    * @param    state       The earliest state that a subscriber can be in and still have the whole 
    *                       corresponding issue.
    * @param    issueIndex  The index of the corresponding issue in issues.
    */
    public void setStateTable ( String state, int issueIndex )
    {
        if (state == null)
            return; //$LOG$ the error - something Really bad happened
        if ( (issueIndex >= 0) && (issueIndex < issues.size()) )
        {
            Integer index = new Integer( issueIndex );
            stateTable.put(state, index);
        }
        else
        {
            // either the issue doesn't exist or its id was set improperly
        }
    }
    
    /**
    * Method called to get the index of an issue from stateTable using its state.
    *
    * @param    state       The earliest state that a subscriber can be in and still have the whole 
    *                       corresponding issue.
    */
    public int getStateTable ( String state )
    {
        if (state == null)
            return -1; //$LOG$ the error - state should not be null
        Integer index = (Integer) stateTable.get(state);
        if (index == null)
        {
            // $LOG$ the error - state was not in the table
            return -1;
        }
        return index.intValue();
    }
    
    /**
    * Method called to return the status of an item in checklist.
    *
    * @param   key  Integer corresponding to the place in the array of
    *               the item whose status the method is searching for.   
    */
    public int getCheckItemStatus( int key )
    {
        if ( key != -1 )
            return checklist.getItem( key );
        else
            return -1;
    }
        
    /**
    * Method called to set the status of an item in checklist to notPresentInChecklist. 
    *
    * @param    index   The offset of the item to set.
    */
    public void setCheckItemNotPresent( int index)
    {
        if ( index > -1 )
            checklist.setItem( index, notPresentInChecklist );
        else
            return;// else do nothing
    }
    
    /**
    * Method called to set the status of an item in checklist to presentInChecklist. 
    *
    * @param    index   The offset of the item to set.
    */
    public void setCheckItemPresent( int index)
    {
        if ( index > -1 )
            checklist.setItem( index, presentInChecklist );
        else
            return;// else do nothing
    }
    
    /**
    * Method called to put the finishing touches on an issue. 
    *
    * @param    issueToFinish   The issue which has been completed.
    */
    public void issueDone( Issue issueToFinish )
    {
        for (int count = 0; count <= issueElementMark; count++)
        {
            if (this.getCheckItemStatus(count) == presentInChecklist)
            {
                // if the element is present in checklist, it must be marked active in the Issue
                issueToFinish.setItemActive(count);
                this.setCheckItemNotPresent(count); // checklist must be cleared for its next use
            }
            else if (this.getCheckItemStatus(count) == vacantInChecklist)
            {
                // the element has been deleted from the collection and its spot has been opened up
                issueToFinish.setItemVacant(count);
                this.setCheckItemNotPresent(count); // checklist must be cleared for its next use
            }
            else if (currentIssue >= 0 )
                // if the element is not present in checklist, must find if it was vacant, present,
                // deleted, or absent in the previous issue and fill the new issue in accordingly
                // it is not necessary to make any changes to checklist because it is already set
                // to notPresentInChecklist
            {
                Issue lastIssue = getIssue(currentIssue);
                if (lastIssue.getElementStatus(count)== Issue.ACTIVE )
                    issueToFinish.setItemDeleted(count);
                else if (lastIssue.getElementStatus(count)== Issue.DELETED )
                    issueToFinish.setItemAbsent(count);
                else if (lastIssue.getElementStatus(count)== Issue.ABSENT )
                    issueToFinish.setItemAbsent(count);
                else if (lastIssue.getElementStatus(count)== Issue.VACANT )
                    issueToFinish.setItemVacant(count);
            }
        } // end for
        this.setIssue( issueToFinish);        
  
        issueToFinish.finishIssue( issueElementMark, issueToFinish.getState() );

        this.currentIssue = issueToFinish.getId();
        this.currentState = issueToFinish.getState();
    }
 
    /**
    * Method called to put the finishing touches on a newly updated issue. 
    * 
    * @param    issueToFinish   The issue which has been completed.
    */
    public void issueUpdated( Issue issueToFinish )
    {
        for (int count = 0; count <= issueElementMark; count++)
        {
            if (this.getCheckItemStatus(count) == presentInChecklist)
            {
                // if the element is present in checklist, it must be marked active in the Issue
                issueToFinish.setItemActive(count);
                this.setCheckItemNotPresent(count); // checklist must be cleared for its next use
            }
            else if (this.getCheckItemStatus(count) == vacantInChecklist)
            {
                // the element has been deleted from the collection and its spot has been opened up
                issueToFinish.setItemVacant(count);
                this.setCheckItemNotPresent(count); // checklist must be cleared for its next use
            }
            else if (currentIssue >= 0 )
                // if the element is not present in checklist, must find if it was vacant, present,
                // deleted, or absent in the previous issue and fill the new issue in accordingly
                // it is not necessary to make any changes to checklist because it is already set
                // to notPresentInChecklist
            {
                Issue lastIssue = getIssue(currentIssue);
                if (lastIssue.getElementStatus(count)== Issue.ACTIVE )
                    issueToFinish.setItemActive(count);
                else if (lastIssue.getElementStatus(count)== Issue.DELETED )
                    issueToFinish.setItemAbsent(count);
                else if (lastIssue.getElementStatus(count)== Issue.ABSENT )
                    issueToFinish.setItemAbsent(count);
                else if (lastIssue.getElementStatus(count)== Issue.VACANT )
                    issueToFinish.setItemVacant(count);
            }
        } // end for
 
        issueToFinish.finishIssue( issueElementMark, issueToFinish.getState() );

        this.currentIssue = issueToFinish.getId();
        this.currentState = issueToFinish.getState();
    }
    
    /**
    * Method called to move the subscriber straight from state ICE-INITIAL or ICE-ANY to the target state
    * NOTE: This method relies on the fact that each issue element is always indexed by an integer
    *  which is unique to that issue element within the period of the collection (also its id).
    *
    * @param currentState   The current state of the subscriber.
    * @param targetState    The target state of the subscriber.
    * @param pack           The package which will contain the update.
    */
    public void generateInitialList(String currentState, Issue targetState, Package pack)
    {
        int vacant = Issue.VACANT;   
        int active = Issue.ACTIVE;   
        int deleted = Issue.DELETED;   
        int absent = Issue.ABSENT; 
        if (targetState.getFinished() == false)
            return; // the issue isnt ready yet
        pack.setPreviousState(currentState);
        PackageElement startIssue = new PackageElement();
        startIssue.newIssue( targetState ); // a new issue is being sent
        pack.setPackageElement( startIssue );

        for( int count = 0; count <= targetState.getElementCount(); count++ )
        {
            int newStateStatus = targetState.getElementStatus( count );
            if ( newStateStatus == active )
            {
                PackageElement packageElement = new PackageElement();
                packageElement.packageElementAdd( (IssueElement)this.issueElementArray.get(count) );  
                pack.setPackageElement( packageElement );
            }
            // if vacant, deleted, or absent in the targetState, no need to add
        } // end for
        PackageElement issueComplete = new PackageElement();
        issueComplete.endIssue( targetState ); // an end issue is being sent
        pack.setPackageElement( issueComplete );
        pack.setNewState(targetState.getState());
    }
    
    /**
    * Method called to find which elements need to be added or removed to update the subscriber.
    * NOTE: This method relies on the fact that each issue element is always indexed by an integer
    *  which is unique to that issue element within the period of the collection (also its id).
    *
    * @param oldState   The beginning state of the subscriber.
    * @param newState   The target state of the subscriber.
    * @param pack       The package which will contain the update.
    */
    public void generateUpdateList( Issue oldState, Issue newState, Package pack )
    {
   
        final byte ADD = 0;
        final byte DEL = 1;
        final byte NOP = 2;
        final byte ERR = 3;
        final byte E_AS  = ERR + 0;  // add skipped.  (valid when n > 1 )
        final byte E_ADS = ERR + 1;  // add then delete skipped ( valid n > 2)
        final byte E_DS  = ERR + 2;  // delete skipped ( valid n > 1 )
        final byte E_DBS = ERR + 3;  // active -> deleted -> absent -> vacant ( valid n > 3 )
        
        int vacant = Issue.VACANT;   
        int active = Issue.ACTIVE;   
        int deleted = Issue.DELETED;   
        int absent = Issue.ABSENT;   
    
        if ( (oldState.getFinished() == false) || (newState.getFinished() == false) )
            return; // one of the issues isnt ready yet
        int count = 0;
        pack.setPreviousState(oldState.getState());
        PackageElement startIssue = new PackageElement();
        startIssue.newIssue( newState ); // a new issue is being sent
        pack.setPackageElement( startIssue );
        
                            /* Vacant, Active, Deleted, Absent */
        byte actionPeriod1Collections[][] =  
        /*                              new state           */
        { /*old state*/  /* Vacant, Active, Deleted, Absent */
          /* Vacant  */  {   NOP,    ADD,     E_AS,   E_ADS },
          /* Active  */  {   E_DBS,  NOP,     DEL,    E_DS },
          /* Deleted */  {   NOP,    ADD,     NOP,     NOP },
          /* Absent  */  {   NOP,    ADD,     E_AS,    NOP }
        };

        
        for( count = 0; count <= newState.getElementCount(); count++ )
        {
            int oldStateStatus = oldState.getElementStatus( count );
            int newStateStatus = newState.getElementStatus( count );
            
            switch ( actionPeriod1Collections[oldStateStatus][newStateStatus] )
            {
                case NOP:
                    break;
                case ADD:
                    PackageElement packageElement = new PackageElement();
                    packageElement.packageElementAdd( (IssueElement)this.issueElementArray.get(count) );  
                    pack.setPackageElement( packageElement );
                    break;
                case DEL:
                    packageElement = new PackageElement();
                    packageElement.packageElementRemove( (IssueElement)this.issueElementArray.get(count) );   
                    pack.setPackageElement( packageElement );
                    break;
                // all error recovery techniques are based on the idea that the target state 
                //  is not the faulty state, so keep going and hope that the errors abate 
                case E_AS:  // add skipped.  ( v -> d, ab -> d ) (valid when n > 1 )
                    {
                        // $LOG$ 
                        // warning error - the collection doesn't change - no data loss
                        // indicates either a logic error or data corruption
                    }
                    break;
                case E_ADS: // add then delete skipped ( v -> ab ) ( valid n > 2)
                    {
                        // $LOG$
                        // warning error - the collection doesn't change - no data loss
                        // indicates either a logic error or data corruption
                    }
                    break;
                case E_DS:  // delete skipped ( a -> ab ) ( valid n > 1 )
                    {
                        // $LOG$
                        // error with corrective action taken
                        // delete it
                        packageElement = new PackageElement();
                        packageElement.packageElementRemove( (IssueElement) this.issueElementArray.get(count) );   
                        pack.setPackageElement( packageElement );
                    }
                    break;
                case E_DBS:  // active -> deleted -> absent -> vacant ( valid n > 3 )
                    {
                        // $LOG$
                        // serious error with corrective action taken
                        // This is really sick!  
                        // delete it again and hope against hope!
                        // Log the deletion and pray!
                        packageElement = new PackageElement();
                        packageElement.packageElementRemove( (IssueElement)this.issueElementArray.get(count) );   
                        pack.setPackageElement( packageElement );
                    } 
                    break;
            }
        }
        PackageElement issueComplete = new PackageElement();
        issueComplete.endIssue( newState ); // an end issue is being sent
        pack.setPackageElement( issueComplete );
        pack.setNewState(newState.getState());
    }
    
    /**
    * Method called to update the collection.
    * NOTE: this implementation only handles cumulative 1 (replacement) collections , so 'remove'
    * completely deletes an issue element from the collection
    *
    * @param pack   The package which holds the updates for the collection.
    */
    public void updateCollection( Package pack )
    {
        int add = PackageElement.ADD;
        int remove = PackageElement.REMOVE;
        int newIssue = PackageElement.NEWISSUE;
        int endIssue = PackageElement.ENDISSUE;
                    
        for( int count = 0; count<pack.getPackageLength(); count++ )
        {
            PackageElement pkgE = pack.getPackageElement(count);
            int action = pkgE.getAction();
            if (action == add)
            // adds an element to issueElementArray and marks it as present in the checklist
            {
                IssueElement elm = pkgE.getElement();
                this.addIssueElement(elm);
            }
            else if (action == remove) 
            // removes an element from issueElementArray and marks it as deleted in the checklist
            {                 
                int id = pkgE.getId();
                this.vacantIssueElement(id); // since this is a cumulative 1 collection, anything that is removed -> vacant
            }
            else if (action == newIssue) 
            {
            }     
            else if (action == endIssue) // creates and finishes the issue
            {                
                String issueName = pkgE.getIssueName();
                Issue issue = new Issue(issueName);
                issue.setState( pack.getNewState() );
      // this will change when not a cumulative 1 collection          
                if (this.currentIssue >= 0 )
                    issues.add( this.currentIssue, issue);
                else 
                    issues.add( 0, issue);
                 
                int index = issues.indexOf(issue);
                issue.setId( index );
                this.setStateTable( issue.getState(), index);
                this.issueUpdated( issue );
            }  
        } // end for
        this.setCurrentState( pack.getNewState() );
    }       
}