/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/collections/CollectionManager.java,v 1.3 2000/10/19 17:00:22 andrewf Exp $

<<ICECOPYRIGHT>>
*/

/*----------------------------------------------------------------------------------------+
 |                                                                                        |
 |                              Reference Implementation of ICE                           |
 |                                                                                        |
 |                     Collections -- Collection Manager Class Definition                 |
 |                                                                                        |
 +----------------------------------------------------------------------------------------*/ 


package com.adobe.rice.common.collections;

import com.adobe.rice.common.*;
import com.adobe.rice.util.ExceptionMessages;

import java.util.*;
import java.lang.*;
import com.adobe.rice.kernel.*;

//$JDK1.1$ import com.sun.java.util.collections.*;

public class CollectionManager extends IceCubeManager
{ 
    public static final String cumulativeOne = "n=1"; // used in makeUpdate
    public static final String initialState = "ICE-INITIAL";
    public static final String anyState = "ICE-ANY";
    
    private ArrayList collectionArray;     // an ArrayList of all of the collections that
                                                  // have been created
    private int collectionArrayMark;       // marks the place of the last collection added
                                                  // to collectionArray
    private Hashtable nameTable;           // maps the name of a collection to the index of
                                                  // that collection in collectionArray
    private Hashtable subscriptionTable;   // maps a subscriber's subscriptionId to the index
                                                  // of the desired collection in collectionArray
    private Hashtable subscriptionArchive; // maps a subscriptionId to the last packageArray 
                                                  // that that subscriber was sent
    
    public CollectionManager() throws DispatcherException
    {
          super ("CollectionManager", MessageDispatcher.instance ());
          
        //if (collectionArray == null) 
        // if collectionArray == null, then nothing else has been initialized, either 
        //{
            collectionArray = new ArrayList();
            collectionArrayMark = -1;
            nameTable = new Hashtable();
            subscriptionTable = new Hashtable(); 
            subscriptionArchive = new Hashtable();
      //  }
   }

    /**
    * Method called to create a collection with the given name. It places the new collection in 
    * collectionArray and also sets it in nameTable so that the collection may be found by name
    * in the future. 
    *
    * @param    collectionName      The name of the collection which is being created.
    */
    public Collection createCollection( String collectionName )
    {
        Collection newCollection = new Collection(collectionName);
        collectionArray.add(newCollection);
        collectionArrayMark++;
        newCollection.setId(collectionArrayMark);
        Integer index = new Integer(collectionArrayMark);
        nameTable.put(collectionName, index);  
        return newCollection;
    }
    
    /**
    * Method called to use the name of a collection to return that collection.
    *
    * @param    collectionName      The name of a collection. 
    */
    public Collection getCollection( String collectionName )
    {
         Integer index = (Integer) nameTable.get(collectionName);
         if (index == null)
            return null; // $LOG$ the error - the collection is not in nameTable
         int collectionIndex = index.intValue();
         if (collectionIndex < collectionArray.size())
            return null; // $LOG$ the error - the collection is not in collectionArray
         return (Collection) collectionArray.get(collectionIndex);
    }
    
    /**
    * Method called to find a collection by subscription identifier and return the collection.
    *
    * @param    subscriptionId      The subscription identifier of a collection, which is 
    *                               used as a key to return the corresponding collection. 
    */
    public Collection getSubscriptionTable( String subscriptionId )
    {
        Integer index = (Integer) subscriptionTable.get(subscriptionId);
        if (index == null)
            return null; // $LOG$ the error - the subscription id is not in subscriptionTable
        int collectionIndex = index.intValue(); 
        if (collectionIndex >= collectionArray.size())
            return null; // $LOG$ the error - the collection is not in collectionArray
        return (Collection) collectionArray.get( collectionIndex );
    }

    /**
    * Method called to put a subscriptionId and the index of the corresponding collection into 
    * collectionTable.
    *
    * @param    subscriptionId          The subscription identifier of a collection, which will 
    *                                   be the key for that collection in this table. 
    * @param    collectionSubscribedTo  The index of the collection which is to be added to the 
    *                                   collection table as the value of subscriptionId.
    */
    public void setSubscriptionTable( String subscriptionId, Collection collectionSubscribedTo )
    {
        int index = collectionSubscribedTo.getId();
        if (index >= 0)
        {
            Integer collectionIndex = new Integer( index );
            subscriptionTable.put( subscriptionId, collectionIndex );
        }
        else
        {
            // either the collection does not exist or its id was not set properly
        }
    }

    /**
    * Method called to find a collection by name and return that collection.
    *
    * @param    collectionName      The name of a collection, which is used as a key to 
    *                               return the corresponding collection. 
    */
    public Collection getNameTable( String collectionName )
    {
        Integer index = (Integer) nameTable.get(collectionName);
        if (index == null)
            return null; // $LOG$ the error - the subscription id is not in subscriptionTable
        int collectionIndex = index.intValue(); 
        if (collectionIndex < collectionArray.size())
            return null; // $LOG$ the error - the collection is not in collectionArray
        return (Collection) collectionArray.get( collectionIndex );
    }

    /**
    * Method called to put the name of a collection and the index of the corresponding collection 
    * into collectionTable.
    *
    * @param    collectionName          The name of a collection, which will be the key for that 
    *                                   collection in this table. 
    * @param    collectionSubscribedTo  The index of the collection which is to be added to the 
    *                                   collection table as the value of collectionName.
    */
    public void setNameTable( String collectionName, Collection collectionSubscribedTo )
    {
        int index = collectionSubscribedTo.getId();
        if (index >= 0)
        {
            Integer collectionIndex = new Integer( index );
            nameTable.put( collectionName, collectionIndex );
        }
        else
        {
            // either the collection does not exist or its id was not set properly
        }
    }
    /** 
    * Method called to get a package from subscriptionArchive.
    *
    * @param subId  Subscription identifier of the desired packageArray.
    */
    public ArrayList getArchivedPackages( String subscriptionId )
    {
       if ( subscriptionArchive.containsKey(subscriptionId) )
       {
            PackageManifest packages = (PackageManifest) subscriptionArchive.get(subscriptionId);
            return packages.getPackageArray();
       }
       else
            return null;
    }
    
    /**
    * Method called to put a packageArray in subscriptionArchive.  
    *
    * @param packagesToArchive  The ArrayList of packages which is to be put into 
    *                           subscriptionArchive.
    */
    public void archivePackages( ArrayList packagesToArchive )
    {
        Package firstPackage = (Package)packagesToArchive.get(0);
        if ( firstPackage.getSubscriptionId() != "" )
        {
            PackageManifest archive = new PackageManifest(packagesToArchive);
            subscriptionArchive.put( firstPackage.getSubscriptionId(), archive );
        }
        else
        {
            return; // the subscription id hasnt been put in, so there's no way to know where it goes
        }
    }
    
    /**
    * Method called to remove a package from subscriptionArchive.  
    *
    * @param idOfPackagesToRemove   The subscription id of the packages to be removed from 
    *                               subscriptionArchive.
    */    
    public void removeArchivedPackages( String idOfPackagesToRemove )
    {
        if (subscriptionArchive.containsKey( idOfPackagesToRemove))
            this.subscriptionArchive.remove( idOfPackagesToRemove );
        else
            return; // the corresponding packages weren't in
    }    
   
    /**
    * Method called to return an ArrayList of all of the packages that are needed to update
    * the subscriber. 
    *
    * @param subscriptionId         The subscription identifier identifies the subscription 
    *                               that this package belongs to.
    * @param currentSubscriberState The current state of the subscriber.
    */
    public ArrayList makeUpdate(String subscriptionId, String currentSubscriberState)
    {
        Collection collectionSubscribedTo = getSubscriptionTable(subscriptionId);
        SubscriptionManagerInterface smi = new SubscriptionManagerInterface();
        String targetState = smi.getTargetState( subscriptionId, collectionSubscribedTo, currentSubscriberState);
        
        if ( currentSubscriberState.equals(targetState) )
            return null; // send a "no update necessary" message
        ArrayList pastPackages = this.getArchivedPackages(subscriptionId);
        Package containsAllPackageElements = new Package();
        if (pastPackages != null)
        {
            for ( int count = 0; count < pastPackages.size(); count++ )
            // check the packages that were sent in the last update.  
            {
                Package archivedPackage = (Package)pastPackages.get(count);
                if ( currentSubscriberState.equals(archivedPackage.getPreviousState()) )
                // if the current state of the subscriber is the same as the previous state of any 
                // of the packages, the packages weren't received correctly and need to be resent
                {
//$$FIX$$ this may not yet move the subscriber to their target state
                    // return an arrayList of the packages in the archive, starting with this
                    // one and ending with the last one in that list.
                    ArrayList packed = new ArrayList();
                    for ( int nextPackage = 0; (count+nextPackage) < pastPackages.size(); nextPackage++ )
                    {
                        packed.add( (Package) pastPackages.get(count+nextPackage) );
                        // the sizes of the packages have already been correctly measured, there is no need to repack
                    } // end for
                    packed.trimToSize();
                    this.archivePackages(packed);
                    return packed; // *NOTE* this may not completely update the subscriber, but it will move them out of an intermedite state.
                } 
            } // end for
        }  
        String subscriptionType = SubscriptionManagerInterface.getSubscriptionType( subscriptionId );       
        int maximumPackageSize = SubscriptionManagerInterface.findMaximumPackageSize( subscriptionId );

        if ( ( (currentSubscriberState.equals(initialState)) && (subscriptionType.equals(cumulativeOne)) )
                || currentSubscriberState.equals(anyState) )
        { 
            // if the subscriber is in ICE-ANY or is in ICE-INITIAL of a cumulativeOne subscription, 
            //  only the entire current issue needs to be sent
            int target = collectionSubscribedTo.getStateTable(targetState);
            Issue targetIssue = collectionSubscribedTo.getIssue(target);
            collectionSubscribedTo.generateInitialList(currentSubscriberState, targetIssue, containsAllPackageElements);
        }
        else if ( currentSubscriberState.equals(initialState) ) //$FIX$ this is where support for n>1 from the initial state should be added
        {
            return null; // this implementation only handles cumulative 1 (replacement) collections
        }
        else
        // if the method goes this far, it can be safely assumed that the subscriber is not currently state
        // at an intermediate point between issues
        {
            int currentSubscriberIssue = collectionSubscribedTo.getStateTable(currentSubscriberState); 
            int currentTargetIssue = collectionSubscribedTo.getStateTable(targetState);
            

            if ( subscriptionType.equals(cumulativeOne) )
            {
                Issue targetIssue = collectionSubscribedTo.getIssue( currentTargetIssue );
                Issue subscriberIssue = collectionSubscribedTo.getIssue( currentSubscriberIssue );
                collectionSubscribedTo.generateUpdateList( subscriberIssue, targetIssue, containsAllPackageElements );

            }
            else //$FIX$ this is where support for n>1 should be added
            {
                return null; // this implementation only handles cumulative 1 (replacement) collections
            }  
        }
        // all the correct info is in containsAllPackageElements 
        ArrayList packages = this.packagePreparation( subscriptionId, containsAllPackageElements, maximumPackageSize );
        packages.trimToSize();
        this.archivePackages(packages);
        return packages; 
        }
        
    /**
    * Method called to sort all of the package elements into packages of an appropriate size
    * for sending. 
    *
    * @param subscriptionId         The id that tells which subscription these packages are for
    * @param subscriberStartState   The state which the subscriber starts in <--- not nec?
    * @param extended               The package containing all of the package elements that it 
    *                               will be neccessary to send.
    * @param maximumPackageCapacity The maximum size of the packages to be sent.
    */
    public ArrayList packagePreparation( String subscriptionId, Package extended, int maximumPackageCapacity )
    {
        int add = PackageElement.ADD;
        int remove = PackageElement.REMOVE;
        int newIssue = PackageElement.NEWISSUE;
        int endIssue = PackageElement.ENDISSUE;
        
        int packageSize = 0;
        char append = 'a';
        Package newPackage = new Package( subscriptionId, extended.getPreviousState()); // the package that will be added to the package ArrayList
        ArrayList finishedPackages = new ArrayList();
        String currentStateOfPackage = extended.getPreviousState(); // the state of the package is determined by whether an endIssue is being sent
        // in that package - if it is, the id of that issue is used as the newState of the package. if no issue is
        // ended over the duration of the package, a letter is concatonated with the state
        for (int count=0; count<extended.getPackageLength(); count++)
        {
            int itemSize;
            PackageElement packageElement = (PackageElement) extended.getPackageElement(count);
            if ( packageElement.getAction() == add  )
            {
                IssueElement issueElement = packageElement.getElement();
                Object toHash = issueElement.getItem(0);
//$$FIX$$ either want to pass the hash, or the actual items and add them up, depending
                itemSize = StorageManagerInterface.getItemSize(toHash);
            }
            else // remove, newIssue, and endIssue commands don't require sending an entire element, so they take up less space
            {
                if (packageElement.getAction() == endIssue ) 
                {
                    // if endIssue pops up, set stateOfThePackage equal to the state of that issue
                    Collection clxn = this.getSubscriptionTable(subscriptionId);
                    int id = packageElement.getId();
                    Issue issue = clxn.getIssue(id);
                    currentStateOfPackage = issue.getState();
                }
                itemSize = StorageManagerInterface.getSizeOfElement();
            }
            packageSize += itemSize;
            if ( packageSize <= maximumPackageCapacity )
            {
                newPackage.setPackageElement(packageElement);
                continue;
            }
            else
            {
                if ( currentStateOfPackage.equals( newPackage.getPreviousState() ) )
                {
                    // add something to the end of currentStateOfPackage
                    currentStateOfPackage = currentStateOfPackage+append;
                    append++;
                }
                //else an endIssue happened sometime during the package, so that's the newState
                newPackage.setNewState( currentStateOfPackage );
                newPackage.trimPackage();
                finishedPackages.add(newPackage);
                newPackage = new Package( subscriptionId, newPackage.getNewState() );
                newPackage.setPackageElement(packageElement);
                packageSize = itemSize;
            }
        }
        newPackage.trimPackage();
        newPackage.setNewState( extended.getNewState() );
        finishedPackages.add(newPackage);
        return finishedPackages;
    }
    
    /**
    * Method called to decompose a Package into individual adds, removes, etc.  It returns true 
    * when the package has been successfully decomposed and the commands have been executed.
    *
    * @param update     The package which contains instructions for updating the collection.
    */
    public boolean unwrapPackage(Package update)
    {
        String subscId = update.getSubscriptionId();
        Collection collectionSubscribedTo = this.getSubscriptionTable(subscId);
        String updateOldState = update.getPreviousState();
        if (updateOldState.equals(collectionSubscribedTo.getCurrentState()))
        {
           // the package's oldState and the collection's currentState match up, like they're supposed to
           if ( cumulativeOne.equals(SubscriptionManagerInterface.getSubscriptionType(subscId)) )
           {
                collectionSubscribedTo.updateCollection( update );
                collectionSubscribedTo.setCurrentState( update.getNewState() );
                return true;
           }
           else
           {
            return false; 
            // temporary: this implementation only handles cumulative 1 (replacement) collections
           }
        }
        else
        {
            return false; 
            // this is a very bad thing - the update and the current state do not match 
            // $LOG$ the error
        }
        
    }
    
    
    // Execution Activity Logic
    //
    public void preInitialize() { }
    public void initialize() { }
    public void postInitialize() { }
    //
    public void run() { }
    //
    public void preFinalize() { }
    public void finalize() { }
    public void postFinalize() { }

}