/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/collections/Issue.java,v 1.1.1.1 2000/09/19 19:39:44 bhunt Exp $

<<ICECOPYRIGHT>>
*/

/*----------------------------------------------------------------------------------------+
 |                                                                                        |
 |                              Reference Implementation of ICE                           |
 |                                                                                        |
 |                           Collections -- Issue Class Definition                        |
 |                                                                                        |
 +----------------------------------------------------------------------------------------*/ 

package com.adobe.rice.common.collections;

import com.adobe.rice.common.storage.logging.*;
import com.adobe.rice.common.policies.*;
import com.adobe.rice.common.*;

//$JDK1.1$ import com.sun.java.util.collections.*;
import java.util.*;
import java.lang.Object;

public class Issue 
{
    public static final int VACANT = 0; 
    public static final int ACTIVE = 1;
    public static final int DELETED = 3;
    public static final int ABSENT = 2;
    
    private String name;            // name of the issue
    private int    id;              // the number of this issue, used to index it in the issue array
    private BitVector itemStatus;   // list of statuses of items that make up the issue
    private boolean finished;       // indicator of whether the issue has been completed
    private String state;           // the state which marks that this issue was received
        
    public Issue()
    {
        name = "";
        id = -1;
        itemStatus = new BitVector();
        finished = false;
        state = CollectionManager.initialState;
    }
    
    public Issue( String issueName )
    {
        name = issueName;
        id = -1;
        itemStatus = new BitVector();
        finished = false;
        state = CollectionManager.initialState;
    }
    
    public String getName() { return name; }
    public void setName( String newName ) { name = newName; }
    public int getId() { return id; }
    public void setId( int newId ) { id = newId; }
    public void setState ( String newState ) { state = newState; }
    public String getState () { return state; }
    public boolean getFinished() {return finished;} 
    
    /**
     * Method returns the number of elements in this issue.
     **/
    public int getElementCount() { return itemStatus.getSize(); }
        
    /**
    * Method called to find an element's status by id.
    *
    * @param   key  Integer corresponding to the id of the PackageElement whose
    *               status the method is searching for.   
    */
    public int getElementStatus( int key )
    {
        if ( key > -1 )
            return itemStatus.getItem( key );
        else
            return -1;
    }
    
     
    /**
    * Method called to set the item status to vacant. 
    *
    * @param    index   The offset of the item to set.
    */
    public void setItemVacant( int index )
    {
        if ( index > -1 )
            itemStatus.setItem( index, VACANT );
        // else do nothing
    }
    
    /**
    * Method called to set the item status to active. 
    *
    * @param    index   The offset of the item to set.
    */
    public void setItemActive( int index )
    {
        if ( index > -1 )
            itemStatus.setItem( index, ACTIVE );
        // else do nothing
    }
    
    /**
    * Method called to set the item status to deleted. 
    *
    * @param    index   The offset of the item to set.
    */
    public void setItemDeleted( int index )
    {
        if ( index > -1 )
            itemStatus.setItem( index, DELETED );
        // else do nothing
    }
     
    /**
    * Method called to set the item status to absent. 
    *
    * @param    index   The offset of the item to set.
    */
    public void setItemAbsent( int index )
    {
        if ( index > -1 )
            itemStatus.setItem( index, ABSENT );
        // else do nothing
    }
    
    /**
    * Method called to declare an issue to be finished. 
    *
    * @param    numberOfCellsUsedInIssue   The number of elements and spaces that make up the finished issue.
    */
    public void finishIssue( int numberOfCellsUsedInIssue, String issueState ) 
    { 
        if ( finished==true )
        {
            //big problem - the indicated issue has already been made!
            return;
        }
        if ( numberOfCellsUsedInIssue > itemStatus.getSize() )
        {
           // $FIX$  The itemStatus size must always be greater than or equal the number of items!
           // $LOG$ log the error and carry on!
        }
        finished = true; 
        state = issueState;
    }
}