/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/collections/Model.java,v 1.1.1.1 2000/09/19 19:39:44 bhunt Exp $

<<ICECOPYRIGHT>>
*/

/*----------------------------------------------------------------------------------------+
 |                                                                                        |
 |                              Reference Implementation of ICE                           |
 |                                                                                        |
 |                           Collections -- Model Class Definition                        |
 |                                                                                        |
 +----------------------------------------------------------------------------------------*/ 

package com.adobe.rice.common.collections;

import com.adobe.rice.common.*;
import com.adobe.rice.util.ExceptionMessages;

import java.util.*;
//$JDK1.1$ import com.sun.java.util.collections.*;

public class Model
{
    private ArrayList   subscriptionElements; 
    private Hashtable   nameTable;
    
    public Model()
    {
        subscriptionElements = new ArrayList();
        nameTable = new Hashtable();
    }
    
    /**
    * Method called to set the name of a subscription element and the index of that element in the array
    * in nameTable.
    * @param    nameToSet   String containing the name of the model element
    * @param    index       The index of the subscription element in subscriptionElements.
    */
    private void setIndex(String nameToSet, int index)
    {
        if (index < 0)
            return; // invalid index
        else if (index > subscriptionElements.size())
            return; // invalid index
        else
        {
            Integer ind = new Integer(index);
            nameTable.put(nameToSet, ind);
        }
    }
    
    /**
    * Method called to use the name of a subscription element to find the index of that element in the array.
    *
    @param  nameToFind  String containing the name of the subscription element that 
    *                   the function is searching for the index of.
    */
    private int getIndex(String nameToFind)
    {
        Integer ind = (Integer) nameTable.get(nameToFind);
        if (ind == null)
        {
              return -1; 
        }
         else  
         {
            int index = ind.intValue();
            return index;
         }
    }

    /**
    * Method called to put the desired subscription element into the model. 
    *
    * @param   subscriptionElementToAdd   The subscription element which is to be added to the model.
    */
    public void setSubscriptionElement( SubscriptionElement subscriptionElementToAdd )
    {
        String name = subscriptionElementToAdd.getName();
        if ( getIndex(name) >= 0 ) 
        // the subscription element is already in the model: it will be replaced with the new element
        {
            subscriptionElements.set( getIndex(name), subscriptionElementToAdd );
            subscriptionElementToAdd.setId( getIndex(name) );
        }
        else
        // the subscription element will be added to the end of the model        
        {
            subscriptionElements.add(subscriptionElementToAdd); 
            int index = subscriptionElements.indexOf(subscriptionElementToAdd);
            setIndex(name, index);
            subscriptionElementToAdd.setId( index );
        }
        return;
    }
    
    /**
    * Method called to set the subscription element X into subscription element (group) Y.  This
    * also puts subscription element X into the model. It will return -1 if subscription element Y is
    * not a group.
    *
    * @param   subscriptionElementToAdd     The subscription element which is to be added to the model.
    * @param   subscriptionElementToAddInto The subscription element which the first element is to be added into.
    */
    public int setSubscriptionElementInGroup( SubscriptionElement subscriptionElementToAdd,  SubscriptionElement subscriptionElementToAddInto )
    {
        int success = subscriptionElementToAddInto.putIntoGroup(subscriptionElementToAdd);
        
        String name = subscriptionElementToAdd.getName();
        if ( getIndex(name) >= 0 ) 
        {
            int index = getIndex(name);
            SubscriptionElement check = (SubscriptionElement) subscriptionElements.get(index);
            if ( subscriptionElementToAdd.getItemGroup() == check.getItemGroup() )
            // the subscription element is already in the model: it will be replaced with the new element
            {
                
                subscriptionElements.set( getIndex(name), subscriptionElementToAdd );
                subscriptionElementToAdd.setId( getIndex(name) );
                return success;
            }
        }
        // the subscription element will be added to the end of the model        
        subscriptionElements.add(subscriptionElementToAdd); 
        int index = subscriptionElements.indexOf(subscriptionElementToAdd);
        setIndex(name, index);
        subscriptionElementToAdd.setId( index );
        return success;
    }
    
    /**
    * Method called to find a subscription element by name and return the subscription element.
    *
    * @param    nameToFind  String containing the name of the subscription element that 
    *                      the function is searching for.
    */
    public SubscriptionElement getSubscriptionElement( String nameToFind )
    {
        if (nameToFind == null)
            return null;
        int index = getIndex(nameToFind);
        if (index >= 0)
            return (SubscriptionElement) subscriptionElements.get(index);
        else
            return null;
    }

        
}