/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/collections/PackageElement.java,v 1.1.1.1 2000/09/19 19:39:44 bhunt Exp $

<<ICECOPYRIGHT>>
*/

/*----------------------------------------------------------------------------------------+
 |                                                                                        |
 |                              Reference Implementation of ICE                           |
 |                                                                                        |
 |                         Collections -- Package Element Class Definition                     |
 |                                                                                        |
 +----------------------------------------------------------------------------------------*/ 

package com.adobe.rice.common.collections;

import com.adobe.rice.common.*;
import com.adobe.rice.util.ExceptionMessages;

import java.util.*;
//$JDK1.1$ import com.sun.java.util.collections.*;

public class PackageElement
{
    public static final int ADD = 1;
    public static final int REMOVE = 2;
    public static final int NEWISSUE = 0;
    public static final int ENDISSUE = 3;
    
    private int action;             // to contain add, remove, newIssue, endIssue
    private IssueElement element;   // the issue element affected if action is add 
    private int id;                 // the id of the affected issue if action is remove or newIssue or endIssue
    private String issueName;       // the name of the issue if action is newIssue
    
    public PackageElement()
    {
        action = -1;
        element = null;
        id = -1;
        issueName = null;
    }
    
    public int getAction() {return action;}
    public IssueElement getElement() { return element; }
    public int getId() { return id; }
    public String getIssueName() { return issueName; }
    
    /**
    * Method to set the action of PackageElement to add, and set element to the indicated issue element.
    *
    * @param issueElementToAdd     The issue element which is to be added by the receiver of the package.
    */
    public void packageElementAdd( IssueElement issueElementToAdd )
    {
        if (issueElementToAdd == null)
            return;
        action = ADD;
        element = issueElementToAdd;
    }
    
    /**
    * Method to set the action of PackageElement to remove, and element to the indicated issue element.
    *
    * @param issueElementToRemove     The issue element which is to be removed by the receiver of the package.
    */
    public void packageElementRemove( IssueElement issueElementToRemove)
    {
        if (issueElementToRemove == null)
            return;
        action = REMOVE;
        id = issueElementToRemove.getId();
    }

    /**
    * Method to set the action of PackageElement to newIssue, and issue to the issueId of the indicated issue.
    *
    * @param toStart     The issue which is to be begun by the receiver of the package.
    */    
    public void newIssue( Issue toStart )
    {
        if (toStart == null)
            return;
        action = NEWISSUE;
        id = toStart.getId();
        issueName = toStart.getName();
    }
    
    /**
    * Method to set the action of PackageElement to endIssue, and issue to the issueId of the indicated issue.
    *
    * @param toEnd     The issue which is to be ended by the receiver of the package.
    */        
    public void endIssue( Issue toEnd )
    {
        if (toEnd == null)
            return;
        action = ENDISSUE;
        id = toEnd.getId();
        issueName = toEnd.getName();
    }
}