/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/collections/PackageManifest.java,v 1.1.1.1 2000/09/19 19:39:44 bhunt Exp $

<<ICECOPYRIGHT>>
*/

/*----------------------------------------------------------------------------------------+
 |                                                                                        |
 |                              Reference Implementation of ICE                           |
 |                                                                                        |
 |                      Collections -- PackageManifest Class Definition                   |
 |                                                                                        |
 +----------------------------------------------------------------------------------------*/ 

package com.adobe.rice.common.collections;

import com.adobe.rice.common.*;
import com.adobe.rice.util.ExceptionMessages;

import java.util.*;
//$JDK1.1$ import com.sun.java.util.collections.*;

public class PackageManifest
{
    private static final int unsent = 0;
    private static final int sent = 1;
    private static final int sendAcknowledged = 2;
    private static final int sentAndOk = 3;
    
    private ArrayList packageArray;
    private BitVector packageStatus;
  
    public PackageManifest()
    {
        packageArray = new ArrayList();
        packageStatus = new BitVector();
    }
  
    public PackageManifest( ArrayList packageArrayToArchive )
    {
        packageArray = packageArrayToArchive;
        packageStatus = new BitVector();
    }
    
    /**
    * Method called to put a packageArray into PackageManifest.
    */
    public void setPackageArray( ArrayList packageArrayToArchive)
    {
        packageArray.clear();
        packageArray = packageArrayToArchive; //even if the array is null, we want to have it recorded.
    }
    
    /**
    * Method called to return a packageArray from PackageManifest.
    */
    public ArrayList getPackageArray()
    {
        return packageArray;
    }
    
    /**
    * Method called to return a package from inside packageArray.
    *
    * @param indexOfPackageInPackageArray   The number which corresponds to the index 
    *                                       of the package in packageArray.
    */
    public Package getPackage(int indexOfPackageInPackageArray)
    {
        if ((indexOfPackageInPackageArray < 0) || (indexOfPackageInPackageArray > packageArray.size()))
            return null;
        else
            return (Package) packageArray.get(indexOfPackageInPackageArray);
    }
    
    /**
    * Method called to set the status of a specific package in packageArray to the specified status.
    *
    * @param indexOfPackageInPackageArray   The number which corresponds to the index 
    *                                       of the package in packageArray.
    * @param statusOfPackage                The status to which the package is to be set.
    */
    public void setPackageStatus(int indexOfPackageInPackageArray, int statusOfPackage)
    {
        if ((indexOfPackageInPackageArray < 0) || (indexOfPackageInPackageArray > packageArray.size()))
            return;
        packageStatus.setItem(indexOfPackageInPackageArray, statusOfPackage);
    }
    
    /**
    * Method called to return the status of a specific package in packageArray.
    *
    * @param indexOfPackageInPackageArray   The number which corresponds to the index 
    *                                       of the package in packageArray.
    */
    public int getPackageStatus(int indexOfPackageInPackageArray)
    {
        if ((indexOfPackageInPackageArray < 0) || (indexOfPackageInPackageArray > packageArray.size()))
            return -1;
        return packageStatus.getItem(indexOfPackageInPackageArray);
    }
}