/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/collections/SubscriptionElement.java,v 1.1.1.1 2000/09/19 19:39:44 bhunt Exp $

<<ICECOPYRIGHT>>
*/

/*----------------------------------------------------------------------------------------+
 |                                                                                        |
 |                              Reference Implementation of ICE                           |
 |                                                                                        |
 |                     Collections -- Subscription Element Class Definition               |
 |                                                                                        |
 +----------------------------------------------------------------------------------------*/ 

package com.adobe.rice.common.collections;

import com.adobe.rice.common.*;
import com.adobe.rice.util.ExceptionMessages;

import java.util.*;
//$JDK1.1$ import com.sun.java.util.collections.*;

public class SubscriptionElement 
{
    private String name;        // name of the subscription element.
    private int id;             // the number for this model element.
    private boolean current;    // false if the subscription element is out of date 
    private int issueElementReferenced; // index of the issue element which refers to this subscription element
    private int useCount;       // number of times this element is used at the moment
    private boolean itemGroup;  // true if the subscription element is an item-group
    private Object item;        // if not a group, this item that makes up the subscription element 
                                // if a group, this is null
    private int hash;           // if not a group, this is the hash number of the item
                                // if a group, this is -1
    public LinkedList head;     // if a group, this is the list of elements in the group
                                // if not a group, this is null
    private SubscriptionElement inGroup; // if in a group, this that group
                                // if not in a group, this is null
    
    public SubscriptionElement()
    {
        name = "";
        id = -1;
        itemGroup = false;
        item = null;
        hash = -1;
        current = true;
        issueElementReferenced = -1;
        useCount = 0;
        head = null;
        inGroup = null;
    }

    public SubscriptionElement( String subscriptionElementName, boolean isGroup )
    // to use if subscription element is a group
    {
        name = subscriptionElementName;
        id = -1;
        itemGroup = isGroup;
        item = null;
        hash = -1;
        current = true;
        issueElementReferenced = -1;
        useCount = 0;
        head = new LinkedList();
        inGroup = null;
    }
    
    public SubscriptionElement( String subscriptionElementName, Object seItem )
    // to use if subscription element is not a group
    {
        name = subscriptionElementName;
        id = -1;
        itemGroup = false;
        item = seItem;
        hash = seItem.hashCode();
        current = true;
        issueElementReferenced = -1;
        useCount = 0;
        head = null;
        inGroup = null;
    }
    
    public void setName( String newName ) { name = newName; }
    public String getName() { return name; }
    public void setId( int newId ) { id = newId; }
    public int getId() { return id; }
    public void setCurrent( boolean notCurrent ) { current = notCurrent; }
    public boolean getCurrent() { return current; }
    public void setIssueElementReferenced( int ie ) { issueElementReferenced = ie; }
    public int getIssueElementReferenced() { return issueElementReferenced; }
    public void setUseCount( int count ) { useCount = count; }
    public int getUseCount() { return useCount; }
    public boolean getItemGroup() { return itemGroup; }
    public Object getItem() { return item; }
    public int getHash() { return hash; }
    public void setInGroup( SubscriptionElement newGroup ) { inGroup = newGroup; }
    public SubscriptionElement getInGroup() { return inGroup; }
    
    /**
    * If the subscription element is a group, this method adds elements to the group and returns 0.  
    * If the subscription element is not a group, it will return -1.
    *
    * @param se The subscription element which is to be added to the group.
    */
    public int putIntoGroup(SubscriptionElement se)
    {
        if (this.itemGroup == false)
        {
            return -1;
        }
        se.setInGroup(this);
        LinkedList list = head;
        int index = list.indexOf(se);
        if ( index == -1)
        // the subscription element is not already in the list - add it
        {
            list.addLast(se);
            System.out.println("not already in list");
        }
        else
        // the subscription element is already in the list
        {
            return alreadyInList(list, se, index);
        }
        return 0;
    }
     
    /**
    * This method is called to find the subscription element that is in the list and respond according to 
    * what it finds.
    *
    * @param list   The linked list that the subscription element is in.
    */
    public int alreadyInList(LinkedList list, SubscriptionElement se, int index)
    {
        SubscriptionElement former = (SubscriptionElement)list.get(index);
        if ( former.getItemGroup() != se.getItemGroup() )
        {
            return -1; // error - one element is an item, the other is a group - they can't be compared
        }
        if ( former.getItemGroup() == false ) 
        // both elements are items (not groups), so compare hash numbers
        {
            if ( former.getHash() == se.getHash() )
            // their items are the same - no change neccessary
            {
                return 0;
            }
            else
            // their items are different - replace the old with the new
            {
                se.setUseCount( former.getUseCount() ); // the new SE is replacing the old one
                former.setUseCount(0); // the old SE is no longer current
                int toChange = former.getIssueElementReferenced();
                se.setIssueElementReferenced(toChange);
//$$FIX$$ change the IERefd's SERef?!
                return 0;
            }
        }
        else
        // both elements are groups, so elements in group must be compared
        {
            LinkedList older = former.head;
            LinkedList newer = se.head;
            for ( int count = 0; count<older.size(); count++ )
            {
                SubscriptionElement past = (SubscriptionElement) older.get(count);
                
                index = newer.indexOf(past);
                if ( index == -1)
                // the subscription element is not already in the list - add it
                {
                    newer.addLast(se);
                    System.out.println("not already in list");
                }
                else
                // the subscription element is already in the list
                {
                    int ret = alreadyInList(newer, past, index);
                    if (ret == -1)
                    {
                        return ret;
                    }
                }
            } // end for 
            return 0;
        }
    }

    
}