/*
 As of 2000/08/24, this method works properly
*/
// this is to test packagePreparation
// if you would like to test just checklist or BitVector, it would be better to use iceCheck
// if you would like to test just stateTable, it would be better to use iceTable
// if you would like to test just issueElementArray, it would be better to use iceIssue
// if you would like to test generateUpdateList, it would be better to use icePkgList
// if you would like to test makeUpdate, it would be better to use iceUpdate
// if you would like to test makeUpdate and unwrapPackage, or want an example of how to create, 
//   fill, and update a collection from "scratch," use iceTest


// NOTE: the states should vary a tiny bit, due to the fact that they contain the current date, 
//   but the last digit should match (the last digit = issueId+1 )
// output: final IssueElementMark for issue 0 = 2
//         issue 0 is finished and has been added to 'issues'.
//         final IssueElementMark for issue 1 = 5
//         issue 1 is finished and has been added to 'issues'.
//         status of name00
//          in issue 0 is 1
//          in issue 1 is 3
//         status of name01
//          in issue 0 is 1
//          in issue 1 is 3
//         status of name02
//          in issue 0 is 1
//          in issue 1 is 3
//         status of name10
//          in issue 0 is 0
//          in issue 1 is 1
//         status of name11
//          in issue 0 is 0
//          in issue 1 is 1
//         status of name12
//          in issue 0 is 0
//          in issue 1 is 1
//         prev state = ASHAMBLE2000-08-231
//         new state = ASHAMBLE2000-08-231a
//         prev state = ASHAMBLE2000-08-231a
//         new state = ASHAMBLE2000-08-231ab
//         prev state = ASHAMBLE2000-08-231ab
//         new state = ASHAMBLE2000-08-232
//         done

package com.adobe.rice.common.collections;
import java.util.*;
import java.io.*;
import com.adobe.rice.kernel.*;

public class icePackage
{
    public static void main( String[] args )
    { 
        try
        {
           CollectionManager clxnMgr = new CollectionManager();
           String collectionName = "bob";    
           Collection bob = clxnMgr.createCollection(collectionName);
           for (int is = 0; is<2; is++)
           {
               Model model = bob.getModel();
               for ( int count = 0; count < 3; count++ )
               {
                 String nm = "name"+is+count;
                 String ref = "ref"+is+count;
                
                 SubscriptionElement se = new SubscriptionElement(ref, nm);
                 model.setSubscriptionElement(se);
                
                 IssueElement IElement = new IssueElement(nm, ref );
                 IElement.finishIssueElement();
                 bob.setIssueElement(IElement);
               } // end for
        
               Issue jules = new Issue("issue"+is);
               bob.issueDone( jules ); 
               System.out.println( "final IssueElementMark for issue "+bob.getCurrentIssue()+ " = " + bob.getIssueElementMark() );
            
               Issue inIssues = (Issue) bob.getIssue( bob.getCurrentIssue() );
               if ( inIssues.getFinished() == true )
                  System.out.println( "issue " + inIssues.getId()+ " is finished and has been added to 'issues'." );        
           } // end for
        
           Issue currentIssue = bob.getIssue( bob.getCurrentIssue() );
           int numIssueElements = currentIssue.getElementCount();
           for (int index=0; index<=(numIssueElements); index++) 
           {
               IssueElement jen = bob.getIssueElement(index);
               System.out.println("status of " + jen.getName());
               for (int count = 0; count <= currentIssue.getId(); count++)
               {
                   Issue issue = bob.getIssue( count );
                   System.out.println("   in issue " + issue.getId() + " is " + issue.getElementStatus(index) );
               }
           } // end for
        
           // corr. to generateUpdateList
           String subscId = "subscriptionId";
           int subState = (currentIssue.getId()-1);  // try 0, 1, 2, 3; 3 should come out w/o an update nec.
           Issue from = bob.getIssue(subState);
           Issue to = bob.getIssue(bob.getCurrentIssue());
           clxnMgr.setSubscriptionTable(subscId, bob);

           Package pkgList= new Package();
           bob.generateUpdateList( from, to, pkgList ); 
        
           // corr. to packagePreparation
           int maximumPackageSize = SubscriptionManagerInterface.findMaximumPackageSize( subscId );
        
           ArrayList packages = clxnMgr.packagePreparation( subscId, pkgList, maximumPackageSize);
        
        for(int count = 0; count<packages.size(); count++)
        {
            Package pkg = (Package)packages.get(count);
            System.out.println("prev state = "+pkg.getPreviousState());
            System.out.println("new state = "+pkg.getNewState());
        }
        System.out.println( "done" );
        return;
        }
        catch (DispatcherException e)
        {
            e.printStackTrace ();
        }
    }
    
    /*
    * Method to print the contents of an Issue
    */
    public static void printIssue( Collection clxn, Issue issue)
    {
        System.out.println("issue "+issue.getId()+" of state "+issue.getState());
        
        for (int count=0; count<=clxn.getIssueElementMark(); count++)
        {
            IssueElement ie = clxn.getIssueElement(count);
            System.out.print("  issue element "+ie.getName());
            System.out.println(" status "+issue.getElementStatus(count));
        }
    }
}