/*
 As of 2000/08/24, this method works properly
*/
// this is to test makeUpdate (which includes generateUpdateList and packagePreparation)
// if you would like to test just checklist or BitVector, it would be better to use iceCheck
// if you would like to test just stateTable, it would be better to use iceTable
// if you would like to test just issueElementArray, it would be better to use iceIssue
// if you would like to test generateUpdateList, it would be better to use icePkgList
// if you would like to test packagePreparation, it would be better to use icePackage
// if you would like to test makeUpdate and unwrapPackage, or want an example of how to create, 
//   fill, and update a collection from "scratch," use iceTest


// NOTE: the states should vary a tiny bit, due to the fact that they contain the current date, 
//   but the last digit should match (the last digit = issueId+1 )
// output: issue state ASHAMBLE2000-08-231
//         final IssueElementMark for issue 0 = 2
//         issue 0 is finished and has been added to 'issues'.
//         issue state ASHAMBLE2000-08-232
//         final IssueElementMark for issue 1 = 5
//         issue 1 is finished and has been added to 'issues'.
//         status of name00
//          in issue 0 is 1
//          in issue 1 is 3
//         status of name01
//          in issue 0 is 1
//          in issue 1 is 3
//         status of name02
//          in issue 0 is 1
//          in issue 1 is 3
//         status of name10
//          in issue 0 is 0
//          in issue 1 is 1
//         status of name11
//          in issue 0 is 0
//          in issue 1 is 1
//         status of name12
//          in issue 0 is 0
//          in issue 1 is 1
//         prev state = ASHAMBLE2000-08-231
//         new state = ASHAMBLE2000-08-231a
//         new issue: issue1
//         remove 0
//         remove 1
//         remove 2
//         add name10
//         prev state = ASHAMBLE2000-08-231a
//         new state = ASHAMBLE2000-08-231ab
//         add name11
//         prev state = ASHAMBLE2000-08-231ab
//         new state = ASHAMBLE2000-08-232
//         add name12
//         end issue: issue1
//         done
//         last arraylist resent


package com.adobe.rice.common.collections;
import com.adobe.rice.kernel.*;
import java.util.*;
import java.io.*;

public class iceUpdate
{
    public static void main( String[] args )
    { 
        try
        {
        CollectionManager clxnMgr = new CollectionManager();
        String clxnName = "bob";
        String subscId = "subscriptionId";
        Collection bob = clxnMgr.createCollection(clxnName);
        clxnMgr.setSubscriptionTable(subscId, bob);
        
        for (int is = 0; is<2; is++)
        {
            
            for ( int count = 0; count < 3; count++ )
            {
                String nm = "name"+is+count;
                String ref = "ref"+is+count;
                IssueElement iElement = new IssueElement(nm, ref );
                bob.setIssueElement(iElement);
                iElement.finishIssueElement();
             } // end for
        
            Issue jules = new Issue("issue"+is);
            bob.issueDone( jules ); 
            System.out.println("issue state "+jules.getState());
            System.out.println( "final IssueElementMark for issue "+bob.getCurrentIssue()+ " = " + bob.getIssueElementMark() );
            
            Issue inIssues = (Issue) bob.getIssue( bob.getCurrentIssue() );
            if ( inIssues.getFinished() == true )
                System.out.println( "issue " + inIssues.getId()+ " is finished and has been added to 'issues'." );        
        } // end for
        
        Issue currentIssue = bob.getIssue( bob.getCurrentIssue() );
        int numIssueElements = currentIssue.getElementCount();
        for (int index=0; index<=(numIssueElements); index++) 
        {
            IssueElement jen = bob.getIssueElement(index);
            System.out.println("status of " + jen.getName());
            for (int count = 0; count <= currentIssue.getId(); count++)
            {
                Issue issue = bob.getIssue( count );
                System.out.println("   in issue " + issue.getId() + " is " + issue.getElementStatus(index) );
            }
        } // end for
        
        // corr. to makeUpdate
        int issueId = currentIssue.getId() - 1; 
        Issue subIssue =  bob.getIssue(issueId);
        String subState =  subIssue.getState();
        
        // if want to test the situation where the subscriber is already updated, 
        // comment out the above three lines and instead use the following:
        // String subState = bob.getCurrentState();
        // this will cut the output off after (status of name12) "in issue 1 is 1"
        // it can be also tested for String subState = CollectionManager.initialState; 
        ArrayList packages = clxnMgr.makeUpdate(subscId, subState);
        if (packages == null)
        {
            System.out.println(" done - early exit");
            return; // a new package was not necessary
        }
        String testState = "";
        for(int count = 0; count<packages.size(); count++)
        {
            Package pkg = (Package)packages.get(count);
            System.out.println("prev state = "+pkg.getPreviousState());
            System.out.println("new state = "+pkg.getNewState());
            testState =  pkg.getPreviousState();
            for(int index =0; index<pkg.getPackageLength(); index++)
            {
                PackageElement elm = pkg.getPackageElement(index);
                int axn = elm.getAction();
                if (axn == 0)
                {
                    System.out.println("new issue: "+elm.getIssueName());
                }
                else if (axn == 1)
                {
                    System.out.print("add ");
                    IssueElement is = elm.getElement();
                    System.out.println(is.getName());
                }
                else if (axn == 2)
                {
                    System.out.println("remove "+elm.getId());
                }
                else if (axn == 3)
                {
                    System.out.println("end issue: "+elm.getIssueName());
                }
                else
                {
                    System.out.println("big problem "+axn);  
                }
            } // end for
        } // end for
        System.out.println( "done" );
        
        
        // testing the archive - testState is an intermediate state- not a whole number
        // if this goes correctly, the arraylist of packages that were supposed to be sent as the
        // last update will be checked and, since the states should match, that package should be 
        // tried again
        ArrayList pack = clxnMgr.makeUpdate(subscId, testState);
        if (pack == null)
            System.out.println("last arraylist resent");
        else
            System.out.println("testState must have been legal?!");
        return;
        }
        catch (DispatcherException e)
        {
            e.printStackTrace ();
        }   
    }
}