/*
$Author: andrewf $
$Log: EnumerationEnum.java,v $
Revision 1.6  2000/11/20 21:26:12  andrewf
some work on changing parameters

Revision 1.5  2000/11/13 18:36:58  andrewf
Rewrote EnumerationEnum and EnumerationParameter
to support ICE 1.2 - simplified.

Revision 1.4  2000/11/09 19:11:50  andrewf
Corrected errors in EnumerationParameter

Revision 1.3  2000/10/26 00:03:30  andrewf
added documentation and copyright

$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/negotiation/EnumerationEnum.java,v 1.6 2000/11/20 21:26:12 andrewf Exp $


ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

One or more classes in ICEcubes have been written by:

V. Bruce Hunt,                  Technical Lead & Vice Chair, ICE-AG, 
    vbhunt@sourceforge.net      Adobe Systems, Inc.
    
Jonathan Herbach                Developer           
    jherbach@adobe.com          Princeton University & Adobe Systems, Inc.
    
Will Conant                     Developer
                                University of Utah & Adobe Systems Inc.

Andrew Francis                  Developer           
    afrancis@adobe.com          Adobe Systems, Inc.
    
Anna Shamble                    Developer           
    ashamble@adobe.com          Santa Clara University & Adobe Systems, Inc.

Authors of various classes are noted in the headings 

/*

   EnumerationEnum is the class that supports section 4.5.4
   of the ICE Specification for.

   WARNING - API developers - are you sure that you want to
             instanciate, or override this class?
*/

package com.adobe.rice.common.negotiation;
import java.util.*;
import java.math.*;
import com.adobe.rice.util.OrderedSet;

public class EnumerationEnum extends EnumerationSet
{
    private OrderedSet    set;
    private Hashtable  table;
    private int selection = -1;
    
   
	public EnumerationEnum (Collection newSet,
                            int        selection)                                                  
    {
       setSet (newSet);
	   setSelection (selection);
    }
	
	
	/**************** getter/setter functions ***************/
	private void setTable (Hashtable newTable)
	{
	    table = newTable;
	}
	
	
	private Hashtable getTable ()
	{
	    return table;
	}
	
	
	private Set getSet ()
    {
        return set;
    }
    
    private void setSet (Collection newSet)
    {
        set = new OrderedSet (newSet);   
    }    
    
    private void setSelection (int newSelection)
    {
        selection = newSelection;
    }
    
    public int getSelection ()
    {
        return selection;
    }
	
    /**************** private methods ****************/
	
	private boolean isValidSelection (Set defaults,
                                      int selection)
    {
       boolean result = false;
       int count = defaults.size ();
       
       if (selection > 0)
       {
          result = (selection == count) ? true : false;
       }
       else if (selection == Setiable.zero_or_more)
       {
          result = (count >= 0) ? true : false; 
       }
       else if (selection == Setiable.zero_or_one)
       {
          result = (count == 0 || count == 1) ? true : false;
       }
       else if (selection == Setiable.one_or_more)
       {
          result = (count >= 1) ? true : false;
       }
       else 
       {
          // perhaps an exception should be thrown 
          result = false;
       }   
       return result;
    }
	   
    /******************** public methods *******************/
    public boolean equals (Object operand)
    {
        return (operand instanceof EnumerationEnum &&
                getSelection () == ((EnumerationEnum) operand).getSelection () &&
                getSet ().equals (((EnumerationEnum) operand).getSet ()));
    }
    
    public int rangeSize () 
    {
	   return (int) set.size ();
    }
 
    // the need to support the ordinal values make
    // Enumeration's in () more sophisticated
    // FIX re-visit this
    //
    public boolean in (Set values) 
    { 
       boolean                 result = false;
       int                     count;
       int                     selection;
       String                  symbol;
       boolean                 temp;
       
       /* first, check the count */
       result = isValidSelection (values, getSelection ());
     
       if (result == true)
       {
          result = getSet ().containsAll (values);
       }
             
       return result;
    }
    
    // re-visit
    public String next (String value) 
    {
	   throw new UnImplementedException ();
    }
   
    // re-visit
    public String previous (String value) 
    {
       throw new UnImplementedException ();
    }
    
    public String minimum ()
    {
        return (String) set.first();
    }
    
    public String maximum ()
    {
        return (String) set.last();
    }

    public String toString() 
    {	
        return getSet ().toString () +  getSelection ();
    } 
    
    /* ensure we haven't busted anything */
    public static void main (String argv[])
    {
        EnumerationEnum a;
        EnumerationEnum x;
        EnumerationEnum y;
        EnumerationEnum z;
        
        boolean         result;
        int   n;
        
	    Object list1 [] = {"a","b","c","d","e"};
	    Object list2 [] = {"a","b","c"};
	    Object list3 [] = {"e","f","g"};
	    
	    OrderedSet foo;
	    OrderedSet bar;
	    OrderedSet foobar;
	  
	    
	    LinkedList f = new LinkedList ();
	    LinkedList g = new LinkedList ();
	    LinkedList h = new LinkedList ();
	    
	    for (n = 0 ; n < list1.length  ; n++)
	    {
	        f.add (list1 [n]);
	    }
	    
	    for (n = 0 ; n < list2.length  ; n++)
	    {
	        g.add (list2 [n]);
	    }
	    
	    for (n = 0 ; n < list3.length ; n++)
	    {
	        h.add (list3 [n]);
	    }
	    
	    foo = new OrderedSet (f); 
	    bar = new OrderedSet (g);
	    foobar = new OrderedSet (h);
	    
	    
	    a = new EnumerationEnum (foo, Setiable.one_or_more);
	    x = new EnumerationEnum (foo, Setiable.one_or_more);
	    y = new EnumerationEnum (bar, Setiable.one_or_more);
	    z = new EnumerationEnum (bar, 3);
	    
	    
	    result = a.in (bar);
	    System.out.print ("a.in (bar) should be true ");
	    System.out.println ((result == true) ? "[PASSED]" : "[FAILED]");
	                         
	    result = a.in (foo);
	    System.out.print ("a.in (foo) should be true ");
	    System.out.println ((result == true) ? "[PASSED]" : "[FAILED]");
	    
	    result = a.in (foobar);
	    System.out.print ("a.in (foobar) should be false ");
	    System.out.println ((result == false) ? "[PASSED]" : "[FAILED]");
    
        result = a.equals (a);
        System.out.print ("a.equals (a) should be true ");
        System.out.println ((result == true) ? "[PASSED]" : "[FAILED]");
    
        result = a.equals (x);
        System.out.print ("a.equals (x) should be true ");
        System.out.println ((result == true) ? "[PASSED]" : "[FAILED]");
    
        result = x.equals (a);
        System.out.print ("x.equals (a) should be true ");
        System.out.println ((result == true) ? "[PASSED]" : "[FAILED]");
    
        result = x.equals (y);
        System.out.print ("x.equals (y) should be false ");
        System.out.println ((result == false) ? "[PASSED]" : "[FAILED]"); 
       
        result = y.equals (x);
        System.out.print ("y.equals (x) should be false ");
        System.out.println ((result == false) ? "[PASSED]" : "[FAILED]"); 
  
        result = y.equals (z);
        System.out.print ("y.equals (z) should be false ");
        System.out.println ((result == false) ? "[PASSED]" : "[FAILED]"); 
    }
}    
    