/*
$Log: EnumerationParameter.java,v $
Revision 1.6  2000/11/20 21:26:12  andrewf
some work on changing parameters

Revision 1.5  2000/11/13 18:36:58  andrewf
Rewrote EnumerationEnum and EnumerationParameter
to support ICE 1.2 - simplified.

Revision 1.4  2000/11/09 19:11:50  andrewf
Corrected errors in EnumerationParameter

Revision 1.3  2000/10/26 00:03:30  andrewf
added documentation and copyright

$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/negotiation/EnumerationParameter.java,v 1.6 2000/11/20 21:26:12 andrewf Exp $
$Author: andrewf $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org
*/


package com.adobe.rice.common.negotiation;
import  java.util.*;
import  com.adobe.rice.util.OrderedSet;

//EnumerationParameter implements section 4.5.1
//of the ICE Specification
//
//Note API developers do not instanciate this
//class directly - the ICE payloadDisassembler does
//
//<dl>
//<dt> boolean in (String value) </dt> 
//<dd> is this string a member of the Enumeration's set?
//      example: 
//      weekdays = {"monday","tuesday","wednesday","sunday"}
//      in ("friday") is false </dd>
//
//<dt> boolean in (Set subset) </dt>
//<dd> is this set a subset of the Enumeration's set </dd>
// 
//<dt> String maximum () </dt>
//<dd> return the symbolic value with the highest ordinal value</dd>
//
//<dt> String minimum () </dt>
//<dd> return the symbolic value with the lowest ordinal value </dd>
//
//<dt> String previous (String value)
//<dd> return the value in the enumeration previous to value.
//     Similar to Pascal's ______ function </dd>
//
//<dt> String next (String value)
//<dd> return the value in the enumeration after value.
//     Similar to Pascal's succ () function </dd>
//</dl>

public class EnumerationParameter extends Parameter
{   
    private boolean hasDefaults = false;
    
    // selector 
    private int selection;
    
    // defaults
    private Set defaultValues;
    
    // range
    private EnumerationEnum range;
    
    
    public EnumerationParameter (String         name, 
	                             String         id, 
	                             int            status,
	                             Set            range,
	                             Set            defaultValues,
	                             boolean        hasDefaults,
	                             int            setSelection,
	                             NegotiationStrategy icePolicy,
	                             Application iceApplication) 
	                             throws OutOfRange,
	                                    CloneNotSupportedException,
	                                    InternalError
	            
	{
	    // initialize the underlying Parameter
	    super(name, id, status, icePolicy, iceApplication);
	      
	    // ICE compiler should be not passing null 
	    // range sets
		if (range == null)
		{
		   throw new InternalError ();   
		}
		
		if (hasDefaults && defaultValues == null)
		{
		   throw new InternalError ();
		}
		
		if (hasDefaults && !isValidDefault (range, defaultValues, 
		                                    setSelection))
		{
		   throw new InternalError ();   
		}
		
		setRange (new EnumerationEnum (range, setSelection));
		
		// create an empty set anyway
		if (!hasDefaults)
		{
		   defaultValues = new OrderedSet ();
		}
		
		setDefaultValues (defaultValues);
		setHasDefaultValues (hasDefaults);
		setSelection (setSelection);
	}


    // Special constructor for payloadDisassembler
    public EnumerationParameter (String         name, 
	                             String         id, 
	                             int            status,
	                             EnumerationSet defaultValues,
	                             IceSet         rangeValues)
	                             throws OutOfRange, InternalError
	{
	    super (name, id, status, null, null);
	}  
    

    /**************** getter/setter methods *****************/
    
	private void setSelection (int selection)
	{
	   this.selection = selection;   
	}
		
	public int getSelection ()
	{
	   return selection;
	}
	
	protected void setHasDefaultValues (boolean hasDefaults)
    {
        this.hasDefaults = hasDefaults;
    }
    
    public boolean hasDefaultValues ()
    {
        return hasDefaults;
    }
    
    //getDefaultValues and setDefaultValues are required for 
    //accepting IceParameters as per section 4.5.x of ICE 
    //specification
    
    public Set getDefaultValues ()
    {
        return defaultValues;   
    }

    public void setDefaultValues (Set defaultValues)
    {
        // perhaps I should change this to a clone?
        this.defaultValues = defaultValues;    
    }
    
    public void setRange (IceSet range)
    {
        this.range = (EnumerationEnum) range;
    }
    
    public IceSet getRange ()
    {
        return range;
    }
     
    // hasEquivalentValues determines if two IceParameters
    // have equivalent ice-default-values
    public boolean hasEquivalentValues (Parameter sender)
    {
        return (getDefaultValues ().equals (sender.getDefaultValues ()));
    }
    
    // are sender's default values within receiver's range 
    // of values?
    public boolean in (Parameter sender)
    {
        return getRange ().in (sender.getDefaultValues ());
    }
 
    // return the size of the range set
    public int rangeSize ()
    {
        return getRange ().rangeSize ();
    }
 
    // return the size of the defaultValues
    public int defaultValueSize ()
    {
        return getDefaultValues ().size ();   
    }
    
    /******************* "API" methods *****************/
    
    // is this value in the Enumeration's set of values
    public boolean in (String value)
    {
        TreeSet list;
        // wrap the value in set
        list = new TreeSet ();
        list.add (value);
        return getRange ().in (list);
    }
    
    // is this a subset of the Enumeration's set?
    public boolean in (Set values)
    {
        return getRange ().in (values);
    }
    
    // what is minimum range value
    public String minimum ()
    {
        return (String)((EnumerationEnum) getRange ()). minimum ();
    }
    
    // what is maximum range value
    public String maximum ()
    {
        return (String)((EnumerationEnum) getRange ()).maximum ();
    }
    
    public String previous (String value)
    {
        return (String)((EnumerationEnum) getRange ()).previous (value);
    }
    
    public String next (String value)
    {
        return (String)((EnumerationEnum) getRange ()).next (value);
    }
	
    // I have to think about this one
	public String toString ()
	{
	    return super.toString () + "range: " + 
	           getRange ().toString () + "defaults: " + 
	           getDefaultValues ().toString ();    
	}
}

