/*
$Log: IceNegotiable.java,v $
Revision 1.4  2000/11/22 00:46:33  andrewf
Changed IceNegotiable to Negotiable

Revision 1.3  2000/10/26 00:03:30  andrewf
added documentation and copyright

$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/negotiation/IceNegotiable.java,v 1.4 2000/11/22 00:46:33 andrewf Exp $
$Author: andrewf $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org
*/

//  IceNegotiable is the abstract class for all Negotiable classes
  
package com.adobe.rice.common.negotiation;

public abstract class IceNegotiable implements Negotiable
{
    final static String [] s = {"UNKNOWN",
                                "ACCEPT",
                                "NEGOTIATE",
                                "REQUIRE",
                                "REJECT"};
                                
    private int             status;
    private String          id;
    private String          name;
    private boolean         negotiability;
    private int             errorStatus;
    private boolean         isErred;
    private Application     application;
    private IceSession      session;
    
    public IceNegotiable (String name, 
                          String id, 
                          int status) 
    {
        boolean result;
        setName (name);
        setId (id);
        setStatus (status);
        
        result = (status == IceNegotiable.REQUIRE) ? false : true;
        setNegotiability (result);
        setErrorStatus (0);
        setIsErred (false);
    }
    
    public IceNegotiable (String         name, 
                          String         id, 
                          int            status,
                          IceSession     session,
                          Application application) 
    {
        boolean result;
        setName (name);
        setId (id);
        setStatus (status);
        
        result = (status == IceNegotiable.REQUIRE) ? false : true;
        setNegotiability (result);
        setErrorStatus (0);
        setIsErred (false);
        setSession (session);
        setApplication (application);
    }
    
    private void setSession (IceSession newSession)
    {
        session = newSession;
    }
    
    private IceSession getSession ()
    {
        return session;   
    }
    
    private Application getApplication ()
    {
        return application;
    }
    
    private void setApplication (Application newApplication)
    {
        application = newApplication;    
    }
    
    public String getName ()
    {
        return name;
    }

    private void setName (String newName)
    {
        name = newName;   
    }
  
    public boolean getIsErred ()
    {
        return isErred;
    }

    private void setIsErred (boolean flag)
    {
        isErred = flag;
    }
    
    public String getId ()
    {
        return id;
    }
    
    private void setId (String newId)
    {
        id = newId;
    }
    
    public int getStatus ()
    {
        return status;
    }
    
    protected void setErrorStatus (int status)
    {
        errorStatus = status;    
    }
    
    protected void setStatus (int newStatus)
    {
        status = newStatus;
    }
    
    
    public boolean getNegotiability ()
    {
        return negotiability;
    }
    
    private void setNegotiability (boolean newNegotiability)
    {
        negotiability = newNegotiability;
    }
       
    public int getErrorStatus ()
    {
        return errorStatus;
    }
        
    public boolean isNegotiable ()
    {
        return getNegotiability ();
    }
  
    // review
    public void accepted ()
    {
        setStatus (IceNegotiable.ACCEPT);
    }
  
    public void accept (Negotiable sender) 
                throws InternalError
    {
        setStatus (IceNegotiable.ACCEPT); 
        setErrorStatus (IceNegotiable.OK);
    }
    
    public boolean isAcceptable (Negotiable sender)
    {
        return false;
    }
  
    public boolean isAccepted ()
    {
       return (status == IceNegotiable.ACCEPT);
    }
    
    public boolean isRejected ()
    {
        return (status == IceNegotiable.REJECT);
    }
    
    public void reject (int status)
    {
        setIsErred (true);
        setErrorStatus (status);
        setStatus (IceNegotiable.REJECT);
    }
    
    public boolean isRequired ()
    {
        return (status == IceNegotiable.REQUIRE);
    }

     
    public String toString ()
    {
        return getName () + " " + getId () + 
               " " + s[getStatus ()] + " " + 
               (negotiability ? "Negotiable" : "Non-negotiable") + " ";                                 
    }
    
    public boolean isError ()
    {
        return getIsErred ();
    }
    
    public Object Application (String key)
    {
        return application.get (key);
    }
    
    // should be defined here
    public Object Session (String key)
    {
        return session.get (key);   
    }
}