/*
$Log: NegotiationManager.java,v $
Revision 1.1  2000/11/09 19:12:39  andrewf
Negotiation framework classes

Revision 1.2  2000/10/26 00:03:30  andrewf
added documentation and copyright

$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/negotiation/NegotiationManager.java,v 1.1 2000/11/09 19:12:39 andrewf Exp $
$Author: andrewf $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org
*/


package com.adobe.rice.common.negotiation;
import com.adobe.rice.kernel.*;

public class NegotiationManager extends IceCubeManager
{
    private static NegotiationManager negotiator;
    private static NegotiationProxy   proxy;
    
    private boolean debugFlag = false;
    
    // FIX - re-examine this strategy
    static
    {
        try
        {
            proxy = new NegotiationProxy ();
            negotiator = new NegotiationManager ();
        }
        catch (DispatcherException de)
        {
            de.printStackTrace ();
        }
    }
        
    /*
       NOTE - Visual Cafe seems to not allow inner classes
       to access protected methods
    */
    class NegotiationRunner extends IceRunner
    {
        private IceMessage message;
        private IceMessage response;
        
        public NegotiationRunner (IceMessage message) 
                                  throws DispatcherException
        {
           super (Thread.currentThread ().getName (),
                  MessageDispatcher.instance ());
            
           this.message = message;      
        }
        
        public void run ()
        {
           IceMessage reply = null;
           
           switch (message.getAction ())
           {
              case com.adobe.rice.kernel.Constants.iceOffer: 
              if (debugFlag)
              {  
                 String i = "ice-negotiation request received <br>";
                 System.out.println (i);
                 reply = new IceMessage (message, i);
              }
              
              case com.adobe.rice.kernel.Constants.iceChangeSubscription:
              if (debugFlag)
              {  
                 String i = "ice-negotiation request received <br>";
                 System.out.println (i);
                 reply = new IceMessage (message, i);
              }
              
              break;
               
              default:
              if (debugFlag)
              {
                 String i = "unknown request received <br>";
                 System.out.println (i);
                 reply = new IceMessage (message, i);
              } 
           }
           
           reply (reply);
        }
    }
  
    /*************** body of the SubscriptionManager *********/
  
    private IceMessage isAuthorized (IceMessage message)
    {
        IceMessage reply = new IceMessage ("authorized");
     
        return reply;
    }
  
    private IceMessage changeSubscription (IceMessage message)
    {
        IceMessage reply = new IceMessage ("received changeSubscription");
        return reply;
    }
   
    private IceMessage newSubscription (IceMessage message)
    {
        IceMessage reply = new IceMessage (message, "received newSubscription");
        
        return reply;
    }
    
    private IceMessage getSubscription (IceMessage message)
    {
        IceMessage reply = new IceMessage (message, "get subscription");
       
        return reply;
    }
        
    /**************** constructor *****************************/
   
    public static Negotiator instance ()
    {
        return proxy;
    }
   
   
	protected NegotiationManager () throws DispatcherException
	{
		super ("negotiation-manager", 
		        MessageDispatcher.instance ());
	}
	
    public void finalize()
    {
        // This method is derived from class com.adobe.rice.kernel.IceCubeManager
        // to do: code goes here
    }

    public void initialize()
    {
        // This method is derived from interface com.adobe.rice.common.Manager
        // to do: code goes here
    }

    public void postFinalize()
    {
        // This method is derived from interface com.adobe.rice.common.Manager
        // to do: code goes here
    }

    public void postInitialize()
    {
        // This method is derived from interface com.adobe.rice.common.Manager
        // to do: code goes here
    }

    public void preFinalize()
    {
        // This method is derived from interface com.adobe.rice.common.Manager
        // to do: code goes here
    }

    public void preInitialize()
    {
        // This method is derived from interface com.adobe.rice.common.Manager
        // to do: code goes here
    }

    // for experimental purposes, we will start with an
    // iterative Subscription manager
    
    public void run()
    {
       IceMessage request;
       IceMessage reply = null;
       Runnable   runner;
       
       // have to come up with an efficient method
       // shutting down the manager
       
       while (true)
       {
           try
           {
              request = (IceMessage) receive ();
              runner = new NegotiationRunner (request);
              new Thread (null, runner, getName ()).start ();
           }
           catch (DispatcherException e)
           {
              e.printStackTrace ();
           }
       }
    }   
}