/*
$Log: NumericSpan.java,v $
Revision 1.3  2000/11/20 21:26:12  andrewf
some work on changing parameters

Revision 1.2  2000/10/26 00:03:31  andrewf
added documentation and copyright

$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/negotiation/NumericSpan.java,v 1.3 2000/11/20 21:26:12 andrewf Exp $
$Author: andrewf $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org
*/


package com.adobe.rice.common.negotiation;
import java.util.*;

     
// NumericSpan implements the ice-span of section 4.5.3
// It is assumed that the Ice "compiler" has determined 
// what the precision is. For now, doubles are used by 
// default.

public class NumericSpan extends NumericSet
{
	private double minimum;
	private double maximum;
	private double precision;
      
    public NumericSpan (double minimum, 
                        double maximum, 
                        double precision) 
    {
	   this.minimum =  minimum;
	   this.maximum =  maximum;
	   this.precision = precision;
    }
   
    /**************** public methods ********************/
    public boolean isValid (IceSet operand)
    {
        NumericSpan o = (NumericSpan) operand;
        return (precision == o.precision &&
                maximum >= o.maximum &&
                minimum <= o.minimum);
    }
     
    public boolean equals (Object operand)
    {
        return (operand instanceof NumericSpan && 
                maximum ().equals (((NumericSpan) operand).maximum ()) &&
                minimum ().equals (((NumericSpan) operand).minimum ()) &&
                (precision == ((NumericSpan) operand).precision));
    }
    
    
    // returns the size of the range
    public int rangeSize ()
    {
	   return (int) ((maximum - minimum)  / precision + 1);
    }
                 
    public boolean in (double value) 
    { 
	   return (value >= minimum && value <= maximum) 
	           ? true : false;
    }

    // this is somewhat of a hack 
    public boolean in (Set values)
    {
       double   value;
       boolean  result = false;
       
       if (values.size () == 1)
       {
          Iterator i;
          i = values.iterator ();
          value = ((Double) i.next ()).doubleValue ();
          result = in (value);
       }
       
       return result;
    }

    public Double maximum() 
    {
 	    return new Double (maximum);
    }

    public Double minimum () 
    {
        return new Double (minimum);
    }
    
    // The next () method returns the next number in the numeric 
    // span set if the number does not exist, NaN is returned
    public Double next (double value) 
    {
	   double newValue;

	   newValue = value + precision;
	   newValue = newValue > maximum ? Double.NaN : newValue;
	   return new Double (newValue);
    }
  
    // The previous () method returns the next number in the 
    // numeric span set if the number does not exist, NaN is 
    // returned
    public Double previous (double value) 
    {
       double newValue;
	
	   newValue = value - precision;
	   newValue = newValue < minimum ? Double.NaN : newValue;
	   return new Double (newValue);
    }
  
    public String toString() 
    {	
	    return "[" + Double.toString (minimum) + ' ' +
	           Double.toString (maximum) + ' ' + ' ' +
	           Double.toString (precision) + "]";
    }
}
