/*
$Log: Parameter.java,v $
Revision 1.5  2000/11/22 00:46:33  andrewf
Changed IceNegotiable to Negotiable

Revision 1.4  2000/11/20 21:26:12  andrewf
some work on changing parameters

Revision 1.3  2000/10/26 00:03:31  andrewf
added documentation and copyright

$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/negotiation/Parameter.java,v 1.5 2000/11/22 00:46:33 andrewf Exp $
$Author: andrewf $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org
*/


package com.adobe.rice.common.negotiation;
import java.util.*;

public abstract class Parameter extends IceNegotiable 
                                implements Setiable
{  
    // Parameters use an icePolicy 
    private NegotiationStrategy icePolicy;
    private Application         application;
    
    /* November 15th additions */
      
    private boolean             isOperational = false;
    private boolean             isExtension   = false;
	
    public Parameter (String name, 
                      String id, 
                      int status,
                      NegotiationStrategy icePolicy,
                      Application         application)
                      throws InternalError
	{   
		super (name, id, status);
		
		// neither name, application, or policy can be null
		if (application == null || name == null || icePolicy == null)
		{
		   throw new InternalError ();
		}   
		
		// set the policy
		setStrategy (icePolicy);
  
        // and the handle to application
	    this.application = application;
	}

    /******************* protected methods ******************/
    
    // isValidSelection is a utility method for enforcing
    // select meanings defined in section 4.5.4 of ICE 1.1
    // specification
    final protected boolean isValidSelection (Set defaults,
                                              int selection)
    {
       boolean result = false;
       int count = defaults.size ();
       
       if (selection > 0)
       {
          result = (selection == count) ? true : false;
       }
       else if (selection == Setiable.zero_or_more)
       {
          result = (count >= 0) ? true : false; 
       }
       else if (selection == Setiable.zero_or_one)
       {
          result = (count == 0 || count == 1) ? true : false;
       }
       else if (selection == Setiable.one_or_more)
       {
          result = (count >= 1) ? true : false;
       }
       else 
       {
          // perhaps an exception should be thrown 
          result = false;
       }   
       return result;
    }
    
    
    // isValidDefault is a utility method used to enforce
    // 4.4.5 negotiation rules.
    final protected boolean isValidDefault (Set rangeValues, 
                                            Set defaultValues, 
                                            int selection)
	{
	   
       // value is acceptable if the selection is correct
       // and default is a subset of the range
	   return (isValidSelection (defaultValues, selection)
	           && rangeValues.containsAll (defaultValues));
    }
    
    
    protected NegotiationStrategy getStrategy ()
    {
        return icePolicy;
    }
    
    protected void setStrategy (NegotiationStrategy newIcePolicy)
    {
        icePolicy = newIcePolicy;
    }
    
    protected abstract boolean hasDefaultValues ();
    protected abstract void    setHasDefaultValues (boolean flag);
    
    /******************* public methods *******************/
    public abstract Set     getDefaultValues ();
    public abstract void    setDefaultValues (Set defaultValues);
    public abstract IceSet  getRange ();
    public abstract void    setRange (IceSet newRange);
    public abstract int     rangeSize ();
    public abstract boolean in (Set values);
      
    public boolean isValid (Parameter sender)
    {
        return getRange ().isValid (sender.getRange ());
    }
      
    // set is the assignment operator
    // it is expected that subclasses will add additional
    // checks before calling this (super) class
    public void set (Parameter parameter)
    {
        setStatus (parameter.getStatus ());
        setRange (parameter.getRange ());
        setDefaultValues (parameter.getDefaultValues ());
        setHasDefaultValues (parameter.hasDefaultValues());
    }
    
    //IceParameters are value equivalent if all of their 
    //non-identifier components are equal.
    public  boolean equals (Object sender)
    {
        return  (getName ().equals (((Parameter) sender).getName ()) &&
                 getDefaultValues ().equals (((Parameter) sender).getDefaultValues ()) &&
                 getRange ().equals (((Parameter) sender).getRange ())  &&
                 getStatus () == ((Parameter)sender).getStatus ());
    }
    
    public String toString ()
    {
       return super.toString ();
    }
    
    // accept IceParameter     
    public void accept (Negotiable sender) 
                throws InternalError
    {   
        // change errorCode to OK and status to ACCEPTED
        super.accept (sender);
        
        // and use the defaultValue of the sender
        setDefaultValues (((Parameter) sender).getDefaultValues ());
    }
    
    //  IceParameter uses two design patterns: Strategy
    //  and Template. IcePolicies are strategies, that
    //  is, classes that encapsulate negotiation algorithms
    //  In turn, negotiate (), isAcceptable (), and propose ()
    //  are user defined methods used in Offer's template
    //  pattern. 
    
    // is this negotiable acceptable?
    public boolean isAcceptable (Negotiable sender)
    {
        return getStrategy ().isAcceptable (sender, this);
    }
            
    // initiate a negotiation based on the sender        
    public Negotiable negotiate (Negotiable sender) throws
                      Sorry, OutOfRange, InternalError
    {
        Negotiable reply = null;
        
        try
        {
            reply = getStrategy ().negotiate (sender, this);
        } 
        catch (InternalError e)
        {
            // let a higher level handle this
            throw e;
        }
        
        return reply;
    }
    
    // propose a counter negotiable based on the sender
    public Negotiable propose (Negotiable sender)
                               throws InternalError 
    {
        return getStrategy ().propose (this, sender);
    }
    
    /* November 15 extension */
    public boolean isOperational ()
    {
        return isOperational;
    }
    
    public boolean isExtension ()
    {
        return isExtension;
    }
}

