/*
$Log: Resolver.java,v $
Revision 1.3  2000/11/22 00:46:33  andrewf
Changed IceNegotiable to Negotiable

Revision 1.2  2000/10/26 00:03:31  andrewf
added documentation and copyright

$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/negotiation/Resolver.java,v 1.3 2000/11/22 00:46:33 andrewf Exp $
$Author: andrewf $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org
*/


package com.adobe.rice.common.negotiation;
import java.util.*;

public class Resolver 
{
    private Hashtable   negotiableTable;   // table of all negotiables
    private Hashtable   unresolvedTable;   // unresolved lookup table
    private Hashtable   resolvedTable;     // resolved lookup table
    private LinkedList  resolved;          // ordered list of resolved negotiables
    private LinkedList  unresolved;        // ordered list of unresolved
    private Iterator    i;
    private boolean     debugFlag = false;
    private boolean     hasRequired = true;
    
    private void initialize ()
    {
       negotiableTable = new Hashtable ();
       unresolvedTable = new Hashtable ();
       resolvedTable = new Hashtable ();
       resolved = new LinkedList ();
       unresolved = new LinkedList ();
    }
    
    
    public Resolver ()
    {
        initialize ();
    }
    
    
    public Resolver  (Collection initialCollection)
    {
       initialize ();
        
       Iterator iterator = initialCollection.iterator ();   
       IceNegotiable element;
       
       while (iterator.hasNext ())
       {
          element = (IceNegotiable) iterator.next ();
          
          // put in master table
          negotiableTable.put (element.getName (), element); 
          
          if (element.isAccepted ())
          {
             resolvedTable.put (element.getName (), element);
             resolved.addLast (element);
          }
          else
          {
             unresolvedTable.put (element.getName (), element);
             unresolved.addLast (element);        
          }
                  
          if (element.isRequired ())
          {
             // if one non-negotiable is detected, the
             // offer is non-negotiable at that time
             hasRequired = true; 
          }
       }
    }
    
    
    //september 22nd change
    public void initialiseResolvedList ()
    {
        i = resolved.iterator (); 
    }
    
    // pre-condition List is initialized
    public boolean hasResolvedNegotiables ()
    {
        return i.hasNext ();
    }
    
    public Negotiable nextResolvedNegotiable ()
    {
        return (Negotiable) i.next ();   
    }
 
    public Negotiable getResolvedNegotiable (String name)
    {
        return (Negotiable) resolvedTable.get (name);   
    }
    
    public boolean hasUnresolvedNegotiables ()
    {
        return !(unresolved.isEmpty ());
    }
    
    // unresolved lists should get smaller
    public Negotiable nextUnresolvedNegotiable ()
    {
        IceNegotiable element;
        
        element = (IceNegotiable) unresolved.removeFirst ();
        unresolvedTable.remove (element.getName ());
        return element;   
    }
 
    public Negotiable getUnresolvedNegotiable (String name)
    {
        return (Negotiable) unresolvedTable.get (name);   
    }
 
    
    public Negotiable removeUnresolvedNegotiable (String key)
    {
        Negotiable element;
        boolean result;
        
        element = (Negotiable) unresolvedTable.remove (key);
        if (element != null)
        {
            result = unresolved.remove (element);
        }
        
        return element;
    }
 
    public void PushUnresolved (Negotiable element)
    {
        unresolved.addFirst (element);
        unresolvedTable.put (element.getName (), element);
    }
 
    //
    // September 24, rethink what this returns, the boolean
    // tells the programmer nothing about what went wrong
    //
    public boolean addResolvedNegotiable (Negotiable element)
    {
        boolean result;
       
        // add to the list
        result = resolved.add (element);
        if (result)
        {
           // and to the table
           result = (resolvedTable.put (element.getName (), element) != null)
                     ? true : false;
        
           // and remove from the unresolved list and table
           unresolved.remove (element);
           unresolvedTable.remove (element.getName ());
        }
        
        return result;
    }
    
    
//    public Negotiable removeResolvedNegotiable (String key)
//    {
//        return resolved.remove (key);
//    }
    
    public Negotiable getNegotiable (String name)
    {
        return (Negotiable) negotiableTable.get (name);
    }
    
    public boolean hasRequiredNegotiables ()
    {
        return hasRequired;   
    }
    
    //for now return negotiable
    public int getStatus ()
    {
        return IceNegotiable.NEGOTIATE;
    }
    
    //August 12th addition, need to get the focus
    //of negotiations - corresponds to section 
    public Negotiable getFocus ()
    {
        return (Negotiable) unresolved.get (0); 
    }
    
    // for debugging/testing purposes
    public String toString ()
    {
        return "resolved:" + resolved.toString() + "\n" +
               "unresolved: " + unresolved.toString () + "<-\n";
    }
    
    //for debugging
    public void printFocus ()
    {
        System.out.println (getFocus ());
    }
    
    public void debug ()
    {
       System.out.println (resolved);
       System.out.println (unresolved);
    }
    
    public static void main (String argv[])
    {
        
        
    }
}