/*
$Log: TimeSpan.java,v $
Revision 1.3  2000/11/20 21:26:12  andrewf
some work on changing parameters

Revision 1.2  2000/10/26 00:03:32  andrewf
added documentation and copyright

$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/negotiation/TimeSpan.java,v 1.3 2000/11/20 21:26:12 andrewf Exp $
$Author: andrewf $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org
*/


package com.adobe.rice.common.negotiation;
import java.util.*;

     
// TimeSpan implements the ice-span of section 4.5.2
// of the ICE specification. In the content of Time,
// precision refers to the ICE Format used, IceTime,
// IceDate, IceDateTime.

public class TimeSpan extends TimeSet
{
	private Date minimum;
	private Date maximum;
	private long precision;
      
    public TimeSpan (Date minimum, 
                     Date maximum, 
                     long precision) 
    {
	   this.minimum =  minimum;
	   this.maximum =  maximum;
	   this.precision = precision;
    }
   
    /**************** public methods ********************/
    public boolean equals (Object operand)
    {
        return (operand instanceof TimeSpan && 
                minimum.equals (((TimeSpan) operand).minimum) &&
                maximum.equals (((TimeSpan) operand).maximum) &&
                precision == ((TimeSpan) operand).precision);
    }
    
    // returns the size of the range
    public int rangeSize ()
    {
	   return (int) ((maximum.getTime () - minimum.getTime ())  
	                 / precision + 1);
    }
                 
    public boolean in (Date value) 
    { 
	   return (value.compareTo (minimum) >= 0
	           && value.compareTo (maximum) <= 0) 
	           ? true : false;
    }

    // this is somewhat of a hack 
    public boolean in (Set values)
    {
       Date     value;
       boolean  result = false;
       
       if (values.size () == 1)
       {
          Iterator i;
          i = values.iterator ();
          value = (Date) i.next ();
          result = in (value);
       }
       
       return result;
    }

    public Date maximum() 
    {
 	    return maximum;
    }

    public Date minimum () 
    {
        return minimum;
    }
    
    // The next () method returns the next number in the numeric 
    // span set if the number does not exist, NaN is returned
    public Date next (Date value) 
    {
       Date newValue;
       
       newValue = (Date) value.clone ();
       newValue.setTime (precision);
	   newValue = newValue.after (maximum) 
	              ? newValue : new Date (0);
	   
	   return newValue;
    }
  
    // The previous () method returns the next number in the 
    // Time span set if the number does not exist, perhaps
    // epoch should be returned or something that indicates
    // an error 
    public Date previous (Date value) 
    {
       Date newValue;
      
       newValue = (Date) value.clone ();
       newValue.setTime (-precision);
	   newValue = newValue.before (minimum) 
	              ? newValue : new Date (0);
	   
	   return newValue;
    }
  
    public String toString() 
    {	
	    return minimum.toString () + ' ' +
	           maximum.toString () + ' ' +
	           precision;
    }
}
