package com.adobe.rice.common.negotiation.tests;
import com.adobe.rice.common.negotiation.*;
import java.util.*;

public class EnumerationParameterTest
{
    public static void main (String argv[])
    {
        test2 ();  
    }
 
    static private TreeSet constructSymbolSet (String string)
    {
        TreeSet set = new TreeSet ();
        StringTokenizer tokens = new StringTokenizer (string);
        String element;
       
        
        while (tokens.hasMoreTokens ())
        {
            element = tokens.nextToken ();
            set.add (element);
        }
          
        return set;
    } 
    
    public static boolean test1 ()
    {
       boolean result = false;
       EnumerationParameter span;
       EnumerationParameter noDefaults;
       EnumerationParameter enum;
       String  rangeList = "monday tuesday wednesday thursday friday saturday sunday";
       String  defaultList = "monday wednesday friday sunday";
       TreeSet ranges;
       TreeSet defaults;
       
       // Parameters does not allow null values
       SampleIceApplication bar = new SampleIceApplication ();
       EchoPolicy foo = new EchoPolicy ();
        
       try
       {
          
          span = new EnumerationParameter ("test", 
                                           "P0", 
                                           Negotiable.NEGOTIATE, 
                                           constructSymbolSet (rangeList), 
                                           constructSymbolSet (defaultList), 
                                           true,
                                           Setiable.one_or_more, 
                                           foo, bar);
	              
          System.out.println (span);   
       } 
       catch (com.adobe.rice.common.negotiation.InternalError e)
       {
          System.out.println ("Caught an Internal Error ");
       }
       catch (OutOfRange e)
       {
          System.out.println ("Caught an OutOfRange Error");
       }
       catch (CloneNotSupportedException e)
       {
          System.out.println ("Caught CloneNotSupportedException e");
       }
       return result;
    }
   
    
    static public boolean test2 ()
    {
       boolean result = false;
       EnumerationParameter parameter1;
       EnumerationParameter parameter2;
       EnumerationParameter parameter3;
       EnumerationParameter parameter4;
       EnumerationParameter parameter5;
       EnumerationParameter parameter6;
       EnumerationParameter parameter7;
       EnumerationParameter parameter8;
       EnumerationParameter parameter9;
      
       String range1   = "monday tuesday wednesday thursday friday saturday sunday";
       String defaultRange1 = "monday";
       String range2   = "monday tuesday wednesday thursday friday";
       String default1 = "monday tuesday wednesday thursday friday";
       String default2 = "saturday sunday";
       String default3 = "monday tuesday wednesday thursday";
      
       // Parameters does not allow null values
       SampleIceApplication bar = new SampleIceApplication ();
       EchoPolicy foo = new EchoPolicy ();
        
       System.out.println ("TESTING EQUALITY, ASSIGNMENT AND INCLUSION");   
       
       try 
       {
          System.out.println ("TESTING hasEquivalents");
         
          parameter1 = new EnumerationParameter ("test",
                                                 "P0",
                                                 Negotiable.NEGOTIATE,
                                                
                                                 constructSymbolSet (range1),
                                                 constructSymbolSet (range1),
                                                 true,
                                                 Setiable.one_or_more,
                                                 foo,
                                                 bar);
                                                 
         // parameter2 is the same as parameter one
         parameter2 = new EnumerationParameter ("test",
                                                 "P1",
                                                 Negotiable.NEGOTIATE,
                                                 
                                                 constructSymbolSet (range1),
                                                 constructSymbolSet (range1),
                                                 true,
                                                 Setiable.one_or_more,
                                                 foo,
                                                 bar); 
                                                 
         // parameter3 has a different default
         parameter3 = new EnumerationParameter ("test",
                                                 "P2",
                                                 Negotiable.NEGOTIATE,
                                                
                                                 constructSymbolSet (range2),
                                                 constructSymbolSet (default3),
                                                 true,
                                                 Setiable.one_or_more,
                                                 foo,
                                                 bar);                                         
         
         // parameter4 has no defaults
         parameter4 = new EnumerationParameter ("test",
                                                 "P2",
                                                 Negotiable.NEGOTIATE,
                                                
                                                 constructSymbolSet (range1),
                                                 constructSymbolSet (""),
                                                 false,
                                                 Setiable.zero_or_more,
                                                 foo,
                                                 bar);   
          
        result = parameter1.hasEquivalentValues (parameter1);
        System.out.print ("self.hasEquivalentValues (self) should be true: " + result);
        System.out.println ((result == true) ? "[PASSED]" : "[FAILED]");
        
        result = parameter1.hasEquivalentValues (parameter2);
        System.out.print ("parameter1.hasEquivalentValues (parameter2) should be true: " + result);
        System.out.println ((result == true) ? "[PASSED]" : "[FAILED]");  
         
        result = parameter2.hasEquivalentValues (parameter1);
        System.out.print ("testing communicativity parameter2.hasEquivalentValues (parameter1) should be true: " + result);
        System.out.println ((result == true) ? "[PASSED]" : "[FAILED]");   
         
        result = parameter1.hasEquivalentValues (parameter3);
        System.out.print ("parameter1.hasEquivalentValues (parameter3) should be false: " + result);
        System.out.println ((result == false) ? "[PASSED]" : "[FAILED]");   
         
        result = parameter1.hasEquivalentValues (parameter4);
        System.out.print ("parameter1.hasEquivalentValues (parameter4) should be false: " + result);
        System.out.println ((result == false) ? "[PASSED]" : "[FAILED]");  
      
       
        System.out.println ("TESTING inclusion");
       
        parameter1 = new EnumerationParameter ("parameter1",
                                                "P0",
                                                Negotiable.NEGOTIATE,
                                                
                                                constructSymbolSet (range1),
                                                constructSymbolSet (range1),
                                                true,
                                                7,
                                                foo,
                                                bar);
                                                 
         // parameter2 is the same as parameter one
         parameter2 = new EnumerationParameter ("parameter2",
                                                 "P1",
                                                 Negotiable.NEGOTIATE,
                                                
                                                 constructSymbolSet (range1),
                                                 constructSymbolSet (range1),
                                                 true,
                                                 7,
                                                 foo,
                                                 bar); 
                                                 
        /*******************************************************************/
       
        parameter3 = new EnumerationParameter ("parameter3",
                                                "P2",
                                                Negotiable.NEGOTIATE,
                                               
                                                constructSymbolSet (range1),
                                                constructSymbolSet (range1),
                                                true,
                                                Setiable.zero_or_more,
                                                foo,
                                                bar);
       
                                                 
        // parameter4 has smaller default
        parameter4 = new EnumerationParameter ("parameter4",
                                               "P3",
                                               Negotiable.NEGOTIATE,
                                              
                                               constructSymbolSet (default1),
                                               constructSymbolSet (default1),
                                               true,
                                               Setiable.zero_or_more,
                                               foo,
                                               bar);  
        
         
        parameter5 = new EnumerationParameter ("parameter5",
                                                "P2",
                                                Negotiable.NEGOTIATE,
                                               
                                                constructSymbolSet (range1),
                                                constructSymbolSet (range1),
                                                true,
                                                Setiable.one_or_more,
                                                foo,
                                                bar);
       
                                                 
        // parameter4 has smaller default
        parameter6 = new EnumerationParameter ("parameter6",
                                               "P3",
                                               Negotiable.NEGOTIATE,
                                              
                                               constructSymbolSet (range1),
                                               constructSymbolSet (default1),
                                               true,
                                               Setiable.zero_or_more,
                                               foo,
                                               bar);   
         
         
        parameter7 = new EnumerationParameter ("parameter7",
                                               "P3",
                                               Negotiable.NEGOTIATE,
                                              
                                               constructSymbolSet (range1),
                                               constructSymbolSet (""),
                                               false,
                                               Setiable.zero_or_more,
                                               foo,
                                               bar);   
             
        parameter8 = new EnumerationParameter ("parameter8",
                                               "P3",
                                               Negotiable.NEGOTIATE,
                                               
                                               constructSymbolSet (range2),
                                               constructSymbolSet (defaultRange1),
                                               true,
                                               Setiable.zero_or_one,
                                               foo,
                                               bar);                                           
                                               
                                               
        parameter9 = new EnumerationParameter ("parameter9",
                                               "P3",
                                               Negotiable.NEGOTIATE,
                                            
                                               constructSymbolSet (default2),
                                               constructSymbolSet (default2),
                                               true,
                                               Setiable.zero_or_more,
                                               foo,
                                               bar);    
         
         
        result = parameter1.in (parameter1);
        System.out.print ("self.in (self) should be true: " + result);
        System.out.println ((result == true) ? "[PASSED]" : "[FAILED]");
       
        result = parameter2.in (parameter1);
        System.out.print ("parameter2.in (parameter1) should be true: " + result);
        System.out.println ((result == true) ? "[PASSED]" : "[FAILED]");
       
        result = parameter1.in (parameter2);
        System.out.print ("parameter1.in (parameter2) should be true: " + result);
        System.out.println ((result == true) ? "[PASSED]" : "[FAILED]");
       
        result = parameter3.in (parameter4);
        System.out.print ("parameter3.in (parameter4) should be true subset: " + result);
        System.out.println ((result == true) ? "[PASSED]" : "[FAILED]");
        
        result = parameter4.in (parameter3);
        System.out.print ("parameter4.in (parameter3) should be false superset: " + result);
        System.out.println ((result == false) ? "[PASSED]" : "[FAILED]");
       
        result = parameter1.in (parameter4);
        System.out.print ("parameter1.in (parameter4) should be false not a subset: " + result);
        System.out.println ((result == false) ? "[PASSED]" : "[FAILED]");
       
        result = parameter5.in (parameter7);
        System.out.print ("parameter5.in (parameter7) should be false needs at least one: " + result);
        System.out.println ((result == false) ? "[PASSED]" : "[FAILED]");
       
        result = parameter6.in (parameter7);
        System.out.print ("parameter6.in (parameter7) should be true accepts zero: " + result);
        System.out.println ((result == true) ? "[PASSED]" : "[FAILED]");    

        result = parameter8.in (parameter9);
        System.out.print ("parameter8.in (parameter9) should be false no intersection: " + result);
        System.out.println ((result == false) ? "[PASSED]" : "[FAILED]");    
       
        System.out.println ("Testing equality");
        
        /* TO TEST EQUALS */
        parameter1 = new EnumerationParameter ("parameter1",
                                                "P0",
                                                Negotiable.NEGOTIATE,
                
                                                constructSymbolSet (range1),
                                                constructSymbolSet (range1),
                                                true,
                                                Setiable.zero_or_more,
                                                foo,
                                                bar);
                                                 
         // parameter2 is the same as parameter one
         parameter2 = new EnumerationParameter ("parameter1",
                                                 "P1",
                                                 Negotiable.NEGOTIATE,
                                                
                                                 constructSymbolSet (range1),
                                                 constructSymbolSet (range1),
                                                 true,
                                                 Setiable.zero_or_more,
                                                 foo,
                                                 bar); 
                                                 
        // parameter2 is the same as parameter one
         parameter3 = new EnumerationParameter ("parameter3",
                                                 "P1",
                                                 Negotiable.NEGOTIATE,
                                                
                                                 constructSymbolSet (range1),
                                                 constructSymbolSet (range1),
                                                 true,
                                                 Setiable.zero_or_more,
                                                 foo,
                                                 bar);                                          
                                                 
        // parameter2 is the same as parameter one
         parameter4 = new EnumerationParameter ("parameter1",
                                                 "P1",
                                                 Negotiable.NEGOTIATE,
                                                 
                                                 constructSymbolSet (range2),
                                                 constructSymbolSet (range2),
                                                 true,
                                                 Setiable.zero_or_more,
                                                 foo,
                                                 bar);  
                                                 
                                                 
         // parameter2 is the same as parameter one
         parameter5 = new EnumerationParameter ("parameter1",
                                                 "P1",
                                                 Negotiable.NEGOTIATE,
                                                
                                                 constructSymbolSet (range1),
                                                 constructSymbolSet (default2),
                                                 true,
                                                 Setiable.zero_or_more,
                                                 foo,
                                                 bar);                                         
                                                 
                                                 
                                                 
        result = parameter1.equals (parameter1);
        System.out.print ("parameter1.equals (parameter1) should be true self comparison: " + result);
        System.out.println ((result == true) ? "[PASSED]" : "[FAILED]");                                             
        
        result = parameter1.equals (parameter2);
        System.out.print ("parameter1.equals (parameter2) should be true same type name range default: " + result);
        System.out.println ((result == true) ? "[PASSED]" : "[FAILED]");                                                                                      
          
        result = parameter2.equals (parameter1);
        System.out.print ("parameter2.equals (parameter1) should be true communicativity: " + result);
        System.out.println ((result == true) ? "[PASSED]" : "[FAILED]");                                                                                     
        
        result = parameter1.equals (parameter3);
        System.out.print ("parameter1.equals (parameter3) should be false different name: " + result);
        System.out.println ((result == false) ? "[PASSED]" : "[FAILED]");                                                                                            
                                                 
        result = parameter1.equals (parameter4);
        System.out.print ("parameter1.equals (parameter4) should be false different range: " + result);
        System.out.println ((result == false) ? "[PASSED]" : "[FAILED]");                                                                  
    
        result = parameter1.equals (parameter5);
        System.out.print ("parameter1.equals (parameter5) should be false different value: " + result);
        System.out.println ((result == false) ? "[PASSED]" : "[FAILED]");                                  
       
       }
       catch (com.adobe.rice.common.negotiation.InternalError e)
       {
          System.out.println ("Caught InternalError ");
       }
       catch (OutOfRange e)
       {
          System.out.println ("Caught OutOfRange error");
       }
       catch (CloneNotSupportedException e)
       {
          System.out.println ("Caught CloneNotSupportedException error");
       }
       
       return result;
    }
}