package com.adobe.rice.common.negotiation.tests;
import com.adobe.rice.common.negotiation.*;
import java.util.*;

public class NumericParameterTest
{
    public static void main (String argv[])
    {
        test2 ();
        
    }
 
    static private TreeSet constructNumericSet (String string)
    {
        TreeSet set = new TreeSet ();
        StringTokenizer tokens = new StringTokenizer (string);
        String element;
       
        try 
        {
           while (tokens.hasMoreTokens ())
           {
               element = tokens.nextToken ();
               set.add (new Double ((Double.parseDouble (element))));
           }
        }
        catch (NumberFormatException e)
        {
            set = null;
        }
             
        return set;
    } 
    
    public static boolean test1 ()
    {
       boolean result = false;
       NumericParameter span;
       NumericParameter noDefaults;
       NumericParameter enum;
       String  rangeList = "-5 -4 -3 -2 -1 0 1 2 3 4 5";
       String  defaultList = "-3 -1 3 5";
       TreeSet ranges;
       TreeSet defaults;
       
       // Parameters does not allow null values
       SampleIceApplication bar = new SampleIceApplication ();
       EchoPolicy foo = new EchoPolicy ();
        
       try
       {
          
          span = new NumericParameter ("test", "P0", Negotiable.NEGOTIATE, 0, 
                              100, 50, 1, true, foo, bar);
	              
          System.out.println (span); 
         
          ranges = constructNumericSet (rangeList);
          defaults = constructNumericSet (defaultList);
          
          enum = new NumericParameter ("test", "P1", Negotiable.NEGOTIATE, defaults,
                              ranges, Setiable.one_or_more, true, foo, bar);
                              
          System.out.println (enum);
          
          noDefaults = new NumericParameter ("test", "P0", Negotiable.NEGOTIATE, 0, 
                                     100, 50, 1, false, foo, bar);
                                     
          System.out.println ("No default scalar value: " + noDefaults);
          
          noDefaults = new NumericParameter ("test", "P1", Negotiable.NEGOTIATE, defaults,
                                    ranges, Setiable.one_or_more, false, foo, bar);
                                     
          System.out.println ("No default set of values:" + noDefaults);
          
       } 
       catch (com.adobe.rice.common.negotiation.InternalError e)
       {
          System.out.println ("Caught an Internal Error ");
       }
       catch (OutOfRange e)
       {
          System.out.println ("Caught an OutOfRange Error");
       }
       
       return result;
    }
   
    static public boolean test2 ()
    {
       boolean result = false;
       NumericParameter span1;
       NumericParameter span2;
       NumericParameter span3;
       NumericParameter span4;
       NumericParameter span5;
       NumericParameter span6;
       
       NumericParameter noDefaults;
       NumericParameter enum;
       String  rangeList1    = "-5 -4 -3 -2 -1 0 1 2 3 4 5";
       String  rangeList2    = "-5 -4 -3";
       String  rangeList3;
       
       String  defaultList1 = "-3 -1 5";
       String  defaultList2 = "-5 -4";
       String  defaultList3;
       
       TreeSet range1;
       TreeSet range2;
       TreeSet range3;
       TreeSet range4;
       TreeSet default1;
       TreeSet default2;
       TreeSet default3;
       Set     temp;
       
       // Parameters does not allow null values
       SampleIceApplication bar = new SampleIceApplication ();
       EchoPolicy foo = new EchoPolicy ();
        
       System.out.println ("TESTING EQUALITY, ASSIGNMENT AND INCLUSION");   
       
       try 
       {
          span1 = new NumericParameter ("test", "P0", Negotiable.NEGOTIATE, 0, 
                               100, 50, 1, true, foo, bar);
                            
          span2 = new NumericParameter ("test", "P1", Negotiable.NEGOTIATE, 0, 
                               100, 50, 1, true, foo, bar);
                            
          span3 = new NumericParameter ("test", "P1", Negotiable.NEGOTIATE, 0, 
                               100, 75, 1, true, foo, bar);
                               
          span4 = new NumericParameter ("test", "P1", Negotiable.NEGOTIATE, 0, 
                               100, 75, 0, false, foo, bar);
        
          System.out.println ("TESTING hasEquivalentValues () on spans");
       
          result = span1.hasEquivalentValues (span1);  
          System.out.print ("self: span1.hasEquivalentValues(span1) should be true: " + result);
          System.out.println ( (result == true) ? "[PASSED]" : "[FAILED]");
      
          result = span1.hasEquivalentValues (span2);
          System.out.print ("equivalent value: span1.hasEquivalentValues(span2) should be true: " + result);
          System.out.println ( (result == true) ? "[PASSED]" : "[FAILED]");
       
          result = span1.hasEquivalentValues (span3);
          System.out.print ("different value: span1.hasEquivalentValues(span3) should be false: " + result);
          System.out.println ( (result == true) ? "[FAILED]" : "[PASSED]"); 
       
          result = span1.hasEquivalentValues (span4);
          System.out.print ("no default: span1.hasEquivalentValues(span4) should be false: " + result);
          System.out.println ( (result == true) ? "[FAILED]" : "[PASSED]"); 
       
          System.out.println ("TESTING hasEquivalents on enums");
          
          range1 = constructNumericSet (rangeList1);
          default1 = constructNumericSet (defaultList1);
          span1 = new NumericParameter ("test", "P0", Negotiable.NEGOTIATE, default1, 
                               range1, Setiable.one_or_more, true, foo, bar);
                            
          range1 = constructNumericSet (rangeList1);
          default1 = constructNumericSet (defaultList1);                  
          span2 = new NumericParameter ("test", "P1", Negotiable.NEGOTIATE, default1, 
                               range1, Setiable.one_or_more, true, foo, bar);
                            
          default2 = constructNumericSet (defaultList2);                  
          span3 = new NumericParameter ("test", "P2", Negotiable.NEGOTIATE, default2, 
                               range1, Setiable.one_or_more, true, foo, bar);
                               
          span4 = new NumericParameter ("test", "P1", Negotiable.NEGOTIATE, default2, 
                               range1, Setiable.one_or_more, false, foo, bar);          
       
          System.out.println ("TESTING INCLUSION");
          
          span1 = new NumericParameter ("test", "P0", Negotiable.NEGOTIATE, 0, 
                               100, 50, 1, true, foo, bar);
                            
          span2 = new NumericParameter ("test", "P1", Negotiable.NEGOTIATE, 0, 
                               100, 50, 1, true, foo, bar);
                            
          span3 = new NumericParameter ("test", "P2", Negotiable.NEGOTIATE, 0, 
                               200, 175, 1, true, foo, bar);
                               
          span4 = new NumericParameter ("test", "P3", Negotiable.NEGOTIATE, 0, 
                               100, 75, 0, false, foo, bar);
                               
          result = span1.in (span1);
          System.out.print ("span1.in (span1) should be true " + result);
          System.out.println ((result == true) ? "[PASSED]" : "[FAILED]");
          
          result = span1.in (span2);
          System.out.print ("span1.in (span2) should be true " + result);
          System.out.println ((result == true) ? "[PASSED]" : "[FAILED]");
          
          result = span1.in (span3);
          System.out.print ("span1.in (span3) should be false " + result);
          System.out.println ((result == true) ? "[FAILED]" : "[PASSED]");
          
          result = span1.in (span4);
          System.out.print ("span1.in (span3) should be false " + result);
          System.out.println ((result == false) ? "[PASSED]" : "[FAILED]");
     
          rangeList1 = "-6 -5 -4 -3 -2 100 150";
          range1 = constructNumericSet (rangeList1);
          span1 = new NumericParameter ("test", "P0", Negotiable.NEGOTIATE, range1, 
                               range1, Setiable.one_or_more, true, foo, bar);
                            
          rangeList2 = "-4 -3 -2";  
          range2 = constructNumericSet (rangeList2);
          span2 = new NumericParameter ("test", "P1", Negotiable.NEGOTIATE, range2, 
                               range2, Setiable.one_or_more, true, foo, bar);
                            
          rangeList3 = "200 300 400";
          defaultList3 = "200 300";
          range3 = constructNumericSet (rangeList3); 
          default3 = constructNumericSet (defaultList3);
          span3 = new NumericParameter ("test", "P2", Negotiable.NEGOTIATE, default3, 
                               range3, Setiable.one_or_more, true, foo, bar);
                               
                               
          result = span1.in (span1);
          System.out.print ("span1.in (span1) should be true " + result);
          System.out.println ((result == true) ? "[PASSED]" : "[FAILED]");
       
          result = span1.in (span2);
          System.out.print ("span1.in (span2) should be true " + result);
          System.out.println ((result == true) ? "[PASSED]" : "[FAILED]");
       
          result = span1.in (span3);
          System.out.print ("span1.in (span3) should be false " + result);
          System.out.println ((result == false) ? "[PASSED]" : "[FAILED]");
       
          range1 = constructNumericSet (rangeList1);
          span1 = new NumericParameter ("test", "P0", Negotiable.NEGOTIATE, range1, 
                               range1, 7, true, foo, bar);
       
          result = span4.in (span2);
          System.out.print ("span4.in (span2) should be false " + result);
          System.out.println ((result == false) ? "[PASSED]" : "[FAILED]");          
   
          System.out.println ("TESTING EQUALS AND ASSIGNMENT");
       
          span1 = new NumericParameter ("test", "P0", Negotiable.NEGOTIATE, 0, 
                               100, 50, 1, true, foo, bar);
                            
          span2 = new NumericParameter ("test", "P2", Negotiable.NEGOTIATE, 0, 
                               100, 50, 1, true, foo, bar);
                            
          span3 = new NumericParameter ("test", "P3", Negotiable.NEGOTIATE, 0, 
                               100, 75, 1, true, foo, bar);
                               
          span4 = new NumericParameter ("test", "P4", Negotiable.NEGOTIATE, 10, 
                               90, 50, 1, true, foo, bar);
                               
          span5 = new NumericParameter ("good", "P5", Negotiable.ACCEPT,0,
                               100, 50, 1, true, foo, bar);
                               
          String  defaultList5 = "0 50 100";                     
          String  rangeList5 = "50";
          Set     range5;
          Set     default5;
          range5 = constructNumericSet (rangeList5);
          default5 = constructNumericSet (rangeList5);
          span6 = new NumericParameter ("test", "P5", Negotiable.NEGOTIATE,default5,
                               range5, Setiable.one_or_more, true, foo, bar);                     
                           
          System.out.println ("TESTING EQUALITY OF SPANS");
          
          result = span1.equals (span2);
          System.out.print ("span1 equals span2: should be true result: " + result);
          System.out.println (result == true ? "[PASSED]" : "[FAILED]");
          
          result = span1.equals (span3);
          System.out.print ("span1 equals span3: should be false different defaults result: " + result);
          System.out.println (result == false ? "[PASSED]" : "[FAILED]");
          
          result = span1.equals (span4);
          System.out.print ("span1 equals span4: should be false different ranges result: " + result);
          System.out.println (result == false ? "[PASSED]" : "[FAILED]");                     
                               
          result = span1.equals (span5);
          System.out.print ("span1 equals span5: should be false different names result: " + result);
          System.out.println (result == false ? "[PASSED]" : "[FAILED]");  
          
          result = span1.equals (span6);
          System.out.print ("span1 equals span6: should be false different range types result: " + result);
          System.out.println (result == false ? "[PASSED]" : "[FAILED]");
 
          System.out.println ("TESTING EQUALITY OF ENUMS");
       
          rangeList1 = "-6 -5 -4 -3 -2 100 150";
          range1 = constructNumericSet (rangeList1);
          span1 = new NumericParameter ("test", "P0", Negotiable.NEGOTIATE, range1, 
                               range1, Setiable.one_or_more, true, foo, bar);
                            
          String rangeList6 = "-6 -5 -4 -3 -2 100 150";
          range2 = constructNumericSet (rangeList6);
          
          span2 = new NumericParameter ("test", "P1", Negotiable.NEGOTIATE, range2, 
                               range1, Setiable.one_or_more, true, foo, bar);
          
          span3 = new NumericParameter ("test", "P2", Negotiable.NEGOTIATE, default3, 
                               range3, Setiable.one_or_more, true, foo, bar);
       
          rangeList1 = "-6 -5 -4";
          range4 = constructNumericSet (rangeList1);
          span4 = new NumericParameter ("test", "P2", Negotiable.NEGOTIATE, range4, 
                               range1, Setiable.one_or_more, true, foo, bar);
       
          span5 = new NumericParameter ("test", "P0", Negotiable.NEGOTIATE, range1, 
                               range1, Setiable.zero_or_more, true, foo, bar);
       
          result = span1.equals (span2);
          System.out.print ("span1 equals span2: should be true result: " + result);
          System.out.println (result == true ? "[PASSED]" : "[FAILED]");
          
          result = span1.equals (span3);
          System.out.print ("span1 equals span3: should be false different defaults result: " + result);
          System.out.println (result == false ? "[PASSED]" : "[FAILED]");
          
          result = span1.equals (span4);
          System.out.print ("span1 equals span4: should be false different ranges result: " + result);
          System.out.println (result == false ? "[PASSED]" : "[FAILED]");                     
                               
          result = span1.equals (span5);
          System.out.print ("span1 equals span5: should be false different selectors result: " + result);
          System.out.println (result == false ? "[PASSED]" : "[FAILED]");  
       }
       catch (com.adobe.rice.common.negotiation.InternalError e)
       {
          System.out.println ("Caught InternalError ");
       }
       catch (OutOfRange e)
       {
          System.out.println ("Caught OutOfRange error");
       }
       
       return result;
    }
}