/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/payloadAssembly/AssemblyElement.java,v 1.4 2000/12/02 23:24:24 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.payloadAssembly;
//
// java imports
//
import java.util.*;

//
// icecubes  imports
//
import com.adobe.rice.IceCubes;
import com.adobe.rice.util.*;
import com.adobe.rice.common.*;
import com.adobe.rice.common.XMLassembly.*;
import com.adobe.rice.common.payloadAssembly.*;
import com.adobe.rice.common.policies.*;
import com.adobe.rice.common.storage.logging.*;     // so we can write to the log.

public class AssemblyElement extends Element
{
    //
    // This is the Assembly bottleneck
    // its function is to provide a means to intercept any of the
    // methods and parameters in an XMLassembly Element.
    //
    int nest = 0;
        
    public AssemblyElement( String name, Element parent, DocType aDTD ) throws ElementException
    {
        super( name, parent, aDTD );
        nest = 0;
    }
    
    public AssemblyElement( String name, String prefix, Element parent, DocType aDTD ) throws ElementException
    {
        super( name, prefix, parent, aDTD );
        nest = 0;
    }

    public void setXMLnest( int n ) { nest = n; }
    public int getXMLnest() { return nest; }
    
    public String addText( String text )
    {
        String result = null;
                
        ContentReference cref = null;
        try
        {
            cref = new ContentReference("CharData", text );
            // NOTE:  The following assumes that assembly text elements
            // are for human consumption.
            cref.setOkToFormat(true);
        }
        catch ( ContentException ce ) 
        {
            Object parms[] = { this.getName(), ce.getMessage() };
            AssemblyMessages.log(AssemblyMessages.get("characterDataBad"), 
                                        parms, 40000000 );
            result = ce.getMessage();
        }
        
        if ( result == null )
        {
            this.open();
            try
            {
                this.add(cref);
            }
            catch ( ElementException ee )
            {
                Object parms[] = { "chardata", this.getName() + " content", ee.getMessage() };
                AssemblyMessages.log(AssemblyMessages.get("contentInvalid"), 
                            parms, 30000000 );
                result = ee.getMessage();
            }
            finally
            {
                this.close();
            }
        }
        return result;
    }
    
    public String getContent()
    {
        String result = "";
        
        for ( int i = 0; i < this.getCount(); i++ )
        {
            Object obj = this.getContent(i);
            if ( obj instanceof ContentReference )
            {
                ContentReference cr = (ContentReference) obj;
                result += cr.getText();
            }
            else if ( obj instanceof Element )
            {
                result += ( (Element) obj ).toXMLstring();
            }
            else
            {
                // it's a PI or Comment -- ignore it.
            }
        }
        
        return result;
        
    }
    
    public String setText( String text )
    {
        String result = null;   
        ContentReference cref = null;
        
        try 
        {
            cref = new ContentReference("CharData", text );
            // NOTE:  The following assumes that assembly text elements
            // are for human consumption.
            cref.setOkToFormat(true);
        }
        catch ( ContentException ce ) 
        {
            Object parms[] = { this.getName(), ce.getMessage() };
            AssemblyMessages.log(AssemblyMessages.get("characterDataBad"), 
                                        parms, 40000000 );
            result = ce.getMessage();
        }
        
        this.open();    // open the element.
   
        if ( this.getCount() > 0 )
            this.clearContent();
        try 
        {
            this.add(cref);  // add in the content.
        }
        catch ( ElementException ee )
        {
                Object parms[] = { "chardata",this.getName() + " content", ee.getMessage() };
                AssemblyMessages.log(AssemblyMessages.get("contentInvalid"), 
                            parms, 30000000 );
                result = ee.getMessage();
        }
        finally
        {
            this.close();
        }
        return result;
    }
}
