/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/payloadAssembly/header/Receiver.java,v 1.2 2000/12/02 23:25:34 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.payloadAssembly.header;
//
// java imports
//
import java.util.*;

//
// icecubes  imports
//
import com.adobe.rice.IceCubes;
import com.adobe.rice.util.*;
import com.adobe.rice.common.*;
import com.adobe.rice.common.XMLassembly.*;
import com.adobe.rice.common.payloadAssembly.*;
import com.adobe.rice.common.policies.*;
import com.adobe.rice.common.storage.logging.*;     // so we can write to the log.

public class Receiver extends AssemblyElement implements XMLexpandable
{
    //
    // <!ELEMENT ice-receiver EMPTY > 
    // <!ATTLIST ice-receiver receiver-id CDATA    #REQUIRED
    //                        name        CDATA    #REQUIRED >
    //
    UUID receiverId;
    String attributeName;       // not to be confused with element name!
        
    public Receiver( Element parent, DocType dtd ) throws ElementException
    {
        super("ice-receiver", parent, dtd );
        
        this.setOkToFormat(true);       // ok to format.
        this.setIsEmpty(true);          // the receiver is empty!

        try 
        {
            receiverId = new UUID();
            this.setAttribute("receiver-id", receiverId.toString());
            this.attributeName = "_Undefined_Receiver_Name_";
            this.setAttribute("name", this.attributeName);
        }
        catch ( AttributeException ae )
        {
            throw new ElementException( ae.getMessage() );
        }
     
    }
    
    public Receiver( UUID rid, Element parent, DocType dtd  ) throws ElementException
    {
        this( parent, dtd );
        this.receiverId = rid;
        try {
            this.setAttribute("receiver-id", this.receiverId.toString() );
            this.setAttribute("name", this.attributeName );
        }
        catch ( AttributeException ae )
        {
            throw new ElementException( ae.getMessage() );
        }
    }
    
    public Receiver( String receiverName, Element parent, DocType dtd  ) throws ElementException
    {
        this(parent, dtd);
        this.attributeName = receiverName;
        try 
        {
            this.setAttribute( "receiver-id", this.receiverId.toString() );
            this.setAttribute("name", this.attributeName );
        }
        catch ( AttributeException ae )
        {
            throw new ElementException( ae.getMessage() );
        }
    }
    
    
    public Receiver( UUID rid, String rName, Element parent, DocType dtd  ) throws ElementException
    {
        this(parent, dtd);
        this.receiverId = rid;
        this.attributeName = rName;
        try 
        {
            this.setAttribute( "receiver-id", this.receiverId.toString() );
            this.setAttribute("name", this.attributeName );
        }
        catch ( AttributeException ae )
        {
            throw new ElementException( ae.getMessage() );
        }
    }
    
    //
    // Setters & Getters
    //
    public String getAttributeName() { return this.getAttribute("name"); }
    
    public void setAttributeName( String n ) throws ElementException
    { 
        this.attributeName = n;
        try
        {
            this.setAttribute("name", this.attributeName);
        }
        catch ( AttributeException ae )
        {
            throw new ElementException( ae.getMessage() );
        }
        
    }
    public UUID getReceiverId() 
    { 
        return this.receiverId; 
        
    }
    
    public void setReceiverId( UUID rid ) throws ElementException
    { 
        this.receiverId = rid;
        try
        {
            this.setAttribute("receiver-id", this.receiverId.toString() );
        }
        catch ( AttributeException ae )
        {
            throw new ElementException( ae.getMessage() );
        }
        
    }
    //
    // XML Expandable Implementation -- override
    //
    
    public String toXMLstring()
    {
        if ( this.receiverId == null )
        {  
            // The receiver-id attribute is required.
           Object[] parms = { "ice-receiver", "receiver-id", "UUID" };    
           AssemblyMessages.log(AssemblyMessages.get("missingRequiredAttribute"), 
               parms, ImpactRange.SERIOUS  );
        }
        if ( this.attributeName == null )
        {   // it is in error to expand this without a name!
            Object[] parms = { "ice-receiver", "name", "CDATA" };
            AssemblyMessages.log(AssemblyMessages.get("missingRequiredAttribute"),
                parms, ImpactRange.SERIOUS  );
        }
        return super.toXMLstring();
    }    

}