/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/payloadAssembly/header/Sender.java,v 1.3 2000/12/02 23:25:33 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.payloadAssembly.header;

import java.util.*;

//
// icecubes  imports
//
import com.adobe.rice.IceCubes;
import com.adobe.rice.util.*;
import com.adobe.rice.common.*;
import com.adobe.rice.common.XMLassembly.*;
import com.adobe.rice.common.payloadAssembly.*;
import com.adobe.rice.common.policies.*;
import com.adobe.rice.common.storage.logging.*;     // so we can write to the log.
import com.adobe.rice.common.payloadAssembly.*;     // parent

public class Sender extends AssemblyElement implements XMLexpandable
{
    private static UUID mySenderId;     // hidden single copys for performance.
    private static String myName;
    private static String myRole;
    private static String myLocation;
    
    // <!ELEMENT ice-sender EMPTY >
    // <!ATTLIST ice-sender sender-id CDATA      #REQUIRED 
    //                      name      CDATA      #REQUIRED
    //                      role      (subscriber | syndicator) #REQUIRED
    //                      location  CDATA      #IMPLIED                 >
        
    public Sender( Element parent, DocType aDTD ) throws ElementException
    {
        super("ice-sender", parent, aDTD );
        
        this.setOkToFormat(true);       // ok to format.
        
        this.setIsEmpty(true);          // the sender is empty 
        
        // note that by default, we will use the configuration UUID
        //
        // get this processor's UUID
        //
        // It is expensive to convert this every time, so do it once.
        if ( mySenderId == null )
        {
            String ourUUIDstring = (String) PolicyManager.instance().getSelection("OrganizationUUID");
            mySenderId = new UUID(ourUUIDstring);
        }
        
        if ( myName == null )
            myName = (String) PolicyManager.instance().getSelection("OrganizationName");
        
        if ( myRole == null )
            myRole = (String) PolicyManager.instance().getSelection("Role");
            
        if ( myLocation == null )
            myLocation = (String) PolicyManager.instance().getSelection("Location");
        
        //
        // now setup the sender element
        //
        try
        {
            this.setAttribute("sender-id", mySenderId.toString() );
            this.setAttribute("name", myName);
            this.setAttribute("role", myRole);
            this.setAttribute("location", myLocation );
        }
        catch ( AttributeException ae )
        {
            throw new ElementException( ae.getMessage() );
        }
    }
    
    public Sender( UUID sid, Element parent, DocType aDTD ) throws ElementException
    {
        this(parent, aDTD);
        try
        {
            this.setAttribute("sender-id", sid.toString() );
        }
        catch ( AttributeException ae )
        {
            throw new ElementException( ae.getMessage() );
        }
    }
    
    public Sender( String senderName, Element parent, DocType aDTD ) throws ElementException
    {
        this(parent, aDTD);
        try
        {
            this.setAttribute("name", senderName);
        }
        catch ( AttributeException ae )
        {
            throw new ElementException( ae.getMessage() );
        }
    }
    
    public Sender( String senderName, String senderRole, Element parent, DocType aDTD ) throws ElementException
    {
        this( parent, aDTD );
        try
        {
            this.setAttribute("name", senderName);
            this.setAttribute("role", senderRole);
        }
        catch ( AttributeException ae )
        {
            throw new ElementException( ae.getMessage() );
        }
    }
    
    public Sender( UUID sid, String sName, String sRole, Element parent, DocType aDTD ) throws ElementException
    {
        this( parent, aDTD );
        try
        {
            this.setAttribute("sender-id", sid.toString() );
            this.setAttribute("name", sName);
            this.setAttribute("role", sRole);
        }
        catch ( AttributeException ae )
        {
            throw new ElementException( ae.getMessage() );
        }
    }
    
    //
    // Setters & Getters
    //
    public String getAttributeName() { return this.getAttribute("name"); }
    public void setAttributeName( String n ) throws AttributeException
    { 
            this.setAttribute("name", n); 
    }
    public String getRole() { return this.getAttribute("role"); }
    public void setRole( String r ) throws AttributeException 
    {
        this.setAttribute("role", r ); 
    }
    public UUID getSenderId() 
    { 
        return new UUID( this.getAttribute("sender-id") );
    }
    public void setSenderId( UUID sid ) throws AttributeException { this.setAttribute("sender-id", sid.toString()); }
    
    public void setLocation( String loc ) throws AttributeException
    {
        this.setAttribute("location", loc );
    }
    public String getLocation() { return this.getAttribute("location"); }
    //
    // XML Expandable Implementation overrride
    //
    
    public String toXMLstring()
    {
        if ( this.getAttribute("sender-id") == null )
        {  // The sender-id attribute is required and is missing.
           Object[] parms = { "ice-sender", "sender-id", "UUID" };    
           AssemblyMessages.log(AssemblyMessages.get("missingRequiredAttribute"), 
               parms, ImpactRange.SERIOUS );
        }
        if ( this.getAttribute("name") == null )
        {  // The name attribute is required and is missing.
           Object[] parms = { "ice-sender", "name", "string" };    
           AssemblyMessages.log(AssemblyMessages.get("missingRequiredAttribute"), 
               parms, ImpactRange.SERIOUS  );
        }
        if ( this.getAttribute("role") == null )
        {  // The name attribute is required and is missing.
           Object[] parms = { "ice-sender", "role", "{syndicator|subscriber}" };    
           AssemblyMessages.log(AssemblyMessages.get("missingRequiredAttribute"), 
               parms, ImpactRange.SERIOUS  );
        }
        // the location is optional in ice 1.0 and 1.01 and is required in
        // ice 1.1 and above.
        // We need a general piece of apparatus to find out the ice version
        // of this particular interchange.
        // $FIX$
        if ( (this.getAttribute("location") == null) && ( true /* version >= 1.1 */ ) )
        {   // see comment just above.
           Object[] parms = { "ice-sender", "location", "URL" };    
           AssemblyMessages.log(AssemblyMessages.get("missingRequiredAttribute"), 
               parms, ImpactRange.SERIOUS  );
        }
        return  super.toXMLstring();
    }    

    //
    // static default setters and getters
    //
    public static void setDefaultRole( String r ) { myRole = r; }
    public static String getDefaultRole( ) { return myRole; }
    public static void setDefaultLocation( String loc ) { myLocation = loc; }
    public static String getDefaultLocation() { return myLocation; }
    public static void setDefaultSenderId( UUID uuid ) { mySenderId = uuid; }
    public static UUID getDefaultSenderId() { return mySenderId; }
    public static void setDefaultName( String nm ) { myName = nm; }
    public static String getDefaultName() { return myName; }
    
}