/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/payloadAssembly/requests/Notify.java,v 1.2 2000/12/02 23:25:13 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.payloadAssembly.requests;

import java.util.*;

//
// icecubes  imports
//
import com.adobe.rice.IceCubes;
import com.adobe.rice.common.*;
import com.adobe.rice.common.XMLassembly.*;
import com.adobe.rice.common.policies.*;
import com.adobe.rice.common.storage.logging.*;     // so we can write to the log.
import com.adobe.rice.common.payloadAssembly.*;     // our parent


public class Notify extends AssemblyElement 
{
    // <!ELEMENT ice-notify (ice-text+) >
    // <!ATTLIST ice-notify
    //                       priority (1 | 2 | 3 | 4 | 5) #REQUIRED
    // >
    String priority = "3";
    
    public Notify( Element parent, DocType dtd ) throws ElementException
    {
        super("ice-notify", parent, dtd );
        this.setOkToFormat(true);       // ok to format.
        
        this.priority = (String) Tray.getPolicyManager().getSafeSelection("NotifyPriority", 
                           AssemblyConstants.fixedNotifyPriority);
        try 
        {
            this.open();
            this.setAttribute("priority", this.priority );
        }
        catch( AttributeException ae )
        {
           // log the fact that we got an attribute execption...
           Object[] parms = { "ice-notify", ae.getMessage() };    
           AssemblyMessages.log(AssemblyMessages.get("missingRequiredAttribute"), 
               parms, ImpactRange.CRITICAL );
           throw new ElementException( ae.getMessage() ); 
        }
        finally
        {
            this.close();
        }
        
        // now add an empty text element
        if ( false ) // $CHECK$ think about this -- why not let the caller add it later ala below?
        {
            this.open();
            try
            {
                this.add( (Element) new Text( this, dtd ) );
            }
            catch ( ElementException ee )
            {
                // { "contentInvalid", "Unable to add content type {0}, to element, {1}, exception {2}." },
                Object[] parms = 
                { 
                        "ice-text", 
                        "ice-notify", 
                        ee.getMessage() 
                };    
                AssemblyMessages.log(
                        AssemblyMessages.get("contentInvalid"), 
                        parms, 
                        ImpactRange.SERIOUS 
                );
                throw ee; 
            }
            finally
            {
                this.close();
            }
        }
    }
    
    public void addTextElement( String theText ) throws ElementException
    {
        this.open();
        try { this.add( new Text( this, this.getDtd() ) ); }
        catch ( ElementException ee ) 
        { 
            // { "contentInvalid", "Unable to add content type {0}, to element, {1}, exception {2}." },
            Object[] parms = 
            { 
                    "ice-text", 
                    "ice-notify", 
                    ee.getMessage() 
            };    
            AssemblyMessages.log(
                    AssemblyMessages.get("contentInvalid"), 
                    parms, 
                    ImpactRange.SERIOUS 
            );
            throw ee; 
        }
        finally { this.close(); }
    }
    
    public void setPriority( String newPriority ) throws AttributeException
    {
        try 
        {
            this.open();
            this.setAttribute("priority", newPriority );
        }
        catch( AttributeException ae )
        {
           // log the fact that we got an attribute execption...
           // { "attributeValueSetFailed",  "Element {0}, attribute {1} set value to {2} failed with exception {3}."},
           Object[] parms = 
           { 
                "ice-notify", 
                "priority", 
                newPriority, 
                ae.getMessage() 
           };    
           AssemblyMessages.log(
                AssemblyMessages.get("attributeValueSetFailed"), 
                parms, 
                ImpactRange.SERIOUS 
           );
           throw ae;
        }
        finally
        {
            this.close();
        }
    }
    
    public void setPriority( int newPriority ) throws AttributeException
    {
        Integer theIntegerPriority = new Integer( newPriority );
        
        try 
        {
            this.open();
            this.setAttribute("priority", theIntegerPriority.toString() );
        }
        catch( AttributeException ae )
        {
           // log the fact that we got an attribute execption...
           // { "attributeValueSetFailed",  "Element {0}, attribute {1} set value to {2} failed with exception {3}."},
           Object[] parms = 
           { 
                "ice-notify", 
                "priority", 
                theIntegerPriority.toString(), 
                ae.getMessage() 
           };    
           AssemblyMessages.log(
                AssemblyMessages.get("attributeValueSetFailed"), 
                parms, 
                ImpactRange.SERIOUS 
           );
        }
        finally
        {
            this.close();
        }
    }
    
    public String getPriority()
    {
        return this.getAttribute("priority");
    }
}
