/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/payloadAssembly/requests/Text.java,v 1.1 2000/12/02 23:25:13 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.payloadAssembly.requests;

import java.util.*;

//
// icecubes  imports
//
import com.adobe.rice.IceCubes;
import com.adobe.rice.common.*;
import com.adobe.rice.common.XMLassembly.*;
import com.adobe.rice.common.policies.*;
import com.adobe.rice.common.storage.logging.*;     // so we can write to the log.
import com.adobe.rice.common.payloadAssembly.*;     // our parent


public class Text extends AssemblyElement 
{
    // <!ELEMENT ice-text (#PCDATA) >
    // <!ATTLIST ice-text
    //                    xml:lang CDATA #IMPLIED >
    //
    String xmlLang = "en"; // OPTIONAL - see spec 3.4.2
    
    
    public Text( Element parent, DocType dtd ) throws ElementException
    {
        super("ice-text", parent, dtd );
        // note that we set the default - okToFormat to true because 
        // the text in ice outside of packages is intended for human consumption.
        // This is not (directly) the case with ice-items and so there we do NOT
        // turn on okToFormat by default.
        this.setOkToFormat(true);       // ok to format.
        
        this.xmlLang = (String) Tray.getPolicyManager().getSafeSelection("Language", 
                           AssemblyConstants.fixedIceLanguage);
        try 
        {
            this.open();
            this.setAttribute("xml:lang", this.xmlLang );
        }
        catch( AttributeException ae )
        {
           // log the fact that we got an attribute execption...
           Object[] parms = { "ice-text", ae.getMessage() };    
           AssemblyMessages.log(AssemblyMessages.get("missingRequiredAttribute"), 
               parms, ImpactRange.CRITICAL );
        }
        finally
        {
            this.close();
        }
    }
    
    public void setXmlLang( String lang )
    {
        try
        {
            this.open();
            this.setAttribute("xml:lang", lang);
            this.xmlLang = lang;
        }
        catch ( AttributeException ae )
        {
            Object[] parms = { "ice-text", "xml:lang", ae.getMessage() };
            AssemblyMessages.log(AssemblyMessages.get("attributeSetFailed"),
                parms, ImpactRange.SERIOUS  );
        }
        finally 
        {
            this.close();
        }
    }
    
    public String getXmlLang() { return this.getAttribute("xml:lang"); }
    
    // note that String setText( String newText ) is inherited
    // from AssemblyElement.
    
}
