/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/payloadAssembly/responses/Code.java,v 1.3 2000/12/02 23:24:48 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.payloadAssembly.responses;

import java.util.*;

//
// icecubes  imports
//
import com.adobe.rice.IceCubes;
import com.adobe.rice.common.*;
import com.adobe.rice.common.XMLassembly.*;
import com.adobe.rice.common.policies.*;
import com.adobe.rice.common.storage.logging.*;     // so we can write to the log.
import com.adobe.rice.common.payloadAssembly.*;     // our parent


public class Code extends AssemblyElement 
{
    String messageId = "_UNSET_";               // REQUIRED - see spec 3.4.2
    String numeric = "_UNSET_";                 // REQUIRED - see spec 3.4.2
    String packageId;                           // CONDITIONAL - see spec 3.4.2
    String payloadId;                           // CONDITIONAL - see spec 3.4.2
    String phrase = "_UNSET_";                  // REQUIRED - see spec 3.4.2
    String xmlLang = "en";                      // OPTIONAL - see spec 3.4.2
    
    public Code(Element parent, DocType dtd ) throws ElementException
    {
        super("ice-code",parent, dtd );
        this.setOkToFormat(true);       // ok to format.
        // put in the stuff containing the codes and the phrases for ice.
        xmlLang = (String) Tray.getPolicyManager().getSafeSelection("Language", 
                           AssemblyConstants.fixedIceLanguage);
        try 
        {
            this.setAttribute("message-id", messageId );
            this.setAttribute("numeric", numeric );
            // this.setAttribute("package-id", packageId );
            // this.setAttribute("payload-id", payloadId );
            this.setAttribute("phrase", phrase );
            this.setAttribute("xml:lang", xmlLang );
        }
        catch( AttributeException ae )
        {
           // log the fact that we got an attribute execption...
           Object[] parms = { "ice-code", ae.getMessage() };    
           AssemblyMessages.log(AssemblyMessages.get("missingRequiredAttribute"), 
               parms, ImpactRange.SERIOUS  );
        }
        
    }
    
    //
    // setters and getters for attributes
    //
    public void setNumeric( String num )
    {
        try
        {
            this.open();
            this.setAttribute("numeric", num);
            this.numeric = num;
        }
        catch ( AttributeException ae )
        {
            Object[] parms = { "ice-code", "numeric", ae.getMessage() };
            AssemblyMessages.log(AssemblyMessages.get("attributeSetFailed"),
                parms, ImpactRange.SERIOUS  );
        }
        finally 
        {
            this.close();
        }
    }
    
    public String getNumeric() { return this.getAttribute("numeric"); }
    
    public void setPhrase( String text )
    {
        try
        {
            this.open();
            this.setAttribute("phrase", text);
            this.phrase = text;
        }
        catch ( AttributeException ae )
        {
            Object[] parms = { "ice-code", "phrase", ae.getMessage() };
            AssemblyMessages.log(AssemblyMessages.get("attributeSetFailed"),
                parms, ImpactRange.SERIOUS  );
        }
        finally 
        {
            this.close();
        }
    }
    
    public String getPhrase() { return this.getAttribute("phrase"); }
    
    public void setMessageId( String msgId )
    {
        try
        {
            this.open();
            this.setAttribute("message-id", msgId);
            this.messageId = msgId;
        }
        catch ( AttributeException ae )
        {
            Object[] parms = { "ice-code", "message-id", ae.getMessage() };
            AssemblyMessages.log(AssemblyMessages.get("attributeSetFailed"),
                parms, ImpactRange.SERIOUS  );
        }
        finally 
        {
            this.close();
        }
    }
    
    public String getMessageId() { return this.getAttribute("message-id"); }
    
    public void setPayloadId( String payId )
    {
        //
        // follow the rules for setting the payload Id in a code. ( see spec, 3.4.2 )
        //
        if ( ( this.numeric != null ) && 
             ( this.numeric.length() > 0 ) && 
             ( this.numeric.charAt(0) != '3' ) )
        {
            try
            {
                this.open();
                this.setAttribute("payload-id", payId);
                this.payloadId = payId;
            }
            catch ( AttributeException ae )
            {
                Object[] parms = { "ice-code", "payload-id", ae.getMessage() };
                AssemblyMessages.log(AssemblyMessages.get("attributeSetFailed"),
                    parms, ImpactRange.SERIOUS  );
            }
            finally 
            {
                this.close();
            }
        }
    }
    
    public String getPayloadId() { return this.getAttribute("payload-id"); }
    
    public void setPackageId( String pkgId )
    {
        try
        {
            this.open();
            this.setAttribute("package-id", pkgId);
            this.packageId = pkgId;
        }
        catch ( AttributeException ae )
        {
            Object[] parms = { "ice-code", "package-id", ae.getMessage() };
            AssemblyMessages.log(AssemblyMessages.get("attributeSetFailed"),
                parms, ImpactRange.SERIOUS  );
        }
        finally 
        {
            this.close();
        }
    }
    
    public String getPackageId() { return this.getAttribute("package-id"); }
    
    public void setXmlLang( String lang )
    {
        try
        {
            this.open();
            this.setAttribute("xml:lang", lang);
            this.xmlLang = lang;
        }
        catch ( AttributeException ae )
        {
            Object[] parms = { "ice-code", "xml:lang", ae.getMessage() };
            AssemblyMessages.log(AssemblyMessages.get("attributeSetFailed"),
                parms, ImpactRange.SERIOUS  );
        }
        finally 
        {
            this.close();
        }
    }
    
    public String getXmlLang() { return this.getAttribute("xml:lang"); }
    
}
