/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/payloadDisassembly/DisassemblyMessages.java,v 1.2 2000/12/02 23:28:45 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.payloadDisassembly;

//
// java imports:
//
import java.util.*;

//
// icecubes imports:
//
import com.adobe.rice.common.storage.logging.*;
import com.adobe.rice.common.payloadDisassembly.*;

public class DisassemblyMessages extends ListResourceBundle 
{
	public static String get(String message) 
	{
		ResourceBundle res = ResourceBundle.getBundle("com.adobe.rice.common.payloadDisassembly.DisassemblyMessages");
		return res.getString(message);
	}
	
	public Object[][] getContents() { return contents; }
	
	static Object[][] contents =
	{
        //
        // generic payload error messages
        //
        {  "invalid",  "Purported {0} with value, {1}, is invalid!, {2}!" },
        // Purported <element type> with value, <bad value>, is invalid! <specfic error message>!
    
        //
        // Models for element errors
        //
        { "elementInvalid", "Element, {0} is invalid because {1}!" },
        { "elementUnknown", "Element, {0} is not defined, it will be ignored."},
        { "contentInvalid", "Attempt to add element {0} as content in element {1} failed with error {2}!" },
    
    
        //
        // Fragments for element errors:
        //
        { "noCharactersInModel", "element cannot contain characters" },
        { "unimplemented", "element is not implemented" },
    
        //
        // Fragments for attribute errors
        //
        { "attributeUnknown",      "Attribute with name {0}, and value {1} on element, {2}, is unknown."},
        { "attributeTypeInvalid",  "Attribute, {0}, must have {1} type."},
        { "attributeRangeInvalid", "Attribute, {0}, with value {1} must be in range {2} to {3}." },
        { "attributeEnumInvalid",  "Attribute, {0}, with value {1} is not a valid choice." },
        { "attributeInvalid",      "Attribute, {0}, with value {1} is invalid because: {2}." },
        { "attributeMissing",      "Attribute, {0}, on element {1} is required and is missing!" },
        { "attributeWarnMissing",  "Attribute, {0}, on element {1} is missing. {2} "},
        //
        // Fragments for content model errors
        //
        { "UnknownElementReturned", "Element {0} returned a result; but is not in Element {1}'s "
            + "content model." },
        //
        // Fragments for UUIDhandler:
        //
        { "UUIDcharactersInvalid",  " Characters must be hexadecimal - or blanks " },
        //
        // Miscellaneous Fragments
        //
        { "ResponseLog",            "Response to Payload {0} Request ->|{1}|<-" },
        { "ResponseSendFailed",     "Response send for Payload {0} had I/O Error with message: {1}!"},
        //
        // header error messages
        //
        { "SenderMisMatch",         "The UUID in Sender-id is not known for the named sender" },
        { "lookupSenderByName",     "It will be looked by name." },
        { "locationRequired",       "ICE 1.1 and above require the location." }
    };
    
	static String thePackageName = "common.payloadDisassembly";
	
	public void setupLog( String thePackage )
	{
	    thePackageName = thePackage;
	}
	
	public static void log( String prototype, Object[] parms, int theErrorNumber )
	{
        LogMsg logmsg = new LogMsg( prototype , parms );
        logmsg.prolog( thePackageName, theErrorNumber, ImpactRange.getImpact(theErrorNumber) );
        logmsg.log();
        if ( theErrorNumber > ImpactRange.impactOf("serious") )
            logmsg.flush(); // when this should be done!
    }
	
	public static void log( String prototype, Object[] parms, int theErrorNumber, 
	                 String theErrorText )
	{
        LogMsg logmsg = new LogMsg( prototype , parms );
        logmsg.prolog( thePackageName, theErrorNumber, theErrorText) ;
        logmsg.log();
        if ( theErrorNumber > ImpactRange.impactOf("serious") )
            logmsg.flush(); // when this should be done!
	}
	
    
    //
    // THE FOLLOWING ARE DEPRECATED AND SHOULD BE CHANGED ASAP TO USE THE LOCALIZABLE EQUIVALENTS
    // $FIX$
    
    //
    // generic payload error messages
    //
    public static final String invalid = "Purported {0} with value, {1}, is invalid!, {2}!";
    // Purported <element type> with value, <bad value>, is invalid! <specfic error message>!
    
    //
    // Models for element errors
    //
    public static final String elementInvalid = "Element, {0} is invalid because {1}!";
    public static final String elementUnknown = "Element, {0} is not defined, it will be ignored.";
    
    
    //
    // Fragments for element errors:
    //
    public static final String noCharactersInModel = "element cannot contain characters";
    public static final String unimplemented = "element is not implemented";
    
    //
    // Fragments for attribute errors
    //
    public static final String attributeUnknown = "Attribute with name {0}, and value {1} on element, {2}, is unknown.";
    public static final String attributeTypeInvalid = "Attribute, {0} must have {1} type.";
    public static final String attributeRangeInvalid = "Attribute, {0} with value {1} "
        + "must be in range {2} to {3}.";
    public static final String attributeEnumInvalid = "Attribute, {0} with value {1} is not "
        + "a valid choice.";
    public static final String attributeInvalid = "Attribute, {0} with value {1} is invalid "
        + "because: {2}.";
    
    //
    // Fragments for content model errors
    //
    public static final String UnknownElementReturned = "Element {0} returned a result; but is not in Element {1}'s "
            + "content model.";
    //
    // Fragments for UUIDhandler:
    //
    public static final String UUIDcharactersInvalid = " Characters must be hexadecimal - or blanks ";
}