/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/payloadDisassembly/HeaderHandler.java,v 1.8 2000/12/02 23:28:45 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.payloadDisassembly;

import java.util.*;

//
// icecubes  imports
//
import com.adobe.rice.IceCubes;
import com.adobe.rice.common.*;
import com.adobe.rice.common.policies.*;
import com.adobe.rice.common.XMLdisassembly.*;
import com.adobe.rice.common.payloadDisassembly.request.*;
import com.adobe.rice.common.payloadDisassembly.*;
import com.adobe.rice.common.storage.logging.*;     // so we can write to the log.

public class HeaderHandler extends Handler
{
    //
    // class wide variables
    //
    static final int SENDER = 0;
    static final int RECEIVER = 1;
    static final int USERAGENT = 2;
    //
    // Instance variables
    //
	SenderData sender;
	ReceiverData receiver;
	UserAgentData userAgent;
	
	HeaderData header;
	
	//
	// constructors
	//
	
	private boolean isAuthorized ()
	{
	    boolean result = true;
	    // System.out.println ("A CALL TO SUBSCRIPTION MANAGER GOES HERE");
	    return true;
	}
	

	public HeaderHandler (XMLDisassembler disassembler, Handler parent) 
	{
		super (disassembler, parent );

		/*
            <!ELEMENT ice-header (ice-sender, ice-receiver? , ice-user-agent? ) >	
		*/
		
		this.element = "ice-header";
		
		ParserAttributes al = new ParserAttributes();
		
		// al.add( new AttributeEntry("ModelAttribute", "CDATA" ) );
		
		this.elementAttributes = al;
	}
	
	public void startElement(String namespaceURI, String localName, 
                             String name, ParserAttributes attributes) 
	{
        TagEntry contentTags[] = null; 
	    
	    // debug (name + " START ELEMENT<br>");
	    this.setupAttributes(attributes);
		/*
            <!ELEMENT ice-header (ice-sender, ice-receiver? , ice-user-agent? ) >	
		*/
		// make sure content handlers are registered.
		// do this as late as possible
		contentTags = new TagEntry[] 
		{
		    // create a TagEntry for each element in ModelContent's content model here.
		    // Note that the order of the TagEntrys corresponds to the indexes used
		    // below -- if TAGNAME is the tag name in the first TagEntry, then it is
		    // index 0 below in result.
		    //
		    // replace the UnimplementedHandler in each TagEntry with an appropriate handler when you implement it.
            new TagEntry("ice-sender", new SenderHandler (disassembler, this)),
            new TagEntry("ice-receiver", new ReceiverHandler( disassembler, this)),
            new TagEntry("ice-user-agent", new UserAgentHandler( disassembler, this))
		};
		this.setContent( new Content(contentTags, disassembler) );
		//
		// Begin Semantic Processing Here:
		//
	}
	
	public void endElement(String namespaceURI, String localName, 
                           String name) 
	{
	    // debug (name + " END ELEMENT<br>");
	    
	    //
	    // Complete Semantic Processing Here:
	    //
	    
	    if ( isAuthorized() )
	    {
	        if ( this.sender == null )
	        {
	            // A null sender indicates a 301 level error.
	            // We should always be able to obtain the header.
	        }
	    }
	    else
	    {
	        // An unauthorized Payload indicates a "405 Unrecognized sender"
	        // message.  
	        
	    }
	    
	    //
	    // construct headerData for the rest of processing
	    //
	    header = new HeaderData( new Date(), this.sender, this.receiver, this.userAgent );
	    //
	    // construct the final result array of objects for return to the 
	    // parent here.  By default, send back this object.
	    //
	    Object[] results = 
	    { 
	        new Integer(this.getErrorCount()),
	        new Integer(this.getErrorLevel()),
	        this.header 
	    };
	    // next-to-last - call the result 
	    disassembler.result( name, results );
	    // last - unregister all my content handlers.
	    this.getContent().clearHandlers();
	}
    /**
	* Function called for element character data.
	*
	* @param    name    String containing name of element chardata is for.
	* @param	ch		array of character data.
	* @param	start	begining index of character data.
	* @param	length	length of character data.
	*
	* @see		SAXManager
     **/
	public void characters( String name, char ch[], int start, int length) 
	{
	    if ( name.equals(this.element) )
	    {
	        // Content Model of a Payload header cannot contain characters.
            Object[] parms = { name, DisassemblyMessages.noCharactersInModel };    
            LogMsg logmsg = new LogMsg( DisassemblyMessages.attributeUnknown , parms );
            logmsg.prolog( "common.payloadDisassembly", 200000000, "warning" );
            logmsg.log();
	    }
    }
    /**
     * Function called when a sub-handler completes and returns it's work.
	 *
	 * @param   name        String containing the name of an element just completed.
	 * @param   objectArray Array of objects, with are the result of the element just
	 *                      completed.  Presumably, the called handler's result 
	 *                      method knows how to handle the objects in the array,
	 *                      with the name of the completed element being the
	 *                      distinguishing hint.    
	 *
	 **/
	public void result( String name, Object[] objectArray ) 
    {
        int i = content.index(name);
        if ( i >= 0 )
        {
            Integer elementErrorCount = (Integer) objectArray[0];
            Integer elementErrorLevel = (Integer) objectArray[1];
                
            this.bumpErrorCount(elementErrorCount.intValue() );
            this.adjustErrorLevel(elementErrorLevel.intValue() );
        }
      
        switch ( i )
        {
            // When an element in the content of this element completes, it calls this 
            // routine with it's results.  That is if element foo has element bar as
            // content, then when element bar contained by foo completes, the result
            // is handed back to foo through this routine.
            //
            // Handle the results here.  Note -- The theory is that this handler is
            // little more than a packager of the effort of the lower level routines.
            // Build the lower level routines to handle everything that is locally possible
            // about the element.  The result routine (this one) should be designed to
            // integrate the lower level elements into a larger whole.  At this level,
            // we can do little but schedule the returning response.
            
            // The index here is the index of the elements in contentTags with the
            // first starting at 0.
            
            case SENDER: // ice-sender
                this.sender = (SenderData) objectArray[2];
                break;
            case RECEIVER: // ice-receiver
                this.receiver = (ReceiverData) objectArray[2];
                break;
            case USERAGENT: // ice-user-agent
                this.userAgent = (UserAgentData) objectArray[2];
                break;
            default: // unknown name
	            // 
                Object[] parms = { name, this.element };    
                LogMsg logmsg = new LogMsg( DisassemblyMessages.UnknownElementReturned , parms );
                logmsg.prolog( "common.payloadDisassembly", 200000000, "warning" );
                logmsg.log();
                // if the message is important enough to want to assure that it is written:
                // logmsg.flush(); 
                break;
        }
    }
}
