/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/payloadDisassembly/PayloadHandler.java,v 1.8 2000/12/02 23:28:45 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.payloadDisassembly;

//
// Java libraries
//
import java.io.*;

//
// External Libraries
//

//
// icecubes  imports
//
import com.adobe.rice.util.*;
import com.adobe.rice.IceCubes;
import com.adobe.rice.common.*;
import com.adobe.rice.common.policies.*;
import com.adobe.rice.common.XMLassembly.*;
import com.adobe.rice.common.XMLdisassembly.*;
import com.adobe.rice.common.transportation.*;
import com.adobe.rice.common.payloadAssembly.*;
import com.adobe.rice.common.payloadAssembly.responses.*;
import com.adobe.rice.common.payloadDisassembly.*;
import com.adobe.rice.common.storage.logging.*;     // so we can write to the log.
//import com.adobe.rice.kernel.*;


public class PayloadHandler extends Handler 
{
    //
    // Class wide variables
    //
		/*
            <!ELEMENT ice-payload ( ice-header,  
                                       ( ice-request+ |  
                                         ice-response+ | 
                                         ice-unsolicited-now | 
                                         ice-unsolicited-request+ |  
                                         ice-unsolicited-response+ 
                                       ) 
                                     ) 
            >  
		*/
    static final int HEADER = 0;
    static final int REQUEST = 1;
    static final int RESPONSE = 2;
    static final int UNSOLICITEDNOW = 3;
    static final int UNSOLICITEDREQUEST = 5;
    static final int UNSOLICITEDRESPONSE = 6;
    
    //
    // Private instance variables:
    //
	private	String		routing;
	
	private HeaderData  theHeaderData;
	
	private PayloadDoc  theResponsePayloadDoc;
	
	private String      theICEversion;
	
	private String      thePayloadID;
	
	private Integer     payloadLevelError = new Integer( 0 );
	
	private boolean     thisIsRequest = false;
	
	private boolean     thisIsResponse = false;
	
	private int         logLevel = 0; 
	
	

	public PayloadHandler(XMLDisassembler disassembler, String routing) 
	{		
		//  The payload handler is the root element of ICE payloads;
		//  therefore it has no parent so we call super with a null to indicate
		//  that this handler is the root document deconstruction handler.
		super(disassembler, null);
		
		this.routing = routing;

		/*
            <!ELEMENT ice-payload ( ice-header,  
                                       ( ice-request+ |  
                                         ice-response+ | 
                                         ice-unsolicited-now | 
                                         ice-unsolicited-request+ |  
                                         ice-unsolicited-response+ 
                                       ) 
                                     ) 
            >  
            <!ATTLIST ice-payload  
                      payload-id      CDATA #REQUIRED 
                      timestamp       CDATA #REQUIRED 
                      ice.version     CDATA #REQUIRED 
            >		
		*/
		
		this.element = "ice-payload";
		
		ParserAttributes al = new ParserAttributes();
		
		al.add( new AttributeEntry("payload-id", "CDATA" ) );
		al.add( new AttributeEntry("timestamp", "CDATA" ) );
		al.add( new AttributeEntry("ice.version", "CDATA" ) );
		
		this.elementAttributes = al;
		
		this.logLevel = (( Integer ) PolicyManager.instance().getSelection("LoggingLevel")).intValue();
		
		// 
		// the payload handler is unique in that it includes handlers for sax parsing errors.
		// (of course any of the children may register handlers as well.)
		//
		// warning
		// error
		// fatal
		//
		
		
	}
	
	public void startElement(String namespaceURI, String localName, 
                             String name, ParserAttributes attributes) 
	{
        TagEntry contentTags[] = null; 
        
        this.setupAttributes(attributes);
        
        this.theICEversion = this.getAttributeValue("ice.version");
        this.thePayloadID = this.getAttributeValue("payload-id");
        
		/*
            <!ELEMENT ice-payload ( ice-header,  
                                       ( ice-request+ |  
                                         ice-response+ | 
                                         ice-unsolicited-now | 
                                         ice-unsolicited-request+ |  
                                         ice-unsolicited-response+ 
                                       ) 
                                     ) 
            >  
		
		Note that payloads are homogeneous, in the sense that elements from the above list 
		MUST NOT be mixed together in a single payload. For example, a single payload can 
		contain multiple ice-request elements, but it cannot contain
        ice-request elements and ice-response elements. The DTD representation shown above 
        enforces this constraint.
		
		*/
		//
		// now build a response payload
		//
		this.theResponsePayloadDoc = PayloadFactory.instance().get( this.theICEversion );
		//
		// Make sure we have a payload response
		//
		if ( this.theResponsePayloadDoc == null )
		{
		    //
		    // This is an ICE 320 error, Incompatible version 
		    //
		    this.payloadLevelError = new Integer(320);
		    //
		    // Just get a standard version: 
		    //
		    this.theResponsePayloadDoc = PayloadFactory.instance().get();
		    if ( this.theResponsePayloadDoc == null )
		    {
		        //
		        // ICE cubes is in terrible trouble.  this should NEVER happen
		        // it means that we cannot construct a standard payload!
		        // This is a fatal error.  We must
		        // 1.  log the occurrence.
		        // 2.  Abort the connection
		        // 3.  Notify the operator
		        // 4.  shutdown ICE cubes.
		        // $LOG$ - why couldn't we add a code?
                Object parms[] = {this.thePayloadID, this.theICEversion };
                DisassemblyMessages.log(
                    AssemblyMessages.get("NoResponseDoc"), 
                    parms, ImpactRange.FATAL );
                // $FIX$ we should shutdown now and figure out why this is broken.
		    }
		    //
		    // build a payload level 320 response 
		    //
		    this.makeResponse( null, "320" );
		}
		//
		// setup initial preparations
		//
		this.theHeaderData = null;  // we don't have a header yet!
		
		//
		// setup the preparation for all of the children.  From ICE's perspective,
		// all the children need to know about are the parent element and the
		// header data of the response document.  The header data is needed to lookup 
		// the sender so that processing can be constrained to the sender's relationship 
		// with this instance of icecubes.  The parent element is needed in case a new
		// sub-element of the parent needs to be added.  This enforces the parallel
		// construction of requests/responses and assures that there is a one-for-one
		// relationship between requests and response and that one request/response pair 
		// cannot alter another.
		//
		// implementation note:  We rely on Java's pass by reference to send the
		// (eventual) header data when a header is processed.  That is, the
		// HeaderData object won't actually be set until the header is processed;
		// however we need to initialize all of the children now; so send them the
		// reference and by the time any of the using tags require header data, it
		// will be set through the magic of references.
		// <aside>
		// Technically, since the content model of a payload requires that the
		// header appear first, we could defer registering the request, response
		// and unsolicited tags until we process the header.  Leave that for
		// a future optimization. $FUTURE$
		// </aside>
		//
		Object prep[] = 
		{ 
		    this.payloadLevelError,
		    this.theHeaderData,
		    this.theResponsePayloadDoc.getRoot()
		};
		this.setPrepared(prep);
		// make sure content handlers are registered.
		// do this as late as possible
		contentTags = new TagEntry[] 
		{
		    // create a TagEntry for each element in ModelContent's content model here.
		    // Note that the order of the TagEntrys corresponds to the indexes used
		    // below -- if TAGNAME is the tag name in the first TagEntry, then it is
		    // index 0 below in result.
		    //
		    // replace the UnimplementedHandler in each TagEntry with an appropriate handler when you implement it.
            new TagEntry("ice-header", new HeaderHandler(disassembler, this)),
            new TagEntry("ice-request", new RequestHandler( disassembler, this)),
            new TagEntry("ice-response", new ResponseHandler( disassembler, this)),
            new TagEntry("ice-unsolicited-now",  new UnsolicitedNowHandler( disassembler, this)),
            new TagEntry("ice-unsolicited-request",  new UnsolicitedRequestHandler( disassembler, this)),
            new TagEntry("ice-unsolicited-response",  new UnsolicitedResponseHandler( disassembler, this))
		};
		this.setContent( new Content(contentTags, disassembler));
	}
	
	
	public void endElement(String namespaceURI, String localName, 
                             String name ) 
	{
	    //
	    // Complete Semantic Processing Here:
	    //
	    /*
	    Depending on the result, the payload handler should.
	    
	    1.  if this payload is a request payload, and the response
	        document is properly built, send back the repsonse document.
	        
	    2.  if this payload is a response payload, find the associated
	        ask and let it process the response data.
	    
	    */
	    if ( this.thisIsRequest )
	    {
	        // send the response document back!
	        Writer out = this.disassembler.getOutput();
	        
	        String packetResponse = this.theResponsePayloadDoc.toXMLstring();
	        
	        if ( logLevel > DisassemblyConstants.LOG_PACKETS )
	        {
	            Object[] parms = { this.thePayloadID, packetResponse };
	            DisassemblyMessages.log(
	                DisassemblyMessages.get("ResponseLog"), 
	                parms, 
	                ImpactRange.INFORMATION
	            );
	        }
	        try { out.write( packetResponse );  out.close(); }
	        catch ( IOException ioe ) 
	        { 
	            // log the error -- presumably it is a connection failure
	            // so we hope that it is a serious, but recoverable error.
	            // we can also predice that the caller will attempt one more
	            // time.
	            Object[] parms = { this.thePayloadID, ioe.toString() };
	            DisassemblyMessages.log(
	                DisassemblyMessages.get("ResponseSendFailed"),
	                parms,
	                ImpactRange.SERIOUS
	            );
	        }

	    }
	    else if ( thisIsResponse )
	    {
	        // we have processed the response and now we can retire this
	        // response.
	    }
	    
	    //
	    // construct the final result array of objects for return to the 
	    // parent here.  By default, send back this object.
	    //
	    Object[] results = 
	    { 
	        new Integer(this.getErrorCount()),
	        new Integer(this.getErrorLevel()),
	        this 
	    };
	    
	    // next-to-last - call the result 
	    disassembler.result( name, results );
	    // last - unregister all my content handlers.
	    this.getContent().clearHandlers();
	}
    /**
	* Function called for element character data.
	*
	* @param    name    String containing name of element chardata is for.
	* @param	ch		array of character data.
	* @param	start	begining index of character data.
	* @param	length	length of character data.
	*
	* @see		SAXManager
     **/
	public void characters( String name, char ch[], int start, int length) 
	{
	    if ( name.equals(this.element) )
	    {
	        // Content Model of an ice-payload cannot contain characters.
            Object[] parms = { name, DisassemblyMessages.noCharactersInModel };    
            LogMsg logmsg = new LogMsg( DisassemblyMessages.attributeUnknown , parms );
            logmsg.prolog( "common.payloadDisassembly", 200000000, "warning" );
            logmsg.log();
	    }
    }
    /**
     * Function called when a sub-handler completes and returns it's work.
	 *
	 * @param   name        String containing the name of an element just completed.
	 * @param   objectArray Array of objects, with are the result of the element just
	 *                      completed.  Presumably, the called handler's result 
	 *                      method knows how to handle the objects in the array,
	 *                      with the name of the completed element being the
	 *                      distinguishing hint.    
	 *
	 **/
	public void result( String name, Object[] objectArray ) 
    {
        int i = content.index(name);
        if ( i >= 0 )
        {
            Integer elementErrorCount = (Integer) objectArray[0];
            Integer elementErrorLevel = (Integer) objectArray[1];
                
            this.bumpErrorCount(elementErrorCount.intValue() );
            this.adjustErrorLevel(elementErrorLevel.intValue() );
        }
        if ( this.payloadLevelError.intValue() == 0 )
        switch ( i )
        {
            // When an element completes, it calls this routine with it's results.
            // Handle the results here.  Note -- The theory is that this handler is
            // little more than a packager of the effort of the lower level routines.
            // Build the lower level routines to handle everything that is locally possible
            // about the element.  The result routine (this one) should be designed to
            // integrate the lower level elements into a larger whole.  At this level,
            // we can do little but schedule the returning response.
            
            case HEADER: // ice-header
                // grab the header
                theHeaderData = (HeaderData) objectArray[2];
                // 
                // now, all the children must be able to access the header data
                //
		        Object prep[] = 
		        { 
		            this.payloadLevelError,
		            this.theHeaderData,
		            this.theResponsePayloadDoc.getRoot()
		        };
		        this.setPrepared(prep);
                break;
            case REQUEST: // ice-request
                this.thisIsRequest = true;
                break;
            case RESPONSE: // ice-response
                this.thisIsResponse = true;
                break;
            case UNSOLICITEDNOW: // ice-unsolicited-now
                break;
            case UNSOLICITEDREQUEST: // ice-unsolicited-request
                break;
            case UNSOLICITEDRESPONSE: // ice-unsolicited-response
                break;
            default: // unknown name
	            // 
                Object[] parms = { name, this.element };    
                LogMsg logmsg = new LogMsg( DisassemblyMessages.UnknownElementReturned , parms );
                logmsg.prolog( "common.payloadDisassembly", 200000000, "warning" );
                logmsg.log();
                // if the message is important enough to want to assure that it is written:
                // logmsg.flush(); 
                break;
        }
    }
    
	/*-------------------------------------------------------------------+
     |                                                                   |
	 |             E r r o r H a n d l e r  I n t e r f a c e            |
	 |                                                                   |
	 +-------------------------------------------------------------------*/
		 
	/**
     * Receive notification of a warning.
     *
     * SAX parsers will use this method to report conditions that are 
     * not errors or fatal errors as defined by the XML 1.0 recommendation. 
     * The default behaviour is to take no action.
     *
     * The SAX parser must continue to provide normal parsing events 
     * after invoking this method: it should still be possible for the 
     * application to process the document through to the end.
     *
     * @param localName - The local name of the tag - active with the warning.
     * @param exception - The warning information encapsulated in a SAX 
     *                    parse exception.
     *
     * @return boolean - returns true to indicate it is ok to bubble up the error.
     * 
     * @exception SAXException - Any SAX exception, possibly wrapping another exception.
     * 
     * @see SAXParseException
     *
	 **/
    public boolean warning( String name, Exception exception )
    {
        // we should log the error 
        Object[] parms = new Object[] { name, exception.getMessage() };    
        DisassemblyMessages.log( 
            DisassemblyMessages.get("elementInvalid"),
            parms,
            ImpactRange.WARNING 
        );
        return false;       // the bubble buck stops here.
    }
    /**
     * Receive notification of a recoverable error.
     *
     * This corresponds to the definition of "error" in section 1.2 of 
     * the W3C XML 1.0 Recommendation. For example, a validating parser 
     * would use this callback to report the violation of a validity 
     * constraint. The default behaviour is to take no action.
     *
     * The SAX parser must continue to provide normal parsing events 
     * after invoking this method: it should still be possible for the 
     * application to process the document through to the end. If the 
     * application cannot do so, then the parser should report a fatal 
     * error even if the XML 1.0 recommendation does not require it to 
     * do so.
     * 
     * @param name - The local name of the tag - active with the error.
     * @param exception - The error information encapsulated in a SAX parse exception.
     *
     * @return boolean - returns true to indicate it is ok to bubble up the error.
     * 
     * @exception SAXException - Any SAX exception, possibly wrapping another exception.
     *
     * @see SAXParseException
     *
     **/
    public boolean error( String name, Exception exception )
    {
        // This is the place to put the following ICE errors:
        // 303 - validation failure
        // 400 - 407 errors (these are at the request/response level or below)
        // we should log the error 
        Object[] parms = new Object[] { name, exception.getMessage() };    
        DisassemblyMessages.log( 
            DisassemblyMessages.get("elementInvalid"),
            parms,
            ImpactRange.SERIOUS
        );
        //
        // The parser will attempt to carry on from here.
        // We should note the error and continue.
        return false;   // the bubble buck stops here
    }
    
    /**
     * Receive notification of a non-recoverable error.
     *
     * This corresponds to the definition of "fatal error" in section 
     * 1.2 of the W3C XML 1.0 Recommendation. For example, a parser would 
     * use this callback to report the violation of a well-formedness 
     * constraint.
     *
     * The application must assume that the document is unusable after
     * the parser has invoked this method, and should continue (if at 
     * all) only for the sake of collecting addition error messages: 
     * in fact, SAX parsers are free to stop reporting any other events 
     * once this method has been invoked.
     *
     * @param name - The local name of the tag - active with the fatal error.
     * @param exception - The fatal error information encapsulated in a SAX parse exception.
     *
     * @return boolean - returns true to indicate it is ok to bubble up the error.
     * 
     * @exception SAXException - Any SAX exception, possibly wrapping another exception.
     *
     * @see SAXParseException
     *
     **/
        
    public boolean fatalError( String name, Exception exception )
    {
        // This is the place to put the following ICE errors:
        // 301 - incomplete/cannot parse
        // 302 - well formedness errors
        Object[] parms = new Object[] { name, exception.getMessage() };    
        DisassemblyMessages.log( 
            DisassemblyMessages.get("elementInvalid"),
            parms,
            ImpactRange.CRITICAL
        );
        //
        // now, we should attempt to set the code and return the response
        // if a request is underway.
        //
        // Note that if this routine is called, the parse of the incomming
        // document has likely terminated.  (is this true for xerces?)
        //
        return false;   // the bubble buck stops here.
    }
    
	/*-------------------------------------------------------------------+
     |                                                                   |
	 |                    U T I L I T Y   R O U T I N E S                |
	 |                                                                   |
	 +-------------------------------------------------------------------*/
		 
    
    private void makeResponse( String messageId, String numericCode )
    {
		    AssemblyElement     root = (AssemblyElement) this.theResponsePayloadDoc.getRoot();
		    DocType             dtd  = this.theResponsePayloadDoc.getDtd();
		    Response            response;
		    Code                code = null;
		    
		    this.theResponsePayloadDoc.open();
		        
		        try
		        {
		            response = new Response( root, dtd );
		            root.add(response);
		            try
		            {
		                response.open();
		                    code = new Code(response, dtd);
		                    response.add( code );
		                response.close();
		            }
		            catch ( ElementException ele ) 
		            {
		                // $LOG$ - why couldn't we add a code?
                        Object parms[] = {"ice-code", "ice-response", ele.getMessage() };
                        DisassemblyMessages.log(
                            DisassemblyMessages.get("contentInvalid"), 
                            parms, ImpactRange.FATAL );
                        // $FIX$ we should shutdown now and figure out why this is broken.
		            }
		        }
		        catch ( ElementException ele )
		        {
		            // $LOG$ - why couldn't we add a response?
                    Object parms[] = {"ice-response", "ice-payload", ele.getMessage() };
                    DisassemblyMessages.log(
                        DisassemblyMessages.get("contentInvalid"), 
                        parms, ImpactRange.FATAL );
                    // $FIX$ we should shutdown now and figure out why this is broken.
		        }
		        code.setPhrase( IceCodes.get(numericCode) );
		        code.setNumeric( numericCode );
		        code.setMessageId( messageId );
		        //
		        // According to the ICE spec (3.4.2 ), expressing the payloadId in 
		        // the code is conditional.  This is handled by the code object.
		        // The payload handler ALWAYS sets the value.
		        // 
		        code.setPayloadId( this.thePayloadID );
		        
		    this.theResponsePayloadDoc.close();
    }

}






