/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/payloadDisassembly/RequestHandler.java,v 1.8 2000/12/02 23:28:45 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.payloadDisassembly;

//
// Java Libraries:
//

import java.util.*;

//
// icecubes  imports
//
import com.adobe.rice.IceCubes;

import com.adobe.rice.util.*;
import com.adobe.rice.common.*;
import com.adobe.rice.common.policies.*;

import com.adobe.rice.common.XMLassembly.*;
import com.adobe.rice.common.payloadAssembly.*;
import com.adobe.rice.common.payloadAssembly.responses.*;

import com.adobe.rice.common.XMLdisassembly.*;
import com.adobe.rice.common.payloadDisassembly.*;
import com.adobe.rice.common.payloadDisassembly.request.*;
import com.adobe.rice.common.payloadDisassembly.response.*;

import com.adobe.rice.common.storage.logging.*;     // so we can write to the log.
//import com.adobe.rice.kernel.*;


public class RequestHandler extends Handler 
{
    //
    // class wide variables
    //
	/*
	<!ELEMENT ice-request  ( ice-cancel |
                             ice-change-subscription |
                             ice-code |
                             ice-get-catalog |
                             ice-get-events |
                             ice-get-package |
                             ice-get-sequence |
                             ice-get-status |
                             ice-nop |
                             ice-notify |
                             ice-offer |
                             ice-package+ |
                             ice-send-confirmations |
                             ice-repair-item 
                           )
        >
    */
    
    /* NOTE - Merge this with my constants */
    
    static final int CANCEL = 0;
    static final int CHANGESUBSCRIPTION = 1;
    static final int CODE = 2;
    static final int GETCATALOG = 3;
    static final int GETEVENTS = 4;
    static final int GETPACKAGE = 5;
    static final int GETSEQUENCE = 6;
    static final int GETSTATUS = 7;
    static final int NOP = 8;
    static final int NOTIFY = 9;
    static final int OFFER = 10;
    static final int PACKAGE = 11;
    static final int SENDCONFIRMATIONS = 12;
    static final int REPAIRITEM = 13;
   
    //
    // instance variables:
    //
    
    String theRequestId = null;
   
    Integer payloadLevelError = null;
    HeaderData headerData = null;
    Payload responsePayload = null;
        
    Integer responseLevelError = null;
    Response response = null;
    Code code = null;
    
    //
    // constructors
    //
	
	public RequestHandler(XMLDisassembler disassembler, Handler parent) 
	{
		super(disassembler, parent);
		
		this.element = "ice-request";
		
		/*
		<!ATTLIST ice-request  request-id  CDATA #REQUIRED  >
		*/
		
		ParserAttributes al = new ParserAttributes();
		
		al.add( new AttributeEntry("request-id", "CDATA" ) );
		
		this.elementAttributes = al;
		
		// 
	}
	
	public void startElement(String namespaceURI, String localName, 
                             String name, ParserAttributes attributes) 
    {
        TagEntry contentTags[] = null; 
        
        this.setupAttributes(attributes);
        // make sure content handlers are registered.
		// do this as late as possible
		/*
		<!ELEMENT ice-request  ( ice-cancel |
                                 ice-change-subscription |
                                 ice-code |
                                 ice-get-catalog |
                                 ice-get-events |
                                 ice-get-package |
                                 ice-get-sequence |
                                 ice-get-status |
                                 ice-nop |
                                 ice-notify |
                                 ice-offer |
                                 ice-package+ |
                                 ice-send-confirmations |
                                 ice-repair-item 
                               )
         >
        */
		//
		// pull in the response payload 
		//      and information about this payload
		//		
		Object[] parentPrep = this.parent.getPrepared();				
		// the objects in the parentPrep array are:
		// payloadLevelError,
		// HeaderData,
		// Payload
		// 
		this.payloadLevelError = (Integer)    parentPrep[ 0 ];
		this.headerData        = (HeaderData) parentPrep[ 1 ];
		this.responsePayload   = (Payload)    parentPrep[ 2 ];
		
		//
		// now construct a response for this request
		//
		// See "2.3.3 Multiple Requests in a Single Payload" in the ICE spec for the
		// requirements.  We enforce restriction 3 here by constructing a response
		// element in the response document for each request.
		//
		// Setup the message identifer to be the request-id as defined by
		// section 3.4.2, message-id: CONDITIONAL. The request-id of the request referenced 
		// by this code, ...
		// A Sender MUST supply this in all cases except when reporting a payload level
        // (3xx) code. (which is handled above this case.
        //
        // assume that the request will go well and setup a default
        // code of 200 (ok)!
        //
		this.makeResponse(this.getAttributeValue("request-id"), "200");
		//
		// now prepare for the children
		//
		// the objects in the Prep array are:
		// payloadLevelError,
		// theHeaderData,
		// response
		// code
		Object prep[] =
		{
		    this.responseLevelError,
		    this.headerData,
		    this.response,
		    this.code
		};
		this.setPrepared(prep);
		//
		contentTags = new TagEntry[] 
		{
		    // create a TagEntry for each element in ModelContent's content model here.
		    // Note that the order of the TagEntrys corresponds to the indexes used
		    // below -- if TAGNAME is the tag name in the first TagEntry, then it is
		    // index 0 below in result.
		    //
		    // replace the UnimplementedHandler below with an appropriate handler when you implement it.
            new TagEntry("ice-cancel", new CancelHandler( disassembler, this)),
            new TagEntry("ice-change-subscription", new ChangeSubscriptionHandler( disassembler, this)),
            new TagEntry("ice-code", new CodeHandler( disassembler, this)),
            new TagEntry("ice-get-catalog",  new GetCatalogHandler( disassembler, this)),
            new TagEntry("ice-get-events",  new GetEventsHandler( disassembler, this)),
            new TagEntry("ice-get-package",  new GetPackageHandler( disassembler, this)),
            new TagEntry("ice-get-sequence",  new GetSequenceHandler( disassembler, this)),
            new TagEntry("ice-get-status",  new GetStatusHandler( (XMLDisassembler) disassembler, (Handler) this)),
            new TagEntry("ice-nop", new NOPHandler(disassembler, this)),
            new TagEntry("ice-notify",  new NotifyHandler( disassembler, this)),
            new TagEntry("ice-offer",  new OfferHandler( disassembler, this)),
            new TagEntry("ice-package",  new PackageHandler( disassembler, this)),
            new TagEntry("ice-send-confirmations",  new SendConfirmationsHandler( disassembler, this)),
            new TagEntry("ice-repair-item",  new RepairItemHandler( disassembler, this))
		};
		this.setContent( new Content(contentTags, disassembler));
	}
	
	public void endElement(String namespaceURI, String localName, String name ) 
    {
        
	    //
	    // construct the final result array of objects for return to the 
	    // parent here.  By default, send back this object.
	    //
	    Object[] results = 
	    { 
	        new Integer(this.getErrorCount()),
	        new Integer(this.getErrorLevel()),
	        this 
	    };
	    
	    if (messagesPending ())
	    {
	       //response.setUnsolicitedFlag ();   
	    }
	    
	    //logIceEvent (request, response);
	   
	   
	    // debug (name + " END ELEMENT<br>");
	    // next-to-last - call the result 
	    disassembler.result( name, results );
	    // last - unregister all my content handlers.
	    this.getContent().clearHandlers();
	}
    /**
	* Function called for element character data.
	*
	* @param    name    String containing name of element chardata is for.
	* @param	ch		array of character data.
	* @param	start	begining index of character data.
	* @param	length	length of character data.
	*
	* @see		SAXManager
     **/
	public void characters( String name, char ch[], int start, int length) 
	{
	    if ( name.equals(this.element) )
	    {
	        // Content Model of an ice-request  cannot contain characters.
            Object[] parms = { name, DisassemblyMessages.noCharactersInModel };    
            LogMsg logmsg = new LogMsg( DisassemblyMessages.elementInvalid , parms );
            logmsg.prolog( "common.payloadDisassembly", ImpactRange.WARNING, "warning" );
            logmsg.log();
	    }
    }
    /**
     * Function called when a sub-handler completes and returns it's work.
	 *
	 * @param   name        String containing the name of an element just completed.
	 * @param   objectArray Array of objects, with are the result of the element just
	 *                      completed.  Presumably, the called handler's result 
	 *                      method knows how to handle the objects in the array,
	 *                      with the name of the completed element being the
	 *                      distinguishing hint.    
	 *
	 **/
	public void result( String name, Object[] objectArray ) 
    {
        int i = content.index(name);
        if ( i >= 0 )
        {
            Integer elementErrorCount = (Integer) objectArray[0];
            Integer elementErrorLevel = (Integer) objectArray[1];
                
            this.bumpErrorCount(elementErrorCount.intValue() );
            this.adjustErrorLevel(elementErrorLevel.intValue() );
        }
        switch ( i )
        {
            // When an element completes, it calls this routine with it's results.
            // Handle the results here.  Note -- The theory is that this handler is
            // little more than a packager of the effort of the lower level routines.
            // Build the lower level routines to handle everything that is locally possible
            // about the element.  The result routine (this one) should be designed to
            // integrate the lower level elements into a larger whole.  At this level,
            // we can do little but schedule the returning response.
	        /*
	        <!ELEMENT ice-request  ( ice-cancel |
                                    ice-change-subscription |
                                    ice-code |
                                    ice-get-catalog |
                                    ice-get-events |
                                    ice-get-package |
                                    ice-get-sequence |
                                    ice-get-status |
                                    ice-nop |
                                    ice-notify |
                                    ice-offer |
                                    ice-package+ |
                                    ice-send-confirmations |
                                    ice-repair-item 
                                )
                >
            */
            case CANCEL:                // ice-cancel
                break;
            case CHANGESUBSCRIPTION:    // ice-change-subscription
                break;
            case CODE:                  // ice-code 
                break;
            case GETCATALOG:            // ice-get-catalog
                break;
            case GETEVENTS:             // ice-get-events
                break;
            case GETPACKAGE:            // ice-get-package 
                break;
            case GETSEQUENCE:           // ice-get-sequence
                break;
            case GETSTATUS:             // ice-get-status 
                break;
            case NOP:                   // ice-nop
                break;
            case NOTIFY:                // ice-notify
                break;
            case OFFER:                 // ice-offer
                break;
            case PACKAGE:               // ice-package
                break;
            case SENDCONFIRMATIONS:     // ice-send-confirmations
                break;
            case REPAIRITEM:            // ice-repair-item
                break;
            default:                    // unknown name
	            // 
                Object[] parms = { name, this.element }; 
                DisassemblyMessages.log(
                    DisassemblyMessages.get("UnknownElementReturned"), 
                    parms, 
                    ImpactRange.WARNING
                );
                break;
        }
    }
    
    //
    // Request Handler Utility Routines:
    //
    
    private boolean messagesPending ()
    {
        boolean result = false;
        
        return result;
    }
    
    //private void logIceEvent (Request  request, 
    //                          Response response)
    //{
    //        
    //}
    
    private void makeResponse( String messageId, String numericCode )
    {
		    DocType             dtd  = this.responsePayload.getDtd();
		    
		    this.responsePayload.open();
		        try
		        {
		            this.response = new Response( this.responsePayload, dtd );
		            responsePayload.add(this.response);
		            try
		            {
		                this.code = new Code(this.response, dtd);
		                this.response.open();
		                    this.response.add( this.code );
		                this.response.close();
		            }
		            catch ( ElementException ele ) 
		            {
		                // $LOG$ - why couldn't we add a code?
                        Object parms[] = {"ice-code", "ice-response", ele.getMessage() };
                        DisassemblyMessages.log(
                            DisassemblyMessages.get("contentInvalid"), 
                            parms, ImpactRange.FATAL );
                        // $FIX$ we should shutdown now and figure out why this is broken.
		            }
		        }
		        catch ( ElementException ele )
		        {
		            // $LOG$ - why couldn't we add a response?
                    Object parms[] = {"ice-response", "ice-payload", ele.getMessage() };
                    DisassemblyMessages.log(
                        DisassemblyMessages.get("contentInvalid"), 
                        parms, ImpactRange.FATAL );
                    // $FIX$ we should shutdown now and figure out why this is broken.
		        }
		        this.code.setPhrase( IceCodes.get(numericCode) );
		        this.code.setNumeric( numericCode );
		        this.code.setMessageId( messageId );
		        
		        Policy language = PolicyManager.instance().getPolicy("Language");
		        this.code.setXmlLang( (String)language.getSelection() );
		        //
		        // According to the ICE spec (3.4.2 ), expressing the payloadId in 
		        // the code is conditional.  This is handled by the code object.
		        // The payload handler ALWAYS sets the value.
		        // 
		        // code.setPayloadId( this.thePayloadID );
		        
		    this.responsePayload.close();
    }
    
}
