/*
$Log: SenderHandler.java,v $
Revision 1.3  2000/12/02 23:28:44  bhunt
Latest Assembly/Disassembly updates

Revision 1.2  2000/11/27 20:16:20  bhunt
New PayloadAssembly updates

Revision 1.1  2000/10/26 19:40:12  andrewf
added message classes for sender, receiver, header
moved senderHandler and userAgentHandler to payload level

Revision 1.1  2000/09/26 05:49:52  andrewf
request handler stubs

$Date: 2000/12/02 23:28:44 $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.payloadDisassembly;

//
// java imports:
//
import java.util.*;
import java.net.*;

//
// icecubes  imports
//
import com.adobe.rice.IceCubes;
import com.adobe.rice.util.*;
import com.adobe.rice.common.*;
import com.adobe.rice.common.policies.*;
import com.adobe.rice.common.XMLdisassembly.*;
import com.adobe.rice.common.payloadDisassembly.*;
import com.adobe.rice.common.storage.logging.*;     // so we can write to the log.

public class SenderHandler extends com.adobe.rice.common.XMLdisassembly.Handler  
{
    //
    // class wide variables
    //
    
    //
    // private class instance variables
    //
    SenderData sender;
	//
	// Constructors
	//
	public SenderHandler(XMLDisassembler disassembler, Handler parent) 
	{
		super (disassembler, parent);
        // <!ELEMENT ice-sender EMPTY>

        // <!ATTLIST ice-sender
        //           name                 CDATA    #REQUIRED
        //           role                 (subscriber | syndicator) #REQUIRED
        //           sender-id            CDATA    #IMPLIED
        //           location             CDATA    #IMPLIED
       
		this.element = "ice-sender";
	
		ParserAttributes al = new ParserAttributes();
		
		al.add (new AttributeEntry("name", "CDATA" ));
		al.add (new AttributeEntry("role", "CDATA" ));
		al.add (new AttributeEntry("sender-id", "CDATA"));
		al.add (new AttributeEntry("location", "CDATA"));
		
		this.elementAttributes = al;
		
	}
	
	public void startElement(String namespaceURI, String localName, 
                             String rawName,      ParserAttributes attributes)
	{
	    //
	    // pull in the attributes
	    //
	    this.setupAttributes(attributes);
		// make sure content handlers are registered.
		// do this as late as possible
		// Sender has EMPTY content.
		
		//
		// Begin Semantic Processing Here:
		//
		sender = new SenderData();
		
		String temp;
		
		temp = getAttributeValue("name");
		if ( temp == null )
		{
		    this.bumpErrorCount();
		    this.setErrorLevel(DisassemblyConstants.ERROR);
		    Object[] parms = { "name", this.element };
		    DisassemblyMessages.log(
		        DisassemblyMessages.get("attributeMissing"),
		        parms,
		        ImpactRange.SERIOUS
		    );
		    temp = "UNKNOWN_SENDER";
		}
		sender.setName( temp );
		
		temp = getAttributeValue("role");
		if ( (temp != null) && ( temp.length() > 0) )
		{
		    sender.setRole(temp);
		}
		else
		{
		    this.bumpErrorCount();
		    this.setErrorLevel(DisassemblyConstants.ERROR);
		    Object[] parms = { "role", this.element };
		    DisassemblyMessages.log(
		        DisassemblyMessages.get("attributeMissing"),
		        parms,
		        ImpactRange.SERIOUS
		    );
		}
		
		temp = getAttributeValue("sender-id");
		// UUID senderNamedUUID = senderLookup( sender.getName() );
		UUID senderNamedUUID = new UUID();
		UUID senderUUID = new UUID();
		if ( temp != null )
		{
		    senderUUID.set(temp);
		}
		else
		{
		    this.bumpErrorCount();
		    this.setErrorLevel(DisassemblyConstants.WARNING);
		    Object[] parms = 
		    { 
		        "sender-id", 
		        this.element,
		        DisassemblyMessages.get("lookUpSenderByName")
		    };
		    DisassemblyMessages.log(
		        DisassemblyMessages.get("attributeWarnMissing"),
		        parms,
		        ImpactRange.SERIOUS
		    );
		}
		    
		if ( !senderNamedUUID.equals(senderUUID) )
		{
		    // $POLICY$  It should be a matter of policy as to how to
		    // treat the mismatch.  The sender could have changed its
		    // name -- or merged with another.  We should probably believe
		    // the UUID and warn about the mismatch.  
		    // That is what we'll do for now -- until guided by policy.
		    this.bumpErrorCount();
		    this.setErrorLevel(DisassemblyConstants.ERROR);
		    Object[] parms = 
		    { 
		        "sender-id", 
		        temp, 
		        DisassemblyMessages.get("SenderMisMatch") 
		    };
		    DisassemblyMessages.log(
		        DisassemblyMessages.get("attributeInvalid"),
		        parms,
		        ImpactRange.SERIOUS
		    );
		}
		
		this.sender.setSenderId(senderUUID);
		    
		temp = getAttributeValue("location");
		if ( temp == null )
		{
		    this.bumpErrorCount();
		    this.setErrorLevel(DisassemblyConstants.WARNING);
		    Object[] parms = 
		    { 
		        "location", 
		        this.element,
		        DisassemblyMessages.get("locationRequired")
		    };
		    DisassemblyMessages.log(
		        DisassemblyMessages.get("attributeWarnMissing"),
		        parms,
		        ImpactRange.WARNING
		    );
		    temp = "UNKNOWN_LOCATION";
		}
		else
		{
		    URL senderURL = null;
		    
		    try { senderURL = new URL(temp); }
		    catch( MalformedURLException mue )
		    {
		        this.bumpErrorCount();
		        this.setErrorLevel(DisassemblyConstants.ERROR);
		        Object[] parms = 
		        { 
		            "ice-sender location", 
		            temp,
		            mue.getMessage()
		        };
		        DisassemblyMessages.log(
		            DisassemblyMessages.get("attributeInvalid"),
		            parms,
		            ImpactRange.SERIOUS
		        );
		    }
		    
		    sender.setLocation( senderURL );
		}
		TagEntry[] contentTags = null;
		this.setContent( new Content(contentTags, disassembler) );
	}
	
	public void endElement(String namespaceURI,
                           String localName,
                           String name)
	{
	    //
	    // Complete Semantic Processing Here:
	    //
	    
	    //
	    // construct the final result array of objects for return to the 
	    // parent here.  By default, send back this object.
	    //
	    Object[] results = 
	    { 
	        new Integer(this.getErrorCount()),
	        new Integer( this.getErrorLevel()),
	        this.sender
	    };
	    // next-to-last - call the result 
	    disassembler.result( name, results );
	    // last - unregister all my content handlers.
	    this.getContent().clearHandlers();
	}
    /**
	* Function called for element character data.
	*
	* @param    name    String containing name of element chardata is for.
	* @param	ch		array of character data.
	* @param	start	begining index of character data.
	* @param	length	length of character data.
	*
	* @see		SAXManager
     **/
	public void characters( String name, char ch[], int start, int length) 
	{
	    if ( name.equals(this.element) )
	    {
	        // Content Model of a ModelElement cannot contain characters.
            Object[] parms = { name, DisassemblyMessages.noCharactersInModel };    
            LogMsg logmsg = new LogMsg( DisassemblyMessages.elementInvalid , parms );
            logmsg.prolog( "payloadDisassembly.request", 200000000, "warning" );
            logmsg.log();
	    }
    }
    /**
     * Function called when a sub-handler completes and returns it's work.
	 *
	 * @param   name        String containing the name of an element just completed.
	 * @param   objectArray Array of objects, with are the result of the element just
	 *                      completed.  Presumably, the called handler's result 
	 *                      method knows how to handle the objects in the array,
	 *                      with the name of the completed element being the
	 *                      distinguishing hint.    
	 *
	 **/
	public void result( String name, Object[] objectArray ) 
    {
        int i = content.index(name);
        if ( i >= 0 )
        {
            Integer elementErrorCount = (Integer) objectArray[0];
            Integer elementErrorLevel = (Integer) objectArray[1];
                
            this.bumpErrorCount(elementErrorCount.intValue() );
            this.adjustErrorLevel(elementErrorLevel.intValue() );
        }
        switch ( i )
        {
            // When an element in the content of this element completes, it calls this 
            // routine with it's results.  That is if element foo has element bar as
            // content, then when element bar contained by foo completes, the result
            // is handed back to foo through this routine.
            //
            // Handle the results here.  Note -- The theory is that this handler is
            // little more than a packager of the effort of the lower level routines.
            // Build the lower level routines to handle everything that is locally possible
            // about the element.  The result routine (this one) should be designed to
            // integrate the lower level elements into a larger whole.  At this level,
            // we can do little but schedule the returning response.
            
            // The index here is the index of the elements in contentTags with the
            // first starting at 0.
            
            default: // unknown name
	            // 
                Object[] parms = { name, this.element };    
                LogMsg logmsg = new LogMsg( DisassemblyMessages.UnknownElementReturned , parms );
                logmsg.prolog( "payloadDisassembly.request", 200000000, "warning" );
                logmsg.log();
                // if the message is important enough to want to assure that it is written:
                // logmsg.flush(); 
                break;
        }
    }
}
                                                                                                                           