/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/payloadDisassembly/UnimplementedHandler.java,v 1.6 2000/12/02 23:28:44 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.payloadDisassembly;

import java.util.*;
import org.xml.sax.*;

//
// icecubes  imports
//
import com.adobe.rice.IceCubes;
import com.adobe.rice.common.*;
import com.adobe.rice.common.policies.*;
import com.adobe.rice.common.XMLdisassembly.*;
import com.adobe.rice.common.payloadDisassembly.*;
import com.adobe.rice.common.storage.logging.*;     // so we can write to the log.

public class UnimplementedHandler extends Handler 
{
    //
    // class wide variables
    //
    //
    // private class instance variables
    //
    
	// none so far
	
	//
	// constructors
	//
	
	public UnimplementedHandler(XMLDisassembler disassembler, Handler handler ) 
	{
		super(disassembler, handler);
		
		this.element = "@ANY_UNKNOWN@";
		
		ParserAttributes al = new ParserAttributes();
		
		al.add( new AttributeEntry("@ANY_UNKNOWN@", "CDATA" ) );
		
		this.elementAttributes = al;
	}
	
	public void startElement(String namespaceURI, String localName, 
                             String name, ParserAttributes attributes) 
    {
        TagEntry contentTags[] = null; 
	    AttributeEntry ae;
	    String choice;
	    int logLevel = ( (Integer) Tray.policyManager.getSelection("LoggingLevel") ).intValue();
	    
	    // Unimplemented Element in parse.
        Object[] parms = { name, DisassemblyMessages.unimplemented };    
        LogMsg logmsg = new LogMsg( DisassemblyMessages.elementInvalid , parms );
        logmsg.prolog( "common.payloadDisassembly", 300000000, "error" );
        logmsg.log();
        logmsg = null;
        {
            this.setupAttributes(attributes);
	    }
		// make sure content handlers are registered.
		// do this as late as possible
		contentTags = new TagEntry[] 
		{
		    // replace the UnimplementedHandler in each TagEntry with an appropriate handler when you implement it.
            new TagEntry("$ANY$", this)
		};
		this.setContent( new Content(contentTags, disassembler));
	}
	
	public void endElement(String namespaceURI, String localName, String name ) 
    {
	    //
	    // construct the final result array of objects for return to the 
	    // parent here.  By default, send back this object.
	    //
	    Object[] results = 
	    { 
	        new Integer(this.getErrorCount()),
	        new Integer(this.getErrorLevel()),
	        this 
	    };
	    // next-to-last - call the result 
	    disassembler.result( name, results );
	    // last - unregister all my content handlers.
	    this.getContent().clearHandlers();
	}
	
    /**
	* Function called for element character data.
	*
	* @param    name    String containing name of element chardata is for.
	* @param	ch		array of character data.
	* @param	start	begining index of character data.
	* @param	length	length of character data.
	*
	* @see		SAXManager
    **/
	public void characters( String name, char ch[], int start, int length) 
	{
	    // Content Model of a Unknown Element cannot contain characters.
        Object[] parms = { name, DisassemblyMessages.noCharactersInModel };    
        LogMsg logmsg = new LogMsg( DisassemblyMessages.elementInvalid , parms );
        logmsg.prolog( "common.payloadDisassembly", 300000000, "error" );
        logmsg.log();
    }
    
    /**
     * Function called when a sub-handler completes and returns it's work.
	 *
	 * @param   name        String containing the name of an element just completed.
	 * @param   objectArray Array of objects, with are the result of the element just
	 *                      completed.  Presumably, the called handler's result 
	 *                      method knows how to handle the objects in the array,
	 *                      with the name of the completed element being the
	 *                      distinguishing hint.    
	 *
	 * @see		Handler in package XMLdisassembly
	 *
	 **/
	public void result( String name, Object[] objectArray ) 
    {
	    // 
        Object[] parms = { name, this.element };    
        LogMsg logmsg = new LogMsg( DisassemblyMessages.UnknownElementReturned , parms );
        logmsg.prolog( "common.payloadDisassembly", 300000000, "error" );
        logmsg.log();
        // if the message is important enough to want to assure that it is written:
        // logmsg.flush(); 
    }
	
}
