/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/payloadDisassembly/request/EnumHandler.java,v 1.6 2000/12/02 23:30:08 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.payloadDisassembly.request;

import java.util.*;
import org.xml.sax.*;

//
// icecubes  imports
//
import com.adobe.rice.IceCubes;
import com.adobe.rice.common.*;
import com.adobe.rice.common.policies.*;
import com.adobe.rice.common.XMLdisassembly.*;
import com.adobe.rice.common.payloadDisassembly.*;
import com.adobe.rice.common.storage.logging.*;     // so we can write to the log.
import com.adobe.rice.common.negotiation.*;

public class EnumHandler extends com.adobe.rice.common.XMLdisassembly.Handler  
{
    // elements in IceEnum's content model
    private final int iceDefaultNone  = 0;
    private final int iceDefaultValue = 1;
    private final int iceEnumItem     = 2;
    
    private int expectedDataType = -1; 
   
    private Collection enumerationSet;
    private Collection defaultSet;
    private boolean    hasDefaults = true;
    
    private IceSet     set;
    private int        selection = Integer.MIN_VALUE;
    private EnumerationEnum foo;
  
    private int getSelection ()
    {
        int     select = Integer.MIN_VALUE;
        boolean flag = false;
        String  selection;
        
        try
        {
           selection = (getAttributeValue ("select")).toLowerCase ();
		
		   for (int n = 0; n < DisassemblyConstants.selectors.length ; n++)
		   {
		       if (selection.equals (DisassemblyConstants.selectors[n]))
		       {
		          select = DisassemblyConstants.selectorValues[n];
		          flag = true;
		          break;
		       }
		   }
		   
		   if (flag == false)
		   {
		      select = Integer.parseInt (selection);
		   }
		
        }
        catch (NumberFormatException e)
        {
           // signal error
           select = Integer.MIN_VALUE; 
        }
        
        return select;
    }
   
   
   
   
   	
	public EnumHandler (XMLDisassembler disassembler, 
	                    Handler parent) 
	{
		super(disassembler, parent);
			
        // <!ELEMENT ice-enum (ice-default-none | ice-default-value+,
        //                     ice-enum-item+)     >
        //
        // <!ATTLIST ice-enum
        //           id          ID   CDATA     #IMPLIED
        //           name             CDATA     #IMPLIED
        //           description      CDATA     #IMPLIED
        //           select           CDATA     #REQUIRED>
                     
		this.element = "ice-enum";
	
		ParserAttributes al = new ParserAttributes();
		
		al.add (new AttributeEntry ("id", "ID" ));
		al.add (new AttributeEntry ("ref", "IDREF" ));
		al.add (new AttributeEntry ("name", "CDATA" ));
		al.add (new AttributeEntry ("description", "CDATA"));
		al.add (new AttributeEntry ("select", "CDATA" ));
		
		this.elementAttributes = al;
		
		// collect enumeration items in a set
		enumerationSet = new LinkedList ();
		
		defaultSet = new LinkedList ();
		
	}
	
	public void startElement(String name, String localName,
	                         String rawName, ParserAttributes attributes) 
	{
	    Object [] prepared;
	    
	    // System.out.println ("GOT TO ENUM HANDLER");
	    // debug (rawName + " START ELEMENT<br>");
	    this.setupAttributes(attributes);
		
		TagEntry[] contentTags = new TagEntry[] 
		{
           new TagEntry ("ice-default-none", new DefaultNoneHandler (disassembler, this)),
           new TagEntry ("ice-default-value", new DefaultValueHandler( disassembler, this)),
           new TagEntry ("ice-enum-item", new EnumItemHandler (disassembler, this))
		};
		
		this.setContent( new Content(contentTags, disassembler));
		
		selection = getSelection ();
		
		// pass type information down to the children 
	    prepared = parent.getPrepared ();
		expectedDataType = ((Integer) prepared[0]).intValue ();
		setPrepared (prepared);
		
		//System.out.println ("EnumHandler "+ expectedDataType);
	}
	
	public void endElement(String namespaceURI,
	                       String localName,
	                       String name) 
	{
	    System.out.println ("EXIT ENUM HANDLER");
	    // semantic processing
	    // System.out.println ("EnumerationSet " + enumerationSet);
	    // System.out.println ("DefaultSet " + defaultSet);
	    // System.out.println ("hasDefaults " + hasDefaults);
	    
	    switch (expectedDataType)
	    {
	    case DisassemblyConstants.numeric:
	         set = new NumericEnum (enumerationSet, selection); 
	         break;
	         
	    case DisassemblyConstants.string:
	         set = new EnumerationEnum (enumerationSet, selection);
	         break;
	         
	    case DisassemblyConstants.time:
	         set = new TimeEnum (enumerationSet, selection);
	         break;
	    }
	    Object[] results = 
	    { 
	        new Integer(this.getErrorCount()),
	        new Integer(this.getErrorLevel()),
	        set, 
	        new Boolean (hasDefaults), 
	        defaultSet 
	    };
	    
	    // debug (name + " END ELEMENT<br>");
	    // next-to-last - call the result 
	    disassembler.result( name, results );
	    // last - unregister all my content handlers.
	    this.getContent().clearHandlers();
	    enumerationSet = null;
	}
    /**
	* Function called for element character data.
	*
	* @param    name    String containing name of element chardata is for.
	* @param	ch		array of character data.
	* @param	start	begining index of character data.
	* @param	length	length of character data.
	*
	* @see		SAXManager
     **/
	public void characters( String name, char ch[], int start, int length) 
	{
	    if ( name.equals(this.element) )
	    {
	        // Content Model of a ModelElement cannot contain characters.
            Object[] parms = { name, Messages.noCharactersInModel };    
            LogMsg logmsg = new LogMsg( Messages.elementInvalid , parms );
            logmsg.prolog( "payloadDisassembly.request", 200000000, "warning" );
            logmsg.log();
	    }
    }
    /**
     * Function called when a sub-handler completes and returns it's work.
	 *
	 * @param   name        String containing the name of an element just completed.
	 * @param   objectArray Array of objects, with are the result of the element just
	 *                      completed.  Presumably, the called handler's result 
	 *                      method knows how to handle the objects in the array,
	 *                      with the name of the completed element being the
	 *                      distinguishing hint.    
	 *
	 **/
	public void result (String name, Object[] objectArray ) 
    {
        int i = content.index(name);
        if ( i >= 0 )
        {
            Integer elementErrorCount = (Integer) objectArray[0];
            Integer elementErrorLevel = (Integer) objectArray[1];
                
            this.bumpErrorCount(elementErrorCount.intValue() );
            this.adjustErrorLevel(elementErrorLevel.intValue() );
        }
        switch ( i )
        { 
            case iceDefaultNone:  // iceDefaultNone
                // System.out.println (name + " " + objectArray[0]);
                hasDefaults = ((Boolean) objectArray[0]).booleanValue () ? true : false;
                break;
            
            case iceDefaultValue: // iceDefaultValue
                // System.out.println (name + " " + objectArray[0]);
                
                defaultSet.add (objectArray [0]);
                break;
            
            case iceEnumItem:     // iceEnumItem
                // System.out.println (name + " " + objectArray[0]);
                
                enumerationSet.add (objectArray [0]);
                break;
            
            default: // unknown name
	            // 
                Object[] parms = { name, this.element };    
                LogMsg logmsg = new LogMsg( Messages.UnknownElementReturned , parms );
                logmsg.prolog( "payloadDisassembly.request", 200000000, "warning" );
                logmsg.log();
                // if the message is important enough to want to assure that it is written:
                // logmsg.flush(); 
                break;
        }
    }
}
                                                                                                                           