/*
$Log: EnumItemHandler.java,v $
Revision 1.6  2000/12/02 23:30:07  bhunt
Latest Assembly/Disassembly updates

Revision 1.5  2000/11/20 21:27:41  andrewf
testing assembling ice-negotiables

Revision 1.4  2000/11/10 21:13:36  andrewf
mistake in package name

Revision 1.3  2000/11/09 19:15:30  andrewf
Added semantics to handlers for assembling ice-parameter
objects

Revision 1.2  2000/11/01 02:43:38  andrewf
adding changes to handlers to test MessageDispatcher

Revision 1.1  2000/09/24 22:45:32  andrewf
payloadDisassembly request handler additions

$Date: 2000/12/02 23:30:07 $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.payloadDisassembly.request;

import java.util.*;
import org.xml.sax.*;

//
// icecubes  imports
//
import com.adobe.rice.IceCubes;
import com.adobe.rice.common.*;
import com.adobe.rice.common.policies.*;
import com.adobe.rice.common.XMLdisassembly.*;
import com.adobe.rice.common.payloadDisassembly.*;
import com.adobe.rice.common.storage.logging.*;     // so we can write to the log.

public class EnumItemHandler extends com.adobe.rice.common.XMLdisassembly.Handler  
{
    String itemElement;
    int    expectedDataType = -1;
    
	public EnumItemHandler (XMLDisassembler disassembler, Handler parent) 
	{
		super (disassembler, parent);
		
        //  <!ELEMENT ice-enum-item (ice-span? | #PCDATA | ice-extension)>
        //  <!ATTLIST ice-enum-item 
        //            id            ID        #IMPLIED
        //            ref           IDREF     #IMPLIED>
         
		this.element = "ice-enum-item";
	
		ParserAttributes al = new ParserAttributes ();
		
		al.add (new AttributeEntry ("id", "ID" ));
		al.add (new AttributeEntry ("ref","IDREF"));
		
		this.elementAttributes = al;
		
	}
	
	public void startElement(String namespaceURI, String localName,
	                         String rawName, ParserAttributes attributes) 
	{
	    //
	    // pull in the attributes
	    //
	    // System.out.println ("GOT TO ENUM ITEM HANDLER");
	    // debug (rawName + " START ELEMENT<br>"); 
	    this.setupAttributes(attributes);
		// make sure content handlers are registered.
		// do this as late as possible
		TagEntry[] contentTags = new TagEntry[] 
		{
		    // create a TagEntry for each element in ModelContent's content model here.
		    // Note that the order of the TagEntrys corresponds to the indexes used
		    // below -- if TAGNAME is the tag name in the first TagEntry, then it is
		    // index 0 below in result.
		    //
		    // replace the UnimplementedHandler in each TagEntry with an appropriate handler when you implement it.
            new TagEntry("ice-extension", new ExtensionHandler (disassembler, this))
		};
		this.setContent( new Content(contentTags, disassembler));
		
		prepared = parent.getPrepared ();
		expectedDataType = ((Integer) prepared[0]).intValue ();		
	}
	
	public void endElement(String namespaceURI,
	                       String localName,
	                       String name) 
	{
	    //
	    // Complete Semantic Processing Here:
	    //
	    
	    //
	    // construct the final result array of objects for return to the 
	    // parent here.  By default, send back this object.
	    //
	    //debug (name +" END ELEMENT<br>"); 
	    Object[] results = 
	    { 
	        new Integer(this.getErrorCount()),
	        new Integer(this.getErrorLevel()),
	        this.itemElement
	    };
	    // next-to-last - call the result 
	    disassembler.result( name, results );
	    // last - unregister all my content handlers.
	    this.getContent().clearHandlers();
	}
	
    /**
	* Function called for element character data.
	*
	* @param    name    String containing name of element chardata is for.
	* @param	ch		array of character data.
	* @param	start	begining index of character data.
	* @param	length	length of character data.
	*
	* @see		SAXManager
     **/
	public void characters( String name, char ch[], int start, int length) 
	{
	    if ( name.equals (this.element) )
	    {
	        itemElement = (new String (ch, start, length)).trim ();
	    }
    }
    
    /**
     * Function called when a sub-handler completes and returns it's work.
	 *
	 * @param   name        String containing the name of an element just completed.
	 * @param   objectArray Array of objects, with are the result of the element just
	 *                      completed.  Presumably, the called handler's result 
	 *                      method knows how to handle the objects in the array,
	 *                      with the name of the completed element being the
	 *                      distinguishing hint.    
	 *
	 **/
	public void result( String name, Object[] objectArray ) 
    {
        int i = content.index(name);
        if ( i >= 0 )
        {
            Integer elementErrorCount = (Integer) objectArray[0];
            Integer elementErrorLevel = (Integer) objectArray[1];
                
            this.bumpErrorCount(elementErrorCount.intValue() );
            this.adjustErrorLevel(elementErrorLevel.intValue() );
        }
       Object[] parms = { name, this.element };    
       LogMsg logmsg = new LogMsg( Messages.UnknownElementReturned , parms );
       logmsg.prolog( "payloadDisassembly.request", 200000000, "warning" );
       logmsg.log();
    }
}
                                                                                                                           