/*
$Log: NegotiableHandler.java,v $
Revision 1.6  2000/12/02 23:30:07  bhunt
Latest Assembly/Disassembly updates

Revision 1.5  2000/11/20 21:27:41  andrewf
testing assembling ice-negotiables

Revision 1.4  2000/11/10 21:13:36  andrewf
mistake in package name

Revision 1.3  2000/11/09 19:15:30  andrewf
Added semantics to handlers for assembling ice-parameter
objects

Revision 1.2  2000/11/01 02:43:39  andrewf
adding changes to handlers to test MessageDispatcher

Revision 1.1  2000/09/24 22:48:38  andrewf
payloadDisassembly request handler updates

$Date: 2000/12/02 23:30:07 $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.payloadDisassembly.request;

import java.util.*;
import org.xml.sax.*;
import java.math.*;

//
// icecubes  imports
//
import com.adobe.rice.IceCubes;
import com.adobe.rice.common.*;
import com.adobe.rice.common.policies.*;
import com.adobe.rice.common.XMLdisassembly.*;
import com.adobe.rice.common.payloadDisassembly.*;
import com.adobe.rice.common.storage.logging.*;     // so we can write to the log.
import com.adobe.rice.common.negotiation.*;


public class NegotiableHandler extends com.adobe.rice.common.XMLdisassembly.Handler  
{
    private final int iceText = 0;
    private final int iceEnum = 1;
    private final int iceSpan = 2;
    
    private String text;
    private IceSet parameterSet;
    private Set    defaultValues;
      
    private int getDataType (String type)
    {
        int result = -1;
        
        for (int n = 0; n < DisassemblyConstants.datatypes.length ; n++)
        {
            if (type.equals (DisassemblyConstants.datatypes[n]))
            {
               result = n;
               break;
            }    
        }
        
        return result;
    }
    
    
	public NegotiableHandler (XMLDisassembler disassembler, 
	                          Handler parent) 
	{
		super (disassembler, parent);
		
		//<!ELEMENT ice-negotiable   ((ice-text?, (ice-enum | ice-span), ice-text?>
		//<!ATTLIST ice-negotiable 
		//          id           ID         #IMPLIED
		//          type         NMTOKEN    #REQUIRED
		//          max          CDATA      #IMPLIED
		//          min          CDATA      #IMPLIED>
		
		this.element = "ice-negotiable";
	
		ParserAttributes al = new ParserAttributes();
		
		al.add (new AttributeEntry ("id", "CDATA"));
		al.add (new AttributeEntry ("type", "NMTOKEN"));
		al.add (new AttributeEntry ("max", "CDATA"));
		al.add (new AttributeEntry ("min", "CDATA"));
		al.add (new AttributeEntry ("datatype","CDATA"));
		
		this.elementAttributes = al;	
	}
	
	public void startElement(String name, String localName,
	                         String rawName, ParserAttributes attributes) 
	{
	    int dataType;
	    
	    // System.out.println ("GOT TO NEGOTIABLE");
	    // debug (rawName + " START ELEMENT<br>");
	    this.setupAttributes(attributes);
		
		// make sure content handlers are registered.
		// do this as late as possible
		
		TagEntry[] contentTags = new TagEntry[] 
		{
            new TagEntry("ice-text", new TextHandler (disassembler, this)),
            new TagEntry("ice-enum", new EnumHandler (disassembler, this)),
            new TagEntry("ice-span", new SpanHandler (disassembler, this))
		};
		
		this.setContent (new Content (contentTags, disassembler));
		
		dataType = getDataType (getAttributeValue ("datatype"));
		if (dataType != -1)
		{
		   Object prepared [] = { new Integer (dataType) };
		   this.setPrepared (prepared);
		}
		else
		{
		   /* this is an error */   
		}
	}
	
	public void endElement(String name) 
	{
	    // we have all the information we need to 
	    // construct a parameter
	    
	   /* switch (expectedDataType)
	    {
	       case DisassemblyConstant.numeric:
	       break;
	       case DisassemblyConstant.time:
	       break;
	       case DisassemblyConstant.string: 
	       break;
	    }    */
	    Object[] results = 
	    { 
	        new Integer(this.getErrorCount()),
	        new Integer(this.getErrorLevel()),
	        this 
	    };
	    
	    // debug (name + " END ELEMENT<br>");
	    // next-to-last - call the result 
	    disassembler.result( name, results );
	    // last - unregister all my content handlers.
	    this.getContent().clearHandlers();
	}
    /**
	* Function called for element character data.
	*
	* @param    name    String containing name of element chardata is for.
	* @param	ch		array of character data.
	* @param	start	begining index of character data.
	* @param	length	length of character data.
	*
	* @see		SAXManager
     **/
	public void characters( String name, char ch[], int start, int length) 
	{
	    if ( name.equals(this.element) )
	    {
	        // Content Model of a ModelElement cannot contain characters.
            Object[] parms = { name, Messages.noCharactersInModel };    
            LogMsg logmsg = new LogMsg( Messages.elementInvalid , parms );
            logmsg.prolog( "payloadDisassembly.request", 200000000, "warning" );
            logmsg.log();
	    }
    }
    /**
     * Function called when a sub-handler completes and returns it's work.
	 *
	 * @param   name        String containing the name of an element just completed.
	 * @param   objectArray Array of objects, with are the result of the element just
	 *                      completed.  Presumably, the called handler's result 
	 *                      method knows how to handle the objects in the array,
	 *                      with the name of the completed element being the
	 *                      distinguishing hint.    
	 *
	 **/
	public void result( String name, Object[] objectArray ) 
    {
        int i = content.index(name);
        if ( i >= 0 )
        {
            Integer elementErrorCount = (Integer) objectArray[0];
            Integer elementErrorLevel = (Integer) objectArray[1];
                
            this.bumpErrorCount(elementErrorCount.intValue() );
            this.adjustErrorLevel(elementErrorLevel.intValue() );
        }
        // System.out.println ("In Negotiable result");
        switch ( i )
        {
           
            case iceText: // ice-delivery-policy
                 break;
                
            case iceEnum: 
            case iceSpan:
                 // System.out.println ("size: " + objectArray.length);
                 parameterSet = (IceSet) objectArray [0];
                 System.out.println ("Range:" + parameterSet);
                 System.out.println (objectArray[1]);
                 if ( ((Boolean)objectArray[1]).booleanValue ())
                 {
                    System.out.println (objectArray[2]);
                 }
                 
                 break;
                 
            default: // unknown name
	            // 
                Object[] parms = { name, this.element };    
                LogMsg logmsg = new LogMsg( Messages.UnknownElementReturned , parms );
                logmsg.prolog( "payloadDisassembly.request", 200000000, "warning" );
                logmsg.log();
                // if the message is important enough to want to assure that it is written:
                // logmsg.flush(); 
                break;
        }
    }
}
                                                                                                                           