/*
$Log: SpanHandler.java,v $
Revision 1.6  2000/12/02 23:30:06  bhunt
Latest Assembly/Disassembly updates

Revision 1.5  2000/11/20 21:27:41  andrewf
testing assembling ice-negotiables

Revision 1.4  2000/11/10 21:13:37  andrewf
mistake in package name

Revision 1.3  2000/11/09 19:15:30  andrewf
Added semantics to handlers for assembling ice-parameter
objects

Revision 1.2  2000/11/01 02:43:39  andrewf
adding changes to handlers to test MessageDispatcher

Revision 1.1  2000/09/26 05:49:52  andrewf
request handler stubs

$Date: 2000/12/02 23:30:06 $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.payloadDisassembly.request;

import java.util.*;
import org.xml.sax.*;

//
// icecubes  imports
//
import com.adobe.rice.IceCubes;
import com.adobe.rice.common.*;
import com.adobe.rice.common.policies.*;
import com.adobe.rice.common.XMLdisassembly.*;
import com.adobe.rice.common.payloadDisassembly.*;
import com.adobe.rice.common.storage.logging.*;     // so we can write to the log.
import com.adobe.rice.common.negotiation.*;

public class SpanHandler extends Handler  
{
    
    private final int iceDefaultNone = 0;
    private final int iceDefaultValue = 1;
    private final int iceSpanMin = 2;
    private final int iceSpanMax = 3;
  
    private Object maximum;
    private Object minimum;
    private Object defaultValue = null;
    private IceSet set;
    private double precision;
    
    private boolean hasDefaults = true;
    
    //
    private int expectedDataType = -1;
    
 	public SpanHandler (XMLDisassembler disassembler, 
 	                    Handler parent) 
	{
		super (disassembler, parent);
		
		Object prepared [];
		
        // <!ELEMENT ice-span (ice-default-value | ice-default-none,
        //                     ice-span-min?, ice-span-max?)     >

        // <!ATTLIST ice-span
        //           id                   ID       #IMPLIED
        //           name                 CDATA    #IMPLIED
        //           description          CDATA    #IMPLIED  >
        
		this.element = "ice-span";
	
		ParserAttributes al = new ParserAttributes();
		
		al.add (new AttributeEntry("id", "ID" ));
		al.add (new AttributeEntry("name", "CDATA" ));
		al.add (new AttributeEntry("description", "CDATA"));
	
		this.elementAttributes = al;
	}
	
	public void startElement(String name, String localName,
	                         String rawName, ParserAttributes attributes) 
	{
	    Object prepared [];
	    
	    // get the expected type
	    prepared = parent.getPrepared ();
		expectedDataType = ((Integer) prepared[0]).intValue ();
	   
	    System.out.println ("GOT TO SPAN HANDLER");
	    // debug (rawName + " START ELEMENT<br>");
	    this.setupAttributes(attributes);
		// make sure content handlers are registered.
		// do this as late as possible
		TagEntry[] contentTags = new TagEntry[] 
		{
		    // create a TagEntry for each element in ModelContent's content model here.
		    // Note that the order of the TagEntrys corresponds to the indexes used
		    // below -- if TAGNAME is the tag name in the first TagEntry, then it is
		    // index 0 below in result.
		    //
		    // replace the UnimplementedHandler in each TagEntry with an appropriate handler when you implement it.
            new TagEntry ("ice-default-none", new DefaultNoneHandler (disassembler, this)),
            new TagEntry ("ice-default-value", new SpanDefaultValueHandler (disassembler, this)),
            new TagEntry ("ice-span-min", new SpanMinHandler (disassembler, this)),
            new TagEntry ("ice-span-max", new SpanMaxHandler (disassembler, this))
		};
		this.setContent( new Content(contentTags, disassembler));
		//
		// Begin Semantic Processing Here:
		//
	}
	
	public void endElement(String namespaceURI, String localName,
	                       String name) 
	{
	       
	    // System.out.println ("IN SPAN END");   
	    switch (expectedDataType)
	    {
	    case DisassemblyConstants.numeric:
	    
	        //   System.out.println ("hasDefault: " + hasDefaults);
	        //   System.out.println ("SpanMin: " + minimum);
	        //   System.out.println ("SpanMax: " + maximum);
	        //   System.out.println ("DefaultValue: " + defaultValue);  
	    
	         set = new NumericSpan (((Double) minimum).doubleValue (),
	                               ((Double) maximum).doubleValue (),
	                               precision);	                                
	         break;
	         
	    case DisassemblyConstants.string:
	         
	         // not sure how to handle this?
	         break;
	         
	    case DisassemblyConstants.time:
	    
	         set = new TimeSpan ((Date) minimum, (Date) maximum, (long) precision);                    
	         break;
	    }
	    
	   
	    // construct the final result array of objects for return to the 
	    // parent here.  By default, send back this object.
	        
	    System.out.println ("->" + defaultValue);
	    
	    Object[] results = 
	    { 
	        new Integer(this.getErrorCount()),
	        new Integer(this.getErrorLevel()),
	        this.set, 
	        new Boolean (hasDefaults), 
	        defaultValue
	    };
	    // debug (name + " END ELEMENT <br>");
	    // next-to-last - call the result 
	    disassembler.result( name, results );
	    // last - unregister all my content handlers.
	    this.getContent().clearHandlers();
	}
    /**
	* Function called for element character data.
	*
	* @param    name    String containing name of element chardata is for.
	* @param	ch		array of character data.
	* @param	start	begining index of character data.
	* @param	length	length of character data.
	*
	* @see		SAXManager
     **/
	public void characters( String name, char ch[], int start, int length) 
	{
	    if ( name.equals(this.element) )
	    {
	        // Content Model of a ModelElement cannot contain characters.
            Object[] parms = { name, Messages.noCharactersInModel };    
            LogMsg logmsg = new LogMsg( Messages.elementInvalid , parms );
            logmsg.prolog( "payloadDisassembly.request", 200000000, "warning" );
            logmsg.log();
	    }
    }
    /**
     * Function called when a sub-handler completes and returns it's work.
	 *
	 * @param   name        String containing the name of an element just completed.
	 * @param   objectArray Array of objects, with are the result of the element just
	 *                      completed.  Presumably, the called handler's result 
	 *                      method knows how to handle the objects in the array,
	 *                      with the name of the completed element being the
	 *                      distinguishing hint.    
	 *
	 **/
	public void result( String name, Object[] objectArray ) 
    {
        int i = content.index(name);
        if ( i >= 0 )
        {
            Integer elementErrorCount = (Integer) objectArray[0];
            Integer elementErrorLevel = (Integer) objectArray[1];
                
            this.bumpErrorCount(elementErrorCount.intValue() );
            this.adjustErrorLevel(elementErrorLevel.intValue() );
        }
        switch (i)
        {
            case iceDefaultNone:
                 
                 hasDefaults = false;
                 break;
            
            case iceDefaultValue:
                
                 defaultValue = objectArray[0];
                 break;
                    
            case iceSpanMin: 
                 minimum = objectArray[0];
                 break;
                          
            case iceSpanMax:
                 
                 maximum = objectArray [0];
                 break;
                    
            default: // unknown name
	            // 
                Object[] parms = { name, this.element };    
                LogMsg logmsg = new LogMsg( Messages.UnknownElementReturned , parms );
                logmsg.prolog( "payloadDisassembly.request", 200000000, "warning" );
                logmsg.log();
                // if the message is important enough to want to assure that it is written:
                // logmsg.flush(); 
                break;
        }
    }
}
                                                                                                                           