/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/policies/ManagerPolicies.java,v 1.5 2000/11/27 20:18:56 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/
/**
 * ManagerPolicies.java
 *
 *
 * Created: Sun Nov  7 14:50:34 1999
 *
 * @author V. Bruce Hunt
 * @version
 */

package com.adobe.rice.common.policies;


import com.adobe.rice.common.Tray;
import com.adobe.rice.common.Manager;

import com.adobe.rice.common.asks.*;
import com.adobe.rice.common.storage.*;
import com.adobe.rice.common.storage.logging.*; 
import com.adobe.rice.common.scheduling.*; 
import com.adobe.rice.common.transportation.*; 
import com.adobe.rice.common.catalog.*;

import com.adobe.rice.subscriber.*;

import com.adobe.rice.syndicator.*;

import com.adobe.rice.kernel.*;

import java.lang.*;
import java.io.*;

public class ManagerPolicies
{
    private static final String[] builtInManagerNames = 
    {
        "Policy",
        "Storage",
        "Schedule",
        "Transport",
        "Catalog",
        "Subscriber",
        "Collection",
        "Subscription",
        "Negotiation",
        "Ask"
    };
    
    private static SimplePolicy[] builtInManagerPolicies;
    
    private static Manager[] builtInManagers;
    
    private boolean initialized = false;
    private Tray tray = null;
    
    //
    // The vector of ICEcubes standard managers:
    
    private static StorageManager storageManager;
    private static ScheduleManager scheduleManager;
    private static TransportManager transportManager;
    // private static CatalogManager catalogManager;
    // private static SubscriberManager subscriberManager;
    // private static CollectionManager collectionManager;
    // private static SubscriptionManager subscriptionManager;
    // private static NegotiationManager negotiationManager;
    private static AskManager askManager;
    
    public SimplePolicy pStorageManager;
    public SimplePolicy pScheduleManager;
    public SimplePolicy pTransportManager;
    // public SimplePolicy pCatalogManger;
    // public SimplePolicy pSubscriberManager;
    // public SimplePolicy pSyndicatorManager;
    // public SimplePolicy pCollectionManager;
    // public SimplePolicy pSubscriptionManager;
    // public SimplePolicy pNegotiationManager;
    public SimplePolicy pAskManager;
    
    
    public ManagerPolicies() 
    {
        int length = ManagerPolicies.builtInManagerNames.length;
        
        this.builtInManagerPolicies = new SimplePolicy[ length ];
        this.builtInManagers = new Manager[ length ];
    }
    
    public void setTray( Tray tray ) { this.tray = tray; }
    
    public void initialize( )
    {
        storageManager = StorageManager.instance();
        scheduleManager = ScheduleManager.instance();
        transportManager = TransportManager.instance();
        askManager = AskManager.instance();
        
        // catalogManager = new CatalogManager();
        // subscriberManager = new SubscriberManager();
        // collectionManager = new CollectionManager();
        // subscriptionManager = new SubscriptionManager();
        // negotiationManager = new NegotiationManger();
        
        // 
        // Setup the simple manager policies
        //
        pStorageManager       = new SimplePolicy( StorageManager.class , storageManager );
        pScheduleManager      = new SimplePolicy( MasterSchedule.class , scheduleManager );
        pTransportManager     = new SimplePolicy( TransportManager.class, transportManager );
        // pCatalogManager      = new SimplePolicy( CatalogManager.class, catalogManager );
        // pSubscriberManager   = new SimplePolicy( SubscriberManager.class, subscriberManager);
        // pSyndicatorManager   = new SimplePolicy( SyndicatorManager.class, syndicatorManager);
        // pCollectionManager   = new SimplePolicy( CollectionManager.class, collectionManager);
        // pSubscriptionManager = new SimplePolicy( SubscriptionManager.class, subscriptionManager);
        // pNegotiationManager  = new SimplePolicy( NegotiationManager.class, negotiationManager);
        pAskManager           = new SimplePolicy( AskManager.class, askManager );
        //
        
        //
        // Register the "manager" simple policies as global policies
        //
        PolicyManager pm = tray.getPolicyManager();
        
        pm.setPolicy("Storage", pStorageManager );
        pm.setPolicy("Schedule", pScheduleManager );
        pm.setPolicy("Transport", pTransportManager);
        //
        // pm.setPolicy("Catalog", pCatalogManager);
        // pm.setPolicy("Subscriber", pSubscriberManager);
        // pm.setPolicy("Syndicator", pSyndicatorManager);
        // pm.setPolicy("Collection", pCollectionManager);
        // pm.setPolicy("Subscription", pSubscriptionManager);
        // pm.setPolicy("Negotiation", pNegotiationManager);
        pm.setPolicy("Ask", pAskManager);
        //
        
        //
        // now register the managers
        //
        
        tray.registerManager( "Storage", (Manager) storageManager );  
        tray.registerManager( "Schedule", (Manager) scheduleManager );
        tray.registerManager( "Transport", (Manager) ManagerPolicies.transportManager );
        // Tray.registerManager( "Catalog", (Manager) ManagerPolicies.catalogManager );
        // Tray.registerManager( "Subscriber", (Manager) ManagerPolicies.subscriberManager );
        // Tray.registerManager( "Syndicator", (Manager) ManagerPolicies.syndicatorManager );
        // Tray.registerManager( "Collections", (Manager) ManagerPolicies.collectionManager );
        // Tray.registerManager( "Subscriptions", (Manager) ManagerPolicies.subscriptionManager );
        // Tray.registerManager( "Negotiation", ManagerPolicies.negotiationManager );
        tray.registerManager("Ask", ManagerPolicies.askManager );
        //
        initialized = true;
    }
    
} // ManagerPolicies









