/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/policies/Policy.java,v 1.1.1.1 2000/09/19 19:39:46 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/


package com.adobe.rice.common.policies;

import com.adobe.rice.util.ExceptionMessages;

import java.util.Observable;

public abstract class Policy extends Observable 
{
    private String  name;                       // name of the policy
    private String  packageName;                // name of package this policy applies to
    private Class	expectedClass;              // Class of policy
	
    public Policy(Class expectedClass) 
    {
        this.expectedClass = expectedClass;
    }
    
    public Policy(Class expectedClass, String policyName )
    {
        this.expectedClass = expectedClass;
        this.name = policyName;
    }
    
    public Policy(Class eC, String pN, String pKn )
    {
        this.expectedClass = eC;
        this.name = pN;
        this.packageName = pKn;
    }
	
    public final void select(Object o) throws PolicyException 
    {
        if (!this.expectedClass.isInstance(o))
            throw new PolicyException(ExceptionMessages.getExceptionMessage("ArgWrongType"));
		
        setSelection(o);
	
        notifyObservers();
    }
	
    public abstract Object getSelection();
	
    protected abstract void setSelection(Object o) throws PolicyException;
    
    public String getName() { return name; }
    
    public void setName( String newName ) { name = newName; }
    
    public String getPackageName() { return packageName; }
    
    public void setPackageName( String newPackage ) { packageName = newPackage; }
    
    public Class getExpectedClass( ) { return expectedClass; }
    
}

