/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/policies/RangedDoublePolicy.java,v 1.1.1.1 2000/09/19 19:39:46 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org


*/

package com.adobe.rice.common.policies;

import com.adobe.rice.util.ExceptionMessages;

public class RangedDoublePolicy extends RangedPolicy 
{
    
    public RangedDoublePolicy(double min, double max) 
    {
        
	    super(Double.class, new Double(min), new Double(max) );
    }
	
	public RangedDoublePolicy( String name, double min, double max )
	{
	    super(Double.class, name, new Double(min), new Double(max) );
	}
	
	public RangedDoublePolicy( String name, String pkg, double min, double max )
	{
	    super(Double.class, name, pkg, new Double(min), new Double(max) );
	}
	
	//
		
	
	public void setSelection( double d ) throws PolicyException
	{
	    if ( ( d < min.doubleValue() ) || ( d > max.doubleValue() ) )
	        throw new PolicyException(ExceptionMessages.getExceptionMessage("ArgOutOfRange"));
	    selection = new Double( d );
	}
	
	public void setSelection( Double cd ) throws PolicyException
	{
	    double d = cd.doubleValue();
	    if ( ( d < this.min.doubleValue() ) || ( d > this.max.doubleValue() ) )
	        throw new PolicyException(ExceptionMessages.getExceptionMessage("ArgOutOfRange"));
	    selection = cd;
	}
	
    public void setSelection(Object o) throws PolicyException 
    {
        if ( RangedPolicy.isNumber( o.getClass() ) )
        {
	        double value = ((Number)o).doubleValue();		
	        if ( (value < min.doubleValue()) || (value > max.doubleValue()) )
	            throw new PolicyException(ExceptionMessages.getExceptionMessage("ArgOutOfRange"));
	        selection = (Number) o;
	    }
	    else
	        throw new PolicyException(ExceptionMessages.getExceptionMessage("ArgWrongType"));
    }
}


