/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/policies/RangedPolicy.java,v 1.1.1.1 2000/09/19 19:39:47 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org


*/

package com.adobe.rice.common.policies;

import com.adobe.rice.util.ExceptionMessages;

public class RangedPolicy extends Policy 
{
    protected Number		max;
    protected Number		min;
    protected Number		selection;
    
    public RangedPolicy(Class c, Number mini, Number maxi ) 
    {
        // What does the statement below do?
	    super(c);
	    // how do you validate that c is a number?
	    if ( isNumber(c) && ( c == mini.getClass()) && ( c == maxi.getClass() ) )
	    {
	        if ( mini.doubleValue() <= maxi.doubleValue() )
	        {
                this.min = mini;
                this.max = maxi;
            }
        }
    }
    
    public RangedPolicy( Class c, String n, Number mini, Number maxi )
    {
        super(c, n );
	    if ( isNumber(c) && ( c == mini.getClass()) && ( c == maxi.getClass() ) )
	    {
	        if ( mini.doubleValue() <= maxi.doubleValue() )
	        {
                this.min = mini;
                this.max = maxi;
            }
            
        }
    }
    
    public RangedPolicy(Class c, String n, String pkg, Number mini, Number maxi )
    {
        super(c,n,pkg);
     
	    if ( isNumber(c) && ( c == mini.getClass()) && ( c == maxi.getClass() ) )
	    {
	        if ( mini.doubleValue() <= maxi.doubleValue() )
	        {
                this.min = mini;
                this.max = maxi;
            }
        }
    }
        
    public Object getSelection() { return selection; }
    
    public void setSelection(Object o) throws PolicyException 
    {
        if ( o.getClass() == this.getExpectedClass() )
        {
    	    double value = ((Double)o).doubleValue();		
    	    if ( (value < min.doubleValue()) || (value > max.doubleValue()) )
	        throw new PolicyException(ExceptionMessages.getExceptionMessage("ArgOutOfRange"));
	        selection = (Number) o;
	    }
	    else
	        throw new PolicyException(ExceptionMessages.getExceptionMessage("ArgWrongType"));	    
    }
    
    public void setMin( Number mini ) throws PolicyException
    {
        if ( (mini.getClass() == this.getExpectedClass()) )
        {
            if ( min.doubleValue() <= selection.doubleValue() )
                min = mini;
            else
                throw new PolicyException(ExceptionMessages.getExceptionMessage("ArgOutOfRange"));
        }
        else
            throw new PolicyException(ExceptionMessages.getExceptionMessage("ArgWrongType"));
    }
    
    public Number getMin() { return min; }
    
    public void setMax( Number maxi ) throws PolicyException
    {
        if ( (maxi.getClass() == this.getExpectedClass()) )
        {
            if ( max.doubleValue() >= selection.doubleValue() )
                max = maxi;
            else
                throw new PolicyException(ExceptionMessages.getExceptionMessage("ArgOutOfRange"));
        }
        else
            throw new PolicyException(ExceptionMessages.getExceptionMessage("ArgWrongType"));
    }
    
    public Number getMax() { return max; }
    
    protected static boolean isNumber( Class c )
    {
        if ( c == Byte.class )
            return true;
        else if ( c == Short.class )
            return true;
        else if ( c == Integer.class )
            return true;
        else if ( c == Long.class )
            return true;
        else if ( c == Float.class )
            return true;
        else if ( c == Double.class )
            return true;
        else if ( c == Character.class )
            return true;
        else
            return false;
    }
}


