/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/policies/configure/ClassHandler.java,v 1.2 2000/11/22 02:33:46 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.policies.configure;

import java.util.*;
import org.xml.sax.*;

//
// icecubes  imports
//
import com.adobe.rice.IceCubes;
import com.adobe.rice.common.*;
import com.adobe.rice.common.policies.*;
import com.adobe.rice.common.XMLdisassembly.*;
import com.adobe.rice.common.storage.logging.*;     // so we can write to the log.

public class ClassHandler extends Handler 
{
    //
    // Class wide varibles
    //
        
    //
    // instance variables
    //
    
    private String theClassName = "";
    private String theClassUuid = "";
    
	public ClassHandler(XMLDisassembler disassembler, Handler parent) 
	{		
		super(disassembler, parent);
		
		this.element = "class";
	}
	
	public void startElement(String namespaceURI, String localName, 
                             String name,         ParserAttributes attributes) 
	{
        TagEntry contentTags[] = null; 
	    AttributeEntry ae;
	    String choice;
	    int logLevel = ( (Integer) Tray.policyManager.getSelection("LoggingLevel") ).intValue();
	    
		if (name.equals(this.element)) 
		{
		    // run through the attributes and get their values
		    for ( int i = 0; i < attributes.getLength(); i++ )
		    {
                String aName = attributes.getName(i);
                String aType = attributes.getType(i);
                String aValue = attributes.getValue(i);
		        ae = this.findAttribute(aName);
		        if ( ae != null )
		        {
		            ae.setValue((ae.get()+aValue));
		            ae.setType(aType);
		        }
		        else /* This attribute is not defined */
		        {   /* $POLICY$ */
		            if ( logLevel > 6 )
		            {
                        Object[] parms = { aName, aValue, this.element };    
                        LogMsg logmsg = new LogMsg( Messages.attributeUnknown , parms );
                        logmsg.prolog( "common.policies.configure", 200000000, "warning" );
                        logmsg.log();
		            }
		            
		            choice = (String) Tray.policyManager.getSafeSelection("AttributeNew", "exception" );
		            if ( choice.equals("add") )
		            {
		                this.elementAttributes.add(new AttributeEntry(aName, aType, aValue ));
		            }
		            else if ( choice.equals("ignore") )
		            {
		            }
		            else /* if ( choice.equals("exception") ) */ // throw an exception here
		            {
		                // $FIX$ -- throw an exception here.
		            }
		        }
		    }
		    ae = this.findAttribute("name");
		    if ( ae != null )
		        theClassName += ae.get();
		    ae = this.findAttribute("uuid");
		    if ( ae != null )
		        theClassUuid += ae.get();
		}
		// now make sure content handlers are registered.
		contentTags = new TagEntry[] 
		{
            new TagEntry("name", new NameHandler(disassembler, this)),
            new TagEntry("uuid", new UuidHandler(disassembler, this)) 
		};
		this.setContent( new Content(contentTags, disassembler));
		//
		// start element semantic processing begins here.
		//
	}
	
	public void endElement(String namespaceURI, String localName, String name  )
	{
	    Object[] results = { theClassName, theClassUuid };
	    // next-to-last - call the result 
	    disassembler.result( name, results );
	    // last - unregister all my content handlers.
	    this.getContent().clearHandlers();
	}
	
    /**
	* Function called for element character data.
	*
	* @param    name    String containing name of element chardata is for.
	* @param	ch		array of character data.
	* @param	start	begining index of character data.
	* @param	length	length of character data.
	*
	* @see		SAXManager
    **/
	public void characters( String name, char ch[], int start, int length) 
	{
	    if ( name.equals(this.element) )
	    {
	        theClassName += String.copyValueOf(ch, start, length);
	    }
    }
    
    /**
     * Function called when a sub-handler completes and returns it's work.
	 *
	 * @param   name        String containing the name of an element just completed.
	 * @param   objectArray Array of objects, with are the result of the element just
	 *                      completed.  Presumably, the called handler's result 
	 *                      method knows how to handle the objects in the array,
	 *                      with the name of the completed element being the
	 *                      distinguishing hint.    
	 *
	 **/
	public void result( String name, Object[] objectArray ) 
    {
        int i = content.index(name);
        switch ( i )
        {
            case 0: // name
                theClassName += ( (String) objectArray[0] );
                break;
            case 1: // uuid
                theClassUuid += ( (String) objectArray[0] );
                break;
            default: // unknown name
	            // 
                Object[] parms = { name, this.element };    
                LogMsg logmsg = new LogMsg( Messages.UnknownElementReturned , parms );
                logmsg.prolog( "common.policies.configure", 200000000, "warning" );
                logmsg.log();
                // if the message is important enough to want to assure that it is written:
                // logmsg.flush(); 
                break;
        }
    }
}








