/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/policies/configure/ConstantHandler.java,v 1.2 2000/11/22 02:33:47 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.policies.configure;

import java.util.*;
import org.xml.sax.*;

//
// icecubes  imports
//
import com.adobe.rice.IceCubes;
import com.adobe.rice.common.*;
import com.adobe.rice.common.policies.*;
import com.adobe.rice.common.XMLdisassembly.*;
import com.adobe.rice.common.storage.logging.*;     // so we can write to the log.

public class ConstantHandler extends Handler 
{
    
    //
    // class wide variables
    //
    static final int NAME = 0;
    static final int VALUE = 1;
    static final int DESCRIPTION = 2;
    
    //
    // Instance Variables
    //
    private String theConstantName;
    private String theConstantNameAttribute;
    private String theConstantNameElement;
    
    private String theConstantValueString;
    
    private Object theConstantValue;
    private Object theConstantValueAttribute;
    private Object theConstantValuePcdata;
    private Object theConstantValueElement;
    
    private String theConstantDescription;
    private String theConstantType;
    
	public ConstantHandler(XMLDisassembler disassembler, Handler parent) 
	{		
		super(disassembler, parent);
		
		// All the ConstantHandler understands is constant.		
		this.element = "constant";
	
		ParserAttributes al = new ParserAttributes();
		
		al.add( new AttributeEntry("name", "CDATA" ) );
		al.add( new AttributeEntry("value", "CDATA" ) );
		al.add( new AttributeEntry("description", "CDATA" ) );
		al.add( new AttributeEntry("type", "CDATA", "int" ) );
		
		this.elementAttributes = al;
	}
	
	public void startElement(String namespaceURI, String localName, 
                             String name,      ParserAttributes attributes) 
	{
        TagEntry contentTags[] = null; 
	    AttributeEntry ae;

		if (name.equals(this.element)) 
		{
		    // run through the attributes and get their values
		    this.setupAttributes(attributes);

            // setup the attributes ...
            this.theConstantNameAttribute = this.getAttributeValue("name");
	        this.theConstantDescription = this.getAttributeValue("description");
	        this.theConstantType = this.getAttributeValue("type");
	        this.theConstantValueString = this.getAttributeValue("value");
            if ( ( this.theConstantType != null) && ( this.theConstantValueString != null) )
            {
                theConstantValueAttribute = Constants.convertValue( this.theConstantType, this.theConstantValueString );
            }
		    this.theConstantValueString = null;  // kill off the string value so we can use it later.    
		    // now make sure content handlers are registered.
		    contentTags = new TagEntry[] 
		    {
                new TagEntry("name", new NameHandler( disassembler, this)),
                new TagEntry("value", new ValueHandler( disassembler, this)),
                new TagEntry("description", new DescriptionHandler( disassembler, this))
		    };
		    this.setContent(new Content(contentTags, disassembler));
		}
		
	}
	
	public void endElement(String namespaceURI, String localName, String rawName )
	{
	    Object result[];
	    SimplePolicy thePolicy = null;
	    //
	    // Convert the value to its type:
	    //
	    if (( this.theConstantValueString != null ) &&
	        !this.theConstantValueString.equals("")    )
            theConstantValuePcdata = Constants.convertValue( this.theConstantType, this.theConstantValueString);
	    //
	    // select the constant value according to the rules:
	    //        choose first contained element, then pcdata, then attribute
	    //
	    if ( theConstantValueElement == null )
	    {
	        if ( theConstantValuePcdata == null )
	        {
	            theConstantValue = theConstantValueAttribute;
	        }
	        else
	        {
	            theConstantValue = theConstantValuePcdata;
	        }
	    }
	    else
	    {
	        theConstantValue = theConstantValueElement;
	    }
	    //
	    // select the correct name value according to the rules:
	    //      choose first element, then attribute (pcdata not allowed for name)
	    //
	    
	    if ( theConstantNameElement == null )
	    {
	        this.theConstantName = this.theConstantNameAttribute;
	    }
	    else
	    {
	        this.theConstantName = this.theConstantNameElement;
	    }
	    if ( theConstantName == null )
	    {
	        this.bumpErrorCount(1);
	        this.adjustErrorLevel(Constants.ERROR);
	        
            Object[] parms = { this.element, "Name"+Messages.emptyContent };    
            LogMsg logmsg = new LogMsg( Messages.elementInvalid, parms );
            logmsg.prolog( "common.policies.configure", 300000000, "error" );
            logmsg.log();
            // if the message is important enough to want to assure that it is written:
            // logmsg.flush(); 
            this.theConstantName = "_unnamedConstant_";
	    }
	    if ( theConstantValue == null ) // a policy with a null object is not allowed.
	    {
	        this.bumpErrorCount(1);
	        this.adjustErrorLevel(Constants.SEVERE);
	        
            Object[] parms = { this.element, "Value "+Messages.emptyContent };    
            LogMsg logmsg = new LogMsg( Messages.elementInvalid, parms );
            logmsg.prolog( "common.policies.configure", 400000000, "severe error" );
            logmsg.log();
            // if the message is important enough to want to assure that it is written:
            // logmsg.flush(); 
	    }
	    else // now create a policy to hand back!
	    {   
	        thePolicy = new SimplePolicy(theConstantValue.getClass(), theConstantValue );
	        thePolicy.setName(this.theConstantName);
	    }
	    result = new Object[] 
	    {
	        new Integer( this.getErrorCount() ),
	        new Integer( this.getErrorLevel() ),
	        thePolicy,
	        this.theConstantDescription
	    };
	    // next-to-last - call the result 
	    disassembler.result( rawName, result);
	    // last - unregister all my content handlers.
	    this.content.clearHandlers();
	}
	
    /**
	* Function called for element character data.
	*
	* @param    name    String containing name of element chardata is for.
	* @param	ch		array of character data.
	* @param	start	begining index of character data.
	* @param	length	length of character data.
	*
	* @see		SAXManager
     **/
	public void characters( String name, char ch[], int start, int length) 
	{
	    if ( name.equals(this.element) )
	    {
	        theConstantValueString = String.copyValueOf(ch, start, length);
	    }
    }
    
    /**
     * Function called when a sub-handler completes and returns it's work.
	 *
	 * @param   name        String containing the name of an element just completed.
	 * @param   objectArray Array of objects, with are the result of the element just
	 *                      completed.  Presumably, the called handler's result 
	 *                      method knows how to handle the objects in the array,
	 *                      with the name of the completed element being the
	 *                      distinguishing hint.    
	 *
	 **/
	public void result( String name, Object[] objectArray ) 
    {
        int i = content.index(name);
        if ( i >= 0 )
        {
            Integer elementErrorCount = (Integer) objectArray[0];
            Integer elementErrorLevel = (Integer) objectArray[1];
                
            this.bumpErrorCount(elementErrorCount.intValue() );
            this.adjustErrorLevel(elementErrorLevel.intValue() );
        }
        switch ( i )
        {
        case NAME: // name
            theConstantNameElement = ( (String) objectArray[2] );
            break;
        case VALUE: // value
            theConstantValueElement = 
                Constants.convertValue( this.theConstantType, (String) objectArray[2] );
            break;
        case DESCRIPTION: // description
            theConstantDescription = ( (String) objectArray[2] );
            break;
        default: // unknown name
    	    // 
    	    this.bumpErrorCount(1);
    	    this.adjustErrorLevel( Constants.WARNING );
	        
            Object[] parms = { name, this.element };    
            LogMsg logmsg = new LogMsg( Messages.UnknownElementReturned , parms );
            logmsg.prolog( "common.policies.configure", 200000000, "warning" );
            logmsg.log();
            // if the message is important enough to want to assure that it is written:
            // logmsg.flush(); 
            break;
        }
    }
    
    //
    // utility functions
    //
    /**
     * Return the type of this range
     *
     * @param void
     * @return String - the name of the type
     **/
    public String getType()
    {
        AttributeEntry ae = this.findAttribute("type");
        if ( ae != null )
            return ae.getValue();
        return "";
    }
    
}
