/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/policies/configure/DescriptionHandler.java,v 1.2 2000/11/22 02:33:47 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.policies.configure;

import java.util.*;
import org.xml.sax.*;

//
// icecubes  imports
//
import com.adobe.rice.IceCubes;
import com.adobe.rice.common.*;
import com.adobe.rice.common.policies.*;
import com.adobe.rice.common.XMLdisassembly.*;
import com.adobe.rice.common.storage.logging.*;     // so we can write to the log.

public class DescriptionHandler extends Handler 
{
    //
    // private class instance variables
    //
    	
    private String theDescriptionName = "";
    private String theDescription = "";
    private String theDescriptionSummary = "";
    
	public DescriptionHandler(XMLDisassembler disassembler, Handler parent) 
	{		
		super(disassembler, parent);
		
		// All the DescriptionHandler understands is description
		this.element = "description";
	
		ParserAttributes al = new ParserAttributes();
		
		al.add( new AttributeEntry("name", "CDATA" ) );
		al.add( new AttributeEntry("summary", "CDATA" ) );
		
		this.elementAttributes = al;
	}
	
	public void startElement(String namespaceURI, String localName, 
                             String rawName,      ParserAttributes attributes) 
	{
	    // 
	    // merge in the attributes
	    //
	    this.setupAttributes(attributes);
	    //
	    this.theDescriptionName = this.getAttributeValue("name");
	    this.theDescriptionSummary = this.getAttributeValue("summary");
	    
		  
		// make sure content handlers are registered.
		// do this as late as possible
		TagEntry[] contentTags = null;
		//
		this.setContent( new Content(contentTags, disassembler));
		//
		// Begin Semantic Processing Here:
		//
	}
	
	public void endElement(String namespaceURI, String localName, 
                           String name       )
	{
	    Object[] result = 
	        {   new Integer(this.getErrorCount()), 
	            new Integer(this.getErrorLevel()),
	            this.theDescription,
	            this.theDescriptionName,
	            this.theDescriptionSummary
	        };
	    // next-to-last - call the result 
	    disassembler.result( name, result);
	    // last - unregister all my content handlers.
        this.content.clearHandlers();
	}
	
    /**
	* Function called for element character data.
	*
	* @param    name    String containing name of element chardata is for.
	* @param	ch		array of character data.
	* @param	start	begining index of character data.
	* @param	length	length of character data.
	*
	* @see		SAXManager
     **/
	public void characters( String name, char ch[], int start, int length) 
	{
	    if ( name.equals( this.element ) )
	    {
	        theDescription += String.copyValueOf(ch, start, length);
	    }
    
    }
    
    /**
     * Function called when a sub-handler completes and returns it's work.
	 *
	 * @param   name        String containing the name of an element just completed.
	 * @param   objectArray Array of objects, with are the result of the element just
	 *                      completed.  Presumably, the called handler's result 
	 *                      method knows how to handle the objects in the array,
	 *                      with the name of the completed element being the
	 *                      distinguishing hint.    
	 *
	 * @see		Handler in package XMLdisassembly
	 *
	 **/
	public void result( String name, Object[] objectArray ) 
    {
        int i = content.index(name);
        switch ( i )
        {
        default: // unknown name
	        // 
	        this.bumpErrorCount(1);
	        this.adjustErrorLevel(Constants.ERROR);
	        
            Object[] parms = { name, this.element };    
            LogMsg logmsg = new LogMsg( Messages.UnknownElementReturned , parms );
            logmsg.prolog( "common.policies.configure", 300000000, "error" );
            logmsg.log();
            // if the message is important enough to want to assure that it is written:
            // logmsg.flush(); 
            break;
        }
    }
}








