/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/policies/configure/EnumerationHandler.java,v 1.3 2000/11/22 02:33:47 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.policies.configure;

import java.util.*;
import org.xml.sax.*;

//
// icecubes  imports
//
import com.adobe.rice.IceCubes;
import com.adobe.rice.common.*;
import com.adobe.rice.common.policies.*;
import com.adobe.rice.common.policies.configure.*;
import com.adobe.rice.common.XMLdisassembly.*;
import com.adobe.rice.common.storage.logging.*;     // so we can write to the log.

public class EnumerationHandler extends Handler 
{
    //
    // class wide variables
    //
    static final int NAME = 0;
    static final int DESCRIPTION = 1;
    static final int SELECT = 2;
    static final int ITEM = 3;
    
    private class EnumItem
    {
        Object item;
        String name;
        String id;
        String description;
        
        EnumItem()
        {
            item = null;
            name = null;
            id = null;
            description = null;
        }
        EnumItem( Object theItem, String theName, String theId, String theDescription )
        {
            this();
            item = theItem;
            name = theName;
            id = theId;
            description = theDescription;
        }
        
        void setItem( Object theItem ) { item = theItem; }
        Object getItem() { return item; }
        void setName( String theName ) { name = theName; }
        String getName() { return name; }
        void setId( String theId ) { id = theId; }
        String getId() { return id; }
        void setDescription( String theDesc ) { description = theDesc; }
        String getDescription() { return description; }
    }
    //
    // private class instance variables
    //
	HashMap itemMap;
	
	String theType;
	
	String theName;
	String theNameAttribute;
	String theNameElement;
	
	String theSelect;
	String theSelectAttribute;
	String theSelectElement;
	
	String theId;
	
	String theDescription;
    //
    // Constructors
    //
	public EnumerationHandler(XMLDisassembler disassembler, Handler parent) 
	{		
		super(disassembler, parent);
		
		this.element = "enumeration";
	
		ParserAttributes al = new ParserAttributes();
		
		al.add( new AttributeEntry("id", "ID" ) );
		al.add( new AttributeEntry("name", "CDATA" ) );
		al.add( new AttributeEntry("select", "IDREF" ) );
		al.add( new AttributeEntry("type", "CDATA"));
		
		this.elementAttributes = al;
	}
	
	public void startElement(String namespaceURI, String localName, 
                             String name, ParserAttributes attributes ) 
	{
        TagEntry contentTags[] = null; 
	    
		if ( name.equals(this.element) )
		{
		    this.setupAttributes(attributes);
		    
		    theNameAttribute = getAttributeValue("name");
		        
		    theType = getAttributeValue("type");
		    if ( theType == null )
		        theType = Constants.DEFAULT_ITEM_TYPE;
		        
            theId = getAttributeValue("id");
            
            theSelectAttribute = getAttributeValue("select");
            
		    // make sure content handlers are registered.
		    // do this as late as possible
		    contentTags = new TagEntry[] 
		    {
		        // create a TagEntry for each element in ModelContent's content model here.
		        // Note that the order of the TagEntrys corresponds to the indexes used
		        // below -- if TAGNAME is the tag name in the first TagEntry, then it is
		        // index 0 below in result.
		        //
		        // replace the UnimplementedHandler in each TagEntry with an appropriate handler when you implement it.
                new TagEntry("name", new NameHandler( disassembler, this)),
                new TagEntry("description", new DescriptionHandler( disassembler, this)),
                new TagEntry("select", new SelectHandler( disassembler, this)),
                new TagEntry("item", new ItemHandler( disassembler, this))
		    };
		    this.setContent( new Content(contentTags, disassembler));
		}
		//
		// Begin Semantic Processing Here:
		//
		// tell the children about the type of this enumeration
		this.setPrepared( new Object[] { theType } );
		// setup the item map to store items
		//
		itemMap = new HashMap(Constants.EXPECTED_ENUM_ITEMS);
	}
	
	public void endElement(String namespaceURI, String localName, 
                             String name )
	{
	    Class policyClass = this.getClassFromType(this.theType);
	    EnumItem selectedItem = null;
	    ChoicePolicy theChoicePolicy = null;
	    //
	    // Complete Semantic Processing Here:
	    //
	    // get name according to precedence rules:
	    if ( this.theNameElement == null )
	        this.theName = this.theNameAttribute;
	    else
	        this.theName = this.theNameElement;
	        
	    if ( this.theName == null )
	    {
	        // error - enumeration name cannot be null
	        this.bumpErrorCount(1);
	        this.adjustErrorLevel(Constants.ERROR);
    	        
            Object[] parms = { this.element, Messages.elementMustHaveName }; 
            LogMsg logmsg = new LogMsg( Messages.elementInvalid , parms );
            logmsg.prolog( "common.policies.configure", 300000000, "error" );
            logmsg.log();
            
            // give it a phony name.
            this.theName = "_MissingName_";

	    }
	    // get select according to precedence rules 
	    if ( this.theSelectElement == null )
	        this.theSelect = this.theSelectAttribute;
	    else
	        this.theSelect = this.theSelectElement;
	    if ( this.theSelect == null )
	    {
	        // error - at least one item must be selected.
	        this.bumpErrorCount(1);
	        this.adjustErrorLevel(Constants.ERROR);
    	        
            Object[] parms = { this.element, Messages.noItemSelected }; 
            LogMsg logmsg = new LogMsg( Messages.elementInvalid , parms );
            logmsg.prolog( "common.policies.configure", 300000000, "error" );
            logmsg.log();
	    }
	    // make the choice policy if there is something to make.
	    
	    if ( ( policyClass != null) && ( !this.itemMap.isEmpty() ) )
	    {
    	    // create a new ChoicePolicy and set its name.
	        theChoicePolicy = new ChoicePolicy( policyClass );
	        theChoicePolicy.setName(theName);
	        // now incorporate the choices from the items.
	        // 1. get a collection of the values
	        Collection policyItems = this.itemMap.values();
	        // 2. construct iterator to run through the item hashMap.
	        // 3. iterate through the items adding them to the choice policy
	        // use only the item object; the other interesting information
	        // except the id is not used.
	        
	        for ( Iterator i = policyItems.iterator(); i.hasNext(); )
	        {
	            EnumItem ei = (EnumItem) i.next();
	            theChoicePolicy.add(ei.getItem());
	            if ( ei.getId().equals(theSelect) )
	                selectedItem = ei;
	        }
	        if ( selectedItem == null )
	        {
	            this.bumpErrorCount(1);
	            this.adjustErrorLevel(Constants.ERROR);
    	        
                Object[] parms = { this.element, Messages.noItemSelected }; 
                LogMsg logmsg = new LogMsg( Messages.elementInvalid , parms );
                logmsg.prolog( "common.policies.configure", 300000000, "error" );
                logmsg.log();
	        }
	        else
	        {
	            try 
	            {
                    theChoicePolicy.select(selectedItem.getItem());	
                }
                catch ( PolicyException pe )
                {
	                this.bumpErrorCount(1);
	                this.adjustErrorLevel(Constants.SEVERE);
    	        
                    Object[] parms = { this.element, selectedItem.getName() + Messages.itemSelectionFailed  }; 
                    LogMsg logmsg = new LogMsg( Messages.elementInvalid , parms );
                    logmsg.prolog( "common.policies.configure", 300000000, "error" );
                    logmsg.log();
                }
            }
                
        }
        else // either the itemMap is empty or the policy class is invalid
        {
            Object[] parms;
            LogMsg logmsg = null;
            
            if ( this.itemMap.isEmpty() )
            {
	            this.bumpErrorCount(1);
	            this.adjustErrorLevel(Constants.SEVERE);
        	        
                parms = new Object[] { this.element, Messages.noItemsInEnumeration }; 
                logmsg = new LogMsg( Messages.elementInvalid , parms );
                logmsg.prolog( "common.policies.configure", 400000000, "severe error" );
                logmsg.log();
            }
            if ( policyClass == null )
            {
	            this.bumpErrorCount(1);
	            this.adjustErrorLevel(Constants.SEVERE);
        	        
                parms = new Object[] { this.element, Messages.unknownTypeInEnumeration }; 
                logmsg = new LogMsg( Messages.elementInvalid , parms );
                logmsg.prolog( "common.policies.configure", 400000000, "severe error" );
                logmsg.log();
            }
            logmsg.flush();
        }
	        
	    Object[] results = 
	    {
	        new Integer(this.getErrorCount()),
	        new Integer(this.getErrorLevel()),
	        theChoicePolicy
	    };
	    //
	    // call result here after completing semantic processing
	    //
	    disassembler.result( name, results );
	    //
	    // last - unregister all my content handlers.
	    //
	    this.content.clearHandlers();
	}
	
    /**
	* Function called for element character data.
	*
	* @param    name    String containing name of element chardata is for.
	* @param	ch		array of character data.
	* @param	start	begining index of character data.
	* @param	length	length of character data.
	*
	* @see		SAXManager
    **/
	public void characters( String name, char ch[], int start, int length) 
	{
	    if ( name.equals(this.element) )
	    {   // no characters are allowed in the enumeration element
    	    this.bumpErrorCount(1);
    	    this.adjustErrorLevel( Constants.WARNING );
	        
            Object[] parms = { this.element, Messages.noCharactersInModel };    
            LogMsg logmsg = new LogMsg( Messages.elementInvalid , parms );
            logmsg.prolog( "common.policies.configure", 200000000, "warning" );
            logmsg.log();
            // if the message is important enough to want to assure that it is written:
            // logmsg.flush(); 
	    }
    }
    
    /**
     * Function called when a sub-handler completes and returns it's work.
	 *
	 * @param   name        String containing the name of an element just completed.
	 * @param   objectArray Array of objects, with are the result of the element just
	 *                      completed.  Presumably, the called handler's result 
	 *                      method knows how to handle the objects in the array,
	 *                      with the name of the completed element being the
	 *                      distinguishing hint.    
	 *
	 * @see		Handler in package XMLdisassembly
	 *
	 **/
	public void result( String name, Object[] objectArray ) 
    {
        int i = content.index(name);
        if ( i >= 0 )
        {
            Integer elementErrorCount = (Integer) objectArray[0];
            Integer elementErrorLevel = (Integer) objectArray[1];
                
            this.bumpErrorCount(elementErrorCount.intValue() );
            this.adjustErrorLevel(elementErrorLevel.intValue() );
        }
        switch ( i )
        {
        case NAME: // name
            this.theNameElement = (String) objectArray[2];
            break;
        case DESCRIPTION:
            this.theDescription += (String) objectArray[2];
            break;
        case SELECT:
            this.theSelectElement = (String) objectArray[2];
            break;
        case ITEM:
            Object theItemObject = objectArray[2];
            String theItemName = (String) objectArray[3];
            String theItemId = (String) objectArray[4];
            String theItemDescription = (String) objectArray[5];
            EnumItem ei = (EnumItem) itemMap.get(theItemId);
            if ( ei != null )
            {   // error - duplicate item... override old one.
                Object[] parms = { this.element, name + Messages.duplicateContentElement };
                LogMsg logmsg = new LogMsg( Messages.elementInvalid, parms );
                logmsg.prolog( "common.policies.configure", 30000000, "error" );
                logmsg.log();
                // override the old one.
                ei.setDescription(theItemDescription);
                ei.setItem(theItemObject);
                ei.setName(theItemName);
            }
            else
            {   // construct a new item
                ei = new EnumItem(theItemObject, theItemName, theItemId, theItemDescription );
                itemMap.put(theItemId, ei );
            }
            break;
        default: // unknown name
    	    // 
    	    this.bumpErrorCount(1);
    	    this.adjustErrorLevel( Constants.WARNING );
	        
            Object[] parms = { name, this.element };    
            LogMsg logmsg = new LogMsg( Messages.UnknownElementReturned , parms );
            logmsg.prolog( "common.policies.configure", 200000000, "warning" );
            logmsg.log();
            // if the message is important enough to want to assure that it is written:
            // logmsg.flush(); 
            break;
        }
    }
    
    /**
     * Return the type of this enumeration
     *
     * @param void
     * @return String - the name of the type
     **/
    public String getType()
    {
        return getAttributeValue("type");
    }
    
    private Class getClassFromType( String typeName )
    {
        int i = Constants.getTypeIndex( typeName );
        
        Class result;
        
        switch ( i )
        {
        case Constants.TYPE_BYTE:
            result = Byte.TYPE;
            break;
        case Constants.TYPE_SHORT:
            result = Short.TYPE;
            break;
        case Constants.TYPE_INT:
            result = Integer.TYPE;
            break;
        case Constants.TYPE_LONG:
            result = Long.TYPE;
            break;
        case Constants.TYPE_FLOAT:
            result = Float.TYPE;
            break;
        case Constants.TYPE_DOUBLE:
            result = Double.TYPE;
            break;
        case Constants.TYPE_BOOLEAN:
            result = Boolean.TYPE;
            break;
        case Constants.TYPE_CHAR:
            result = Character.TYPE;
            break;
        case Constants.TYPE_STRING:
            result = String.class;
            break;
        case Constants.TYPE_STRINGBUFFER:
            result = StringBuffer.class;
            break;
        default:
            // how can we figure out if the type exists and is known
            // so that we can return it?
            try 
            {
                result = Class.forName(typeName);
            }
            catch ( Exception ex )
            {
                result = null;
            }
            break;
        }
        return result;
    }
}
