/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/policies/configure/ItemHandler.java,v 1.2 2000/11/22 02:33:47 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.policies.configure;

import java.util.*;
import org.xml.sax.*;

//
// icecubes  imports
//
import com.adobe.rice.IceCubes;
import com.adobe.rice.common.*;
import com.adobe.rice.common.policies.*;
import com.adobe.rice.common.XMLdisassembly.*;
import com.adobe.rice.common.storage.logging.*;     // so we can write to the log.

public class ItemHandler extends Handler 
{
    //
    // class wide variables
    //
    static final int NAME = 0;
    static final int VALUE = 1;
    static final int DESCRIPTION = 2;
    
    //
    // instance variables
    //
    String theType = Constants.DEFAULT_ITEM_TYPE;   // The type the item is required to be.
    
    Object theValue;
    Object theValueAttribute;
    Object theValueElement;
    Object theValuePcdata;
    
    String theStringValuePcdata;
    String theStringValueAttribute;
    
    
    String theName;
    String theNameAttribute;
    String theNameElement;
    
    String theDescription;
    String theDescriptionAttribute;
    String theDescriptionElement;
    
    String theId;
    //
    // Constructors
    //
	
	public ItemHandler(XMLDisassembler disassembler, Handler parent) 
	{		
		super(disassembler, parent);
		
		// All the ItemHandler understands is item		
		this.element = "item";
	
		ParserAttributes al = new ParserAttributes();
		
		al.add( new AttributeEntry("id", "ID" ) );
		al.add( new AttributeEntry("name", "NMTOKEN" ) );
		al.add( new AttributeEntry("value", "CDATA" ) );
		al.add( new AttributeEntry("description", "CDATA" ) );
		
		this.elementAttributes = al;
		
	}
	
	public void startElement(String namespaceURI, String localName, 
                             String name, ParserAttributes attributes ) 
	{
        TagEntry contentTags[] = null; 
        Object prepare[] = this.parent.getPrepared();
	    
		if (name.equals(this.element)) 
		{
		    // run through the attributes and get their values
		    this.setupAttributes(attributes);
		    
		    if ( (prepare != null) && ( prepare[0] != null ) )
		        theType = (String) prepare[0];
		        
		    theNameAttribute = this.getAttributeValue("name");
		    
		    theId = this.getAttributeValue("id");
		    
		    theStringValueAttribute = this.getAttributeValue("value");
		    if ( this.theStringValueAttribute != null )
    		    theValueAttribute = Constants.convertValue( this.theType, this.theStringValueAttribute );
		        
		    theDescriptionAttribute = this.getAttributeValue("description");    
		}
		// make sure content handlers are registered.
		// do this as late as possible
		contentTags = new TagEntry[] 
		{
		    // create a TagEntry for each element in ModelContent's content model here.
		    // Note that the order of the TagEntrys corresponds to the indexes used
		    // below -- if TAGNAME is the tag name in the first TagEntry, then it is
		    // index 0 below in result.
		    //
            new TagEntry("name", new NameHandler( disassembler, this)),
            new TagEntry("value", new ValueHandler( disassembler, this)),
            new TagEntry("description", new DescriptionHandler( disassembler, this))
		};
		content = new Content(contentTags, disassembler);
		//
		// Begin Semantic Processing Here:
		//
	}
	
	public void endElement(String namespaceURI, String localName, 
                             String name )
	{
	    //
	    // Complete Semantic Processing Here:
	    //
	    if ( this.theStringValuePcdata != null )
	        this.theValuePcdata = Constants.convertValue( this.theType, this.theStringValuePcdata );
	    
	    // choose the value according to the precedence rules.
	    if ( theValueElement != null )
	    {
	        theValue = theValueElement;
	    }
	    else if ( theValuePcdata != null )
	    {
	        theValue = theValuePcdata;
	    }
	    else
	    {
	        theValue = theValueAttribute;
	    }
	    
	    if ( theValue == null )
	    {
	        this.bumpErrorCount(1);
	        this.adjustErrorLevel( Constants.SEVERE );
	        
            Object[] parms = { this.element, Messages.itemValueRequired };    
            LogMsg logmsg = new LogMsg( Messages.elementInvalid , parms );
            logmsg.prolog( "common.policies.configure", 400000000, "severe error" );
            logmsg.log();
            // if the message is important enough to want to assure that it is written:
            // logmsg.flush();
	    }
	    
	    // choose the name according to the precedence rules
	    if ( this.theNameElement != null )
	    {
	        theName = theNameElement;
	    }
	    else
	    {
	        if ( this.theNameAttribute != null )
	            theName = theNameAttribute;
	    }
	    // Note that it is possible to have an unnamed item ( name is null ).
	    // choose the description according to the precedence rules
	    if ( this.theDescriptionElement != null )
	    {
	        this.theDescription = this.theDescriptionElement;
	    }
	    else
	    {
	        if ( this.theDescriptionAttribute != null )
	            this.theDescription = this.theDescriptionAttribute;
	    }
	    // note that it is possible to have a null description.
	    
	    if ( this.theId == null )
	    {
	        this.bumpErrorCount(1);
	        this.adjustErrorLevel( Constants.ERROR );
	        
            Object[] parms = { this.element, Messages.selectionIdRequired };    
            LogMsg logmsg = new LogMsg( Messages.elementInvalid , parms );
            logmsg.prolog( "common.policies.configure", 300000000, "error" );
            logmsg.log();
            // if the message is important enough to want to assure that it is written:
            // logmsg.flush();
	    }
	    
	    Object[] results = 
	    { 
	        new Integer(this.getErrorCount()),
	        new Integer(this.getErrorLevel()),
	        theValue, 
	        theName, 
	        theId, 
	        theDescription 
	    };
	    // next-to-last - call the result 
	    disassembler.result( name, results );
	    // last - unregister all my content handlers.
	    this.getContent().clearHandlers();
	}
	
    /**
	* Function called for element character data.
	*
	* @param    name    String containing name of element chardata is for.
	* @param	ch		array of character data.
	* @param	start	begining index of character data.
	* @param	length	length of character data.
	*
	* @see		SAXManager
     **/
	public void characters( String name, char ch[], int start, int length) 
	{
	    if ( name.equals(this.element) )
	    {
	        this.theStringValuePcdata = new String(ch, start, length);
	    }
    }
    
    /**
     * Function called when a sub-handler completes and returns it's work.
	 *
	 * @param   name        String containing the name of an element just completed.
	 * @param   objectArray Array of objects, with are the result of the element just
	 *                      completed.  Presumably, the called handler's result 
	 *                      method knows how to handle the objects in the array,
	 *                      with the name of the completed element being the
	 *                      distinguishing hint.    
	 *
	 * @see		Handler in package XMLdisassembly
	 *
	 **/
	public void result( String name, Object[] objects ) 
    {
        int i = content.index(name);
        // check for and set any errors
        if ( i >= 0 )
        {
            Integer elementErrorCount = (Integer) objects[0];
            Integer elementErrorLevel = (Integer) objects[1];
                
            this.bumpErrorCount(elementErrorCount.intValue() );
            this.adjustErrorLevel(elementErrorLevel.intValue() );
        }
        switch ( i )
        {
            // When an element in the content of this element completes, it calls this 
            // routine with it's results.  That is if element foo has element bar as
            // content, then when element bar contained by foo completes, the result
            // is handed back to foo through this routine.
            //
            // Handle the results here.  Note -- The theory is that this handler is
            // little more than a packager of the effort of the lower level routines.
            // Build the lower level routines to handle everything that is locally possible
            // about the element.  The result routine (this one) should be designed to
            // integrate the lower level elements into a larger whole.  At this level,
            // we can do little but schedule the returning response.
            
            // The index here is the index of the elements in contentTags with the
            // first starting at 0.
            
        case NAME: // name
            this.theNameElement = (String) objects[2];
            break;
        case VALUE: // value
            String theStringValue = (String) objects[2];
            if ( theStringValue != null )
                this.theValueElement = Constants.convertValue( theType, theStringValue );
            else
                this.theValueElement = null;
            break;
        case DESCRIPTION: // description
            this.theDescriptionElement = (String) objects[2];
            break;
        default: // unknown name
	        // 
	        this.bumpErrorCount(1);
	        this.adjustErrorLevel( Constants.WARNING );
	        
            Object[] parms = { name, this.element };    
            LogMsg logmsg = new LogMsg( Messages.UnknownElementReturned , parms );
            logmsg.prolog( "common.policies.configure", 200000000, "warning" );
            logmsg.log();
            // if the message is important enough to want to assure that it is written:
            // logmsg.flush(); 
            break;
        }
    }
}
