/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/policies/configure/PackageHandler.java,v 1.2 2000/11/22 02:33:47 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

/*
    Initial Checking    2000-08-31  vbhunt
*/

package com.adobe.rice.common.policies.configure;

import java.util.*;
import org.xml.sax.*;

//
// icecubes  imports
//
import com.adobe.rice.IceCubes;
import com.adobe.rice.common.*;
import com.adobe.rice.common.policies.*;
import com.adobe.rice.common.XMLdisassembly.*;
import com.adobe.rice.common.storage.logging.*;     // so we can write to the log.

public class PackageHandler extends Handler 
{
    //
    // class wide variables
    //
    static final int NAME = 0;
    static final int DESCRIPTION = 1;

    //
    // private class instance variables
    //
    
    String theName;
    String theNameAttribute;
    String theNamePcdata;
    String theNameElement;
    
    String theId;
    
    String theDescription;
    String theDescriptionAttribute;
    String theDescriptionElement;
    
    //
    // Constructors
    //
	
	public PackageHandler(XMLDisassembler disassembler, Handler parent) 
	{		
		super(disassembler, parent);
		
		// All the PackageHandler understands is package
		this.element = "package";
	
		ParserAttributes al = new ParserAttributes();
		
		al.add( new AttributeEntry("id", "ID" ) );
		al.add( new AttributeEntry("name", "NMTOKEN" ) );
		al.add( new AttributeEntry("description", "CDATA" ) );
		
		this.elementAttributes = al;
	}
	
	public void startElement(String namespaceURI, String localName, 
                             String name, ParserAttributes attributes) 
	{
        TagEntry contentTags[] = null; 
	    
		if (name.equals(this.element)) 
		{
		    // run through the attributes and get their values
		    this.setupAttributes(attributes);
		    
		    this.theDescriptionAttribute = this.getAttributeValue("description");
		    this.theId = this.getAttributeValue("id");
		    this.theNameAttribute = this.getAttributeValue("name");
		}
		// make sure content handlers are registered.
		// do this as late as possible
		
		contentTags = new TagEntry[] 
		{
            new TagEntry("name", new NameHandler( disassembler, this)),
            new TagEntry("description", new DescriptionHandler( disassembler, this))
		};
		this.setContent( new Content(contentTags, disassembler) );
		//
		// Begin Semantic Processing Here:
		//
	}
	
	public void endElement( String namespaceURI, String localName, String rawName )
	{
	    if ( this.theNameElement == null )
	    {
	        if ( this.theNamePcdata == null )
	            this.theName = this.theNameAttribute;
	        else
	            this.theName = this.theNamePcdata;
	    }
	    else
	        this.theName = this.theNameElement;
	        
	    if ( ( this.theName == null ) || this.theName.equals("") )
	    {
	        this.bumpErrorCount(1);
	        this.adjustErrorLevel(Constants.ERROR);
	        
            Object[] parms = { this.element, "Name " + Messages.emptyContent }; 
            LogMsg logmsg = new LogMsg( Messages.elementInvalid , parms );
            logmsg.prolog( "common.policies.configure", 300000000, "error" );
            logmsg.log();
            
            this.theName = Constants.ALL_PACKAGES;  // set it by default to all packages...
            
	    }
	    // convert the package name for all packages to the global policies name.
	    // this makes sure that a policy designed for all packages gets put in
	    // the global policy name space.
	    if ( this.theName.equals(Constants.ALL_PACKAGES) )
	        this.theName = com.adobe.rice.common.policies.Constants.GLOBAL_POLICY;
	        
	    //
	    // now get the description
	    //
	    if ( this.theDescriptionElement == null )
	    {
	        this.theDescription = this.theDescriptionAttribute;
	    }
	    else
	    {
	        this.theDescription = this.theDescriptionElement;
	    }
	        
	    Object results[] = 
	    {
	        new Integer(this.getErrorCount()),
	        new Integer(this.getErrorLevel()),
	        this.theName,
	        this.theId,
	        this.theDescription
	    };
	    // next-to-last - call the result 
	    disassembler.result( rawName, results );
	    // last - unregister all my content handlers.
	    this.getContent().clearHandlers();
	}
	
    /**
	* Function called for element character data.
	*
	* @param    name    String containing name of element chardata is for.
	* @param	ch		array of character data.
	* @param	start	begining index of character data.
	* @param	length	length of character data.
	*
	* @see		SAXManager
     **/
	public void characters( String name, char ch[], int start, int length) 
	{
	    if ( name.equals(this.element) )
	    {
	        this.theNamePcdata = String.copyValueOf(ch, start, length);
            // the package name can also be in the "#PCDATA"
	    }
    }
    
    /**
     * Function called when a sub-handler completes and returns it's work.
	 *
	 * @param   name        String containing the name of an element just completed.
	 * @param   objectArray Array of objects, with are the result of the element just
	 *                      completed.  Presumably, the called handler's result 
	 *                      method knows how to handle the objects in the array,
	 *                      with the name of the completed element being the
	 *                      distinguishing hint.    
	 *
	 * @see		Handler in package XMLdisassembly
	 *
	 **/
	public void result( String name, Object[] objectArray ) 
    {
        int i = content.index(name);
        if ( i >= 0 )
        {
            Integer elementErrorCount = (Integer) objectArray[0];
            Integer elementErrorLevel = (Integer) objectArray[1];
                
            this.bumpErrorCount(elementErrorCount.intValue() );
            this.adjustErrorLevel(elementErrorLevel.intValue() );
        }
        switch ( i )
        {
            // When an element in the content of this element completes, it calls this 
            // routine with it's results.  That is if element foo has element bar as
            // content, then when element bar contained by foo completes, the result
            // is handed back to foo through this routine.
            //
            // Handle the results here.  Note -- The theory is that this handler is
            // little more than a packager of the effort of the lower level routines.
            // Build the lower level routines to handle everything that is locally possible
            // about the element.  The result routine (this one) should be designed to
            // integrate the lower level elements into a larger whole.  At this level,
            // we can do little but schedule the returning response.
            
            // The index here is the index of the elements in contentTags with the
            // first starting at 0.
            
            case NAME: // name
                // the package name can be in the "name" element
                if ( objectArray[2] != null )
                    this.theNameElement = (String) objectArray[2];
                break;
            case DESCRIPTION: // description
                // the package description can be in the "description" element.
                if ( objectArray[2] != null )
                    this.theDescriptionElement = (String) objectArray[2];
                break;
            default: // unknown name
	            // 
                Object[] parms = { name, this.element };    
                LogMsg logmsg = new LogMsg( Messages.UnknownElementReturned , parms );
                logmsg.prolog( "common.policies.configure", 200000000, "warning" );
                logmsg.log();
                // if the message is important enough to want to assure that it is written:
                // logmsg.flush(); 
                break;
        }
    }
}








