/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/policies/configure/ParameterHandler.java,v 1.2 2000/11/22 02:33:47 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.policies.configure;

import java.util.*;
import org.xml.sax.*;

//
// icecubes  imports
//
import com.adobe.rice.IceCubes;
import com.adobe.rice.common.*;
import com.adobe.rice.common.policies.*;
import com.adobe.rice.common.XMLdisassembly.*;
import com.adobe.rice.common.storage.logging.*;     // so we can write to the log.

public class ParameterHandler extends Handler 
{
    //
    // class wide variables
    //
    static final int NAME = 0;
    static final int VALUE = 1;
    static final int DESCRIPTION = 2;
    
    //
    // private class instance variables
    //
    String theName = "";
    String theNameAttribute = "";
    String theNameElement = "";
    
    String theValue = "";
    String theValueAttribute = "";
    String theValueElement = "";
    
    String theSummary = "";
    
    String theDescription = "";
    
	//
	// constructors
	//
	
	public ParameterHandler(XMLDisassembler disassembler, Handler parent) 
	{		
		super(disassembler, parent);
		
		// All the ParameterHandler understands is itself
		this.element = "parameter";
	
		ParserAttributes al = new ParserAttributes();
		
		al.add( new AttributeEntry("name", "NMTOKEN" ) );
		al.add( new AttributeEntry("value", "CDATA" ) );
		al.add( new AttributeEntry("summary", "CDATA" ) );
		
		this.elementAttributes = al;
	}
	
	public void startElement(String namespaceURI, String localName, 
                             String name, ParserAttributes attributes ) 
	{
        TagEntry contentTags[] = null; 
	    
		if (name.equals(this.element)) 
		{
		    // run through the attributes and get their values
		    this.setupAttributes(attributes);
		    
		    this.theNameAttribute = this.getAttributeValue("name");
		    this.theValueAttribute = this.getAttributeValue("value");
		    this.theSummary = this.getAttributeValue("summary");
		}
		// make sure content handlers are registered.
		// do this as late as possible
		
		contentTags = new TagEntry[] 
		{
		    // create a TagEntry for each element in ModelContent's content model here.
		    // Note that the order of the TagEntrys corresponds to the indexes used
		    // below -- if TAGNAME is the tag name in the first TagEntry, then it is
		    // index 0 below in result.
		    //
		    // replace the UnimplementedHandler in each TagEntry with an appropriate handler when you implement it.
            new TagEntry("name", new NameHandler( disassembler, this)),
            new TagEntry("value", new ValueHandler( disassembler, this)),
            new TagEntry("description", new DescriptionHandler( disassembler, this))
		};
		this.setContent( new Content(contentTags, disassembler) );
		//
		// Begin Semantic Processing Here:
		//
	}
	
	public void endElement(String namespaceURI, String localName, String name )
	{
	    // get the name according to the precedence rules
	    if ( this.theNameElement.equals("") )
	    {
	        this.theName = this.theNameAttribute;
	    }
	    else
	    {
	        this.theName = this.theNameElement;
	    }
	    // check the name to make sure it is not null:
	    if ( this.theName.equals("") )
	    {
	        // error, name cannot be null!
	        this.bumpErrorCount(1);
	        this.adjustErrorLevel(Constants.ERROR);
	        
            Object[] parms = { this.element, "Name " + Messages.emptyContent }; 
            LogMsg logmsg = new LogMsg( Messages.elementInvalid , parms );
            logmsg.prolog( "common.policies.configure", 300000000, "error" );
            logmsg.log();
	    }
	    // get the parameter value according to the precedence rules
	    if ( this.theValueElement.equals("") )
	    {
	        this.theValue = this.theValueAttribute;
	    }
	    else
	    {
	        this.theValue = this.theValueElement;
	    }
	    
	    if ( this.theValue.equals("") )
	    {
	        // error, name cannot be null!
	        this.bumpErrorCount(1);
	        this.adjustErrorLevel(Constants.ERROR);
	        
            Object[] parms = { this.element, "Value " + Messages.emptyContent }; 
            LogMsg logmsg = new LogMsg( Messages.elementInvalid , parms );
            logmsg.prolog( "common.policies.configure", 300000000, "error" );
            logmsg.log();
	    }
	    // setup the result object array
	    Object[] result = 
	    {   
	        new Integer(this.getErrorCount()), 
	        new Integer(this.getErrorLevel()),
	        theName,
	        theValue,
	        theSummary,
	        theDescription
	    };
	    // next-to-last - call the result 
	    disassembler.result( name, result );
	    // last - unregister all my content handlers.
	    this.getContent().clearHandlers();
	}
	
    /**
	* Function called for element character data.
	*
	* @param    name    String containing name of element chardata is for.
	* @param	ch		array of character data.
	* @param	start	begining index of character data.
	* @param	length	length of character data.
	*
	* @see		SAXManager
     **/
	public void characters( String name, char ch[], int start, int length) 
	{
	    if ( name.equals(this.element) )
	    {
	        // Content Model of an ice-payload cannot contain characters.
            Object[] parms = { name, Messages.noCharactersInModel };    
            LogMsg logmsg = new LogMsg( Messages.attributeUnknown , parms );
            logmsg.prolog( "common.payloadDisassembly", 200000000, "warning" );
            logmsg.log();
	    }
    }
    
    /**
     * Function called when a sub-handler completes and returns it's work.
	 *
	 * @param   name        String containing the name of an element just completed.
	 * @param   objectArray Array of objects, with are the result of the element just
	 *                      completed.  Presumably, the called handler's result 
	 *                      method knows how to handle the objects in the array,
	 *                      with the name of the completed element being the
	 *                      distinguishing hint.    
	 *
	 * @see		Handler in package XMLdisassembly
	 *
	 **/
	public void result( String name, Object[] objectArray ) 
    {
        int i = content.index(name);
        if ( i >= 0 )
        {
            Integer elementErrorCount = (Integer) objectArray[0];
            Integer elementErrorLevel = (Integer) objectArray[1];
                
            this.bumpErrorCount(elementErrorCount.intValue() );
            this.adjustErrorLevel(elementErrorLevel.intValue() );
        }
        switch ( i )
        {
            // When an element in the content of this element completes, it calls this 
            // routine with it's results.  That is if element foo has element bar as
            // content, then when element bar contained by foo completes, the result
            // is handed back to foo through this routine.
            //            
            // The index here is the index of the elements in contentTags with the
            // first starting at 0.
            case NAME: // name
                this.theNameElement = (String) objectArray[2];
                break;
            case VALUE: // value
                this.theValueElement = (String) objectArray[2];
                break;
            case DESCRIPTION: // description
                this.theDescription += (String) objectArray[2];
                break;
            default: // unknown name
	            // 
                Object[] parms = { name, this.element };    
                LogMsg logmsg = new LogMsg( Messages.UnknownElementReturned , parms );
                logmsg.prolog( "common.policies.configure", 200000000, "warning" );
                logmsg.log();
                // if the message is important enough to want to assure that it is written:
                // logmsg.flush(); 
                break;
        }
    }
}
