/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/policies/configure/PolicyHandler.java,v 1.4 2000/12/02 23:30:47 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.policies.configure;

import java.util.*;
import org.xml.sax.*;

//
// icecubes  imports
//
import com.adobe.rice.IceCubes;
import com.adobe.rice.common.*;
import com.adobe.rice.common.policies.*;
import com.adobe.rice.common.XMLdisassembly.*;
import com.adobe.rice.common.storage.logging.*;     // so we can write to the log.

public class PolicyHandler extends Handler 
{
    //
    // class wide variables
    //
    static final int PACKAGE = 0;
    static final int RANGE = 1;
    static final int ENUM = 2;
    static final int CONSTANT = 3;
    static final int CHOICE = 4;
    static final int METHOD = 5;
    static final int METHODENUM = 6;
    static final int DESCRIPTION = 7;
    /*
    static final int PARMCHOICE = 8;
    */
    
    static final String CONFIGURE = "common.policies.configure";
    
    //
    // private class instance variables
    //
    
    private RangedPolicy rangePolicy = null;
    private ChoicePolicy choicePolicy = null;
    private SimplePolicy constantPolicy = null;
    
    private Object methodPolicy = null;             // $FIX$ -- implement!
    private Object methodEnumPolicy = null;         // $FIX$ -- implement!
    
	//
	// package declaration
	//
	
	private String packageName;
	
	private String packageNameAttribute;
	private String packageNameElement;
	
	//
	// name of the policy
	//
	private String policyName;
	
	//
	// description declaration
	//
	
	private String description = "";
	
	//
	// Constructors
	// 
    
	public PolicyHandler(XMLDisassembler disassembler, Handler parent) 
	{		
		super(disassembler, parent);
		
		// PolicyHandler only handels policy elements.
		this.element = "policy";
	
		ParserAttributes al = new ParserAttributes();
	
		al.add( new AttributeEntry("id", "ID" ) );
		al.add( new AttributeEntry("name", "NMTOKEN" ) );
		al.add( new AttributeEntry("package", "CDATA" ) );
		al.add( new AttributeEntry("override", "NMTOKENS" ) );
		
		this.elementAttributes = al;
	}
	
	public void startElement(String namespaceURI, String localName, 
                             String name, ParserAttributes attributes) 
	{
        TagEntry contentTags[] = null; 
	    
		if (name.equals(this.element)) 
		{
		    // run through the attributes and get their values
		    this.setupAttributes(attributes);
		    
		    this.packageNameAttribute = this.getAttributeValue("package");
		    this.policyName = this.getAttributeValue("name");
		    
		}
		// make sure content handlers are registered.
		// do this as late as possible
		
		contentTags = new TagEntry[] 
		{
		    // create a TagEntry for each element in ModelContent's content model here.
		    // Note that the order of the TagEntrys corresponds to the indexes used
		    // below -- if TAGNAME is the tag name in the first TagEntry, then it is
		    // index 0 below in result.
		    //
		    // replace the UnimplementedHandler in each TagEntry with an appropriate handler when you implement it.
            new TagEntry("package", new PackageHandler( disassembler, this)),
            new TagEntry("range", new RangeHandler( disassembler, this)),
            new TagEntry("enumeration", new EnumerationHandler( disassembler, this)),
            new TagEntry("constant", new ConstantHandler( disassembler, this)),
            new TagEntry("choice", new EnumerationHandler( disassembler, this)),
            new TagEntry("method", new MethodHandler( disassembler, this)),
            new TagEntry("methods", new MethodEnumerationHandler( disassembler, this)),
            new TagEntry("description", new DescriptionHandler(disassembler, this))
		};
		this.setContent( new Content(contentTags, disassembler) );
		//
		// Begin Semantic Processing Here:
		//
	}
	
	public void endElement(String namespaceURI, String localName, String name )
	{
	    //
	    // Complete Semantic Processing Here:
	    //
	    PolicyManager pm = PolicyManager.getPolicyManager();
	    Policy policyObject = null;
	    
	    // 
	    // make sure that there is a policy name and package!
	    //
        if ( (this.policyName == null) || ( this.policyName.equals("") ) )
        {
	        // Policy constructed is in error, it should be a SimplePolicy
            Object[] parms = { name+" "+this.policyName , Messages.elementMustHaveName };    
            LogMsg logmsg = new LogMsg( Messages.elementInvalid, parms );
            logmsg.prolog( "common.policies.configure", 300000000, "Error" );
            logmsg.log();
            //    
            this.bumpErrorCount(1);
            this.adjustErrorLevel(Constants.ERROR);
            // this is an error because icecubes will not be able to access the policy.
            this.policyName = "_MISSING_POLICY_NAME_";
        }
        
        if ( this.packageNameElement == null )
            this.packageName = this.packageNameAttribute;
        else
            this.packageName = this.packageNameElement;
            
        if ( this.packageName == null )
        {
	        // Policy constructed is in error, it should be a SimplePolicy
            Object[] parms = { name+" "+this.policyName, this.policyName };    
            LogMsg logmsg = new LogMsg( Messages.policyPackageMissing, parms );
            logmsg.prolog( "common.policies.configure", 200000000, "warning" );
            logmsg.log();
            //    
            this.bumpErrorCount(1);
            this.adjustErrorLevel(Constants.WARNING);
            // this is a warning because the reasonable default is all packages...
            this.packageName = Constants.ALL_PACKAGES;
        }
	    // check for constant policy
	    if ( this.constantPolicy != null )
	    {
	        if ( this.constantPolicy.getClass().getName().equals( SimplePolicy.class.getName() ) )
	        {
	            // SimplePolicy sp = (SimplePolicy) this.constantPolicy ;
	            // pm.setPolicy(sp.getPackageName(), sp.getName(), sp );
	            policyObject = (Policy) this.constantPolicy;
	        }
	        else
	        {
	            // Policy constructed is in error, it should be a SimplePolicy
                Object[] parms = { name+" "+this.policyName, constantPolicy.getClass().getName(), "SimplePolicy" };    
                LogMsg logmsg = new LogMsg( Messages.policyClassError, parms );
                logmsg.prolog( "common.policies.configure", 400000000, "Severe" );
                logmsg.log();
                //    
                this.bumpErrorCount(1);
                this.adjustErrorLevel(Constants.SEVERE);
	        }
	    }
	    // check for range policy
	    if ( this.rangePolicy != null )
	    {
	        if ( ( RangedPolicy.class.getName().equals( this.rangePolicy.getClass().getSuperclass().getName() ) ) ||
	             ( RangedPolicy.class.getName().equals( this.rangePolicy.getClass().getName() ) ) 
	           )
	        {
	            // RangedPolicy rp = (RangedPolicy) this.rangePolicy ;
	            // pm.setPolicy( this.packageName, rp.getName(), rp );
	            policyObject = (Policy) this.rangePolicy;
	        }
	        else
	        {
	            // Policy constructed is in error, it should be a RangedPolicy
                Object[] parms = 
                { 
                    name+" "+this.policyName, 
                    rangePolicy.getClass().getSuperclass().getName(), 
                    RangedPolicy.class.getName() 
                };    
                LogMsg logmsg = new LogMsg( Messages.policyClassError, parms );
                logmsg.prolog( "common.policies.configure", 400000000, "Severe" );
                logmsg.log();
                //    
                this.bumpErrorCount(1);
                this.adjustErrorLevel(Constants.SEVERE);
	        }
	    }
	    // check for enum or choice policy
	    if ( this.choicePolicy != null )
	    {
	        if ( this.choicePolicy.getClass().getName().equals( ChoicePolicy.class.getName() ) )
	        {
	            // ChoicePolicy cp = (ChoicePolicy) this.choicePolicy ;
	            // pm.setPolicy(cp.getPackageName(), cp.getName(), cp );
	            policyObject = (Policy) this.choicePolicy;
	        }
	        else
	        {
	            // Policy constructed is in error, it should be a ChoicePolicy
                Object[] parms = { name+" "+this.policyName, 
                                   choicePolicy.getClass().getName(), 
                                   ChoicePolicy.class.getName() 
                                 };    
                LogMsg logmsg = new LogMsg( Messages.policyClassError, parms );
                logmsg.prolog( "common.policies.configure", 400000000, "Severe" );
                logmsg.log();
                //    
                this.bumpErrorCount(1);
                this.adjustErrorLevel(Constants.SEVERE);
	        }
	    }
	    if ( policyObject == null )
	    {
	        // A lower level error prevented the policy from being set.
            Object[] parms = { name+" "+this.policyName };    
            LogMsg logmsg = new LogMsg( Messages.policyNotSet, parms );
            logmsg.prolog( "common.policies.configure", 400000000, "Severe" );
            logmsg.log();
            //    
            this.bumpErrorCount(1);
            this.adjustErrorLevel(Constants.SEVERE);
                
	    }
        // static final int METHOD = 5;
        // static final int METHODENUM = 6;
        //
        // now set the name and package:
        policyObject.setName(this.policyName);
        policyObject.setPackageName(this.packageName);
        //
        pm.setPolicy(policyObject.getPackageName(), policyObject.getName(), policyObject);
	    //
	    // call result here after completing semantic processing
	    //
	    Object[] results = 
	    {
	        new Integer(this.getErrorCount()),
	        new Integer(this.getErrorLevel()),
	        policyObject
	    };
	    //
	    // now clear out the policies so we don't do them twice
	    //
	    this.rangePolicy = null;
	    this.choicePolicy = null;
	    this.constantPolicy = null;
	    //
	    // next-to-last - call the result 
	    disassembler.result( name, results );
	    // last - unregister all my content handlers.
	    this.getContent().clearHandlers();
	}
	
    /**
	 * Function called for element character data.
	 *
	 * @param    name    String containing name of element chardata is for.
	 * @param	ch		array of character data.
	 * @param	start	begining index of character data.
	 * @param	length	length of character data.
	 *
	 * @see		SAXManager
     **/
	public void characters( String name, char ch[], int start, int length) 
	{
	    // Content Model of a Policy cannot contain characters.
        Object[] parms = { name+" "+this.policyName, Messages.noCharactersInModel };    
        LogMsg logmsg = new LogMsg( Messages.elementInvalid , parms );
        logmsg.prolog( "common.policies.configure", 200000000, "warning" );
        logmsg.log();
        //
        this.bumpErrorCount(1);
        this.adjustErrorLevel(Constants.WARNING);
    }
    
    /**
     * Function called when a sub-handler completes and returns it's work.
	 *
	 * @param   name        String containing the name of an element just completed.
	 * @param   objectArray Array of objects, with are the result of the element just
	 *                      completed.  Presumably, the called handler's result 
	 *                      method knows how to handle the objects in the array,
	 *                      with the name of the completed element being the
	 *                      distinguishing hint.    
	 *
	 * @see		Handler in package XMLdisassembly
	 *
	 **/
	public void result( String name, Object[] objectArray ) 
    {
        int i = content.index(name);
        if ( i >= 0 )
        {
            Integer elementErrorCount = (Integer) objectArray[0];
            Integer elementErrorLevel = (Integer) objectArray[1];
                
            this.bumpErrorCount(elementErrorCount.intValue() );
            this.adjustErrorLevel(elementErrorLevel.intValue() );
        }
        switch ( i )
        {
            // When an element in the content of this element completes, it calls this 
            // routine with it's results.  That is if element foo has element bar as
            // content, then when element bar contained by foo completes, the result
            // is handed back to foo through this routine.
            //
            // Handle the results here.  Note -- The theory is that this handler is
            // little more than a packager of the effort of the lower level routines.
            // Build the lower level routines to handle everything that is locally possible
            // about the element.  The result routine (this one) should be designed to
            // integrate the lower level elements into a larger whole.  At this level,
            // we can do little but schedule the returning response.
            
            // The index here is the index of the elements in contentTags with the
            // first starting at 0.

        case PACKAGE: // package
            this.packageNameElement = (String) objectArray[2];
            break;
        case RANGE: // range
            this.rangePolicy = (RangedPolicy) objectArray[2];
            break;
        case ENUM: // enumeration
            this.choicePolicy = (ChoicePolicy) objectArray[2];
            break;
        case CONSTANT: // constant
            this.constantPolicy = (SimplePolicy) objectArray[2];
            break;
        case CHOICE: // choice
            this.choicePolicy = (ChoicePolicy) objectArray[2];
            break;
        case METHOD: // method
            this.methodPolicy = objectArray[2];  // $FIX$ implement
            break;
        case METHODENUM: // methods
            this.methodEnumPolicy = objectArray[2]; // $FIX$ implement
            break;
        case DESCRIPTION: // description
            this.description += (String) objectArray[2];
            break;
        default: // unknown name
	        // 
	        this.bumpErrorCount(1);
	        this.adjustErrorLevel(Constants.WARNING);
	        
            Object[] parms = { name+" "+this.policyName, this.element };    
            LogMsg logmsg = new LogMsg( Messages.UnknownElementReturned , parms );
            logmsg.prolog( "common.policies.configure", 200000000, "warning" );
            logmsg.log();
            // if the message is important enough to want to assure that it is written:
            // logmsg.flush(); 
            break;
        }
    }
}








