/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/policies/configure/RangeHandler.java,v 1.2 2000/11/22 02:33:47 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.policies.configure;

//
// icecubes  imports
//
import com.adobe.rice.IceCubes;
import com.adobe.rice.common.*;
import com.adobe.rice.common.policies.*;
import com.adobe.rice.common.XMLdisassembly.*;
import com.adobe.rice.common.storage.logging.*;     // so we can write to the log.

public class RangeHandler extends Handler 
{
    //
    // class wide variables
    //
    public static final int NAME        = 0;
    public static final int MIN         = 1;
    public static final int MAX         = 2;
    public static final int DEFAULT     = 3;
    public static final int DESCRIPTION = 4;
    
    //
    // private class instance variables
    //
    
    private String theName;            // name
    private String theNameElement;
    private String theNameAttribute;
    
    private String theId;              // id
    
    private String theDescription;     // description 
    
    private String theType;
    
    private String theMinString;       // string version of MIN
    private Number theMin;
    private Number theMinAttribute;
    private Number theMinElement;
    
    private String theMaxString;       // string version of MAX
    private Number theMax;
    private Number theMaxAttribute;
    private Number theMaxElement;
    
    private String theDefaultString;   // string version of default
    private Number theDefault;
    private Number theDefaultAttribute;
    private Number theDefaultElement;
    
    
	public RangeHandler(XMLDisassembler disassembler, Handler parent) 
	{		
		super(disassembler, parent);
		
		// All the RangeHandler understands is range.
		this.element = "range";
	
		ParserAttributes al = new ParserAttributes();
		
		al.add( new AttributeEntry("id", "ID" ) );
		al.add( new AttributeEntry("name", "NMTOKEN" ) );
		al.add( new AttributeEntry("default", "CDATA" ) );
		al.add( new AttributeEntry("min", "CDATA" ) );
		al.add( new AttributeEntry("max", "CDATA" ) );
		al.add( new AttributeEntry("type", "CDATA", "int" ) );
		
		
		this.elementAttributes = al;
	}
	
	public void startElement(String namespaceURI, String localName, 
                             String name, ParserAttributes attributes) 
	{
        TagEntry contentTags[] = null; 
	    AttributeEntry ae;
	    
		if (name.equals(this.element)) 
		{
		    this.setupAttributes(attributes);
		    
		    theType = this.getAttributeValue("type");
		    if ( theType != null )
		    {
		    
		        theDefaultString = this.getAttributeValue("default");
		        if ( theDefaultString != null )
    		        theDefaultAttribute = (Number) Constants.convertValue( theType, theDefaultString );
		    
		        theMinString = this.getAttributeValue("min");
		        if ( theMinString != null )
		            theMinAttribute = (Number) Constants.convertValue( theType, theMinString );
		        
		        theMaxString = this.getAttributeValue("max");
		        if ( theMaxString != null )
		            theMaxAttribute = (Number) Constants.convertValue( theType, theMaxString );
		    }
		    
		    theNameAttribute = this.getAttributeValue("name");
		    
		    theId = this.getAttributeValue("id");
		} 
		// make sure content handlers are registered.
		// do this as late as possible
		contentTags = new TagEntry[] 
		{
            new TagEntry("name", new NameHandler(disassembler, this)),
            new TagEntry("min", new MinHandler(disassembler, this)),
            new TagEntry("max", new MaxHandler(disassembler, this)),
            new TagEntry("default", new DefaultHandler(disassembler, this)),
            new TagEntry("description", new DescriptionHandler(disassembler, this))
		};
		this.setContent( new Content(contentTags, disassembler) );
		//
		// Begin Semantic Processing Here:
		//
	}
	
	public void endElement(String namespaceURI, String localName, String name )
	{   
	    RangedPolicy rangedPolicy = null;
	    Object[] result = null;
	    
	    // now process the content to create a "RangedPolicy"
	    if ( name.equals(this.element))
	    {
	        if ( (this.theNameElement == null) || theNameElement.equals("") )
	            theName = theNameAttribute;
	        else
	            theName = theNameElement;
	            
	        if ( theMinElement == null )
	        {
	            if ( theMinAttribute == null )
	                theMin = (Number) Constants.extrema( theType, true /* min */);
	            else
	                theMin = theMinAttribute;
	        }
	        else
	            theMin = theMinElement;
	            
	        if ( theMaxElement == null )
	        {
	            if ( theMaxAttribute == null )
	                theMax = (Number) Constants.extrema( theType, false /* max */ );
	            else
	                theMax = theMaxAttribute;
	        }
	        else
	            theMax = theMaxElement;
	            
	        if ( theDefaultElement == null )
	        {
	            if ( theDefaultAttribute == null )
	                theDefault = null;
	            else
	                theDefault = theDefaultAttribute;
	        }
	        else
	            theDefault = theDefaultElement;
	            
	        // now create a ranged policy for the input:
	        
	        switch( Constants.getTypeIndex( theType ) )
	        {
                case Constants.TYPE_BYTE:
                    rangedPolicy = new RangedBytePolicy(theName, theMin.byteValue(), theMax.byteValue());
                    break;
                case Constants.TYPE_SHORT:
                    rangedPolicy = new RangedShortPolicy(theName, theMin.shortValue(), theMax.shortValue());
                    break;
                case Constants.TYPE_INT:
                    rangedPolicy = new RangedIntegerPolicy(theName, theMin.intValue(), theMax.intValue() );
                    break;
                case Constants.TYPE_LONG:
                    rangedPolicy = new RangedLongPolicy(theName, theMin.longValue(), theMax.longValue() );
                    break;
                case Constants.TYPE_FLOAT:
                    rangedPolicy = new RangedFloatPolicy(theName, theMin.floatValue(), theMax.floatValue() );
                    break;
                case Constants.TYPE_DOUBLE:
                    rangedPolicy = new RangedDoublePolicy(theName, theMin.doubleValue(), theMax.doubleValue() );
                    break;
                default:
                    Object[] parms = { name, theName, theType };    
                    LogMsg logmsg = new LogMsg( Messages.unknownTypeInRange , parms );
                    logmsg.prolog( "common.policies.configure", 200000000, "error" );
                    logmsg.log();
                    break;
	        }
	        if ( (theDefault != null) && (rangedPolicy != null) )
	        {
	            try 
	            {
	                rangedPolicy.select(theDefault);
	            }
	            catch( PolicyException pe )
	            {
                    Object[] parms = { name, theName, pe.getMessage() };    
                    LogMsg logmsg = new LogMsg( Messages.selectionError , parms );
                    logmsg.prolog( "common.policies.configure", 200000000, "error" );
                    logmsg.log();
	            }
	        }
	        else
	        {
                Object[] parms = { name, theName, this.theDefaultString };    
                LogMsg logmsg = new LogMsg( Messages.defaultValueInvalid , parms );
                logmsg.prolog( "common.policies.configure", 200000000, "error" );
                logmsg.log();
                // recover by killing the rangedPolicy
                rangedPolicy = null;
	        }
	        
	        result = new Object[] 
	        { 
	            new Integer( this.getErrorCount() ),
	            new Integer( this.getErrorLevel() ),
	            rangedPolicy, 
	            this.theDescription 
	        };
	        
	    }
	    // next-to-last - call the result 
	    disassembler.result( name, result );
	    // last - unregister all my content handlers.
	    this.getContent().clearHandlers();
	}
	
    /**
	* Function called for element character data.
	*
	* @param    name    String containing name of element chardata is for.
	* @param	ch		array of character data.
	* @param	start	begining index of character data.
	* @param	length	length of character data.
	*
	* @see		SAXManager
     **/
	public void characters( String name, char ch[], int start, int length) 
	{
	    // Content Model of a Range cannot contain characters.
        Object[] parms = { name, Messages.noCharactersInModel };    
        LogMsg logmsg = new LogMsg( Messages.attributeUnknown , parms );
        logmsg.prolog( "common.policies.configure", 200000000, "warning" );
        logmsg.log();
    }
    
    /**
     * Function called when a sub-handler completes and returns it's work.
	 *
	 * @param   name        String containing the name of an element just completed.
	 * @param   objectArray Array of objects, with are the result of the element just
	 *                      completed.  Presumably, the called handler's result 
	 *                      method knows how to handle the objects in the array,
	 *                      with the name of the completed element being the
	 *                      distinguishing hint.    
	 *
	 * @see		Handler in package XMLdisassembly
	 *
	 **/
	public void result( String name, Object[] objectArray ) 
    {
        int i = content.index(name);
        if ( i >= 0 )
        {
            Integer elementErrorCount = (Integer) objectArray[0];
            Integer elementErrorLevel = (Integer) objectArray[1];
                
            this.bumpErrorCount(elementErrorCount.intValue() );
            this.adjustErrorLevel(elementErrorLevel.intValue() );
        }
        switch( i )
        {
        case NAME: // name
            theNameElement = (String) objectArray[2];
            break;
        case MIN: // min
            theMinString = (String) objectArray[2];
            theMinElement = (Number) Constants.convertValue( theType, theMinString );
            break;
        case MAX: // max
            theMaxString = (String) objectArray[2];
            theMaxElement = (Number) Constants.convertValue( theType, theMaxString );
            break;
        case DEFAULT: // default
            theDefaultString = (String) objectArray[2];
            theDefaultElement = (Number) Constants.convertValue( theType, theDefaultString );
            break;
        case DESCRIPTION: // description
            theDescription = (String) objectArray[2];
            break;
        default: // unknown name
	        // 
	        this.bumpErrorCount(1);
	        this.adjustErrorLevel(Constants.WARNING);
	        
            Object[] parms = { name, this.element };    
            LogMsg logmsg = new LogMsg( Messages.UnknownElementReturned , parms );
            logmsg.prolog( "common.policies.configure", 200000000, "warning" );
            logmsg.log();
            // if the message is important enough to want to assure that it is written:
            // logmsg.flush(); 
            break;
        }
        
    }
    //
    // utility functions
    //
    /**
     * Return the type of this range
     *
     * @param void
     * @return String - the name of the type
     **/
    public String getType()
    {
        AttributeEntry ae = this.findAttribute("type");
        if ( ae != null )
            return ae.getValue();
        return "";
    }
    
}








