/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/policies/configure/UuidHandler.java,v 1.3 2000/11/22 02:33:48 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.policies.configure;

import java.util.*;
import org.xml.sax.*;

//
// icecubes  imports
//
import com.adobe.rice.IceCubes;
import com.adobe.rice.util.*;                       // for UUID
import com.adobe.rice.common.*;
import com.adobe.rice.common.policies.*;
import com.adobe.rice.common.XMLdisassembly.*;
import com.adobe.rice.common.storage.logging.*;     // so we can write to the log.

public class UuidHandler extends Handler 
{
    //
    // class wide variables
    //
    //
    // private class instance variables
    //
    private String theUUID = "";
    private String theUUIDattribute = "";
    private String theUUIDpcdata = "";
    
    private UUID actualUUID = new UUID();
    
	public UuidHandler(XMLDisassembler disassembler, Handler parent) 
	{		
		super(disassembler, parent);
		
		// All the UuidHandler understands is the uuid
		this.element = "uuid";
	
		ParserAttributes al = new ParserAttributes();
		
		al.add( new com.adobe.rice.common.XMLdisassembly.AttributeEntry("value", "CDATA" ) );
		
		this.elementAttributes = al;
	}
	
	public void startElement(String namespaceURI, String localName, 
                             String name, ParserAttributes attributes) 
	{
        TagEntry contentTags[] = null; 
	    
	    this.setupAttributes(attributes);
	    
	    theUUIDattribute = this.getAttributeValue("value");
	        
		// make sure content handlers are registered.
		// do this as late as possible
		this.setContent( new Content(contentTags, disassembler));
	}
	
	public void endElement(String namespaceURI, String localName, 
                             String name ) /* throws ConfigException */
	{
	    // select uuid value based on priority
	    if ( theUUIDpcdata.equals("") )
	        theUUID = theUUIDattribute;
	    else
	        theUUID = theUUIDpcdata;
	    // go parse the UUID to make sure it's valid. 
	    this.actualUUID.set(theUUID); 
	        
	    // next-to-last - call the result 
	    Object[] result = 
	        {   new Integer(this.getErrorCount()), 
	            new Integer(this.getErrorLevel()),
	            actualUUID
	        };
	    disassembler.result( name, result );
	    // last - unregister all my content handlers.
	    this.getContent().clearHandlers();
	}
	
    /**
	* Function called for element character data.
	*
	* @param    name    String containing name of element chardata is for.
	* @param	ch		array of character data.
	* @param	start	begining index of character data.
	* @param	length	length of character data.
	*
	* @see		SAXManager
     **/
	public void characters( String name, char ch[], int start, int length) 
	{
	    if ( name.equals("uuid") )
	    {
	        theUUIDattribute += String.copyValueOf(ch, start, length);
	    }
    }
    
    /**
     * Function called when a sub-handler completes and returns it's work.
	 *
	 * @param   name        String containing the name of an element just completed.
	 * @param   objectArray Array of objects, with are the result of the element just
	 *                      completed.  Presumably, the called handler's result 
	 *                      method knows how to handle the objects in the array,
	 *                      with the name of the completed element being the
	 *                      distinguishing hint.    
	 *
	 * @see		Handler in package XMLdisassembly
	 *
	 **/
	public void result( String name, Object[] objectArray ) 
    {
        int i = content.index(name);
        switch ( i )
        {
            default: // unknown name
	            // 
                Object[] parms = { name, this.element };    
                LogMsg logmsg = new LogMsg( Messages.UnknownElementReturned , parms );
                logmsg.prolog( "common.policies.configure", 200000000, "warning" );
                logmsg.log();
                // if the message is important enough to want to assure that it is written:
                // logmsg.flush(); 
                break;
        }
    }
    
}








