/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/scheduling/MasterSchedule.java,v 1.3 2000/11/27 20:19:41 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.scheduling;

import com.adobe.rice.common.*;
import com.adobe.rice.common.policies.*;
//$JDK1.1$ import com.sun.java.util.collections.*;
import com.adobe.rice.util.time.*;
import com.adobe.rice.util.*;
import java.util.*;

/**
 * MasterSchedule maintains ScheduleItems in a storage-optimzed form. This is	
 * achieved because ICE provides for an inherent means of repetition in the 
 * system.
 */
//123456789012345678901234567890123456789012345678901234567890123456789012345678
//Jonathan's anal attempt to keep as much code as possible under the 80 char lmt

public class MasterSchedule	extends Thread implements PolicyObserver
{

    // data structures used
    private Cube future,present;
	
    // local policy copies
    private IceDuration polDispatchWindowSize;
    private IceTime polMaintTime;
	
    // other local members
    private IceDateTime nextMaint;
    private IceDateTime nextDisp;
	
	
    //
    //  Construction
    //
	
    public MasterSchedule() 
    {
        future = new Cube();
        present = new Cube();
        
	    constructPolicies();
	    constructBoundary();
    }

    private void constructBoundary() 
    {
	    // set up initial boundary cases for the run()
	    nextMaint=new IceDateTime();
	    nextDisp=new IceDateTime();
    }
	
    // POLICY STUFF
    private void constructPolicies() 
    {
	    // set up the window for dispatch. this will be obeyed as best as possible.
	    SimplePolicy dws = new SimplePolicy(IceDurationDay.class);
	    PolicyManager pm = (PolicyManager) Tray.getManager("Policy");

	    pm.setPolicy("MasterSchedule-DispatchWindowsSize",dws);
	    dws.addObserver(this);
	    try {
	        dws.select(new IceDurationDay("P3600S")); // will implicity call update
	        this.polDispatchWindowSize = (IceDuration) dws.getSelection();
	    }
	    catch (PolicyException pe) {}
		
	    // set up the time for maintinence.
	    // it might be dumb to change this setting to later in the day, no?
	    SimplePolicy mnt = new SimplePolicy(IceTime.class);
	    pm.setPolicy("MasterSchedule-MaintenanceTime",mnt);
	    mnt.addObserver(this);
	    try {
	        mnt.select(new IceTime("12:00:00")); // will implicity call update
	        this.polMaintTime = (IceTime) mnt.getSelection();
	    }
	    catch (PolicyException pe) {}
    }

    public void update(Observable policy, Object hint) 
    {
	    // $TODO$ include a switch statement to go through and update whatever
	
			
	    // we changed some policy. wake run() to notify
	    this.interrupt();
    }
	
    //
    // Other Methods
    //
	
	
    public void	addSchedulable(Schedulable scheduable) throws IllegalArgumentException 
    {
	    // if it starts later
	    if (scheduable.getScheduledStartdate().getDate().after(IceDate.getDateTomorrow()))
	        future.insertSchedulable(scheduable);
	    // if it stops later
	    else if (! scheduable.getScheduledStopdate().getDate().before(new IceDate()))
	        present.insertSchedulable(scheduable);
	    // then it stopped long ago
	    else 
	        throw new IllegalArgumentException();
    }
    
    public void	removeSchedulable(String id) 
    {
	    future.removeSchedulable(id);
	    present.removeSchedulable(id);
    }
	
    /** Export the next PerishableDispatchSchedule.
     * @param date Export records with this IceDate
     * @param starttime Export records which start greater/equal to this starttime
     * @param endtime Export records which start before this endtime  
     */
    public void exportDispatcher(IceDate date, 
				 IceTime starttime,
				 IceTime endtime) 
	{
	    // TODO -- why don't I have to declare throws IllegalArgumentException?
	    ListIterator contents;
	    PerishableDispatchSchedule dispatch;
	    dispatch = new PerishableDispatchSchedule(new IceDateTime(date,starttime),
						  new IceDateTime(date,endtime));
	    contents = present.getLogicalIterator(date,starttime,endtime);
	    try {
	        while(contents.hasNext())
		        dispatch.add((DispatchScheduleItem) contents.next());
	    }
	    catch (ObjectImmutableException e) {}
	
	    dispatch.start();
    }

	
	
    public void run() 
    {
	    // we can subsequently deal with killing this run loop if needed	
	    while (true)
	    { 
	        IceDuration sleepamt;
	    
	        // perform any Maintenance that is needed
	        synchronized (nextMaint) 
	        {
		        if (! nextMaint.before(new IceDateTime())) 
		        {
		            performMaintenance();
		            // the next Maintenance should be tomorrow at the spec time.
		            nextMaint = new IceDateTime(IceDate.getDateTomorrow(),
						                        polMaintTime);
		        }
	        }

	        // perform any dispatch that is needed
	        synchronized (nextDisp) 
	        {
		        if (! nextDisp.before(new IceDateTime())) 
		        {
		            IceTime windowend= nextDisp.getTime().add(polDispatchWindowSize);
		            exportDispatcher(new IceDate(),
				                     nextDisp.getTime(),
				                     windowend);

		            // next time will be the next reasonable time or start of next day
		            if (windowend == IceTime.getTimeEndOfDay()) 
			            nextDisp = new IceDateTime(nextDisp.getDate().nextDay(),
						                           IceTime.getTimeStartOfDay());
		            else 
			            nextDisp = new IceDateTime(nextDisp.getDate(),windowend);	
		        }
	        }

			
	        // TODO think more if this will ever result in deadlock...
	        synchronized (nextDisp) 
	        {
		        synchronized (nextMaint) 
		        {
		    
		        // determine how long to sleep!
		    
		            if (nextDisp.before(nextMaint)) 
			            sleepamt = nextDisp.subtract(new IceDateTime());
		            else
			            sleepamt = nextMaint.subtract(new IceDateTime());
		        }
	        }
			
	        // sleep until we have to do something
	        try 
	        {
		        this.sleep(sleepamt.getTotal());
	        }
	        catch (InterruptedException ie) 
	        {
		        // Nada. update() was probably called.
	        }
	    }
    }
	
	
	
	
    /** Move those schedules with a startdate &lt;tomorrow from future to 
     * present and delete those schedules in present with an endday<yesterday, 
     * too.
     * 
     * Note that for efficiency reasons this will not clean up everything, but 
     * they will be cleaned up eventually! And besides, we ensure everything that 
     * we are considering for dispatch remains efficient.
     */
    private void performMaintenance() 
    {
	    Iterator expired, usable;
		
	    expired = present.getExpiredIterator(IceDate.getDateYesterday(), false);
	    usable = future.getExpiredIterator(IceDate.getDateTomorrow(), true);
		
	    // unschedule those Schedulables which have expired
	    while (expired.hasNext()) 
	    {
	        Schedulable cur = (Schedulable) expired.next();
	        present.removeSchedulable(cur.getScheduledId());
	    }
	    // move those schedulables that are now relevent to the active Cube
	    while (usable.hasNext()) 
	    {
	        Schedulable cur = (Schedulable) usable.next();
	        future.removeSchedulable(cur.getScheduledId());
	        present.insertSchedulable(cur);
	    }													  
    }
	
	
    private class Cube	 
    {
	    List    	dayColumns[][];
	    List        carryover;  // for interpreted iterators
	    Map         map;
	    
	    public Cube()	  
	    {
	        /* The grid at the base of the cube consists of nine days: ["Any", 
	        Monday, ..., Sunday]. Let these be denoted [0,7]. There are also 
	        33 dates: ["Any", 1, 2, 3, ..., 28,29, 30, 31,"Last"]. Let those 
	        be denoted [0,32].
	        */
	        map = Collections.synchronizedMap(new HashMap());
	        dayColumns = new List[8][33];
	        carryover = Collections.synchronizedList(new LinkedList());
	        for (int i=0;i<8;i++)
	        {
		        for (int j=0;j<33;j++)
		        {
		            List ls = new LinkedList();
		            dayColumns[i][j] = Collections.synchronizedList(ls);
		        }
		    }
	    }

	    public void	insertSchedulable(Schedulable schedulable)	 
	    {
	        List chain = new ArrayList();
	        boolean newschedulable = true;
	        // check if this is a collision, and just append
	        synchronized(map) 
	        {
		        if (map.containsKey(schedulable.getScheduledId()))
		            chain=(List)map.get(schedulable.getScheduledId());
		        // but don't append if this schedulable already exists
		        Iterator li = chain.iterator();
		        while (li.hasNext()) 
		            if (li.next().equals(schedulable))
			            newschedulable=false;
	        }

	        // now do the real work!
	        ListIterator days =	(ListIterator) schedulable.getDays().getListIterator();
	        IceDay day;
	        if (newschedulable) 
	        {
		        chain.add(schedulable);
		        map.put(schedulable.getScheduledId(),chain);
		        // for each column needed, insert into the sorted list
		        while (days.hasNext()) 
		        {
		            day =(IceDay) days.next();
		            insertRecord(day.getWeekday(),day.getMonthday(),schedulable);
		        }
	        }
	    }
	    
	    public void	removeSchedulable(String id)	 
	    {
	        boolean containskey =false;
	        List chain = null;
	        
	        synchronized(map) 
	        {
		        if (map.containsKey(id))
		        {
		            containskey=true;
		            chain = (List) map.get(id);
		            map.remove(id);
		        }
	        }
	        if (containskey)
	        {
		        Iterator ci = chain.iterator();
		        while (ci.hasNext()) 
		        {
		            Schedulable schedulable = (Schedulable) ci.next();
		            IceDay day;
		            ListIterator days =	(ListIterator) schedulable.getDays().getListIterator();
		            // for each column needed, remove from the sorted list
		            while (days.hasNext()) 
		            {
			            day =(IceDay) days.next();
			            removeRecords(day.getWeekday(),day.getMonthday(),schedulable);
		            }
		        }
	        }
	    }
	    
	    /**
	     *Provides an iterator to many Schedulables for a window of logical 
	     *time [start, end) today or tomorrow. This is what should be used 
	     *to generate DispatchScheduleItems.
	     **/
	    public ListIterator getLogicalIterator(IceDate date,
					                           IceTime starttime,
					                           IceTime endtime) 
			   throws IllegalArgumentException 
	    {
	        if ((!date.equals(new IceDate())) && (!date.equals(IceDate.getDateTomorrow())))
		        throw new IllegalArgumentException();
	        if (!starttime.before(endtime))			
		        throw new IllegalArgumentException();
	    
	        IceDateTime now = new IceDateTime();
	        IceDateTime resultStart, resultEnd;
	        resultStart=new IceDateTime(date,starttime);
	        resultEnd=new IceDateTime(date,endtime);
	    
	        List results = new ArrayList();
			
	        // take all relevent DispatchScheduleItems in the "carryover" bin
	        // and schedule them in this resultslist
	        synchronized(carryover) 
	        {
		        ListIterator carryIt = (ListIterator)carryover.iterator();
		        while (carryIt.hasNext()) 
		        {
		            DispatchScheduleItem cur = (DispatchScheduleItem) carryIt.next();
		            // do some cleanup now
		            if (cur.getStarttime().before(now)) 
		            {
			            carryIt.remove();
			            continue;
		            }
		            if (! cur.getStarttime().before(resultStart)
			            &&
			            cur.getStarttime().before(resultEnd)
			           )
			            results.add(cur);
		        }
	        }
	        // now go through the real day columns and make dispatchscheduleitems
	        // and either return them now or dump them into the carryover bin.
	        ListIterator colIt = (ListIterator) (new IceDayGroup(date)).getListIterator();
	        while (colIt.hasNext()) 
	        {
		        IceDay day = (IceDay) colIt.next();
		
		        List daycol = dayColumns[day.getWeekday()][day.getMonthday()];
		        synchronized(daycol) 
		        {
		            ListIterator dayIt =  (ListIterator)daycol.iterator();
		            while (dayIt.hasNext()) 
		            {
			            Schedulable sch = (Schedulable)dayIt.next();	
			            if (! sch.getScheduledStarttime().before(resultStart.getTime())
			                &&
			                sch.getScheduledStarttime().before(resultEnd.getTime())
			               ) 
                        {
			                // get all those "non-carryover" and add them.
			                ListIterator schIt = seperateSchedulables(resultStart,
								      resultEnd,
								      sch);
			                while (schIt.hasNext())
				                insDSI(results,(DispatchScheduleItem)schIt.next());
			            }
		            }
		        }
	        }
	        return (ListIterator) results.listIterator();
	    }
		
	   /** Gives you an iterator to Scheduables which can be used for subsequent 
	    * reorginization. Be sure to note efficiency caveat below.
	    * 
	    * Example 1:
	    * If we want to locate records which have a startdate of before Tuesday 
	    * 3Aug99, call with cutoff equal to the cutoff=3Aug99 and isStartdate=true. 
	    * This will return an iterator to all Scheduables which begin on Tuesday 
	    * 3Aug99.
	    * 
	    * Example 2:
	    * If we want to then locate all records which have a stopdate before Sunday 
	    * 1Aug99,  call with cutoff=1Aug99 and isStartdate=false.
	    * 
	    * For efficiency reasons, we will only consider those records occuring ON 
	    * the cutoff. So consider a schedule containing the following Scheduables 
	    * which are supposed to run on Tuesdays:
	    * "AAA" (startdate=2Aug99), "BBB" (startdate=3Aug99), "CCC" (startdate=5Aug99).
	    * And also these which are supposed to run on Mondays:
	    * "XXX" (statdate=2Aug99), "YYY" (startdate=3Aug99), "ZZZ" (startdate=5Aug99).
	    * If we called with cutoff=3Aug99 and isStartdate=true, AAA and BBB would 
	    * be returned. CCC, with startdate>3Aug99 would not. XXX, YYY, and ZZZ would 
	    * not be returned with this call. (But an subsequent call to 9Aug99 would 
	    * return XXX, YYY, ZZZ, but NOT CCC.)
	    * 
	    * @author Jonathan Herbach
	    * @param cutoff the (inclusive) date to be considered
	    * @param isStartdate whether we should consider the cutoff as the startdate 
	    * or the stopdate.
	    */
	    public Iterator getExpiredIterator(IceDate cutoff, boolean isStartdate )
	    {
	        List results = new ArrayList();
	        IceDayGroup days = new IceDayGroup(cutoff);
	        ListIterator daysIt = (ListIterator) days.getListIterator();
	        while (daysIt.hasNext()) 
	        {
		        IceDay day = (IceDay) daysIt.next();
		        List column = dayColumns[day.getWeekday()][day.getMonthday()];
		        synchronized(column) 
		        {
		            ListIterator colIt = (ListIterator) column.iterator();	
		            while (colIt.hasNext()) 
		            {
			            Schedulable cur = (Schedulable) colIt.next();
			            // if it is before/equal to the startdate or
			            // before/equal to the stopdate then it's wanted
			            if (
			                (! cur.getScheduledStartdate().getDate().after(cutoff)
			                    && isStartdate )
			                ||
			                (! cur.getScheduledStopdate().getDate().after(cutoff)
			                    && !isStartdate )
			            )
			                results.add(cur);
		            }
		        }
	        }
	        return results.iterator();
	    }
		
	    private void insertRecord(int weekday, int monthday, Schedulable schedulable) 
	    {
	        boolean added = false;
	        synchronized(dayColumns[weekday][monthday]) 
	        {
		        ListIterator column = (ListIterator) dayColumns[weekday][monthday].iterator();
		        while (column.hasNext()) 
		        {
		            Schedulable curval = (Schedulable) column.next();
		            if (curval.getScheduledStarttime().before(schedulable.getScheduledStarttime())) 
		            {
			            dayColumns[weekday][monthday].add(column.previousIndex(),schedulable);
			            added=true;
			            break;
                    }
		        }
		        if (!added)
		            dayColumns[weekday][monthday].add(schedulable);
	        }
	    }
	
	    private void removeRecords(int weekday, int monthday, Schedulable schedulable) 
	    {
	        synchronized(dayColumns[weekday][monthday]) 
	        {
		        ListIterator column = (ListIterator) dayColumns[weekday][monthday].iterator();
		        while (column.hasNext()) 
		        {
		            Schedulable curval = (Schedulable) column.next();
		            if (curval.equals(schedulable))
			            column.remove();
		        }
	        }
	    }
		
		
	    private void insDSI(List destination, DispatchScheduleItem item) 
	    {
	        synchronized (destination) 
	        {
		        boolean alreadyexists=false;
		        ListIterator it = (ListIterator) destination.iterator();				
		        while (it.hasNext())
		        {
		            DispatchScheduleItem cur = (DispatchScheduleItem)it.next();
		            if (cur.equals(item)) 
		            {
			            alreadyexists= true; // do not add
			            break;
		            }
		            if (item.getStarttime().before(cur.getStarttime())) 
			            break; // and add
		        }
		        if (!alreadyexists)
		            it.add(item);
	        }
	    }
		
	    private	ListIterator seperateSchedulables(IceDateTime resultStart,
						  IceDateTime resultEnd,
						  Schedulable schedulable) 
		{
	        List returnable = new ArrayList();
	        ListIterator schIt = schedulable.getSchedulables(resultStart.getDate());
	        // for all scheduables
	        while (schIt.hasNext()) 
	        {
		        DispatchScheduleItem cur =(DispatchScheduleItem) schIt.next();	
		        // put things outside the DispatchWindow into carryover
		        if (! cur.getStarttime().before(resultEnd)) 
		        {
		            insDSI(carryover,cur);
		        }
		        else // it will go into the return pile
		            returnable.add(cur);
	        }
	        return (ListIterator) returnable.iterator();
	    }
    }
}





