/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/scheduling/ScheduleStats.java,v 1.2 2000/11/22 21:16:53 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.scheduling;

import java.util.*;
//$JDK1.1$ import com.sun.java.util.collections.*;
import com.adobe.rice.util.time.*;
import com.adobe.rice.util.*;


public class ScheduleStats { 
    private List[][] dayColumns;
    private MasterSlaveSemiphore dayColumnsFlag;
    private int arraysize;
	
    public static class StatType {
	static final byte MINVAL = -1;
	public static final byte TOTAL =0;
	public static final byte CPU = 1;
	public static final byte MEMORY = 2;
	public static final byte BANDWITH = 3; //TODO  is this http? smtp???
	static final byte MAXVAL = 4;
    }

    // yeah, this is for the inner Stat class
    private static final double NULLSTATVALUE =-100;
    
    private class Stat {
	private double[] values;
	private Stat() {
	    int total = StatType.MAXVAL-StatType.MINVAL+1;
	    
	    values=new double[total];
	    for (int i=0;i<total;i++)
		values[i]= NULLSTATVALUE; 
	}
    }
	
    public void construct() {
	// TODO set up policy information for the granularity of stats
	arraysize=86400;
		
	constructStorage();
	dayColumnsFlag = new MasterSlaveSemiphore();
    }
	
    private void constructStorage() {
	/* The grid at the base of the cube consists of nine days: ["Any", 
	   Monday, ..., Sunday]. Let these be denoted [0,7]. There are also 
	   33 dates: ["Any", 1, 2, 3, ..., 28,29, 30, 31,"Last"]. Let those 
	   be denoted [0,32].
	   
	   Each base will contain a list of all the possible times in the day.
	   Each slot will contain a Stat.
		
	   Note that these don't need to be synchronized lists -- 
	   we only will be changing the Stat objects contained.
	*/
	dayColumnsFlag.startMaster();
		
	dayColumns = new ArrayList[8][33];
	for (int i=0;i<8;i++)
	    for (int j=0;j<33;j++){
		dayColumns[i][j]=new ArrayList();
		((ArrayList)dayColumns[i][j]).ensureCapacity(arraysize);
		for (int k=0;k<arraysize;k++)
		    ((ArrayList)dayColumns[i][j]).set(k, new Stat());
	    }
	dayColumnsFlag.stopMaster();
    }
	
    public void changeGranularity() {
	// TODO low priority: change the capacity of the arrays, and keep old data.
	
	/* possible approaches.
	   increasing granularity, just create more slots with expanding.
	   decreasing, just average or some arbitrary pick? bezier viz. bruce?
	*/
		
	// this will currently throw out old data; it's much easier. 8)
	constructStorage();
    }
	
    public double getStat(byte stat, IceDay day,
			  IceTime start, IceTime stop)
	throws NoStatisticException{
	dayColumnsFlag.startSlave();
	int avgcount=0;
	double avgvalue =0;
		
	int rangeStart, rangeStop;
	rangeStart = start.getRatio(arraysize);
	rangeStop = stop.getRatio(arraysize);
	ArrayList thiscol = (ArrayList) 
	    dayColumns[day.getWeekday()][day.getMonthday()];
	
	for (int i=rangeStart;i<=rangeStop;i++) {
	    Stat thisstat=(ScheduleStats.Stat) thiscol.get(i);
	    synchronized(thisstat){
		if (thisstat.values[stat] != NULLSTATVALUE) {
		    avgvalue+=thisstat.values[stat];
		    avgcount++;
		}
	    }
	}
	dayColumnsFlag.stopSlave();
	if (avgcount==0)
	    throw new NoStatisticException();

	avgvalue/=avgcount;
	return avgvalue;
    }
	
    public void setStat(byte stat, double newvalue) throws IllegalArgumentException{
	//assert
	if (stat >=StatType.MAXVAL ||
	    stat <=StatType.MINVAL ||
	    newvalue <0 ||
	    newvalue >1)
	    throw new IllegalArgumentException();
	dayColumnsFlag.startSlave();

	IceDateTime now = new IceDateTime();		
	IceDayGroup days = new IceDayGroup(now.getDate());
	Iterator dayIt= days.getListIterator();
	while (dayIt.hasNext()) {
	    IceDay thisday = (IceDay) dayIt.next();
	    
	    if (! (thisday.getMonthday() == IceDay.Monthday.ANY)
		&&
		! (thisday.getWeekday() == IceDay.Weekday.ANY)
		) {
		
		ArrayList thiscol=(ArrayList) 
		    dayColumns[thisday.getWeekday()][thisday.getMonthday()];
		Stat thisstat=(ScheduleStats.Stat)
		    thiscol.get(now.getTime().getRatio(arraysize));
		// the synchronization should happen on the stat @ the time we want.
		synchronized(thisstat) {
		    thisstat.values[stat]= setStatValue(stat,
							thisstat.values[stat],
							newvalue);
		}
	    }
	}
	// this will take care of the ANY things that we skipped
	fixAny(stat,now);
	dayColumnsFlag.stopSlave();
    }
	
    private double setStatValue(byte stat, double oldvalue, double newvalue) {
		
	//TODO figure out policy on computing new values, please.
	return 5;	
    }
    private void fixAny(byte stat, IceDateTime now) {
	// no dayColumnsFlag because this is a subroutine of the setStat atom
	IceDayGroup days = new IceDayGroup(now.getDate());
	Iterator dayIt= days.getListIterator();
	while (dayIt.hasNext()) {
	    IceDay thisday = (IceDay) dayIt.next();
	    int avgcount=0; // for counting how many we average...

	    if (thisday.getMonthday() == IceDay.Monthday.ANY
		&& thisday.getWeekday() == IceDay.Weekday.ANY) {
		double avgvalue=0;
		int i,j;
		// iterate through (all columns-ANY) and grab the values so
		// we can come back and create an average of them for [ANY,ANY].
		for (i=1;i<8;i++) {
		    for (j=1;j<33;j++) {
			ArrayList thiscol = (ArrayList) 
			    dayColumns[i][j];
			Stat thisstat=(ScheduleStats.Stat)
			    thiscol.get(now.getTime().getRatio(arraysize));
			// the synchronization should happen on the stat @ the time we want.
			synchronized(thisstat) {
			    if (thisstat.values[stat] != NULLSTATVALUE) {
				avgvalue+=thisstat.values[stat];
				avgcount++;
			    }
			}
		    }
		}
		if (avgcount==0)
		    avgvalue=NULLSTATVALUE;
		else 
		    avgvalue/=avgcount;
		// now set the [ANY][ANY] column stat
		ArrayList finalcol= (ArrayList) dayColumns[0][0];
		Stat finalstat = (ScheduleStats.Stat)
		    finalcol.get(now.getTime().getRatio(arraysize));
		synchronized(finalstat) {
		    finalstat.values[stat]=avgvalue;
		}
	    }	
	    else if (thisday.getMonthday() == IceDay.Monthday.ANY) {
		double avgvalue=0;
		int i, j;
		/* iterate through all columns with this Weekday and grab the 
		   values so we can come back and create an avg of them for any.
		*/
		i = thisday.getWeekday();
		for (j=1;j<33;j++) {
		    ArrayList thiscol = (ArrayList) 
			dayColumns[i][j];
		    Stat thisstat=(ScheduleStats.Stat)
			thiscol.get(now.getTime().getRatio(arraysize));
		    // the synchronization should happen on the stat @ the time we want.
		    synchronized(thisstat) {
			if (thisstat.values[stat] != NULLSTATVALUE) {
			    avgvalue+=thisstat.values[stat];
			    avgcount++;
			}
		    }
		}
		if (avgcount==0)
		    avgvalue=NULLSTATVALUE;
		else 
		    avgvalue/=avgcount;
		// now set the [Weekday][ANY] column stat
		ArrayList finalcol= (ArrayList) 
		    dayColumns[thisday.getWeekday()][0];
		Stat finalstat = (ScheduleStats.Stat)
		    finalcol.get(now.getTime().getRatio(arraysize));
		synchronized(finalstat) {
		    finalstat.values[stat]=avgvalue;
		}
	    }	
	    else if (thisday.getWeekday() == IceDay.Weekday.ANY) {
		double avgvalue=0;
		int i, j;
		/* iterate through all columns with this Weekday and grab the 
		   values so we can come back and create an avg of them for any.
		*/
		j = thisday.getMonthday();
		for (i=1;j<8;j++) {
		    ArrayList thiscol = (ArrayList) 
			dayColumns[i][j];
		    Stat thisstat=(ScheduleStats.Stat)
			thiscol.get(now.getTime().getRatio(arraysize));
		    // the synchronization should happen on the stat @ the time we want.
		    synchronized(thisstat) {
			if (thisstat.values[stat] != NULLSTATVALUE) {
			    avgvalue+=thisstat.values[stat];
			    avgcount++;
			}
		    }
		}
		if (avgcount==0)
		    avgvalue=NULLSTATVALUE;
		else 
		    avgvalue/=avgcount;
		// now set the [ANY][Monthday] column stat
		ArrayList finalcol= (ArrayList) 
		    dayColumns[0][thisday.getMonthday()];
		Stat finalstat = (ScheduleStats.Stat)
		    finalcol.get(now.getTime().getRatio(arraysize));
		synchronized(finalstat) {
		    finalstat.values[stat]=avgvalue;
		}
	    }
	}
    }
}
