/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/storage/StorageManager.java,v 1.5 2000/11/22 21:17:20 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.storage;

//
// icecubes imports
//
import com.adobe.rice.util.*;

import com.adobe.rice.kernel.*;

import com.adobe.rice.common.*;
import com.adobe.rice.common.storage.*;
import com.adobe.rice.common.scheduling.*;
import com.adobe.rice.common.catalog.*;
import com.adobe.rice.common.policies.*;
import com.adobe.rice.common.storage.logging.*;

//
// java imports
//
import java.io.*;

//

public class StorageManager extends IceCubeManager 
{
    private static StorageManager storageManager;
    
    private DescriptionStore      descriptionStore;
    private SubscriptionStore     subscriptionStore;
    private CatalogStore          catalogStore;
    private ScheduleStore         scheduleStore;
    private CollectionStore       collectionStore;
    private LogStore              logStore;
    private DebugLogStore         debugLogStore;

    // constructors -- implement the singleton pattern

    private StorageManager() throws FileNotFoundException, IOException, DispatcherException
    {
        super("StorageManager");
        
	    debugLogStore = new DebugLogStore();
	    try { 
	        logStore = new StoreLog();
	    } 
	    catch ( FileNotFoundException fnf ) { throw fnf; }
	    catch ( IOException ioe ) { throw ioe; };
	    
    }
	
    // methods

    public static StorageManager instance() 
    { 
        if ( storageManager == null )
        {
            try 
            {
                storageManager = new StorageManager();
            }
            catch ( FileNotFoundException fnfe ) {}
            catch ( IOException ioe ) {}
            catch ( DispatcherException de ) {}
        }
        return StorageManager.storageManager;
    }
    
    public void setStorageManager(StorageManager storageManager) {
	    StorageManager.storageManager = storageManager;
    }
    
    
    public static LogStore getLogStore() { return storageManager.logStore; }
	
	
    //storage functions
	
    public void storeDescription(IceDescription description) {
	    descriptionStore.storeDescription(description);
    }
	
    public IceDescription retrieveDescription(UUID iceID) {
	    return descriptionStore.retrieveDescription(iceID);
    }
	
    public void storeSubscription(Subscription subscription) {
	    subscriptionStore.storeSubscription(subscription);
    }
	
    public Subscription retrieveSubscription(String subscriptionID, UUID iceID) {
	return subscriptionStore.retrieveSubscription(subscriptionID, iceID);
    }
    
    public void storeCatalog(Catalog catalog) {
	catalogStore.storeCatalog(catalog);
    }
	
    public Catalog retrieveCatalog(String catalogID) {
	return catalogStore.retrieveCatalog(catalogID);
    }
	
    public void storeSchedule(MasterSchedule schedule) {
	scheduleStore.storeSchedule(schedule);
    }
	
    public MasterSchedule retrieveSchedule() {
	return scheduleStore.retrieveSchedule();
    }
	
    public void storeItem(String collectionID, String subscriptionElement) {
	collectionStore.storeItem(collectionID, subscriptionElement);
    }
	
    public Package retrievePackage(String collectionID, String currentState) {
	    return collectionStore.retrievePackage(collectionID, currentState);
    }
	
	
    public void appendLog(LogEntry entry) throws IOException {
	    logStore.appendLogEntry( entry );
    }
	
    public void appendDebugLog(LogEntry entry) {
	
    }
    //registration functions
	
    public void registerDescriptionStore(DescriptionStore descriptionStore) {
	    this.descriptionStore = descriptionStore;
    }
	
    public void registerSubscriptionStore(SubscriptionStore subscriptionStore) {
	this.subscriptionStore = subscriptionStore;
    }
	
    public void registerCatalogStore(CatalogStore catalogStore) {
	this.catalogStore = catalogStore;
    }
	
    public void registerScheduleStore(ScheduleStore scheduleStore) {
	this.scheduleStore = scheduleStore;
    }
	
    public void registerCollectionStore(CollectionStore collectionStore) {
	this.collectionStore = collectionStore;
    }
	
    public void registerLogStore(LogStore logStore) {
	this.logStore = logStore;
    }

    //
    // implement Manager
    //
    //
    // Execution Activity Logic
    //
    public void preInitialize() { }
    public void initialize() { }
    public void postInitialize() { }
    //
    public void run() { }
    //
    public void preFinalize() { }
    public void finalize() { }
    public void postFinalize() { }

	
}





