/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/storage/StoreLog.java,v 1.2 2000/11/22 02:34:55 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

/**
 * StoreLog.java
 *
 *
 * Created:  Saturday Nov 6 22:34:49 
 *
 * @author Will Conant
 * @author Bruce Hunt
 * @version 2
 */
package com.adobe.rice.common.storage;

import com.adobe.rice.common.policies.*;
import com.adobe.rice.common.storage.logging.*;
import java.io.*;

public class StoreLog implements LogStore {

    // This implementation uses a simple file to store logs.
    // $FUTURE$
    // It could/should be rewritten to use a policy to determine the 
    // type of storage to use ( Database, File, Internet, .... )
    //
    FileWriter rawLogFile = null;	// the raw log file
    BufferedWriter logFile = null;

    public StoreLog() throws FileNotFoundException, 
                             IllegalArgumentException, 
                             IOException 
    {
	    //
	    // get the log directory and file name from StoragePolices.
	    //
	    String  logFileName = (String) StoragePolicies.logDirectory.getSelection(); 
	    logFileName += System.getProperty("file.separator") + (String) StoragePolicies.logFileName.getSelection();
	    // get the log buffersize as well.
	    Integer logFileBufferSize = (Integer) StoragePolicies.logFileBufferSize.getSelection();
	    try { rawLogFile = new FileWriter( logFileName, true );	} 
	    catch ( FileNotFoundException fnf ) { throw fnf; }
	    catch ( IOException ioe ) { throw ioe; }
	    catch ( SecurityException se ) { throw se; }
	    try {
	        logFile = new BufferedWriter( rawLogFile, logFileBufferSize.intValue() );
	    } catch ( IllegalArgumentException iae ) { throw iae; }
    }
    
    public void appendLogEntry( LogEntry logEntry ) throws IOException 
    {
	    if ( ((String) StoragePolicies.logStyle.getSelection()).equals("xml") )
	        logFile.write( logEntry.toXMLstring() );
	    else
	        logFile.write( logEntry.toString() );
	        
	    //
	    // Automatically flush if the log level is high; or the impact is above serious.
	    // This way, the log is updated reasonably.
	    //
	    // $POLICY$ - make it a matter of policy?
	    //
	    PolicyManager pm = PolicyManager.getPolicyManager();
	    int loggingLevel = ( (Integer) (pm.getSafeSelection("LoggingLevel", new Integer(9))) ).intValue();
	    if ( (loggingLevel > 5 ) || ( logEntry.getImpact() >= ImpactRange.SERIOUS ) )
	        flush();
    }

    public void close() throws IOException { logFile.close(); }
    public void flush() throws IOException { logFile.flush(); }
    public void newline() throws IOException { logFile.newLine(); }
    
} // StoreLog









