/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/storage/Subscription.java,v 1.1.1.1 2000/09/19 19:39:49 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.storage;

import java.io.*;
import com.adobe.rice.util.time.*;

public class Subscription implements Serializable {
	public static final byte EXP_PRIORITY_FIRST = 1;
	public static final byte EXP_PRIORITY_LAST = 2;
	
	private String currentState;
	private IceDate expirationDate;
	private byte expirationPriority;
	private int quantityRemaining;
	private String subscriptionID;
	private Offer originalOffer;
	
	public Subscription() {
		
	}
	
	/**
	* Sets the state of the Subscription.
	* 
	* @param	newState	String representing new state of the Subscription.
	*/
	public void setCurrentState(String newState) {
		currentState = newState;
	}
	
	/**
	* Returns a string representing the current state of the Subscription.
	*/
	public String getCurrentState() {
		return currentState;
	}
	
	/**
	* Sets the expiration date of the Subscription.
	* 
	* @param	date	IceDate representing the Subscription's expiration date.
	*/
	public void setExpirationDate(IceDate date) {
		
		expirationDate = date;
	}
	
	/**
	* Returns the expiration date of the Subscription.
	*/
	public IceDate getExpirationDate() {
		return expirationDate;
	}
	
	/**
	* Sets the expiration priority of the Subscription.
	*
	* @param	priority	Either EXP_PRIORITY_FIRST or EXP_PRIORITY_LAST.
	*/
	public void setExpirationPriority(byte priority) {
		if (priority < 1 || priority > 2) {
			throw new IllegalArgumentException("Must be first or last priority.");
		}
		
		expirationPriority = priority;
	}
	
	/**
	* Returns the expiration priority of the Subscription
	* (either EXP_PRIORITY_FIRST or EXP_PRIORITY_LAST).
	*/
	public byte getExpirationPriority() {
		return expirationPriority;
	}
	
	/**
	* Sets the number of deliveries remaining for the Subscription.
	*
	* @param	remaining	number of remaining deliveries.
	*/
	public void setQuantityRemaining(int remaining) {
		if (remaining < 0) {
			throw new IllegalArgumentException("Quantity remaining cannot be negative.");
		}
		
		quantityRemaining = remaining;
	}
	
	/**
	* Returns the number of deleveries ramaining for the Subscription.
	*/
	public int getQuantityRemaining() {
		return quantityRemaining;
	}
	
	/**
	* Sets the subscription id of the Subscription.
	* 
	* @param	id	String representing the subscription id.
	*/
	public void setSubscriptionID(String id) {
		subscriptionID = id;
	}
	
	/**
	* Returns a String containing the subscription id.
	*/
	public String getSubscriptionID() {
		return subscriptionID;
	}
	
	/**
	* Sets the original offer of the Subscription.
	* 
	* @param	offer	a reference to the offer for the subscription.
	*/
	public void setOriginalOffer(Offer offer) {
		originalOffer = offer;
	}
	
	/**
	* Returns a reference to the original offer of the Subscription.
	*/
	public Offer originalOffer() {
		return originalOffer;
	}
}