/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/storage/logging/ImpactRange.java,v 1.3 2000/11/22 02:35:21 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

/**
 * ImpactRange.java
 *
 *
 * Created: Sat Nov  6 08:23:15 1999
 *
 * @author V. Bruce Hunt
 * @version 1.0
 */

package com.adobe.rice.common.storage.logging;

public class ImpactRange  
{
    int	   min;
    int	   max;
    String name;
    
    public ImpactRange( ) 
    {
	    min = 0;
	    max = 0;
	    name = "";
    }

    ImpactRange( int iMin, int iMax, String iName ) {
	    min = iMin; 
	    max = iMax;
	    name = iName;
    }
    
    public static final int COMMENTARY   = 0;
    public static final int INFORMATION  = 100000000;
    public static final int WARNING      = 200000000;
    public static final int SERIOUS      = 300000000;
    public static final int CRITICAL     = 400000000;
    public static final int DAMAGING     = 500000000;
    public static final int FATAL        = 600000000;
    public static final int CATASTROPHIC = 700000000;
    
    public static final int SPAN         =  99999999;

    static final ImpactRange impactRange[] = 
    { 
	    new ImpactRange( COMMENTARY,   COMMENTARY+SPAN,   "Commentary"  ),
	    new ImpactRange( INFORMATION,  INFORMATION+SPAN,  "Information" ),
	    new ImpactRange( WARNING,      WARNING+SPAN,      "Warning"     ),
	    new ImpactRange( SERIOUS,      SERIOUS+SPAN,      "Serious"     ),
	    new ImpactRange( CRITICAL,     CRITICAL+SPAN,     "Critical"    ),
	    new ImpactRange( DAMAGING,     DAMAGING+SPAN,     "Damaging"    ),
	    new ImpactRange( FATAL,        FATAL+SPAN,        "Fatal"       ),
	    new ImpactRange( CATASTROPHIC, CATASTROPHIC+SPAN, "Catastrophic"),
    };

    public static String getImpact( int lImpact ) 
    {
	    for ( int i = 0; i < impactRange.length; i++ )
	    {
	        if ( lImpact > impactRange[i].max )
		    continue;
	        if ( impactRange[i].min <= lImpact && lImpact <= impactRange[i].max )
		    return impactRange[i].name;
	    }
	    return "Unknown";
    }
    
    public static int getImpactBase( int theImpactValue )
    {
        for ( int i = 0; i < impactRange.length; i++ )
        {
            if ( theImpactValue > impactRange[i].max ) 
                continue;
            if ( (impactRange[i].min <= theImpactValue) && 
                 (theImpactValue <= impactRange[i].max)  )
                return impactRange[i].min;
        }
        long value = theImpactValue;
        value /= impactRange[1].min;
        value *= impactRange[1].min;
        int intval = (int) value;
        return intval;
    }
    
    public static int impactOf( String imp )
    {
        for ( int i = 0; i < impactRange.length; i++ )
        {
            if ( impactRange[i].name.equals(imp) )
                return impactRange[i].min;
        }
        return impactRange[ impactRange.length-1 ].min;
    }
    
} // ImpactRange
