/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/storage/logging/LogEntry.java,v 1.2 2000/11/22 02:35:21 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.storage.logging;

//$JDK1.1$ import com.sun.java.util.collections.*;
import com.adobe.rice.common.XMLassembly.*;
import java.io.*;
import java.util.*;
import java.text.*;

public class LogEntry implements Serializable, XMLexpandable {
    private String		category;
    private int			impact;
    private String 		messageName;
    private List		fields;
	
    /**LogEntry defines a single log entry.  A LogEntry has four important components,
     ** <em>category</em>, <em>impact</em>, <em> message name</em> and <em>fields</em>.
     ** <dl>
     **   <dt>category</dt>
     **      <dd>The category is intended to define a single functional area of ICE or
     **          in the application processor.  This would typically be a string of the form:
     **          <em>packageName.className</em> or for small packages, <em>packageName</em>
     **		     For example, the ICE functional areas
     **          consist in part of <em>Syndicator.aClass</em>, <em>Subscriber.aClass</em>,
     **          <em>Subscription.aClass</em>, <em>Transportation.aClass</em>, 
     **          <em>PayloadAssembly.aClass</em>, <em>Scheduling.aClass</em> and so forth.  
     **		     An application
     **	         processor defines it's own categories; it is recommended that pattern:
     **          <em>packageName.clasName</em> be used there as well.  This is intended to be used
     **	         by a reader as comprehension context so that the issue addressed by the 
     **          log entry can be quickly comprehended for any needed action. </dd>
     **	  <dt>impact</dt>
     **      <dd>Intended to be a measure of the imact, the logged behavior could have on the
     **          system's operation spanning commentary through system meltdown.  
     **		     In general the impact has the following associations:
     **		 <dl>
     **        <dt>Commentary</dt><dd>Integers in range 0 to 99.999,999.  This is detailed
     **            operational information that is expected to have little or no system
     **            impact with the exception that it may be extremely detailed information
     **            used in support of a higher impact entry. Intervention is not 
     **		       required.</dd>
     **	       <dt>Information</dt><dd>Integers in range 100,000,000 to 199,999,999.  This is 
     **            summary operational information that is expected to provide a precis of 
     **            system operation and may be useful for tracking performance, internal 
     **            resource use, external resource use, external and internal details
     **            in support of a higher level message. Intervention is not directly
     **		       required.</dd>
     **        <dt>Warning</dt><dd>Integers in range 200,000,000 to 299,999,999.  Information 
     **		       intended to highlight possible unexpected or unusual behavior that should
     **	           be examined for correction.  The system can recover from this and
     **            provides the warning to signal longer term possiblility of more detrimental
     **            consequences.  Intervention may be required.  
     **		       An example might be a warning when some fraction of 
     **            of storage capacity is exceeded; or when several repeated attempts to 
     **            access a subscription fail and should be examined by an operator.</dd>
     **        <dt>Serious</dt><dd>Integers in range 300,000,000 to 399,999,999.  Information 
     **            exposing clearly detrimental system behavior.  The system <em>may</em> be able to 
     **		       recover but correct operation is not assured.  Intervention is likely to
     **            be required. An example could be very low delivery bandwidth indicating
     **            that it is unlikely that scheduled deliverys can be made.
     **		   <dt>Critical</dt><dd>Integers in range 400,000,000 to 499,999,999. Information
     **		       logging behavior that indicates dangerous system behavior.  The system's
     **		       <em>ability</em> to recover is in doubt.  Intervention is required.
     **		       An example is dangerously low memory or storage resources.
     **		       </dd>
     **		   <dt>Damaging</dt><dd>Integers in range 500,000,000 to 599,999,999.  Information
     **		       logging behavior that is damaging to the system.  The system is unable to 
     **		       recover and is reporting damage done.  Intervention is mandatory to 
     **		       repair the system for proper operation. </dd>
     **        <dt>Fatal</dt><dd>Integer in range 600,000,000 to 699,999,999.  Information
     **		       logging behavior that requires system shutdown due to damage.  The system
     **		       is terminating because it cannot continue; recovery without loss is 
     **            highly unlikely.  Intervention is mandatory to repair the system for 
     **            operation. </dd>
     **        <dt>Catastrophic</dt><dd>Integer in range 700,000,000 to 799,999,999.  Information
     **            logging behavior that shows fatal damage and significant loss.  The system
     **            behavior is unpredictable and is attempting to shut down for a previous
     **            fatal condition.  System reconstruction from external sources will be
     **            required.  Intervention is mandatory. </dd>
     **		</dl>
     **	     </dd>
     **	  <dt>Message Name</dt>
     **      <dd>The name of the message.</dd>
     **   <dt>Fields</dt>
     **      <dd>A list of fields.  Each field consists of a name and a "loggable" object.
     **          The name is a string and the object is any object satisfying the "loggable"
     **		 interface.  A field is used to describe a part of the message.
     **	     </dd>
     ** </dl>
     ** Ice-cubes defines the following categories as the name of Rice Packages:
     ** <ul>
     **	   <li>Scheduling
     **	   <li>Payload Assembly
     **    <li>Payload Disassembly
     **    <li>Transportation
     **    <li>Storage
     **    <li>Negotiation
     **    <li>Subscription Management
     **    <li>Catalog Management
     **    <li>
     **	</ul>
     **/
    public LogEntry() {
	    impact = 0;
	    fields = new LinkedList();
    }

    public LogEntry( String lCategory, int lImpact, String lMessageName ) {
	    this();
	    category = lCategory;
	    impact = lImpact;
	    messageName = lMessageName;
    }
	
    public void setCategory(String category) {
	    this.category = category;
    }
    
    public String getCategory() { return this.category; }
	
    public void setImpact(int impact) {
	    this.impact = impact;
    }
    
    public int getImpact() { return this.impact; }
	
    public void setMessage(String messageName) {
	    this.messageName = messageName;
    }
    
    public String getMessage( ) { return this.messageName; }
    
    public List getFields() { return this.fields; }
	
    public void addField(String fieldName, Loggable loggableObject) {
	    fields.add(new LogField(fieldName, loggableObject.toLog() ));
    }

    // TODO a simple string dump. this can be changed later. jdh
    public String toString() {
	    StringBuffer sb = new StringBuffer();
	    DateFormat date = DateFormat.getDateTimeInstance( DateFormat.MEDIUM, DateFormat.MEDIUM);
    	
	    sb.append( date.format( new Date() ) + 
		    ": " + category + 
		    ", " + impact + 
		    ", " + messageName + 
		    ": " );
    	
	    Iterator it = fields.iterator();
	    while (it.hasNext()) {
	        LogField lf = (LogField) it.next();
	        // TODO lf.fieldValue needs better specifics.
	        sb.append("[" + lf.name + ": " + lf.value + "]"); 
	    }
	    return sb.toString();
    }

    public boolean canDoCharData() { return false; }
    public String toCDSect() { return ""; }
    public String toCharData() { return ""; }

    public String toXMLstring() {
	    StringBuffer sb = new StringBuffer();
	    //
	    // Output a logEntry in XML format
	    // 
	    DateFormat date = DateFormat.getDateTimeInstance( DateFormat.MEDIUM, DateFormat.MEDIUM);
	    sb.append( "<logentry>\n" +
		    "  <date>" + date.format( new Date() ) + "</date>\n" +
		    "  <category>" + category + "</category>\n" +
		    "  <impact>\n" +
		    "    <severity>" + ImpactRange.getImpact( impact ) + "</severity>\n" +
		    "    <value>" + (new Integer(impact)).toString() + "</value>\n" +
		    "  </impact>\n" +
		    "  <message>\n" +
		    "     <name>" + messageName + "</name>\n" 
	        ) ;
	    Iterator it = fields.iterator();
	    while ( it.hasNext() ) {
	        LogField lf = (LogField) it.next();
	        sb.append( "    <field>\n" +
		        "      <name>" + lf.name + "</name>\n" +
		        "      <value>" + lf.value + "</value>\n" +
		        "    </field>\n" );
	    }
	    sb.append("  </message>\n" +
		    "</logentry>\n"
	        );
	    return sb.toString();
    }
	
    private class LogField {
	    String name;
	    String value;
    	
	    public LogField(String fname, String fvalue) {
	        name = fname;
	        value = fvalue;
	    }
    }
}

