/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/storage/logging/LogMsg.java,v 1.2 2000/11/22 02:35:21 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/
/**
 * LogMsg.java
 *
 *
 * Created: Fri Nov  5 14:16:26 1999
 *
 * @author V. Bruce Hunt
 * @version
 */
package com.adobe.rice.common.storage.logging;

import java.text.*;
import java.io.*;
import com.adobe.rice.common.storage.*;

/**
 ** Place a MessageFormat Message into the Rice log.
 ** This class implements an extension of the MessageFormat class to make
 ** it easy and fast for Rice objects to construct log messages.
 **
 ** The model for use of this class is as follows:<br>
 ** <tt><pre>
 ** String pattern = " Can't contact Subscriber {0} for scheduled package delivery {1} at {2,time}.";
 ** Object[] parms = { subscriber.toString(), subscriber.currentPackage.toString(), Date.date() };    
 ** LogMsg logmsg = new LogMsg( pattern, parms );
 ** logmsg.prolog( "scheduling.Delivery", 1010, "deliveryPump" );
 ** logmsg.log();
 ** // if the message were important enough to want to assure that it was written:
 ** logmsg.flush(); 
 ** </pre></tt>
 ** <br>
 ** 
 **/
public class LogMsg extends MessageFormat implements Loggable 
{
    Object[] arguments;

    String category;
    int	   impact;
    String messageName;

    static LogStore storeLog;
    
    public LogMsg( String pattern, Object[] args ) 
    {
	    super( pattern );
	    arguments = args;
	    if ( storeLog == null ) 
	    {
	        storeLog = StorageManager.getLogStore();
	    }
    }
    
    //
    // methods
    //
    /** Set initial log category and impact items.
     ** 
     **/
    public void prolog( String mCategory, int mImpact, String mMessageName )
    {
	    category = mCategory;
	    impact = mImpact;
	    messageName = mMessageName;
    }
    //
    // format and output the logmsg to the log.
    //
    public void log() {
	    LogEntry le = new LogEntry( category, impact, messageName);
	    le.addField("log msg", this );
	    try {
	        storeLog.appendLogEntry( le );
	    } catch ( IOException ioe ) {
	        // got an I/O error on the long.
	        // Output the message on the console:  GET OPERATOR's ATTENTION
	        String msg = "!!!! LOG IO ERROR !!!! for msg: ";

	        msg += category + ", " + ImpactRange.getImpact(impact) + ( new Integer(impact)).toString();
	        msg += ", " + messageName + ": " ;
	        System.out.println( msg + this.toLog() );
	    }
    }
    //
    // Implement Loggable:
    //
    public String toLog()
    {
	    String logString = null;

	    if ( arguments != null )
	    {
	        logString = "<logmsg>" + this.format( arguments ) + "</logmsg>";
	    }
	    else
	    {
	        logString = "<logmsg> **** --&gt;|Arguments missing in LogMsg|&lt;<---**** </logmsg>";
	    }
	    return logString;
    }
    // 
    // make sure that message get written to log before proceeding
    //
    public void flush()
    {
	    try { storeLog.flush(); }
	    catch ( IOException ioe ) { };
    }
} // LogMsg






