/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/transportation/TransportProtocol.java,v 1.3 2000/12/02 23:31:13 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.transportation;

//
// java imports
//
import java.io.*;
import java.net.*;

//
// icecubes imports
//
import com.adobe.rice.common.*;
import com.adobe.rice.common.payloadAssembly.*;
import com.adobe.rice.common.XMLdisassembly.*;
import com.adobe.rice.common.payloadDisassembly.*;
import com.adobe.rice.kernel.*;


/** The TransportProtocol interface will allow the TransportManager to give serialized 
 * payloads to TransportProtocols (e.g., HTTP, SMTP). */


public interface TransportProtocol 
{
    /** 
     * sendRequest is called internally from icecubes to initiate a Request/Response
     * protocol interchange.  Its function is to manage the specifics of the protocol
     * to construct the interchange.
     *
     * @param requestPayload - A Payload that contains the request.
     * @param location - A URL of where to send the request.
     *
     * @return Reader - the Reader that contains the response.
     *
     **/
	public Reader sendRequest( Payload requestPayload, URL location);
	/**
	 * receiveRequest is called from the "driver" (usually a servlet) when a
	 * request is received from an external source.  A response -- or at least
	 * a close() MUST be issued to close and complete the connection.
	 *
	 * @param requestReader - A Reader that contains the request.
	 * @param responseWriter - A Writer where the response is to be written.
	 *
	 **/
	public void receiveRequest( Reader inputRequest, Writer ouputResponse );
	
	public boolean canUseProtocol(String protocolName);
}
