/*
$Log: IceMessage.java,v $
Revision 1.2  2000/10/25 22:04:27  andrewf
added documentation and instructions

$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/kernel/IceMessage.java,v 1.2 2000/10/25 22:04:27 andrewf Exp $
$Author: andrewf $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org
*/

/*
   IceMessage represents the information stored in a message
   that is sent and received between IceCubeManagers

<dl>

<dt>IceMessage (String recipient, Object body) </dt>
<dd>Compose an IceMessage. recipient is the name of the
    queue/manager that the message will be delivered to.
    body is the message body</dd>

<dt>IceMessage (IceMessage originator, Object body)</dt>
<dd>Send a message, as a response to a previously sent
    message. originator contains information about the
    origing message - ie return address, transaction Id
    body - the new message </dd>

<dt>setAction (int action) </dt>
<dd>set action associates a type with an IceMessage. Action
    is a hint about the nature of the IceMessage and the
    underlying dispatcher makes no attempt to do verification
    </dd>

<dt>String getSender () </dt>
<dd>get the sender of the Message </dd>

<dt>getRecipient () </dt>
<dd>get the recipient </dd>

<dt>setBody (Object body)</dt>
<dd>set the body of the IceMessage </dd>

<dt>Object getMessage () </dt>
<dd>get the body of the message </dd>

<dt>Object getReturnAddress () </dt>
<dd>get the returnAddress, typically used by the 
    MessageDispatcher </dd>

<dt>setReturnAddress (Object returnAddress) </dt>
<dd>set the returnAddress. Typically used by the 
    MessageDispatcher</dd>

<dt>int setTransactionId (long id) </dt>
<dd>assign a transaction Id to the IceMessage</dd>

<dt>void getTransactionId () </dt>
<dd>get the transaction Id associated with the IceMessage </dd>

<dt>void setTimeout () </dt>
<dd>indicate that a call has timedout </dd>

<dt>boolean getTimeout () </dt>
<dd>has the call timedout ? </dd>

</dl>
*/

package com.adobe.rice.kernel;

public class IceMessage
{
    private String   recipient;     // the recipient is a mailslot or queue
    private String   sender;        // name of thread or manager
    private Object   senderBuffer;  // sender's buffer
    private Object   replyBuffer;   // the reply buffer
    private Object   returnAddress; // the dispatcher manipulates this
    private long     transactionId; // transactionId;
    private boolean  hasTimedout;
    private int      action;

    public IceMessage (Object body)
    {
        hasTimedout  = false;
        senderBuffer = body;
    }
    
    public IceMessage (boolean timedout)
    {
        hasTimedout = timedout;
    }
        
    public IceMessage (IceMessage originator, Object body)
    {
        hasTimedout  = false;
        senderBuffer = body;
        transactionId = originator.transactionId;
        returnAddress = originator.returnAddress;
    }

    public IceMessage (String recipient, Object body)
    {
        hasTimedout = false;
        this.recipient = recipient;
        senderBuffer = body;
    }
                       
    public void setAction (int action)
    {
        this.action = action;
    }
                       
    public int getAction ()
    {
        return action;
    }
                                           
    public String getSender ()
    {
        return sender;
    }
    
    public void setSender (String sender)
    {
        this.sender = sender;
    }
                                        
    public String getRecipient ()
    {
        return recipient;
    }
    
    public void setRecipient (String recipient)
    {
        this.recipient = recipient; 
    }
    
    public void setBody (Object body)
    {
        senderBuffer = body;
    }
    
    public Object getMessage ()
    {
        return senderBuffer;
    }
 
    /* TO BE REMOVED */
    public void setReply (Object message)
    {
        replyBuffer = message;
    }
    
    public Object getReturnAddress ()
    {
        return returnAddress;
    }
    
    public void setReturnAddress (Object returnAddress)
    {
        this.returnAddress = returnAddress;
    }

    public void setTransactionId (long id)
    {
        transactionId = id;
    }
    
    public long getTransactionId ()
    {
        return transactionId;
    }
    
    public void setTimedout ()
    {
        hasTimedout = true;
    }
    
    public boolean hasTimedout ()
    {
        return hasTimedout;
    }
}