package com.adobe.rice.kernel;

public abstract class IceRunner implements Runnable
{
    private MessageDispatcher dispatcher;
    private String            name;
    
    public IceRunner (String name, MessageDispatcher dispatcher)
                      throws DispatcherException
    {
        // Managers must have names and a dispatcher
        if (name == null || dispatcher == null)
           throw new DispatcherException ("null arguments not permitted");
           
        this.name = name;
        this.dispatcher = dispatcher;
    }
    
    // should throw an exception on a null name or message
    protected long send (String name, IceMessage message)
    {
        return dispatcher.asyncSend (name, message);    
    }
    
    protected IceMessage receive () throws DispatcherException
    {
        return dispatcher.receive ();
    }
    
    protected IceMessage receive (long timeout) 
                         throws DispatcherException
    {
        return dispatcher.receive (timeout);
    }
    
    protected IceMessage receive (String queueName)
                         throws DispatcherException
    {
        return dispatcher.receive (queueName);
    }
    
    protected IceMessage receive (String queueName,
                                  long   timeout)
                         throws DispatcherException
    {
        return dispatcher.receive (queueName, timeout);
    } 
    
    
    protected IceMessage synchronousSend (String     queueName, 
                                          IceMessage message)
                         throws DispatcherException
    {
        return dispatcher.synchronousSend (queueName, message);  
        
    }
    
    protected IceMessage synchronousSend (String     queueName, 
                                          IceMessage message,
                                          long       msecs)
                         throws DispatcherException
    {
        return dispatcher.synchronousSend (queueName, message, msecs);          
    }
    
    protected long reply (IceMessage message)
    {
        return dispatcher.reply (message);   
    }
    
    public String getName ()
    {
        return name;
    }
}