/*
$Log: MailSlot.java,v $
Revision 1.2  2000/10/25 22:04:27  andrewf
added documentation and instructions

$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/kernel/MailSlot.java,v 1.2 2000/10/25 22:04:27 andrewf Exp $
$Author: andrewf $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org


*/

package com.adobe.rice.kernel;
import  java.util.*;

// a MailSlot is the lightweight version of the
// mail queue. It assumes there is room for only
// one buffer and only one thread waiting. MailSlot
// is optimized for thread to thread communications
//
//
// TODO - create free list of slots
//      - perhaps throw an exception if a second
//      - thread attempts to send a message


public class MailSlot implements Mailbox
{
    private Object buffer;
    
    public MailSlot ()
    {
        buffer = null;     
    }
    
    // make the assumption that there
    // there is only one thread waiting
    public synchronized void add (Object element)
    {
        buffer = element;
        notify ();
    }
    
    public synchronized Object remove () 
                        throws InterruptedException
    {
        Object element = null;
       
        try
        {   
           while (buffer == null) { wait (); }
           element = buffer;
        }
        catch (InterruptedException e)
        {
           element = null;
        }
        
        buffer = null;
        return element;
    }
    
    
    public synchronized Object remove (long msecs)
                               throws InterruptedException
    {
        Object element;
        
        if (Thread.interrupted ())
        {
           throw new InterruptedException ();
        }
              
        try 
        {
           // do an initial check
           if (buffer != null)
           {
              element = buffer;
              buffer = null;
           }
           else
           {  
              long timeout;
              long expiration = System.currentTimeMillis() + msecs;
                   
              for (;;)
              {
                 timeout = expiration - System.currentTimeMillis();
                 
                 if (buffer != null)
                 {
                    element = buffer;
                    buffer  = null;
                    break;
                 }
                
                 if (timeout <= 0)
                 {
                    element = null;
                    break;
                 }
                 
                 wait (timeout);
              }
           }
        }
        catch (InterruptedException e)
        {
            element = null;
        }
        
        return element;
    }    
    
      
    public synchronized Object asyncRemove () 
    {
        Object element = null;
       
         
        if (buffer != null)
        {
           element = buffer;
           buffer = null;
        }
        
        return element;
    }   
}