/*
$Log: IceDispatcherTest2.java,v $
Revision 1.3  2000/10/27 02:49:01  bhunt
Updated tests- new message dispatcher

Revision 1.2  2000/10/25 22:06:31  andrewf
added documentation

$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/kernel/tests/IceDispatcherTest2.java,v 1.3 2000/10/27 02:49:01 bhunt Exp $
$Author: bhunt $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org
*/

package com.adobe.rice.kernel.tests;
import com.adobe.rice.kernel.*;

// test the synchronous RPC feature
// 
// Three managers, asynchronous manager A sends message to B
// Manager B, spawns a worker thread to respond to A but
// sends an addition message to a third manager that spawns
// a thread to respond to this request.


public class IceDispatcherTest2
{   
    public static void main (String argv [])
    {
        class TestServlet extends IceCubeManager
        {
           private String banner;
           
           TestServlet (String name, String banner) throws DispatcherException
           {
               super (name, MessageDispatcher.instance ());
               this.banner = banner;
           }
            
           public void run ()
           {
               IceMessage message = new IceMessage (banner);
               IceMessage reply = null;
               
               System.out.println (getName () + " starting");
               System.out.println (" sending message to Manager");
               
               try
               {
                   reply = synchronousSend ("manager-a", message);
               }
               catch (DispatcherException e)
               {
                  e.printStackTrace ();
               }
                
               System.out.println (getName () + " received " + reply.getMessage ());
           }
           
           public void preInitialize() {}
           public void initialize() {}
           public void postInitialize() {}
           public void preFinalize(){}
           public void finalize(){}
           public void postFinalize(){}
        }
          
        class TestManager2 extends IceCubeManager 
        {
           private String     banner;
          
           
           class WorkerThread extends IceCubeManager
           {
               private IceMessage myMessage;
               
               WorkerThread (String name, IceMessage message) 
                            throws DispatcherException
               {
                   super (name,MessageDispatcher.instance ()); 
                   Thread.currentThread ().setName (name);
                   this.myMessage = message;
               }
               
               public void run ()
               {
                   IceMessage reply;
                   IceMessage query = new IceMessage ("simulated RPC request");
                   IceMessage newMessage;
                   String result;
                   
                   try
                   {
                      // generate a message to simulate RPC 
                      reply = synchronousSend ("manager-b", query);
                      
                      // now reply to original message
                      newMessage = new IceMessage (myMessage, reply.getMessage ()
                                                   + " " + myMessage.getMessage ());
                      reply (newMessage);
                   }
                   catch (DispatcherException e)
                   {
                      e.printStackTrace ();
                   }
               }  
                public void preInitialize() {}
                public void initialize() {}
                public void postInitialize() {}
                public void preFinalize(){}
                public void finalize(){}
                public void postFinalize(){}
           }
           
           TestManager2 (String name, String banner) throws DispatcherException
           {
               super (name, MessageDispatcher.instance ());
               this.banner = banner;
           }
            
           public void run ()
           {
               IceMessage message;
               IceMessage newMessage = null;
               WorkerThread workerThread = null; 
                
               System.out.println (getName () + " starting");
               
               while (true)
               {
                  try
                  {
                  
                    System.out.println (getName () + " about to receive");
                    
                    // receive a message
                    message = receive ();
                    
                    // and start a new worker thread
                    workerThread = new WorkerThread (getName (), message);
                    new Thread (workerThread).start ();
                     
                  }                     
                  catch (DispatcherException e)
                  {
                    e.printStackTrace ();
                  }
               }
           }
            public void preInitialize() {}
            public void initialize() {}
            public void postInitialize() {}
            public void preFinalize(){}
            public void finalize(){}
            public void postFinalize(){}
        }
    
    
        class TestManager3 extends IceCubeManager
        {
           private String banner;
           
           TestManager3 (String name, String banner) throws DispatcherException
           {
               super (name, MessageDispatcher.instance ());
               this.banner = banner;
           }
            
           public void run ()
           {
               IceMessage message;
               IceMessage newMessage = null;
                
               System.out.println (getName () + " starting");
               
               while (true)
               {
                  try
                  {
                  
                    System.out.println (getName () + " about to receive");
                    
                    // receive a message
                    message = receive ();
                  
                    // and send a response
                    newMessage = new IceMessage (message,"manager B processed ");
                    reply (newMessage);     
                  }                     
                  catch (DispatcherException e)
                  {
                    e.printStackTrace ();
                  }
               }
           }
           public void preInitialize() {}
           public void initialize() {}
           public void postInitialize() {}
           public void preFinalize(){}
           public void finalize(){}
           public void postFinalize(){}
        }
    
 
        try
        {
           Runnable  manager = new TestManager2 ("manager-a","I got the message"); 
           Runnable  manager1 = new TestManager3 ("manager-b","I got the message"); 
           Runnable  servlet;
           String name = "servlet";
           
           
           servlet = new TestServlet (name, "hello world from servlet " + name);
           new Thread (servlet, name).start ();
                   
           
           new Thread (manager,"manager-a").start ();
           new Thread (manager1,"manager-b").start ();
           
        }
        catch (DispatcherException e)
        {
           e.printStackTrace ();
        }
    }
}
