package com.adobe.rice.kernel.tests;
import com.adobe.rice.kernel.*;

// test the synchronous RPC timeout features
// 
// Three managers, asynchronous manager A sends message to B
// Manager B, spawns a worker thread to respond to A but
// sends an addition message to a third manager that spawns
// a thread to respond to this request.


public class IceDispatcherTest3
{   
    public static void main (String argv [])
    {
        class TestServlet extends IceCubeManager
        {
           private String banner;
           
           TestServlet (String name, String banner) throws DispatcherException
           {
               super (name, MessageDispatcher.instance ());
               this.banner = banner;
           }
            
           public void run ()
           {
               IceMessage message = new IceMessage (banner);
               IceMessage reply = null;
               
               System.out.println (getName () + " starting");
               System.out.println (" sending message to Manager");
               
               try
               {
                   long now = System.currentTimeMillis ();
                   reply = synchronousSend ("manager-a", message, 1000);
                   if (System.currentTimeMillis () - now > now)
                   {
                      System.out.println ("reply should have timed out " + now);
                   }
                   
                   if (reply.hasTimedout())
                   {
                      System.out.println ("Send has timed out");
                   }
               }
               catch (DispatcherException e)
               {
                  e.printStackTrace ();
               }
                
               System.out.println (getName () + " received " + reply.getMessage ());
           }
            public void preInitialize() {}
            public void initialize() {}
            public void postInitialize() {}
            public void preFinalize(){}
            public void finalize(){}
            public void postFinalize(){}
        }
          
        class TestManager2 extends IceCubeManager
        {
           private String     banner;
          
           
           class WorkerThread extends IceCubeManager
           {
               private IceMessage myMessage;
               
               WorkerThread (String name, IceMessage message) 
                            throws DispatcherException
               {
                   super (name,MessageDispatcher.instance ()); 
                   Thread.currentThread ().setName (name);
                   this.myMessage = message;
               }
               
               public void run ()
               {
                   IceMessage reply;
                   String result;
                   long r;
                   
                   try
                   {
                      // make sure that other party timed out
                      Thread.currentThread ().sleep (10000);
                      reply = new IceMessage (myMessage," late sending message");
                      r = reply (reply);
                      if (r == -2)
                      {
                         System.out.println ("Return address non-existant");
                      }
                      else
                      {
                         System.out.println ("Reply: " + r);
                      }
                   }
                   catch (InterruptedException e)
                   {
                      e.printStackTrace ();
                   }
               } 
                public void preInitialize() {}
                public void initialize() {}
                public void postInitialize() {}
                public void preFinalize(){}
                public void finalize(){}
                public void postFinalize(){}
           }
           
           TestManager2 (String name, String banner) throws DispatcherException
           {
               super (name, MessageDispatcher.instance ());
               this.banner = banner;
           }
            
           public void run ()
           {
               IceMessage message;
               IceMessage newMessage = null;
               WorkerThread workerThread = null; 
                
               System.out.println (getName () + " starting");
               
               while (true)
               {
                  try
                  {
                  
                    System.out.println (getName () + " about to receive");
                    
                    // receive a message
                    message = receive ();
                    
                    // and start a new worker thread
                    workerThread = new WorkerThread (getName (), message);
                    new Thread (workerThread).start ();
                     
                  }                     
                  catch (DispatcherException e)
                  {
                    e.printStackTrace ();
                  }
               }
           }
            public void preInitialize() {}
            public void initialize() {}
            public void postInitialize() {}
            public void preFinalize(){}
            public void finalize(){}
            public void postFinalize(){}
           
        }
    
    
        try
        {
           Runnable  manager = new TestManager2 ("manager-a","I got the message"); 
           Runnable  servlet;
           String name = "servlet";
           
           
           servlet = new TestServlet (name, "hello world from servlet " + name);
           new Thread (servlet, name).start ();
                   
           
           new Thread (manager,"manager-a").start ();           
        }
        catch (DispatcherException e)
        {
           e.printStackTrace ();
        }
    }
}
