/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/util/IceCodes.java,v 1.3 2000/12/02 23:37:25 bhunt Exp $


ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.util;

import java.util.*;

public class IceCodes extends ListResourceBundle 
{	
	public static String get(String message) 
	{
		ResourceBundle res = ResourceBundle.getBundle("com.adobe.rice.util.IceCodes");
		return res.getString(message);
	}
	
	public Object[][] getContents() 
	{
		return contents;
	}
	
	static Object[][] contents = 
	{
	    
	    {"2__", "Successful Completion Codes" },
	    
		{"200", "OK" },         /* The operation completed successfully. */
        {"201", "Confirmed" },  /* The operation is confirmed. See Section 3.8 Confirmation for a discussion. */
        {"202", "Package sequence state already current" }, /* A Subscriber requested a package update, 
                                but the Subscriber is already in the current package sequence state, i.e.,
                                there are no updates at the moment. */
        /*
        3xx: Payload level Status Codes 

        These indicate something about the ice-payload itself, as opposed to the individual requests 
        and responses within the payload. These codes have one very explicit and important semantic: 
        they are used when the payload could not be properly interpreted, meaning that even if there 
        were multiple requests in the payload, there will be only one ice-code in the response. For 
        example, if the payload had been corrupted, it might be so corrupted that it isn't even 
        possible to determine how many requests it contains, let alone respond to them individually. 
        */
        { "3__", "Payload level status codes" },
        
        { "300", "Generic catastrophic payload error" }, /* Generic status code indicating inability 
                            to comprehend the received payload. Usually, it is better to send a more
                            specific code if possible. */
        { "301", "Payload incomplete/cannot parse" }, /* The payload sent is severely garbled and 
                            cannot be parsed. For example, if a binary file were sent instead of an
                            XML payload, this would be an appropriate response. */
        { "302", "Payload not well formed XML" }, /* The payload sent is recognizable as XML, but is
                            not well formed per the definition of XML. This is available as both a 
                            payload level error and as a request level (4xx) error. Whether a given
                            implementation attempts to interpret not well formed XML so as to 
                            generate request level (4xx) errors versus payload level (3xx) errors is
                            a quality of implementation issue. */
                            /* $FIX$ - How does this implementation handle this QOI issue? */
        { "303", "Payload validation failure" }, /* The payload failed validation according to the 
                            DTD. This is available as both a payload level error and as a request
                            level (4xx) error. Whether a given implementation attempts to interpret
                            not well formed XML so as to generate request level (4xx) errors versus 
                            payload level (3xx) errors is a quality of implementation issue. Note 
                            that Receivers SHOULD perform validation on incoming ICE payloads, but 
                            are not required to. Senders MUST send only valid ICE payloads or they are 
                            in error; however, the ability to detect invalid payloads is a 
                            quality-of-implementation issue for the Receiver, and Senders MUST NOT 
                            assume the Receiver will perform an XML validation on their payloads. */
                            /* This implementation validates on sending and receiving. */
        { "320", "Incompatible version" }, /* The ICE protocol version used in the request is not 
                            supported. NOTE: The ICE protocol versions are transmitted as part of the
                            payload header, implementations may look there to decide what appropriate
                            corrective actions to take. Implementations must follow the version rules 
                            defined in Appendix B. For a discussion of ICE versions see also 
                            ice.version in section 3.5.2, Payload Element Definition */
        { "331", "Failure fetching external data" }, /* The receiver could not follow an external 
                            reference (URL) given to it by the sender as an external entity reference. 
                            Note that in ICE 1.0 and ICE 1.1 only the Subscriber is permitted to reply 
                            with this code.  A Syndicator MUST NOT reply with this code. */ 
        { "390", "Payload temporary redirect" }, /* Used with redirection; see 3.4.5. */
        { "391", "Payload permanent redirect" },  /* Used with redirection; see 3.4.5. */
        
        /* 
        4xx: Request level Status Codes 

        These indicate errors caused by an inability to carry out an individual request. 
        Note that in some cases there are similar errors between the 3xx and 4xx class; 
        the difference is whether or not the error is supplied as a single, payload level error
        code (3xx) or whether it is supplied as a prerequisite code. 
        
        */
        { "4__", "Request Level Status Codes" },

        { "400", "Generic request error" }, /* Generic status code indicating inability to comprehend
                            the request. Usually, it is better to send a more specific code if 
                            possible. */
        { "401", "Incomplete/cannot parse" }, /* The request sent is severely garbled and cannot be 
                            parsed. Note that in most cases, a payload level error (301) might
                            be more appropriate. */
        { "402", "Not well formed XML" }, /* The request sent is recognizable as XML, but is not well
                            formed per the definition of XML. This is available as both a payload 
                            level error and as a request level (4xx) error. Whether a given 
                            implementation attempts to interpret not well formed XML so as to generate
                            request level (4xx) errors versus payload level (3xx) errors is a quality 
                            of implementation issue. */
        { "403", "Validation failure" }, /* The request failed validation according to the DTD. This
                            is available as both a payload level error and as a request level (4xx) 
                            error. Whether a given implementation attempts to interpret not well 
                            formed XML so as to generate request level (4xx) errors versus payload 
                            level (3xx) errors is a quality of implementation issue. Note that 
                            Receivers SHOULD perform validation on incoming ICE payloads, but are not 
                            required to. Senders MUST send only valid ICE payloads or they are in 
                            error; however, the ability to detect invalid payloads is a 
                            quality-of-implementation issue for the Receiver, and Senders MUST NOT 
                            assume the Receiver will perform an XML validation on their payloads. */
        { "404", "This error intentionally left blank" }, 
        { "405", "Unrecognized sender" }, 
        { "406", "Unrecognized subscription" },
        { "407", "Unrecognized operation" }, 
        { "408", "Unrecognized operation arguments" },
        { "409", "Not available under this subscription" }, /* The Requester has referenced something
                            not covered by the subscription referenced in the request. */
        { "410", "Not found" }, /* Generic error for being unable to find something. */
        { "411", "Unrecognized package sequence state" }, /* The package sequence identifier supplied
                            by the Sender is not understood by the Receiver. */
        { "412", "Unauthorized" },
        { "413", "Forbidden" },
        { "414", "Business term violation" },
        { "420", "Constraint failure" }, /* Compliant implementations MUST NOT send this message if 
                            the constraint was not specified in the negotiated subscription. */ 
        { "422", "Schedule violation, try again later" }, /* The request was made at an incorrect 
                            time. For example, trying to get a package update outside of the agreed 
                            upon timing window. */
        { "430", "Not confirmed" }, /* Generic error indicating the operation is not confirmed. See
                            3.8 Confirmation for a discussion. */
        { "431", "Failure fetching external data" }, /* The receiver could not follow an external 
                            reference (URL) given to it by the sender.  Note that in ICE 1.0 only 
                            the Subscriber is permitted to reply with this code.  A Syndicator 
                            MUST NOT reply with this code. */
        { "440", "Sorry" }, /* Used during parameter negotiation. See 4.5.1 Basic Negotiation Flow
                            for a discussion. The offer is refused. */ 
        { "441", "Counter-proposal" }, /* Used during parameter negotiation. See 4.5.1 Basic 
                            Negotiation Flow for a discussion. */
        { "442", "Renegotiation in progress" }, /* Used during parameter renegotiation. 
                            See 4.6.2 ice-change-subscription for a discussion. */
        { "450", "Range type invalid" }, /* Used during parameter negotiation. An ice-range can only 
                            be of type numerical, lexical, time, or enumeration. */
        { "451", "Undefined Reference" }, /* Used during parameter negotiation. An ice-range content 
                            element references an undefined or invalid element.  For example, an 
                            ice-span may reference a previously defined ice-span or ice-enum 
                            (typically in an ice-range-define). This error occurs when the 
                            reference is not defined. An element MUST be defined before it
                            can be referenced. */
        { "452", "Invalid Reference" }, /* Used during parameter negotiation. Several ice elements
                            in parameter negotiation MAY reference previously defined ice elements. 
                            The reference is to an element which cannot be used for the referring 
                            elements semantic operation. For example, a numeric ice-span that 
                            references a lexical ice-enum is invalid if the ice processor cannot 
                            convert the lexical value to a number within the span's range. */
        { "453", "Span selection out of range" }, /* Used during parameter negotiation. The value
                            selected in a span is less than the minimum or greater than the 
                            maximum; or is outside the boundary of any point sub-spans in the range.
                            */
        { "454", "Selection error" }, /* Used during parameter negotiation. The number of items 
                            selected in an enumeration does NOT match the select attribute. For 
                            example, the select attribute my require one-or-more items to be 
                            selected but the number selected in the response was zero. */
        { "455", "Limit invalid" }, /* Used during parameter negotiation. An ice-limit is not 
                            valid because it does not reference a known ID. This is also a 
                            Payload validation error; and this error is a refinement of that more
                            generic error. */
        { "456", "Enumeration selection empty or invalid" }, /* Used during parameter negotiation. 
                            The number of selectable ice-enum-items is not compatible with the 
                            select attribute. For example, the select attribute might indicate that
                            at least one ice-enum-item must be selected, but the actual number of 
                            items available is less than the number of ice-enum-items in the 
                            enumeration. */
        { "457", "Span invalid" }, /* Used during parameter negotiation. A Span has values that 
                            violate it's structure. For example, a span point MUST NOT have a 
                            value that is less than the span minimum nor larger than the span 
                            maximum. This error code is sent when such a structural violation is 
                            detected. */
        { "458", "Time formats incompatible for comparison." }, /* Used during parameter negotiation. 
                            Two time values to be compared during negotiation are in incomparable
                            formats. For example, a duration is being compared with an ICE date or time. */
        /* 
        5xx: Implementation errors and operational failures 

        These indicate errors caused by internal or operational problems, rather than by incorrect
        requests. Note that, like all other codes except for the 3xx series, these must be sent 
        individually with each response; if the error condition or operational problem prevents 
        the Responder from resolving the original payload down to the request level, use a 3xx 
        code instead.
        
        */
        
        { "5__", "Implementation errors and operational failures" },
        
        { "500", "Generic internal responder error" }, /* Catch-all for general problems; 
                            recovery/retry behavior unspecified. */
        { "501", "Temporary responder problem" }, /* Too busy, update in progress etc. 
                            Eventually an identical retry request might succeed. */
        { "503", "Not implemented" }, /* The entity does not implement the requested operation. */ 

        /* 
        6xx: Pending State 

        These codes indicate a state condition where the Subscriber is expected to send something
        to the Syndicator, or vice versa.
        
        */
        { "6__", "Pending State Codes" },

        { "601", "Unsolicited messages must be processed now" }, /* The sender has unsolicited 
                        messages for the receiver and the receiver has not yet requested them.
                        The sender has decided (based on implementation specific policy) to 
                        refuse to process any more requests until the unsolicited messages 
                        are collected. See Unsolicited Messages for a detailed explanation 
                        of unsolicited message operation. */
        { "602", "Excessive confirmations outstanding" }, /* The Syndicator had requested 
                        confirmation of package delivery, and now refuses to perform any 
                        additional operations until the Subscriber supplies the confirmations 
                        (positive or negative).  */
        { "603", "No more confirmations to send" }, /* The Subscriber received an 
                        ice-send-confirmations message but believes it has sent all of the 
                        confirmations already. */
        { "604", "No more unsolicited messages" }, /* The Subscriber sent an 
                        ice-unsolicited-now but the Syndicator has no unsolicited 
                        messages to send. */ 
        /*
        7xx: Local Use Codes 

        These codes are reserved for use by the local ICE implementation and MUST NOT ever 
        be sent to another ice processor over the transport medium. The intent is that this 
        range of codes can be used by the local ICE implementation software to communicate 
        transport level error conditions, or other specific local conditions, using the 
        ice-code mechanism in a way guaranteed to not collide with any other usage of 
        ice-code values. 
        
        */
        { "7__", "Local Use Codes" },
        
        /*
        8xx  Reserved Codes
        
        These codes are reserved for future use.
        
        */
        { "8__", "Reserved Codes" },
        
        /*
        9xx: Experimental Codes 

        ICE implementations MUST NOT use any codes not listed in this specification, 
        unless those codes are in the 9xx range. The 9xx range allows implementations 
        to experiment with new codes and new facilities without fear of collision with future
        versions of ICE. How a given system treats any 9xx code is a quality of implementation issue. 
        
        */
        { "9__", "Experimental Codes" }
	};
}