/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/util/MasterSlaveSemiphore.java,v 1.1.1.1 2000/09/19 19:39:51 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.util;

/** MasterSlaveSemiphore is a flagging system to allow many non-destructive 
 * (e.g., read-only) threads (Slaves) to simultaneous access an object but 
 *  allow a seperate thread (Master) to occaisionally take exclusive control.
 */
public class MasterSlaveSemiphore { 
	private int slavecount;
	private boolean lock;
	
	public MasterSlaveSemiphore() {
		slavecount=0;
		lock=false;
	}

	/** A slave should call this before attempting to access the associated object.
	 */
	public synchronized void startSlave() {
		acquire();
		++slavecount;
		release();
	}
	/** When a slave finishes with the associated object, use this to notify 
	 *  others of completion.
	 */
	
	public synchronized void stopSlave() {
		--slavecount;	
		notifyAll();
	}
	/** The Master should call this before attempting to access the object destructively. 
	 * It will block until exclusive access is provided.
	 */
	public synchronized void startMaster() {
		acquire();
		while (slavecount>0) {
			try {
				wait();
			}
			catch (Exception e) {}
		}
	}
	/** The Master should call this to notify others that that the associated object 
	 * is availible.
	 */
	public synchronized void stopMaster() {
		release();	
	}
	
	private synchronized void acquire() {
		while (lock == true) { 
			try {
				wait();
			}
			catch (Exception e) {}
		}
		lock = true;
		notifyAll();
	}
	private synchronized void release() {
		lock = false;
		notifyAll();
	}
	
}
