/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/util/PayloadIdentifier.java,v 1.2 2000/11/22 02:24:13 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.util;

import java.util.*;
import java.text.*;

//
// icecubes  imports
//
import com.adobe.rice.IceCubes;
import com.adobe.rice.common.*;
import com.adobe.rice.common.policies.*;
import com.adobe.rice.common.storage.logging.*;     // so we can write to the log.


public class PayloadIdentifier
{
    private static int pliCount = 0;
    
    private static String TAG = "PL";
        
    private String domain = "YourDomain.com";
    
    private String utcDateTime = "CCYY-MM-DDTHH:MM:SS.III";
    private String count = "000000"; 
    
    public PayloadIdentifier()
    {
        // use a local counter to make sure that only we get the value
        // of pliCount after it is incremented.
        int localCount;
        // we are using TAG as a convenient class object to syncronize around
        // what we care about is that only one thread at a time increment 
        // pliCount.  Further, we want the smallest block of code possible as
        // the bottleneck so that it can be completed most rapidly.
        synchronized( TAG )
        {
            localCount = ++pliCount;
        }
        // calendar is local -- only needed till we get string representation.
        Calendar calendar = Calendar.getInstance();
        // set it with the current date.
        calendar.setTime( new Date() );
        
        // set the domain name correctly
		domain = (String) Tray.policyManager.getSafeSelection("Domain", "null.com" );
        // 
        count = Integer.toString(localCount);
        // get the utcDateTime
        utcDateTime = this.getUTC(calendar);
        
    }
    
    public String get() { return this.TAG + "-" + this.domain + "-" + utcDateTime + "-" + count; }
    
    public String toString() { return this.get(); }
    
    
	public boolean equals(PayloadIdentifier pid) 
	{
	    // times must be equal
	    if ( !this.utcDateTime.equals(pid.utcDateTime) )
	        return false;
	    // counts must be equal
	    if ( !this.count.equals(pid.count) )
	        return false;
	    // domains must be equal
	    if ( !this.domain.equals(pid.domain) )
    		return false;
        // they must be equal!
    	return true;
	}
	
	private String getUTC( Calendar cal )
	{
	    // create decimal formats for each of elements of a UTC date & time:
	    DecimalFormat utcYearFormat = new DecimalFormat("0000");
	    DecimalFormat utcMonthFormat = new DecimalFormat("00");
	    DecimalFormat utcDayFormat = new DecimalFormat("00");
	    DecimalFormat utcHourFormat = new DecimalFormat("00");
	    DecimalFormat utcMinuteFormat = new DecimalFormat("00");
	    DecimalFormat utcSecondFormat = new DecimalFormat("00");
	    DecimalFormat utcMillisecondFormat = new DecimalFormat("000");
	    
	    // get the elements out of the calendar
	    int year = cal.get(Calendar.YEAR);
	    int month = cal.get(Calendar.MONTH);
	    int day = cal.get(Calendar.DAY_OF_MONTH);
	    int hour = cal.get(Calendar.HOUR_OF_DAY);
	    int minute = cal.get(Calendar.MINUTE);
	    int second = cal.get(Calendar.SECOND);
	    int millisecond = cal.get(Calendar.MILLISECOND);
	    
	    // convert each element to its proper string representation
	    
	    String aUTCyear =   ( utcYearFormat.format(   (long) year )  ).toString();
	    String aUTCmonth =  ( utcMonthFormat.format(  (long) month ) ).toString();
	    String aUTCday =    ( utcDayFormat.format(    (long) day )   ).toString();
	    
	    String aUTChour =   ( utcHourFormat.format(   (long) hour )  ).toString();
	    String aUTCminute = ( utcMinuteFormat.format( (long) minute )).toString();
	    String aUTCsecond = ( utcSecondFormat.format( (long) second )).toString();
	    
	    String aUTCmillisecond = ( utcMillisecondFormat.format((long)millisecond)).toString();
	    
	    String theUTCdate = aUTCyear  + "-" +
	                        aUTCmonth + "-" +
	                        aUTCday;
	                        
	    String theUTCtime = aUTChour   + ":" +
	                        aUTCminute + ":" +
	                        aUTCsecond + "." +
	                        aUTCmillisecond;
	                        
	    return theUTCdate + "T" + theUTCtime;
	}
}
