/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/util/UUID.java,v 1.3 2000/11/22 02:24:13 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.util;
/** Adobe RICE */


public class UUID 
{
    private byte[] uuid;
    
    public UUID()
    {
        uuid = new byte[] 
        {
            0, 0, 0, 0,  0, 0, 0, 0,
            0, 0, 0, 0,  0, 0, 0, 0
        };
    }
    
    public UUID( String aUUID )
    {
        this();
        this.set( aUUID );
    }
    
    public UUID( byte[] aUUID )
    {
        this();
        this.set( aUUID );
    }
    
	public boolean equals(UUID id) 
	{
		for ( int i = 0; i < this.uuid.length; i++ )
		{
		    if ( this.uuid[i] != id.uuid[i] )
		        return false;
		}
		return true;
	}
	
	public void set( int index, byte nibble )
	{
	    if ( ( index >= 0 ) && ( index < this.uuid.length ) )
	    {
	        int odd = index%2;
	        byte val = this.uuid[ index/2 ];
	        if ( odd > 0 )
	        {
	            val &= 0xf0;
	            nibble &= 0x0f;
	        }
	        else
	        {
	            val &= 0x0f;
	            nibble <<= 4;
	        }
	        val |= nibble;
	        this.uuid[ index/2 ] = val;
	    }
	}
	
	public void set( byte[] theBytes )
	{
	    if ( theBytes.length != ( this.uuid.length * 2 ) )
	        return;
	    for ( int i = 0; i < theBytes.length; i++ )
	    {
	        this.set( i, theBytes[i] );
	    }
	}
	
	public void set( String aUUID )
	{
	    // note that if the UUID syntax checking here is minimal
	    // This routine will return at worst, a zero filled UUID.
	    // It will find the hex characters in the string and right
	    // align them into a UUID.
	    this.uuid = this.parseUUID( aUUID );
	}
	
	public byte get( int index )
	{
	    byte val = 0;
	    if ( ( index >= 0 ) && ( index < this.uuid.length * 2 ) )
	    {
	        val = this.uuid[ index / 2 ];
	        if ( (index % 2) > 0 )
	            val &= 0x0f;
	        else
	            val >>= 4;
	    }
	    return val;
	}
	
	public byte[] get()
	{
	    byte result[] = new byte[]
	    {
	        0, 0, 0, 0,    0, 0, 0, 0,
	        0, 0, 0, 0,    0, 0, 0, 0
	    };
	    
	    for ( int i = 0; i < this.uuid.length; i++ )
	    {
	        byte val = this.uuid[i];
	        byte upperNibble = (byte) ( val >> 4 );
	        byte lowerNibble = (byte) ( val & 0x0f );
	        result[ 2 * i ] = upperNibble;
	        result[ (2 * i) + 1 ] = lowerNibble;
	    }
	    return result;
	}
	
	private static final char[] hexChars = new char[]
	{
	    '0', '1', '2', '3', '4', '5', '6', '7', 
	    '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'
	};
	
	
	public String toString()
	{
	    String result = "";
	    for ( int i = 0; i < this.uuid.length; i++ )
	    {
	        byte val = this.uuid[i];
	        byte upperNibble = ( byte ) ( ( val >> 4 ) & 0x0f );
	        byte lowerNibble = ( byte ) ( val & 0x0f );
	        
	        result += hexChars[ upperNibble ];
	        result += hexChars[ lowerNibble ];
	        // 
	        //              6ba7b811-9dad-11d1-80b4-00c04fd430c8 
	        // UUID form is dddddddd-dddd-dddd-dddd-dddddddddddd
	        //               0 1 2 3  4 5  6 7  8 9  A B C D E F
	        switch( i )
	        {
	            case 3:
	            case 5:
	            case 7:
	            case 9:
	                result += "-";
	            default:
	                break;
	        }
	    }
	    return result;
	}
	
    //
    // utility routines
    //
    public byte[] parseUUID( String aUUID )
    {
        boolean inError = false;
        
	    char ch;
	    int i, digitCount = 0;
	    StringBuffer buf = new StringBuffer(32);
	    
	    byte[] theUUID = new byte[16];
	    
	    // validate the UUID here:
	    for( i = 0; i < aUUID.length() ; i++ )
	    {
	        ch = aUUID.charAt(i);
	        if ( digitCount < 32 )
	        {
	            if ( ( ch >= '0' ) &&  ( ch <= '9' ) )
	                { buf.append(ch); digitCount++; continue; }
	            if ( ( ch >= 'A' ) &&  ( ch <= 'F' ) )
	                { buf.append(ch); digitCount++; continue; }
	            if ( ( ch >= 'a' ) && ( ch <= 'f' ) )
	                { buf.append(ch); digitCount++; continue; }
	            // It is ok to have blanks and dashes in the UUID
	            if ( ( ch == ' ' ) || ( ch == '-' ) )
	                continue;
	        }
	        // if execute's past the continue above, the UUID is invalid.
	        if ( !inError )
	        {
	            inError = true;
                Object[] parms = { "UUID", UtilMessages.get("UUIDcharactersInvalid")  };  
                UtilMessages.log( UtilMessages.get("invalid") , parms , 200000000 );
            }
	    }
	    if ( digitCount != 32 )
	    {
	        inError = true;
	        Object[] parms = { "UUID", UtilMessages.get("UUIDtoManyCharacters") };
	        UtilMessages.log( UtilMessages.get("invalid"), parms, 200000000 );
	    }
	    // It can be converted to a UUID -- Convert it now!
	    // run through the character string backward so that theUUID is
	    // aligned on the right.  (i.e. 03ffe9 -> 00 00 00 00  00 03 FF E9 )
	    digitCount = 31;
	    byte wb = 0, tb = 0;
	    
	    for ( i = buf.length()-1; i >= 0 ; i--)
	    {
	        ch = buf.charAt(i);
	        if ( (ch >= 'a') && ( ch <= 'f' ) )
	            wb = (byte) ( 10  + ( ch - 'a' ) );
	        else if ( ( ch >= 'A' ) && ( ch <= 'F' ) )
	            wb = (byte) ( 10  + ( ch - 'A' ) );
	        else if ( ( ch >= '0' ) && ( ch <= '9' ) )
	            wb = (byte) ( ch - '0' );
	        else 
	            continue;
	        if ( ( digitCount % 2 ) == 0 )
	        {   // on an even digit
	            tb = (byte) ( (tb << 4) + wb );
	            theUUID[ digitCount/2 ] = tb;
	        }
	        else
	            tb = wb;
	        digitCount--;
	    }
	    for ( i = digitCount; i >= 0; i-- )
	    {
	        if ( i % 2 == 0 )
	        {
	            tb = (byte) ( ( tb << 4 ) + wb );
	            theUUID[ i / 2 ] = tb;
	            wb = 0;
	        }
	        else
	            tb = wb;
	    }
	    return theUUID;
    }
	
	
}
