/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/util/time/IceDate.java,v 1.2 2000/11/22 02:24:47 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.util.time;

import java.util.*;
import java.text.*;

public class IceDate 
{ 
	Date date;
	
	private SimpleDateFormat informat = new SimpleDateFormat("yyyy'-'MM'-'dd"),
	                         outformat = new SimpleDateFormat("yyyy'-'MM'-'dd");
	
	private final static long timefullday = 86400000;
	
	public IceDate() 
	{
		this.date = new Date();
		datecleanup();
	}
	
	IceDate (long total)
	{
		this.date = new Date( total );
		// date.setTime(total);
		datecleanup();
	}
	
	public IceDate (Date date) 
	{ 
		this.date = date;
		datecleanup();
	}
	
	public IceDate (String iceDateString) 
	{
		setDate(iceDateString);
	}
	
	
	private	static final String INFINITY = " 9999-12-31 ";
	
	/** getDateInfinite returns a new IceDate with a value of positive infinity.
	 */	
	public static IceDate getDateInfinite() 
	{
		IceDate id = new IceDate();
		id.setDate(INFINITY);
		return id;
	}
	/** getDateTomorrow returns a new IceDate with a value of "tomorrow".
	 */	
	public static IceDate getDateTomorrow() 
	{
		IceDate id = new IceDate();
		return id.nextDay();
	}

	/** getDateYesterday returns a new IceDate with a value of "yesterday".
	 */	
	public static IceDate getDateYesterday() 
	{
		IceDate id = new IceDate();
		return id.prevDay();
	}
	
	public IceDate nextDay() 
	{
		IceDate id = new IceDate();
		id.date.setTime(this.date.getTime()+timefullday); // HACK implementation dep
		id.datecleanup();
		return id;
	}

	public IceDate prevDay() 
	{
		IceDate id = new IceDate();
		id.date.setTime(this.date.getTime()-timefullday); // HACK implementation dep
		id.datecleanup();
		return id;
	}
		
	
	private void setDate(String iceDateString) 
	{
		StringBuffer sb = new StringBuffer(30);
		sb.append(iceDateString);
		date = informat.parse(sb.toString(), new ParsePosition(0));
		datecleanup();
	}
	
	public String toString() 
	{
		return outformat.format(date);
	}
	
	long getTotal()
	{
		return date.getTime();
	}
	
	public boolean before(IceDate iceDate) 
	{
		return date.before(iceDate.date);
	}
	
	public boolean equals(IceDate iceDate) 
	{
	    return ( date.getTime() == iceDate.getTotal() );
	}
	
	public boolean after(IceDate iceDate) 
	{
		return date.after(iceDate.date);
	}

	private void datecleanup() 
	{
		// we want to ensure the date is only in whole units of days.
		date.setTime(date.getTime() - (date.getTime()%timefullday)); 
	}
}