/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/util/time/IceDateTime.java,v 1.2 2000/11/27 20:24:25 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.util.time;

import java.util.*;
import java.text.*;

public class IceDateTime 
{ 
	
	private IceDate date;
	private IceTime time;

	public IceDateTime()
	{
		setDateTime(new IceDate(), new IceTime());
	}
	
	public IceDateTime(IceDate iceDate, IceTime iceTime) 
	{
		setDateTime(iceDate,iceTime);
	}
	
	public IceDateTime(String iceDateTime) throws IllegalArgumentException
	{
		setDateTime(iceDateTime);
	}
	
	private void setDateTime(IceDate iceDate, IceTime iceTime) 
	{
		IceTime.cleanupBoundary(iceDate,iceTime);
		time=iceTime;
		date=iceDate;
	}

	private void setDateTime(String iceDateTime) throws IllegalArgumentException  
	{
		StringTokenizer st = new StringTokenizer(iceDateTime,"T",false);
		try 
		{
			date = new IceDate(st.nextToken());
			time = new IceTime(st.nextToken());
		}
		catch (NoSuchElementException e) 
		{
			throw new IllegalArgumentException();
		}
	}

	public IceDuration add(IceDateTime iceDateTime) 
	{
		long total = this.date.getTotal()
					 + this.time.getTotal()
					 + iceDateTime.date.getTotal()
					 + iceDateTime.time.getTotal();
		IceDuration id = new IceDuration();
		id.setTotal(total);
		return id;
	}
	
	public IceDuration subtract(IceDateTime iceDateTime) 
	{
		long total = this.date.getTotal()
					 + this.time.getTotal()
					 - iceDateTime.date.getTotal()
					 - iceDateTime.time.getTotal();
		IceDuration id = new IceDuration();
		id.setTotal(total);
		return id;
	}
	
	public IceDateTime add(IceDuration iceDuration) 
	{
		long total = this.date.getTotal()
					 + this.time.getTotal()
					 + iceDuration.getTotal();
		IceDateTime idt = new IceDateTime();
		idt.date = new IceDate(total);
		idt.time = new IceTime(total);
		return idt;
	}
	public IceDateTime subtract(IceDuration iceDuration) 
	{
		long total = this.date.getTotal()
					 + this.time.getTotal()
					 - iceDuration.getTotal();
		IceDateTime idt = new IceDateTime();
		idt.date = new IceDate(total);
		idt.time = new IceTime(total);
		return idt;
	}
	
	public IceDate getDate() 
	{
		return date;	
	}
	
	public IceTime getTime() 
	{
		return time;	
	}
	
	public boolean before(IceDateTime iceDateTime) 
	{
		return date.before(iceDateTime.date) && time.before(iceDateTime.time);
	}
	
	public boolean equals(IceDateTime iceDateTime) 
	{
		return date.equals(iceDateTime.date) && time.equals(iceDateTime.time);
	}
	
	public boolean after(IceDateTime iceDateTime) 
	{
		return date.after(iceDateTime.date) && time.after(iceDateTime.time);
	}
	
	public String toString() 
	{
		StringBuffer sb = new StringBuffer(date.toString() + "T" + time.toString());
		return sb.toString();
	}
	
	
}
