/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/util/time/IceDayGroup.java,v 1.3 2000/11/22 21:20:19 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.util.time;

//$java1.1.8$ import com.sun.java.util.collections.*;
import java.util.*;
import com.adobe.rice.util.*;

public class IceDayGroup{
	private LinkedList iceDays;
	
	private void construct() {
		iceDays = new LinkedList();
	}

	public IceDayGroup(String weekdays, String monthdays) {
		construct();
		
		java.util.Vector wd = new java.util.Vector();
		java.util.Vector md = new java.util.Vector();
		StringTokenizer wst = new StringTokenizer(weekdays," ",false);
		StringTokenizer mst = new StringTokenizer(monthdays," ",false);
		
		// we need to see if any of the tokens is ANY as that subsumes all other tokens.
		while (wst.hasMoreTokens())
			wd.addElement(wst.nextToken());
		while (mst.hasMoreTokens())
			md.addElement(mst.nextToken());
		
		boolean anyWeekday = false,
			anyMonthday = false;
		for (int i=0;i<wd.size();i++) 
			if (wd.elementAt(i).equals("ANY"))
				anyWeekday=true;
		for (int i=0;i<md.size();i++) 
			if (md.elementAt(i).equals("ANY"))
				anyMonthday=true;
		
		// we must now go and generate cross-product IceDays; a lot of this code
		// deals with shortcutting this st "ANY" crossproducts are made.
		if (anyWeekday && anyMonthday)
			iceDays.add(new IceDay(IceDay.Monthday.ANY,IceDay.Weekday.ANY));
		
		if (anyWeekday && !anyMonthday) {
			for (int i=0;i<md.size();i++) {
				if (md.elementAt(i).equals("LAST"))
					iceDays.add(new IceDay(IceDay.Monthday.LAST,IceDay.Weekday.ANY));
				else 
					iceDays.add(new IceDay(
										   (byte)new Integer((String)md.elementAt(i)).intValue(),
										   IceDay.Weekday.ANY
										   )
								);
			}
		}
		
		if (!anyWeekday && anyMonthday) {
			for (int i=0;i<wd.size();i++) {
					iceDays.add(new IceDay(
										   IceDay.Monthday.ANY,
										   (byte)new Integer((String)wd.elementAt(i)).intValue()
										   )
								);
					
			}
		}
		
		// go through and clean this up so it does a double iteration through them all and deals with it.
		if (!anyWeekday && !anyMonthday) {
			for (int i=0;i<md.size();i++) {
				for (int j=0;j<wd.size();j++) {
					
					if (md.elementAt(i).equals("LAST"))
						iceDays.add(new IceDay(
											   IceDay.Monthday.LAST,
											   (byte) new Integer((String)wd.elementAt(j)).intValue()
											   )
									);
					else
						iceDays.add(new IceDay(
											   (byte)new Integer((String)md.elementAt(i)).intValue(),
											   (byte)new Integer((String)wd.elementAt(j)).intValue()
											   )
									);
				}
			}
			
			
		}
	}
		
		
	/** Create an IceDateGroup consisting of all IceDates describing a specific Calendar
	 */
	public IceDayGroup(IceDate iceDate) {
		
		construct();

		byte weekday=IceDay.Weekday.ANY, monthday=IceDay.Monthday.ANY;
		boolean lastday=false;

		Calendar calendar = Calendar.getInstance();
		calendar.setTime(iceDate.date);

		// deduce weekday and use the IceDate token (M=1,Su=7, as per ICE NMTOKEN spec	)	
		int wd = calendar.get(Calendar.DAY_OF_WEEK);
		switch (wd) {
			case Calendar.MONDAY: {
							    weekday = 1; 
								break;
										 }
			case Calendar.TUESDAY: {
										  weekday = 2 ; 
										  break;
									  }
			case Calendar.WEDNESDAY: {
									    weekday = 3;
										break;
										}
			case Calendar.THURSDAY: {
										   weekday = 4; 
										   break;
									   }
			case Calendar.FRIDAY: {
										 weekday = 5;
										 break;
									 }
			case Calendar.SATURDAY: {
										   weekday = 6;
										   break;
									   }
			case Calendar.SUNDAY: {
										 weekday = 7; 
										 break;
									 }
		}
		// deduce weekday and use the IceDay token (M=1,Su=7, as per ICE NMTOKEN spec		
		int md = calendar.get(Calendar.DAY_OF_MONTH);
		int mth = calendar.get(Calendar.MONTH);
		
		
		// determine whether this day is the "last" of the month
		// if the month has 31 days
		if ((mth==Calendar.JANUARY ||
			 mth==Calendar.MARCH ||
			 mth==Calendar.MAY ||
			 mth==Calendar.JULY ||
			 mth==Calendar.AUGUST ||
			 mth==Calendar.OCTOBER ||
			 mth==Calendar.DECEMBER) && monthday == 31)
			lastday=true;
		
		// if the month has 30 days
		else if ((mth==Calendar.APRIL ||
				  mth==Calendar.JUNE ||
				  mth==Calendar.SEPTEMBER ||
				  mth==Calendar.NOVEMBER) && monthday == 30)
			lastday=true;
		
		/* if it's the damned month, we must consider leap years.
		<http://aa.usno.navy.mil/AA/faq/docs/leap_years.html>
		According to the Gregorian calendar, which is the civil calendar in use today, 
		years evenly divisible by 4 are leap years, with the exception of centurial years 
		that are not evenly divisible by 400. Therefore, the years 1700, 1800, 1900 and 2100 
		are not leap years, but 1600, 2000, and 2400 are leap years.
		*/
		
		else if (mth==Calendar.FEBRUARY) {
			int yr=calendar.get(Calendar.YEAR);
			if (monthday==29) 
				lastday=true;
			// in most cases, each fourth year is a leap year
			if (monthday==28 && yr%4==0)
				lastday=true;
			/*
			<cynical> since we really want ICE to work in 2100 and beyond, if the year is of 
			the form AA00, it is only a leap year if AA is divisible by 4. </cynical>
			*/
			if (yr%100==0 && ((yr/100)%4==0))
				lastday=false;
		}
		
		// now create all relevent IceDates corresponding to the Calendar	
		iceDays.add(new IceDay(monthday,weekday));
		iceDays.add(new IceDay(IceDay.Monthday.ANY,weekday));
		iceDays.add(new IceDay(monthday,IceDay.Weekday.ANY));
		if (lastday) {
			iceDays.add(new IceDay(IceDay.Monthday.LAST,weekday));
			iceDays.add(new IceDay(IceDay.Monthday.LAST,IceDay.Weekday.ANY));
		}
		
	}
	public Iterator getListIterator() 
	{
		return iceDays.listIterator();	
	}
	
}