/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/util/time/IceDuration.java,v 1.2 2000/11/22 02:24:47 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.util.time;

import java.util.*;
public class IceDuration implements Cloneable { 
	Date duration;
	
	protected void construct() {
		duration = new Date();
	}
	
	/** IceDuration is package-level constructor for creating zero-len
	 * duration. Can be used in conjunction with package-level setTotal()
	 */
	public IceDuration() {
		construct();
		setDuration("P0S"); // default internal constructor for friendly package mbrs
	}
	
	
	public IceDuration(String iceDurationString) {
		construct();
		setDuration(iceDurationString);
	}
	protected void setDuration(String iceDurationString) {
		StringBuffer clean = new StringBuffer(); // the string without P or S
		// remove P and S
		clean.append(iceDurationString.toCharArray(),1,iceDurationString.length()-2);
		StringTokenizer st = new StringTokenizer(clean.toString(),",",false);
		Long secs = new Long(st.nextToken());
		long time= 1000*secs.longValue();
		if (st.hasMoreTokens()) { // if subseconds specified
			/* ICE provides fractions of a second. We MUST parse up to nine subsecond 
			digits, and SHOULD handle arbitrary digits. Since Java cannot handle things 
			smaller than milliseconds, "clean up" the string--i.e., round to 3 significant 
			figures of milliseconds and possibly just insert the trivial 000 milliseconds.	
			*/
			Double dsubsec = new Double("," + st.nextToken());
			Integer isubsec = new Integer((int)((1000*dsubsec.doubleValue())+ .5 ));
			time+=isubsec.longValue();	
		}
		duration.setTime(time);		
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();
		long subsecs = duration.getTime() % 1000;
		long secs = duration.getTime()-subsecs;
		
		sb.append("P");
		sb.append(secs);
		if (subsecs>0)
			sb.append(","+ subsecs);
		sb.append("S");
		return sb.toString();
	}
	
	
	public long getTotal(){
		return duration.getTime();
	}
	void setTotal(long total) {
		duration.setTime(total);
	}
	
	public boolean before(IceDuration iceDuration) {
		return duration.before(iceDuration.duration);
	}
	public boolean equals(IceDuration iceDuration) {
		return duration.equals(iceDuration.duration);
	}
	public boolean after(IceDuration iceDuration) {
		return duration.after(iceDuration.duration);
	}
	
	public double divide(IceDuration divisor) {
		long top = this.duration.getTime();
		long bottom = divisor.duration.getTime();
		return (double) top/bottom;
	}
	
	public IceDuration divide(int divisor) {
		long top = this.duration.getTime();
		long bottom = (long) divisor;
		IceDuration quotient = new IceDuration();
		quotient.setTotal(top/bottom);
		return quotient;
	}

	public Object clone() throws CloneNotSupportedException
	{
	    // use default cloning of field-to-field copy
	    return super.clone();
	}
	
}
	
