/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/util/time/IceTime.java,v 1.2 2000/11/22 02:24:47 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.util.time;

import java.util.*;
import java.text.*;

public class IceTime implements Cloneable
{ 
	private Date time;
	private boolean completeday;
	private SimpleDateFormat informat, outformat;
	
	private static long timefullday = 86400000;
	
	private void construct() 
	{
		// note that the INPUT format tacks on the timezone-java-kludge
		informat = new SimpleDateFormat("HH':'mm':'ss','SSS' 'zzz");
		// note that the OUTPUT format is seperated st we can convert back to subsec.
		outformat = new SimpleDateFormat("HH':'mm':'ss','SSS");
		completeday = false;
	}

	private void timecleanup() 
	{
		if (time.getTime() == timefullday) 
		    completeday=true;
		time.setTime(time.getTime()%timefullday); // we want to ensure the time < 1 day.
	}
	
	public IceTime() 
	{
		construct();
		this.time = new Date();
		timecleanup();
	}
	
	public IceTime (Date time) 
	{ 
		construct();
		this.time=time;
		timecleanup();
	}
	
	IceTime (long total)
	{
		construct();
		this.time = new Date();
		time.setTime(total);
		timecleanup();
	}
	
	void setTotal(long total) 
	{
		time.setTime(total);
		timecleanup();
	}
	
	public IceTime(String iceTimeString) 
	{
		/* ICE provides fractions of a second. And, ICE requires (MUST) parse up to 
		   nine subsecond digits, and SHOULD handle arbitrary digits. Since Java cannot 
		   handle things smaller than milliseconds, "clean up" the string--i.e., round 
		   to 3 significant figures of milliseconds and possibly just insert the 
		   trivial 000 milliseconds.	
		*/
		construct();
		// note that the string buffer must be at least 16 characters -- use 20 to be sure. vbh001029
		StringBuffer sb = new StringBuffer(20);		
		sb.append(iceTimeString.toCharArray(),0,8);
		sb.append(",");
		if (iceTimeString.length() > 8) 
		{
			StringTokenizer st = new StringTokenizer(iceTimeString,",",false);
			st.nextToken(); // to ignore the first half
			Double dsubsec = new Double("." + st.nextToken()); // to get a fraction
			// now create a pseudo-"rounded" via truncation integer			
			Integer isubsec = new Integer((int)((1000*dsubsec.doubleValue())+ 0.5 ));
			// handle the case when isubsec is from 0..99
			if ( isubsec.intValue() < 100 )
			    sb.append("0");
			// handle the case when isubsec is from 0..9
			if ( isubsec.intValue() < 10 )
			    sb.append("0");
			sb.append(isubsec.toString());
		}
		else
			sb.append("000");
		sb.append(" GMT"); // yeah, a kludge. needed to parse INTO UTC as per ICE spec.
		/*
		
		time = informat.parse(iceTimeString,new ParsePosition(0));
		
		*/
		
		this.time = informat.parse(sb.toString(), new ParsePosition(0));
		//
		// it is possible that this.time is still not set -- the string passed in
		// was completely invalid.
		if ( this.time == null )
		    this.time = new Date(0l);
		timecleanup();
	}
	
	/** getTimeStartOfDay returns a new IceTime with a value of "start-of-day".
	 */	
	public static IceTime getTimeStartOfDay() {
		IceTime it = new IceTime();
		it.time.setTime(0); // HACK implementation dep
		it.timecleanup();
		return it;
	}
	/** getTimeEndOfDay returns a new IceTime with a value of "end-of-day".
	 *  By the ice spec, this is really 24:00:00 -- midnight ahead
	 */	
	public static IceTime getTimeEndOfDay() {
		IceTime it = new IceTime();
		it.time.setTime(timefullday); // HACK implementation dep
		it.timecleanup();
		return it;
	}
		
	public String toString() {
		return outformat.format(time);
	}
	
	long getTotal(){
		return time.getTime();
	}
	
	/** cleanupBoundary will check and convert "today & midnight tomorrow" to "tomorrow & midnight"
	 * e.g., Wednesday 24:00:00 -->  Thursday 00:00:00
	 */
	public static void cleanupBoundary(IceDate iceDate, IceTime iceTime) {
		if (iceTime.completeday == true) {
			// we're about to subtract a day of TIME and add a day of DATE
			iceTime.completeday=false;
			iceTime.time.setTime(0); // beginning of day
			iceDate.date.setTime(iceDate.getTotal()+timefullday);
		}
	}
		
	
	/** add will add a duration and return a new result. 
	 * Note that since IceTimes are only specified as a time of day, so
	 * if the result will overflow (e.g., 2pm + 30 hours > Midnight), the value 
	 * returned will be the same as getTimeEndOfDay.
	 */
	public IceTime add(IceDuration duration) {
		long total = this.time.getTime() + duration.duration.getTime();
		if (total>=timefullday)
			return getTimeEndOfDay();
		IceTime it = new IceTime();
		it.setTotal(total);
		return it;
	}
	
	public int getRatio(int length) {
		// hack -- implementation dependent
		/* note that this ignores the completeday stuff that was subsequently added.
		this interface was somewhat of a hack needed for schedulestats stuff.
		this shouldn't adversely affect that usage, but if it is subequently used
		by others, perhaps it should be rethough/cleaned up or something.
		jdh 20aug99
		*/
		
		return (int) (length*time.getTime()/timefullday);
	}
	
	
	public boolean before(IceTime iceTime) {
		return ((! this.completeday && iceTime.completeday)
				|| 
				time.before(iceTime.time));
	}
	
	public boolean equals(IceTime iceTime) {
		return (this.completeday==iceTime.completeday 
				&&
				time.equals(iceTime.time));
	}
	
	public boolean after(IceTime iceTime) {
		return ((this.completeday && !iceTime.completeday)
				|| 
				time.after(iceTime.time));
	}
}