/*
$Header: /opt/repository/icecubes/Rice/code/com/adobe/rice/common/payloadAssembly/Payload.java,v 1.5 2000/12/02 23:24:24 bhunt Exp $

ICEcubes - Reference implementation of the Information and Content Exchange Protocol
Copyright (C) 1998 - 2000 ICE Authoring Group

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

The ICE Authoring Group thanks Adobe System's Inc. for supporting 
the initial development of ICEcubes.

This library was written on behalf of the ICE Authoring Group, affialiated
with GCA's IdeaAlliance.  For the latest information on ICE see 

http://www.icestandard.org

*/

package com.adobe.rice.common.payloadAssembly;

//
// java imports
//
import java.io.*;
import java.util.*;

//
// icecubes  imports
//
import com.adobe.rice.IceCubes;
import com.adobe.rice.util.*;
import com.adobe.rice.util.time.*;
import com.adobe.rice.common.*;
import com.adobe.rice.common.XMLassembly.*;
import com.adobe.rice.common.policies.*;
import com.adobe.rice.common.storage.logging.*;     // so we can write to the log.

public class Payload extends AssemblyElement 
{
    PayloadIdentifier payloadId;
    IceDateTime       timestamp;
    String            iceVersion;
    
    // <!ELEMENT ice-payload ( ice-header, ( ice-request+ |  
    //                                       ice-response+ | 
    //                                       ice-unsolicited-now | 
    //                                       ice-unsolicited-request+ |  
    //                                       ice-unsolicited-response+ 
    //                                     ) 
    //                       ) 
    // >  
    // <!ATTLIST ice-payload  
    //                       payload-id      CDATA #REQUIRED 
    //                       timestamp       CDATA #REQUIRED 
    //                       sender-location CDATA #IMPLIED 
    //                       ice.version     CDATA #REQUIRED 
    // >    
    public Payload( Element parent, DocType dtd ) throws ElementException
    {
        super( "ice-payload", parent, dtd );
        this.setOkToFormat(true);       // ok to format.

        this.payloadId = new PayloadIdentifier();
        this.timestamp = new IceDateTime();
        this.iceVersion = Constants.ICE_VERSION;
        try 
        {
            this.setAttribute("payload-id", this.payloadId.toString() );
            this.setAttribute("timestamp", this.timestamp.toString() );
            this.setAttribute("ice.version", this.iceVersion );
        }
        catch( AttributeException ae )
        {
           // log the fact that we got an attribute execption...
           Object[] parms = { "ice-payload", ae.getMessage() };    
           AssemblyMessages.log(AssemblyMessages.get("missingRequiredAttribute"), 
               parms, ImpactRange.CRITICAL );
        }
        //
        // now add in the header
        //
        try
        {
            this.open();
            this.add( new Header( this, dtd ) );
        }
        catch( ElementException ee )
        {
            Object[] parms = { "ice-payload", "ice-header", ee.getMessage() };
            AssemblyMessages.log(AssemblyMessages.get("contentInvalid"), 
                parms, ImpactRange.CRITICAL );
        }
        finally
        {
            this.close();
        }
    }
    
    public Payload( DocType dtd ) throws ElementException
    {
        this( null, dtd );
    }
    
    public PayloadIdentifier getPayloadId() { return this.payloadId; }
    public IceDateTime getTimestamp() { return this.timestamp; }
    public String getIceVersion() { return this.iceVersion; }
}
